/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 *  Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 abstraction library header file (common)
*/


#ifndef _FAPEX_ABST_COMMON_PRIV_H
#define _FAPEX_ABST_COMMON_PRIV_H

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

#ifdef _LINUX_FAPI

#define FAPEX_SYS_THREAD_STATE_INIT     0
#define FAPEX_SYS_THREAD_STATE_ACT      1
#define FAPEX_SYS_THREAD_STATE_WAIT     2
#define FAPEX_SYS_THREAD_STATE_TIMEWAIT 3

#define FAPEX_SYS_THREAD_LIST_MAX   256
#define FAPEX_SYS_THREAD_NAME_SIZE  128

#endif /* _LINUX_FAPI */

/*!
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

#ifdef _LINUX_FAPI

typedef struct _fapex_sys_thread_controlT
{
    int             inited;
    pthread_mutex_t mutexT;
} fapex_sys_thread_controlT;

typedef struct _fapex_sys_thread_listT {
    int32_t         no;
    uint32_t       *stackBuffer;
    uint32_t        stackSize;
    uint32_t        priority;

    FAPEX_SYS_ThreadFunctionT function;

    void           *optArg;
    void           *optData;
    char_t          optName[FAPEX_SYS_THREAD_NAME_SIZE];
    int32_t         optStatus;
    pthread_t       thread_obj;
    pthread_attr_t  thread_attr;
    int32_t         wait_cnt;
    int32_t         lastError;
    uint32_t        actState;       // thread state, running, ready, blocked, etc.
    FAPEX_SYS_EventT   eventPtr;    // pointer to semaphore/mailbox event structure
    uint32_t        delayCount;     // delay in timer ticks, used if state is delayed

    pthread_mutex_t thread_mutex;
    pthread_cond_t  thread_cond;

} fapex_sys_thread_listT;

#endif /* _LINUX_FAPI */

/*!
*************************************************************************
** \brief open sub_id parameters for the Common driver.<br>
**        this is dummy internal structure for FAPEX_IOCCOM_xxx_OPEN_SUBID command.
**
** ioctl command type : #FAPEX_IOCCOM_xxx_OPEN_SUBID
**
*************************************************************************/
typedef struct {
    /*!
        sub id<br>
        \param[in]  sub_id  Device minor_no
        \param[out] sub_id  Device identifier
    */
    int32_t     sub_id;

} FAPEX_COMMON_OpenSubIdParamStrT;

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
FAPEXOPENRET_T fapex_COMMON_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags,
                                 FAPEXOPENARG3_T argptr, uint32_t cmd);
FAPEXIOCRET_T  fapex_COMMON_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd,
                                  FAPEXIOCARG3_T pArg);

#endif /* !_FAPEX_ABST_COMMON_PRIV_H */

