/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_mixer.h
** 
** \version     $Id: abst_libaudec_mixer.h 4190 2009-07-02 16:14:11Z spurke $
**
** \brief       mixer codec specific parameters.
**
** This file contains the declaration of specific parameters for the MIXER codec.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 Fujitsu Semiconductor Europe GmbH, All Rights Reserved.
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_MIXER_H
    #define _FAPEX_ABST_LIBAUDEC_MIXER_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the MIXER decoder.
    **
    ** This data structure covers all parameters which can be set for the MIXER
    ** decoder.
    ** Setting these parameters is optional. If not the codec applies the 
    ** mentioned defaults. 
    **
    ** \sa FAPEX_AUDEC_Ioctl/FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio mixer ID.
           \note This field must be set to FAPEX_AUDEC_CODEC_TYPE_MIXER.
        */
        uint32_t decoder;

        /*!
         * Lower part of the PTS where the fade and pan parameter should be applied
        */
		uint32_t pts_low;

	    /*!
         * Upper par of the PTS where the fade and pan parameter should be applied
        */
		uint32_t pts_high;

        /*!
         * This byte gives the amount how much the mainstream should be faded
         * down. The vaild range is 0-0xFF.
         * A value of 0x00 is no fade at all. Each increment fades the
         * main stream down about nominal 0.3 dB. 0xFE therefore representing
         * approximately 76.2 dB, whilst the value of 0xFF mutes the main
         * stream completely. (Mixer's default: 0dB)
        */
        uint32_t fade;

        /*!
         * \warning Not used yet!
         *
         * This byte allows the sub stream to be panned within the sound stage
         * of the main stream sound. Pan control allows to position the sub
         * stream at any preferred horizontal position within the sound field
		 * of the main stream. The valid range is 0-0xFF.
		 * A value of 0x00 represents centre front. Each
		 * increment represents about 1.4 clockwise looking down on the
		 * listener (see figure below) (Mixer's default: 0)
        */
        uint32_t pan;

    } FAPEX_AUDEC_MixerParamStrT;

#endif
