/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_mpeg.h
** 
** \version     $Id: drv_audec_mpeg.h 4190 2009-07-02 16:14:11Z spurke $
**
** \brief       MPEG codec specific parameters.
**
** This file contains the declaration of specific parameters for the MPEG codec.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 Fujitsu Semiconductor Europe GmbH, All Rights Reserved.
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_MPEG_H
    #define _FAPEX_ABST_LIBAUDEC_MPEG_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the MPEG decoder.
    **
    ** This data structure covers all parameters which can be set for the MPEG
    ** decoder.
    ** Setting these parameters is optional. If not the codec applies the 
    ** mentioned defaults. 
    **
    ** \sa FAPEX_AUDEC_Ioctl/FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio decoder ID.
           \note This field must be set to FAPEX_AUDEC_CODEC_TYPE_MPEG.
        */
        int32_t  decoder;
        
        /*!
           Minimum mute duration in [ms] after a decoding error occured. 
           The codec will only unmute after it received error free frames
           for the given period. (Codec's default: 1200ms)
        */
        uint32_t muteDuration;
        
        /*!
           Number of consecutive frames with same header which must sucessfully 
           decode before the decoder will start. (Codec's default: 1)
           This parameter allows applications to improve the audio error 
           recognition when the decoder is started with bad streams. 
        */
        uint32_t startInsyncFrameCount;

    } FAPEX_AUDEC_MpegParamStrT;

#endif
