/*!
*************************************************************************
**
** \file    fapex/abst_libttx.h
**
** \brief   abstraction library header file (TTX driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBTTX_H
#define _FAPEX_ABST_LIBTTX_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex TTX error code base value */
#define FAPEX_TTX_ERR_BASE              (FAPEX_TTX_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_TTX_ERR_BAD_PARAMETER     (FAPEX_TTX_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_TTX_ERR_OUT_OF_MEMORY     (FAPEX_TTX_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Invalid status. */
#define FAPEX_TTX_ERR_INVALID_STATUS    (FAPEX_TTX_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)


/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the TTX driver
**
** \sa FAPEX_TTX_Open
*************************************************************************/
typedef struct {
    /*!
        input Device minor number<br>
        (#FAPEXDEV_TSD_SUB_MINOR_FIRST..#FAPEXDEV_TSD_SUB_MINOR_LAST).
    */
    int32_t     inputId;

} FAPEX_TTX_OpenParamStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the TTX driver
*************************************************************************/
#define FAPEX_IOCCOM_TTX_OPEN   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_TTX + 0, FAPEX_TTX_OpenParamStrT)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_TTX_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_TTX_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_TTX_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBTTX_H */

