/*!
*************************************************************************
**
** \file    fapex/abst_libviout.h
**
** \brief   abstraction library header file (Video Output driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBVIOUT_H
#define _FAPEX_ABST_LIBVIOUT_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex VIOUT error code base value */
#define FAPEX_VIOUT_ERR_BASE             (FAPEX_VISCALE_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_VIOUT_ERR_BAD_PARAMETER    (FAPEX_VIOUT_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_VIOUT_ERR_OUT_OF_MEMORY    (FAPEX_VIOUT_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Invalid status. */
#define FAPEX_VIOUT_ERR_INVALID_STATUS   (FAPEX_VIOUT_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_VIOUT_ERR_SEMAPHORE_CREATE (FAPEX_VIOUT_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)


/*!
****************************************************************************
** \brief Enumeration type for system mode.
****************************************************************************
*/
typedef enum
{
    FAPEX_VIOUT_SYSTEM_MODE_UNKNOWN = 0,           //! Unknown system mode.
    FAPEX_VIOUT_SYSTEM_MODE_PAL,                   //! System mode is PAL.
    FAPEX_VIOUT_SYSTEM_MODE_NTSC,                  //! System mode is NTSC.
    FAPEX_VIOUT_SYSTEM_MODE_SECAM,                 //! System mode is SECAM.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x2397P,       //!< System mode is 1920x1080p at 23.97 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x24P,         //!< System mode is 1920x1080p at 24 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x25P,         //!< System mode is 1920x1080p at 25 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x2997P,       //!< System mode is 1920x1080p at 29.97 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x30P,         //!< System mode is 1920x1080p at 30 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x50I,         //!< System mode is 1920x1080i at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x50P,         //!< System mode is 1920x1080p at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x5994I,       //!< System mode is 1920x1080i at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x5994P,       //!< System mode is 1920x1080p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x60I,         //!< System mode is 1920x1080i at 60 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1920x1080x60P,         //!< System mode is 1920x1080p at 60 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x2397P,        //!< System mode is 1280x720p at 23.97 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x24P,          //!< System mode is 1280x720p at 24 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x25P,          //!< System mode is 1280x720p at 25 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x2997P,        //!< System mode is 1280x720p at 29.97 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x30P,          //!< System mode is 1280x720p at 30 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x50P,          //!< System mode is 1280x720p at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x5994P,        //!< System mode is 1280x720p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_1280x720x60P,          //!< System mode is 1280x720p at 60 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_720x576x50I,           //!< System mode is 720x576i at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_720x576x50P,           //!< System mode is 720x576p at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_720x480x5994I,         //!< System mode is 720x480i at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_720x480x5994P,         //!< System mode is 720x480p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_VGA_640x480x5994P,     //!< System mode is 640x480p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_VGA_800x600x5994P,     //!< System mode is 800x600p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_VGA_1024x768x5994P,    //!< System mode is 1024x768p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_VGA_1360x768x5994P,    //!< System mode is 1360x768p at 59.94 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_LCD_640x480x50P,       //!< System mode is 640x480p at 50 Hz.
    FAPEX_VIOUT_SYSTEM_MODE_LCD_1024x768x50P,      //!< System mode is 1024x768p at 50 Hz.
} FAPEX_VIOUT_SystemModeEnumT;

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/
/*!
****************************************************************************
** \brief Enumeration type for user defined aspect ratio of the display.
****************************************************************************
*/
typedef enum{
    /*!< Aspect ratio is not defined,so the aspect ratio shall be as it is
            of the stream after application of AFD later filling the whole active region. */
    FAPEX_VIOUT_DISPLAY_AR_NONE   = 0,
    //! Aspect ratio is 4/3
    FAPEX_VIOUT_DISPLAY_AR_4_3,
    //! Aspect ratio is 14/9
    FAPEX_VIOUT_DISPLAY_AR_14_9,
    //! Aspect ratio is 16/9 also called wide screen.
    FAPEX_VIOUT_DISPLAY_AR_16_9,
    //! Aspect ratio is 20/9 or also called 2.21
    FAPEX_VIOUT_DISPLAY_AR_221_100,
    //! Aspect ratio is 235/100 or also called 2.35
    FAPEX_VIOUT_DISPLAY_AR_235_100

}FAPEX_VISCALE_AspectRatioEnumT;

/*!
****************************************************************************
** \brief Enumeration type for user defined scaling methods.
****************************************************************************
*/
typedef enum {
    /*! User can specify the preferred active area.*/
    FAPEX_VISCALE_SM_NONE = 0,
    //! Scaling method specified as Pillar or letter box.
    FAPEX_VISCALE_SM_PLBOX,
    //! Scaling method specified as Fullscreen zoom.
    FAPEX_VISCALE_SM_FSZOOM,
    //! Scaling method specified as fullscreen center cut out.
    FAPEX_VISCALE_SM_FSCCO,
    //! Scaling method specified as let TV scale.
    FAPEX_VISCALE_SM_LTVS

}FAPEX_VISCALE_ScalingMethodEnumT;

/*!
*************************************************************************
** \brief Parameters that have changed are intimated by this structure.
*************************************************************************
*/
typedef struct {
    /*! Coded Aspect ratio of the frame*/
    FAPEX_VISCALE_AspectRatioEnumT        codedAspectRatio;
    /*! Display Aspect ratio*/
    FAPEX_VISCALE_AspectRatioEnumT        displayAspectRatio;
    /*! User defined scaling method*/
    FAPEX_VISCALE_ScalingMethodEnumT      scalingMethod;
    /*! Current and new value of the Afd*/
    uint32_t                              currAfd;
    /*! Current and new value of the Wss*/
    uint32_t                              currWss;
} FAPEX_VISCALE_ParamChangeInfoT;

/*!
*******************************************************************************
** \brief Enumerated types for Hnadling of Wss signal on analog SD encoder.
*******************************************************************************
*/
typedef enum {
    /*!   Unknown handling state of Wss Signal.*/
    FAPEX_VISCALE_UNKNOWN_WSS_STATE = (int32_t)-1,
    /*!   User defined Wss signal  is 8.*/
    FAPEX_VISCALE_WSS_ENABLE_4_3_FULLFORMAT = 8,
    /*!   User defined Wss signal  is 1.*/
    FAPEX_VISCALE_WSS_ENABLE_14_9_CENTRE = 1,
    /*!   User defined Wss signal  is 11.*/
    FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_CENTRE = 11,
    /*!   User defined Wss signal  is 13.*/
    FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_GT_CENTRE = 13,
    /*!   User defined Wss signal  is 14.*/
    FAPEX_VISCALE_WSS_ENABLE_14_9_FULLFORMAT = 14,
    /*!   User defined Wss signal  is 7.*/
    FAPEX_VISCALE_WSS_ENABLE_16_9_ANAMORPHIC = 7,
    /*!   User defined Wss signal  is 2.*/
    FAPEX_VISCALE_WSS_ENABLE_14_9_TOP = 2,
    /*!   User defined Wss signal  is 4.*/
    FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_TOP = 4,
    /*!   The Wss Signaling is disabled.*/
    FAPEX_VISCALE_WSS_DISABLE = 0

}FAPEX_VISCALE_WssStateEnumT;

/*!
*************************************************************************
** \brief HDMI Video parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_update_avi_info "FAPEX_IOCCOM_VIOUT_UPDATE_AVI_INFO",
**
** \sa FAPEX_VIOUT_Open, FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        HDMI Video Input Type<br>
        \param #FAPEX_HDMI_RGB     RGB
        \param #FAPEX_HDMI_YUV422  YUV422
    */
    uint32_t  inputType;
    /*!
        HDMI Video Input Type<br>
        \param #FAPEX_HDMI_RGB     RGB
        \param #FAPEX_HDMI_YUV422  YUV422
        \param #FAPEX_HDMI_YUV444  YUV444
    */
    uint32_t  outputType;
    /*!
        HDMI Scan information<br>
        \param #FAPEX_HDMI_SCAN_NO_DATA  No data
        \param #FAPEX_HDMI_OVERSCAN      overscanned display
        \param #FAPEX_HDMI_UNDERSCAN     underscanned display
    */
    uint32_t  scanInfo;
    /*!
        HDMI Active format Discription
    */
    uint32_t  activeFormat;

    /*!
        HDMI Picture aspect ratio
        \param #FAPEX_VIOUT_DISPLAY_AR_NONE     Aspect ratio is not defined,
        \param #FAPEX_VIOUT_DISPLAY_AR_4_3      Aspect ratio is set to 4/3
        \param #FAPEX_VIOUT_DISPLAY_AR_14_9     Aspect ratio is set to 14/9
        \param #FAPEX_VIOUT_DISPLAY_AR_16_9     Aspect ratio is set to 16/9.
        \param #FAPEX_VIOUT_DISPLAY_AR_221_100  Aspect ratio is 20/9 or also called 2.21
        \param #FAPEX_VIOUT_DISPLAY_AR_235_100  Aspect ratio is 235/100 or also called 2.35
    */
    uint32_t  pictureAr;

} FAPEX_VIOUT_HdmiParamStrT;

/*!
*************************************************************************
** \brief open parameters for the VIOUT driver
**
** \sa FAPEX_VIOUT_Open
*************************************************************************/
typedef struct {
    /*!
        Video Output Type enable flag<br>
        The value of enableFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_VIOUT_ENABLE_NORMAL  default output
        \param #FAPEX_VIOUT_ENABLE_HDMI    Enable HDMI output
    */
    uint32_t enableFlag;

    /*!
        output Device minor number for HD Encoder<br>
        (#FAPEXDEV_VIENC_SUB_MINOR_FIRST..#FAPEXDEV_VIENC_SUB_MINOR_LAST).
    */
    int32_t  outputId;

    /*!
        HDMI Video Input Type<br>
        \param #FAPEX_HDMI_RGB     RGB
        \param #FAPEX_HDMI_YUV422  YUV422
    */
    FAPEX_VIOUT_HdmiParamStrT   hdmiParams;

} FAPEX_VIOUT_OpenCmnParamStrT;

#define FAPEX_VIOUT_ENABLE_NORMAL    (0x0000) //!< default output
#define FAPEX_VIOUT_ENABLE_HDMI      (0x0001) //!< Enable HDMI output

/*!
*************************************************************************
** \brief open sub_id parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_open_subid "FAPEX_IOCCOM_VIOUT_OPEN_SUBID"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Video Output layer id<br>
        \param[in]  sub_id  Device minor_no (#FAPEXDEV_VIOUT_SUB_MINOR_FIRST .. #FAPEXDEV_VIOUT_SUB_MINOR_LAST).
        \param[out] sub_id  Device identifier for VIOUT layer.
    */
    int32_t     sub_id;
    /*!
        scaler type<br>
        \param #FAPEX_VIOUT_SCALER_VIDEO         Video scaler
        \param #FAPEX_VIOUT_SCALER_SD            SD scaler
        \param #FAPEX_VIOUT_SCALER_OSD1          OSD1 scaler
        \param #FAPEX_VIOUT_SCALER_OSD2          OSD2 scaler
        \param #FAPEX_VIOUT_SCALER_OSD3          OSD3 scaler
        \param #FAPEX_VIOUT_SCALER_OSD4          OSD4 scaler
        \param #FAPEX_VIOUT_SCALER_CURSOR_PLANE  Cursor Plane
        \param #FAPEX_VIOUT_SCALER_UNKNOWN       Unknown scaler
    */
    int32_t     scalerType;
    /*!
        topology type<br>
        \param #FAPEX_VIOUT_OVERLAY_UNKNOWN   Unknown state of topology descriptor.
        \param #FAPEX_VIOUT_OVERLAY_NO_OSD    None of the OSD layers are overlaid on SD output so all are on HD path.
        \param #FAPEX_VIOUT_OVERLAY_OSD1      OSD1 scaler overlaid on SD output.
        \param #FAPEX_VIOUT_OVERLAY_OSD2      OSD2 scaler overlaid on SD output.
        \param #FAPEX_VIOUT_OVERLAY_OSD3      OSD3 scaler overlaid on SD out.
        \param #FAPEX_VIOUT_OVERLAY_OSD4      OSD4 scaler overlaid on SD output.
        \param #FAPEX_VIOUT_OVERLAY_CURSOR    Cursor overlaid on SD output.
        \param #FAPEX_VIOUT_VO_BYPASS_DISABLE Configuration parameter for turning off VO scaler bypass mode.
        \param #FAPEX_VIOUT_VO_BYPASS_ENABLE  Configuration parameter for bypassing the VO scaler.
    */
    int32_t     topologyType;

} FAPEX_VIOUT_OpenSubParamStrT;

// scaler type
#define FAPEX_VIOUT_SCALER_UNKNOWN        (-1)          //!< Unknown scaler
#define FAPEX_VIOUT_SCALER_VIDEO            0           //!< Video scaler
#define FAPEX_VIOUT_SCALER_SD               1           //!< SD scaler
#define FAPEX_VIOUT_SCALER_WBU              2
#define FAPEX_VIOUT_SCALER_OSD1             3           //!< OSD1 scaler
#define FAPEX_VIOUT_SCALER_OSD2             4           //!< OSD2 scaler
#define FAPEX_VIOUT_SCALER_OSD3             5           //!< OSD1 scaler
#define FAPEX_VIOUT_SCALER_OSD4             6           //!< OSD2 scaler
#define FAPEX_VIOUT_SCALER_CURSOR_PLANE     7           //!< Cursor Plane

// topology type
#define FAPEX_VIOUT_OVERLAY_UNKNOWN   (-1)  //!< Unknown state of topology descriptor.
#define FAPEX_VIOUT_OVERLAY_NO_OSD      0   //!< None of the OSD layers are overlaid on SD output so all are on HD path.
#define FAPEX_VIOUT_OVERLAY_OSD1        1   //!< OSD1 scaler overlaid on SD output.
#define FAPEX_VIOUT_OVERLAY_OSD2        2   //!< OSD2 scaler overlaid on SD output.
#define FAPEX_VIOUT_OVERLAY_OSD3        3   //!< OSD3 scaler overlaid on SD out.
#define FAPEX_VIOUT_OVERLAY_OSD4        4   //!< OSD4 scaler overlaid on SD output.
#define FAPEX_VIOUT_OVERLAY_CURSOR      5   //!< Cursor overlaid on SD output.
#define FAPEX_VIOUT_VO_BYPASS_DISABLE   6   //!< Configuration parameter for turning off VO scaler bypass mode.
#define FAPEX_VIOUT_VO_BYPASS_ENABLE    7   //!< Configuration parameter for bypassing the VO scaler.

/*!
****************************************************************************
** \brief   Endianness specifiers
**
**          Pixels are processed 32-bit wise by the OSD hardware.
**          Within one 32-bit word, the pixel that is leftmost visible on the
**          display is per default stored in the most significant bits
**          of the 32-bit word.
**          One should also note that system memory access is always
**          little endian. That means that byte-aligned pointers will reference
**          the bytes in a 32-bit word from the least significant byte to the
**          most significant byte for ascending addresses.
**          Smilarly, 16-bit aligned pointers will first reference the 16 least
**          significant bits for the even address and the 16 most significant
**          bytes for the odd address.
**          The order in which the OSD hardware constructs 32-bit words from
**          groups of 4 bytes can be swapped by this setting.
****************************************************************************
*/
typedef enum {
    FAPEX_VISCALE_ENDIAN_DEFAULT = 0,
    FAPEX_VISCALE_ENDIAN_SWAP
}FAPEX_VISCALE_EndiannessT;
/*!
****************************************************************************
** \brief   Pixel order specifiers
**
**          Per default, the most significant bits store the leftmost and
**          the least significant bits store the rightmost pixel within a
**          32-bit word.
**          The pixel order within a 32-bit word can be reversed by this
**          option parameter.
****************************************************************************
*/
typedef enum {
    FAPEX_VISCALE_PIXEL_ORDER_DEFAULT = 0,
    FAPEX_VISCALE_PIXEL_ORDER_REVERSE
}FAPEX_VISCALE_PixelOrderT;
/*!
*************************************************************************
** \brief scaler parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_set_scaler_params "FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS",
** \ref viout_cmd_get_scaler_params "FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        display mode or scaler mode<br>
        \param #FAPEX_VIOUT_SCALER_MODE_NONE    Unknown Scaler mode
        \param #FAPEX_VIOUT_SCALER_MODE_PLBOX   Pillar or letter box
        \param #FAPEX_VIOUT_SCALER_MODE_FSZOOM  Fullscreen zoom
        \param #FAPEX_VIOUT_SCALER_MODE_FSCCO   fullscreen center cut out
        \param #FAPEX_VIOUT_SCALER_MODE_LTVS    let TV scale
        \param #FAPEX_VIOUT_OSD_MODE_32_24BIT   Tru colour 32 or 24 bit OSD
        \param #FAPEX_VIOUT_OSD_MODE_5551       Tru colour 16 bit 5551 OSD
        \param #FAPEX_VIOUT_OSD_MODE_4444       Tru colour 14 bit 444 OSD
        \param #FAPEX_VIOUT_OSD_MODE_16BIT      16 bit OSD
        \param #FAPEX_VIOUT_OSD_MODE_CLUT_4BIT  4 bit clut OSD
        \param #FAPEX_VIOUT_OSD_MODE_CLUT_8BIT  8 bit clut OSD
        \param #FAPEX_VIOUT_OSD_MODE_UNKNOWN    Unknown OSD mode
        \param #FAPEX_VIOUT_CURSOR_MODE_1BIT    1 bit Cursor
        \param #FAPEX_VIOUT_CURSOR_MODE_2BIT    2 bit Cursor
        \param #FAPEX_VIOUT_CURSOR_MODE_4BIT    4 bit Cursor
        \param #FAPEX_VIOUT_CURSOR_MODE_8BIT    8 bit Cursor
        \param #FAPEX_VIOUT_CURSOR_MODE_UNKNOWN Unknown Cursor mode
    */
    int32_t     mode;
    /*!
        OSD pixel order<br>
        \param #FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN  Unknown layout format of OSD in memory
        \param #FAPEX_VIOUT_OSD_LAYOUT_ARGB     RGB mode with alpha as most significant byte
        \param #FAPEX_VIOUT_OSD_LAYOUT_RGBA     RGB mode with alpha as least significant byte
        \param #FAPEX_VIOUT_OSD_LAYOUT_ACbYCr   CrYCb mode with alpha as most significant byte
        \param #FAPEX_VIOUT_OSD_LAYOUT_CbYCrA   CrYCb mode with alpha as least significant byte
    */
    int32_t     osdByteOrder;

    /*!
        Number of tiles in the horizontal direction in the cursor.
    */
    uint32_t    cpTileNumX;

    /*!
        Number of tiles in the vertical direction in the cursor.
    */
    uint32_t    cpTileNumY;

    /*!
        0:interlace, 1:prograssive
    */

    uint32_t    flagFrameField;

    /*!
        bits per pixel
    */
    uint32_t    bpp;

    /*!
        Input picture start pixel position used for cropping on video plane.<br>
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowX0;

    /*!
        Input picture start line position used for cropping on video plane.<br>
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowY0;

    /*!
        Input picture width in terms of luma pixels.<br>
        When this value is gotten by the \ref viout_cmd_get_scaler_params
        "FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS" command,
        this value is uncropped input picture width.
    */
    uint32_t    inputWindowWidth;

    /*!
        Input picture height in terms of luma lines.<br>
        When this value is gotten by the \ref viout_cmd_get_scaler_params
        "FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS" command,
        this value is uncropped input picture height.
    */
    uint32_t    inputWindowHeight;

    /*!
        Output picture start position in horizontal direction
    */
    uint32_t    outputWindowX0;

    /*!
        Output picture start position in vertical direction
    */
    uint32_t    outputWindowY0;

    /*!
        Output picture width in terms of luma pixels after scaling.<br>
        For Cursor plane it is equal to picture width as no scaling is
        supported on cursor plane.
    */
    uint32_t    outputWindowWidth;

    /*!
        Output picture height in terms of luma lines after scaling.<br>
        For Cursor plane it is equal to picture height as no scaling is
        supported on cursor plane.
    */
    uint32_t    outputWindowHeight;

    //! Defines the endianness scheme when reading 32-bit words.
    FAPEX_VISCALE_EndiannessT             endianness;
    //! Defines the order of the pixels within a 32-bit word.
    FAPEX_VISCALE_PixelOrderT             pixelOrder;

} FAPEX_VIOUT_ScalerParamStrT;

// Video scaler mode
#define FAPEX_VIOUT_SCALER_MODE_NONE     0
#define FAPEX_VIOUT_SCALER_MODE_PLBOX    1    //!< Pillar or letter box
#define FAPEX_VIOUT_SCALER_MODE_FSZOOM   2    //!< Fullscreen zoom
#define FAPEX_VIOUT_SCALER_MODE_FSCCO    3    //!< fullscreen center cut out
#define FAPEX_VIOUT_SCALER_MODE_LTVS     4    //!< let TV scale

// OSD/Cursor scaler mode
#define FAPEX_VIOUT_OSD_MODE_32_24BIT    0    //!< Tru colour 32 or 24 bit OSD
#define FAPEX_VIOUT_OSD_MODE_5551        1    //!< Tru colour 16 bit 5551 OSD
#define FAPEX_VIOUT_OSD_MODE_4444        2    //!< Tru colour 14 bit 444 OSD
#define FAPEX_VIOUT_OSD_MODE_16BIT       4    //!< 16 bit OSD
#define FAPEX_VIOUT_OSD_MODE_CLUT_4BIT  10    //!< 4 bit clut OSD
#define FAPEX_VIOUT_OSD_MODE_CLUT_8BIT  11    //!< 8 bit clut OSD
#define FAPEX_VIOUT_OSD_MODE_UNKNOWN    15    //!< Unknown OSD mode
#define FAPEX_VIOUT_CURSOR_MODE_1BIT     0    //!< 1 bit Cursor
#define FAPEX_VIOUT_CURSOR_MODE_2BIT     1    //!< 2 bit Cursor
#define FAPEX_VIOUT_CURSOR_MODE_4BIT     2    //!< 4 bit Cursor
#define FAPEX_VIOUT_CURSOR_MODE_8BIT     3    //!< 8 bit Cursor
#define FAPEX_VIOUT_CURSOR_MODE_UNKNOWN  4    //!< Unknown Cursor mode

// OSD pixel order type
#define FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN (-1)   //!< Unknown layout format of OSD in memory
#define FAPEX_VIOUT_OSD_LAYOUT_ARGB      1    //!< RGB mode with alpha as most significant byte
#define FAPEX_VIOUT_OSD_LAYOUT_RGBA      2    //!< RGB mode with alpha as least significant byte
#define FAPEX_VIOUT_OSD_LAYOUT_ACbYCr    3    //!< CrYCb mode with alpha as most significant byte
#define FAPEX_VIOUT_OSD_LAYOUT_CbYCrA    4    //!< CrYCb mode with alpha as least significant byte


/*!
*************************************************************************
** \brief alpha parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_set_alpha "FAPEX_IOCCOM_VIOUT_SET_ALPHA"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        alpha control mode<br>
        \param #FAPEX_VIOUT_PARAMETER_GET       Returns the parameters
        \param #FAPEX_VIOUT_PARAMETER_SET       Applies the parameters
        \param #FAPEX_VIOUT_PARAMETER_INCREASE  Increase by step size as specified
        \param #FAPEX_VIOUT_PARAMETER_DECREASE  decrease by step size as specified
        \param #FAPEX_VIOUT_PARAMETER_ON        Turns on a specified parameter or feature
        \param #FAPEX_VIOUT_PARAMETER_OFF       Turns off a specified parameter or feature
    */
    int32_t     mode;
    /*!
        alpha value (0..255)
    */
    uint32_t    alpha;

} FAPEX_VIOUT_AlphaParamStrT;

// parameter control mode
#define FAPEX_VIOUT_PARAMETER_GET       0   //!< Returns the parameters
#define FAPEX_VIOUT_PARAMETER_SET       1   //!< Applies the parameters
#define FAPEX_VIOUT_PARAMETER_INCREASE  2   //!< Increase by step size as specified
#define FAPEX_VIOUT_PARAMETER_DECREASE  3   //!< decrease by step size as specified
#define FAPEX_VIOUT_PARAMETER_ON        4   //!< Turns on a specified parameter or feature
#define FAPEX_VIOUT_PARAMETER_OFF       5   //!< Turns off a specified parameter or feature

/*!
*************************************************************************
** \brief fade parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_set_fading "FAPEX_IOCCOM_VIOUT_SET_FADING",
** \ref viout_cmd_get_fading "FAPEX_IOCCOM_VIOUT_GET_FADING"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        fade mode<br>
        \param #FAPEX_VIOUT_VIDEO_FADE_IN   For fading in video
        \param #FAPEX_VIOUT_VIDEO_FADE_OUT  For fading out video
    */
    int32_t   mode;

    /*!
        fading step size (0-255)
    */
    uint32_t  stepSize;

} FAPEX_VIOUT_FadingParamStrT;

#define FAPEX_VIOUT_VIDEO_FADE_IN   0   //!< For fading in video
#define FAPEX_VIOUT_VIDEO_FADE_OUT  1   //!< For fading out video

/*!
*************************************************************************
** \brief resize parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_set_resize "FAPEX_IOCCOM_VIOUT_SET_RESIZE"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Input picture start pixel position used for cropping on video plane.
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowX0;

    /*!
        Input picture start line position used for cropping on video plane.
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowY0;

    /*!
        Input picture width in terms of luma pixels used for cropping.
        For Cursor plane it is equal to picture width.
    */
    uint32_t    inputWindowWidth;

    /*!
        Input picture height in terms of luma lines used for cropping.
        For Cursor plane it is equal to picture height.
    */
    uint32_t    inputWindowHeight;

    /*!
        Output picture start position in horizontal direction
    */
    uint32_t    outputWindowX0;

    /*!
        Output picture start position in vertical direction
    */
    uint32_t    outputWindowY0;

    /*!
        Output picture width in terms of luma pixels after scaling.
        For Cursor plane it is equal to picture width as no scaling is
        supported on cursor plane.
    */
    uint32_t    outputWindowWidth;

    /*!
        Output picture height in terms of luma lines after scaling.
        For Cursor plane it is equal to picture height as no scaling is
        supported on cursor plane.
    */
    uint32_t    outputWindowHeight;

} FAPEX_VIOUT_ResizeParamStrT;

/*!
*************************************************************************
** \brief position parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_set_position "FAPEX_IOCCOM_VIOUT_SET_POSITION"
** \ref viout_cmd_get_position "FAPEX_IOCCOM_VIOUT_GET_POSITION"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Input picture start pixel position used for cropping on video plane.
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowX0;

    /*!
        Input picture start line position used for cropping on video plane.
        Cropping at top left corner is allowed for Video plane only,
        for OSD and Cursor it s set to 0 internally.
    */
    uint32_t    inputWindowY0;

    /*!
        Output picture start position in horizontal direction
    */
    uint32_t    outputWindowX0;

    /*!
        Output picture start position in vertical direction
    */
    uint32_t    outputWindowY0;

} FAPEX_VIOUT_PosParamStrT;

/*!
*************************************************************************
** \brief fill rectangle parameters for the VIOUT driver
**
** The related ioctl command type :
** \ref viout_cmd_fill_rectangle "FAPEX_IOCCOM_VIOUT_FILL_RECTANGLE"
**
** \sa FAPEX_VIOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        The colour of the rectangle.
        It must achieve the requirements for the counts of bits per pixel
        in the currently selected colour mode of the OSD/CP.
    */
    uint32_t    colour;

    /*!
        The x coordinate of the top left corner of the rectangle.
    */
    uint32_t    x0;

    /*!
        The y coordinate of the top left corner of the rectangle.
    */
    uint32_t    y0;

    /*!
        The width of the rectangle. The whole rectangle must be enclosed by the OSD/CP.
    */
    uint32_t    width;

    /*!
        The height of the rectangle. The whole rectangle must be enclosed by the OSD/CP.
    */
    uint32_t    height;

} FAPEX_VIOUT_FillRectParamStrT;


#define FAPEX_VIOUT_MAX_HRES              1920
#define FAPEX_VIOUT_MAX_VRES              1088
#define FAPEX_VIOUT_OSD1_MAX_LINE_BYTE    (960 * 8)
#define FAPEX_VIOUT_OSD2_MAX_LINE_BYTE    (960 * 8)	// change by tiger from 240
#define FAPEX_VIOUT_OSD3_MAX_LINE_BYTE    (240 * 8)
#define FAPEX_VIOUT_OSD4_MAX_LINE_BYTE    (240 * 8)
#define FAPEX_VIOUT_OSD1_MAX_SIZE         (FAPEX_VIOUT_OSD1_MAX_LINE_BYTE * FAPEX_VIOUT_MAX_VRES)
#define FAPEX_VIOUT_OSD2_MAX_SIZE         (FAPEX_VIOUT_OSD2_MAX_LINE_BYTE * FAPEX_VIOUT_MAX_VRES)
#define FAPEX_VIOUT_OSD3_MAX_SIZE         (FAPEX_VIOUT_OSD3_MAX_LINE_BYTE * FAPEX_VIOUT_MAX_VRES)
#define FAPEX_VIOUT_OSD4_MAX_SIZE         (FAPEX_VIOUT_OSD4_MAX_LINE_BYTE * FAPEX_VIOUT_MAX_VRES)
#define FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE   (4 * 1024)


/*!
*************************************************************************
** \brief ioctl specific command for the VIOUT driver
*************************************************************************/
#define FAPEX_IOCCOM_VIOUT_OPEN                 _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  0,  FAPEX_VIOUT_OpenCmnParamStrT)
#define FAPEX_IOCCOM_VIOUT_OPEN_SUBID           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_SUBID_VIOUT, FAPEX_VIOUT_OpenSubParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_PLANEORDER       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  1,  int32_t[6])
#define FAPEX_IOCCOM_VIOUT_GET_PLANEORDER       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  1,  int32_t[6])
#define FAPEX_IOCCOM_VIOUT_SET_BGCOLOUR         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  2,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_GET_BGCOLOUR         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  2,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_GET_SYSTEM_MODE      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  3,  int32_t)
#define FAPEX_IOCCOM_VIOUT_ENABLE               _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  4)
#define FAPEX_IOCCOM_VIOUT_DISABLE              _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  5)
#define FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  6,  FAPEX_VIOUT_ScalerParamStrT)
#define FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS    _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  6,  FAPEX_VIOUT_ScalerParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  7,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_SET_ALPHA            _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  8,  FAPEX_VIOUT_AlphaParamStrT)
#define FAPEX_IOCCOM_VIOUT_GET_ALPHA            _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT +  9,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_SET_FADING           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 10,  FAPEX_VIOUT_FadingParamStrT)
#define FAPEX_IOCCOM_VIOUT_GET_FADING           _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 10,  FAPEX_VIOUT_FadingParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_RESIZE           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 11,  FAPEX_VIOUT_ResizeParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_POSITION         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 12,  FAPEX_VIOUT_PosParamStrT)
#define FAPEX_IOCCOM_VIOUT_GET_POSITION         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 12,  FAPEX_VIOUT_PosParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_FILLCOLOUR       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 13,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_GET_FILLCOLOUR       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 13,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_FILL_RECTANGLE       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 14,  FAPEX_VIOUT_FillRectParamStrT)
#define FAPEX_IOCCOM_VIOUT_SET_CLUT             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 15,  uint32_t[256])
#define FAPEX_IOCCOM_VIOUT_GET_CLUT             _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 15,  uint32_t[256])
#define FAPEX_IOCCOM_VIOUT_SET_HILIGHT          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 16,  uint32_t)
#define FAPEX_IOCCOM_VIOUT_FLUSH_PLANE          _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 17)
#define FAPEX_IOCCOM_VIOUT_UPDATE_AVI_INFO      _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 18,  FAPEX_VIOUT_HdmiParamStrT)
#define FAPEX_IOCCOM_VIOUT_HDMI_RESTART         _IO (FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 19)
#define FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 20,  FAPEX_VISCALE_WssStateEnumT)
#define FAPEX_IOCCOM_VIOUT_SET_BRIGHTNESS       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 21,  int32_t)
#define FAPEX_IOCCOM_VIOUT_SET_CONTRAST         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 22,  int32_t)
#define FAPEX_IOCCOM_VIOUT_SET_SATURATION       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIOUT + 23,  int32_t)
/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T  FAPEX_VIOUT_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T   FAPEX_VIOUT_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T   FAPEX_VIOUT_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);
FAPEXMMPRET_T   FAPEX_VIOUT_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                                 FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                                 FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset);
FAPEXMUNMPRET_T FAPEX_VIOUT_Munmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBVIOUT_H */

