/*!
*****************************************************************************
**
** \file        sys_config_mb86h61_devkit.h
**
** \brief       Fujitsu DEVKIT specific board configuration for MB86H61.
**
** This file contains the settings for the Fujitsu DEVKIT board of MB86H61.
** It defines all available drivers and their required parameter settings.<br>
** In case of \b Linux environment, this file is included by sys_config.h
** with configuration of \b "make menuconfig".<br>
** In case of \b RTOS environment, this file is included by sys_config.h if
** the following is set within the main configuration file Config.mak:
** - DECODER   = MB86H61
** - BOARDNAME = mb86h61_dk01
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH<br>
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
#ifndef FAPEX_CONFIG_MB86H61_DEVKIT_H
    #define FAPEX_CONFIG_MB86H61_DEVKIT_H
    //***********************************************************************
    //***********************************************************************
    //** Define all needed hardware frequencies
    //***********************************************************************
    //***********************************************************************

    #define FAPEX_SYS_CLOCK_FREQUENCY  396000000
    #define FAPEX_SYS_TIMER_FREQUENCY  99000000
    #define FAPEX_SYS_UART_FREQUENCY   99000000

    //***********************************************************************
    //***********************************************************************
    //** Define the memory mapping for all hardware blocks
    //***********************************************************************
    //***********************************************************************
    #define DDR_MEMORY_START_1  0x20000000
    #define DDR_MEMORY_START_2  0x40000000

    #ifndef _LINUX_CONFIG
    #define DDR_MEMORY_BYTES_1  0x08000000
    #define DDR_MEMORY_BYTES_2  0x08000000
    #else
    #define DDR_MEMORY_BYTES_1  CONFIG_MEM_SIZE_RAMIFS
    #define DDR_MEMORY_BYTES_2  CONFIG_MEM_SIZE_RAMIFV
    #endif /* _LINUX_CONFIG */

    //Memories available on board
    #define FAPEX_SYS_MMAP_DDR_START                DDR_MEMORY_START_1 // fixed!
    #define FAPEX_SYS_MMAP_DDR2_START               DDR_MEMORY_START_2 // fixed!
    #define FAPEX_SYS_MMAP_DDR_BYTES                DDR_MEMORY_BYTES_1
    #define FAPEX_SYS_MMAP_DDR2_BYTES               DDR_MEMORY_BYTES_2

    #if (CONFIG_MEM_SIZE_RAMIFS != 0)
    #define PROGRAM_MEMORY_START                    DDR_MEMORY_START_1
    #else
    #define PROGRAM_MEMORY_START                    DDR_MEMORY_START_2
    #endif

    // Audio decoder memory's location and size.
    #define FAPEX_SYS_MMAP_ARC_APPLICATION_START    ARC_PROGRAM_LOAD_ADDRESS
    #define FAPEX_SYS_MMAP_ARC_APPLICATION_BYTES    0x00400000UL // 4 MB

    // Video decoder memory's location and size.
    #define FAPEX_SYS_MMAP_VIDEO_DECODER_START      (PROGRAM_MEMORY_START+0x00400000UL)
    #define FAPEX_SYS_MMAP_VIDEO_DECODER_BYTES      0x01100000UL // 17 MB

    // Frame memorys location.
    #if (CONFIG_MEM_SIZE_RAMIFS != 0)
    #define FAPEX_SYS_MMAP_VIDEO_FRAME_SIZE         (0x04000000UL)
    #define MEMORY_OFFSET_VIDEO                     0
    #else
    #define FAPEX_SYS_MMAP_VIDEO_FRAME_SIZE         CONFIG_FIRM_FRAME_BUFFER_SIZE
    #define MEMORY_OFFSET_VIDEO                     (FAPEX_SYS_MMAP_DDR2_BYTES - FAPEX_SYS_MMAP_VIDEO_FRAME_SIZE)
    #endif
    #define FAPEX_SYS_MMAP_VIDEO_FRAME_START        (FAPEX_SYS_MMAP_DDR2_START + MEMORY_OFFSET_VIDEO)

    // Video Scalers IPC memory's location and size of about 4 MB.
    #define FAPEX_SYS_MMAP_VIDEO_SCALER_START       (0xFFFFFFFFUL)
    #define FAPEX_SYS_MMAP_VIDEO_SCALER_SIZE        (0x00400000UL)

    //BM memory's location and size.
    #define FAPEX_SYS_MMAP_BM01_GLOBAL_OFFSET       FAPEX_SYS_MMAP_DDR_START
    #define FAPEX_SYS_MMAP_BM23_GLOBAL_OFFSET       FAPEX_SYS_MMAP_DDR_START

    #define FAPEX_SYS_MMAP_BM_I2S0_START          (PROGRAM_MEMORY_START+0x01500000UL)
    #define FAPEX_SYS_MMAP_BM_I2S0_BYTES          0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_I2S1_START          (PROGRAM_MEMORY_START+0x01510000UL)
    #define FAPEX_SYS_MMAP_BM_I2S1_BYTES          0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_I2S2_START          (PROGRAM_MEMORY_START+0x01520000UL)
    #define FAPEX_SYS_MMAP_BM_I2S2_BYTES          0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_I2S3_START          (PROGRAM_MEMORY_START+0x01530000UL)
    #define FAPEX_SYS_MMAP_BM_I2S3_BYTES          0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_SPDIF_START         (PROGRAM_MEMORY_START+0x01540000UL)
    #define FAPEX_SYS_MMAP_BM_SPDIF_BYTES         0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_TELETEXT_START      (PROGRAM_MEMORY_START+0x01550000UL)
    #define FAPEX_SYS_MMAP_BM_TELETEXT_BYTES      0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_AUDIO_STREAM_START  (PROGRAM_MEMORY_START+0x01560000UL)
    #define FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_SUBTITLE_START      (PROGRAM_MEMORY_START+0x01570000UL)
    #define FAPEX_SYS_MMAP_BM_SUBTITLE_BYTES      0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_AUDIO2_STREAM_START (PROGRAM_MEMORY_START+0x01580000UL)
    #define FAPEX_SYS_MMAP_BM_AUDIO2_STREAM_BYTES 0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX00_STREAM_START  (PROGRAM_MEMORY_START+0x01590000UL)
    #define FAPEX_SYS_MMAP_BM_MIX00_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX01_STREAM_START  (PROGRAM_MEMORY_START+0x015a0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX01_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX02_STREAM_START  (PROGRAM_MEMORY_START+0x015b0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX02_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX03_STREAM_START  (PROGRAM_MEMORY_START+0x015c0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX03_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX04_STREAM_START  (PROGRAM_MEMORY_START+0x015d0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX04_STREAM_BYTES  0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX00_0_START       (PROGRAM_MEMORY_START+0x015e0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX00_0_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX00_1_START       (PROGRAM_MEMORY_START+0x015f0000UL)
    #define FAPEX_SYS_MMAP_BM_MIX00_1_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX01_0_START       (PROGRAM_MEMORY_START+0x01600000UL)
    #define FAPEX_SYS_MMAP_BM_MIX01_0_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX01_1_START       (PROGRAM_MEMORY_START+0x01610000UL)
    #define FAPEX_SYS_MMAP_BM_MIX01_1_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX02_0_START       (PROGRAM_MEMORY_START+0x01620000UL)
    #define FAPEX_SYS_MMAP_BM_MIX02_0_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX02_1_START       (PROGRAM_MEMORY_START+0x01630000UL)
    #define FAPEX_SYS_MMAP_BM_MIX02_1_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX03_0_START       (PROGRAM_MEMORY_START+0x01640000UL)
    #define FAPEX_SYS_MMAP_BM_MIX03_0_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX03_1_START       (PROGRAM_MEMORY_START+0x01650000UL)
    #define FAPEX_SYS_MMAP_BM_MIX03_1_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX04_0_START       (PROGRAM_MEMORY_START+0x01660000UL)
    #define FAPEX_SYS_MMAP_BM_MIX04_0_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_MIX04_1_START       (PROGRAM_MEMORY_START+0x01670000UL)
    #define FAPEX_SYS_MMAP_BM_MIX04_1_BYTES       0x00010000UL // 64 KB
    #define FAPEX_SYS_MMAP_BM_SECTION_START       (PROGRAM_MEMORY_START+0x01680000UL)
    #define FAPEX_SYS_MMAP_BM_SECTION_BYTES       0x00100000UL // 1 MB
    #define FAPEX_SYS_MMAP_BM_SECTION_END         (FAPEX_SYS_MMAP_BM_SECTION_START+FAPEX_SYS_MMAP_BM_SECTION_BYTES)

    // Free available memory's location and size.
    #define FAPEX_SYS_MMAP_SPARE_RESERVED_START   (PROGRAM_MEMORY_START+0x01780000UL)
    #define FAPEX_SYS_MMAP_SPARE_RESERVED_BYTES   0x00000000UL // 0 KB

    // uncashed heap memory's location and size.
    #define FAPEX_SYS_MMAP_UNCASHED_HEAP_START    (PROGRAM_MEMORY_START+0x01780000UL)
    #define FAPEX_SYS_MMAP_UNCASHED_HEAP_SIZE     0x00100000UL // 1 MB

    // OSD/Cursor memory's location and size.
    #define FAPEX_SYS_MMAP_CURSOR_PLANE_START     (PROGRAM_MEMORY_START+0x01880000UL)
    #define FAPEX_SYS_MMAP_CURSOR_PLANE_BYTES     0x00004000UL // 16 KB

    #define FAPEX_SYS_MMAP_OSD_PLANE_START        (PROGRAM_MEMORY_START+0x01884000UL)
    #define FAPEX_SYS_MMAP_OSD_PLANE_BYTES        0x01400000UL // 21 MB
    #define FAPEX_SYS_MMAP_OSD_PLANE_END          (PROGRAM_MEMORY_START+0x02e00000UL)
    #define PROGRAM_LOAD_ADDRESS                  (PROGRAM_MEMORY_START+0x02e00000UL) // BASE + 46M

    #define FAPEX_SYS_MMAP_OSD1_PLANE_START       FAPEX_SYS_MMAP_OSD_PLANE_START
    #define FAPEX_SYS_MMAP_OSD1_PLANE_BYTES       FAPEX_SYS_MMAP_OSD_PLANE_BYTES // 21M for OSD1
    #if (CONFIG_MEM_SIZE_RAMIFS != 0)
    #define FAPEX_SYS_MMAP_OSD2_PLANE_START       0x47000000UL // (FAPEX_SYS_MMAP_OSD_PLANE_START+FAPEX_SYS_MMAP_OSD1_PLANE_BYTES)
    #define FAPEX_SYS_MMAP_OSD2_PLANE_BYTES       0x00800000UL // 8M  for OSD2
    #else
    #define FAPEX_SYS_MMAP_OSD2_PLANE_START       (PROGRAM_MEMORY_START+0x02d04000UL)
    #define FAPEX_SYS_MMAP_OSD2_PLANE_BYTES       0x000fc000UL // 1008 KB
    #endif

    #define FAPEX_SYS_MMAP_ARM_APPLICATION_START  PROGRAM_LOAD_ADDRESS
    #define FAPEX_SYS_MMAP_ARM_APPLICATION_BYTES  0x05200000UL // 82 MB

    /************************************************************************/
    /*the follow osd setting does not define in the sys_init_mb86h61_devkit.h */
    /************************************************************************/

#ifndef _LINUX_FAPI /* for drivers */
#ifndef _LINUX_CONFIG
    #define FAPEX_SYS_MMAP_OSD_BORDER_SIZE        0x00000400UL // 1 KB border
#else
    #ifndef CONFIG_MB86HXX_LARGEMEM
      #error Please check with "System Type ---> Large Memory Mapping" of make configuration!
    #endif

#if (CONFIG_FIRM_AREA_SIZE < (PROGRAM_LOAD_ADDRESS - PROGRAM_MEMORY_START))
  #error Please check the kernel memory size with "System Type ---> Memory Size for Video/Audio Firmware" of make configuration!
#endif

    #define FAPEX_SYS_MMAP_OSD_BORDER_SIZE        (1 << PAGE_SHIFT)
#endif /* !_LINUX_CONFIG */
#endif /* !_LINUX_FAPI */

    /* confirm the "drv_videc_priv.h"!! */
    #define FAPEX_SYS_MMAP_BM_VIDEO_STREAM_START  (FAPEX_SYS_MMAP_VIDEO_DECODER_START + 0x00900000UL)
    #define FAPEX_SYS_MMAP_BM_VIDEO_STREAM_BYTES  0x00800000UL // 8 MB

    /* setup OSD parameter */
    #define FAPEX_SYS_MMAP_OSD_BORDER_MASK        ~(FAPEX_SYS_MMAP_OSD_BORDER_SIZE - 1)

    //***********************************************************************
    //***********************************************************************
    //** Define all needed drivers
    //** - some might already be excluded from application
    //***********************************************************************
    //***********************************************************************

#ifndef _LINUX_CONFIG
    #define FAPI_SYS_INIT_UART         1
#else
    #define FAPI_SYS_INIT_UART         0
#endif /* !_LINUX_CONFIG */

    #define FAPI_SYS_INIT_TIMER        1
    #define FAPI_SYS_INIT_INTR         1
    #define FAPI_SYS_INIT_ICC          1
    #define FAPI_SYS_INIT_BM           1
    #define FAPI_SYS_INIT_FLASH        1
    #define FAPI_SYS_INIT_AUOUT        1
    #define FAPI_SYS_INIT_VIDEC        1
    #define FAPI_SYS_INIT_VISCALE      1

    /* switch for Lowlevel Drivers */
    #define FAPICORE_INIT_GPIO         1
    #define FAPICORE_INIT_DMA          1
    #define FAPICORE_INIT_FPC          1
    #define FAPICORE_INIT_UPI          1
    #define FAPICORE_INIT_CLKPWR       1
    #define FAPICORE_INIT_FLASH        1
    #define FAPICORE_INIT_BOOT         1
    #define FAPICORE_INIT_GPREG        1
    #define FAPICORE_INIT_HDMI         1
    #define FAPICORE_INIT_HDMICEC      0
    #define FAPICORE_INIT_I2C          1
    #define FAPICORE_INIT_ICC          1
    #define FAPICORE_INIT_INTR         1
    #define FAPICORE_INIT_IR           1
    #define FAPICORE_INIT_SSP          1
    #define FAPICORE_INIT_TIMER        1
    #define FAPICORE_INIT_BLTE         1
    #define FAPICORE_INIT_ADC          1
    #define FAPICORE_INIT_PV           1
    #define FAPICORE_INIT_CEC          0

#ifndef _LINUX_CONFIG
    #define FAPICORE_INIT_MMU          1
    #define FAPICORE_INIT_ETH          0
    #define FAPICORE_INIT_UART         1
    #define FAPICORE_INIT_USB          0
#else
    #define FAPICORE_INIT_MMU          0
    #define FAPICORE_INIT_ETH          0
    #define FAPICORE_INIT_UART         0
    #define FAPICORE_INIT_USB          0
#endif /* !_LINUX_CONFIG */

    /* BM HCB setting */
    #define FAPEX_HCB_BLK_I2S       0 /* BM0 */
    #define FAPEX_HCB_BLK_SPDIF     0 /* BM0 */
    #define FAPEX_HCB_BLK_TTX       0 /* BM0 */
    #define FAPEX_HCB_BLK_AUDEC     0 /* BM0 */
    #define FAPEX_HCB_BLK_VIDEC     0 /* BM0 */

    //***********************************************************************
    //***********************************************************************
    //** Set board specific parameters
    //** There must be a macro for each driver with board specific settings.
    //** This macro has to be called by FAPI_SYS_SET_BOARD_PARAMS below.
    //***********************************************************************
    //***********************************************************************

    #if FAPI_SYS_INIT_UART == 1
        #define FAPI_SYS_UART_SET_BOARD_PARAMS                          \
            uart_funcStr.SetClockFrequency( FAPEX_SYS_UART_FREQUENCY ); \
            uart_funcStr.SetBaudrate( 0, FAPI_UART_BAUDRATE_115200 );   \
            uart_funcStr.SetDataBits( 0, FAPI_UART_DATABITS_8 );        \
            uart_funcStr.SetStopBits( 0, FAPI_UART_STOPBITS_1 );        \
            uart_funcStr.SetParity( 0, FAPI_UART_PARITY_NONE );         \
            uart_funcStr.SetFlowControl( 0, FAPI_UART_FLOWCTRL_NONE );  \
            uart_funcStr.SetStdioBlockIndex( 0 );
    #else
        #define FAPI_SYS_UART_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_TIMER == 1
        #define FAPI_SYS_TIMER_SET_BOARD_PARAMS                       \
            timer_funcStr.SetClockFrequency( FAPEX_SYS_TIMER_FREQUENCY );
    #else
        #define FAPI_SYS_TIMER_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_INTR == 1
#ifndef _LINUX_CONFIG
        #define FAPI_SYS_INTR_SET_BOARD_PARAMS                                           \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER0      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER1      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER2      , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_UART        , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_BLTE        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SATA        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SDIO        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SSP         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_ICC         , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_FPC_IR      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_I2C         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_AMCPU_AXP   , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_GPIO        , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_DMA         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_HDMAC       , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_USB         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_ETH         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_AUDIO       , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_BM          , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TSD         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO0         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO1         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO2_HDMI_CEC, FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_POLLING_MODE, FAPI_INTR_RECEIVER_ARM );  \
                                                                                         \
            intr_funcStr.SetPriority( FAPI_INTR_TIMER0   , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_TIMER1   , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_TIMER2   , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_UART0    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_UART1    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_SSP0     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_SSP1     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_ICC0     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_ICC1     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_FPC_IR   , FAPI_INTR_PRIORITY_LOW  );    \
            intr_funcStr.SetPriority( FAPI_INTR_I2C0     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_I2C1     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_I2C2     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_AMCPU_AXP, FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_GPIO     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_DMA      , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_USB      , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_ETH      , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_AUDIO    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_BM01     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_BM23     , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_TSD01    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_TSD23    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_DISP0    , FAPI_INTR_PRIORITY_HIGHEST ); \
            intr_funcStr.SetPriority( FAPI_INTR_DISP1    , FAPI_INTR_PRIORITY_HIGHEST ); \
            intr_funcStr.SetPriority( FAPI_INTR_DISP2    , FAPI_INTR_PRIORITY_HIGH );    \
            intr_funcStr.SetPriority( FAPI_INTR_POLLING_MODE, FAPI_INTR_PRIORITY_HIGHEST );
#else  /*_LINUX_CONFIG*/
        #define FAPI_SYS_INTR_SET_BOARD_PARAMS                                           \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER0      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER1      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TIMER2      , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_UART        , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_BLTE        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SATA        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SDIO        , FAPI_INTR_RECEIVER_NONE ); \
            intr_funcStr.SetReceiver( FAPI_INTR_SSP         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_ICC         , FAPI_INTR_RECEIVER_ARM/*FAPI_INTR_RECEIVER_NONE*/ ); \
            intr_funcStr.SetReceiver( FAPI_INTR_FPC_IR      , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_I2C         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_AMCPU_AXP   , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_GPIO        , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_DMA         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_HDMAC       , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_USB         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_ETH         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_AUDIO       , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_BM          , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_TSD         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO0         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO1         , FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_VO2_HDMI_CEC, FAPI_INTR_RECEIVER_ARM  ); \
            intr_funcStr.SetReceiver( FAPI_INTR_POLLING_MODE, FAPI_INTR_RECEIVER_ARM  );
#endif /* !_LINUX_CONFIG */

 #endif  /*FAPI_SYS_INIT_INTR*/


    #if FAPI_SYS_INIT_ICC == 1
        #define FAPI_SYS_ICC_SET_BOARD_PARAMS              \
            icc_funcStr.SetClkDivider( FAPI_ICC0, 1 );     \
            icc_funcStr.SetResetInverted(FAPI_ICC0,0);     \
            icc_funcStr.SetVccInverted( FAPI_ICC0, 1 ); //if used 8004 this reset invert(SetResetInverted) must remove .if used 74HC145D this reset invert must used
    #else
        #define FAPI_SYS_ICC_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_BM == 1
        #define FAPI_SYS_BM_SET_BOARD_PARAMS \
            bm_funcStr.SetBufferPurpose(FAPI_BM_I2S0_BUFFER, FAPI_BM0,                                                 \
                                        FAPEX_SYS_MMAP_BM_I2S0_START, FAPEX_SYS_MMAP_BM_I2S0_BYTES);                   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_I2S1_BUFFER, FAPI_BM0,                                                 \
                                        FAPEX_SYS_MMAP_BM_I2S1_START, FAPEX_SYS_MMAP_BM_I2S1_BYTES);                   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_I2S2_BUFFER, FAPI_BM0,                                                 \
                                        FAPEX_SYS_MMAP_BM_I2S2_START, FAPEX_SYS_MMAP_BM_I2S2_BYTES);                   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_I2S3_BUFFER, FAPI_BM0,                                                 \
                                        FAPEX_SYS_MMAP_BM_I2S3_START, FAPEX_SYS_MMAP_BM_I2S3_BYTES);                   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_SPDIF_BUFFER, FAPI_BM0,                                                \
                                        FAPEX_SYS_MMAP_BM_SPDIF_START, FAPEX_SYS_MMAP_BM_SPDIF_BYTES);                 \
            bm_funcStr.SetBufferPurpose(FAPI_BM_TELETEXT_BUFFER, FAPI_BM0,                                             \
                                        FAPEX_SYS_MMAP_BM_TELETEXT_START, FAPEX_SYS_MMAP_BM_TELETEXT_BYTES);           \
            bm_funcStr.SetBufferPurpose(FAPI_BM_AUDIO_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_AUDIO_STREAM_START, FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_SPDIF_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_AUDIO2_STREAM_START, FAPEX_SYS_MMAP_BM_AUDIO2_STREAM_BYTES); \
            bm_funcStr.SetBufferPurpose(FAPI_BM_VIDEO_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_VIDEO_STREAM_START, FAPEX_SYS_MMAP_BM_VIDEO_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX00_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_MIX00_STREAM_START, FAPEX_SYS_MMAP_BM_MIX00_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX01_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_MIX01_STREAM_START, FAPEX_SYS_MMAP_BM_MIX01_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX02_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_MIX02_STREAM_START, FAPEX_SYS_MMAP_BM_MIX02_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX03_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_MIX03_STREAM_START, FAPEX_SYS_MMAP_BM_MIX03_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX04_STREAM_BUFFER, FAPI_BM0,                                         \
                                        FAPEX_SYS_MMAP_BM_MIX04_STREAM_START, FAPEX_SYS_MMAP_BM_MIX04_STREAM_BYTES);   \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX00_0_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX00_0_START, FAPEX_SYS_MMAP_BM_MIX00_0_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX00_1_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX00_1_START, FAPEX_SYS_MMAP_BM_MIX00_1_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX01_0_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX01_0_START, FAPEX_SYS_MMAP_BM_MIX01_0_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX01_1_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX01_1_START, FAPEX_SYS_MMAP_BM_MIX01_1_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX02_0_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX02_0_START, FAPEX_SYS_MMAP_BM_MIX02_0_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX02_1_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX02_1_START, FAPEX_SYS_MMAP_BM_MIX02_1_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX03_0_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX03_0_START, FAPEX_SYS_MMAP_BM_MIX03_0_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX03_1_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX03_1_START, FAPEX_SYS_MMAP_BM_MIX03_1_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX04_0_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX04_0_START, FAPEX_SYS_MMAP_BM_MIX04_0_BYTES);             \
            bm_funcStr.SetBufferPurpose(FAPI_BM_MIX04_1_BUFFER, FAPI_BM0,                                              \
                                        FAPEX_SYS_MMAP_BM_MIX04_1_START, FAPEX_SYS_MMAP_BM_MIX04_1_BYTES);

    #else
        #define FAPI_SYS_BM_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_FLASH == 1
        #define FAPI_SYS_FLASH_SET_BOARD_PARAMS \
            flash_funcStr.SetEnable(FAPI_FLASH_DEVICE_PFLASH, 0);   \
            flash_funcStr.SetEnable(FAPI_FLASH_DEVICE_SFLASH, 1);   \
            flash_funcStr.SetUpiIndex(FAPI_FLASH_DEVICE_PFLASH, 0);
    #else
        #define FAPI_SYS_FLASH_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_AUOUT == 1
        #define FAPI_SYS_AUOUT_SET_BOARD_PARAMS                  \
            auout_funcStr.SetSpeakers(FAPI_AUOUT_I2S_0,          \
                                      FAPI_AUOUT_SPEAKER_LEFT,   \
                                      FAPI_AUOUT_SPEAKER_RIGHT); \
            auout_funcStr.SetInternalDac(FAPI_AUOUT_I2S_0);
    #else
        #define FAPI_SYS_AUOUT_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_VIDEC == 1
        #define FAPI_SYS_VIDEC_SET_BOARD_PARAMS                        \
            videc_funcStr.SetBoardMemory(FAPEX_SYS_MMAP_DDR_START,                          \
                                         FAPEX_SYS_MMAP_DDR_BYTES,                          \
                                         FAPEX_SYS_MMAP_DDR2_START,                         \
                                         FAPEX_SYS_MMAP_DDR2_BYTES,                         \
                                         FAPEX_SYS_MMAP_DDR2_START + MEMORY_OFFSET_VIDEO,   \
                                         FAPEX_SYS_MMAP_DDR2_BYTES - MEMORY_OFFSET_VIDEO);  \
            videc_funcStr.PreInit(FAPI_VIDEC_DECODER_NUMBER_0,        \
                                  FAPEX_SYS_MMAP_VIDEO_DECODER_START, \
                                  FAPEX_SYS_MMAP_VIDEO_FRAME_START);
    #else
        #define FAPI_SYS_VIDEC_SET_BOARD_PARAMS
    #endif

    #if FAPI_SYS_INIT_VISCALE == 1
        #define FAPI_SYS_VISCALE_SET_BOARD_PARAMS                 \
            viscale_funcStr.SetIpcMemory(FAPEX_SYS_MMAP_VIDEO_SCALER_START);
    #else
        #define FAPI_SYS_VISCALE_SET_BOARD_PARAMS
    #endif

#if 0
    #define FAPI_SYS_SET_BOARD_PARAMS    \
        atexit( CPU_Polling );           \
        FAPI_SYS_UART_SET_BOARD_PARAMS   \
        FAPI_SYS_TIMER_SET_BOARD_PARAMS  \
        FAPI_SYS_GPIO_SET_BOARD_PARAMS   \
        FAPI_SYS_INTR_SET_BOARD_PARAMS   \
        FAPI_SYS_BM_SET_BOARD_PARAMS     \
        FAPI_SYS_FLASH_SET_BOARD_PARAMS  \
        FAPI_SYS_AUOUT_SET_BOARD_PARAMS  \
        FAPI_SYS_VIDEC_SET_BOARD_PARAMS

#endif


#endif /* FAPEX_CONFIG_MB86H61_DEVKIT_H */
