/*!
*************************************************************************
**
** \file    abst_libboardifc.c
**
** \brief   abstraction library source file (BOARDIFC driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"
#include "fapex/abst_libboardifc.h"
int32_t fd_bd = -1;
char_t Manufact[128];
char_t    Model[128];
FAPEX_BOARD_HwPlatformInfosStrT hwInfo={
    0,//!< MagicAndVersion
    Manufact,                   //!< pHwManufactString
    Model,                      //!< pHwModelString
    NULL,                       //!< pOptionalAdditionalInfo
    0,                          //!< flagUsbHostOnly
    0,  //!< typeOfFlashAvailable
    0,                          //!< AudioConfigurationInfoA
    0,                          //!< AudioConfigurationInfoB
    2,                          //!< numberOfAvlFrontEnds
    0,                          //!< numberOfAvlCiSlots
    
    {
        { 0,  //!< defaultFeType
          0, //!< tsPathInputConfig
          0,                        //!< flagCiPathAval
          0 },                      //!< inputOptionsCiPath
        { 0,  //!< defaultFeType
          0, //!< tsPathInputConfig
          0,                        //!< flagCiPathAval
          0 }                       //!< inputOptionsCiPath
    },
    0,           //!< flagHardDiskInterface
    0,                              //!< flgDdr2MemoryInfos
    0           //!< video/scaler optimize info
};

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (NULL)
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_BOARDIFC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    fd_bd = fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_BOARDIFC_OPEN);
    return fd_bd;
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_BOARDIFC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_BOARDIFC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_BOARDIFC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor boardifc_cmd_get_chip_id
** \li #FAPEX_IOCCOM_CHIP_ID<br>
**      get the chip infomation.<br>
**         
** \anchor boardifc_cmd_open
** \li #FAPEX_IOCCOM_BOARDIFC_OPEN<br>
**      open the boardifc.<br>
**
** \anchor boardifc_cmd_get_boardinfo
** \li #FAPEX_IOCCOM_BOARDIFC_GETBOARDINFO<br>
**      get board infomation.<br>
**      Set board info to \e pArg.
**
** \anchor boardifc_cmd_set_init_detect_func
** \li #FAPEX_IOCCOM_BOARDIFC_INIT_DETECT_FUNCTION<br>
**      set board detect function.<br>
**      Execute the board init dection function.
**
** \anchor boardifc_cmd_set_platform_support
** \li #FAPEX_IOCCOM_PLATFORM_SUPPORT<br>
**      register board information.<br>
**      register  HW-platform board-information.
**
** \anchor boardifc_cmd_set_frontend
** \li #FAPEX_IOCCOM_FRONTEND_SUPPORT<br>
**      set frontend.<br>
**      register frontend information.<br>
**
** \anchor boardifc_cmd_set_platform_support_ext
** \li #FAPEX_IOCCOM_PLATFORM_SUPPORT_EXT<br>
**      execute the board support function.<br>
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_BOARDIFC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}


