/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_fe_demo/src/main.c
**
** \version     $Id$
**
** \brief       Demo Application for the FE subsystem
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2009-2010 by Fujitsu Semiconductor WSC
**
********************************************************************************
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fapex/abst_types.h>
#include "fapex/sys_config.h"
#include "fapex/abst_libgpio.h"
#include "fapex/abst_libicc.h"
#include "fapex/abst_lib.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */

#define APP_VERSION      "1.0.0"

#define APP_SHELL_THREAD_STACKSIZE         2048
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"


#define APP_CommandStringSize   256


#define APP_CardIndex           0
#define APP_ATRSize             256
#define APP_HISTSize            256
#define APP_ICCCardClock        4515000
#define APP_ICCParErrResendFlag 1
#define APP_ICCDataBit          8

static FAPEX_SYS_ThreadT appShellThreadHandle;


static int32_t fd_ICC0   = -1;


static FAPEX_ICC_OpenParamStrT        iccParamStrT =
{
/*protocolMode*/    0,
};
static FAPEX_ICC_OpenSubIdParamStrT   iccSbuIdParamStrT =
{
/*sub_id*/          FAPEXDEV_ICC_SUB_MINOR_FIRST,
/*cardIndex*/       APP_CardIndex,
/*atrSize */         APP_ATRSize,
/*histSize */        APP_HISTSize,
/*cardInCallback*/   NULL,
/*cardOutCallback*/  NULL,
};
static FAPEX_ICC_IOportParamStrT     iccIOprotParamStrT =
{
/*cardClock*/       APP_ICCCardClock,
/*baudRate*/       APP_ICCCardClock / 372,
/*convention*/      FAPEX_ICC_CONVENTION_DIRECT,
/*parity*/          FAPEX_ICC_PARITY_EVEN,
/*parityErrResend*/  APP_ICCParErrResendFlag,
/*dataBits*/        APP_ICCDataBit,
/*flow control*/     FAPEX_ICC_FLOWCONTROL_OFF,
};

static uint8_t histBytes[256];
static uint8_t atrBuffer[256];

static  FAPEX_ICC_AtrParamStrT      iccATRParamStrT =
{
/*convention*/     0,
/*protocol*/       0,
/*FI*/            0,
/*DI*/            0,
/*PI1*/           0,
/*PI2*/           0,
/*II*/            0,
/*N*/            0,
/*XI*/            0,
/*UI*/            0,
/**histBuffer*/     histBytes,
/*histBytes*/      256,
/**atrBuffer*/     atrBuffer,
/*atrBytes*/      256,
};


static uint8_t  SendDataCMD[] = {0x80,0x46,0x00,0x00,0x04,0x07,0x00,0x00,0x08};
static uint8_t  ReadDataCMD[] = {0x00,0xC0,0x00,0x00,0x08,};

static void appShellThreadFunction(void* optArg);


#ifdef _LINUX_FAPI
//***************************************************************************
#if 0       /* not use, remove warning */
static void print_usage(char *name)
{
    FAPI_SYS_PRINT_MSG("usage: %s [-s hd_mode (sd_mode)]\n", name);
}
#endif
#endif


/**
*******************************************************************************
**
** \brief Register a new command.
**
** This function registers the given command to the internal command table.
** An already existing command will be unregistered first and replaced by the
** given parameters. The buffers for the brief and the description texts will
** be dynamically allocated to avoid buffer overflows.
**
** \param command     The command (string) to register.
** \param brief       A short (one line) help message.
** \param description The full command description, maybe NULL.
** \param function    The function to be called when the user enters the
**                    "command" (with optional parameters), this functions
**                    must accept the argc, argv argument pair for additional
**                    optional parameters [strings].
**
*******************************************************************************
*/

static int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler, command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}

static uint32_t APP_GetStatusCommand( int32_t argc, char_t* argv[] )
{
    FAPEXOPENRET_T retVal = 0;
    uint32_t  CardStatus  = 0;

    /*get smart card status*/
    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_STATUS, (FAPEXIOCARG3_T)&CardStatus );
    if ( retVal >= 0 )
    {
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_GET_STATUS Success CardStatus = 0x%x\n",CardStatus);
    }
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_GET_STATUS Fail\n");

    return 1;
}

static uint32_t APP_ColdResetCommand( int32_t argc, char_t* argv[] )
{
    FAPEXOPENRET_T retVal      = 0;
    int32_t        cnt         = 0;

    /*reset smart card*/
    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, FAPEX_IOCCOM_RESET, FAPEX_ICC_RESET_MODE_COLD );
    if ( retVal == FAPEX_ICC_STATUS_FINISHED_RESET_COLD )
    {
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_ICC_RESET_MODE_COLD Success\n");
        retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_ATR, (FAPEXIOCARG3_T)&iccATRParamStrT );
        if ( retVal >= 0 )
        {
            FAPI_SYS_PRINT_MSG("========>Cold Reset ATR Numbern:%d\n",iccATRParamStrT.atrBytes);
            for ( cnt=0; cnt<iccATRParamStrT.atrBytes; cnt++ )
            {
                FAPI_SYS_PRINT_MSG("0x%02x,",iccATRParamStrT.atrBuffer[cnt]);
            }
            FAPI_SYS_PRINT_MSG("\n");
        }
        else
            FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_GET_ATR Fail\n");
    }
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_ICC_RESET_MODE_COLD Fail retVal =%d\n",retVal);

    return 1;
}

static uint32_t APP_WarmResetCommand( int32_t argc, char_t* argv[] )
{
    FAPEXOPENRET_T retVal = 0;
    int32_t  cnt          = 0;

    /*reset smart card*/
    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, FAPEX_IOCCOM_RESET, FAPEX_ICC_RESET_MODE_WARM );
    if ( retVal == FAPEX_ICC_STATUS_FINISHED_RESET_WARM )
    {
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_ICC_RESET_MODE_WARM Success\n");
        retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_ATR, (FAPEXIOCARG3_T)&iccATRParamStrT );
        if ( retVal >= 0 )
        {
            FAPI_SYS_PRINT_MSG("========>Warm Reset ATR Numbern:%d\n",iccATRParamStrT.atrBytes);
            for ( cnt=0; cnt<iccATRParamStrT.atrBytes; cnt++ )
            {
                FAPI_SYS_PRINT_MSG("0x%02x,",iccATRParamStrT.atrBuffer[cnt]);
            }
            FAPI_SYS_PRINT_MSG("\n");
        }
        else
            FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_GET_ATR Fail\n");
    }
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_ICC_RESET_MODE_WARM Fail retVal =%d\n",retVal);

    return 1;
}

static uint32_t APP_SendCMDCommand( int32_t argc, char_t* argv[] )
{
    FAPEXOPENRET_T           retVal = 0;
    FAPEX_ICC_DataParamStrT  iccDataParamStrT = {NULL,0,NULL,0};
    uint8_t                  sendCMDBuf[256]  = {0};
    uint8_t                  sendSize         = 0;
    uint8_t                  readBuf[256]     = {0};
    uint32_t                 cnt = 0;
    int32_t                 tmp;

    sendSize = argc-1;
    for ( cnt=0;cnt<argc;cnt++ )
    {
        sscanf(argv[cnt+1], "%x", &tmp);
        sendCMDBuf[cnt] = (uint8_t)tmp;
    }

    FAPI_SYS_PRINT_MSG(">>>Send Command:\n");
    for ( cnt=0;cnt<sendSize;cnt++ )
        FAPI_SYS_PRINT_MSG("0x%02x,",sendCMDBuf[cnt]);

    iccDataParamStrT.sendBuffer    = sendCMDBuf;
    iccDataParamStrT.sendSize      = sizeof(sendCMDBuf);
    iccDataParamStrT.receiveBuffer = readBuf;
    iccDataParamStrT.receiveSize   = sizeof(readBuf);

    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_DATA, (FAPEXREADARG3_T)&iccDataParamStrT );
    if ( retVal >= 0 )
    {
        FAPI_SYS_PRINT_MSG("\n========>receiveSize=%d\n",iccDataParamStrT.receiveSize);
        for ( cnt=0;cnt<iccDataParamStrT.receiveSize;cnt++ )
        {
            FAPI_SYS_PRINT_MSG("0x%02x,",iccDataParamStrT.receiveBuffer[cnt]);
        }
        FAPI_SYS_PRINT_MSG("\n");
    }

    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_GET_DATA Fail retVal =%d\n",retVal);

    return 1;
}

static uint32_t APP_SendCMDCommand2( int32_t argc, char_t* argv[] )
{
    FAPEXOPENRET_T           retVal = 0;
    FAPEX_ICC_DataParamStrT  iccDataParamStrT = {NULL,0,NULL,0};
    uint8_t                  readBuf[256]     = {0};
    uint32_t                 cnt = 0;


    FAPI_SYS_PRINT_MSG(">>>Send Command:\n");
    for ( cnt=0;cnt<sizeof(SendDataCMD);cnt++ )
        FAPI_SYS_PRINT_MSG("0x%02x,",SendDataCMD[cnt]);

    iccDataParamStrT.sendBuffer    = SendDataCMD;
    iccDataParamStrT.sendSize      = sizeof(SendDataCMD);
    iccDataParamStrT.receiveBuffer = readBuf;
    iccDataParamStrT.receiveSize   = sizeof(readBuf);

    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_DATA, (FAPEXREADARG3_T)&iccDataParamStrT );
    if ( retVal >= 0 )
    {
        FAPI_SYS_PRINT_MSG("\n========>receiveSize=%d\n",iccDataParamStrT.receiveSize);
        for ( cnt=0;cnt<iccDataParamStrT.receiveSize;cnt++ )
        {
            FAPI_SYS_PRINT_MSG("0x%02x,",iccDataParamStrT.receiveBuffer[cnt]);
        }
        FAPI_SYS_PRINT_MSG("\n");
    }
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_GET_DATA Fail retVal =%d\n",retVal);


    FAPI_SYS_PRINT_MSG(">>>Send Command:\n");
    for ( cnt=0;cnt<sizeof(ReadDataCMD);cnt++ )
        FAPI_SYS_PRINT_MSG("0x%02x,",ReadDataCMD[cnt]);

    iccDataParamStrT.sendBuffer    = ReadDataCMD;
    iccDataParamStrT.sendSize      = sizeof(ReadDataCMD);
    iccDataParamStrT.receiveBuffer = readBuf;
    iccDataParamStrT.receiveSize   = sizeof(readBuf);

    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_DATA, (FAPEXREADARG3_T)&iccDataParamStrT );
    if ( retVal >= 0 )
    {
        FAPI_SYS_PRINT_MSG("\n========>receiveSize=%d\n",iccDataParamStrT.receiveSize);
        for ( cnt=0;cnt<iccDataParamStrT.receiveSize;cnt++ )
        {
            FAPI_SYS_PRINT_MSG("0x%02x,",iccDataParamStrT.receiveBuffer[cnt]);
        }
        FAPI_SYS_PRINT_MSG("\n");
    }
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_GET_DATA Fail retVal =%d\n",retVal);

    return 1;
}


static uint32_t APP_HaltCommand( int32_t argc, char_t* argv[] )
{
	FAPEXOPENRET_T			 retVal = 0;

	FAPI_SYS_PRINT_MSG("[APP_HaltCommand] leave ICC demo app\n");

	retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_CLOSE_SUBID, (FAPEXIOCARG3_T)NULL );
	if ( retVal < 0 )
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl Close SubID failed \n");

	retVal = FAPEX_ICC_Close(fd_ICC0);
	if ( retVal < 0 )
	    FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl Close CommonID failed \n");

	exit( 1 );

	return 0;
}


static void APP_IccUserInsertCallback(void)
{
    FAPI_SYS_PRINT_MSG("Card Insert come to APP_IccUserInsertCallback\n");
}

static void APP_IccUserRemoveCallback(void)
{
    FAPI_SYS_PRINT_MSG("Card Remove come to APP_IccUserRemoveCallback\n");
}

/*
********************************************************************************
** The application's initialisation thread.
**
** This function is required to initialise the entire system.
** Within a non-RTOS environment is is called a a generic function,
** within an RTOS environment it is called within thread context.
********************************************************************************
*/
static void appInitialize( void )
{
    FAPEXOPENRET_T          retVal = 0;

//    int32_t                 size = 0;



#if 0
    FAPEX_GPIO_OpenParamStrT   paramsGPIO;
    int32_t		            GPIO_ID;
    while(1)
    {
        paramsGPIO.pin = 48;
        paramsGPIO.function = FAPEX_GPIO_OUT_0;//0;
        GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);
        FAPI_SYS_SLEEP(200);
        FAPEX_GPIO_Close(GPIO_ID);
        paramsGPIO.function = FAPEX_GPIO_OUT_1;//0;
        GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);
        FAPEX_GPIO_Close(GPIO_ID);
    };
#endif

    /*open common descriptor */
    fd_ICC0 = FAPEX_ICC_Open( FAPEXDEV_ICC_CMN_MINOR, O_RDWR, &iccParamStrT );
    if ( fd_ICC0 >= 0 )
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Open Success\n");
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Open Fail vaule=%d\n",fd_ICC0);

    /*open sub  descriptor */
    iccSbuIdParamStrT.cardInCallback  = APP_IccUserInsertCallback;
    iccSbuIdParamStrT.cardOutCallback = APP_IccUserRemoveCallback;
    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)FAPEXDEV_ICC_SUB_MINOR_FIRST, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_OPEN_SUBID, (FAPEXIOCARG3_T)&iccSbuIdParamStrT );
    if ( retVal >= 0 )
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl Open SubID Success iccSbuIdParamStrT.sub_id = %d\n",iccSbuIdParamStrT.sub_id);
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl Open SubID Fail vaule=%d\n",retVal);

    /*set IO port*/
    retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)iccSbuIdParamStrT.sub_id, (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_SET_IOPORT, (FAPEXIOCARG3_T)&iccIOprotParamStrT );
    if ( retVal >= 0 )
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_SET_IOPORT Success \n");
    else
        FAPI_SYS_PRINT_MSG("##FAPEX_ICC_Ioctl FAPEX_IOCCOM_ICC_SET_IOPORT Fail\n");

}


/*
*****************************************************************************
**
** \brief Shell thread
**
** This function monitors the input command.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT,
                                                             500,
                                                             NULL );
    if( !appShellInputHandler )
    {
        exit(1);
    }
#else
    char_t    args_buff[APP_CommandStringSize];
    int32_t   i, c;

    appInitialize();

    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n"   \
        "as optional argument to ?\n"                                           \
        "Example: ?       displays all available commands, one per line\n"     \
        "Example: ? !     displays a more detailed help for the ! command\n" \
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n" \
    );
#endif
    (void)appRegisterCommand(
        "GS",
        APP_GetStatusCommand,
        "get status of ICC module",
        "This function will print out the ICC module status\n"\
        "no param"\
        "Example:GS"\

    );


    (void)appRegisterCommand(
        "CR",
        APP_ColdResetCommand,
        "cold reset the smart card",
        "This function will print out the ATR value\n"\
        "no param"\
        "Example:CR"\
    );

    (void)appRegisterCommand(
        "WR",
        APP_WarmResetCommand,
        "warm reset the smart card",
        "This function will print out the ATR value\n"\
        "no param"\
        "Example:WR"\
    );

    (void)appRegisterCommand(
        "SCMD",
        APP_SendCMDCommand,
        "send command to smart card(include send or read command)",
        "send command to smart card\n"\
        "Example:SCMD 80 46 00 00 04 07 00 00 08\n"\
    );

    (void)appRegisterCommand(
        "SCMD2",
        APP_SendCMDCommand2,
        "send command to smart card",
        "send the SendDataCMD&ReadDataCMD command to smart card\n"\
        "no param"\
        "Example:SCMD2"\
    );

    (void)appRegisterCommand(
        "exit",
        APP_HaltCommand,
        "Exit application.",
        "Usage: exit\n"
    );


#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#else
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );

    while(1)
    {
        i = 0;
        while( !feof(stdin) )
        {
            c = getchar();
            args_buff[i] = c;
            if(c == '\n')
            {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) )
            {
                i --;
            }
            else if(i < (APP_CommandStringSize - 1))
            {
                i ++;
            }
        }
        SHELL_DoCommand( args_buff );
        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */

}


//***************************************************************************
//***************************************************************************
//** Main Program
//***************************************************************************
//***************************************************************************
int32_t main( int32_t argc, char_t* argv[] )
{
    int32_t  result = 0;

    FAPI_SYS_PRINT_MSG("\n%lld, main() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));


    FAPI_SYS_PRINT_MSG("\033[2J\n\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# icc demo application (FAPex version)\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# program version: %s\n", APP_VERSION);
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2007-2010 by Fujitsu Semiconductor Europe GmbH\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2008-2010 by Fujitsu Semiconductor Limited\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("\n");

    appShellThreadHandle = FAPEX_SYS_CreateThread( NULL,
                                                   APP_SHELL_THREAD_STACKSIZE,
                                                   APP_SHELL_THREAD_PRIORITY,
                                                   appShellThreadFunction,
                                                   0, 0, APP_SHELL_THREAD_NAME );
    if(appShellThreadHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG( "Error: FAPEX_SYS_CreateThread (shell)\n" );
        exit(1);
    }
    FAPI_SYS_PRINT_MSG("\n%lld, thread start\n",
                         FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_StopPrintThread();
    FAPEX_SYS_StartMultitasking();

    FAPEX_SYS_WaitEndThread( appShellThreadHandle );

    exit( result );
}






