/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_osd_demo/src/main.c
**
** \version     $Id$
**
** \brief       Demo Application for the DRAW subsystem
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007-2011 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2011 by Fujitsu Semiconductor Limited
**
********************************************************************************
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libviout.h>
#include <val/val.h>
#include <osdhandler/osdhandler.h>
#include <fontserver/fontserver.h>
#include <fapex/abst_libclkpwr.h>
#include <fapex/abst_libtsd.h>

#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */

#if (USE_FREETYPE == 1)
#include <fontserver/fontserver_engine_ft.h>
#include <ftf/ftf.h>
#else
#include <fontserver/fontserver_engine_fbf.h>
#include <fbf/fbf_zwo_bold.h>
#endif // USE_FREETYPE
#include <draw/draw.h>

#define APP_VERSION      "1.0.5"
#define APP_PREFIX       "[paint]"
#define APP_SHELL_PROMPT "[paint] ==> "
#define DRV_ID_CLKPWR   (FAPEXDEV_CLKPWR_MINOR_FIRST)
//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
int32_t fd_TSD[] = { -1, -1, -1, -1 };
extern int32_t fd_VIOUT;

#ifdef _LINUX_FAPI
struct timeval  tValue;
#endif
static VAL_DataT*                  appValDataPtr            = NULL;
static VAL_OpenT*                  appValOpenPtr            = NULL;

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define THREAD_PRIO_HDMI FAPEX_SYS_PRIORITY_MEDIUM

#ifndef _LINUX_FAPI
//***** BITMAP file addresses ****************
#define BITMAP_01BPP_ADDRESS    0x40000000
#define BITMAP_04BPP_ADDRESS    0x40010000
#define BITMAP_08BPP_ADDRESS    0x40050000
#define BITMAP_16BPP_ADDRESS    0x400C0000
#define BITMAP_24BPP_ADDRESS    0x40190000
#define BITMAP_32BPP_ADDRESS    0x402C0000
#define PICTURE_CCIR_ADDRESS    0x40450000
#endif

#define APP_HD_SYSTEM_MODE      VAL_1920x1080x50I
#define APP_SD_SYSTEM_MODE      VAL_SD_PAL
#define APP_TIME_DISPLAY        2500
#define APP_TIME_SETUP          400
#define APP_TIME_BREAK          200
#define APP_TXT_COLOUR          1
#define APP_TXT_BLACK           0
#define APP_FULL_STRING         0
#define APP_OSD1                1
#define APP_CP                  5
#define APP_FONT_IDX            0

#define DRV_ID_VIOUT_CP    (FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 2)
#define DRV_ID_VIOUT_OSD1  (FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 3)
#define DRV_ID_VIOUT_OSD2  (FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 4)
#define DRV_ID_VIENC_HD    (FAPEXDEV_VIENC_SUB_MINOR_FIRST + 0)

#define DRAW_TEST_PATTERN_BASIC_MAXNUM      9
#define DRAW_TEST_PATTERN_MOVE              100
#define DRAW_TEST_PATTERN_RESIZE            200
#define DRAW_TEST_PATTERN_RECTANGLE_START   300
#define DRAW_TEST_PATTERN_RECTANGLE         301
#define DRAW_TEST_PATTERN_FONT_START        400
#define DRAW_TEST_PATTERN_FONT              401
#define DRAW_TEST_PATTERN_CLUT              500

#define DISPLAY_ENABLE_FLAG_NONE        0x0000
#define DISPLAY_ENABLE_FLAG_OSD1        0x0001
#define DISPLAY_ENABLE_FLAG_OSD2        0x0002
#define DISPLAY_ENABLE_FLAG_OSD3        0x0004
#define DISPLAY_ENABLE_FLAG_OSD4        0x0008
#define DISPLAY_ENABLE_FLAG_CP          0x0010

#define APP_ColorFgBlack     "\033[30m"
#define APP_ColorFgRed       "\033[31m"
#define APP_ColorFgPurple    "\033[35m"
#define APP_CommandStringSize 64

#if (USE_FREETYPE == 0)
#define PRINT_STR_OFFSET_X  0
#define PRINT_STR_OFFSET_Y  0
#else
#define PRINT_STR_OFFSET_X  4
#define PRINT_STR_OFFSET_Y  20
#endif

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_SHELL_THREAD_STACKSIZE         (4 * 1024)
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"
#define APP_DRAW_THREAD_STACKSIZE          (8 * 1024)
#define APP_DRAW_THREAD_PRIORITY           FAPEX_SYS_PRIORITY_MEDIUM
#define APP_DRAW_THREAD_NAME               "app:draw"

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

// FONTSERVER handle:
FAPI_SYS_HandleT appFontserverHandle = NULL;

// FONTSERVER font index:
uint32_t appFontIndex = 0;

// Video system mode for the HD encoder:
VAL_HdEncoderT hd_system_mode;
VAL_SdEncoderT sd_system_mode;

/* Addresses for engine interfaces. */
uint32_t appEngineData[1];

// Resolution of the system mode (will be set up later):
static uint32_t out_h_res = 0;
static uint32_t out_v_res = 0;

// Switch that decides whether to read the source as a frame or field:
static uint32_t read_frame = 0;

// coordinates for the display (OSD/CP with text description):
static uint32_t display__x = 0;
static uint32_t display__y = 0;

// resolution for the display
static uint32_t display__width  = 0;
static uint32_t display__height = 0;

static uint32_t defaultClut[] =
{
    0x800080FF, /* black */ 0x80FF80FF, /* white */
    0x808080FF, /* gray */  0x80EC80FF,
    0x64E584FF, 0x48DF89FF, 0x2CD98DFF, 0x10D392FF,
    0x92CB97FF, 0x76C59CFF, 0x5ABFA0FF, 0x3EB9A5FF,
    0x22B2A9FF, 0xA5ABAFFF, 0x89A5B3FF, 0x6D9FB8FF,
    0x5198BCFF, 0x3592C1FF, 0xB78BC6FF, 0x9B85CBFF,
    0x7F7ECFFF, 0x6378D4FF, 0x4772D8FF, 0xCA6BDEFF,
    0xAE64E2FF, 0x925EE7FF, 0x7658EBFF, 0x5A52F0FF,
    0x89DB64FF, 0x6DD568FF, 0x51CF6DFF, 0x35C871FF,
    0x19C276FF, 0x9CBB7BFF, 0x80B580FF, 0x64AE84FF,
    0x48A889FF, 0x2CA28DFF, 0xAE9B93FF, 0x929497FF,
    0x768E9CFF, 0x5A88A0FF, 0x3E82A5FF, 0xC17AAAFF,
    0xA574AFFF, 0x896EB3FF, 0x6D68B8FF, 0x5161BCFF,
    0xD35AC2FF, 0xB754C6FF, 0x9B4ECBFF, 0x7F47CFFF,
    0x6341D4FF, 0x93CB48FF, 0x77C44CFF, 0x5BBE51FF,
    0x3FB855FF, 0x23B25AFF, 0xA5AA5FFF, 0x89A464FF,
    0x6D9E68FF, 0x51986DFF, 0x359171FF, 0xB88A77FF,
    0x9C847BFF, 0x807E80FF, 0x647784FF, 0x487189FF,
    0xCA6A8EFF, 0xAE6493FF, 0x925D97FF, 0x76579CFF,
    0x5A51A0FF, 0xDD4AA6FF, 0xC143AAFF, 0xA53DAFFF,
    0x8937B3FF, 0x6D31B8FF, 0x9CBA2CFF, 0x80B430FF,
    0x64AE35FF, 0x48A739FF, 0x2CA13EFF, 0xAF9A43FF,
    0x939448FF, 0x778D4CFF, 0x5B8751FF, 0x3F8155FF,
    0xC17A5BFF, 0xA5735FFF, 0x896D64FF, 0x6D6768FF,
    0x51616DFF, 0xD45972FF, 0xB85377FF, 0x9C4D7BFF,
    0x804780FF, 0x644084FF, 0xE6398AFF, 0xCA338EFF,
    0xAE2D93FF, 0x922697FF, 0x76209CFF, 0xA6AA10FF,
    0x8AA314FF, 0x6E9D19FF, 0x52971DFF, 0x369122FF,
    0xB88927FF, 0x9C832CFF, 0x807D30FF, 0x647735FF,
    0x487039FF, 0xCB693FFF, 0xAF6343FF, 0x935D48FF,
    0x77564CFF, 0x5B5051FF, 0xDD4956FF, 0xC1435BFF,
    0xA53C5FFF, 0x893664FF, 0x6D3068FF, 0xF0296EFF,
    0xD42272FF, 0xB81C77FF, 0x9C167BFF, 0x801080FF,
    0x80FF80FF, 0x800080FF, 0x808080FF, 0x807E80FF,
    0x727A82FF, 0x647784FF, 0x567486FF, 0x487189FF,
    0x896D8BFF, 0x7B6A8EFF, 0x6D6790FF, 0x5F6492FF,
    0x516194FF, 0x925D97FF, 0x845A99FF, 0x76579CFF,
    0x68549EFF, 0x5A51A0FF, 0x9B4DA3FF, 0x8D4AA5FF,
    0x7F47A7FF, 0x7144AAFF, 0x6341ACFF, 0xA53DAFFF,
    0x973AB1FF, 0x8937B3FF, 0x7B34B5FF, 0x6D31B8FF,
    0x847572FF, 0x767274FF, 0x686F76FF, 0x5A6C78FF,
    0x4C697BFF, 0x8E657DFF, 0x806280FF, 0x725F82FF,
    0x645C84FF, 0x565986FF, 0x975589FF, 0x89528BFF,
    0x7B4F8EFF, 0x6D4C90FF, 0x5F4992FF, 0xA04595FF,
    0x924297FF, 0x843F99FF, 0x763C9CFF, 0x68389EFF,
    0xA935A1FF, 0x9B32A3FF, 0x8D2FA5FF, 0x7F2BA7FF,
    0x7128AAFF, 0x896D64FF, 0x7B6A66FF, 0x6D6768FF,
    0x5F646AFF, 0x51616DFF, 0x925D6FFF, 0x845A72FF,
    0x765774FF, 0x685476FF, 0x5A5078FF, 0x9C4D7BFF,
    0x8E4A7DFF, 0x804780FF, 0x724382FF, 0x644084FF,
    0xA53D87FF, 0x973A89FF, 0x89368BFF, 0x7B338EFF,
    0x6D3090FF, 0xAE2D93FF, 0xA02995FF, 0x922697FF,
    0x842399FF, 0x76209CFF, 0x8E6556FF, 0x806258FF,
    0x725F5AFF, 0x645B5CFF, 0x56585FFF, 0x975561FF,
    0x895264FF, 0x7B4E66FF, 0x6D4B68FF, 0x5F486AFF,
    0xA0456DFF, 0x92416FFF, 0x843E72FF, 0x763B74FF,
    0x683876FF, 0xAA3479FF, 0x9C317BFF, 0x8E2E7DFF,
    0x802B80FF, 0x722882FF, 0xB32485FF, 0xA52187FF,
    0x971E89FF, 0x891B8BFF, 0x7B188EFF, 0x935D48FF,
    0x85594AFF, 0x77564CFF, 0x69534EFF, 0x5B5051FF,
    0x9C4C53FF, 0x8E4956FF, 0x804658FF, 0x72435AFF,
    0x64405CFF, 0xA53C5FFF, 0x973961FF, 0x893664FF,
    0x7B3366FF, 0x6D3068FF, 0xAE2C6BFF, 0xA0296DFF,
    0x92266FFF, 0x842372FF, 0x762074FF, 0xB81C77FF,
    0xAA1979FF, 0x9C167BFF, 0x8E137DFF, 0x801080FF
};

static int32_t  appDrawEnable = 0;
static int32_t  appDrawContrl = 0;

#ifndef _LINUX_FAPI
static FAPI_SYS_ShellInputHandler* appShellInputHandler;
#endif

static FAPEX_SYS_ThreadT  appShellThreadHandle;
static FAPEX_SYS_ThreadT  appDrawThreadHandle;

#ifdef _LINUX_FAPI
static FAPEX_SYS_FFILE  *fpt_bitmap_01bpp;
static FAPEX_SYS_FFILE  *fpt_bitmap_04bpp;
static FAPEX_SYS_FFILE  *fpt_bitmap_08bpp;
static FAPEX_SYS_FFILE  *fpt_bitmap_16bpp;
static FAPEX_SYS_FFILE  *fpt_bitmap_24bpp;
static FAPEX_SYS_FFILE  *fpt_bitmap_32bpp;
static FAPEX_SYS_FFILE  *fpt_picture_CCIR;

static uint32_t                 data_buff_size = 0;
#endif

static uint32_t                 layer_id = 0;
static uint32_t                 layer_bpp;
static uint32_t                 layer_width;
static uint32_t                 layer_height;
static uint32_t                 layer_px[6]  = {0,0,0,0,0,0};
static uint32_t                 layer_py[6]  = {0,0,0,0,0,0};
static OSDHANDLER_StartParamsT  start_params;

static OSDHANDLER_ModeParamT    fading[6] = { OSDHANDLER_PARAMETER_ON,
                                              OSDHANDLER_PARAMETER_ON,
                                              OSDHANDLER_PARAMETER_ON,
                                              OSDHANDLER_PARAMETER_ON,
                                              OSDHANDLER_PARAMETER_ON,
                                              OSDHANDLER_PARAMETER_ON };
static uint8_t              osdInfo1[64];
static uint8_t              osdInfo2[64];
static DRAW_BmpFileHeadT    bmpHdr;
static uint32_t             *data_buff = NULL;

static uint32_t             display_flag = DISPLAY_ENABLE_FLAG_NONE;
static uint32_t             enable_flag  = DISPLAY_ENABLE_FLAG_NONE;
static int32_t              move_x = 0;
static int32_t              move_y = 0;
static float                resize_ratio_x = 1.0;
static float                resize_ratio_y = 1.0;

static uint64_t             draw_pic_time_start;
static uint64_t             draw_pic_count;

static FONTSERVER_InformationT  font_info;
static uint32_t                 font_draw_frame_count;
static uint32_t                 font_draw_code_count;
static uint8_t                  font_draw_code;
static uint32_t                 font_draw_colour;
static uint32_t                 font_draw_x0;
static uint32_t                 font_draw_y0;

static uint32_t             clut_type;

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static uint32_t calcProp_H_Value(uint32_t arg);
static uint32_t calcProp_V_Value(uint32_t arg);
static void setUpOutputResolution(uint32_t layer, OSDHANDLER_StartParamsT* startParamsPtr);

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   wakeup draw thread
*****************************************************************************
*/
static void wakeup_DrawThread(void)
{
    FAPEX_SYS_WakeupThread( appDrawThreadHandle );
}

/*!
*****************************************************************************
** \brief   exit command
*****************************************************************************
*/
static uint32_t APP_ExitCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
        appDrawEnable = -1;
        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   end command
*****************************************************************************
*/
static uint32_t APP_EndCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
        if(appDrawEnable > 0) {
            appDrawEnable = 1;
        }
        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   pause command
*****************************************************************************
*/
static uint32_t APP_PauseCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
        appDrawContrl = 1;
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   move command
*****************************************************************************
*/
static uint32_t APP_MoveCommand( int32_t argc, char_t* argv[] )
{
    if( (argc == 1) || (argc == 3) ) {
        if( argc == 1 ) {
            move_x = (rand() & 0x000007F) - 64;
            move_y = (rand() & 0x000007F) - 64;
        }
        else {
            sscanf(argv[1], "%d", &move_x);
            sscanf(argv[1], "%d", &move_y);
        }

        appDrawEnable = DRAW_TEST_PATTERN_MOVE;

        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   resize command
*****************************************************************************
*/
static uint32_t APP_ResizeCommand( int32_t argc, char_t* argv[] )
{
    if( (argc == 2) || (argc == 3) ) {
        sscanf(argv[1], "%f", &resize_ratio_x);
        if(argc == 3) {
            sscanf(argv[2], "%f", &resize_ratio_y);
        }
        else {
            resize_ratio_y = resize_ratio_x;
        }
        appDrawEnable = DRAW_TEST_PATTERN_RESIZE;

        if(appDrawContrl == 1) {
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   draw rectangle command
*****************************************************************************
*/
static uint32_t APP_RectangleCommand( int32_t argc, char_t* argv[] )
{
    int32_t  tmp_id;

    if( argc > 0 ) {
        if( argc == 2 ) {
            tmp_id = atoi(argv[1]);
            if((tmp_id > 0) && (tmp_id < 5)) {
                FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
                layer_id = tmp_id;
            }
            else {
                layer_id = 0;
            }
        }
        appDrawEnable = DRAW_TEST_PATTERN_RECTANGLE_START;

        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   draw Font command
*****************************************************************************
*/
static uint32_t APP_FontCommand( int32_t argc, char_t* argv[] )
{
    int32_t  tmp_id;

    if( argc == 2 ) {
        tmp_id = atoi(argv[1]);
        if((tmp_id > 0) && (tmp_id < 5)) {
            FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
            layer_id = tmp_id;
        }
        else {
            layer_id = 0;
        }
        appDrawEnable = DRAW_TEST_PATTERN_FONT_START;

        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   draw CLUT command
*****************************************************************************
*/
static uint32_t APP_ClutCommand( int32_t argc, char_t* argv[] )
{
    int32_t  tmp_id;

    if( (argc == 2) || (argc == 3) ) {
        tmp_id = atoi(argv[1]);
        if((tmp_id > 0) && (tmp_id < 6)) {
            FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
            layer_id = tmp_id;
        }
        else {
            layer_id = 0;
        }
        if(argc == 3) {
            clut_type = atoi(argv[2]);
        }
        else {
            clut_type = 0;
        }
        appDrawEnable = DRAW_TEST_PATTERN_CLUT;

        if(appDrawContrl == 1) {
            appDrawContrl = 0;
            /* wakeup appDrawThreadHandle */
            wakeup_DrawThread();
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   hilight command
*****************************************************************************
*/
static uint32_t APP_HilightCommand( int32_t argc, char_t* argv[] )
{
    static OSDHANDLER_ModeParamT  hilight_flag = OSDHANDLER_PARAMETER_OFF;
    uint32_t  errorCode;
    int32_t   result;

    if( argc == 1 ) {
        hilight_flag = (hilight_flag == OSDHANDLER_PARAMETER_ON) ?
                        OSDHANDLER_PARAMETER_OFF : OSDHANDLER_PARAMETER_ON;

        /**** set hilight for CP layer ****/
        result = OSDHANDLER_CursorHilight(hilight_flag, &errorCode);
        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_MSG("Error set hilight Cursor Plane: %d\n", result);
        }
    }
    return 1;
}

/*!
*****************************************************************************
** \brief   set alpha command
*****************************************************************************
*/
static uint32_t APP_AlphaCommand( int32_t argc, char_t* argv[] )
{
    uint32_t  alpha;
    uint32_t  tmp_id;
    int32_t   result;

    if( argc == 3 ) {
        tmp_id = atoi(argv[1]);
        if((tmp_id > 0) && (tmp_id < 5)) {
            FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
        }
        else if(tmp_id == 5) {
            FAPI_SYS_PRINT_MSG("%sselect CP%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        }
        else {
            tmp_id = 0;
        }
        alpha = atoi(argv[2]);

        /**** set alpha ****/
        result = OSDHANDLER_GlobalAlpha( tmp_id, OSDHANDLER_PARAMETER_SET, alpha );
        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_MSG("Error set alpha: %d\n", result);
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   set fading command
*****************************************************************************
*/
static uint32_t APP_FadeCommand( int32_t argc, char_t* argv[] )
{
    uint32_t  tmp_id;
    uint32_t  errorCode;
    int32_t   result;

    if( argc == 2 ) {
        tmp_id = atoi(argv[1]);
        if((tmp_id > 0) && (tmp_id < 5)) {
            FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
        }
        else if(tmp_id == 5) {
            FAPI_SYS_PRINT_MSG("%sselect CP%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        }
        else {
            tmp_id = 0;
        }
        fading[tmp_id] = (fading[tmp_id] == OSDHANDLER_PARAMETER_ON) ?
                            OSDHANDLER_PARAMETER_OFF : OSDHANDLER_PARAMETER_ON;

        /**** set fading ****/
        result = OSDHANDLER_PlaneFading( tmp_id, fading[tmp_id], &errorCode );
        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_MSG("Error set fading: %d\n", result);
            return 0;
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   set enable command
*****************************************************************************
*/
static uint32_t APP_EnableCommand( int32_t argc, char_t* argv[] )
{
    uint32_t  enable_mask;
    uint32_t  tmp_id;
    uint32_t  errorCode;
    int32_t   result;

    if( argc == 2 ) {
        tmp_id = atoi(argv[1]);
        if((tmp_id > 0) && (tmp_id < 5)) {
            FAPI_SYS_PRINT_MSG("%sselect OSD%d%s\n", APP_ColorFgPurple, tmp_id, APP_ColorFgBlack);
            enable_mask = 0x00000001 << (tmp_id - 1);
        }
        else {
            tmp_id      = 0;
            enable_mask = DISPLAY_ENABLE_FLAG_NONE;
        }

        /**** set enable ****/
        if(enable_flag & enable_mask) {
            printf("set enable\n");
            result = OSDHANDLER_ShowLayer( tmp_id, &errorCode );
            enable_flag &= ~enable_mask;
        }
        else {
            printf("set disable\n");
            result = OSDHANDLER_HideLayer( tmp_id, &errorCode );
            enable_flag |= enable_mask;
        }
        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_MSG("Error set enable: %d\n", result);
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   set background colour command
*****************************************************************************
*/
static uint32_t APP_BackgroundCommand( int32_t argc, char_t* argv[] )
{
    uint32_t  pre_value, new_value;
    uint32_t  y, cr, cb;
    int32_t   result;

    if( argc == 4 ) {
        /**** get background colour ****/
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)fd_VIOUT,
                                    FAPEX_IOCCOM_VIOUT_GET_BGCOLOUR,
                                    (FAPEXIOCARG3_T)&pre_value );
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_ERR("Error get background colour: %d\n", result);
        }
        else {
            y  = atoi(argv[1]);
            cr = atoi(argv[2]);
            cb = atoi(argv[3]);

            if(y  >= 256) y  = 255;
            if(cr >= 256) cr = 255;
            if(cb >= 256) cb = 255;

            new_value = (y << 16) | (cr << 8) | cb;

            /**** set background colour ****/
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)fd_VIOUT,
                                        FAPEX_IOCCOM_VIOUT_SET_BGCOLOUR,
                                        (FAPEXIOCARG3_T)new_value );
            if(result == FAPI_OK) {
                FAPI_SYS_PRINT_MSG("  {0x%02X, 0x%02X, 0x%02X} -> {0x%02X, 0x%02X, 0x%02X}\n",
                                   (pre_value >> 16), (pre_value >> 8) & 0x00FF, pre_value & 0x00FF, y, cr, cb);
            }
            else {
                FAPI_SYS_PRINT_ERR("Error set background colour: %d\n", result);
            }
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   set Plane Order command
*****************************************************************************
*/
static uint32_t APP_OrderCommand( int32_t argc, char_t* argv[] )
{
    uint32_t  i, j;
    int32_t   pre_order[6], new_order[6], tmp_order;
    int32_t   result;

    if( (argc == 1) || (argc == 6) ) {
        /**** get plane order ****/
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)fd_VIOUT,
                                    FAPEX_IOCCOM_VIOUT_GET_PLANEORDER,
                                    (FAPEXIOCARG3_T)pre_order );
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_ERR("Error get plane order: %d\n", result);
        }
        else {
            new_order[0] = pre_order[0];   // video
            if(argc == 1) {
                for(i=1; i<6; i++) {
                    new_order[i] = pre_order[i];
                }
                for(i=0; i<6; i++) {
                    if((i != 3) && (i != 4)) {
                        j = rand() % 4;
                        if(j == 3) j=5;
                        tmp_order    = new_order[j];
                        new_order[j] = new_order[i];
                        new_order[i] = tmp_order;
                    }
                }
            }
            else {
                for(i=1; i<5; i++) {
                    new_order[i] = atoi(argv[i]);
                }
                new_order[5] = atoi(argv[i]); // cp
            }
            result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)fd_VIOUT,
                                        FAPEX_IOCCOM_VIOUT_SET_PLANEORDER,
                                        (FAPEXIOCARG3_T)new_order );
            if(result == FAPI_OK) {
                FAPI_SYS_PRINT_MSG("  {%d, %d, %d} -> {%d, %d, %d}\n",
                                   pre_order[1], pre_order[2], pre_order[5],
                                   new_order[1], new_order[2], new_order[5]);
            }
            else {
                FAPI_SYS_PRINT_ERR("Error set plane order: %d\n", result);
            }
        }
        return 1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   Calculates the proportional value for a horizontal value
**          (such as resolution or position) relative to the 1920x1080I mode
**          depending on the current system mode.
*****************************************************************************
*/
static uint32_t calcProp_H_Value(uint32_t arg)
{
    if(hd_system_mode != VAL_1920x1080x50I)
    {
        return((arg * out_h_res) / 1920);
    }
    else
    {
        return(arg);
    }
}

/*!
*****************************************************************************
** \brief   Calculates the proportional value for a vertical value
**          (such as resolution or position) relative to the 1920x1080I mode
**          depending on the current system mode.
*****************************************************************************
*/
static uint32_t calcProp_V_Value(uint32_t arg)
{
    if(hd_system_mode != VAL_1920x1080x50I) // 1080i-mode: output res: 540
    {
        return((arg * out_v_res) / 540);
    }
    else
    {
        return(arg);
    }
}

/*!
*****************************************************************************
** \brief   setup resolution
*****************************************************************************
*/
static void setUpOutputResolution(uint32_t layer, OSDHANDLER_StartParamsT* startParamsPtr)
{
    if(hd_system_mode != VAL_1920x1080x50I)
    {
        startParamsPtr->outputWidth = \
            (startParamsPtr->inputWidth * out_h_res) / 1920;
    }
    else
    {
        startParamsPtr->outputWidth = 0; // set up automatically
    }
    if(hd_system_mode != VAL_1920x1080x50I)
    {
        startParamsPtr->outputHeight = \
            (startParamsPtr->inputHeight * out_v_res) / 1080;
    }
    else
    {
        startParamsPtr->outputHeight = 0; // set up automatically
    }
    layer_px[layer] = startParamsPtr->xPos;
    layer_py[layer] = startParamsPtr->yPos;
}

#ifdef _LINUX_FAPI
/*!
*****************************************************************************
** \brief   read file data
*****************************************************************************
*/
static void readFileData(FAPEX_SYS_FFILE *fpt, uint32_t **buff, uint32_t *buff_size)
{
    uint32_t size;

    FAPEX_SYS_FSeek(fpt, 0, FAPEX_SYS_FSEEK_END);
    size = FAPEX_SYS_FTell(fpt);
    FAPEX_SYS_FRewind(fpt);

    if(size > *buff_size) {
        if(*buff != NULL) {
            FAPI_SYS_FREE(*buff);
            *buff = NULL;
        }
        *buff_size = size;
    }
    if(*buff == NULL) {
        *buff = (uint32_t *)FAPI_SYS_MALLOC(*buff_size);
    }
    if(*buff != NULL) {
        FAPEX_SYS_FRead(*buff, sizeof(uint8_t), size, fpt);
    }
}
#endif /* _LINUX_FAPI */

/*!
*****************************************************************************
** \brief   draw test pattern 0
*****************************************************************************
*/
static int32_t draw_test0(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 4 bpp ********
#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_04BPP_ADDRESS;
#else
    fpt_bitmap_04bpp = fopen("data/04bpp_fujitsu_MB86H60.bmp", "rb");
    if(fpt_bitmap_04bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/04bpp_fujitsu_MB86H60.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_04bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_04bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    // start OSD without picture
    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_CLUT_4BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(240);
    start_params.yPos        = calcProp_V_Value(60);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    // give a message that gives information about the picture
    sprintf((char_t*)osdInfo1, "OSD%d: 4-bpp CbYCrA CLUT", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 10+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/

    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 4-bit bitmap on OSD
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("CLUT of bitmap converted from ARBG to CbYCrA\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d \n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 1
*****************************************************************************
*/
static int32_t draw_test1(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 8 bpp ********
#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_08BPP_ADDRESS;
#else
    fpt_bitmap_08bpp = fopen("data/08bpp_fujitsu_MB86H60.bmp", "rb");
    if(fpt_bitmap_08bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/08bpp_fujitsu_MB86H60.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_08bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_08bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    // start OSD without a picture and print an info string
    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_CLUT_8BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(860);
    start_params.yPos        = calcProp_V_Value(40);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo2, "OSD%d: 8-bpp CbYCrA CLUT", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo2, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 30+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 8-bit bitmap on OSD
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo2);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("CLUT of bitmap converted from ARBG to CbYCrA\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 1
*****************************************************************************
*/
static int32_t clear_test1(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t  result;

    // stop OSD and set OSD1 in front of OSD2
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    result = OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET, 5, 1, 2, 3, 4, 0,
                                   &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set Plane Order: %d\n", result);
        return result;
    }

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 10+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 2
*****************************************************************************
*/
static int32_t draw_test2(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 16 bpp CbYCrA 5551 ********
#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_16BPP_ADDRESS;
#else
    fpt_bitmap_16bpp = fopen("data/16bpp_Fujitsu1.bmp", "rb");
    if(fpt_bitmap_16bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/16bpp_Fujitsu1.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_16bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_16bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_5551, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(240);
    start_params.yPos        = calcProp_V_Value(80);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 16-bpp CbYCrA (5551)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 10+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 16-bit bitmap while converting colours to CbYCrA mode
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("colors converted from bitmap format\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 2
*****************************************************************************
*/
static int32_t clear_test2(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSD and set OSD2 in front of OSD1
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    result = OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET, 5, 2, 1, 3, 4, 0,
                                   &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set Plane Order: %d\n", result);
        return result;
    }

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo2, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 30+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 3
*****************************************************************************
*/
static int32_t draw_test3(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 16 bpp ARGB 1555 ********
#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_16BPP_ADDRESS;
#else
    fpt_bitmap_16bpp = fopen("data/16bpp_Fujitsu1.bmp", "rb");
    if(fpt_bitmap_16bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/16bpp_Fujitsu1.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_16bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_16bpp);
#endif

    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_5551, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_ARGB, &errorCode);
    start_params.xPos        = calcProp_H_Value(860);
    start_params.yPos        = calcProp_V_Value(120);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo2, "OSD%d: 16-bpp ARGB (1555)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo2, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 30+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 16-bit bitmap in the native ARGB mode
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo2);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("original bitmap colors\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 3
*****************************************************************************
*/
static int32_t clear_test3(int32_t osdId1, int32_t osdId2)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSDs and keep OSD2 set in front of OSD1
    (void)OSDHANDLER_StopLayer(osdId1, &errorCode);
    (void)OSDHANDLER_StopLayer(osdId2, &errorCode);
    display_flag &= ~((0x00000001 << (osdId1 - 1)) | (0x00000001 << (osdId2 - 1)));

    result = OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET, 5, 2, 1, 3, 4, 0,
                                   &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set Plane Order: %d\n", result);
        return result;
    }

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 10+PRINT_STR_OFFSET_Y, &result);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo2, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 30+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 4
*****************************************************************************
*/
static int32_t draw_test4(int32_t osdId)
{
    OSDHANDLER_LayerParamsT   layer_params;
    uint32_t  *osd_ptr, *pic_ptr;
    uint32_t  ii;
    uint32_t  errorCode;
    int32_t   result;

    // ******** 16 bpp CCIR mode test picture ********
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_16BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(700);
    start_params.yPos        = calcProp_V_Value(80);
    start_params.inputWidth  = 512;
    start_params.inputHeight = 576;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 16-bpp test picture for CCIR mode", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

#ifndef _LINUX_FAPI
    ii = 0x00024000UL; // number of words of the test file
    data_buff = (uint32_t*)PICTURE_CCIR_ADDRESS;
#else
    // draw the test picture:
    fpt_picture_CCIR = fopen("data/16bpp_CCIR_test.out", "rb");
    if(fpt_picture_CCIR == NULL) {
        FAPI_SYS_PRINT_MSG("data/16bpp_CCIR_test.out open error\n");
        return -1;
    }

    // read data
    ii = 0;
    pic_ptr = data_buff;
    do {
        result = fscanf(fpt_picture_CCIR, "0x%08X,\n", pic_ptr++);
        ii ++;
        if(ii >= data_buff_size) {
            break;
        }
    } while(result != EOF);

    fclose(fpt_picture_CCIR);
#endif /* !_LINUX_FAPI */

    result = OSDHANDLER_GetLayerParameters(osdId, &layer_params);
    if(result != FAPI_OK) {
        FAPI_SYS_PRINT_MSG("Error get layer parameter of the OSD%d: %d\n", osdId, result);
        return result;
    }

    ii = ((layer_params.xRes * layer_params.bpp) >> 5) * layer_params.yRes;
    osd_ptr = (uint32_t *)layer_params.pictureBase;
    pic_ptr = data_buff;

    while(--ii) {
        *osd_ptr++ = *pic_ptr++;
    }

    FAPI_SYS_PRINT_MSG("%s OSD%d: 16-bpp test picture for CCIR mode\n", APP_PREFIX, osdId);

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 4
*****************************************************************************
*/
static int32_t clear_test4(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSD
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 5
*****************************************************************************
*/
static int32_t draw_test5(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 24 bpp CbYCrA ********
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_24BPP_ADDRESS;
#else
    fpt_bitmap_24bpp = fopen("data/24bpp_Fujitsu1.bmp", "rb");
    if(fpt_bitmap_24bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/24bpp_Fujitsu1.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_24bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_24bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_32_24BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(480);
    start_params.yPos        = calcProp_V_Value(60);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 24/32-bpp CbYCr[A] (8888)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 24-bit bitmap while converting colours to CbYCrA mode
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("colors converted from bitmap format\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 5
*****************************************************************************
*/
static int32_t clear_test5(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSD
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 6
*****************************************************************************
*/
static int32_t draw_test6(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 24 bpp (A)RGB ********
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_24BPP_ADDRESS;
#else
    fpt_bitmap_24bpp = fopen("data/24bpp_Fujitsu2.bmp", "rb");
    if(fpt_bitmap_24bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/24bpp_Fujitsu2.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_24bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_24bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_32_24BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_ARGB, &errorCode);
    start_params.xPos        = calcProp_H_Value(480);
    start_params.yPos        = calcProp_V_Value(60);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 24/32-bpp [A]RGB (8888)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 24-bit bitmap
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("original bitmap colors\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 6
*****************************************************************************
*/
static int32_t clear_test6(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSD
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 7
*****************************************************************************
*/
static int32_t draw_test7(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 32 bpp CbYCrA ********
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_32BPP_ADDRESS;
#else
    fpt_bitmap_32bpp = fopen("data/24bpp_Fujitsu1.bmp", "rb");
    if(fpt_bitmap_32bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/24bpp_Fujitsu1.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_32bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_32bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_32_24BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(560);
    start_params.yPos        = calcProp_V_Value(60);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 32-bpp CbYCrA (8888)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 32-bit bitmap while converting colours to CbYCrA mode
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("colors converted from bitmap format\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 7
*****************************************************************************
*/
static int32_t clear_test7(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // stop OSD
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern 8
*****************************************************************************
*/
static int32_t draw_test8(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 32 bpp ARGB ********
    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_32BPP_ADDRESS;
#else
    fpt_bitmap_32bpp = fopen("data/24bpp_Fujitsu2.bmp", "rb");
    if(fpt_bitmap_32bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/24bpp_Fujitsu2.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_32bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_32bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_32_24BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_ARGB, &errorCode);
    start_params.xPos        = calcProp_H_Value(560);
    start_params.yPos        = calcProp_V_Value(60);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(osdId, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    sprintf((char_t*)osdInfo1, "OSD%d: 32-bpp ARGB (8888)", osdId);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    // draw the 32-bit bitmap ARGB
    result = DRAW_Bitmap(osdId, 0, 0, DRAW_DO_NOT_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("original bitmap colors\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while drawing on OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_FlushPicture(osdId);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the OSD%d: %d\n", osdId, result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   clear test pattern 8
*****************************************************************************
*/
static int32_t clear_test8(void)
{
    uint32_t  errorCode;

    // stop OSDs
    (void)OSDHANDLER_StopLayer(1, &errorCode);
    (void)OSDHANDLER_StopLayer(2, &errorCode);
    (void)OSDHANDLER_StopLayer(3, &errorCode);
    (void)OSDHANDLER_StopLayer(4, &errorCode);
    (void)OSDHANDLER_StopLayer(5, &errorCode);
    display_flag = DISPLAY_ENABLE_FLAG_NONE;

    return 0;
}

/*!
*****************************************************************************
** \brief   draw test pattern 9
*****************************************************************************
*/
static int32_t draw_test9(int32_t osdId)
{
    uint32_t  errorCode;
    int32_t   result;

    // ******** 1 bpp for the Cursor Plane ********
#ifndef _LINUX_FAPI
    data_buff = (uint32_t*)BITMAP_01BPP_ADDRESS;
#else
    fpt_bitmap_01bpp = fopen("data/01bpp_test.bmp", "rb");
    if(fpt_bitmap_01bpp == NULL) {
        FAPI_SYS_PRINT_MSG("data/01bpp_test.bmp open error\n");
        return -1;
    }
    readFileData(fpt_bitmap_01bpp, &data_buff, &data_buff_size);

    fclose(fpt_bitmap_01bpp);
#endif
    (void)DRAW_ParseBitmap(&bmpHdr, (uint8_t*)data_buff);

    (void)OSDHANDLER_SetColourMode(APP_CP, OSDHANDLER_CURSOR_MODE_1BIT,  &errorCode);
    (void)OSDHANDLER_SetLayoutMode(APP_CP, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos        = calcProp_H_Value(800);
    start_params.yPos        = calcProp_V_Value(160);
    start_params.inputWidth  = bmpHdr.width;
    start_params.inputHeight = bmpHdr.height;
    setUpOutputResolution(APP_CP, &start_params);

    result = OSDHANDLER_StartLayer(APP_CP, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the Cursor Plane: %d\n", result);
        return result;
    }

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    (void)OSDHANDLER_StopLayer(osdId, &errorCode);
    display_flag &= ~(0x00000001 << (osdId - 1));

    (void)OSDHANDLER_SetColourMode(osdId, OSDHANDLER_OSD_MODE_CLUT_4BIT, &errorCode);
    (void)OSDHANDLER_SetLayoutMode(osdId, OSDHANDLER_OSD_LAYOUT_CbYCrA,  &errorCode);
    start_params.xPos         = display__x;
    start_params.yPos         = display__y;
    start_params.inputWidth   = display__width;
    start_params.inputHeight  = display__height;
    start_params.outputWidth  = 0; // set up automatically
    start_params.outputHeight = 0; // set up automatically
    setUpOutputResolution(1, &start_params);

    result = OSDHANDLER_StartLayer(osdId, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD%d: %d\n", osdId, result);
        return result;
    }

    result = OSDHANDLER_ShowLayer(osdId, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD%d: %d\n", osdId, result);
        return result;
    }
    display_flag |= 0x00000001 << (osdId - 1);

    strcpy((char_t*)osdInfo1, "Cursor Plane: 1-bpp bitmap (CbYCrA)");

    (void)FONTSERVER_SetLayerId(appFontserverHandle, osdId);
    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);

    // draw the 01-bit bitmap
    result = DRAW_Bitmap(APP_CP, 0, 0, DRAW_SET_UP_CLUT, (uint8_t*)data_buff, &errorCode);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
        FAPI_SYS_PRINT_MSG("%s       ", APP_PREFIX);
        FAPI_SYS_PRINT_MSG("CLUT of bitmap converted from ARBG to CbYCrA\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
        return result;
    }

    /**** flush plane area ****/
    result = OSDHANDLER_FlushPicture(5);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush the Cursor Plane: %d\n", result);
    }
    else {
        display_flag |= DISPLAY_ENABLE_FLAG_CP;
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw test pattern reset
*****************************************************************************
*/
static int32_t draw_test_reset(int32_t osdId1, int32_t osdId2)
{
    uint32_t  errorCode;
    int32_t   result;

    (void)FONTSERVER_SetLayerId(appFontserverHandle, APP_CP);

    // stop the Cursor Plane and restore it as a display
    (void)OSDHANDLER_StopLayer(osdId1, &errorCode);
    (void)OSDHANDLER_StopLayer(osdId2, &errorCode);
    if((layer_id != osdId1) && (layer_id != osdId2)) {
        (void)OSDHANDLER_StopLayer(layer_id, &errorCode);
    }
    (void)OSDHANDLER_StopLayer(APP_CP, &errorCode);
    display_flag = DISPLAY_ENABLE_FLAG_NONE;

    (void)OSDHANDLER_SetColourMode(APP_CP, OSDHANDLER_CURSOR_MODE_1BIT,  &errorCode);
    (void)OSDHANDLER_SetLayoutMode(APP_CP, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.xPos         = display__x;
    start_params.yPos         = display__y;
    start_params.inputWidth   = display__width;
    start_params.inputHeight  = display__height;
    start_params.frameBased   = read_frame;
    start_params.outputWidth  = 0;
    start_params.outputHeight = 0;
    setUpOutputResolution(APP_CP, &start_params);

    result = OSDHANDLER_StartLayer(APP_CP, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the Cursor Plane: %d\n", result);
        return result;
    }

    /* set Colour LUT */
    result = OSDHANDLER_SetClut(APP_CP, defaultClut, &errorCode);
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("Error Set CLUT(CP) %d\n", result);
        return result;
    }

    /**** start CP layer ****/
    result = OSDHANDLER_ShowLayer(APP_CP, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d\n", result);
    }

    FAPI_SYS_PRINT_MSG("\n");

    return result;
}

/*!
*****************************************************************************
** \brief   select layer descriptor
*****************************************************************************
*/
static int32_t select_layer(void)
{
    if(layer_id > 0) {
        return 0;
    }

    /* select layer */
    if(display_flag & DISPLAY_ENABLE_FLAG_OSD1) {
        FAPI_SYS_PRINT_MSG("%sselect OSD1%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        layer_id = 1;
    }
    else if(display_flag & DISPLAY_ENABLE_FLAG_OSD2) {
        FAPI_SYS_PRINT_MSG("%sselect OSD2%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        layer_id = 2;
    }
    else if(display_flag & DISPLAY_ENABLE_FLAG_OSD3) {
        FAPI_SYS_PRINT_MSG("%sselect OSD3%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        layer_id = 3;
    }
    else if(display_flag & DISPLAY_ENABLE_FLAG_OSD4) {
        FAPI_SYS_PRINT_MSG("%sselect OSD4%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        layer_id = 4;
    }
    else if(display_flag & DISPLAY_ENABLE_FLAG_CP) {
        FAPI_SYS_PRINT_MSG("%sselect CP%s\n", APP_ColorFgPurple, APP_ColorFgBlack);
        layer_id = 5;
    }
    else {
        layer_id = 0;
        return -1;
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   move test
*****************************************************************************
*/
static int32_t move_test(void)
{
    OSDHANDLER_LayerParamsT   params;
    int32_t   move_pt;
    uint32_t  errorCode;
    int32_t   result;

    /* select layer */
    if(select_layer() < 0) {
        return -1;
    }

    /**** get scaler parameter ****/
    result = OSDHANDLER_GetLayerParameters( layer_id, &params );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error get scaler parameter: %d\n", result);
        return result;
    }
    params.yRes >>= (1 - read_frame);

    move_pt = layer_px[layer_id] + move_x;
    if(move_pt < 0) {
        layer_px[layer_id] = 0;
        move_x = (rand() & 0x0000001F) + 1;
        if(move_y >= 0) {
            move_y += (rand() & 0x0000007);
        }
        else {
            move_y -= (rand() & 0x0000007);
        }
    }
    else if(move_pt > (out_h_res - params.xRes)) {
        layer_px[layer_id] = out_h_res - params.xRes;
        move_x = - (rand() & 0x0000001F) - 1;
        if(move_y >= 0) {
            move_y += (rand() & 0x0000007);
        }
        else {
            move_y -= (rand() & 0x0000007);
        }
    }
    else {
        layer_px[layer_id] = move_pt;
    }

    move_pt = layer_py[layer_id] + move_y;
    if(move_pt < 0) {
        layer_py[layer_id] = 0;
        move_y =   (rand() & 0x0000001F) + 1;
        if(move_x >= 0) {
            move_x += (rand() & 0x0000007);
        }
        else {
            move_x -= (rand() & 0x0000007);
        }
    }
    else if(move_pt > (out_v_res - params.yRes)) {
        layer_py[layer_id] = out_v_res - params.yRes;
        move_y = - (rand() & 0x0000001F) - 1;
        if(move_x >= 0) {
            move_x += (rand() & 0x0000007);
        }
        else {
            move_x -= (rand() & 0x0000007);
        }
    }
    else {
        layer_py[layer_id] = move_pt;
    }

    /**** set position ****/
    result = OSDHANDLER_Reposition(layer_id, layer_px[layer_id], layer_py[layer_id], &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set position: %d\n", result);
    }

    FAPI_SYS_SLEEP(50);

    return result;
}

/*!
*****************************************************************************
** \brief   resize test
*****************************************************************************
*/
static int32_t resize_test(void)
{
    OSDHANDLER_LayerParamsT  params;
    uint32_t  errorCode;
    int32_t   result;

    FAPI_SYS_SLEEP(APP_TIME_DISPLAY);

    /* select layer */
    if(select_layer() < 0) {
        return -1;
    }

    /**** get scaler parameter ****/
    result = OSDHANDLER_GetLayerParameters( layer_id, &params );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error get scaler parameter: %d\n", result);
        return result;
    }
    params.yRes >>= (1 - read_frame);

    /**** set resize ****/
    result = OSDHANDLER_Rescale( layer_id,
                                 (uint32_t)(resize_ratio_x * params.xRes),
                                 (uint32_t)(resize_ratio_y * params.yRes),
                                 &errorCode );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set resize: %d\n", result);
    }

    FAPI_SYS_SLEEP(APP_TIME_DISPLAY);

    return result;
}

/*!
*****************************************************************************
** \brief   draw rectangle test initialization
*****************************************************************************
*/
static int32_t draw_rect_init(void)
{
    OSDHANDLER_LayerParamsT  params;
    uint32_t  h_size, size, max_h_size, max_size;
    uint32_t  out_h_size, out_v_size;
    uint32_t  errorCode;
    int32_t   result;

    draw_pic_count = 0;

    /* select layer */
    if(select_layer() < 0) {
        return -1;
    }

    /**** stop layer ****/
    (void)OSDHANDLER_StopLayer(layer_id, &errorCode);

    /**** get scaler parameter ****/
    result = OSDHANDLER_GetLayerParameters( layer_id, &params );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error get scaler parameter: %d\n", result);
        return result;
    }
    layer_bpp = params.bpp;

    /* set limit size */
    if(layer_id == 5) {
        max_size   = FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE;
        max_h_size = FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE / display__height;
        out_h_size = out_h_res;
        out_v_size = out_v_res;
    }
    else {
        out_h_size = out_h_res;
        out_v_size = out_v_res << (1 - read_frame);
        if(layer_id == 1) {
            max_size   = FAPEX_VIOUT_OSD1_MAX_SIZE;
            max_h_size = FAPEX_VIOUT_OSD1_MAX_LINE_BYTE;
        }
        else {
            max_size   = FAPEX_VIOUT_OSD2_MAX_SIZE;
            max_h_size = FAPEX_VIOUT_OSD2_MAX_LINE_BYTE;
        }
    }

    /**** set scaler parameter ****/
    start_params.inputWidth  = out_h_size;
    start_params.inputHeight = out_v_size;

    /* adjust horizontal size */
    h_size = (start_params.inputWidth * params.bpp) >> 3;
    while(h_size > max_h_size) {
        h_size >>= 1;
        start_params.inputWidth >>= 1;
    }

    /* adjust vertical size */
    size = h_size * start_params.inputHeight;
    while(size > max_size) {
        size >>= 1;
        start_params.inputHeight >>= 1;
    }

    start_params.xPos = (out_h_size - start_params.inputWidth) >> 1;

    if(layer_id != 5) {
        start_params.yPos = (out_v_size - start_params.inputHeight) >> 1;
    }
    else {
        start_params.yPos = display__y;
    }
    start_params.outputWidth  = 0; // set up automatically
    start_params.outputHeight = 0; // set up automatically

    result = OSDHANDLER_StartLayer(layer_id, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        if(result == FAPEX_VIOUT_ERR_OUT_OF_MEMORY) {
            layer_id = 0;
        }
        FAPI_SYS_PRINT_MSG("Error set scaler parameter: %d\n", result);
        return result;
    }

    /**** set Colour LUT ****/
    result = OSDHANDLER_SetClut(layer_id, defaultClut, &errorCode);
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("Error set CLUT: %d\n", result);
        return result;
    }

    /**** start layer ****/
    result = OSDHANDLER_ShowLayer(layer_id, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting: %d\n", result);
    }
    else {
        FAPI_SYS_PRINT_MSG("Draw random rectangles\n");
        draw_pic_time_start = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw rectangle test
*****************************************************************************
*/
static int32_t draw_rect(void)
{
    uint32_t  x0, y0, width, height, colour;
    uint32_t  max_width, max_height;
    int32_t   result;

    colour = (rand() << 16) | rand();
    if(layer_bpp != 32) {
        colour &= (1 << layer_bpp) - 1;
    }
    max_width  = start_params.inputWidth  - 1;
    max_height = start_params.inputHeight - 1;

    x0 = rand() % max_width;
    y0 = rand() % max_height;
    width  = (rand() % (max_width  - x0)) + 1;
    height = (rand() % (max_height - y0)) + 1;

    /**** draw rectangle ****/
    result = OSDHANDLER_BitBlitFill( layer_id, x0, y0, width, height, colour );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error draw rectangle: %d\n", result);
        return result;
    }
    draw_pic_count += width * height;

    /**** flush plane area ****/
    result = OSDHANDLER_FlushPicture(layer_id);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush plane: %d\n", result);
    }
    return result;
}

/*!
*****************************************************************************
** \brief   draw Font test initialization
*****************************************************************************
*/
static int32_t draw_font_init(void)
{
    OSDHANDLER_LayerParamsT  params;
    uint32_t  errorCode;
    int32_t   result;

    if(layer_id == 0) {
        return -1;
    }

    (void)OSDHANDLER_StopLayer(layer_id, &errorCode);

    result = FONTSERVER_GetFontInfo(appFontserverHandle, &font_info);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error get font information: %d\n", result);
        return result;
    }
    FAPI_SYS_PRINT_MSG("\n----< Font Information >----\n");
    FAPI_SYS_PRINT_MSG(" Font Name   : %s\n", font_info.fontNamePtr);
    FAPI_SYS_PRINT_MSG(" Copyright   : %s\n", font_info.copyrightInfoPtr);
    FAPI_SYS_PRINT_MSG(" weight type : 0x%02X\n", font_info.weightName);
    FAPI_SYS_PRINT_MSG(" slant type  : 0x%02X\n", font_info.slantName);

#if (USE_FREETYPE == 0)  /* #649 */
    font_draw_x0     = 0;
    font_draw_y0     = 0;
#else
    font_draw_x0     = font_info.maxCharWidth;
    font_draw_y0     = font_info.maxCharHeight;
#endif
    font_draw_code   = 0x20;
    font_draw_colour = 0;
    font_draw_frame_count = 0;
    font_draw_code_count  = 0;

    (void)FONTSERVER_SetLayerId(appFontserverHandle, layer_id);

    /**** set scaler parameter ****/
    if(layer_id == 1) {
        (void)OSDHANDLER_SetColourMode(layer_id, OSDHANDLER_OSD_MODE_CLUT_4BIT, &errorCode);
    }
    else {
        (void)OSDHANDLER_SetColourMode(layer_id, OSDHANDLER_OSD_MODE_CLUT_8BIT, &errorCode);
    }
    (void)OSDHANDLER_SetLayoutMode(layer_id, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.inputWidth   = out_h_res;
    start_params.inputHeight  = out_v_res << (1 - read_frame);
    start_params.xPos         = 0;
    start_params.yPos         = 0;
    start_params.outputWidth  = 0; // set up automatically
    start_params.outputHeight = 0; // set up automatically

    result = OSDHANDLER_StartLayer(layer_id, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setup the OSD: %d\n", result);
        return result;
    }

    /**** start layer ****/
    result = OSDHANDLER_ShowLayer(layer_id, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD: %d\n", result);
        return result;
    }

    /**** get scaler parameter ****/
    result = OSDHANDLER_GetLayerParameters( layer_id, &params );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error get scaler parameter: %d\n", result);
        return result;
    }
    layer_bpp    = params.bpp;
    layer_width  = params.xRes;
    layer_height = params.yRes;

    draw_pic_time_start = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);

    return result;
}

/*!
*****************************************************************************
** \brief   draw Font test
*****************************************************************************
*/
static int32_t draw_font(void)
{
    uint8_t   ch[2];
    int32_t   ch_width;
    int32_t   result = 0;

    /* check position */
    if((font_draw_x0 + font_info.maxCharWidth) >= layer_width) {
#if (USE_FREETYPE == 0)  /* #649 */
        font_draw_x0  = 0;
#else
        font_draw_x0  = font_info.maxCharWidth;
#endif
        font_draw_y0 += font_info.maxCharHeight;
    }
    if((font_draw_y0 + font_info.maxCharHeight) >= layer_height) {
#if (USE_FREETYPE == 0)  /* #649 */
        font_draw_x0   = 0;
        font_draw_y0   = 0;
#else
        font_draw_x0   = font_info.maxCharWidth;
        font_draw_y0   = font_info.maxCharHeight;
#endif
        font_draw_code = 0x20;
        font_draw_frame_count ++;
        if((font_draw_frame_count & 0x01) != 0) {
            font_draw_colour = 0;
        }
        else {
            font_draw_colour = (font_draw_frame_count >> 1) & ((1 << layer_bpp) - 1);
        }
//      FAPI_SYS_SLEEP(APP_TIME_SETUP);
        return 1;
    }

    ch[1] = '\0';

    do {
        ch[0] = font_draw_code ++;

        ch_width = FONTSERVER_GetStringPixelWidth(appFontserverHandle, ch, 1,
                                                  &result);
        if(ch_width > 0) {
            /* print character */
            (void)FONTSERVER_SetColour(appFontserverHandle, font_draw_colour);
            (void)FONTSERVER_PrintChar(appFontserverHandle, ch,
                                       font_draw_x0, font_draw_y0, &result);
            if (result != FAPI_OK)
            {
                FAPI_SYS_PRINT_MSG("Error print character: (%d, %d) %d\n",
                                   font_draw_x0, font_draw_y0, result);
                return result;
            }
        }

        /* next character number & colour */
        if(font_draw_code == 0) {
            font_draw_code = 0x20;
            if((font_draw_frame_count & 0x01) == 0) {
                font_draw_colour ++;
                font_draw_colour &= ((1 << layer_bpp) - 1);
            }
        }
        else if(font_draw_code == 0x7F) {
            font_draw_code = 0xA0;
        }
    } while(ch_width == 0);

    /* next horizontal point */
    font_draw_x0 += ch_width;
    font_draw_code_count ++;

    return 0;
}

/*!
*****************************************************************************
** \brief   draw clut test
*****************************************************************************
*/
static int32_t draw_clut(void)
{
    uint32_t  out_h_size, out_v_size;
    uint32_t  max_width, max_size;
    uint32_t  step_x, step_y, width, height;
    int32_t   x, y;
    uint32_t  colour;
    uint32_t  errorCode;
    int32_t   result;

    /* select layer */
    if(select_layer() < 0) {
        return -1;
    }

    /**** stop layer ****/
    (void)OSDHANDLER_StopLayer(layer_id, &errorCode);

    /**** set scaler parameter ****/
    if(layer_id == 5) {
        (void)OSDHANDLER_SetColourMode(layer_id, OSDHANDLER_CURSOR_MODE_8BIT, &errorCode);
        out_h_size = out_h_res;
        out_v_size = out_v_res;
        max_width = FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE / (4 * 16);
        max_size  = FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE;
    }
    else {
        (void)OSDHANDLER_SetColourMode(layer_id, OSDHANDLER_OSD_MODE_CLUT_8BIT, &errorCode);
        out_h_size = out_h_res;
        out_v_size = out_v_res << (1 - read_frame);
        if(layer_id == 1) {
            max_width = FAPEX_VIOUT_OSD1_MAX_LINE_BYTE;
            max_size  = FAPEX_VIOUT_OSD1_MAX_SIZE;
        }
        else {
            max_width = FAPEX_VIOUT_OSD2_MAX_LINE_BYTE;
            max_size  = FAPEX_VIOUT_OSD2_MAX_SIZE;
        }
    }
    width = (out_h_size >> 4) << 4;
    while(width > max_width) {
        width = (width >> 5) << 4;
    }
    max_size /= width;

    height = (out_v_size >> 4) << 4;
    while(height > max_size) {
        height = (height >> 5) << 4;
    }
    (void)OSDHANDLER_SetLayoutMode(layer_id, OSDHANDLER_OSD_LAYOUT_CbYCrA, &errorCode);
    start_params.inputWidth  = width;
    start_params.inputHeight = height;
    start_params.xPos        = (out_h_size - width)  >> 1;
    if(layer_id != 5) {
        start_params.yPos = (out_v_size - height) >> 1;
    }
    else {
        start_params.yPos = display__y;
    }
    start_params.outputWidth  = 0; // set up automatically
    start_params.outputHeight = 0; // set up automatically

    result = OSDHANDLER_StartLayer(layer_id, &start_params, &errorCode);
    if (result != FAPI_OK)
    {
        if(result == FAPEX_VIOUT_ERR_OUT_OF_MEMORY) {
            layer_id = 0;
        }
        FAPI_SYS_PRINT_MSG("Error set scaler parameter: %d\n", result);
        return result;
    }

    if(clut_type != 0) {
        /**** set Colour LUT ****/
        result = OSDHANDLER_SetClut(layer_id, defaultClut, &errorCode);
        if(result < 0) {
            FAPI_SYS_PRINT_MSG("Error set CLUT: %d\n", result);
            return result;
        }
    }

    /**** start layer ****/
    result = OSDHANDLER_ShowLayer(layer_id, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the OSD/CP: %d\n", result);
        return result;
    }

    step_x = width  >> 4;
    step_y = height >> 4;
    colour = 0;

    for(y=0; y<height; y+=step_y) {
        for(x=0; x<width; x+=step_x) {
            /**** draw rectangle ****/
            result = OSDHANDLER_BitBlitFill(layer_id, x, y, step_x, step_y, colour);
            if (result != FAPI_OK)
            {
                FAPI_SYS_PRINT_MSG("Error draw rectangle: %d\n", result);
                return result;
            }
            colour ++;
        }
    }

    /**** flush plane area ****/
    result = OSDHANDLER_FlushPicture(layer_id);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error flush plane: %d\n", result);
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   draw sample main program
*****************************************************************************
*/
static int32_t draw_sample(uint8_t font_size)
{
    FONTSERVER_InitParamsT  fontserver_init;
    FONTSERVER_OpenParamsT  fontserver_open;
    int32_t   pattern_no, pre_pattern_no;
    uint32_t  errorCode;
    int32_t   result;
    int32_t   osd_id1, osd_id2;

    memset(&start_params,    0, sizeof(OSDHANDLER_StartParamsT));
    memset(&fontserver_init, 0, sizeof(FONTSERVER_InitParamsT));
    memset(&fontserver_open, 0, sizeof(FONTSERVER_OpenParamsT));
    memset(osdInfo1,         0, sizeof(osdInfo1));
    memset(osdInfo2,         0, sizeof(osdInfo2));
    memset(&bmpHdr,          0, sizeof(DRAW_BmpFileHeadT));

    // init the FONTDRIVER:
#if (USE_FREETYPE == 1)
    appEngineData[0] = (uint32_t)FONTSERVER_ENGINE_FT_INTERFACE_PTR;
#else
    appEngineData[0] = (uint32_t)FONTSERVER_ENGINE_FBF_INTERFACE_PTR;
#endif
    fontserver_init.numEngines = 1;
    fontserver_init.enginesPtr = appEngineData;
    FAPI_SYS_PRINT_MSG("Initialzing FONTSERVER...");
    result = FONTSERVER_Init(&fontserver_init);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error: %d\n", result);
        return(1);
    }

    // Register a font with the FONTSERVER:
    FAPI_SYS_PRINT_MSG("Registering FBF font... ");
    appFontIndex = FONTSERVER_RegisterFont(
#if (USE_FREETYPE == 1)
                                           (uint32_t*)Fujitsu_Zwo_Bold_DATA,
                                           Fujitsu_Zwo_Bold_SIZE,
#else
                                           (uint32_t*)Fujitsu_Zwo_Bold_FBF_24_DATA,
                                           Fujitsu_Zwo_Bold_FBF_24_SIZE,
#endif
                                           &result);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error: %d\n", result);
        return(-1);
    }

    // Open a FONTSERVER handle:
    fontserver_open.version = FONTSERVER_VERSION;
    FAPI_SYS_PRINT_MSG("Opening FONTSERVER handle... ");
    appFontserverHandle = FONTSERVER_Open(&fontserver_open, &result);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error: %d\n", result);
        return(-1);
    }

    // Set up FONTSERVER parameters:
    result = 0;
    result |= FONTSERVER_SetFont(appFontserverHandle, appFontIndex);
    result |= FONTSERVER_SetSize(appFontserverHandle, font_size);
    result |= FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    result |= FONTSERVER_SetLayerId(appFontserverHandle, APP_CP);
    result |= FONTSERVER_SetEncoding(appFontserverHandle, FONTSERVER_ENCODING_8_BIT);
    result |= FONTSERVER_SetStyle(appFontserverHandle, FONTSERVER_STYLE_NORMAL);
    result |= FONTSERVER_SetAntiAliasing(appFontserverHandle, FONTSERVER_ANTI_ALIAS_OFF);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setting up fontserver configuration.\n");
        return(-1);
    }

    // Initialize DRAW subsystem:
    FAPI_SYS_PRINT_MSG("Initialzing DRAW...");
    result = DRAW_Init();
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error: %d\n", result);
        return(1);
    }

    // start the Cursor Plane (only internally; does not display it yet):
    start_params.xPos         = display__x;
    start_params.yPos         = display__y;
    start_params.inputWidth   = display__width;
    start_params.inputHeight  = display__height;
    start_params.outputWidth  = 0;
    start_params.outputHeight = 0;
    start_params.frameBased   = read_frame;

    result = OSDHANDLER_StartLayer(APP_CP, &start_params, &errorCode);
   
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while starting the Cursor Plane: %d errorCode: 0x%08X\n" ,\
                            result, errorCode);
        return(1);
    }

    // print a welcome message:
    strcpy((char_t*)osdInfo1, "This is a demo application.");
    FAPI_SYS_PRINT_MSG("%s %s\n", APP_PREFIX, osdInfo1);
    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("FONTSERVER_PrintString() error: %d\n", result);
        return(1);
    }
    display_flag |= DISPLAY_ENABLE_FLAG_CP;

    /**** enable CP layer ****/
    (void)OSDHANDLER_ShowLayer(APP_CP, &errorCode);

    FAPI_SYS_SLEEP(APP_TIME_DISPLAY);

    (void)FONTSERVER_SetColour(appFontserverHandle, APP_TXT_BLACK);
    (void)FONTSERVER_PrintString(appFontserverHandle, osdInfo1, APP_FULL_STRING,
                                 20+PRINT_STR_OFFSET_X, 20+PRINT_STR_OFFSET_Y, &result);
    display_flag &= ~DISPLAY_ENABLE_FLAG_CP;

    (void)OSDHANDLER_ShowLayer(APP_CP, &errorCode);

    osd_id1 = 1;    osd_id2 = 2;    // TO TEST other osd, change this
    pattern_no = pre_pattern_no = 0;
    while(appDrawEnable >= 0)
    {
        switch(pattern_no) {
            case 0:
                draw_test0(osd_id1);
                break;
            case 1:
                draw_test1(osd_id2);
                break;
            case 2:
                draw_test2(osd_id1);
                break;
            case 3:
                draw_test3(osd_id2);
                break;
            case 4:
                draw_test4(osd_id1);
                break;
            case 5:
                draw_test5(osd_id1);
                break;
            case 6:
                draw_test6(osd_id1);
                break;
            case 7:
                draw_test7(osd_id1);
                break;
            case 8:
                draw_test8(osd_id1);
                break;
            case 9:
                draw_test9(osd_id1);
                break;
            case DRAW_TEST_PATTERN_MOVE:
            case DRAW_TEST_PATTERN_RESIZE:
            case DRAW_TEST_PATTERN_RECTANGLE_START:
            case DRAW_TEST_PATTERN_RECTANGLE:
            case DRAW_TEST_PATTERN_FONT_START:
            case DRAW_TEST_PATTERN_FONT:
            case DRAW_TEST_PATTERN_CLUT:
                break;
            default:
                draw_test_reset(osd_id1, osd_id2);
        }

        if(appDrawContrl == 1) {
            /* suspend appDrawThreadHandle */
            FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );
        }

        /* test pattern check */
        if(appDrawEnable == 0) {
            FAPI_SYS_SLEEP(APP_TIME_SETUP);
        }
        else if(appDrawEnable == 1) {  /* end command */
            uint64_t  time_value;

            /* optional draw end */
            if( (pattern_no == DRAW_TEST_PATTERN_RECTANGLE)
             && (draw_pic_count != 0) ) {
                time_value = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
                time_value -= draw_pic_time_start;
                FAPI_SYS_PRINT_MSG("draw rate %.3f [Mpixel/sec]\n\n",
                                   (float)draw_pic_count / (1000  * time_value));
            }
            else if( (pattern_no == DRAW_TEST_PATTERN_FONT)
                  && (font_draw_code_count != 0) ) {
                time_value = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
                time_value -= draw_pic_time_start;
                FAPI_SYS_PRINT_MSG("draw rate %.3f [Kchar/sec]\n\n",
                                   (float)font_draw_code_count / time_value);
            }
            else if( (pattern_no == DRAW_TEST_PATTERN_CLUT)
                  && (layer_id == 5) ) {
                /* for reset test pattern */
                pre_pattern_no = 9;
            }
            start_params.outputWidth  = 0;
            start_params.outputHeight = 0;
            pattern_no    = pre_pattern_no;
            appDrawEnable = 0;
            if(layer_id != 0) {
                result = OSDHANDLER_SetFillColour(layer_id, 0);
                if (result != FAPI_OK)
                {
                    FAPI_SYS_PRINT_MSG("Error set fill colour. %d\n", result);
                }
                layer_id = 0;
            }
        }
        else {
            if(pattern_no <= DRAW_TEST_PATTERN_BASIC_MAXNUM) {
                pre_pattern_no = pattern_no;
            }
            if(pattern_no < DRAW_TEST_PATTERN_RECTANGLE_START) {
                pattern_no = appDrawEnable;
            }
        }

        switch(pattern_no) {
            case 0:
            case 9:
                break;
            case 1:
                clear_test1(osd_id1);
                break;
            case 2:
                clear_test2(osd_id2);
                break;
            case 3:
                clear_test3(osd_id1, osd_id2);
                break;
            case 4:
                clear_test4(osd_id1);
                break;
            case 5:
                clear_test5(osd_id1);
                break;
            case 6:
                clear_test6(osd_id1);
                break;
            case 7:
                clear_test7(osd_id1);
                break;
            case 8:
                clear_test8();
                break;
            case DRAW_TEST_PATTERN_MOVE:
                result = move_test();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'move' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                    appDrawEnable = 1;
                }
                pattern_no --;
                break;
            case DRAW_TEST_PATTERN_RESIZE:
                resize_test();
                appDrawEnable = 1;
                pattern_no --;
                break;
            case DRAW_TEST_PATTERN_RECTANGLE_START:
                result = draw_rect_init();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'rect' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                    appDrawEnable = 1;
                }
                else {
                    result = draw_rect();
                    if(result < 0) {
                        FAPI_SYS_PRINT_MSG("%sThe 'rect' command was canceled!\n%s",
                                           APP_ColorFgRed, APP_ColorFgBlack);
                        appDrawEnable = 1;
                    }
                }
                break;
            case DRAW_TEST_PATTERN_RECTANGLE:
                result = draw_rect();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'rect' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                    appDrawEnable = 1;
                }
                pattern_no --;
                break;
            case DRAW_TEST_PATTERN_FONT_START:
                result = draw_font_init();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'font' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                    appDrawEnable = 1;
                }
                else {
                    result = draw_font();
                    if(result < 0) {
                        FAPI_SYS_PRINT_MSG("%sThe 'font' command was canceled!\n%s",
                                           APP_ColorFgRed, APP_ColorFgBlack);
                        appDrawEnable = 1;
                    }
                }
                break;
            case DRAW_TEST_PATTERN_FONT:
                result = draw_font();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'font' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                    appDrawEnable = 1;
                }
                pattern_no --;
                break;
            case DRAW_TEST_PATTERN_CLUT:
                result = draw_clut();
                if(result < 0) {
                    FAPI_SYS_PRINT_MSG("%sThe 'clut' command was canceled!\n%s",
                                       APP_ColorFgRed, APP_ColorFgBlack);
                }
                appDrawContrl = 1;  /* pause */
                pattern_no --;
                break;
            default:
                pattern_no = -1;
        }
        pattern_no ++;

        if(appDrawEnable == 0) {
            FAPI_SYS_SLEEP(APP_TIME_BREAK);
        }
    }

    /**** stop VIOUT ****/
    result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)fd_VIOUT,
                                FAPEX_IOCCOM_STOP,
                                (FAPEXIOCARG3_T)NULL );
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_ERR("Error while stopping the VIOUT: %d\n", result);
    }
    return result;
}

//***************************************************************************
//***************************************************************************
//** The application's  Draw thread.
//***************************************************************************
//***************************************************************************
static void appDrawThreadFunction(void* optArg)
{
    OSDHANDLER_InitParamsT   handler_params;
    VAL_InitParamsT          initParams;
    uint8_t   font_size;
    uint8_t   tsdcnt = 0;
    uint32_t  errorCode,error_code;
    int32_t   result;
    int32_t  fd_CLKPWR;

    VAL_UpParamsT     vidParams;
    VAL_OpenParamsT   vidOpenParams;
    VAL_DecoderTypeT  decoder;

    FAPEX_TSD_OpenParamStrT   tsdOpenParam;
    memset (&tsdOpenParam, 0, sizeof(FAPEX_TSD_OpenParamStrT));

    for ( tsdcnt=0;tsdcnt<FAPEX_TSD_MAX;tsdcnt++ )
    {
        if ( fd_TSD[tsdcnt]<0 )   //open TSD first time
        {
            /* input settings */
            tsdOpenParam.useStc           = 1;
            tsdOpenParam.tsSelect         = FAPEX_TSD_INPUT_CH_A;
            tsdOpenParam.syncByteDistance = 0;
            tsdOpenParam.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
            tsdOpenParam.oflags           = 0;

            fd_TSD[tsdcnt] = FAPEX_TSD_Open( FAPEX_TSD0 + tsdcnt, O_RDWR, &tsdOpenParam );
            if(fd_TSD[tsdcnt] < 0)
            {
                error_code = -1;
                FAPI_SYS_PRINT_MSG("FAPEX_TSD_Open...error\n");
            }
        }
    }

    /* Val_Init */
    FAPI_SYS_PRINT_MSG("Initializing VAL...");
    memset (&initParams, 0, sizeof(initParams));
    initParams.threadPriority = THREAD_PRIO_HDMI;

    error_code = VAL_Init(&initParams);
    if (error_code)
    {
        FAPI_SYS_PRINT_MSG("%s VAL_Init(): (%d) \n", APP_PREFIX, error_code);
    }
    
    // VAL Goup
    vidParams.hdSystemType                    = VAL_1920x1080x50I;
    vidParams.sdSystemType                    = VAL_SD_PAL;
    vidParams.displayAspectRatio[VAL_VOUT_SD] = FAPEX_VIOUT_DISPLAY_AR_NONE;
    vidParams.displayAspectRatio[VAL_VOUT_HD] = FAPEX_VIOUT_DISPLAY_AR_16_9;
    vidParams.scalingMethod                   = FAPEX_VIOUT_SCALER_MODE_PLBOX;

    appValDataPtr = VAL_GoUp (&vidParams, &result);
    if(appValDataPtr == NULL) {
        FAPI_SYS_PRINT_MSG("%s VAL_GoUp failed. %d\n", APP_PREFIX, result);
        return;
    }

    decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
    decoder.system        = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
    vidOpenParams.environment = VAL_ENV_LIVE;
    vidOpenParams.decoder     = decoder;
    vidOpenParams.enableAfd   = 1;

    appValOpenPtr = VAL_Open(appValDataPtr, &vidOpenParams, &result);
    if(appValOpenPtr == NULL) {
        FAPI_SYS_PRINT_MSG("%s VAL_Open failed. %d\n", APP_PREFIX, result);
    }

    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("Error: %d\n", result);
    }

    // Open CLKPWR
    fd_CLKPWR = FAPEX_CLKPWR_Open( DRV_ID_CLKPWR, O_RDWR, (FAPEXOPENARG3_T)NULL );
    /* power on all 6 video DACs. */
    if(fd_CLKPWR > 0)
    {
        int32_t result;
        uint64_t component_mask = FAPEX_CLKPWR_DAC0 | FAPEX_CLKPWR_DAC1 | FAPEX_CLKPWR_DAC2 |
                                  FAPEX_CLKPWR_DAC3 | FAPEX_CLKPWR_DAC4 | FAPEX_CLKPWR_DAC5;
        result = FAPEX_CLKPWR_Ioctl(fd_CLKPWR, FAPEX_IOCCOM_CLKPWR_START,
                                    (FAPEXIOCARG3_T)&component_mask);
        if(result < 0)
            FAPI_SYS_PRINT_MSG("ERROR:[main.c] Video DACS power up %d\n",result);
    }
    // get the system resolution
    // and set the font depending on the system mode.
    // Select smaller fonts for lower resoluted system modes,
    // this is necessary since the Cursor Plane can not be rescaled.
    switch(hd_system_mode)
    {
        case(VAL_1920x1080x2397P):
        case(VAL_1920x1080x24P):
        case(VAL_1920x1080x25P):
        case(VAL_1920x1080x2997P):
        case(VAL_1920x1080x30P):
            out_h_res = 1920;
            out_v_res = 1080;
            read_frame = 1;
            font_size  = 24;
            display__x = 700;
            display__y = 800;
            display__width = 512;
            display__height = 64;
            break;
        case(VAL_1280x720x2397P):
        case(VAL_1280x720x24P):
        case(VAL_1280x720x25P):
        case(VAL_1280x720x2997P):
        case(VAL_1280x720x30P):
        case(VAL_1280x720x50P):
        case(VAL_1280x720x5994P):
        case(VAL_1280x720x60P):
            out_h_res = 1280;
            out_v_res =  720;
            read_frame = 1;
            font_size  = 24;
            display__x = 380;
            display__y = 530;
            display__width = 512;
            display__height = 64;
            break;
        case(VAL_720x576x50I):
            out_h_res =  720;
            out_v_res =  288;
            read_frame = 0;
            font_size  = 18;
            display__x = 160;
            display__y = 210;
            display__width = 384;
            display__height = 64;
            break;
        case(VAL_720x576x50P):
            out_h_res =  720;
            out_v_res =  576;
            read_frame = 1;
            font_size  = 18;
            display__x = 160;
            display__y = 426;
            display__width = 384;
            display__height = 64;
            break;
        case(VAL_720x480x5994I):
            out_h_res =  720;
            out_v_res =  280;
            read_frame = 0;
            font_size  = 18;
            display__x = 160;
            display__y = 180;
            display__width = 384;
            display__height = 64;
            break;
        case(VAL_720x480x5994P):
            out_h_res =  720;
            out_v_res =  480;
            read_frame = 1;
            font_size  = 18;
            display__x = 160;
            display__y = 360;
            display__width = 384;
            display__height = 64;
            break;
        case(VAL_1920x1080x50I):
        case(VAL_1920x1080x5994I):
        case(VAL_1920x1080x60I):
        default:
            out_h_res = 1920;
            out_v_res =  540;
            read_frame = 0;
            font_size  = 24;
            display__x = 700;
            display__y = 400;
            display__width = 512;
            display__height = 64;
            break;
    }

    memset(&handler_params, 0, sizeof(OSDHANDLER_InitParamsT));

    // init the OSDHANDLER:
    handler_params.osd1InitParams.inUse          = 1;
    handler_params.osd1InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_4BIT;
    handler_params.osd1InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd1InitParams.defaultClutPtr = defaultClut;
    handler_params.osd1InitParams.useGlobalAlpha = 1;
    handler_params.osd1InitParams.globalAlpha    = 255;
    
    handler_params.osd2InitParams.inUse          = 1;
    handler_params.osd2InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_8BIT;
    handler_params.osd2InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd2InitParams.defaultClutPtr = defaultClut;
    handler_params.osd2InitParams.useGlobalAlpha = 1;
    handler_params.osd2InitParams.globalAlpha    = 255;
    
    handler_params.osd3InitParams.inUse          = 1;
    handler_params.osd3InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_8BIT;
    handler_params.osd3InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd3InitParams.defaultClutPtr = defaultClut;
    handler_params.osd3InitParams.useGlobalAlpha = 1;
    handler_params.osd3InitParams.globalAlpha    = 255;
    
    handler_params.osd4InitParams.inUse          = 1;
    handler_params.osd4InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_8BIT;
    handler_params.osd4InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd4InitParams.defaultClutPtr = defaultClut;
    handler_params.osd4InitParams.useGlobalAlpha = 1;
    handler_params.osd4InitParams.globalAlpha    = 255;
    
    handler_params.cpInitParams.inUse            = 1;
    handler_params.cpInitParams.colourMode       = OSDHANDLER_CURSOR_MODE_1BIT;
    handler_params.cpInitParams.osdLayout        = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.cpInitParams.defaultClutPtr   = defaultClut;
    handler_params.cpInitParams.useGlobalAlpha   = 1;
    handler_params.cpInitParams.globalAlpha      = 255;

    result = OSDHANDLER_Init(&handler_params, &errorCode);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while initializing the OSDHANDLER. %d\n", result);
    }
    else {
        /**** draw test sample ****/
        draw_sample(font_size);

        appDrawEnable = -1;

        /**** close OSDHANDLER ****/
        result = OSDHANDLER_Exit();
        if(result != 0) {
            FAPI_SYS_PRINT_MSG("%s OSDHANDLER_Exit failed. %d\n", APP_PREFIX, result);
        }
        if(appValOpenPtr != NULL) {
            result = VAL_Close(appValOpenPtr);
            if(result != 0) {
                FAPI_SYS_PRINT_MSG("%s VAL_Close failed. %d\n", APP_PREFIX, result);
            }
        }
        result = VAL_GoDown(appValDataPtr);
        if(result != 0) {
            FAPI_SYS_PRINT_MSG("%s VAL_GoDown failed. %d\n", APP_PREFIX, result);
        }
    }
}

/**
*******************************************************************************
**
** \brief Register a new command.
**
** This function registers the given command to the internal command table.
** An already existing command will be unregistered first and replaced by the
** given parameters. The buffers for the brief and the description texts will
** be dynamically allocated to avoid buffer overflows.
**
** \param command     The command (string) to register.
** \param brief       A short (one line) help message.
** \param description The full command description, maybe NULL.
** \param function    The function to be called when the user enters the
**                    "command" (with optional parameters), this functions
**                    must accept the argc, argv argument pair for additional
**                    optional parameters [strings].
**
*******************************************************************************
*/
int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler, command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}

//***************************************************************************
//***************************************************************************
//** The application's  Shell thread.
//***************************************************************************
//***************************************************************************
static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT,
                                                             500,
                                                             NULL );
    if( !appShellInputHandler ) {
        exit(1);
    }
#else
    char_t    args_buff[APP_CommandStringSize];
    int32_t   i, c;

    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n"   \
        "as optional argument to ?\n"                                           \
        "Example: ?       displays all available commands, one per line\n"     \
        "Example: ? !     displays a more detailed help for the ! command\n" \
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n" \
    );
#endif

    (void)appRegisterCommand(
        "exit",
        APP_ExitCommand,
        "Exit application.",
        "Usage: exit\n"
    );

    (void)appRegisterCommand(
        "pause",
        APP_PauseCommand,
        "Pause the display. (option)",
        "Usage: pause\n"
    );

    (void)appRegisterCommand(
        "move",
        APP_MoveCommand,
        "Move the window. (option)",
        "Usage: move\n" \
        "       move [init_delta_x] [init_delta_y]\n"
    );

    (void)appRegisterCommand(
        "resize",
        APP_ResizeCommand,
        "Resize the window.",
        "Usage: resize [scale_ratio]\n" \
        "       resize [x_scale_ratio] [y_scale_ratio]\n"
    );

    (void)appRegisterCommand(
        "rect",
        APP_RectangleCommand,
        "Draw rectangle. (option)",
        "Usage: rect\n" \
        "       rect <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4, 5:CP)>\n"
    );

    (void)appRegisterCommand(
        "font",
        APP_FontCommand,
        "Draw Font. (option)",
        "Usage: font <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4)>\n"
    );

    (void)appRegisterCommand(
        "clut",
        APP_ClutCommand,
        "Draw CLUT. (option)",
        "Usage: clut <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4, 5:CP)>\n"
    );

    (void)appRegisterCommand(
        "hilight",
        APP_HilightCommand,
        "Hilight On/OFF switch.",
        "Usage: hilight\n"
    );

    (void)appRegisterCommand(
        "alpha",
        APP_AlphaCommand,
        "set Alpha value",
        "Usage: alpha <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4, 5:CP)> [alpha_value]\n"
    );

    (void)appRegisterCommand(
        "fade",
        APP_FadeCommand,
        "Fading switch",
        "Usage: fade <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4, 5:CP)>\n"
    );

    (void)appRegisterCommand(
        "enable",
        APP_EnableCommand,
        "Enable/Disable switch",
        "Usage: enable <layer_index(1:OSD1, 2:OSD2, 3:OSD3, 4:OSD4, 5:CP)>\n"
    );

    (void)appRegisterCommand(
        "back",
        APP_BackgroundCommand,
        "set Background colour",
        "Usage: back [Y] [Cr] [Cb]\n"
    );

    (void)appRegisterCommand(
        "order",
        APP_OrderCommand,
        "set Plane Order",
        "Usage: order\n" \
        "       order [OSD1_order] [OSD2_order] [OSD3_order] [OSD4_order] [CP_order]\n"
    );

    (void)appRegisterCommand(
        "end",
        APP_EndCommand,
        "End optional test pattern.",
        "Usage: end\n"
    );

    appDrawThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_DRAW_THREAD_STACKSIZE,
            APP_DRAW_THREAD_PRIORITY,
            appDrawThreadFunction,
            0, 0, APP_DRAW_THREAD_NAME );
    if(appDrawThreadHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG( "Error: FAPEX_SYS_CreateThread (draw)\n" );
        exit(1);
    }

#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#else
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );

    while(appDrawEnable >= 0) {
        i = 0;

        /* get command string */
        while( !feof(stdin) ) {
            c = getchar();
            args_buff[i] = c;
            if(c == '\n') {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) ) {
                i --;
            }
            else if(i < (APP_CommandStringSize - 1)) {
                i ++;
            }
        }
        SHELL_DoCommand( args_buff );

        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */

    FAPEX_SYS_WaitEndThread( appDrawThreadHandle );
}

#ifdef _LINUX_FAPI
//***************************************************************************
static void print_usage(char *name)
{
    FAPI_SYS_PRINT_MSG("usage: %s [-s hd_mode (sd_mode)]\n", name);
}
#endif

//***************************************************************************
//***************************************************************************
//** Main Program
//***************************************************************************
//***************************************************************************
int main(int argc, char **argv)
{
    int32_t  i;
    hd_system_mode = APP_HD_SYSTEM_MODE;
    sd_system_mode = APP_SD_SYSTEM_MODE;

#ifndef _LINUX_FAPI
    {
        int32_t  result;

        result = FAPEX_common_init();
        if ( result != FAPI_OK ) {
            exit( result );
        }
    }
#else
    /* initial input settings */
    if(argc > 1) {

        for(i=1; i<argc; i++) {
            if(strcmp("-s", argv[i]) == 0) {
                i ++;
                hd_system_mode = atoi(argv[i]);
                if(argc > (i+1)) {
                    if( (strlen(argv[i+1]) == 1)
                     && (isdigit(*argv[i+1])) ) {
                        i ++;
                        sd_system_mode = atoi(argv[i]);
                    }
                }
            }
            else {
                print_usage(argv[0]);
                exit(1);
            }
        }
    }
#endif

    FAPI_SYS_PRINT_MSG("\033[2J\n\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# osd demo application (FAPex version)\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# program version: %s\n", APP_VERSION);
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2007-2011 by Fujitsu Semiconductor Europe GmbH\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2008-2011 by Fujitsu Semiconductor Limited\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("\n");

    appShellThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_SHELL_THREAD_STACKSIZE,
            APP_SHELL_THREAD_PRIORITY,
            appShellThreadFunction,
            0, 0, APP_SHELL_THREAD_NAME );
    if(appShellThreadHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG( "Error: FAPEX_SYS_CreateThread (shell)\n" );
        exit(1);
    }

    FAPEX_SYS_StartMultitasking();

    FAPEX_SYS_WaitEndThread( appShellThreadHandle );
    if (data_buff != NULL) {
        FAPI_SYS_FREE(data_buff);
    }

    for ( i=0;i<FAPEX_TSD_MAX;i++ ) {
        if (fd_TSD[i] > 0) {
            FAPEX_TSD_Close(fd_TSD[i]);
        }
    }

    return 0;
}
