/*
********************************************************************************
**
** \file        ./sample_application/fapex_av_demo_fs/src/gui.c
**
** \version     $Id$
**
** \brief       GUI with GTK+.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2009-2010 by Fujitsu Semiconductor Limited
**
********************************************************************************
*/
#define GTK_DEBUG  0

#ifdef GUI_ENABLE_STANDALONE
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>

#define DISABLE_GUI_FULL_SCREEN_WINDOW

typedef enum
{
    VAL_ENV_NONE = -1,
    VAL_ENV_LIVE =  0,
    VAL_ENV_PLAY,
    VAL_ENV_NUMS,
} VAL_EnvironmentT;

#else

#include <fapex/abst_types.h>
#include <fapex/abst_libir.h>
#include <fapex/abst_libvidec.h>
#include <fapex/abst_libauout.h>
#include <val/val.h>
#include "servicelist.h"
#include "gui.h"

#endif /* GUI_ENABLE_STANDALONE */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>

//******************************************************************************
//******************************************************************************
//** Defines and Macros
//******************************************************************************
//******************************************************************************

#if GTK_DEBUG
#define GTK_DPRINT(level, formar, arg...)   \
        {                                   \
            if(GTK_DEBUG >= level)          \
            {                               \
                g_print(formar, ## arg);    \
            }                               \
        }
#else
#define GTK_DPRINT(level, formar, arg...)
#endif

#ifndef DISABLE_GUI_FULL_SCREEN_WINDOW
#define MAIN_WINDOW_WIDTH   gdk_screen_width()
#define MAIN_WINDOW_HEIGHT  gdk_screen_height()
#define MAIN_WINDOW_BORDER  (gdk_screen_width() >> 6)
#define COMBO_ITEM_WIDTH_0  -1
#define COMBO_ITEM_WIDTH_1  -1
#else
#define MAIN_WINDOW_WIDTH   500
#define MAIN_WINDOW_HEIGHT  500
#define MAIN_WINDOW_BORDER  10
#define COMBO_ITEM_WIDTH_0  100
#define COMBO_ITEM_WIDTH_1  120
#endif

#define APP_PROGRESSBAR_UPDATE_INTERVAL          500    /* [msec] */

enum
{
    PAGE_TYPE_UNKNOWN = -1,
    PAGE_TYPE_STREAM,
    PAGE_TYPE_FILE,
    PAGE_TYPE_MAX_NUM
};

enum
{
    LIST_COLUMN_INFO = 0,
    LIST_COLUMN_NAME,
    LIST_COLUMN_SIZE,
    LIST_COLUMN_DATE,
    LIST_COLUMNS_MAX_NUM
};

enum
{
    ITEM_TYPE_UNKNOWN = -1,
    ITEM_TYPE_DIRECTORY,
    ITEM_TYPE_FILE
};

enum
{
    INFO_COLUMN_CATEGORY = 0,
    INFO_COLUMN_KEY,
    INFO_COLUMNS_MAX_NUM
};

enum
{
    PLAYER_STATUS_RESET = -1,
    PLAYER_STATUS_STOPPED,
    PLAYER_STATUS_READY,
    PLAYER_STATUS_RUNNING
};

#define GUI_WINDOW_TABLE_LAYOUT_VMAX        20
#define GUI_WINDOW_TABLE_LAYOUT_INFO_START  0
#define GUI_WINDOW_TABLE_LAYOUT_INFO_END    (GUI_WINDOW_TABLE_LAYOUT_INFO_START + 1)
#define GUI_WINDOW_TABLE_LAYOUT_PRGB_START  (GUI_WINDOW_TABLE_LAYOUT_INFO_END)
#define GUI_WINDOW_TABLE_LAYOUT_PRGB_END    (GUI_WINDOW_TABLE_LAYOUT_PRGB_START + 1)
#define GUI_WINDOW_TABLE_LAYOUT_CTRL_START  (GUI_WINDOW_TABLE_LAYOUT_PRGB_END)
#define GUI_WINDOW_TABLE_LAYOUT_CTRL_END    (GUI_WINDOW_TABLE_LAYOUT_CTRL_START + 1)
#define GUI_WINDOW_TABLE_LAYOUT_NOTE_START  (GUI_WINDOW_TABLE_LAYOUT_CTRL_END)
#define GUI_WINDOW_TABLE_LAYOUT_NOTE_END    (GUI_WINDOW_TABLE_LAYOUT_VMAX)

#define STREAM_PARAM_TYPE_MASK          0xF0
#define STREAM_PARAM_SUB_MASK           0x0F
#define STREAM_PARAM_TYPE_VIDEO         0x00
#define STREAM_PARAM_TYPE_AUDIO         0x10
#define STREAM_PARAM_TYPE_OTHER         0x20

#define STREAM_PARAM_VIDEO_PID          (STREAM_PARAM_TYPE_VIDEO | 0x01)
#define STREAM_PARAM_VIDEO_FRAME_INFO   (STREAM_PARAM_TYPE_VIDEO | 0x02)
#define STREAM_PARAM_VIDEO_FRAME_RATIO  (STREAM_PARAM_TYPE_VIDEO | 0x03)
#define STREAM_PARAM_VIDEO_CODEC_TYPE   (STREAM_PARAM_TYPE_VIDEO | 0x04)
#define STREAM_PARAM_VIDEO_MAX_BITRATE  (STREAM_PARAM_TYPE_VIDEO | 0x05)
#define STREAM_PARAM_VIDEO_CUR_BITRATE  (STREAM_PARAM_TYPE_VIDEO | 0x06)

#define STREAM_PARAM_AUDIO_PID          (STREAM_PARAM_TYPE_AUDIO | 0x01)
#define STREAM_PARAM_AUDIO_CODEC_TYPE   (STREAM_PARAM_TYPE_AUDIO | 0x02)
#define STREAM_PARAM_AUDIO_CHANNEL_INFO (STREAM_PARAM_TYPE_AUDIO | 0x03)
#define STREAM_PARAM_AUDIO_FREQUENCY    (STREAM_PARAM_TYPE_AUDIO | 0x04)

#define STREAM_PARAM_OTHER_PCR_PID      (STREAM_PARAM_TYPE_OTHER | 0x01)
#define STREAM_PARAM_OTHER_TTX_PID      (STREAM_PARAM_TYPE_OTHER | 0x02)

//******************************************************************************
//******************************************************************************
//** Data types
//******************************************************************************
//******************************************************************************
typedef struct
{
    gchar   *id;
    gchar   *name;
    gchar   *pathname;
    gint    type;       //!< ITEM_TYPE_UNKNOWN, ITEM_TYPE_DIRECTORY, ITEM_TYPE_FILE
    gsize   size;
    time_t  date;

} gui_ListItemInfo;

typedef struct
{
    gint    id;
    gchar   *label;

} gui_ObjectInfo;

#ifndef GUI_ENABLE_STANDALONE
typedef struct
{
    uint8_t   name[SERVICELIST_MAX_SERVICE_NAME_LEN]; //!< service name

    int32_t   videoId;          //!< video decoder descriptor
    uint32_t  videoPid;         //!< video PID
    uint32_t  videoType;        //!< stream type as defined in MPEG/DVB

    uint32_t  width;            //!< decoded width
    uint32_t  height;           //!< decoded height
    int32_t   aspectRatio;
    uint32_t  maxBitRate;

    int32_t   audioId;          //!< audio output descriptor
    uint32_t  audioPid;         //!< audio PID
    uint32_t  audioType;        //!< stream type as defined in MPEG/DVB

    int32_t   sampleFreq;
    int32_t   channelCount;

    uint32_t  pcrPid;           //!< PCR PID if 0 then video PID used
    uint32_t  teletextPid;      //!< teletext PID

    uint64_t  playbackSize;
    uint64_t  startTime;

} gui_StreamInfo;
#endif

/* Make gui_ListItemInfo a boxed type so we can automatically
 * manage memory
 */
#define LIST_ITEM_INFO_TYPE list_item_info_get_type()

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************

static GtkWidget    *appMainWindowPtr    = NULL;
static GdkPixmap    *appMainPixMapPtr    = NULL;
static GtkWidget    *appStreamInfoBoxPtr = NULL;
static GtkWidget    *appProgressbar      = NULL;
static GtkWidget    *appPrevButtonPtr    = NULL;
static GtkWidget    *appPlayButtonPtr    = NULL;
static GtkWidget    *appNoteBookPtr      = NULL;
static GtkWidget    *appPageWindowPtr[2] = {NULL, NULL};
static GtkWidget    *appBrowsePagePtr    = NULL;
static GtkWidget    *appDirTreeViewPtr   = NULL;
static GtkWidget    *appInfoPagePtr      = NULL;
static GtkWidget    *appInfoTreeViewPtr  = NULL;
static gchar        *appCurrDirStr       = NULL;
static guint        appTimer             = 0;
static gint         appPlayStatus        = PLAYER_STATUS_STOPPED;
static gint         appFrontPageIndex    = PAGE_TYPE_UNKNOWN;


static gui_ListItemInfo     appPlaybackFile;
static gui_ListItemInfo     appSelectedFile;

#ifndef GUI_ENABLE_STANDALONE
static GUI_CallbackFuncs_T  appPlayerControlFuncs = {NULL, NULL, NULL, NULL};
static gui_StreamInfo       appStreamInfo;
#endif
static VAL_EnvironmentT     appPlayBackMode = VAL_ENV_NONE;

static const char *appMonthStr[] = {
    "Jan", "Feb", "Mar",
    "Apr", "May", "Jun",
    "Jul", "Aug", "Sep",
    "Oct", "Nov", "Dec"
};

static const gui_ObjectInfo appVideoParams[] =
{
    {STREAM_PARAM_VIDEO_PID,          "Video Pid"},
    {STREAM_PARAM_VIDEO_FRAME_INFO,   "Picture Size / Aspect Ratio"},
    {STREAM_PARAM_VIDEO_FRAME_RATIO,  "Frame Ratio"},
    {STREAM_PARAM_VIDEO_CODEC_TYPE,   "Compression Format"},
    {STREAM_PARAM_VIDEO_MAX_BITRATE,  "Max BitRatio"},
    {STREAM_PARAM_VIDEO_CUR_BITRATE,  "Measured BitRatio"},
    {-1,   NULL}
};

static const gui_ObjectInfo appAudioParams[] =
{
    {STREAM_PARAM_AUDIO_PID,          "Audio Pid"},
    {STREAM_PARAM_AUDIO_CODEC_TYPE,   "Audio Format"},
    {STREAM_PARAM_AUDIO_CHANNEL_INFO, "Mode"},
    {STREAM_PARAM_AUDIO_FREQUENCY,    "Sampling Frequency"},
    {-1,   NULL}
};

static const gui_ObjectInfo appOtherParams[] =
{
    {STREAM_PARAM_OTHER_PCR_PID,      "PCR Pid"},
    {STREAM_PARAM_OTHER_TTX_PID,      "TTX Pid"},
    {-1,   NULL}
};

static const char *appAspectStr[] = {
    "", "4:3", "14:9", "16:9", "221:100", "235:100"
};

//******************************************************************************
//******************************************************************************
//** Local functions
//******************************************************************************
//******************************************************************************

/*!
********************************************************************************
** \brief
********************************************************************************/
void set_program_name(const gchar *name)
{
    if(appStreamInfoBoxPtr != NULL)
    {
        GtkLabel  *label;

        label = (GtkLabel *)g_object_get_data(G_OBJECT(appStreamInfoBoxPtr), "pname");
        if(label != NULL)
        {
            gtk_label_set_text(label, name);
        }
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void set_file_name(const gchar *name)
{
    if(appStreamInfoBoxPtr != NULL)
    {
        GtkLabel  *label;

        label = (GtkLabel *)g_object_get_data(G_OBJECT(appStreamInfoBoxPtr), "fname");
        if(label != NULL)
        {
            gtk_label_set_text(label, name);
        }
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void set_dir_name(const gchar *name)
{
    if(appBrowsePagePtr != NULL)
    {
        GtkLabel  *label;

        label = (GtkLabel *)g_object_get_data(G_OBJECT(appBrowsePagePtr), "dname");
        if(label != NULL)
        {
            gtk_label_set_text(label, name);
        }
    }
}

/*!
********************************************************************************
** \brief The function to free the Boxed structure
********************************************************************************/
static void list_item_info_free(gui_ListItemInfo *info)
{
    g_free(info->id);
    g_free(info->name);
    g_free(info->pathname);
    g_free(info);
}

/*!
********************************************************************************
** \brief The function to copy the Boxed structure
********************************************************************************/
static gui_ListItemInfo* list_item_info_copy(gui_ListItemInfo *src)
{
    gui_ListItemInfo *info;

    info = g_new(gui_ListItemInfo, 1);

    info->id       = g_strdup(src->id);
    info->name     = g_strdup(src->name);
    info->pathname = g_strdup(src->pathname);
    info->type     = src->type;
    info->size     = src->size;
    info->date     = src->date;

    return info;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static GType list_item_info_get_type(void)
{
    static GType our_type = 0;

    if(our_type == 0) {
        our_type = g_boxed_type_register_static("gui_ListItemInfo",
                                                (GBoxedCopyFunc)list_item_info_copy,
                                                (GBoxedFreeFunc)list_item_info_free);
    }
    return our_type;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static GtkListStore* change_dir_model(GtkTreeView *treeview)
{
    GDir              *dir;
    GtkListStore      *store;
    gui_ListItemInfo  info;
    GtkTreeIter       iter;
    struct stat       tmp_stat;

    gtk_tree_view_set_model(treeview, NULL);

    store = gtk_list_store_new(2, LIST_ITEM_INFO_TYPE, G_TYPE_STRING);

    gtk_list_store_clear(store);

    if(g_utf8_collate(appCurrDirStr, "/") != 0)
    {
        info.id       = g_strdup(GTK_STOCK_DIRECTORY);
        info.name     = g_strdup_printf("..");
        info.pathname = g_strdup(g_path_get_dirname(appCurrDirStr));
        info.type     = ITEM_TYPE_DIRECTORY;
        info.size     = 0;

        g_lstat(info.pathname, &tmp_stat);
        info.date = tmp_stat.st_mtime;

        gtk_list_store_append(store, &iter);
        gtk_list_store_set(store, &iter,
                           LIST_COLUMN_INFO, &info,
                           LIST_COLUMN_NAME, info.id,
                           -1);
    }

    dir = g_dir_open(appCurrDirStr, 0, NULL);
    if(dir)
    {
        const gchar  *name;

        GTK_DPRINT(4, "[GUI] appCurrDirStr : %s\n", appCurrDirStr);

        while((name = g_dir_read_name(dir)))
        {
            if(name[0] != '.')
            {
                gboolean  is_dir;

                info.pathname = g_build_filename(appCurrDirStr, name, NULL);
                is_dir = g_file_test(info.pathname, G_FILE_TEST_IS_DIR);

                g_lstat(info.pathname, &tmp_stat);

                if(is_dir)
                {
                    info.id   = GTK_STOCK_DIRECTORY;
                    info.type = ITEM_TYPE_DIRECTORY;
                }
                else
                {
                    info.id   = GTK_STOCK_FILE;
                    info.type = ITEM_TYPE_FILE;
                }
                info.name = g_strdup(name);
                info.size = tmp_stat.st_size;
                info.date = tmp_stat.st_mtime;

                gtk_list_store_append(store, &iter);
                gtk_list_store_set(store, &iter,
                                   LIST_COLUMN_INFO, &info,
                                   LIST_COLUMN_NAME, info.id,
                                   -1);
            }
        }
        g_dir_close(dir);
    }
    return store;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static gint sort_func(GtkTreeModel *model,
                      GtkTreeIter  *a,
                      GtkTreeIter  *b,
                      gpointer     user_data)
{
    gui_ListItemInfo  *info_a, *info_b;
    int               result;

    gtk_tree_model_get(model, a, LIST_COLUMN_INFO, &info_a, -1);
    gtk_tree_model_get(model, b, LIST_COLUMN_INFO, &info_b, -1);

    if((info_a->type != ITEM_TYPE_DIRECTORY) && (info_b->type == ITEM_TYPE_DIRECTORY))
    {
        result = 1;
    } 
    else if((info_a->type == ITEM_TYPE_DIRECTORY) && (info_b->type != ITEM_TYPE_DIRECTORY))
    {
        result = -1;
    }
    else
    {
        result = g_utf8_collate(info_a->name, info_b->name);
    }
    return result;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void add_file_data(GtkTreeView *treeview)
{
    GtkListStore  *store;

    // draw Tree View
    store = change_dir_model(treeview);

    gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(store),
                                            sort_func, NULL, NULL);

    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store),
                                         GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
                                         GTK_SORT_ASCENDING);

    gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));

    g_object_unref(store);

    gtk_tree_view_set_rubber_banding(treeview, TRUE);

    // directory path name
    set_dir_name(appCurrDirStr);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void set_func_text_name(GtkTreeViewColumn *tree_column,
                               GtkCellRenderer   *cell,
                               GtkTreeModel      *model,
                               GtkTreeIter       *iter,
                               gpointer          data)
{
    gui_ListItemInfo  *info;

    gtk_tree_model_get(model, iter, LIST_COLUMN_INFO, &info, -1);

    g_object_set(GTK_CELL_RENDERER(cell), "text", info->name, NULL);

    list_item_info_free(info);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void set_func_size(GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer          data)
{
    gui_ListItemInfo  *info;
    gchar             *str;

    gtk_tree_model_get(model, iter, LIST_COLUMN_INFO, &info, -1);

    if(info->type == ITEM_TYPE_FILE)
    {
        str = g_strdup_printf("%d", info->size);
    }
    else
    {
        str = g_strdup_printf(" ");
    }

    g_object_set(GTK_CELL_RENDERER(cell), "text", str, NULL);

    list_item_info_free(info);
    g_free(str);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void set_func_date(GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer          data)
{
    gui_ListItemInfo  *info;
    struct tm         *tmp_tm;
    gchar             *str;

    gtk_tree_model_get(model, iter, LIST_COLUMN_INFO, &info, -1);

    tmp_tm = localtime(&(info->date));

    str = g_strdup_printf("%s %2d %4d %02d:%02d",
                          appMonthStr[tmp_tm->tm_mon],
                          tmp_tm->tm_mday,
                          1900 + tmp_tm->tm_year,
                          tmp_tm->tm_hour,
                          tmp_tm->tm_min);

    g_object_set(GTK_CELL_RENDERER(cell), "text", str, NULL);

    list_item_info_free(info);
    g_free(str);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void progressbar_update_live(GtkProgressBar *progressbar) 
{
    if(appFrontPageIndex == PAGE_TYPE_STREAM)
    {
#ifndef GUI_ENABLE_STANDALONE
        set_program_name((const gchar *)appStreamInfo.name);
#endif
        gtk_widget_queue_clear(appInfoTreeViewPtr);
    }
}

#if ENABLE_H57_TRANSCODER == 1
/*!
********************************************************************************
** \brief
********************************************************************************/
static void progressbar_update_trans(GtkProgressBar *progressbar)
{

    if(appPlayStatus == PLAYER_STATUS_RUNNING) {
        int32_t  prg_rate;

#ifndef GUI_ENABLE_STANDALONE
        prg_rate = TRANS_GetProgressRate();
#else
        prg_rate = (int32_t)(100.0 * gtk_progress_bar_get_fraction(progressbar)) + 1;
#endif
        if(prg_rate >= 0)
        {
            gchar    *str;
            gdouble  new_val;

            new_val = (gdouble)prg_rate / 100.0;

            gtk_progress_bar_set_fraction(progressbar, new_val);

            str = g_strdup_printf("%3d%%", prg_rate);
            gtk_progress_bar_set_text(progressbar, str);
            g_free(str);

            /* update file list view */
            if(appFrontPageIndex == PAGE_TYPE_FILE)
            {
                add_file_data(GTK_TREE_VIEW(appDirTreeViewPtr));
            }
        }
    }
    else if(appPlayStatus == PLAYER_STATUS_READY) {
        gtk_progress_bar_pulse(progressbar);
    }
}
#endif

/*!
********************************************************************************
** \brief
********************************************************************************/
static void change_playback_controller(void)
{
    GtkImage  *image;

    image = (GtkImage *)g_object_get_data(G_OBJECT(appPlayButtonPtr), "icon");

    if(appPlayStatus == PLAYER_STATUS_RUNNING)
    {
        gtk_widget_set_sensitive(appPrevButtonPtr, TRUE);
#if ENABLE_H57_TRANSCODER == 1
        gtk_widget_set_sensitive(appTransButtonPtr, FALSE);
#endif

        gtk_image_set_from_stock(image,
                                 GTK_STOCK_MEDIA_STOP,
                                 GTK_ICON_SIZE_BUTTON);
    }
    else
    {
        gtk_widget_set_sensitive(appPrevButtonPtr, FALSE);
#if ENABLE_H57_TRANSCODER == 1
        gtk_widget_set_sensitive(appTransButtonPtr, TRUE);
#endif

        gtk_image_set_from_stock(image,
                                 GTK_STOCK_MEDIA_PLAY,
                                 GTK_ICON_SIZE_BUTTON);
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void progressbar_update_pvr(GtkProgressBar *progressbar) 
{
    gchar  *str;

    if(appPlayStatus != PLAYER_STATUS_STOPPED)
    {
        gdouble   new_val = 0;

        if(appPlayStatus != PLAYER_STATUS_RUNNING)
        {
            appPlayStatus ++;

            if(appPlayStatus == PLAYER_STATUS_RUNNING)
            {
#ifndef GUI_ENABLE_STANDALONE
                char_t  *argv[2];

                argv[1] = appPlaybackFile.pathname;
                appPlayerControlFuncs.playbackStart(2, argv);
#endif
                GTK_DPRINT(1, "[GUI] Play Start %s\n", appPlaybackFile.name);

                set_file_name(appSelectedFile.name);

                change_playback_controller();
            }
            else
            {
                GTK_DPRINT(1, "[GUI] Play Stop %s\n", appPlaybackFile.name);

#ifndef GUI_ENABLE_STANDALONE
                appPlayerControlFuncs.playbackStop(0, NULL);
#endif
                str = g_strdup_printf("0:00:00");
                gtk_progress_bar_set_text(progressbar, str);
                g_free(str);

                gtk_progress_bar_set_fraction(progressbar, 0.0);

                change_playback_controller();
            }
        }
#ifdef GUI_ENABLE_STANDALONE
        else
        {
            /* running mode */
            new_val = gtk_progress_bar_get_fraction(progressbar) + 0.01;
            if(new_val >= 1.0)
            {
                /* end of playback */
                new_val = 1.0;
                appPlayStatus = PLAYER_STATUS_STOPPED;

                change_playback_controller();
            }
        }
        gtk_progress_bar_set_fraction(progressbar, new_val);

        str = g_strdup_printf("%3d%%", (int)(new_val * 100.0));
        gtk_progress_bar_set_text(progressbar, str);
        g_free(str);
#else
        else
        {
            /* running mode */
            if(appStreamInfo.playbackSize > 0)
            {
                int64_t  stc_value;

                /* set time stamp */
                stc_value = (int64_t)appPlayerControlFuncs.getTimeStamp();

                if(stc_value >= 0)
                {
                    uint32_t  hour, min, sec;

                    stc_value -= appStreamInfo.startTime;
                    if(stc_value < 0)
                    {
                        stc_value += 0x200000000ULL;
                    }

                    stc_value /= 90000ULL;
                    sec = (uint32_t)(stc_value % 60);

                    stc_value = stc_value / 60;
                    min = (uint32_t)(stc_value % 60);

                    hour = (uint32_t)(stc_value / 60);

                    str = g_strdup_printf("%d:%02d:%02d", hour, min, sec);
                    gtk_progress_bar_set_text(progressbar, str);
                    g_free(str);
                }

                /* set progressbar position */
                new_val = (gdouble)appPlayerControlFuncs.getFileReadPosition()
                        / (gdouble)appStreamInfo.playbackSize;
                if(new_val >= 1.0)
                {
                    /* end of playback */
                    new_val = 1.0;
                    appPlayStatus = PLAYER_STATUS_STOPPED;

                    change_playback_controller();
                }
                gtk_progress_bar_set_fraction(progressbar, new_val);
            }
            else
            {
                str = g_strdup_printf("Analyzing...");
                gtk_progress_bar_set_text(progressbar, str);
                g_free(str);

                gtk_progress_bar_pulse(progressbar);
            }
        }

#endif /* GUI_ENABLE_STANDALONE */

        if(appFrontPageIndex == PAGE_TYPE_STREAM)
        {
            gtk_widget_queue_clear(appInfoTreeViewPtr);
        }
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static gboolean progressbar_update(gpointer user_data) 
{
    GtkProgressBar *progressbar = GTK_PROGRESS_BAR(user_data);

    switch(appPlayBackMode)
    {
        case VAL_ENV_LIVE:
            progressbar_update_live(progressbar);
            break;

#if ENABLE_H57_TRANSCODER == 1
        case VAL_ENV_NUMS:  /* Transcode Mode */
            progressbar_update_trans(progressbar);
            break;
#endif
        default:
            progressbar_update_pvr(progressbar);
    }
    return TRUE;
}

/*!
********************************************************************************
** \brief  callback function for mouse event
********************************************************************************/
static void cb_prev_press(GtkButton *button,
                          gpointer  user_data)
{
    GTK_DPRINT(3, "[GUI] cb_prev_press\n");

    appPlayStatus = PLAYER_STATUS_READY;
}

/*!
********************************************************************************
** \brief  callback function for mouse event
********************************************************************************/
static void cb_play_press(GtkButton *button,
                          gpointer  user_data)
{
    GTK_DPRINT(3, "[GUI] cb_play_press %d\n", appPlayStatus);

    if(appPlayStatus == PLAYER_STATUS_STOPPED)
    {
        if(appPlaybackFile.name == NULL)
        {
            if(appSelectedFile.type == ITEM_TYPE_FILE)
            {
                appPlaybackFile.name     = g_strdup(appSelectedFile.name);
                appPlaybackFile.pathname = g_strdup(appSelectedFile.pathname);
                appPlaybackFile.size     = appSelectedFile.size;
#ifndef GUI_ENABLE_STANDALONE
                appStreamInfo.maxBitRate   = 0;
                appStreamInfo.playbackSize = 0;
                appStreamInfo.startTime    = 0;
#endif
            }
            else
            {
                return;
            }
        }
        appPlayStatus = PLAYER_STATUS_READY;
    }
    else if(appPlayStatus == PLAYER_STATUS_RUNNING)
    {
        appPlayStatus = PLAYER_STATUS_RESET;
    }
}

/*!
********************************************************************************
** \brief  configure event (create & resize window) management
********************************************************************************/
static gboolean cb_configure(GtkWidget *widget, GdkEventExpose *event)
{
    if(appMainPixMapPtr)
    {
        gdk_pixmap_unref(GDK_DRAWABLE(appMainPixMapPtr));
    }

    appMainPixMapPtr = gdk_pixmap_new(widget->window,
                                      widget->allocation.width,
                                      widget->allocation.height,
                                      -1);
    gdk_draw_rectangle(GDK_DRAWABLE(appMainPixMapPtr),
                       widget->style->white_gc,
                       TRUE,
                       0, 0,
                       widget->allocation.width,
                       widget->allocation.height);
    return TRUE;
}

/*!
********************************************************************************
** \brief  redraw management
********************************************************************************/
static gboolean cb_expose(GtkWidget *widget, GdkEventExpose *event)
{
    GdkRectangle  *rects;
    gint          i, num_rects;

    gdk_region_get_rectangles(event->region, &rects, &num_rects);

    GTK_DPRINT(4, "[GUI] redraw (%4d, %3d) .. (%4d, %3d) %d\n",
                  event->area.x, event->area.y,
                  event->area.x + event->area.width,
                  event->area.y + event->area.height,
                  num_rects);

    for(i=0; i<num_rects; i++)
    {
        // repaint rectangle
        gdk_draw_pixmap(widget->window,
                        widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
                        appMainPixMapPtr,
                        rects[i].x, rects[i].y,
                        rects[i].x, rects[i].y,
                        rects[i].width, rects[i].height);

        GTK_DPRINT(4, "          -> (%4d, %3d) .. (%4d, %3d)\n",
                      rects[i].x, rects[i].y,
                      rects[i].x + rects[i].width,
                      rects[i].y + rects[i].height);
    }
    g_free(rects);

    return FALSE;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void cb_destroy(GtkWidget *widget, gpointer data)
{
    g_free(appCurrDirStr);
    gtk_timeout_remove(appTimer);

    gtk_main_quit();
}

/*!
********************************************************************************
** \brief  callback function for selecting item
********************************************************************************/
static void cb_selection_changed(GtkTreeSelection *selection,
                                 gpointer user_data)
{
    GtkTreeModel *model;
    GtkTreeIter  iter;

    if(gtk_tree_selection_get_selected(selection, &model, &iter))
    {
        gui_ListItemInfo  *info;

        gtk_tree_model_get(model, &iter, LIST_COLUMN_INFO, &info, -1);

        appSelectedFile.id       = g_strdup(info->id);
        appSelectedFile.name     = g_strdup(info->name);
        appSelectedFile.pathname = g_strdup(info->pathname);
        appSelectedFile.type     = info->type;
        appSelectedFile.size     = info->size;
        appSelectedFile.date     = info->date;

        list_item_info_free(info);

        GTK_DPRINT(3, "[GUI] selection_changed %s (%s)\n",
                      appSelectedFile.pathname, appSelectedFile.name);

        if(appPlaybackFile.name == NULL)
        {
            goto cb_selection_changed_fname;
        }
        if(appPlayStatus == PLAYER_STATUS_STOPPED)
        {
            /* change playback file */
            appPlaybackFile.name     = g_strdup(appSelectedFile.name);
            appPlaybackFile.pathname = g_strdup(appSelectedFile.pathname);
            appPlaybackFile.size     = appSelectedFile.size;
#ifndef GUI_ENABLE_STANDALONE
            appStreamInfo.maxBitRate   = 0;
            appStreamInfo.playbackSize = 0;
            appStreamInfo.startTime    = 0;
#endif

cb_selection_changed_fname:
            if(appSelectedFile.type == ITEM_TYPE_FILE)
            {
                set_file_name(appSelectedFile.name);

                gtk_widget_set_sensitive(appPlayButtonPtr, TRUE);
            }
            else
            {
                set_file_name(" ");

                gtk_widget_set_sensitive(appPlayButtonPtr, FALSE);
            }
        }
    }
}

/*!
********************************************************************************
** \brief  callback function for deciding item
********************************************************************************/
static gboolean cb_selection_decided(GtkWidget *widget,
                                     GdkEventKey *event,
                                     gpointer user_data) 
{
    if(event->keyval == GDK_Return)
    {
        GTK_DPRINT(3, "[GUI] cb_selection_decided\n");

        if(appSelectedFile.type == ITEM_TYPE_FILE)
        {
            appPlaybackFile.name     = g_strdup(appSelectedFile.name);
            appPlaybackFile.pathname = g_strdup(appSelectedFile.pathname);
            appPlaybackFile.size     = appSelectedFile.size;
#ifndef GUI_ENABLE_STANDALONE
            appStreamInfo.maxBitRate   = 0;
            appStreamInfo.playbackSize = 0;
            appStreamInfo.startTime    = 0;
#endif
            appPlayBackMode = VAL_ENV_PLAY;
            appPlayStatus   = PLAYER_STATUS_READY;

            /* set focus */
            gtk_widget_grab_focus(appPlayButtonPtr);
        }
        else
        {
            g_free(appCurrDirStr);
            appCurrDirStr = g_strdup(appSelectedFile.pathname);

            GTK_DPRINT(3, "[GUI] \nnew_path : %s\n", appCurrDirStr);

            add_file_data(GTK_TREE_VIEW(appDirTreeViewPtr));
        }
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void create_file_browser(GtkWidget *page)
{
    {
        GtkWidget  *hbox;
        GtkWidget  *label;

        hbox = gtk_hbox_new(FALSE, 5);
        gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 0);

        label = gtk_label_new(appCurrDirStr);
        gtk_misc_set_padding(GTK_MISC(label), 10, 0);
        gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
        g_object_set_data(G_OBJECT(page), "dname", GTK_LABEL(label));
        gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    }

    {
        GtkCellRenderer    *cell_renderer;
        GtkTreeViewColumn  *column;
        GtkTreeSelection   *selection;

        appPageWindowPtr[PAGE_TYPE_FILE] = gtk_scrolled_window_new(NULL, NULL);

        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(appPageWindowPtr[PAGE_TYPE_FILE]),
                                            GTK_SHADOW_ETCHED_IN);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(appPageWindowPtr[PAGE_TYPE_FILE]),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
        gtk_box_pack_start(GTK_BOX(page), appPageWindowPtr[PAGE_TYPE_FILE], TRUE, TRUE, 0);

        appDirTreeViewPtr = gtk_tree_view_new();
        gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(appDirTreeViewPtr), TRUE);
        gtk_container_add(GTK_CONTAINER(appPageWindowPtr[PAGE_TYPE_FILE]), appDirTreeViewPtr);

        add_file_data(GTK_TREE_VIEW(appDirTreeViewPtr));

        column = gtk_tree_view_column_new();

        // column 1
        gtk_tree_view_column_set_title(column, " Name");

        // column 1-a (icon)
        cell_renderer = gtk_cell_renderer_pixbuf_new();
        g_object_set(cell_renderer, "xpad", 5, NULL);
        gtk_tree_view_column_pack_start(column,
                                        cell_renderer,
                                        TRUE);
        gtk_tree_view_column_set_attributes(column, cell_renderer,
                                            "stock_id", LIST_COLUMN_NAME,
                                            NULL);

        // column 1-b (name)
        cell_renderer = gtk_cell_renderer_text_new();
        g_object_set(cell_renderer, "xpad", 5, NULL);
        gtk_tree_view_column_pack_start(column,
                                        cell_renderer,
                                        TRUE);
        gtk_tree_view_column_set_cell_data_func(column,
                                                cell_renderer,
                                                set_func_text_name,
                                                NULL, NULL);

        gtk_tree_view_append_column(GTK_TREE_VIEW(appDirTreeViewPtr),
                                    column);

        // column 2
        cell_renderer = gtk_cell_renderer_text_new();
        g_object_set(cell_renderer, "xalign", 1.0, NULL);   // right align
        g_object_set(cell_renderer, "xpad", 10, NULL);
        gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(appDirTreeViewPtr),
                                                   -1,
                                                   " Size",
                                                   cell_renderer,
                                                   set_func_size,
                                                   NULL, NULL);

        // column 3
        cell_renderer = gtk_cell_renderer_text_new();
        g_object_set(cell_renderer, "xalign", 1.0, NULL);   // right align
        g_object_set(cell_renderer, "xpad", 10, NULL);
        gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(appDirTreeViewPtr),
                                                   -1,
                                                   " Date",
                                                   cell_renderer,
                                                   set_func_date,
                                                   NULL, NULL);

        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(appDirTreeViewPtr));
        gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

        g_signal_connect(selection, "changed",
                         G_CALLBACK(cb_selection_changed), page);

        gtk_widget_add_events(appDirTreeViewPtr, GDK_KEY_PRESS_MASK);
        g_signal_connect(G_OBJECT(appDirTreeViewPtr), "key-press-event",
                         G_CALLBACK(cb_selection_decided), NULL);
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static GtkTreeModel *create_info_panel_model(void)
{
    GtkTreeStore *treestore;
    GtkTreeIter  iter;
    gint  i;

    treestore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_INT);

    /* Video Information */
    gtk_tree_store_append(treestore, &iter, NULL);
    gtk_tree_store_set(treestore, &iter,
                       INFO_COLUMN_CATEGORY, "Video Information",
                       INFO_COLUMN_KEY,      STREAM_PARAM_TYPE_VIDEO,
                       -1);
    i = 0;
    while(appVideoParams[i].label)
    {
        GtkTreeIter  child_iter;

        gtk_tree_store_append(treestore, &child_iter, &iter);
        gtk_tree_store_set(treestore, &child_iter,
                           INFO_COLUMN_CATEGORY, appVideoParams[i].label,
                           INFO_COLUMN_KEY,      appVideoParams[i].id,
                           -1);
        i++;
    }

    /* Audio Information */
    gtk_tree_store_append(treestore, &iter, NULL);
    gtk_tree_store_set(treestore, &iter,
                       INFO_COLUMN_CATEGORY, "Audio Information",
                       INFO_COLUMN_KEY,      STREAM_PARAM_TYPE_AUDIO,
                       -1);
    i = 0;
    while(appAudioParams[i].label)
    {
        GtkTreeIter  child_iter;

        gtk_tree_store_append(treestore, &child_iter, &iter);
        gtk_tree_store_set(treestore, &child_iter,
                           INFO_COLUMN_CATEGORY, appAudioParams[i].label,
                           INFO_COLUMN_KEY,      appAudioParams[i].id,
                           -1);
        i++;
    }

    /* Other Information */
    gtk_tree_store_append(treestore, &iter, NULL);
    gtk_tree_store_set(treestore, &iter,
                       INFO_COLUMN_CATEGORY, "Other Information",
                       INFO_COLUMN_KEY,      STREAM_PARAM_TYPE_OTHER,
                       -1);
    i = 0;
    while(appOtherParams[i].label)
    {
        GtkTreeIter  child_iter;

        gtk_tree_store_append(treestore, &child_iter, &iter);
        gtk_tree_store_set(treestore, &child_iter,
                           INFO_COLUMN_CATEGORY, appOtherParams[i].label,
                           INFO_COLUMN_KEY,      appOtherParams[i].id,
                           -1);
        i++;
    }
    return GTK_TREE_MODEL(treestore);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void set_func_parameter(GtkTreeViewColumn *tree_column,
                               GtkCellRenderer   *cell,
                               GtkTreeModel      *model,
                               GtkTreeIter       *iter,
                               gpointer          data)
{
    gchar  *str;
    gint   index, type;

    gtk_tree_model_get(model, iter, INFO_COLUMN_KEY, &index, -1);

    type = index & STREAM_PARAM_TYPE_MASK;

    if((index & STREAM_PARAM_SUB_MASK) == 0)
    {
        str = g_strdup_printf(" ");
    }
#ifdef GUI_ENABLE_STANDALONE
    else
    {
        str = g_strdup_printf("%d", rand());
    }
#else
    else if(type == STREAM_PARAM_TYPE_VIDEO)
    {
        FAPEX_VIDEC_StatusStrT  video_info;
        int32_t  result;

        if(appStreamInfo.videoPid == 0)
        {
            str = g_strdup_printf("n.a.");
        }
        else
        {
            switch(index)
            {
                case STREAM_PARAM_VIDEO_PID:
                    str = g_strdup_printf("%d", appStreamInfo.videoPid);
                    break;

                case STREAM_PARAM_VIDEO_FRAME_INFO:
                    if((appStreamInfo.width * appStreamInfo.height) != 0)
                    {
                        if(appStreamInfo.aspectRatio >= 0)
                        {
                            str = g_strdup_printf("%dx%d %s",
                                                  appStreamInfo.width,
                                                  appStreamInfo.height,
                                                  appAspectStr[appStreamInfo.aspectRatio]);
                        }
                        else
                        {
                            str = g_strdup_printf("%dx%d",
                                                  appStreamInfo.width,
                                                  appStreamInfo.height);
                        }
                    }
                    else
                    {
                        str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_VIDEO_FRAME_RATIO:
                    result = FAPEX_VIDEC_Ioctl( appStreamInfo.videoId,
                                                FAPEX_IOCCOM_GET_STATUS,
                                                (FAPEXIOCARG3_T)&video_info );
                    if(result == 0)
                    {
                        str = g_strdup_printf("%5.2f Hz",
                                              (float)video_info.frameRate / 100);
                    }
                    else
                    {
                        str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_VIDEO_CODEC_TYPE:
                    switch(appStreamInfo.videoType)
                    {
                        case VID_MPEG1:
                            str = g_strdup_printf("MPEG-1");
                            break;
                        case VID_MPEG2:
                            str = g_strdup_printf("MPEG-2");
                            break;
                        case VID_H264:
                            str = g_strdup_printf("H.264");
                            break;
                        default:
                            str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_VIDEO_MAX_BITRATE:
                    result = FAPEX_VIDEC_Ioctl( appStreamInfo.videoId,
                                                FAPEX_IOCCOM_GET_STATUS,
                                                (FAPEXIOCARG3_T)&video_info );
                    if(result == 0)
                    {
                        if(appStreamInfo.maxBitRate < video_info.currBitRate)
                        {
                            appStreamInfo.maxBitRate = video_info.currBitRate;
                        }
                        str = g_strdup_printf("%.2f kBit/S",
                                              (float)appStreamInfo.maxBitRate / 1000);
                    }
                    else
                    {
                        str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_VIDEO_CUR_BITRATE:
                    result = FAPEX_VIDEC_Ioctl( appStreamInfo.videoId,
                                                FAPEX_IOCCOM_GET_STATUS,
                                                (FAPEXIOCARG3_T)&video_info );
                    if(result == 0)
                    {
                        str = g_strdup_printf("%.2f kBit/S",
                                              (float)video_info.currBitRate / 1000);
                    }
                    else
                    {
                        str = g_strdup_printf("n.a.");
                    }
                    break;

                default:
                    str = g_strdup_printf(" ");
            }
        }
    }
    else if(type == STREAM_PARAM_TYPE_AUDIO)
    {
        if(appStreamInfo.audioPid == 0)
        {
            str = g_strdup_printf("n.a.");
        }
        else
        {
            switch(index)
            {
                case STREAM_PARAM_AUDIO_PID:
                    str = g_strdup_printf("%d", appStreamInfo.audioPid);
                    break;

                case STREAM_PARAM_AUDIO_CODEC_TYPE:
                    switch(appStreamInfo.audioType)
                    {
                        case AUD_MPEG1:
                            str = g_strdup_printf("MPEG Layer1");
                            break;
                        case AUD_MPEG2:
                            str = g_strdup_printf("MPEG Layer2");
                            break;
                        case AUD_AAC_MPEG2:
                            str = g_strdup_printf("MPEG2 AAC-LC");
                            break;
                        case AUD_AAC_MPEG4:
                            str = g_strdup_printf("MPEG4 AAC-LC");
                            break;
                        case AUD_AC3_ATSC_A52:
                            str = g_strdup_printf("AC-3 ATSC A52");
                            break;
                        case USER_DEF_AC_3:
                            str = g_strdup_printf("AC3");
                            break;
                        case USER_DEF_EAC_3:
                            str = g_strdup_printf("Dolby Digital+");
                            break;
                        default:
                            str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_AUDIO_CHANNEL_INFO:
                    switch(appStreamInfo.channelCount)
                    {
                        case 1:
                            str = g_strdup_printf("Monaural");
                            break;
                        case 2:
                            str = g_strdup_printf("Stereo");
                            break;
                        default:
                            str = g_strdup_printf("n.a.");
                    }
                    break;

                case STREAM_PARAM_AUDIO_FREQUENCY:
                    if(appStreamInfo.sampleFreq > 0)
                    {
                        str = g_strdup_printf("%5.2f kHz",
                                              (float)appStreamInfo.sampleFreq / 1000);
                    }
                    else
                    {
                        str = g_strdup_printf("n.a.");
                    }
                    break;

                default:
                    str = g_strdup_printf(" ");
            }
        }
    }
    else
    {
        switch(index)
        {
            case STREAM_PARAM_OTHER_PCR_PID:
                if(appStreamInfo.pcrPid == 0)
                {
                    str = g_strdup_printf("n.a.");
                }
                else
                {
                    str = g_strdup_printf("%d", appStreamInfo.pcrPid);
                }
                break;

            case STREAM_PARAM_OTHER_TTX_PID:
                if(appStreamInfo.teletextPid == 0)
                {
                    str = g_strdup_printf("n.a.");
                }
                else
                {
                    str = g_strdup_printf("%d", appStreamInfo.teletextPid);
                }
                break;

            default:
                str = g_strdup_printf(" ");
        }
    }
#endif
    g_object_set(GTK_CELL_RENDERER(cell), "text", str, NULL);
    g_free(str);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void add_info_data(GtkTreeView *treeview)
{
    GtkCellRenderer   *renderer;
    GtkTreeViewColumn *column;
    gint              col_offset;

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);

    /**** column for category ****/
    renderer = gtk_cell_renderer_text_new();
    g_object_set(renderer, "xalign", 0.0, NULL);
    g_object_set(renderer, "xpad", 5, NULL);
    g_object_set(renderer, "ypad", 0, NULL);

    col_offset = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeview),
                                    -1, "Media Type", renderer,
                                    "text", INFO_COLUMN_CATEGORY,
                                    NULL);
    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), col_offset - 1);
    gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

    /**** column for data ****/
    renderer = gtk_cell_renderer_text_new();
    g_object_set(renderer, "xalign", 1.0, NULL);
    g_object_set(renderer, "xpad", 10, NULL);
    g_object_set(renderer, "ypad", 0, NULL);

    column = gtk_tree_view_column_new();
    gtk_tree_view_column_pack_start(column, renderer, TRUE);
    gtk_tree_view_column_set_cell_data_func(column, renderer,
                                            set_func_parameter,
                                            NULL, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void cb_page_switch(GtkWidget       *widget,
                           GtkNotebookPage *page,
                           gint            page_no)
{
    GtkNotebook *notebook = GTK_NOTEBOOK(widget);
    gint        old_page_no;
    GtkWidget   *image;

    old_page_no = gtk_notebook_get_current_page(notebook);

    if(page_no == old_page_no)
    {
        return;
    }
    GTK_DPRINT(3, "[GUI] cb_page_switch %d -> %d\n", old_page_no, page_no);

    image = (GtkWidget *)g_object_get_data(G_OBJECT(appStreamInfoBoxPtr), "icon");

    if(page_no == PAGE_TYPE_STREAM)
    {
        if(appPlayBackMode == VAL_ENV_LIVE)
        {
            if(image != NULL)
            {
                GdkPixbuf  *pixbuf;
                GError     *error = NULL;

                pixbuf = gdk_pixbuf_new_from_file("image/air.png", &error);
                if(error)
                {
                    g_print("%s\n", error->message);
                    g_error_free(error);
                }
                else
                {
                    /* set the icon image */
                    gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
                }
            }
            /* clear file name */
            set_file_name(" ");
        }
        else
        {
            if(image != NULL)
            {
                /* set the icon image */
                gtk_image_set_from_stock(GTK_IMAGE(image), GTK_STOCK_FILE,
                                         GTK_ICON_SIZE_DND);
            }
        }
    }
    else
    {
        if(page_no == PAGE_TYPE_FILE)
        {
            set_dir_name(appCurrDirStr);
        }

        if(image != NULL)
        {
            /* set the icon image */
            gtk_image_set_from_stock(GTK_IMAGE(image), GTK_STOCK_FILE,
                                     GTK_ICON_SIZE_DND);
        }
        if(appPlayBackMode == VAL_ENV_LIVE)
        {
            /* clear program name */
            set_program_name(" ");
        }
    }
    appFrontPageIndex = page_no;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void get_expanded_path(GtkTreeView *treeview,
                              GtkTreePath *path,
                              gpointer    user_data) 
{
    GtkTreeModel *model;
    GtkTreeIter  iter;
    GList        **list;
    gint         index;

    list  = user_data;
    model = gtk_tree_view_get_model(treeview);

    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_model_get(model, &iter, INFO_COLUMN_KEY, &index, -1);

    *list = g_list_append(*list, (gpointer)index);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static gboolean is_expanded(GtkTreeView *treeview,
                            gint        index)
{
    GList    *list, *node;
    gboolean ret = FALSE;

    list = NULL;
    gtk_tree_view_map_expanded_rows(treeview, get_expanded_path, &list);

    node = list;
    while(node != NULL)
    {
        if((gint)node->data == index)
        {
            ret = TRUE;
            break;
        }
        node = g_list_next(node);
    }
    g_list_free(list);

    return ret;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void cb_row_activated(GtkTreeView       *treeview,
                             GtkTreePath       *path,
                             GtkTreeViewColumn *column,
                             gpointer          user_data)
{
    GtkTreeModel  *model;
    GtkTreeIter   iter;
    gint          index;

    model = gtk_tree_view_get_model(treeview);

    if(gtk_tree_model_get_iter(model, &iter, path))
    {
        gtk_tree_model_get(model, &iter, INFO_COLUMN_KEY, &index, -1);
        if(is_expanded(treeview, index))
        {
            gtk_tree_view_collapse_row(treeview, path);
        }
        else
        {
            gtk_tree_view_expand_row(treeview, path, FALSE);
        }
        GTK_DPRINT(3, "[GUI] cb_row_activated\n");
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
static void cb_row_changed(GtkTreeView *treeview,
                           GtkTreeIter *iter,
                           GtkTreePath *path,
                           gpointer    user_data)
{
    gtk_widget_queue_clear(appInfoPagePtr);

    GTK_DPRINT(3, "[GUI] cb_row_changed %d\n", (gint)user_data);
}

#ifndef GUI_ENABLE_STANDALONE
/*!
********************************************************************************
** \brief
********************************************************************************/
static void set_focus_dir_tree(uint32_t code)
{
    GtkTreePath        *path;
    GtkTreeViewColumn  *column;
    gboolean           ret;

    gtk_tree_view_get_cursor(GTK_TREE_VIEW(appDirTreeViewPtr),
                             &path, &column);

    if(path != NULL)
    {
        GtkTreeModel  *model;

        if(code == FAPEX_IR_KEY_UP)
        {
            ret = gtk_tree_path_prev(path);
        }
        else
        {
            ret = TRUE;
            gtk_tree_path_next(path);
        }
        if(ret == TRUE)
        {
            gtk_tree_view_set_cursor(GTK_TREE_VIEW(appDirTreeViewPtr),
                                     path, column, FALSE);
        }

        model = gtk_tree_view_get_model(GTK_TREE_VIEW(appDirTreeViewPtr));
        if(model != NULL)
        {
            GtkTreeIter  iter;

            if(!gtk_tree_model_get_iter(model, &iter, path))
            {
                gtk_tree_path_free(path);
            }
        }
    }
    else
    {
        gtk_widget_grab_focus(appDirTreeViewPtr);
    }
}
#endif

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
********************************************************************************
** \brief
********************************************************************************/
#ifdef GUI_ENABLE_STANDALONE
int main(int argc, char *argv[])
#else
int GUI_Open(GUI_CallbackFuncs_T *funcs, const char_t *pathname)
#endif
{
    GtkWidget  *vbox;
    GtkWidget  *hbox;
    GtkWidget  *table;
    GtkWidget  *label_box;
    GtkWidget  *image;
    GtkWidget  *label;
    GtkWidget  *draw_area;

#ifndef GUI_ENABLE_STANDALONE
    if(funcs == NULL)
    {
        return -1;
    }
    memset(&appStreamInfo, 0, sizeof(SERVICELIST_ServiceInfo_T));
    appStreamInfo.videoId = appStreamInfo.audioId = -1;
    memcpy(&appPlayerControlFuncs, funcs, sizeof(GUI_CallbackFuncs_T));
#endif
    memset(&appPlaybackFile, 0, sizeof(gui_ListItemInfo));
    memset(&appSelectedFile, 0, sizeof(gui_ListItemInfo));
    appPlaybackFile.type = appSelectedFile.type = ITEM_TYPE_UNKNOWN;

    gtk_init(NULL, NULL);

    /* create main window */
    appMainWindowPtr = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position(GTK_WINDOW(appMainWindowPtr), GTK_WIN_POS_CENTER);
    gtk_container_set_border_width(GTK_CONTAINER(appMainWindowPtr), MAIN_WINDOW_BORDER);
    gtk_widget_set_size_request(appMainWindowPtr, MAIN_WINDOW_WIDTH, MAIN_WINDOW_HEIGHT);
    gtk_window_set_title(GTK_WINDOW(appMainWindowPtr), "simple player");

#ifdef GUI_ENABLE_STANDALONE
    if(argc > 1)
    {
        appCurrDirStr = g_strdup(argv[1]);
#else
    if(pathname != NULL)
    {
        appCurrDirStr = g_strdup((const gchar*)pathname);
#endif
    }
    else
    {
        appCurrDirStr = g_get_current_dir();
    }
    GTK_DPRINT(3, "[GUI] appCurrDirStr %s\n", appCurrDirStr);

    table = gtk_table_new(GUI_WINDOW_TABLE_LAYOUT_VMAX, 1, FALSE);
    gtk_container_add(GTK_CONTAINER(appMainWindowPtr), table);

    /*******************************/
    /** playback file information **/
    /*******************************/
    {
        GdkPixbuf  *pixbuf;
        GError     *error = NULL;

        hbox = gtk_hbox_new(FALSE, 5);
        gtk_table_attach_defaults(GTK_TABLE(table), hbox,
                                  0, 1,
                                  GUI_WINDOW_TABLE_LAYOUT_INFO_START,
                                  GUI_WINDOW_TABLE_LAYOUT_INFO_END);

        vbox = gtk_vbox_new(FALSE, 5);
        gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);

        if(appPlayBackMode == VAL_ENV_LIVE)
        {
            pixbuf = gdk_pixbuf_new_from_file("image/air.png", &error);
            if(error)
            {
                g_print("%s\n", error->message);
                g_error_free(error);
            }
            else
            {
                image = gtk_image_new_from_pixbuf(pixbuf);
            }
            label = gtk_label_new(" ");
        }
        else
        {
            image = gtk_image_new_from_stock(GTK_STOCK_FILE, GTK_ICON_SIZE_DND);
            label = gtk_label_new("(none)");
        }
        gtk_misc_set_padding(GTK_MISC(image), 5, 1);
        g_object_set(G_OBJECT(image), "xalign", 0.0, NULL);
        g_object_set(G_OBJECT(image), "yalign", 0.0, NULL);
        g_object_set_data(G_OBJECT(hbox), "icon", image);
        gtk_box_pack_start(GTK_BOX(vbox), image, FALSE, FALSE, 0);

        vbox = gtk_vbox_new(FALSE, 5);
        gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);

        /* file name */
        g_object_set(G_OBJECT(label), "xalign", 0.0, NULL);
        g_object_set_data(G_OBJECT(hbox), "fname", GTK_LABEL(label));
        gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

        /* program name */
        label = gtk_label_new(" ");
        g_object_set(G_OBJECT(label), "xalign", 0.0, NULL);
        g_object_set_data(G_OBJECT(hbox), "pname", GTK_LABEL(label));
        gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

        pixbuf = gdk_pixbuf_new_from_file("image/usb.png", &error);
        if(error)
        {
            g_print("%s\n", error->message);
            g_error_free(error);
        }
        else
        {
            GtkWidget  *usb_box;

            usb_box = gtk_hbox_new(FALSE, 0);
            gtk_box_pack_end(GTK_BOX(hbox), usb_box, FALSE, FALSE, 0);

            image = gtk_image_new_from_pixbuf(pixbuf);
            gtk_box_pack_start(GTK_BOX(usb_box), image, FALSE, FALSE, 0);
        }
        appStreamInfoBoxPtr = hbox;
    }

    /******************/
    /** Progress Bar **/
    /******************/
    {
        GtkWidget  *prg_box;

        prg_box = gtk_vbox_new(FALSE, 0);
        gtk_table_attach_defaults(GTK_TABLE(table), prg_box,
                                  0, 1,
                                  GUI_WINDOW_TABLE_LAYOUT_PRGB_START,
                                  GUI_WINDOW_TABLE_LAYOUT_PRGB_END);

        appProgressbar = gtk_progress_bar_new();
        gtk_box_pack_start(GTK_BOX(prg_box), appProgressbar, FALSE, FALSE, 0);

        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(appProgressbar), " ");

        appTimer = gtk_timeout_add(APP_PROGRESSBAR_UPDATE_INTERVAL,
                                   progressbar_update, appProgressbar);
    }

    /****************************/
    /** Player Control Buttons **/
    /****************************/
    {
        GtkWidget  *bbox;
        GdkColor   color;

        hbox = gtk_hbox_new(FALSE, 20);
        gtk_table_attach_defaults(GTK_TABLE(table), hbox,
                                  0, 1,
                                  GUI_WINDOW_TABLE_LAYOUT_CTRL_START,
                                  GUI_WINDOW_TABLE_LAYOUT_CTRL_END);

        bbox = gtk_hbutton_box_new();
        gtk_box_pack_start(GTK_BOX(hbox), bbox, FALSE, FALSE, 0);
        gtk_container_set_border_width(GTK_CONTAINER(bbox), 5);
        gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_START);

        /* prev button */
        appPrevButtonPtr = gtk_button_new();
        gtk_container_add(GTK_CONTAINER(bbox), appPrevButtonPtr);
        gtk_widget_set_sensitive(appPrevButtonPtr, FALSE);

        gdk_color_parse("#E0E0FF", &color);
        gtk_widget_modify_bg(appPrevButtonPtr, GTK_STATE_NORMAL, &color);

        image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS, GTK_ICON_SIZE_LARGE_TOOLBAR);
        gtk_container_add(GTK_CONTAINER(appPrevButtonPtr), image);

        g_signal_connect(G_OBJECT(appPrevButtonPtr), "clicked",
                         G_CALLBACK(cb_prev_press), NULL);

        /* play button */
        appPlayButtonPtr = gtk_button_new();
        gtk_container_add(GTK_CONTAINER(bbox), appPlayButtonPtr);
        gtk_widget_set_sensitive(appPlayButtonPtr, FALSE);

        gtk_widget_modify_bg(appPlayButtonPtr, GTK_STATE_NORMAL, &color);

        image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR);
        g_object_set_data(G_OBJECT(appPlayButtonPtr), "icon", GTK_IMAGE(image));
        gtk_container_add(GTK_CONTAINER(appPlayButtonPtr), image);

        g_signal_connect(G_OBJECT(appPlayButtonPtr), "clicked",
                         G_CALLBACK(cb_play_press), NULL);
    }

    /**************/
    /** NoteBook **/
    /**************/
    appNoteBookPtr = gtk_notebook_new();
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(appNoteBookPtr), TRUE);

    gtk_table_attach_defaults(GTK_TABLE(table), appNoteBookPtr,
                              0, 1,
                              GUI_WINDOW_TABLE_LAYOUT_NOTE_START,
                              GUI_WINDOW_TABLE_LAYOUT_NOTE_END);

    g_signal_connect(G_OBJECT(appNoteBookPtr), "switch_page",
                     G_CALLBACK(cb_page_switch), NULL);

    /* NoteBook-0 : Stream Information */
    {
        GtkTreeModel     *model;
        GtkTreeSelection *selection;

        hbox = gtk_hbox_new(FALSE, 10);
        label_box = gtk_hbox_new(FALSE, 0);

        image = gtk_image_new_from_stock(GTK_STOCK_INFO, GTK_ICON_SIZE_LARGE_TOOLBAR);
        gtk_misc_set_padding(GTK_MISC(image), 5, 1);
        gtk_box_pack_start(GTK_BOX(label_box), image, FALSE, FALSE, 0);

        label = gtk_label_new("Stream Information ");
        gtk_box_pack_start(GTK_BOX(label_box), label, FALSE, FALSE, 0);

        gtk_widget_show_all(label_box);

        gtk_notebook_append_page(GTK_NOTEBOOK(appNoteBookPtr),
                                 hbox, label_box);

        appPageWindowPtr[PAGE_TYPE_STREAM] = gtk_scrolled_window_new(NULL, NULL);
        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(appPageWindowPtr[PAGE_TYPE_STREAM]),
                                            GTK_SHADOW_ETCHED_IN);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(appPageWindowPtr[PAGE_TYPE_STREAM]),
                                       GTK_POLICY_NEVER,
                                       GTK_POLICY_AUTOMATIC);
        gtk_box_pack_start(GTK_BOX(hbox), appPageWindowPtr[PAGE_TYPE_STREAM], TRUE, TRUE, 0);
        appInfoPagePtr = hbox;

        /* create model */
        model = create_info_panel_model();

        /* create tree view */
        appInfoTreeViewPtr = gtk_tree_view_new_with_model(model);

        g_object_unref(model);

        gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(appInfoTreeViewPtr), TRUE);
        gtk_container_add(GTK_CONTAINER(appPageWindowPtr[PAGE_TYPE_STREAM]), appInfoTreeViewPtr);

        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(appInfoTreeViewPtr));
        gtk_tree_selection_set_mode(selection, GTK_SELECTION_NONE);

        add_info_data(GTK_TREE_VIEW(appInfoTreeViewPtr));

        /* expand all rows after the treeview widget has been realized */
        g_signal_connect(appInfoTreeViewPtr, "realize",
                         G_CALLBACK(gtk_tree_view_expand_all), NULL);

        g_signal_connect(appInfoTreeViewPtr, "row-activated",
                         G_CALLBACK(cb_row_activated), NULL);

        g_signal_connect(appInfoTreeViewPtr, "row-collapsed",
                         G_CALLBACK(cb_row_changed), (gpointer)-1);

        g_signal_connect(appInfoTreeViewPtr, "row-expanded",
                         G_CALLBACK(cb_row_changed), (gpointer)1);
    }

    /* NoteBook-1 : File Selection */
    {
        label_box = gtk_hbox_new(FALSE, 0);

        image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_LARGE_TOOLBAR);
        gtk_misc_set_padding(GTK_MISC(image), 5, 1);
        gtk_box_pack_start(GTK_BOX(label_box), image, FALSE, FALSE, 0);

        label = gtk_label_new("File Selection ");
        gtk_box_pack_start(GTK_BOX(label_box), label, FALSE, FALSE, 0);

        gtk_widget_show_all(label_box);

        vbox = gtk_vbox_new(FALSE, 5);

        create_file_browser(vbox);

        gtk_notebook_append_page(GTK_NOTEBOOK(appNoteBookPtr),
                                 vbox, label_box);
        appBrowsePagePtr = vbox;
    }

    /* setup for drawing */
    draw_area = gtk_drawing_area_new();
    gtk_widget_show(draw_area);

    gtk_widget_set_events(draw_area,
                          GDK_EXPOSURE_MASK |
                          GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
                          GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);

    /* setup event handers */
    gtk_signal_connect(GTK_OBJECT(appMainWindowPtr), "configure_event",
                       GTK_SIGNAL_FUNC(cb_configure), NULL);

    g_signal_connect(G_OBJECT(appMainWindowPtr), "expose_event",
                     GTK_SIGNAL_FUNC(cb_expose), NULL);

    g_signal_connect(G_OBJECT(appMainWindowPtr), "destroy",
                     G_CALLBACK(cb_destroy), NULL);

    gtk_widget_show_all(appMainWindowPtr);

    gtk_main();

    return 0;
}

#ifndef GUI_ENABLE_STANDALONE
/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_Close(void)
{
    cb_destroy(NULL, 0);
}

/*!
********************************************************************************
** \brief set playback mode
********************************************************************************/
void GUI_SetPlaybackMode(VAL_EnvironmentT mode)
{
    appPlayBackMode = mode;
    GTK_DPRINT(1, "[GUI] GUI_SetPlaybackMode : %d\n", appPlayBackMode);
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetServiceInfo(SERVICELIST_ServiceInfo_T *servicePtr)
{
    strncpy((char*)appStreamInfo.name, (const char*)servicePtr->name,
            SERVICELIST_MAX_SERVICE_NAME_LEN);
    set_program_name((const gchar *)appStreamInfo.name);

    appStreamInfo.videoPid    = servicePtr->videoPid;
    appStreamInfo.videoType   = servicePtr->videoType;
    appStreamInfo.audioPid    = servicePtr->audioPid[0];
    appStreamInfo.audioType   = servicePtr->audioType[0];
    appStreamInfo.pcrPid      = servicePtr->pcrPid;
    appStreamInfo.teletextPid = servicePtr->teletextPid;
    appStreamInfo.maxBitRate  = 0;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetVidecId(int32_t id)
{
    appStreamInfo.videoId = id;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetAuoutId(int32_t id)
{
    FAPEX_AUOUT_PcmInfoStrT  pcm_info;
    int32_t  result;

    appStreamInfo.audioId = id;

    result = FAPEX_AUOUT_Ioctl( appStreamInfo.audioId,
                                FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                (FAPEXIOCARG3_T)&pcm_info );
    if(result == 0)
    {
        appStreamInfo.sampleFreq   = pcm_info.sampleFreq;
        appStreamInfo.channelCount = pcm_info.channelCount;
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetPlaybackStart(void)
{
    appStreamInfo.playbackSize = appPlaybackFile.size
                               - appPlayerControlFuncs.getFileReadPosition();

    appStreamInfo.startTime = (int64_t)appPlayerControlFuncs.getTimeStamp();
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetRunningCancel(void)
{
    GtkWidget  *dialog;
    gint       tmp_index;
    gchar      *str;
    GdkColor   color;

    switch(appPlayBackMode)
    {
        case VAL_ENV_PLAY:
            str = "Playback was cancelled.";
            break;

        default:
            return;
    }
    GTK_DPRINT(3, "[GUI] %s\n", str);

    tmp_index         = appFrontPageIndex;
    appFrontPageIndex = PAGE_TYPE_UNKNOWN;

    dialog = gtk_message_dialog_new(GTK_WINDOW(appMainWindowPtr),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    str);
    gtk_widget_set_size_request(dialog,
                                MAIN_WINDOW_WIDTH >> 1,
                                MAIN_WINDOW_HEIGHT >> 2);
    gdk_color_parse("yellow", &color);
    gtk_widget_modify_bg(dialog, GTK_STATE_NORMAL, &color);

    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);

    appFrontPageIndex = tmp_index;

    appPlayStatus = PLAYER_STATUS_STOPPED;

    /* wait for update of progress bar */
    FAPI_SYS_SLEEP(2 * APP_PROGRESSBAR_UPDATE_INTERVAL);

    gtk_timeout_remove(appTimer);
    appTimer = gtk_timeout_add(APP_PROGRESSBAR_UPDATE_INTERVAL,
                               progressbar_update, appProgressbar);

    change_playback_controller();
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetVideoFrameParameter(FAPEX_VIDEC_DecFrameInfoStrT *info)
{
    appStreamInfo.width       = info->decodedWidth;
    appStreamInfo.height      = info->decodedHeight;
    appStreamInfo.aspectRatio = info->aspectRatio;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_RemoteControl(uint32_t code)
{
    static int window_visible = 1;

    GTK_DPRINT(3, "[GUI] GUI_RemoteControl : %02x (%d)\n", code, appPlayStatus);

    switch(code) {
        case FAPEX_IR_KEY_PLAY:
            if(appPlayStatus == PLAYER_STATUS_STOPPED) {
                gtk_widget_grab_focus(appPlayButtonPtr);

                gtk_button_clicked(GTK_BUTTON(appPlayButtonPtr));
            }
            break;

        case FAPEX_IR_KEY_STOP:
            if(appPlayStatus == PLAYER_STATUS_RUNNING) {
                gtk_widget_grab_focus(appPlayButtonPtr);

                gtk_button_clicked(GTK_BUTTON(appPlayButtonPtr));
            }
            break;

        case FAPEX_IR_KEY_RIGHT:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Right, 0);
            break;

        case FAPEX_IR_KEY_LEFT:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Left, 0);
            break;

        case FAPEX_IR_KEY_UP:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Up, 0);
            break;

        case FAPEX_IR_KEY_DOWN:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Down, 0);
            break;

        case FAPEX_IR_KEY_EXIT:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Tab, 0);
            break;

        case FAPEX_IR_KEY_OK:
            gtk_test_widget_send_key(appMainWindowPtr, GDK_Return, 0);
            break;

        case FAPEX_IR_KEY_INFO:
            gtk_notebook_set_current_page(GTK_NOTEBOOK(appNoteBookPtr), PAGE_TYPE_STREAM);
            break;

        case FAPEX_IR_KEY_PIPSWAP:
            if(window_visible == 1)
            {
                gtk_widget_hide_all(appMainWindowPtr);
                window_visible = 0;
            }
            else
            {
                gtk_widget_show_all(appMainWindowPtr);
                window_visible = 1;
            }
            break;

        default:
            ;
    }
}

/*!
********************************************************************************
** \brief
********************************************************************************/
void GUI_SetWarning(const char_t *str)
{
    GtkWidget  *dialog;
    GdkColor   color;

    GTK_DPRINT(3, "[GUI] %s\n", str);

    dialog = gtk_message_dialog_new(GTK_WINDOW(appMainWindowPtr),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_WARNING,
                                    GTK_BUTTONS_CLOSE,
                                    (gchar *)str);
    gtk_widget_set_size_request(dialog,
                                MAIN_WINDOW_WIDTH >> 1,
                                MAIN_WINDOW_HEIGHT >> 2);

    gdk_color_parse("orange", &color);
    gtk_widget_modify_bg(dialog, GTK_STATE_NORMAL, &color);

    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
}
#endif /* GUI_ENABLE_STANDALONE */
