/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo_fs/src/hdmi_cec.c
** 
** \version     $Id$
**
** \brief       HDMI-CEC related function implementations.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#include <fapex/abst_types.h>
#include <fapex/abst_libhdmi.h>
#include <fapex/abst_libhdmicec.h>
#include <val/val.h>
#include "hdmi_cec.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define DRV_ID_HDMICEC          (FAPEXDEV_HDMICEC_MINOR_FIRST)

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static int32_t fd_HDMI    = -1;

#ifdef USE_HDMICEC
static int32_t fd_HDMICEC = -1;

static uint32_t logicalAdder[4] = {3, 6, 7, 10};
//static volatile uint32_t StandbyMode = 0;
#endif

static FAPEX_HDMI_EdidInfoStrT  appEdidInfo;


//***************************************************************************
//***************************************************************************
//** Prototype Functions Declaration
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** External Functions Declaration
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
extern VAL_DataT* vidDatPtr;

#ifdef USE_HDMICEC
FAPEX_HDMICEC_MessageParamsT hdmicecMsgParamUserSetting;
#endif

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** Open HDMI.
**
** This function opens the HDMI & HDMI-CEC controller.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_Open(void)
{
    /* open HDMI */
    if(fd_HDMI < 0) {
        fd_HDMI = VAL_GetIdHdmi(vidDatPtr);
    }

    if(fd_HDMI < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open HDMI driver. %d\n", fd_HDMI);
        return fd_HDMI;
    }

    memset(&appEdidInfo, 0, sizeof(FAPEX_HDMI_EdidInfoStrT));

#ifdef USE_HDMICEC
    /* open HDMI-CEC */
    fd_HDMICEC = FAPEX_HDMICEC_Open( DRV_ID_HDMICEC,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)NULL );
    if(fd_HDMICEC < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open HDMI-CEC driver. %d\n", fd_HDMICEC);
        fd_HDMI = -1;
        return fd_HDMICEC;
    }

    /* init the parameter of cec message */
    hdmicecMsgParamUserSetting.cecGiveTunerDeviceStatusParams.Status_Request = 0x01;
    hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Broadcast_Type = 0x02;
    hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[0] = 0x00;
    hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[1] = 0xff;
    hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Broadcast_System = 0x00;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[0] = 0x00;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[1] = 0x01;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[2] = 0x02;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[3] = 0x03;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[4] = 0x04;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[5] = 0x05;
    hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[6] = 0x06;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.deviceType = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[0] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[1] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[2] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[3] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[4] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[5] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[6] = 0x01;
    hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[7] = 0x01;
    hdmicecMsgParamUserSetting.cecSetOSDStringParams.Display_Control = 0x00;
    hdmicecMsgParamUserSetting.cecMenuRequestParams.Menu_Request_Type = 0x00;
    hdmicecMsgParamUserSetting.cecMenuStatusParams.Menu_State = 0x00;
    hdmicecMsgParamUserSetting.cecUserControlPressedParams.UI_Command = 0x00;
    hdmicecMsgParamUserSetting.cecReportPowerStatusParams.Power_Status = 0x00;
    hdmicecMsgParamUserSetting.cecCECVersionParams.CEC_Version = 0x04;
    hdmicecMsgParamUserSetting.cecReportPhysicalAddressParams.Device_Type = 3;
    hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[0] = 0x65;
    hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[1] = 0x6e;
    hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[2] = 0x67;
    hdmicecMsgParamUserSetting.cecFeatureAbortParams.Feature_Opcode = 0x8C;
    hdmicecMsgParamUserSetting.cecFeatureAbortParams.Abort_Reason = 0x02;
    hdmicecMsgParamUserSetting.cecActiveSourceParams.Physical_Address[0] = 0x10;
    hdmicecMsgParamUserSetting.cecActiveSourceParams.Physical_Address[1] = 0x00;
    hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[0] = 0x10;
    hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[1] = 0x00;
    hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[0] = 0x10;
    hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[1] = 0x00;
    hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[0] = 0x20;
    hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[1] = 0x00;
    hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[0] = 0x10;
    hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[1] = 0x00;
    hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[0] = 0x10;
    hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[1] = 0x00;
#endif

    return 0;
}

/*!
*******************************************************************************
** Close the HDMI.
**
** This function close the HDMI video.
**
*******************************************************************************
*/
int32_t HDMI_Close(void)
{
    int32_t result = 0;

    fd_HDMI = -1;

#ifdef USE_HDMICEC
    if(fd_HDMICEC >= 0) {
        result = FAPEX_HDMICEC_Close( fd_HDMICEC );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close HDMI (code %d).\n", result);
            return result;
        }
        fd_HDMICEC = -1;
    }
#endif
    return result;
}

/*!
*******************************************************************************
** Reads EDID.
**
** This function reads EDID.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_ReadEDID(void)
{
    int32_t  status;

    FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                      FAPEX_IOCCOM_HDMI_READ_EDID,
                      (FAPEXIOCARG3_T)&status );
    return status;
}

/*!
*******************************************************************************
** Extracts the delay.
**
** This function extracts the delay and interlaced delay from the EDID and
** calculates the resulting overall dealys.
**
*******************************************************************************
*/
void HDMI_GetDelay(int32_t* delay, int32_t* interlaced_delay)
{
    int32_t  video_delay, audio_delay;

    /* either audio or video is not supported */
    if(!(appEdidInfo.videoLatency) || (appEdidInfo.videoLatency > 251) ||
       !(appEdidInfo.audioLatency) || (appEdidInfo.audioLatency > 251)
      )
    {
        //FAPI_SYS_PRINT_MSG("[HDMI] Latency information is invalid or not supported.\n");
        *delay = 0xDEAD;
        *interlaced_delay = 0xDEAD;
        return;
    }

    /* see HDMI 1.3 spec 8.3.2 for further information */
    video_delay = (int32_t)( (appEdidInfo.videoLatency - 1) * 2);
    audio_delay = (int32_t)( (appEdidInfo.audioLatency - 1) * 2);
    *delay = (video_delay - audio_delay);

    /* either audio or video is not supported */
    if(!(appEdidInfo.interlacedVideoLatency) || (appEdidInfo.interlacedVideoLatency > 251) ||
       !(appEdidInfo.interlacedAudioLatency) || (appEdidInfo.interlacedAudioLatency > 251)
      )
    {
        /* use same settings as progressive */
        *interlaced_delay = (video_delay - audio_delay);
    }
    else
    {
        /* see HDMI 1.3 spec 8.3.2 for further information */
        video_delay = (int32_t)( (appEdidInfo.interlacedVideoLatency - 1) * 2);
        audio_delay = (int32_t)( (appEdidInfo.interlacedAudioLatency - 1) * 2);
        *interlaced_delay = (video_delay - audio_delay);
    }
}

/*!
*******************************************************************************
** Gets physical address in EDID.
**
** This function gets physical address in EDID.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_GetPhysicalAddressInEDID(uint8_t *UpperPhysAddr, uint8_t *LowerPhysAddr)
{
    uint32_t PackphysAddr = 0;
    int32_t  result;

    result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                               FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR,
                               (FAPEXIOCARG3_T)&PackphysAddr );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("FAPI_HDMI_GetPhysicalAddressInEDID status = %d\n", result);
        return result;
    }

    *UpperPhysAddr = (PackphysAddr >> 8) & 0x000000ff;
    *LowerPhysAddr = PackphysAddr & 0x000000ff;

    return result;
}

/*!
*******************************************************************************
** Check HPD.
**
** This function checks HPD.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CheckHPD(void)
{
    int32_t status;

    FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                      FAPEX_IOCCOM_HDMI_CHECK_HPD,
                      (FAPEXIOCARG3_T)&status );
    return status;
}

#ifdef USE_HDMICEC
/*!
*******************************************************************************
** Control the processing of sending message.
**
** This function control the processing of sending message.
**
*******************************************************************************
*/
void HDMI_CEC_ControlSendMessage(void)
{
    uint32_t SendStatus = 0;
    int32_t result;

    while(1) {
        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_GET_SEND_STATUS,
                                      (FAPEXIOCARG3_T)&SendStatus );
        if(result == 0) {
            FAPI_SYS_PRINT_MSG("[Transmission completion]\n");

            if(SendStatus == FAPEX_HDMICEC_SENDING_ERROR) {
                FAPI_SYS_PRINT_MSG("ERROR:[hdmi_cec]:FAPI_HDMI_CheckCecSendStatus "
                                   "return FAPEX_HDMICEC_SENDING_ERROR\n");
            }
            break;
        }
    }
}

/*!
*******************************************************************************
** Control the processing of receiving message.
**
** This function controls the processing of receiving message.
**
** \param pRecMessage   A pointer variable for receiving message.
** \param pRecInitiator A pointer variable for receiving initiator.
** \param pMessageType  A pointer variable for receiving message type.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_ControlRecMessage(uint32_t* pRecMessage, uint32_t* pRecInitiator, uint32_t* pMessageType)
{
    uint32_t RemainRecMsg = 0;
    uint32_t RecStatus    = 0;
    int32_t  result;
    uint64_t start_time;
    uint64_t end_time;
    int32_t  time_difference;

    start_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_100_USEC);

    while(1) {
        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_GET_RECEIVE_STATUS,
                                      (FAPEXIOCARG3_T)&RecStatus );
        if(result == 0) {
            FAPI_SYS_PRINT_MSG("[Reception complete]\n");

            if(RecStatus == FAPEX_HDMICEC_RECEPTION_ERROR) {
                FAPI_SYS_PRINT_MSG("ERROR:[hdmi_cec]:FAPI_HDMI_CheckCecReceiveStatus "
                                   "return FAPEX_HDMICEC_RECEPTION_ERROR\n");
            }
            break;
        }
        end_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_100_USEC);
        time_difference = (int32_t)(end_time - start_time);
        if( time_difference > 90000 ){
            FAPI_SYS_PRINT_MSG("It is a time out.There was not a response from other HDMI device.\n");
            FAPI_SYS_PRINT_MSG("Please enter CEC command on terminal.\n");
            return -1;
        }
    }
    (void)HDMI_CEC_GetMsg(pRecInitiator, pRecMessage, &RemainRecMsg, pMessageType);

    return result;
}

/*!
*******************************************************************************
** Check sending status.
**
** This function checks the sending status.
**
** \return sending status
*******************************************************************************
*/
uint32_t HDMI_CEC_CheckSendStatus(void)
{
    int32_t  result;
    uint32_t status = 0;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_CHECK_SEND_STATUS,
                                  (FAPEXIOCARG3_T)&status );
    if(result == 0) {
        FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_CheckCecSendStatus return = %x\n", status);
        return status;
    }
    return result;
}

/*!
*******************************************************************************
** Check reciving status.
**
** This function checks the rceiving status.
**
** \return receiving status
*******************************************************************************
*/
uint32_t HDMI_CEC_CheckReceiveStatus(void)
{
    int32_t  result;
    uint32_t status = 0;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_CHECK_RECEIVE_STATUS,
                                  (FAPEXIOCARG3_T)&status );
    if(result == 0) {
        FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_CheckCecReceiveStatus return = %x\n", status);
        return status;
    }
    return result;
}

/*!
*******************************************************************************
** Get CEC Message.
**
** This function gets CEC message.
**
** \param pRecInitiator This parameter is set to the logical address of initiator in receiving message.
** \param pRecMessage   This parameter is set to the message(opcode) in receiving message.
** \param pRemainRecMsg This parameter is set to the remaining message count.
** \param pMessageType  This parameter is set to the message type(broadcast or directly).
**
** \return #FAPI_OK
** \return #FAPI_HDMI_ERR_CEC_MESSAGE_BUFFER_EMPTY
*******************************************************************************
*/
int32_t HDMI_CEC_GetMsg(uint32_t* pRecInitiator, uint32_t* pRecMessage, uint32_t* pRemainRecMsg, uint32_t* pMessageType)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    FAPEX_HDMICEC_MessageParamsT   *message;
    int32_t  result;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_GET_MESSAGE,
                                  (FAPEXIOCARG3_T)&params );
    if(result != 0) {
        FAPI_SYS_PRINT_ERR("[hdmi_cec]:FAPI_HDMI_GetCecMessageParameters return = %d\n", result);
        return result;
    }
    message = &params.message;

    FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_GetCecMessageParameters RecInitiator = %x\n", params.initiatorLogicalAddr );
    FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_GetCecMessageParameters RecMessage   = %x\n", params.opcodeName );
    FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_GetCecMessageParameters RemainRecMsg = %d\n", params.remainRecMsgNum );
    FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPI_HDMI_GetCecMessageParameters MessageType  = %x\n", message->message_type );

    *pRecInitiator = params.initiatorLogicalAddr;
    *pRecMessage   = params.opcodeName;
    *pRemainRecMsg = params.remainRecMsgNum;
    *pMessageType  = message->message_type;

    switch(params.opcodeName){
        case FAPEX_HDMICEC_FEATURE_ABORT:
            FAPI_SYS_PRINT_MSG("Feature_Opcode = %x\n",message->cecFeatureAbortParams.Feature_Opcode );
            FAPI_SYS_PRINT_MSG("Abort_Reason   = %x\n",message->cecFeatureAbortParams.Abort_Reason );
            break;
        case FAPEX_HDMICEC_ACTIVE_SOURCE:
            FAPI_SYS_PRINT_MSG("Physical_Address[0] = %x\n",message->cecActiveSourceParams.Physical_Address[0] );
            FAPI_SYS_PRINT_MSG("Physical_Address[1] = %x\n",message->cecActiveSourceParams.Physical_Address[1] );
            break;
        case FAPEX_HDMICEC_ROUTING_INFORMATION:
            FAPI_SYS_PRINT_MSG("Physical_Address[0] = %x\n",message->cecRoutingInformationParams.Physical_Address[0] );
            FAPI_SYS_PRINT_MSG("Physical_Address[1] = %x\n",message->cecRoutingInformationParams.Physical_Address[1] );
            break;
        case FAPEX_HDMICEC_CEC_VERSION:
            FAPI_SYS_PRINT_MSG("CEC_Version = %x\n",message->cecCECVersionParams.CEC_Version );
            break;
        case FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS:
            FAPI_SYS_PRINT_MSG("Physical_Address[0] = %x\n",message->cecReportPhysicalAddressParams.Physical_Address[0] );
            FAPI_SYS_PRINT_MSG("Physical_Address[1] = %x\n",message->cecReportPhysicalAddressParams.Physical_Address[1] );
            FAPI_SYS_PRINT_MSG("Device_Type = %x\n",message->cecReportPhysicalAddressParams.Device_Type );
            break;
        case FAPEX_HDMICEC_SET_MENU_LANGUAGE:
            FAPI_SYS_PRINT_MSG("Language[0] = %x\n",message->cecSetMenuLanguageParams.Language[0] );
            FAPI_SYS_PRINT_MSG("Language[1] = %x\n",message->cecSetMenuLanguageParams.Language[1] );
            FAPI_SYS_PRINT_MSG("Language[2] = %x\n",message->cecSetMenuLanguageParams.Language[2] );
            break;
        case FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS:
            FAPI_SYS_PRINT_MSG("Status_Request = %x\n",message->cecGiveTunerDeviceStatusParams.Status_Request );
            break;
        case FAPEX_HDMICEC_TUNER_DEVICE_STATUS:
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[0] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[0] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[1] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[1] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[2] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[2] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[3] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[3] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[4] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[4] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[5] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[5] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[6] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[6] );
            FAPI_SYS_PRINT_MSG("Tuner_Device_Info[7] = %x\n",message->cecTunerDeviceStatusParams.Tuner_Device_Info[7] );
            break;
        case FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE:
            /* if tuner device is not capable of selecting this service then send message of FAPEX_HDMICEC_FEATURE_ABORT */
            FAPI_SYS_PRINT_MSG("Analogue_Broadcast_Type = %x\n",message->cecSelectAnalogueServiceParams.Analogue_Broadcast_Type );
            FAPI_SYS_PRINT_MSG("Analogue_Frequency[0] = %x\n",message->cecSelectAnalogueServiceParams.Analogue_Frequency[0] );
            FAPI_SYS_PRINT_MSG("Analogue_Frequency[1] = %x\n",message->cecSelectAnalogueServiceParams.Analogue_Frequency[1] );
            FAPI_SYS_PRINT_MSG("Broadcast_System = %x\n",message->cecSelectAnalogueServiceParams.Broadcast_System );
            break;
        case FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE:
            /* if tuner device is not capable of selecting this service then send message of FAPEX_HDMICEC_FEATURE_ABORT */
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[0] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[0] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[1] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[1] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[2] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[2] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[3] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[3] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[4] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[4] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[5] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[5] );
            FAPI_SYS_PRINT_MSG("Digital_Service_Identification[6] = %x\n",message->cecSelectDigitalServiceParams.Digital_Service_Identification[6] );
            break;
        case FAPEX_HDMICEC_MENU_REQUEST:
            FAPI_SYS_PRINT_MSG("Menu_Request_Type = %x\n",message->cecMenuRequestParams.Menu_Request_Type );
            break;
        case FAPEX_HDMICEC_MENU_STATUS:
            FAPI_SYS_PRINT_MSG("Menu_Request_Type = %x\n",message->cecMenuStatusParams.Menu_State );
            break;
        case FAPEX_HDMICEC_REPORT_POWER_STATUS:
            FAPI_SYS_PRINT_MSG("Power_Status = %x\n",message->cecReportPowerStatusParams.Power_Status );
            break;
        case FAPEX_HDMICEC_SET_OSD_STRING:
            FAPI_SYS_PRINT_MSG("OSD_String[ 0] = %x\n",message->cecSetOSDStringParams.OSD_String[0] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 1] = %x\n",message->cecSetOSDStringParams.OSD_String[1] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 2] = %x\n",message->cecSetOSDStringParams.OSD_String[2] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 3] = %x\n",message->cecSetOSDStringParams.OSD_String[3] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 4] = %x\n",message->cecSetOSDStringParams.OSD_String[4] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 5] = %x\n",message->cecSetOSDStringParams.OSD_String[5] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 6] = %x\n",message->cecSetOSDStringParams.OSD_String[6] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 7] = %x\n",message->cecSetOSDStringParams.OSD_String[7] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 8] = %x\n",message->cecSetOSDStringParams.OSD_String[8] );
            FAPI_SYS_PRINT_MSG("OSD_String[ 9] = %x\n",message->cecSetOSDStringParams.OSD_String[9] );
            FAPI_SYS_PRINT_MSG("OSD_String[10] = %x\n",message->cecSetOSDStringParams.OSD_String[10] );
            FAPI_SYS_PRINT_MSG("OSD_String[11] = %x\n",message->cecSetOSDStringParams.OSD_String[11] );
            FAPI_SYS_PRINT_MSG("OSD_String[12] = %x\n",message->cecSetOSDStringParams.OSD_String[12] );
            break;
        case FAPEX_HDMICEC_USER_CONTROL_PRESSED:
            FAPI_SYS_PRINT_MSG("UI_Command = %x\n",message->cecUserControlPressedParams.UI_Command );
            break;
        case FAPEX_HDMICEC_INACTIVE_SOURCE:
            FAPI_SYS_PRINT_MSG("Physical_Address[0] = %x\n",message->cecInactiveSourceParams.Physical_Address[0] );
            FAPI_SYS_PRINT_MSG("Physical_Address[1] = %x\n",message->cecInactiveSourceParams.Physical_Address[1] );
            break;
        case FAPEX_HDMICEC_ROUTING_CHANGE:
            FAPI_SYS_PRINT_MSG("Original_Address[0] = %x\n",message->cecRoutingChangeParams.Original_Address[0] );
            FAPI_SYS_PRINT_MSG("Original_Address[1] = %x\n",message->cecRoutingChangeParams.Original_Address[1] );
            FAPI_SYS_PRINT_MSG("New_Address[0] = %x\n",message->cecRoutingChangeParams.New_Address[0] );
            FAPI_SYS_PRINT_MSG("New_Address[1] = %x\n",message->cecRoutingChangeParams.New_Address[1] );
            break;
        case FAPEX_HDMICEC_SET_STREAM_PATH:
            FAPI_SYS_PRINT_MSG("Physical_Address[0] = %x\n",message->cecSetStreamPathParams.Physical_Address[0] );
            FAPI_SYS_PRINT_MSG("Physical_Address[1] = %x\n",message->cecSetStreamPathParams.Physical_Address[1] );
            break;
        default:
            FAPI_SYS_PRINT_MSG("A message parameter is none.\n");
            break;
    }

    return result;
}

/*!
*******************************************************************************
** Perform message sending and the receive that a user appointed.
**
** This function performs message sending and the receive that a user appointed.
**
** \param message              This parameter is set to the message number. Refer to macro value(FAPEX_HDMICEC_STANDBY... and so on) in drv_hdmi.h.
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendOrReceiveMessage(int32_t message, int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    int32_t  result;
    uint32_t physAddr[2];
    uint32_t PackphysAddr = 0;
    uint32_t ack = 0;

    /* Get physical address in EDID */
    /* Note : Before executing HDMI_CECT10, it is nessesary that HDMI_LinkHDCP is executed. */
    result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                               FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR,
                               (FAPEXIOCARG3_T)&PackphysAddr );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[hdmi_cec]:FAPI_HDMI_GetPhysicalAddressInEDID return = %d\n",result);
        return result;
    }

    physAddr[0] = (PackphysAddr >> 8) & 0x000000ff;
    physAddr[1] = PackphysAddr & 0x000000ff;

    FAPI_SYS_PRINT_MSG("[hdmi_cec]:FAPEX_HDMI_GetPhysicalAddressInEDID physical address = %x\n",PackphysAddr);

    switch(message){
        case FAPEX_HDMICEC_STANDBY:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_STANDBY\n");
            result = HDMI_CEC_SendMessage_Standby(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS\n");
            result = HDMI_CEC_SendMessage_GiveTunerDeviceStatus(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE\n");
            result = HDMI_CEC_SendMessage_SelectAnalogueService(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE\n");
            result = HDMI_CEC_SendMessage_SelectDigitalService(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_TUNER_DEVICE_STATUS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_TUNER_DEVICE_STATUS\n");
            result = HDMI_CEC_SendMessage_TunerDeviceStatus(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_TUNER_STEP_DECREMENT:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_TUNER_STEP_DECREMENT\n");
            result = HDMI_CEC_SendMessage_TunerStepDecrement(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_TUNER_STEP_INCREMENT:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_TUNER_STEP_INCREMENT\n");
            result = HDMI_CEC_SendMessage_TunerStepIncrement(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_SET_OSD_STRING:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_SET_OSD_STRING\n");
            result = HDMI_CEC_SendMessage_SetOSDString(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_MENU_REQUEST:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_MENU_REQUEST\n");
            result = HDMI_CEC_SendMessage_MenuRequest(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_MENU_STATUS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_MENU_STATUS\n");
            result = HDMI_CEC_SendMessage_MenuStatus(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_USER_CONTROL_PRESSED:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_USER_CONTROL_PRESSED\n");
            result = HDMI_CEC_SendMessage_UserControlPressed(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_USER_CONTROL_RELEASED:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_USER_CONTROL_RELEASED\n");
            result = HDMI_CEC_SendMessage_UserControlReleased(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS\n");
            result = HDMI_CEC_SendMessage_GiveDevicePowerStatus(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_REPORT_POWER_STATUS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_REPORT_POWER_STATUS\n");
            result = HDMI_CEC_SendMessage_ReportPowerStatus(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_CEC_VERSION:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_CEC_VERSION\n");
            result = HDMI_CEC_SendMessage_CECVersion(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_GET_CEC_VERSION:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_GET_CEC_VERSION\n");
            result = HDMI_CEC_SendMessage_GetCECVersion(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS\n");
            result = HDMI_CEC_SendMessage_GivePhysicalAddress(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_GET_MENU_LANGUAGE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_GET_MENU_LANGUAGE\n");
            result = HDMI_CEC_SendMessage_GetMenuLanguage(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS\n");
            result = HDMI_CEC_SendMessage_ReportPhysicalAddress(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr,
                                                                (uint8_t)physAddr[0], (uint8_t)physAddr[1]);
            break;
        case FAPEX_HDMICEC_SET_MENU_LANGUAGE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_SET_MENU_LANGUAGE\n");
            result = HDMI_CEC_SendMessage_SetMenuLanguage(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_FEATURE_ABORT:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_FEATURE_ABORT\n");
            result = HDMI_CEC_SendMessage_FeatureAbort(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_ABORT:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_ABORT\n");
            result = HDMI_CEC_SendMessage_Abort(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_ACTIVE_SOURCE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_ACTIVE_SOURCE\n");
            result = HDMI_CEC_SendMessage_ActiveSource(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr,
                                                       (uint8_t)physAddr[0], (uint8_t)physAddr[1]);
            break;
        case FAPEX_HDMICEC_IMAGE_VIEW_ON:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_IMAGE_VIEW_ON\n");
            result = HDMI_CEC_SendMessage_ImageViewOn(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_TEXT_VIEW_ON:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_TEXT_VIEW_ON\n");
            result = HDMI_CEC_SendMessage_TextViewOn(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_INACTIVE_SOURCE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_INACTIVE_SOURCE\n");
            result = HDMI_CEC_SendMessage_InactiveSource(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE\n");
            result = HDMI_CEC_SendMessage_RequestActiveSource(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_ROUTING_CHANGE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_ROUTING_CHANGE\n");
            result = HDMI_CEC_SendMessage_RoutingChange(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_ROUTING_INFORMATION:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_ROUTING_INFORMATION\n");
            result = HDMI_CEC_SendMessage_RoutingInformation(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_SET_STREAM_PATH:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_SET_STREAM_PATH\n");
            result = HDMI_CEC_SendMessage_SetStreamPath(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
            break;
        case FAPEX_HDMICEC_POLLING_MESSAGE:
            FAPI_SYS_PRINT_MSG("Send CEC message of FAPEX_HDMICEC_POLLING_MESSAGE\n");
            result = HDMI_CEC_SendMessage_PollingMessage(send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr, &ack);
            break;
        default:
            FAPI_SYS_PRINT_MSG("No send CEC message.\n");
            break;
    }

    return result;
}

/*!
*******************************************************************************
** Check intigrity of CEC message
**
** This function checks intigrity of CEC message.
**
** \param message      This parameter is set to the message value(opcode).
** \param RecInitiator This parameter is set to the receiving initiator address.
** \param MessageType  This parameter is set to message type if broadcast message is 1 or directly message is 0.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_CheckIntigrityOfMessage(uint32_t message, uint32_t MessageType)
{
    int32_t status = FAPI_OK;

    switch(message){
        case FAPEX_HDMICEC_STANDBY:
            /* Directly addressed Y */
            /* Broadcast Y */
            break;
        case FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_TUNER_DEVICE_STATUS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_TUNER_STEP_DECREMENT:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_TUNER_STEP_INCREMENT:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_SET_OSD_STRING:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_MENU_REQUEST:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_MENU_STATUS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_USER_CONTROL_PRESSED:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_USER_CONTROL_RELEASED:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_REPORT_POWER_STATUS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_CEC_VERSION:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_GET_CEC_VERSION:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_GET_MENU_LANGUAGE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS:
#if 0
            /* Directly addressed N */
            /* Broadcast Y */
            if(MessageType == 0){
                status = -1;
            }
#else
            /* Follower is TV only. Therefore tuner can not receive this message. */
            status = -1;
#endif
            break;
        case FAPEX_HDMICEC_SET_MENU_LANGUAGE:
            /* Directly addressed N */
            /* Broadcast Y */
            if(MessageType == 0){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_FEATURE_ABORT:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_ABORT:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_ACTIVE_SOURCE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_IMAGE_VIEW_ON:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_TEXT_VIEW_ON:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_INACTIVE_SOURCE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE:
            /* Directly addressed N */
            /* Broadcast Y */
            if(MessageType == 0){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_ROUTING_CHANGE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_ROUTING_INFORMATION:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_SET_STREAM_PATH:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_DEVICE_VENDOR_ID:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        case FAPEX_HDMICEC_SET_SYSTEM_AUDIO_MODE:
            /* Directly addressed Y */
            /* Broadcast N */
            if(MessageType == 1){
                status = -1;
            }
            break;
        default:
            break;
    }

    return status;
}

/*!
*******************************************************************************
** exexute Device Installation and Addressing
**
** This function executes Device Installation and Addressing.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_DeviceInstrationAndAddressing(void)
{
    int32_t  result, status;
    uint32_t physAddr[2];
    uint32_t PackphysAddr = 0;
    uint32_t MyDevLogicalAddr = 0;
    volatile int32_t hpd_status = 0;
    uint64_t start_time;
    uint64_t end_time;
    int32_t  time_difference;

    start_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_100_USEC);

    /* HPD */
    do {
        FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                          FAPEX_IOCCOM_HDMI_CHECK_HPD,
                          (FAPEXIOCARG3_T)&hpd_status );
    } while((hpd_status & 0x0f) != 0x06);

    FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                      FAPEX_IOCCOM_HDMI_READ_EDID,
                      (FAPEXIOCARG3_T)&status );
    if(status != FAPI_OK) {
        FAPI_SYS_PRINT_MSG("FAPI_HDMI_ReadEDID status = %d\n", status);
        return status;
    }

    /* Get physical address in EDID */
    /* Note : Before executing HDMI_CECT10, it is nessesary that HDMI_LinkHDCP is executed. */
    result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                               FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR,
                               (FAPEXIOCARG3_T)&PackphysAddr );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("FAPI_HDMI_GetPhysicalAddressInEDID status = %d\n",result);
        return result;
    }

    FAPI_SYS_PRINT_MSG("PackphysAddr = %x\n",PackphysAddr);

    physAddr[0] = (PackphysAddr >> 8) & 0x000000ff;
    physAddr[1] = PackphysAddr & 0x000000ff;

    /* Allocate Logical Address */
    result = HDMI_CEC_AllocateLogicalAddress();
    if(result != 0) {
        FAPI_SYS_PRINT_MSG("HDMI_CEC_AllocateLogicalAddress status = %d\n",result);
    }

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS,
                                  (FAPEXIOCARG3_T)&MyDevLogicalAddr );

    FAPI_SYS_PRINT_MSG("FAPI_HDMI_GetCecLogicalAddress MyDevLogicalAddr = %d\n",MyDevLogicalAddr);

    /* <Report Physical Address> broadcast */
    FAPI_SYS_PRINT_MSG("<Report Physical Address> broadcast\n");
    (void)HDMI_CEC_SendMessage_ReportPhysicalAddress(1, 0, MyDevLogicalAddr, 0, (uint8_t)physAddr[0], (uint8_t)physAddr[1]);

    /* HPD */
    do {
        hpd_status = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                                       FAPEX_IOCCOM_HDMI_CHECK_HPD,
                                       (FAPEXIOCARG3_T)NULL );
        end_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_100_USEC);
        time_difference = (int32_t)(end_time - start_time);
        if( time_difference > 300000 ) {  /* 30 sec */
            FAPI_SYS_PRINT_MSG("time_difference %d\n", time_difference);
            FAPI_SYS_PRINT_MSG("It is a time out.There was not a response to hpd signal.\n");
            return -1;
        }
    } while((hpd_status & 0x0f) == 0x06);

    FAPI_SYS_PRINT_MSG("hpd_status %x\n", hpd_status);

    return result;
}

/*!
*******************************************************************************
** Communicate with CEC message.
**
** This function communicate with CEC message.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_Communication(void)
{
    int32_t  result;
    uint8_t  physAddr[2];
    uint32_t PackphysAddr;
    uint32_t initiatorLogicalAddr = 0;
    uint32_t followerLogicalAddr  = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;
    uint32_t RemainRecMsg = 0;
    uint32_t RecStatus    = 0;

    /* Get logical address (FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1) */
    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS,
                                  (FAPEXIOCARG3_T)&initiatorLogicalAddr );
    if(result != 0){
        return result;
    }

    /* Get physical address in EDID */
    /* Note : Before executing HDMI_CECT10, it is nessesary that HDMI_LinkHDCP is executed. */
    result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                               FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR,
                               (FAPEXIOCARG3_T)&PackphysAddr );
    if(result < 0) {
        return result;
    }

    if(PackphysAddr == 0){
        return -1;
    }

    physAddr[0] = (PackphysAddr >> 8) & 0x000000ff;
    physAddr[1] = PackphysAddr & 0x000000ff;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_GET_RECEIVE_STATUS,
                                  (FAPEXIOCARG3_T)&RecStatus );

    if(result == 0){
        FAPI_SYS_PRINT_MSG("[Reception complete]\n");

        if(RecStatus == FAPEX_HDMICEC_RECEPTION_ERROR) {
            FAPI_SYS_PRINT_MSG("ERROR:[hdmi_cec]:FAPI_HDMI_CheckCecReceiveStatus "
                               "return FAPEX_HDMICEC_RECEPTION_ERROR\n");
        }

        (void)HDMI_CEC_GetMsg(&RecInitiator, &RecMessage, &RemainRecMsg, &MessageType);

        /**** check integrity of message ****/
        result = HDMI_CEC_CheckIntigrityOfMessage(RecMessage, MessageType);
        if(result != FAPI_OK){
            FAPI_SYS_PRINT_MSG("ERROR:[hdmi_cec]:Ignores the invalid CEC Message. %d\n", result);
            return(FAPI_OK);
        }
        followerLogicalAddr = RecInitiator;

        switch(RecMessage){
            case FAPEX_HDMICEC_ABORT:
                /* send message of FAPEX_HDMICEC_FEATURE_ABORT */
            case FAPEX_HDMICEC_GIVE_DEVICE_VENDOR_ID:
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_FeatureAbort\n");
                (void)HDMI_CEC_SendMessage_FeatureAbort(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE:
                /* send message of FAPEX_HDMICEC_ACTIVE_SOURCE */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_ActiveSource\n");

                (void)HDMI_CEC_SendMessage_ActiveSource(1, 0, initiatorLogicalAddr, followerLogicalAddr, (uint8_t)physAddr[0], (uint8_t)physAddr[1]);
                break;
            case FAPEX_HDMICEC_ROUTING_CHANGE:
                /* send message of FAPEX_HDMICEC_ROUTING_INFORMATION */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_RoutingInformation\n");

                (void)HDMI_CEC_SendMessage_RoutingInformation(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_GET_CEC_VERSION:
                /* send message of FAPEX_HDMICEC_CEC_VERSION */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_CECVersion\n");

                (void)HDMI_CEC_SendMessage_CECVersion(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS:
                /* send message of FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_ReportPhysicalAddress\n");

                (void)HDMI_CEC_SendMessage_ReportPhysicalAddress(1, 0, initiatorLogicalAddr, followerLogicalAddr, physAddr[0], physAddr[1]);
                break;
            case FAPEX_HDMICEC_GET_MENU_LANGUAGE:
                /* send message of FAPEX_HDMICEC_SET_MENU_LANGUAGE */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_SetMenuLanguage\n");

                (void)HDMI_CEC_SendMessage_SetMenuLanguage(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS:
                /* send message of FAPEX_HDMICEC_TUNER_DEVICE_STATUS */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_TunerDeviceStatus\n");

                (void)HDMI_CEC_SendMessage_TunerDeviceStatus(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE:
                /* if tuner device is not capable of selecting this service then send message of FAPEX_HDMICEC_FEATURE_ABORT */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_FeatureAbort\n");

                (void)HDMI_CEC_SendMessage_FeatureAbort(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE:
                /* if tuner device is not capable of selecting this service then send message of FAPEX_HDMICEC_FEATURE_ABORT */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_FeatureAbort\n");

                (void)HDMI_CEC_SendMessage_FeatureAbort(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_MENU_REQUEST:
                /* send message of FAPEX_HDMICEC_MENU_STATUS */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_MenuStatus\n");

                (void)HDMI_CEC_SendMessage_MenuStatus(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS:
                /* send message of FAPEX_HDMICEC_REPORT_POWER_STATUS */
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_ReportPowerStatus\n");

                (void)HDMI_CEC_SendMessage_ReportPowerStatus(1, 0, initiatorLogicalAddr, followerLogicalAddr);
                break;
            case FAPEX_HDMICEC_STANDBY:
                FAPI_SYS_PRINT_MSG("Tuner execute the processing of stadby mode.\n");
                break;

            default:
                /* Not sending response */
                FAPI_SYS_PRINT_MSG("No send CEC message.\n");
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_RECEIVE_MESSAGE,
                             (FAPEXIOCARG3_T)&result );
    }
    return result;
}

/*!
*******************************************************************************
** Receive CEC Message.
**
** This function receives CEC message.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_ReceivingMessage(void)
{
    int32_t  status, result;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                         FAPEX_IOCCOM_HDMICEC_RECEIVE_MESSAGE,
                         (FAPEXIOCARG3_T)&status );
    if(status != FAPI_OK) {
        return status;
    }

    result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
    if(result != FAPI_OK) {
        FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
    }

    return result;
}

/*!
*******************************************************************************
** Allocate the logical address on my hdmi device.
**
** This function allocates the logical address on my hdmi device.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_AllocateLogicalAddress(void)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    uint32_t cnt;
    uint32_t send_status = 0;
    int32_t  result;

//  FAPI_SYS_PRINT_MSG("Allocate logical address\n");
    for(cnt=0; cnt<4; cnt++){
        FAPI_SYS_PRINT_MSG("start allocate logical address\n");

        params.initiatorLogicalAddr = logicalAdder[cnt];
        params.followerLogicalAddr  = logicalAdder[cnt];
        params.opcodeName           = FAPEX_HDMICEC_POLLING_MESSAGE;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            return status;
        }

        HDMI_CEC_ControlSendMessage();

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_CHECK_SEND_STATUS,
                                      (FAPEXIOCARG3_T)&send_status );

        if(send_status == FAPEX_HDMICEC_SENDING_COMPLETE) {
            result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                          FAPEX_IOCCOM_HDMICEC_SET_LOGICAL_ADDRESS,
                                          (FAPEXIOCARG3_T)params.initiatorLogicalAddr );
            FAPI_SYS_PRINT_MSG("InitiatorLogicalAddr(my device logical addr) = %d\n", params.initiatorLogicalAddr );
            return result;
        }
    }

    params.initiatorLogicalAddr = 0x0f;
    FAPI_SYS_PRINT_MSG("InitiatorLogicalAddr(my device logical addr) = %d\n", params.initiatorLogicalAddr );

    return result;
}

/*!
*******************************************************************************
** Set the logical address in own device.
**
** This function sets the logical address in own device.
**
** \param MyDevLogicalAddr A value of logical address.
**
*******************************************************************************
*/
int32_t HDMI_CEC_SetLogicalAddress(int32_t MyDevLogicalAddr)
{
    int32_t result;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_SET_LOGICAL_ADDRESS,
                                  (FAPEXIOCARG3_T)MyDevLogicalAddr );
    return result;
}

/*!
*******************************************************************************
** Get the logical address in own device.
**
** This function gets the logical address in own device.
**
** \return A logical address value.
**
*******************************************************************************
*/
uint32_t HDMI_CEC_GetLogicalAddress(void)
{
    uint32_t MyDevLogicalAddr;
    int32_t  result;

    result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                  FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS,
                                  (FAPEXIOCARG3_T)&MyDevLogicalAddr );
    if(result == 0) {
        return MyDevLogicalAddr;
    }
    else {
        return 0;
    }
}

/*!
*******************************************************************************
** Set CEC message parameter.
**
** This function sets CEC message parameter.
**
** \param opcodeName A vlaue of operation code for CEC.
** \param pRecInitiator A value of cec parameter for CEC.
**
*******************************************************************************
*/
void HDMI_CEC_SetCecMessageParameters( uint32_t opcodeName,
                                          uint8_t cecparambuf[])
{
    switch(opcodeName){
       case FAPEX_HDMICEC_STANDBY:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS:
           hdmicecMsgParamUserSetting.cecGiveTunerDeviceStatusParams.Status_Request = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE:
           hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Broadcast_Type = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[0] = cecparambuf[1];
           hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[1] = cecparambuf[2];
           hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Broadcast_System = cecparambuf[3];
           break;
       case FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE:
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[1] = cecparambuf[1];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[2] = cecparambuf[2];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[3] = cecparambuf[3];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[4] = cecparambuf[4];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[5] = cecparambuf[5];
           hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[6] = cecparambuf[6];
           break;
       case FAPEX_HDMICEC_TUNER_DEVICE_STATUS:
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.deviceType = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[0] = cecparambuf[1];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[1] = cecparambuf[2];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[2] = cecparambuf[3];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[3] = cecparambuf[4];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[4] = cecparambuf[5];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[5] = cecparambuf[6];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[6] = cecparambuf[7];
           hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[7] = cecparambuf[8];
           break;
       case FAPEX_HDMICEC_TUNER_STEP_DECREMENT:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_TUNER_STEP_INCREMENT:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_SET_OSD_STRING:
           hdmicecMsgParamUserSetting.cecSetOSDStringParams.Display_Control = cecparambuf[0];
           /***************************************************/
           /*************during correspondence*****************/
           /***************************************************/
           break;
       case FAPEX_HDMICEC_MENU_REQUEST:
           hdmicecMsgParamUserSetting.cecMenuRequestParams.Menu_Request_Type = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_MENU_STATUS:
           hdmicecMsgParamUserSetting.cecMenuStatusParams.Menu_State = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_USER_CONTROL_PRESSED:
           hdmicecMsgParamUserSetting.cecUserControlPressedParams.UI_Command = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_USER_CONTROL_RELEASED:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_REPORT_POWER_STATUS:
           hdmicecMsgParamUserSetting.cecReportPowerStatusParams.Power_Status = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_CEC_VERSION:
           hdmicecMsgParamUserSetting.cecCECVersionParams.CEC_Version = cecparambuf[0];
           break;
       case FAPEX_HDMICEC_GET_CEC_VERSION:
           /* not paramter */
           break;
       case FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS:
           /* not paramter */
           break;
       case FAPEX_HDMICEC_GET_MENU_LANGUAGE:
           /* not paramter */
           break;
       case FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS:
           /***************************************************/
           /*************during correspondence*****************/
           /***************************************************/
           break;
       case FAPEX_HDMICEC_SET_MENU_LANGUAGE:
           hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[1] = cecparambuf[1];
           hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[2] = cecparambuf[2];
           break;
       case FAPEX_HDMICEC_FEATURE_ABORT:
           hdmicecMsgParamUserSetting.cecFeatureAbortParams.Feature_Opcode = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecFeatureAbortParams.Abort_Reason = cecparambuf[1];
           break;
       case FAPEX_HDMICEC_ABORT:
            /* not parameter */
           break;
       case FAPEX_HDMICEC_ACTIVE_SOURCE:
           hdmicecMsgParamUserSetting.cecActiveSourceParams.Physical_Address[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecActiveSourceParams.Physical_Address[1] = cecparambuf[1];
           break;
       case FAPEX_HDMICEC_IMAGE_VIEW_ON:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_TEXT_VIEW_ON:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_INACTIVE_SOURCE:
           hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[1] = cecparambuf[1];
           break;
       case FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE:
           /* not parameter */
           break;
       case FAPEX_HDMICEC_ROUTING_CHANGE:
           hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[1] = cecparambuf[1];
           hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[0] = cecparambuf[2];
           hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[1] = cecparambuf[3];
           break;
       case FAPEX_HDMICEC_ROUTING_INFORMATION:
           hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[1] = cecparambuf[1];
           break;
       case FAPEX_HDMICEC_SET_STREAM_PATH:
           hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[0] = cecparambuf[0];
           hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[1] = cecparambuf[1];
           break;
       default:
           break;
    }
}

/*!
*******************************************************************************
** Send "standby" message in CEC.
**
** This function sends "standby" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_Standby(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_STANDBY;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);

        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Give Tuner Device Status" message in CEC.
**
** This function sends "Give Tuner Device Status" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_GiveTunerDeviceStatus(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS;
        params.message.message_type = 0;
        params.message.cecGiveTunerDeviceStatusParams.Status_Request = hdmicecMsgParamUserSetting.cecGiveTunerDeviceStatusParams.Status_Request;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Select Analogue Service" message in CEC.
**
** This function sends "standby" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_SelectAnalogueService(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE;
        params.message.message_type = 0;
        params.message.cecSelectAnalogueServiceParams.Analogue_Broadcast_Type = hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Broadcast_Type;
        params.message.cecSelectAnalogueServiceParams.Analogue_Frequency[0]   = hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[0];
        params.message.cecSelectAnalogueServiceParams.Analogue_Frequency[1]   = hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Analogue_Frequency[1];
        params.message.cecSelectAnalogueServiceParams.Broadcast_System        = hdmicecMsgParamUserSetting.cecSelectAnalogueServiceParams.Broadcast_System;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Select Digital Service" message in CEC.
**
** This function sends "Select Digital Service" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_SelectDigitalService(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE;
        params.message.message_type = 0;
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[0] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[0];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[1] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[1];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[2] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[2];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[3] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[3];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[4] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[4];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[5] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[5];
        params.message.cecSelectDigitalServiceParams.Digital_Service_Identification[6] = hdmicecMsgParamUserSetting.cecSelectDigitalServiceParams.Digital_Service_Identification[6];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Tuner Device Status" message in CEC.
**
** This function sends "Tuner Device Status" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_TunerDeviceStatus(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_TUNER_DEVICE_STATUS;
        params.message.message_type = 0;
        params.message.cecTunerDeviceStatusParams.deviceType           = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.deviceType;
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[0] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[0];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[1] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[1];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[2] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[2];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[3] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[3];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[4] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[4];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[5] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[5];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[6] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[6];
        params.message.cecTunerDeviceStatusParams.Tuner_Device_Info[7] = hdmicecMsgParamUserSetting.cecTunerDeviceStatusParams.Tuner_Device_Info[7];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Tuner Step Decrement" message in CEC.
**
** This function sends "Tuner Step Decrement" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_TunerStepDecrement(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_TUNER_STEP_DECREMENT;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Tuner Step Increment" message in CEC.
**
** This function sends "Tuner Step Increment" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_TunerStepIncrement(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_TUNER_STEP_INCREMENT;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Set OSD String" message in CEC.
**
** This function sends "Set OSD String" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_SetOSDString(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint8_t  str[13] = "Fujitsu";
    uint32_t RecMessage = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_SET_OSD_STRING;
        params.message.message_type = 0;
        params.message.cecSetOSDStringParams.Display_Control = 0x00;
        params.message.cecSetOSDStringParams.StringLength = 7;
        memcpy(params.message.cecSetOSDStringParams.OSD_String, str, params.message.cecSetOSDStringParams.StringLength);

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Menu Request" message in CEC.
**
** This function sends "Menu Request" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_MenuRequest(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_MENU_REQUEST;
        params.message.message_type = 0;
        params.message.cecMenuRequestParams.Menu_Request_Type = hdmicecMsgParamUserSetting.cecSetOSDStringParams.Display_Control;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Menu Status" message in CEC.
**
** This function sends "Menu Status" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_MenuStatus(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_MENU_STATUS;
        params.message.message_type = 0;
        params.message.cecMenuStatusParams.Menu_State = hdmicecMsgParamUserSetting.cecMenuStatusParams.Menu_State;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "User Control Pressed" message in CEC.
**
** This function sends "User Control Pressed" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_UserControlPressed(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_USER_CONTROL_PRESSED;
        params.message.message_type = 0;
        params.message.cecUserControlPressedParams.UI_Command = hdmicecMsgParamUserSetting.cecUserControlPressedParams.UI_Command;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "User Control Released" message in CEC.
**
** This function sends "User Control Released" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_UserControlReleased(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_USER_CONTROL_RELEASED;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}


/*!
*******************************************************************************
** Send "Give Device Power Status" message in CEC.
**
** This function sends "Give Device Power Status" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_GiveDevicePowerStatus(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Report Power Status" message in CEC.
**
** This function sends "Report Power Status" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_ReportPowerStatus(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_REPORT_POWER_STATUS;
        params.message.message_type = 0;
        params.message.cecReportPowerStatusParams.Power_Status = hdmicecMsgParamUserSetting.cecReportPowerStatusParams.Power_Status;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "CEC Version" message in CEC.
**
** This function sends "CEC Version" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t  HDMI_CEC_SendMessage_CECVersion(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_CEC_VERSION;
        params.message.message_type = 0;
        params.message.cecCECVersionParams.CEC_Version = hdmicecMsgParamUserSetting.cecCECVersionParams.CEC_Version;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Get CEC Version" message in CEC.
**
** This function sends "Get CEC Version" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_GetCECVersion(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_GET_CEC_VERSION;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Give Physical Address" message in CEC.
**
** This function sends "Give Physical Address" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_GivePhysicalAddress(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Get Menu Language" message in CEC.
**
** This function sends "Get Menu Language" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_GetMenuLanguage(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_GET_MENU_LANGUAGE;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Report Physical Address" message in CEC.
**
** This function sends "Report Physical Address" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
** \param UpperPhysAddr        This parameter is set to the upper physical address.
** \param LowerPhysAddr        This parameter is set to the lower physical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_ReportPhysicalAddress(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr, uint8_t UpperPhysAddr, uint8_t LowerPhysAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS;
        params.message.message_type = 1;
        params.message.cecReportPhysicalAddressParams.Physical_Address[0] = UpperPhysAddr; /* Upper */
        params.message.cecReportPhysicalAddressParams.Physical_Address[1] = LowerPhysAddr; /* Lower */
        params.message.cecReportPhysicalAddressParams.Device_Type = 3;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Set Menu Language" message in CEC.
**
** This function sends "Set Menu Language" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_SetMenuLanguage(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_SET_MENU_LANGUAGE;
        params.message.message_type = 1;
        params.message.cecSetMenuLanguageParams.Language[0] = hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[0];
        params.message.cecSetMenuLanguageParams.Language[1] = hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[1];
        params.message.cecSetMenuLanguageParams.Language[2] = hdmicecMsgParamUserSetting.cecSetMenuLanguageParams.Language[2];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Feature Abort" message in CEC.
**
** This function sends "Feature Abort" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_FeatureAbort(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_FEATURE_ABORT;
        params.message.message_type = 0;
        params.message.cecFeatureAbortParams.Feature_Opcode = hdmicecMsgParamUserSetting.cecFeatureAbortParams.Feature_Opcode;
        params.message.cecFeatureAbortParams.Abort_Reason = hdmicecMsgParamUserSetting.cecFeatureAbortParams.Abort_Reason;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Abort" message in CEC.
**
** This function sends "Abort" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_Abort(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_ABORT;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Active Source" message in CEC.
**
** This function sends "Active Source" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_ActiveSource(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr, uint8_t UpperPhysAddr, uint8_t LowerPhysAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_ACTIVE_SOURCE;
        params.message.message_type = 1;
        params.message.cecActiveSourceParams.Physical_Address[0] = UpperPhysAddr; /* Upper */
        params.message.cecActiveSourceParams.Physical_Address[1] = LowerPhysAddr; /* Lower */

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Image View On" message in CEC.
**
** This function sends "Image View On" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_ImageViewOn(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_IMAGE_VIEW_ON;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Text View On" message in CEC.
**
** This function sends "Text View On" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_TextViewOn(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_TEXT_VIEW_ON;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Inactive Source" message in CEC.
**
** This function sends "Inactive Source" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_InactiveSource(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_INACTIVE_SOURCE;
        params.message.message_type = 0;
        params.message.cecInactiveSourceParams.Physical_Address[0] = hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[0];
        params.message.cecInactiveSourceParams.Physical_Address[1] = hdmicecMsgParamUserSetting.cecInactiveSourceParams.Physical_Address[1];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Request Active Source" message in CEC.
**
** This function sends "Request Active Source" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_RequestActiveSource(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE;
        params.message.message_type = 1;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Routing Change" message in CEC.
**
** This function sends "Routing Change" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_RoutingChange(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_ROUTING_CHANGE;
        params.message.message_type = 1;
        params.message.cecRoutingChangeParams.Original_Address[0] = hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[0];
        params.message.cecRoutingChangeParams.Original_Address[1] = hdmicecMsgParamUserSetting.cecRoutingChangeParams.Original_Address[1];
        params.message.cecRoutingChangeParams.New_Address[0]      = hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[0];
        params.message.cecRoutingChangeParams.New_Address[1]      = hdmicecMsgParamUserSetting.cecRoutingChangeParams.New_Address[1];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Routing Information" message in CEC.
**
** This function sends "Routing Information" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_RoutingInformation(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_ROUTING_INFORMATION;
        params.message.message_type = 1;
        params.message.cecRoutingInformationParams.Physical_Address[0] = hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[0];
        params.message.cecRoutingInformationParams.Physical_Address[1] = hdmicecMsgParamUserSetting.cecRoutingInformationParams.Physical_Address[1];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Set Stream Path" message in CEC.
**
** This function sends "Set Stream Path" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_SetStreamPath(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST;
        params.opcodeName           = FAPEX_HDMICEC_SET_STREAM_PATH;
        params.message.message_type = 1;
        params.message.cecSetStreamPathParams.Physical_Address[0] = hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[0];
        params.message.cecSetStreamPathParams.Physical_Address[1] = hdmicecMsgParamUserSetting.cecSetStreamPathParams.Physical_Address[1];

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}

/*!
*******************************************************************************
** Send "Polling" message in CEC.
**
** This function sends "Polling" message in CEC.
**
** \param send_flag            This parameter is set to the sending flag.
** \param rec_flag             This parameter is set to the receiving flag.
** \param InitiatorLogicalAddr This parameter is set to the initiator logical address.
** \param FollowerLogicalAddr  This parameter is set to the follower logical address.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t HDMI_CEC_SendMessage_PollingMessage(int32_t send_flag, int32_t rec_flag, uint32_t InitiatorLogicalAddr, uint32_t FollowerLogicalAddr, uint32_t *ack)
{
    FAPEX_HDMICEC_MessageParamStrT  params;
    int32_t  status;
    int32_t  result = 0;
    uint32_t RecMessage   = 0;
    uint32_t RecInitiator = 0;
    uint32_t MessageType  = 0;
    uint32_t send_status  = 0;

    if(send_flag == 1) {
        params.initiatorLogicalAddr = InitiatorLogicalAddr;
#ifdef DEBUG_SRC_TO_SRC
        params.followerLogicalAddr  = FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1;
#else
        params.followerLogicalAddr  = FollowerLogicalAddr;
#endif
        params.opcodeName           = FAPEX_HDMICEC_POLLING_MESSAGE;
        params.message.message_type = 0;

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_SET_MESSAGE,
                                      (FAPEXIOCARG3_T)&params );
        if(result != 0) {
            return result;
        }

        FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                             FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE,
                             (FAPEXIOCARG3_T)&status );
        if(status != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_SendCecMessage status = %d\n", status);
            return status;
        }

        HDMI_CEC_ControlSendMessage();

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_CHECK_SEND_STATUS,
                                      (FAPEXIOCARG3_T)&send_status );
        if(send_status == FAPEX_HDMICEC_SENDING_COMPLETE) {
            *ack = 0;
            FAPI_SYS_PRINT_MSG("Negative acknowledge\n");
        }
        else {
            *ack = 1;
            FAPI_SYS_PRINT_MSG("Acknowledge\n");
        }
    }

    if(rec_flag == 1) {
        result = HDMI_CEC_ControlRecMessage(&RecMessage, &RecInitiator, &MessageType);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_ControlRecMessage status = %d\n", result);
        }
        return result;
    }

    return result;
}
#endif /* USE_HDMICEC */

/*!
*******************************************************************************
**
** HDMI Task.
**
** Called periodically too check HPD and communicate CEC line.
**
** \return '0' HPD on, '1' communicating CEC, '-1' HPD off.
*******************************************************************************
*/
int32_t HDMI_Task(void)
{
    volatile int32_t hpd_status = 0;
    static   int32_t prv_status = FAPEX_HDMI_STATUS_OK;
    int32_t  status, result;
#ifdef USE_HDMICEC
    uint32_t physAddr[2] = {0, 0};
    uint32_t PackphysAddr = 0;
    uint32_t MyDevLogicalAddr = 0;
#endif

    /* check HPD */
    FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                      FAPEX_IOCCOM_HDMI_CHECK_HPD,
                      (FAPEXIOCARG3_T)&hpd_status );
    if((hpd_status & 0x0f) != 0x06) {
        return -1;
    }

    /* HPD interrupt ? */
    if(hpd_status & 0xf0) {
        /* read EDID data */
        FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                          FAPEX_IOCCOM_HDMI_READ_EDID,
                          (FAPEXIOCARG3_T)&status );

        FAPI_SYS_PRINT_MSG("\nFAPI_HDMI_ReadEDID hpd_status = %x\n",hpd_status);

        if(status != 0) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_ReadEDID status = %d\n", status);
            return status;
        }

        /* get EDID Information */
        result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                                   FAPEX_IOCCOM_HDMI_GET_EDID_INFO,
                                   (FAPEXIOCARG3_T)&appEdidInfo );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR: FAPI_HDMI_GetEDID_Info %d\n", result);
            return result;
        }

#ifdef USE_HDMICEC
        /* Get physical address in EDID */
        /* Note : Before executing HDMI_CECT10, it is nessesary that HDMI_LinkHDCP is executed. */
        result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                                   FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR,
                                   (FAPEXIOCARG3_T)&PackphysAddr );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("FAPI_HDMI_GetPhysicalAddressInEDID status = %d\n",result);
            return result;
        }
        FAPI_SYS_PRINT_MSG("PackphysAddr = %x\n",PackphysAddr);

        physAddr[0] = (PackphysAddr >> 8) & 0x000000ff;
        physAddr[1] = PackphysAddr & 0x000000ff;

        /* Allocate Logical Address */
        result = HDMI_CEC_AllocateLogicalAddress();
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("HDMI_CEC_AllocateLogicalAddress status = %d\n", result);
        }

        result = FAPEX_HDMICEC_Ioctl( (FAPEXIOCARG1_T)fd_HDMICEC,
                                      FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS,
                                      (FAPEXIOCARG3_T)&MyDevLogicalAddr );

        FAPI_SYS_PRINT_MSG("FAPI_HDMI_GetCecLogicalAddress MyDevLogicalAddr = %d\n", MyDevLogicalAddr);

        /* <Report Physical Address> broadcast */
        FAPI_SYS_PRINT_MSG("<Report Physical Address> broadcast\n");

        (void)HDMI_CEC_SendMessage_ReportPhysicalAddress(1, 0, MyDevLogicalAddr, 0, (uint8_t)physAddr[0], (uint8_t)physAddr[1]);
#endif /* USE_HDMICEC */

        return 1;
    }
#ifdef USE_HDMICEC
    else {
        /* Communicate with CEC message. */
        result = HDMI_CEC_Communication();
        if(result != FAPI_OK) {
            if(result != -1) {
                FAPI_SYS_PRINT_MSG("HDMI_CEC_Communication status = %d\n", result);
            }
            return result;
        }
    }
#endif

    /* check HDCO Link */
    result = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)fd_HDMI,
                               FAPEX_IOCCOM_HDMI_LINK_HDCP,
                               (FAPEXIOCARG3_T)&status );
    if(result == 0) {
        if(status != prv_status) {
            FAPI_SYS_PRINT_MSG("FAPI_HDMI_LinkHDCP = %d\n", status);
            prv_status = status;
        }
    }
    return 0;
}

