/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/pvr_dec.c
**
**              $Id: pvr_dec.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       Decoder interface implementation for PVR playback engine.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 715: Symbol 'anySymbol' not referenced
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <pvr/pvr.h>
#include <val/val.h>
#include <aal/aal.h>

#include "abst_libtsd.h"

//#include "av.h"
//#include "pvr_sys.h"

#include "utils.h"

#include "audio.h"

#include "pvr_m.h"

//******************************************************************************
//******************************************************************************
//** Defines and macros
//******************************************************************************
//******************************************************************************
enum    /* NOTE: The maximum number of audio formats is 8, as it is stored in a
                 three-bit-field. */
{
    SYS_AUDIO_MPEG = 0,
    SYS_AUDIO_AC3,
    SYS_AUDIO_AAC,
    SYS_AUDIO_HEAAC,
    SYS_AUDIO_DDPDCV,

    SYS_AUDIO_NUMS,
    SYS_AUDIO_ALL,              //!< Internally used only, not assigned to
                                //!< format variable!
};
/* Audio channel preference: stored in 1-bit data field */
#define SYS_AUDIOCHAN_LEFT              0x00    //!< Left audio channel.
#define SYS_AUDIOCHAN_RIGHT             0x01    //!< Right audio channel.


/* Stream type as defined in MPEG-2/DVB. */
    #define VID_MPEG1                          1
    #define VID_MPEG2                          2
    #define AUD_MPEG1                          3
    #define AUD_MPEG2                          4
    #define PRIV_PES                           6
    #define AUD_AAC_MPEG2                      15
    #define AUD_AAC_MPEG4                      17
    #define VID_H264                           27
    /* Stream type as defined in ATSC A52. */
    #define AUD_AC3_ATSC_A52                   129
    /* Special custom stream types */
    #define USER_DEF_AC_3                      0x6a06
    #define USER_DEF_EAC_3                     0x7a06
//******************************************************************************
//******************************************************************************
//** Global functions
//******************************************************************************
//******************************************************************************

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************


typedef struct
{
    /* current decoder control state parameters */
    uint8_t                     started;
    uint16_t                    pid;            //!< Video stream PID.
    uint8_t                     trackFormat;    //!< Video stream format,
                                                //!< one of
                                                //!< PVR_TRACK_FORMAT_VIDEO_XXX.

    PVR_DecoderModeT            currMode;
    uint8_t                     currModeIndex;
    uint8_t                     decoderStarted;

} PVRDEC_VideoT;

typedef struct
{
    uint8_t                     trackIndex;     //!< PVR track index.
    uint16_t                    pid;            //!< Audio stream PID.
    uint8_t                     format;         //!< One of SYS_AUDIO_XXX.
    uint8_t                     channel;        //!< One of SYS_AUDIOCHAN_XXX.

    /* current decoder control state parameters */
    uint8_t                     started;
    PVR_DecoderModeT            currMode;
    uint8_t                     currModeIndex;
    uint8_t                     decoderStarted;
    int32_t                     decoderStartStatus;

} PVRDEC_AudioT;

typedef struct
{
    PVRDEC_VideoT  video;
    PVRDEC_AudioT  audio;

} PVRDEC_DataT;

static PVRDEC_DataT* pvrDecPtr = NULL;


extern VAL_DataT*     vidDatPtr;
extern VAL_OpenT*     vidOpenPtr;
extern int32_t fd_TSD[];

//******************************************************************************
//******************************************************************************
//** Local Functions Declaration
//******************************************************************************
//******************************************************************************

static void*    decVideoOpen           (PVR_DecoderTypeT type, void* argPtr);
static int32_t  decVideoClose          (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static int32_t  decVideoStart          (PVR_DecoderTypeT type,
                                        uint8_t trackIndex, uint32_t trackData,
                                        PVR_TrackTypeT trackType,
                                        uint8_t trackFormat, void* argPtr,
                                        uint16_t pid, void* handlePtr);
static int32_t  decVideoStop           (PVR_DecoderTypeT type, void* argPtr,
                                        uint8_t freeze, void* handlePtr);
static int32_t  decVideoSetMode        (PVR_DecoderTypeT type, void* argPtr,
                                        PVR_DecoderModeT mode,
                                        uint8_t modeIndex, void* handlePtr);
static int32_t  decVideoShowIFrame     (PVR_DecoderTypeT type, void* argPtr,
                                        const uint8_t* dataPtr,
                                        uint32_t dataBytes, void* handlePtr);
static int32_t  decVideoBufferReset    (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static uint32_t decVideoBufferGetSize  (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decVideoBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static int32_t  decVideoPrepareFrame   (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static uint8_t  decVideoIsFrameDecoded (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static void     decVideoTask           (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static void decVideoRecover            (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);

static int32_t devVideoGetTsdFd(PVR_DecoderTypeT type, void* argPtr, void* handlePtr);

static int32_t  decVideoStartDecoder   (PVRDEC_VideoT* vidPtr,
                                        uint16_t pid, uint8_t trackFormat,
                                        VAL_PlayModeEnumT playMode);

static void*    decAudioOpen           (PVR_DecoderTypeT type, void* argPtr);
static int32_t  decAudioClose          (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static int32_t  decAudioStart          (PVR_DecoderTypeT type,
                                        uint8_t trackIndex, uint32_t trackData,
                                        PVR_TrackTypeT trackType,
                                        uint8_t trackFormat, void* argPtr,
                                        uint16_t pid, void* handlePtr);
static int32_t  decAudioStop           (PVR_DecoderTypeT type, void* argPtr,
                                        uint8_t freeze, void* handlePtr);
static int32_t  decAudioSetMode        (PVR_DecoderTypeT type,
                                        void* argPtr, PVR_DecoderModeT mode,
                                        uint8_t modeIndex, void* handlePtr);
static int32_t  decAudioBufferReset    (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decAudioBufferGetSize  (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decAudioBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static void     decAudioTask           (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static void decAudioRecover           (PVR_DecoderTypeT type, void* argPtr,
                                         void* handlePtr);
static int32_t  decAudioStartDecoder   (PVRDEC_AudioT* audPtr, uint16_t pid,
                                        uint8_t format, uint8_t channel);

static int32_t  decAvSync              (uint8_t enable, PVR_DecoderTypeT source,
                                        void* argPtr);

/* ------------------------------------------------------------------------- */

//******************************************************************************
//******************************************************************************
//** Local Functions
//******************************************************************************
//******************************************************************************

static void* decVideoOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*      datPtr   = (PVRDEC_DataT*)argPtr;
    PVRDEC_VideoT*     vidPtr   = &(datPtr->video);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoOpen: In\n");

    #if 1
    vidPtr->started        = 0;
    vidPtr->pid            = 0;
    vidPtr->trackFormat    = PVR_TRACK_FORMAT_OTHER;
    vidPtr->currMode       = PVR_DECMODE_NONE;
    vidPtr->currModeIndex  = 0;
    vidPtr->decoderStarted = 0;
    #endif
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoOpen: Out\n");
    return vidPtr;
}

static int32_t decVideoClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             err_code = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoClose: In\n");

    if (vidOpenPtr != NULL){
        VAL_Close(vidOpenPtr);
        vidOpenPtr = NULL;
    }

    /* currently nothing to do... */
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoClose: Out\n");
    return err_code;
}

static int32_t decVideoStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    PVRDEC_VideoT*      vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             retVal   = FAPI_OK;
    VAL_PlayModeEnumT   playMode = VAL_PLAYMODE_NONE;
    PVRSYS_FastModeCfgT fastCfg;
    PVRSYS_SlowModeCfgT slowCfg;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStart: (trackFormat %i pid %i type %i)\n",
                                                        trackFormat, pid, type);

    do
    {
        switch ( vidPtr->currMode )
        {
            case PVR_DECMODE_NONE:
            case PVR_DECMODE_PAUSE:
            default:
                playMode = VAL_PLAYMODE_NONE;
                break;

            case PVR_DECMODE_NORMAL:
                playMode = VAL_PLAYMODE_NORMAL;
                break;

            case PVR_DECMODE_FAST:
                retVal = PVRSYS_GetFastModeConfig(vidPtr->currModeIndex,
                                                  &fastCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = fastCfg.videoMode;
                }
                break;

            case PVR_DECMODE_SLOW:
                retVal = PVRSYS_GetSlowModeConfig(vidPtr->currModeIndex,
                                                  &slowCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = slowCfg.videoMode;
                }
                break;

            case PVR_DECMODE_FEEDDATA:
                playMode = VAL_PLAYMODE_FAST_LIB_IONLY;
                break;

            case PVR_DECMODE_FEEDCTRL:
                playMode = VAL_PLAYMODE_FAST_CONTROLLEDFEED;
                break;
        }

        if ( playMode != VAL_PLAYMODE_NONE )
        {
            retVal = decVideoStartDecoder (vidPtr, pid, trackFormat, playMode);
        }

    } while ( 0 );

    vidPtr->trackFormat = trackFormat;
    vidPtr->pid         = pid;
    vidPtr->started     = 1;

    return retVal;
}

static int32_t decVideoStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    PVRDEC_VideoT* vidPtr   = (PVRDEC_VideoT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStop: In\n");

    /* NOTE: Argument freeze is ignored! */

    //(void)AVINT_VideoStop(FFALSE);
    VAL_Stop(vidOpenPtr, freeze);

    vidPtr->started        = 0;
    vidPtr->pid            = 0;
    vidPtr->trackFormat    = PVR_TRACK_FORMAT_OTHER;
    vidPtr->currMode       = PVR_DECMODE_NONE;
    vidPtr->currModeIndex  = 0;
    vidPtr->decoderStarted = 0;
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStop: Out\n");
    return FAPI_OK;
}

static int32_t decVideoSetMode (PVR_DecoderTypeT type,
                                void* argPtr, PVR_DecoderModeT mode,
                                uint8_t modeIndex, void* handlePtr)
{
    PVRDEC_VideoT*      vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             retVal   = FAPI_OK;
    VAL_PlayModeEnumT   playMode = VAL_PLAYMODE_NONE;
    PVRSYS_FastModeCfgT fastCfg;
    PVRSYS_SlowModeCfgT slowCfg;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoSetMode: mode[%i] index[%i] type[%i]\n", \
    					mode, modeIndex, type);
    if ( vidPtr->started )
    {
        switch ( mode )
        {
            case PVR_DECMODE_NONE:
            default:
                playMode = VAL_PLAYMODE_NONE;
                break;

            case PVR_DECMODE_PAUSE:
                playMode = VAL_PLAYMODE_PAUSE;
                break;

            case PVR_DECMODE_NORMAL:
                playMode = VAL_PLAYMODE_NORMAL;
                break;

            case PVR_DECMODE_FAST:
                retVal = PVRSYS_GetFastModeConfig(modeIndex, &fastCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = fastCfg.videoMode;
                    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoSetMode playMode(%d),iFDTime(%d),sF(%d)\n",\
                                            playMode, fastCfg.iFrameDisplayTime, fastCfg.factor);
                }
                else
                {
                    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoSetMode: get fast mode config failed[%d]\n", retVal);
                }
                break;

            case PVR_DECMODE_SLOW:
                retVal = PVRSYS_GetSlowModeConfig(modeIndex, &slowCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = slowCfg.videoMode;
                }
                break;

            case PVR_DECMODE_FEEDDATA:
                playMode = VAL_PLAYMODE_FAST_LIB_IONLY;
                break;

            case PVR_DECMODE_FEEDCTRL:
                playMode = VAL_PLAYMODE_FAST_CONTROLLEDFEED;
                break;
        }

        if ( playMode != VAL_PLAYMODE_NONE )
        {
            if ( vidPtr->decoderStarted )
            {
                if ( vidOpenPtr!= NULL )
                {
                    retVal = VAL_PlayChangeMode (vidOpenPtr,
                                                 playMode);
                }
            }
            else
            {
                /* NOTE: Starting decoder in pause mode is not supported
                         by video decoder driver and VAL layer! */
                if ( playMode != VAL_PLAYMODE_PAUSE )
                {
                    retVal = decVideoStartDecoder (vidPtr, vidPtr->pid,
                                                   vidPtr->trackFormat,
                                                   playMode);
                }
            }
        }
    }

    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[DEC ] decVideoSetMode: failure: err %i\n", retVal);
    }

    vidPtr->currMode      = mode;
    vidPtr->currModeIndex = modeIndex;

    return retVal;
}

static int32_t decVideoShowIFrame (PVR_DecoderTypeT type,
                                   void* argPtr, const uint8_t* dataPtr,
                                   uint32_t dataBytes, void* handlePtr)
{
    //PVRDEC_VideoT*  vidPtr = (PVRDEC_VideoT*)handlePtr;
    int32_t         retVal = FAPI_OK;

    /* ensure LW-alignment of dataPtr */
    if ( (((uint32_t)dataPtr) & 0x3) != 0 )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[DEC ] decVideoShowIFrame:NON_ALIGNED I-FRAME\n");
    }
    else if ( vidOpenPtr != NULL )
    {
        /*lint -save -e826 */
        #if 1
        retVal = VAL_PlayShowIFrame (vidOpenPtr,
                                    (const uint32_t*)dataPtr, dataBytes);
        #else // Martin test at 20100706

        #if 0
        if (vidOpenPtr->decoderStatus == VAL_STATUS_RUNNING)
        {
            VAL_Stop(vidOpenPtr, 1);
        }
        #endif

        retVal = VAL_IFrameDecode(vidOpenPtr, (const uint32_t*)dataPtr, dataBytes);

        if (retVal == FAPI_OK)
        {
            retVal = VAL_IFrameDisplay(vidOpenPtr);
            if (retVal != FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoShowIFrame: display iframe failed[%d]\n",\
                                    retVal);
            }
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoShowIFrame: decode iframe failed[%d]\n",\
                                  retVal);
        }
        #endif
        /*lint -restore */
    }

    return retVal;
}

static int32_t decVideoBufferReset (PVR_DecoderTypeT type,
                                    void* argPtr,
                                    void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoBufferReset: In\n");

    // TODO: to be implemented

    if (vidOpenPtr != NULL)
    {
        FAPEX_TSD_Ioctl(vidOpenPtr->tsdId, \
                        FAPEX_IOCCOM_TSD_RESET_BUFFER,\
                        (FAPEXIOCARG3_T)NULL);
        FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoBufferReset: Out\n");
    }
    return FAPI_OK;
}

static uint32_t decVideoBufferGetSize (PVR_DecoderTypeT type, void* argPtr,
                                       void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    uint32_t       size   = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoBufferGetSize: In\n");

    if ( vidOpenPtr!= NULL )
    {
        if ( VAL_GetBufferSize(vidOpenPtr, &size) != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoBufferGetSize:get buffer size error\n");
            size  = 0;
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoBufferGetSize: Out:size 0x%08x \n", size);

    return size;
}

static uint32_t decVideoBufferGetLevel (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    uint32_t       level  = 0;
    int32_t        result = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoBufferGetLevel: In\n");

    #ifndef _LINUX_FAPI
    if ( avData->video.valOpenPtr != NULL )
    {
        if ( VAL_GetBufferLevel(avData->video.valOpenPtr, &level) != FAPI_OK )
        {

            level  = 0;
        }
    }
    #else
    if ( vidOpenPtr != NULL )
    {
        result = VAL_GetBufferLevel(vidOpenPtr, &level);

        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoBufferGetLevel: get buffer level error(%d)\n",
                                     result);
            level  = 0x1FFFFFFF;
        }
    }
    else
    {
        level = 0x1FFFFFFF;
    }
    #endif

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoBufferGetLevel: level %i \n", level);

    return level;
}

static int32_t decVideoPrepareFrame (PVR_DecoderTypeT type, void* argPtr,
                                    void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    int32_t        retVal = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decVideoPrepareFrame\n");

    if ( vidOpenPtr != NULL )
    {
        retVal = VAL_PlayPrepareNextFrame(vidOpenPtr);
    }

    return retVal;
}

static uint8_t decVideoIsFrameDecoded (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr       = (PVRDEC_VideoT*)handlePtr;
    uint8_t        frameDecoded = 0;

    if ( vidOpenPtr != NULL )
    {
    	#if 1
        if ( VAL_IsFrameAvailable(vidOpenPtr, &frameDecoded)
             != FAPI_OK )
        {
            frameDecoded  = 0;
        }
        #else
        if (VAL_GetFrameDecodedDirectly(vidOpenPtr, &frameDecoded) != FAPI_OK)
        {
        	frameDecoded = 0;
        }
        #endif
    }

    return frameDecoded;
}

static void decVideoTask (PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    FAPI_SYS_PRINT_DEBUG(3, "[DEC ] decVideoTask\n");

    #ifndef _LINUX_FAPI
    (void)AVINT_VideoTask ();
    #else
    VAL_Task(vidOpenPtr);
    #endif
}

static void decVideoRecover(PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    FAPI_SYS_PRINT_DEBUG(3, "[DEC ] decVideoTask\n");

    #ifndef _LINUX_FAPI
    (void)AVINT_VideoTask ();
    #else
    VAL_Recover(vidOpenPtr);
    #endif
}

static int32_t devVideoGetTsdFd(PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    FAPI_SYS_PRINT_DEBUG(3, "[DEC ] devVideoGetTsdFd\n");

    return VAL_GetIdTsd(vidOpenPtr);
}


static int32_t decVideoStartDecoder (PVRDEC_VideoT* vidPtr, uint16_t pid,
                                     uint8_t trackFormat,
                                     VAL_PlayModeEnumT playMode)
{
    int32_t         retVal = FAPI_OK;
    #ifndef _LINUX_FAPI
    uint8_t         streamType;
    #endif
    VAL_OpenParamsT vidOpenParams;
    int32_t         result;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStartDecoder: (playMode %i)\n", playMode);

    #ifndef _LINUX_FAPI
    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_VIDEO_MPEG1:
            streamType = STRTYPE_VIDEO_MPEG1;
            break;

        case PVR_TRACK_FORMAT_VIDEO_MPEG2:
            streamType = STRTYPE_VIDEO_MPEG2;
            break;

        case PVR_TRACK_FORMAT_VIDEO_H264:
            streamType = STRTYPE_VIDEO_H264;
            break;
    }
    #endif
    if (vidOpenPtr == NULL){
        //Open
        memset(&vidOpenParams, 0, sizeof(VAL_OpenParamsT));

        switch (trackFormat)
        {
            case PVR_TRACK_FORMAT_VIDEO_H264:
                FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoOpen: FAPEX_VIDEC_CODEC_TYPE_H264\n");
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_H264;
                break;

            case PVR_TRACK_FORMAT_VIDEO_MPEG1:
            case PVR_TRACK_FORMAT_VIDEO_MPEG2:
                FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoOpen: FAPEX_VIDEC_CODEC_TYPE_MPEG2\n");
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
                break;

            default:
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_UNKNOWN;
                break;

        }


        vidOpenParams.environment = VAL_ENV_PLAY;
        vidOpenParams.decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
        vidOpenParams.enableAfd = 1;
        vidOpenParams.h264AudType = VAL_H264_AUD_TYPE_AUTO;

        vidOpenPtr = VAL_Open(vidDatPtr, &vidOpenParams, &result);
        if (result == FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStartDecoder: VAL_Open success, videcId[%d], tsdId[%d]\n",\
                    vidOpenPtr->videcId, vidOpenPtr->tsdId);
        }
    }

    {
		/*Martin -- 20100803 -- add this for sync in the case of from Live play to PVR play but without reopen VAL*/
		if (vidOpenPtr->openParams.environment != FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR)
		{
	    	FAPEX_VIDEC_Ioctl(vidOpenPtr->videcId,
                               FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_ENV,
	    					  (FAPEXIOCARG3_T)FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR);

			/*Martin -- 20100803 -- add this for conforming  to VAL parameter check in VAL_PlayChangeMode*/
	    	vidOpenPtr->openParams.environment = FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR;
    	}
        retVal = VAL_Start(vidOpenPtr, pid, VAL_PLAYMODE_NORMAL, VAL_STREAM_SEGMENT_FULL, 0);
    }
    if ( retVal == FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decVideoStartDecoder: VAL_Start success\n");
        vidPtr->decoderStarted = 1;
    }
    else
    {
    	FAPI_SYS_PRINT_MSG("[DEC ] decVideoStartDecoder: VAL_Start failed[%d]\n", retVal);
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

static void* decAudioOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*  datPtr = (PVRDEC_DataT*)argPtr;
    PVRDEC_AudioT* audPtr = &(datPtr->audio);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioOpen: In\n");

    if ( AUDIO_Open() == FAPI_OK )
    {
        audPtr->trackIndex         = 0xFF;
        audPtr->pid                = 0;
        audPtr->format             = SYS_AUDIO_MPEG;

        audPtr->started            = 0;
        audPtr->currMode           = PVR_DECMODE_NONE;
        audPtr->currModeIndex      = 0;
        audPtr->decoderStarted     = 0;
        audPtr->decoderStartStatus = FAPI_OK;
    }
    else
    {
        audPtr = NULL;
    }
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioOpen: Out\n");
    return audPtr;
}

static int32_t decAudioClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioClose: In\n");

    (void)AUDIO_Close();

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioClose: Out\n");
    return FAPI_OK;
}

static int32_t decAudioStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    PVRDEC_AudioT*  audPtr    = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal    = FAPI_OK;
    uint8_t         format;
    uint8_t         channel   = (uint8_t)trackData;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioStart: (trackFormat %i pid %i)\n",
                         trackFormat, pid);

    if ( (channel != SYS_AUDIOCHAN_LEFT) && (channel != SYS_AUDIOCHAN_RIGHT) )
    {
        channel = SYS_AUDIOCHAN_LEFT;
    }

    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_AUDIO_MPEG1:
        case PVR_TRACK_FORMAT_AUDIO_MPEG2:
            format = SYS_AUDIO_MPEG;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AC3:
            format = SYS_AUDIO_AC3;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG2:
            format = SYS_AUDIO_AAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG4:
            format = SYS_AUDIO_HEAAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_DDPDCV:
            format = SYS_AUDIO_DDPDCV;
            break;
    }

    switch ( audPtr->currMode )
    {
        case PVR_DECMODE_NONE:
        case PVR_DECMODE_PAUSE:
        default:
            /* do nothing */
            break;

        case PVR_DECMODE_NORMAL:
        	retVal = decAudioStartDecoder (audPtr, pid, format, channel);
       	#if 0
        case PVR_DECMODE_PAUSE:
            retVal = AUDIO_Resume();
            audPtr->decoderStartStatus = retVal;
            audPtr->decoderStarted = 1;
            break;
		#endif
        case PVR_DECMODE_FAST:
        case PVR_DECMODE_SLOW:
            /* not supported yet */
            break;
    }

    audPtr->trackIndex = trackIndex;
    audPtr->pid        = pid;
    audPtr->format     = format;
    audPtr->channel    = channel;
    audPtr->started    = 1;

    return retVal;
}

static int32_t decAudioStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;
    int32_t     result;
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioStop: In\n");

    if ( audPtr->decoderStarted )
    {
        result = AUDIO_Stop();

        if (result != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioStop: AUDIO_Stop error(%d)\n",result);
        }

        audPtr->decoderStarted     = 0;
        audPtr->decoderStartStatus = FAPI_OK;
    }

    audPtr->started       = 0;
    audPtr->currMode      = PVR_DECMODE_NONE;
    audPtr->currModeIndex = 0;
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioStop: Out\n");
    return FAPI_OK;
}

static int32_t decAudioSetMode (PVR_DecoderTypeT type, void* argPtr,
                                PVR_DecoderModeT mode, uint8_t modeIndex,
                                void* handlePtr)
{
    PVRDEC_AudioT*  audPtr = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal = FAPI_OK;

    printf("[DEC ] decAudioSetMode: (mode %i)\n", mode);

    if ( audPtr->started )
    {
        switch ( mode )
        {
            case PVR_DECMODE_NORMAL:
                switch ( audPtr->currMode )
                {
                    case PVR_DECMODE_PAUSE:

                        if ( audPtr->decoderStarted )
                        {
                            if ( audPtr->decoderStartStatus == FAPI_OK )
                            {
                                #ifndef _LINUX_FAPI
                                (void)AVINT_AudioPause(FFALSE);
                                #else
                                (void)AUDIO_Resume();
                                #endif
                            }
                        }
                        else
                        {
                            retVal = decAudioStartDecoder (audPtr, audPtr->pid,
                                                           audPtr->format,
                                                           audPtr->channel);
                        }
                        break;

                    case PVR_DECMODE_FAST:
                    case PVR_DECMODE_SLOW:
                        /* not supported yet */
                        break;

                    case PVR_DECMODE_NONE:
                    case PVR_DECMODE_NORMAL:
                    default:
                        /* do nothing */
                        break;
                }
                break;

            case PVR_DECMODE_PAUSE:
                switch ( audPtr->currMode )
                {
                    case PVR_DECMODE_NORMAL:

                        if ( (audPtr->decoderStarted)
                          && (audPtr->decoderStartStatus == FAPI_OK) )
                        {
                            #ifndef _LINUX_FAPI
                            (void)AVINT_AudioPause(FTRUE);
                            #else
                            (void)AUDIO_Pause();
                            #endif
                        }
                        break;

                    case PVR_DECMODE_FAST:
                    case PVR_DECMODE_SLOW:
                        /* not supported yet */
                        break;

                    case PVR_DECMODE_NONE:
                    case PVR_DECMODE_PAUSE:
                    default:
                        /* do nothing */
                        break;
                }
                break;

            default:
                break;
        }
    }

    audPtr->currMode      = mode;
    audPtr->currModeIndex = modeIndex;

    return retVal;
}

static int32_t decAudioBufferReset (PVR_DecoderTypeT type, void* argPtr,
                                    void* handlePtr)
{
    int32_t err_code;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioBufferReset\n");

    // TODO: to be implemented
    err_code = AUDIO_ResetBuffer();

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decAudioBufferReset: Out[%d]\n", err_code);
    return err_code;
}

static uint32_t decAudioBufferGetSize (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    uint32_t size = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decAudioBufferGetSize: In \n");

    #ifndef _LINUX_FAPI
    if ( avData->audio.audioHandle != NULL )
    {
        size = AAL_GetBufferSize(avData->audio.audioHandle);
    }
    #else
        size = AUDIO_GetBufferSize();
    #endif

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decAudioBufferGetSize: Out: size %i \n", size);
    return size;
}

static uint32_t decAudioBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr)
{
    uint32_t level = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decAudioBufferGetLevel \n");

    #ifndef _LINUX_FAPI
    if ( avData->audio.audioHandle != NULL )
    {
        level = AAL_GetBufferLevel(avData->audio.audioHandle);
    }
    #else
    {
        level = AUDIO_GetBufferLevel();
        if ( (int32_t)level < 0)
        {
            FAPI_SYS_PRINT_MSG("[DEC ] decAudioBufferGetLevel: get audio buffer level error(%d)\n",\
                                    (int32_t)level);
        }
    }
    #endif

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decAudioBufferGetLevel: level %i \n", level);
    return level;
}

static void decAudioTask (PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(3, "[DEC ] decAudioTask\n");

    if ( (audPtr->decoderStarted)
      && (audPtr->decoderStartStatus == FAPI_OK) )
    {
        #ifndef _LINUX_FAPI
        (void)AVINT_AudioTask ();
        #else

        #endif
    }
}

static void decAudioRecover(PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(3, "[DEC ] decAudioTask\n");

    if ( (audPtr->decoderStarted)
      && (audPtr->decoderStartStatus == FAPI_OK) )
    {
        #ifndef _LINUX_FAPI
        (void)AVINT_AudioTask ();
        #else

        #endif
    }
}

static int32_t decAudioStartDecoder (PVRDEC_AudioT* audPtr, uint16_t pid,
                                     uint8_t format, uint8_t channel)
{
    int32_t retVal = FAPI_OK;

    uint32_t audio_stream_type;

    //retVal = AVINT_AudioStart (pid, format, channel, NULL);

    switch ( format )
    {
        case SYS_AUDIO_MPEG:
            audio_stream_type = AUD_MPEG2;
            break;

        case SYS_AUDIO_AC3:
            audio_stream_type = AUD_AC3_ATSC_A52;
            break;

        case SYS_AUDIO_AAC:
            audio_stream_type = AUD_AAC_MPEG2;
            break;

        case SYS_AUDIO_HEAAC:
            audio_stream_type = AUD_AAC_MPEG4;
            break;

        case SYS_AUDIO_DDPDCV:
            audio_stream_type = USER_DEF_AC_3;
            break;

        default:
            audio_stream_type = AUD_MPEG2;
            //retVal = FAPI_ERR_BAD_PARAMETER;
            break;
    }

    retVal = AUDIO_Start(pid, 0, audio_stream_type);

    audPtr->decoderStarted     = 1;
    audPtr->decoderStartStatus = retVal;

    return retVal;
}

/* ------------------------------------------------------------------------- */

static void* decSpdifOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*  datPtr = (PVRDEC_DataT*)argPtr;
    PVRDEC_AudioT* audPtr = &(datPtr->audio);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decSpdifOpen\n");

    /* NOTE: Opening of audio module is done in decAudioOpen! */

    return audPtr;
}

static int32_t decSpdifClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decSpdifClose\n");

    /* NOTE: Closing of audio module is done in decAudioOpen! */

    return FAPI_OK;
}

static int32_t decSpdifStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    //PVRDEC_AudioT*  audPtr    = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal    = FAPI_OK;
    uint8_t         format;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decSpdifStart (trackFormat %i pid %i)\n",
                         trackFormat, pid);

    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_AUDIO_MPEG1:
        case PVR_TRACK_FORMAT_AUDIO_MPEG2:
            format = SYS_AUDIO_MPEG;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AC3:
            format = SYS_AUDIO_AC3;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG2:
            format = SYS_AUDIO_AAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG4:
            format = SYS_AUDIO_HEAAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_DDPDCV:
            format = SYS_AUDIO_DDPDCV;
            break;
    }

    //retVal = AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_PASSTHROUGH, pid, format);

    return retVal;
}

static int32_t decSpdifStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] decSpdifStop\n");

    #ifndef _LINUX_FAPI
    (void)AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_CURRPCM, 0, SYS_AUDIO_NUMS);
    #endif

    return FAPI_OK;
}

static uint32_t decSpdifBufferGetSize (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    uint32_t size = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decSpdifBufferGetSize \n");

    #ifndef _LINUX_FAPI
    if ( avData->audio.audioHandle != NULL )
    {
        size = AAL_GetSPDIFBufferSize(avData->audio.audioHandle);
    }
    #endif

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decSpdifBufferGetSize: size %i \n", size);
    return size;
}

static uint32_t decSpdifBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr)
{
    uint32_t level = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decSpdifBufferGetLevel \n");

    #ifndef _LINUX_FAPI
    if ( avData->audio.audioHandle != NULL )
    {
        level = AAL_GetSPDIFBufferLevel(avData->audio.audioHandle);
    }
    #endif

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decSpdifBufferGetLevel: level %i \n", level);
    return level;
}

/* ------------------------------------------------------------------------- */

static int32_t decAvSync (uint8_t enable, PVR_DecoderTypeT source, void* argPtr)
{
    //PVRDEC_DataT* datPtr = (PVRDEC_DataT*)argPtr;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC ] decAvSync enable %i source %i\n",
                         enable, source);

    // FIXME: needed?
    #ifndef _LINUX_FAPI

    if ( !enable )
    {
        (void)FAPI_SYNC_SetStcRegulation(0);
    }
    #else
    if (enable)
    {

    }
    else
    {

    }
    #endif

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */

int32_t PVRDEC_Init (void)
{
    //DBG_Assert(pvrDecPtr == NULL);

    pvrDecPtr = (PVRDEC_DataT*)malloc(sizeof(PVRDEC_DataT));
    if ( pvrDecPtr == NULL ) return -1;

    memset (pvrDecPtr, 0, sizeof(PVRDEC_DataT));

    return FAPI_OK;
}

void PVRDEC_RegisterFunctions (PVR_TargetControlT* ctrlPtr)
{
    PVR_DecoderControlT* decoderPtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] PVRDEC_RegisterFunctions: In\n")
    DBG_Assert(pvrDecPtr != NULL);

    memset(ctrlPtr, 0, sizeof(PVR_TargetControlT));

    ctrlPtr->argPtr             = pvrDecPtr;
    ctrlPtr->avSync             = decAvSync;

    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_VIDEO);

    decoderPtr->open            = decVideoOpen;
    decoderPtr->close           = decVideoClose;
    decoderPtr->start           = decVideoStart;
    decoderPtr->stop            = decVideoStop;
    decoderPtr->setMode         = decVideoSetMode;
    decoderPtr->feedData        = decVideoShowIFrame;
    decoderPtr->resetBuffer     = decVideoBufferReset;
    decoderPtr->getBufferSize   = decVideoBufferGetSize;
    decoderPtr->getBufferLevel  = decVideoBufferGetLevel;
    decoderPtr->prepareData     = decVideoPrepareFrame;
    decoderPtr->isDataDecoded   = decVideoIsFrameDecoded;
    decoderPtr->task            = decVideoTask;
    decoderPtr->recover         = decVideoRecover;
    decoderPtr->getTsdFd           = devVideoGetTsdFd;

    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_AUDIO);

    decoderPtr->open            = decAudioOpen;
    decoderPtr->close           = decAudioClose;
    decoderPtr->start           = decAudioStart;
    decoderPtr->stop            = decAudioStop;
    decoderPtr->setMode         = decAudioSetMode;
    decoderPtr->resetBuffer     = decAudioBufferReset;
    decoderPtr->getBufferSize   = decAudioBufferGetSize;
    decoderPtr->getBufferLevel  = decAudioBufferGetLevel;
    decoderPtr->task            = decAudioTask;
    decoderPtr->recover         = decAudioRecover;
    decoderPtr->getTsdFd        = NULL;//AUDIO_GetIdAudec;

    #if 0 /*Ignore the 2nd audio*/
    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_AUDIO_2ND);

    decoderPtr->open            = decSpdifOpen;
    decoderPtr->close           = decSpdifClose;
    decoderPtr->start           = decSpdifStart;
    decoderPtr->stop            = decSpdifStop;
    decoderPtr->getBufferSize   = decSpdifBufferGetSize;
    decoderPtr->getBufferLevel  = decSpdifBufferGetLevel;
    #endif
    FAPI_SYS_PRINT_DEBUG(1, "[DEC ] PVRDEC_RegisterFunctions: Out\n")
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
