/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/pvr_list.h
**
**              $Id: pvr_list.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Management of PVR recordings.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _PVR_LIST_H_
#define _PVR_LIST_H_

#include "sys_api.h"
#include "sys_priv.h"

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t PVR_LIST_Init             (void);
int32_t PVR_LIST_AddRecordItem    (const PVR_LIST_ITEM_S* pItem, uint8_t recIdx,
                                   const char_t* dirPath, PVRLST_IDX* idx);
int32_t PVR_LIST_SetItem          (const PVR_LIST_ITEM_S* pItem, PVRLST_IDX idx,
                                   fbool_t endOfRecording);
int32_t PVR_LIST_RemoveItemByIndex(PVRLST_IDX idx);
int32_t PVR_LIST_GetFilename      (PVRLST_IDX idx, char_t* filename,
                                   uint16_t maxLength);
int32_t PVR_LIST_GetIndexByPath   (const char_t* path, PVRLST_IDX* idx);

int32_t  PVR_LIST_SyncDevice      (SYS_DEVICE_E device, const char_t* relPath);
int32_t  PVR_LIST_DeviceRemoved   (SYS_DEVICE_E device);

#ifdef APPL_EDITMARKER_ENABLED

int32_t PVR_LIST_GetMarker        (int16_t lstPos,
                                   PVR_BLOCK_MARKER_ITEM_S* pMarker);
int32_t PVR_LIST_GetMarkerByIdx   (PVRLST_IDX idx,
                                   PVR_BLOCK_MARKER_ITEM_S* pMarker);
int32_t PVR_LIST_SetMarker        (int16_t lstPos,
                                   PVR_BLOCK_MARKER_ITEM_S* pMarker);

#endif /* APPL_EDITMARKER_ENABLED */

#ifdef __cplusplus
}
#endif

#endif /* _PVR_LIST_H_ */

/******************************************************************************/
/* END OF FILE */
