/*!
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo_fs/src/tsd.h
** 
** \version     $Id$
**
** \brief       TSD Driver.
**
** This file contains the declaration of the TSD driver API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#ifndef TSD_H
    #define TSD_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************
    #define DRV_ID_TSD0         (FAPEXDEV_TSD0_CMN_MINOR)
    #define DRV_ID_TSD1         (FAPEXDEV_TSD1_CMN_MINOR)
    #define DRV_ID_TSD2         (FAPEXDEV_TSD2_CMN_MINOR)
    #define DRV_ID_TSD3         (FAPEXDEV_TSD3_CMN_MINOR)

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************
    typedef struct {
        FAPEX_SYS_FFILE     *filePtr;
        int32_t             syncOffset;
        uint32_t            packetSize;

        uint8_t             *chunkBuffer;           //!< double buffer
        uint32_t            chunkBufferSize;
        uint32_t            chunkWriteCount;
        uint32_t            chunkReadCount;
        int32_t             lastWriteSize;

        void                (*inputFunc)(void *dst, const void *src, int32_t n);

        uint8_t             *inputPortBase;
        uint32_t            inputPortOffset;

        int32_t             audioBuffSize;
        int32_t             videoBuffSize;

    } TSD_InputFileInfo;

    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t  TSD_Open(void);
    int32_t  TSD_Close(void);
    int32_t  TSD_Start(FAPEX_TSD_InputParamStrT *inputParams, const char_t *fileName,
                       int32_t startOffset);
    int32_t  TSD_Stop(void);
    int32_t  TSD_Pause(void);
    int32_t  TSD_Restart(void);
    uint64_t TSD_GetFileReadPosition(void);
    uint64_t TSD_GetTimeStamp(void);

    #ifdef __cplusplus
    }
    #endif
#endif
