/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo_fs/src/ttx.c
** 
** \version     $Id$
**
** \brief       Teletext VBI re-insertion Decoder.
**
** This file contains the implementation of the Teletext API. It
** initializes and configures the teletext decoder(s) including all buffers.
** Furthermore it provides a simple API to start/stop and run the teletext
** decoder.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
#undef ENABLE_TTX_DATA_READ

#include <fapex/abst_types.h>
#include <fapex/abst_libttx.h>
#include <fapex/abst_libtsd.h>
#include <fapex/sys_config_mb86h61_devkit.h>
#include "ttx.h"

#ifdef ENABLE_TTX_DATA_READ
#include "demo_app.h"
#endif

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define TTX_TSD_CHANNEL     6
#define DRV_ID_TSD0_TTX_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + TTX_TSD_CHANNEL)
#define DRV_ID_TTX_0        (FAPEXDEV_TTX_MINOR_FIRST)

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_TTX_READ_THREAD_STACKSIZE   1024
#define APP_TTX_READ_THREAD_PRIORITY    FAPEX_SYS_PRIORITY_MEDIUM
#define APP_TTX_READ_THREAD_NAME        "app:ttx_read"

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static int32_t fd_TSD0_TTX = -1;
static int32_t fd_TTXVBI   = -1;

#ifdef ENABLE_TTX_DATA_READ
static FAPEX_SYS_ThreadT    appTtxReadThreadHandle = NULL;
static APP_ThreadStatus     appTtxReadThreadStatus;
static volatile int32_t     appTtxReadEnable       = -1;
#endif

//***************************************************************************
//***************************************************************************
//** External Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
extern int32_t fd_TSD[];

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

#ifdef ENABLE_TTX_DATA_READ
/*!
********************************************************************************
** \brief TS file read thread
********************************************************************************/
static void appTtxReadThreadFunction(void *optArg)
{
    FAPEX_TSD_ReadInfoStrT  read_info;
    FAPEX_SYS_FFILE         *fp;
    uint8_t  *buff_map, *read_ptr;
    uint32_t end_offset, tmp_size;
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "appTtxReadThreadFunction() start\n");

    APP_ThreadStatus_SetValue( &appTtxReadThreadStatus,
                               APP_THREAD_STATUS_OPENED );

    /* get TTX PES buffer mapping address */
    buff_map = (uint8_t*)FAPEX_TSD_Mmap( NULL, FAPEX_SYS_MMAP_BM_TELETEXT_BYTES,
                                         (PROT_READ | PROT_WRITE), MAP_SHARED,
                                         fd_TSD0_TTX, 0 );
    if(buff_map == (void *)-1) {
        result = -1;
        FAPI_SYS_PRINT_ERR("[TTX] Error: TTX PES buffer mmap failed %d.\n", result);

        APP_ThreadStatus_SetValue( &appTtxReadThreadStatus,
                                   APP_THREAD_STATUS_DISABLE );
        return;
    }

    fp = FAPEX_SYS_FOpen("ttx.bin", "wb");
    if(fp == NULL)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[ttx.c] open file failed\n" );
        goto appTtxReadThreadFunction_end;
    }

    while(appTtxReadEnable >= 0) {
        if(appTtxReadEnable == 0) {
            APP_ThreadStatus_SetValue(
                &appTtxReadThreadStatus,
                (APP_THREAD_STATUS_READY | APP_THREAD_STATUS_SUSPENDED) );

            // wait start trigger
            FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

            if(appTtxReadEnable > 0) {
                APP_ThreadStatus_SetValue( &appTtxReadThreadStatus,
                                           APP_THREAD_STATUS_STARTED );
            }
            else {
                APP_ThreadStatus_SetValue( &appTtxReadThreadStatus,
                                           APP_THREAD_STATUS_READY );
                continue;
            }
        }

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                                  FAPEX_IOCCOM_TSD_GET_BUFFER_READINFO,
                                  (FAPEXIOCARG3_T)&read_info );
        if((result == 0) && (read_info.fillSize > 0)) {
            FAPI_SYS_PRINT_DEBUG( 3, "%06x %04x %x\n",
                                     read_info.readOffset,
                                     read_info.bufferStatus,
                                     read_info.fillSize );

            read_ptr   = buff_map + read_info.readOffset;
            end_offset = read_info.readOffset + read_info.fillSize;
            if(end_offset < FAPEX_SYS_MMAP_BM_TELETEXT_BYTES) {
                FAPEX_SYS_FWrite( read_ptr, sizeof(uint8_t),
                                  read_info.fillSize, fp );
            }
            else {
                tmp_size = FAPEX_SYS_MMAP_BM_TELETEXT_BYTES - read_info.readOffset;
                FAPEX_SYS_FWrite( read_ptr, sizeof(uint8_t),
                                  tmp_size, fp );
                tmp_size = end_offset - FAPEX_SYS_MMAP_BM_TELETEXT_BYTES;
                FAPEX_SYS_FWrite( buff_map, sizeof(uint8_t),
                                  tmp_size, fp );
            }
            result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                                      FAPEX_IOCCOM_TSD_SET_BUFFER_READSIZE,
                                      (FAPEXIOCARG3_T)read_info.fillSize );
        }
        else {
            FAPI_SYS_SLEEP(10);
        }
    }
    FAPEX_SYS_FClose(fp);

appTtxReadThreadFunction_end:
    FAPI_SYS_PRINT_DEBUG(1, "appTtxReadThreadFunction() end\n");

    FAPEX_TSD_Munmap(buff_map, FAPEX_SYS_MMAP_BM_TELETEXT_BYTES);

    APP_ThreadStatus_SetValue( &appTtxReadThreadStatus,
                               APP_THREAD_STATUS_DISABLE );
}
#endif

/*!
*******************************************************************************
** Stop TSD0/BM0 channel6.
*******************************************************************************
*/
static int32_t ttxTsdStop(void)
{
    int32_t  result;

    /* stop TSD channel */
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                              FAPEX_IOCCOM_STOP,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Stop TTX TSD channel (code %d).\n", result);
    }
#ifdef ENABLE_TTX_DATA_READ
    else {
        appTtxReadEnable = 0;
    }
#endif
    return result;
}

/*!
*******************************************************************************
** Open TSD0/BM0 channel6.
*******************************************************************************
*/
static int32_t ttxTsdOpen(void)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    FAPEX_TSD_PesParamStrT       pesParam;
    int32_t  result;

    pidChParam.channelNo = TTX_TSD_CHANNEL;
    pidChParam.sub_id    = DRV_ID_TSD0_TTX_CH;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                              FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open TTX TSD channel. %d\n", result);
        return result;
    }
    fd_TSD0_TTX = pidChParam.sub_id;

    /* set Teletext PES filter */
    pesParam.pid   = 0x1FFF;  /* dummy */
    pesParam.flags = FAPEX_TSD_FLTFLAGS_RM_TS_HEADER |
                     FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP |
                     FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
    pesParam.bufferSize       = FAPEX_SYS_MMAP_BM_TELETEXT_BYTES;
    pesParam.bufferThresholdH = pesParam.bufferSize;
    pesParam.bufferThresholdL = 0;
    pesParam.streamType       = FAPEX_TSD_STREAMTYPE_TELETEXT;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                              FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                              (FAPEXIOCARG3_T)&pesParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set TTX PES filter (code %d).\n", result);
        return result;
    }

#ifdef ENABLE_TTX_DATA_READ
    appTtxReadEnable = 0;

    // Open TTX read thread
    APP_ThreadStatus_Init( &appTtxReadThreadStatus );
    appTtxReadThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_TTX_READ_THREAD_STACKSIZE,
            APP_TTX_READ_THREAD_PRIORITY,
            appTtxReadThreadFunction,
            0, 0, APP_TTX_READ_THREAD_NAME );
    if( appTtxReadThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appTtxReadThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[ttx.c] creating thread \"%s\" failed\n",
                            APP_TTX_READ_THREAD_NAME );
        return -1;
    }
#endif

    return 0;
}

/*
*******************************************************************************
** Close TSD0/BM0 channel6.
*******************************************************************************
*/
static int32_t ttxTsdClose(void)
{
    int32_t  result = 0;

    if(fd_TSD0_TTX >= 0) {
#ifdef ENABLE_TTX_DATA_READ
        result = ttxTsdStop();
        if(result < 0) {
            return result;
        }

        appTtxReadEnable = -1;

        /* wait thread end */
        while(1) {
            APP_ThreadStatus_Lock( &appTtxReadThreadStatus );
            if(appTtxReadThreadStatus.value == APP_THREAD_STATUS_DISABLE) {
                APP_ThreadStatus_Unlock( &appTtxReadThreadStatus );
                break;
            }
            else if(appTtxReadThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appTtxReadThreadStatus );

                FAPEX_SYS_WakeupThread( appTtxReadThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTtxReadThreadStatus );
            }
            FAPI_SYS_SLEEP(100);
        }
        APP_ThreadStatus_Exit( &appTtxReadThreadStatus );
#endif

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                                  FAPEX_IOCCOM_CLOSE_SUBID,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close TTX TSD channel. %d\n", result);
        }
        fd_TSD0_TTX = -1;
    }
    return result;
}

/*
*******************************************************************************
** Open Teletext driver.
*******************************************************************************
*/
static int32_t ttxOpen(void)
{
    FAPEX_TTX_OpenParamStrT  ttxOpenParam;

    ttxOpenParam.inputId = DRV_ID_TSD0_TTX_CH;
    fd_TTXVBI = FAPEX_TTX_Open( DRV_ID_TTX_0,
                                O_RDWR,
                                (FAPEXOPENARG3_T)&ttxOpenParam );
    if(fd_TTXVBI < 0) {
        FAPI_SYS_PRINT_ERR("ERROR: Cannot open TTX driver (code %d).\n", fd_TTXVBI);
        return fd_TTXVBI;
    }

    return 0;
}

/*
*******************************************************************************
** Close Teletext driver.
*******************************************************************************
*/
static int32_t ttxClose(void)
{
    int32_t result = 0;

    if(fd_TTXVBI >= 0) {
        result = FAPEX_TTX_Close( fd_TTXVBI );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close TTX driver (code %d).\n", result);
            return result;
        }
        fd_TTXVBI = -1;
    }
    return result;
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** Opens teletext re-insertion.
**
** This function opens the teletext handle.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TTXVBI_Open(void)
{
    int32_t  result;

    result = ttxTsdOpen();

    if(result == 0) {
        result = ttxOpen();
    }
    return result;
}

/*!
*******************************************************************************
**
** Closes teletext re-insertion.
**
** This function closes the teletext handle.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TTXVBI_Close(void)
{
    int32_t  result;

    result = ttxClose();

    if(result == 0) {
        result = ttxTsdClose();
    }
    return result;
}

/*!
*******************************************************************************
** Starts Teletext re-insertion.
**
** This function starts the teletext re-insertion process for the given PID.
**
** \param pid The teletext PID to decode.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TTXVBI_Start(uint32_t pid)
{
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, TTXVBI_Start() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    /* set TTX PES PID */
    pidParam.pid   = pid;
    pidParam.flags = 0;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                              FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                              (FAPEXIOCARG3_T)&pidParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set TTX Pid (code %d).\n", result);
        return result;
    }

    /* start TTX driver */
    result = FAPEX_TTX_Ioctl( (FAPEXIOCARG1_T)fd_TTXVBI,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Start TTX driver (code %d).\n", result);
        return result;
    }

    /* start TSD channel */
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_TTX,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Start TTX TSD channel (code %d).\n", result);
    }
#ifdef ENABLE_TTX_DATA_READ
    else {
        appTtxReadEnable = 1;
        FAPEX_SYS_WakeupThread( appTtxReadThreadHandle );
    }
#endif
    return result;
}

/*!
*******************************************************************************
** Stops AUDIO Processor.
**
** This function stops the audio decoding process.
*******************************************************************************
*/
int32_t TTXVBI_Stop(void)
{
    int32_t result = 0;

    /* stop TTX Driver */
    if(fd_TTXVBI >= 0) {
        result = FAPEX_TTX_Ioctl( (FAPEXIOCARG1_T)fd_TTXVBI,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Stop TTX driver (code %d).\n", result);
            return result;
        }
    }

    /* stop TSD channel */
    if(fd_TSD0_TTX >= 0) {
        result = ttxTsdStop();
    }
    return result;
}

