/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/inc/utils.h
**
**              $Id: utils.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Public common utility functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _UTILS_H_
#define _UTILS_H_
/*----------------------------------------------------------------------------*/
/* Debug output handling                                                      */
/*----------------------------------------------------------------------------*/

#include "fapex/abst_types.h"

#if ( defined APPL_DEBUG_MODE )
    #define DBG_Assert(a)                                            \
        /*lint -save -e506 -e774 */                                  \
        do                                                           \
        {                                                            \
            if ( !(a) )                                              \
            {                                                        \
                FAPI_SYS_PRINT_MSG("ASSERTION FAILED at %s, l.%i\n", \
                                  __FILE__ ,__LINE__ );              \
                FAPEX_SYS_ExitApplication(-1);                                 \
            }                                                        \
        } while (0)                                                  \
        /*lint -restore */

#else

    #define DBG_Assert(a)

#endif /* defined APPL_DEBUG_MODE */


/**
 *
 * \note DBG_ is replaced by FAPI_SYS_PRINT_DEBUG
 *
 */


#ifdef __cplusplus
extern "C" {
#endif

/* utiliy functions */
uint32_t   UTILS_strncpy               (char_t* target,
                                        const char_t* source,
                                        uint32_t size);
uint32_t   UTILS_strlen                ( const char_t* str );
void       UTILS_remWhiteCharAtEnd     (char_t* string,
                                        uint32_t size);

int32_t    UTILS_strcmp                (const char_t* s1,
                                        const char_t* s2);
int32_t    UTILS_strncmpi              (const char_t* s1,
                                        const char_t* s2,
                                        int32_t numChars);
uint64_t   UTILS_GetTimerStampInterval (uint64_t timeStart,
                                        uint64_t timeEnd);
uint64_t   UTILS_GetTimerStamp         (void);
void       UTILS_MemDump               (const char_t* str,
                                        const uint8_t* pBuf,
                                        uint32_t len);
void*      UTILS_Malloc                (uint32_t size);
void*      UTILS_Zalloc                (uint32_t size);
void       UTILS_Free                  (void *p);


#ifdef __cplusplus
}
#endif

#endif /* _UTILS_H_ */
/******************************************************************************/
/* END OF FILE */
