/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_fe_demo/src/main.c
**
** \version     $Id$
**
** \brief       Demo Application for the FE subsystem
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2009-2010 by Fujitsu Semiconductor WSC
**
********************************************************************************
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libgpio.h>
#include "fapex/sys_config.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */


#include "fe/fe.h"
#include "fe/fe_priv.h"

#define APP_VERSION      "1.0.0"

#define APP_SHELL_THREAD_STACKSIZE         (4 * 1024)
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"

#define APP_START_THREAD_STACKSIZE         8*1024
#define APP_START_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGHEST
#define APP_START_THREAD_NAME              "app:sfal_demo"

#define APP_CommandStringSize 64


static FAPEX_SYS_ThreadT  appStartThreadHandle;
static FE_SYSTEM_PARAMS_S FE_SystemParameter;
static feHandleT*  handleT;
static FE_TUNE_PARAMS_S   FE_TuneParameter;
static FAPEX_SYS_ThreadT  appShellThreadHandle;
static FAPI_SYS_HandleT handle;
static int8_t   count = 5;


static void appStartThreadFunction(void* optArg);
static void appShellThreadFunction(void* optArg);
static int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description );
FE_STATUS_E fe_lock(void);
int32_t getPSISection(void);
int32_t testMutiFiltering(void);
        
static void ReTune(void);
static uint32_t APP_ReTuneCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetFreqCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetSymbleRateCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetModeCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetIQCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetPolCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetLNAHighFqCommand( int32_t argc, char_t* argv[] );
static uint32_t APP_SetLNALowFqCommand( int32_t argc, char_t* argv[] );

//***************************************************************************
//***************************************************************************
//** Main Program
//***************************************************************************
//***************************************************************************
int32_t main( int32_t argc, char_t* argv[] )
{
    int32_t  result = 0;

    FAPI_SYS_PRINT_MSG("\n%lld, main() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPI_SYS_PRINT_MSG("\033[2J\n\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# SFAL demo application (FAPex version)\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# program version: %s\n", APP_VERSION);
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2007-2011 by Fujitsu Semiconductor Europe GmbH\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2008-2011 by Fujitsu Semiconductor Limited\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("\n");

    appShellThreadHandle = FAPEX_SYS_CreateThread(  NULL,
                                                    APP_SHELL_THREAD_STACKSIZE,
                                                    APP_SHELL_THREAD_PRIORITY,
                                                    appShellThreadFunction,
                                                    0, 0, APP_SHELL_THREAD_NAME );
    if(appShellThreadHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG( "Error: FAPEX_SYS_CreateThread (shell)\n" );
        exit(1);
    }

    FAPI_SYS_PRINT_MSG("\n%lld, thread start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_StopPrintThread();
    FAPEX_SYS_StartMultitasking();

    FAPEX_SYS_WaitEndThread( appShellThreadHandle );

    exit( result );

}

FE_STATUS_E fe_lock(void)
{
    FE_OpenParamsT  FE_OpenParameter;
    FE_STATUS_E   lockStatus = FE_STATUS_UNKNOWN;
    int32_t  errorCode = 0;

    FE_Init();

    FE_OpenParameter.slotNum = 0;
    FE_OpenParameter.version = FE_VERSION;

    if(FE_DetectSlot(FE_OpenParameter.slotNum) != FAPI_OK)
    {
        handle = NULL;
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE: Not detected!\n");

        return FE_STATUS_UNKNOWN;
    }
    else
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE: detected!\n");

    handle = FE_Open(&FE_OpenParameter,&errorCode);
    if(handle == NULL)
    {
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Open Failed! errorCode: %d\n",errorCode);
        
        return FE_STATUS_UNKNOWN;
    }
    else
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Open Successful!\n");

    handleT = (feHandleT*)handle;
    if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
    {
        FE_SystemParameter.cable.autoFEC           = 1;
        FE_SystemParameter.cable.autoModulation    = 1;
        FE_SystemParameter.cable.defaultIQ         = FE_IQ_AUTO;
        FE_SystemParameter.cable.custom            = NULL;

        FE_TuneParameter.cable.frequency = 336000000;
        FE_TuneParameter.cable.iq = FE_IQ_NORMAL;
        FE_TuneParameter.cable.modulation = FE_MOD_QAM_64;
        FE_TuneParameter.cable.symbolRate = 6875;
    }
    else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
    {
        errorCode = FE_GetSystemParams(handle, &FE_SystemParameter);
        if ( errorCode != FAPI_OK )
            FAPI_SYS_PRINT_MSG("FE_GetSystemParams failed~~!\n");

        //FE_SystemParameter.sat.classic = 1;
        //FE_SystemParameter.sat.posBy22kHz = 0;
        //systemParams.sat.toneburst = 1;
        //FE_SystemParameter.sat.diseqc = 0;
        //FE_SystemParameter.sat.diseqc10Rep = 0;
        //FE_SystemParameter.sat.diseqc11Rep = 1;
        //systemParams.sat.ifFrequency = 1284;
        FE_SystemParameter.sat.lowFrequency = 5150000;
        FE_SystemParameter.sat.highFrequency = 5150000;
        //FE_SystemParameter.sat.autoFEC = 0;
        //FE_SystemParameter.sat.defaultIQ = FE_IQ_AUTO;
        //FE_SystemParameter.sat.custom = NULL;
        //FE_SystemParameter.sat.motorAddress = 0x31;

        errorCode = FE_SetSystemParams(handle, &FE_SystemParameter);
        if (errorCode != FAPI_OK )
        {
            FAPI_SYS_PRINT_MSG("FE_SetSystemParams failed~~!\n");
            exit(1);
        }

        FE_TuneParameter.sat.frequency = 4150000;
        //FE_TuneParameter.sat.ifFrequency = 1284;
        FE_TuneParameter.sat.symbolRate = 27500;
        //FE_TuneParameter.sat.fec = FE_FEC_3_4;
        FE_TuneParameter.sat.pol = FE_POL_HOR;
        //FE_TuneParameter.sat.satID = 0;
        //FE_TuneParameter.sat.orbitalPosition = 192;
        //FE_TuneParameter.sat.westEastFlag = FE_WE_FLAG_EAST;
        //FE_TuneParameter.sat.iq = FE_IQ_AUTO;
        FE_TuneParameter.sat.modulation = FE_MOD_QPSK;
        //FE_TuneParameter.sat.rollOff = FE_ROLL_OFF_AUTO;
    }

    errorCode = FE_Tune(handle,&FE_TuneParameter);
    if(errorCode == 0)
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Tune Successful!\n");
    else
        FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Tune Failed! errorCode: %d\n",errorCode);

    while(count--)
    {
        FE_Wait(200);
        errorCode = FE_GetStatus(handle,&lockStatus);
        if(count > 0)
        {
            FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Get Status errorCode: %d\n",errorCode);
            if(lockStatus == FE_STATUS_LOCKED)
                FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Locked!\n");
            else
                FAPI_SYS_PRINT_MSG("[FE_LOCK] FE Unlock! LockStatus: %d\n",lockStatus);
        }
    }

    return lockStatus; 
}


static void appStartThreadFunction(void* optArg)
{
    FAPI_SYS_PRINT_MSG("\n%lld, appStartThreadFunction() start\n",
                        FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    if(fe_lock() != FE_STATUS_LOCKED)
        exit(1);

}


static int32_t appRegisterCommand( char_t*  command,
                                    uint32_t (*function)(int32_t, char_t**),
                                    char_t*  brief, char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler, command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}

static uint32_t APP_HaltCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
#ifdef USE_WATCHDOG
        {
            int32_t  result;

            result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                         FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP,
                                         (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_MSG("Watchdon Stop Error %d\n", result);
            }
        }
#endif
        exit( 1 );
    }
    return( 0 );
}

static void ReTune(void)
{
    uint32_t errorCode = 0;
    errorCode = FE_Tune(handle,&FE_TuneParameter);
    if(errorCode == 0)
    {
        FAPI_SYS_PRINT_MSG("\nRetune Successful!\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("\nRetune Failed! errorCode: %d\n",errorCode);
    }
    count = 0;
}

static uint32_t APP_ReTuneCommand( int32_t argc, char_t* argv[] )
{
    FAPI_SYS_PRINT_MSG("argc: %d\n",argc);
    if( argc == 1 ) {
        ReTune();
    }
    return 1;
}

static uint32_t APP_SetFreqCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            sscanf(argv[1], "%d", &FE_TuneParameter.cable.frequency);
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", &FE_TuneParameter.sat.frequency);
        }
        ReTune();
    }
    return 1;
}

static uint32_t APP_SetSymbleRateCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.cable.symbolRate);
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.sat.symbolRate);
        }
        ReTune();
    }
    return 1;
}

static uint32_t APP_SetModeCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.cable.modulation);
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.sat.modulation);
        }
        ReTune();
    }
    return 1;
}

static uint32_t APP_SetIQCommand( int32_t argc, char_t* argv[] )
{
    FAPI_SYS_PRINT_MSG("argc: %d\n",argc);
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.cable.iq);
            ReTune();
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            FAPI_SYS_PRINT_MSG("Not Support!!!\n");
        }
    }
    return 1;
}


static uint32_t APP_SetPolCommand( int32_t argc, char_t* argv[] )
{
    FAPI_SYS_PRINT_MSG("argc: %d\n",argc);
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            FAPI_SYS_PRINT_MSG("Not Support!!!\n");
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", (int *)&FE_TuneParameter.sat.pol);
            ReTune();
        }
    }
    return 1;
}

static uint32_t APP_SetLNAHighFqCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            FAPI_SYS_PRINT_MSG("Not Support!!!\n");
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", &FE_SystemParameter.sat.highFrequency);
            ReTune();
        }
    }
    return 1;
}

static uint32_t APP_SetLNALowFqCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 2 ) {
        if(handleT->systemType == FE_SYSTEM_TYPE_CABLE)
        {
            FAPI_SYS_PRINT_MSG("Not Support!!!\n");
        }
        else if(handleT->systemType == FE_SYSTEM_TYPE_SATELLITE)
        {
            sscanf(argv[1], "%d", &FE_SystemParameter.sat.lowFrequency);
            ReTune();
        }
    }
    return 1;
}

static uint32_t APP_TestPSICommand( int32_t argc, char_t* argv[] )
{
    int32_t retVal = 0;

    if( argc == 1 ) {
        retVal = getPSISection();
        if(retVal != FAPI_OK)
            FAPI_SYS_PRINT_MSG("[main] getSection Failed! retVal: %d\n",retVal);
    }

    return 1;
}

static uint32_t APP_TestMutiCommand( int32_t argc, char_t* argv[] )
{
    int32_t retVal = 0;

    if( argc == 1 ) {
        retVal = testMutiFiltering();
        if(retVal != FAPI_OK)
            FAPI_SYS_PRINT_MSG("[main] getSection Failed! retVal: %d\n",retVal);
    }

    return 1;
}

static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT, 500, NULL );
    if( !appShellInputHandler ) {
        exit(1);
    }
#else
    char_t    args_buff[APP_CommandStringSize];
    int32_t   i, c;

    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n"   \
        "as optional argument to ?\n"                                           \
        "Example: ?       displays all available commands, one per line\n"     \
        "Example: ? !     displays a more detailed help for the ! command\n" \
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n" \
    );
#endif

    (void)appRegisterCommand(
        "exit",
        APP_HaltCommand,
        "Exit application.",
        "Usage: exit\n"
    );

    (void)appRegisterCommand(
        "Retune",
        APP_ReTuneCommand,
        "Retune frontend RF.",
        "Usage: Retune\n"
    );

    (void)appRegisterCommand(
        "SetRF",
        APP_SetFreqCommand,
        "Set frontend RF.",
        "Usage: SetRF [freq]\n"
    );

    (void)appRegisterCommand(
        "SetRate",
        APP_SetSymbleRateCommand,
        "Set frontend SymbleRate.",
        "Usage: SetRate [SymbleRate]\n"
    );

    (void)appRegisterCommand(
        "SetMode",
        APP_SetModeCommand,
        "Set frontend Mode.",
        "Usage: SetMode [Mode]\n0-None,1-QPSK,2-QPSK_S2,3-8PSK,4-QAM16,5-QAM32,6-QAM64,7-QAM128,8-QAM256,9-AUTO\n"
    );

    (void)appRegisterCommand(
        "SetIQ",
        APP_SetIQCommand,
        "Set frontend IQ.",
        "Usage: SetIQ [IQ]\n0-None,1-Normal,2-Inverted,3-AUTO\n"
    );

    (void)appRegisterCommand(
        "SetPol",
        APP_SetPolCommand,
        "Set frontend Pol.",
        "Usage: SetPol [Pol]SetPol [Pol].\n0-HOR,1-VER,2-LEFT,3-RIGHT\n"
    );

    (void)appRegisterCommand(
        "SetLNAHigh",
        APP_SetLNAHighFqCommand,
        "Set frontend LNAHigh.",
        "Usage: SetLNAHigh [HLNA]\n"
    );

    (void)appRegisterCommand(
        "SetLNALow",
        APP_SetLNALowFqCommand,
        "Set frontend LNALow.",
        "Usage: SetLNALow [LLNA]\n"
    );

    (void)appRegisterCommand(
        "testPSI",
        APP_TestPSICommand,
        "Test PSI filtering of sfal.",
        "Usage: testPSI\n"
    );

    (void)appRegisterCommand(
        "testMF",
        APP_TestMutiCommand,
        "Test Muti-filter in one channel of sfal.",
        "Usage: testMutiFilter\n"
    );

    appStartThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_START_THREAD_STACKSIZE,
            APP_START_THREAD_PRIORITY,
            appStartThreadFunction,
            0, 0, APP_START_THREAD_NAME );
    if( appStartThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR: creating thread \"%s\" failed\n", APP_START_THREAD_NAME );
        exit(1);
    }

#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#else
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );

    while(1) {
        i = 0;

        /* get command string */
        while( !feof(stdin) ) {
            c = getchar();
            args_buff[i] = c;
            if(c == '\n') {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) ) {
                i --;
            }
            else if(i < (APP_CommandStringSize - 1)) {
                i ++;
            }
        }
        SHELL_DoCommand( args_buff );

        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */

    FAPEX_SYS_WaitEndThread( appStartThreadHandle );
}



