#include <stdio.h>
#include <string.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libgpio.h>
#include "fapex/sys_config.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */


#include "sfal/sfal.h"

#define MAX_FILTER_DEPTH    7

int32_t fd_TSD[4] = {-1,-1,-1,-1};

static uint32_t tsd_init()
{
    FAPEX_TSD_OpenParamStrT  tsdOpenParam;
    int32_t retVal = FAPI_OK;

    /* input settings */
    tsdOpenParam.useStc           = 1;
    tsdOpenParam.tsSelect         = FAPEX_TSD_INPUT_CH_A;
    tsdOpenParam.syncByteDistance = 0;
    tsdOpenParam.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    tsdOpenParam.oflags           = 0;

    fd_TSD[0] = FAPEX_TSD_Open( FAPEX_TSD0, O_RDWR, &tsdOpenParam );
    if(fd_TSD[0] < 0) {
        FAPI_SYS_PRINT_MSG("[main]FAPEX_TSD_Open(): error TSD0\n");
        return -1;
    }

    tsdOpenParam.useStc = 1;
    fd_TSD[1] = FAPEX_TSD_Open( FAPEX_TSD1, O_RDWR, &tsdOpenParam );
    if(fd_TSD[1] < 0) {
        FAPI_SYS_PRINT_MSG("[main]FAPEX_TSD_Open(): error TSD1\n");
        return -1;
    }

    fd_TSD[2] = FAPEX_TSD_Open( FAPEX_TSD2, O_RDWR, &tsdOpenParam );
    if(fd_TSD[2] < 0) {
        FAPI_SYS_PRINT_MSG("[main]FAPEX_TSD_Open(): error TSD2\n");
        return -1;
    }

    fd_TSD[3] = FAPEX_TSD_Open( FAPEX_TSD3, O_RDWR, &tsdOpenParam );
    if(fd_TSD[3]< 0) {
        FAPI_SYS_PRINT_MSG("[main]FAPEX_TSD_Open(): error TSD3\n");
        return -1;
    }

    return retVal;
}


static void sectNotify(FAPI_SYS_HandleT handle,
        	int32_t sectionStatus,
                const uint8_t *pSectionBuffer,
                uint16_t sectionSize,
                void *pPrivateData)
{
    uint32_t i;
    uint8_t sectRecieved = 0;
    uint8_t sectError = 0;
    uint8_t sectTimeout = 0;

    FAPI_SYS_PRINT_MSG("[sfal_test][sectNotify]handle:0x%p,pSectionBuffer:0x%p,sectionSize:0x%x,pPrivateData:0x%p !\n", handle, pSectionBuffer, sectionSize, pPrivateData);

    if (sectionStatus == FAPEX_OK)
    {
        sectRecieved++;

        FAPI_SYS_PRINT_MSG("[sectNotify] section recieve Successful!\n");

        // handle the section
        for (i=0; i< 16; i++)
        //for (i=0; i<sectionSize; i++)
        {
            if(i % 16 == 0)
            {
                printf("\n");
                printf("%3d(0x%x)", i, i);
            }

            if(i % 8 == 0) printf("\t");

            printf("%2x ", pSectionBuffer[i]);
        }
        printf("\n");

        //SFAL_StopFilter(handle);
        }
    else if (sectionStatus == SUBSYS_SFAL_ERR_SECTION_CRC)
    {
        sectError = 1;
        FAPI_SYS_PRINT_MSG("[sectNotify] section CRC error!\n");
    }
    else if (sectionStatus == SUBSYS_SFAL_ERR_SECTION_TIMEOUT)
    {
        sectTimeout = 1;
        FAPI_SYS_PRINT_MSG("[sectNotify] section timeout error!\n");
    }
}

int32_t testMutiFiltering(void)
{
    SFAL_InitT sfal_init;
    SFAL_OpenParamsT appSfalOpenParams;
    FAPI_SYS_HandleT appSfalHandle[4];
    uint8_t filterData [MAX_FILTER_DEPTH];
    uint8_t filterEqual[MAX_FILTER_DEPTH];
    uint8_t filterNonEq[MAX_FILTER_DEPTH];
    uint8_t filter_depth = 7;
    int32_t retVal = FAPI_OK;
    int32_t count = 0;

    tsd_init();

    // Initialise section filtering abstraction layer
    sfal_init.threadPriority   = 80;
    sfal_init.threadSleep      = 0;
    sfal_init.kByteBufferSize  = 256;

    retVal =  SFAL_Init(&sfal_init);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] sfal init Failed! retVal: %d\n", retVal);
        return retVal;
    }

    memset(filterData, 0, filter_depth);
    memset(filterEqual, 0, filter_depth);
    memset(filterNonEq, 0, filter_depth);

    /*------------------ first filter -----------------------*/

    filterData[0] = 0x3b;    // table id
    filterEqual[0] = 0xff;

    appSfalOpenParams.version         = SFAL_VERSION;
    appSfalOpenParams.pid             = 0x385;
    appSfalOpenParams.tsdInput        = FAPEX_TSD3;
    appSfalOpenParams.filterDepth     = filter_depth;
    appSfalOpenParams.matchPtr        = filterData;
    appSfalOpenParams.equalMaskPtr    = filterEqual;
    appSfalOpenParams.notEqualMaskPtr = filterNonEq;


    appSfalHandle[0] =  SFAL_Open(&appSfalOpenParams, &retVal);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Open Failed! retVal: %d\n", retVal);
	    return retVal;
    }

    retVal = SFAL_Start( appSfalHandle[0], 0, SFAL_LOOP_CONTINUOUS, SFAL_CRC_CHECK, NULL, NULL,
			 sectNotify, NULL);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Start Failed! retVal: %d\n",retVal);
        return retVal;
    }
    
    /*------------------ second filter -----------------------*/
    
    memset(filterData, 0, filter_depth);
    memset(filterEqual, 0, filter_depth);

    filterData[0] = 0x3c;    // table id
    filterEqual[0] = 0xff;

    appSfalOpenParams.version         = SFAL_VERSION;
    appSfalOpenParams.pid             = 0x385;
    appSfalOpenParams.tsdInput        = FAPEX_TSD3;
    appSfalOpenParams.filterDepth     = filter_depth;
    appSfalOpenParams.matchPtr        = filterData;
    appSfalOpenParams.equalMaskPtr    = filterEqual;
    appSfalOpenParams.notEqualMaskPtr = filterNonEq;

    appSfalHandle[1] =  SFAL_Open(&appSfalOpenParams, &retVal);
    if(retVal != FAPI_OK)
    {
            FAPI_SYS_PRINT_MSG("[main] SFAL_Open Failed! retVal: %d\n", retVal);
            return retVal;
    }

    retVal = SFAL_Start( appSfalHandle[1], 0, SFAL_LOOP_CONTINUOUS, SFAL_CRC_CHECK, NULL, NULL,
                         sectNotify, NULL);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Start Failed! retVal: %d\n",retVal);
        return retVal;
    }

/*  test */
    retVal = SFAL_Stop(appSfalHandle[0]);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }

#if 0
    retVal = SFAL_Stop(appSfalHandle[1]);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }
#endif
 
    /*------------------ third filter -----------------------*/
#if 0    
    memset(filterData, 0, filter_depth);
    memset(filterEqual, 0, filter_depth);

    //filterData[0] = 0x3c;    // table id
    //filterEqual[0] = 0xff;

    appSfalOpenParams.version         = SFAL_VERSION;
    appSfalOpenParams.pid             = 0x385;
    appSfalOpenParams.tsdInput        = FAPEX_TSD3;
    appSfalOpenParams.filterDepth     = filter_depth;
    appSfalOpenParams.matchPtr        = filterData;
    appSfalOpenParams.equalMaskPtr    = filterEqual;
    appSfalOpenParams.notEqualMaskPtr = filterNonEq;

    appSfalHandle[2] =  SFAL_Open(&appSfalOpenParams, &retVal);
    if(retVal != FAPI_OK)
    {
            FAPI_SYS_PRINT_MSG("[main] SFAL_Open Failed! retVal: %d\n", retVal);
            return retVal;
    }

    retVal = SFAL_Start( appSfalHandle[2], 0, SFAL_LOOP_CONTINUOUS, SFAL_CRC_CHECK, NULL, NULL,
                         sectNotify, NULL);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Start Failed! retVal: %d\n",retVal);
        return retVal;
    }
#endif
    /* start test */
    count = 1000;
    while(count--)
    {
        SFAL_ThreadFunction();
        //sleep(1);
    }


    retVal = SFAL_Stop(appSfalHandle[0]);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }

    retVal = SFAL_Stop(appSfalHandle[1]);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }

    retVal = SFAL_Stop(appSfalHandle[2]);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }


    FAPI_SYS_PRINT_MSG("\n---------------- End of Test -----------------------\n");

    SFAL_Close( appSfalHandle[0] );
    SFAL_Close( appSfalHandle[1] );
    SFAL_Close( appSfalHandle[2] );

    SFAL_Exit();

    return retVal;
}

/* Test for the filter start and stop, change the pid and target of a filter.
 * This is the basic test for SFAL. If it failed, we can't continue other test.
 */
int32_t getPSISection(void)
{
    SFAL_InitT sfal_init;
    SFAL_OpenParamsT appSfalOpenParams;
    FAPI_SYS_HandleT appSfalHandle;
    uint8_t filterData [MAX_FILTER_DEPTH];
    uint8_t filterEqual[MAX_FILTER_DEPTH];
    uint8_t filterNonEq[MAX_FILTER_DEPTH];
    uint8_t filter_depth = 7;
    int32_t retVal = FAPI_OK;
    int32_t count = 0;

    tsd_init();

    // Initialise section filtering abstraction layer
    sfal_init.threadPriority   = 80;
    sfal_init.threadSleep      = 0;
    sfal_init.kByteBufferSize  = 64;

    retVal =  SFAL_Init(&sfal_init);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] sfal init Failed! retVal: %d\n", retVal);
        return retVal;
    }

    memset(filterData, 0, filter_depth);
    memset(filterEqual, 0, filter_depth);
    memset(filterNonEq, 0, filter_depth);

    FAPI_SYS_PRINT_MSG("\n------------------ PAT filtering -----------------------\n");

    //filterData[0] = FAPEX_TSD_PAT_TABLE;    // table id
    //filterData[0] = 0x3b;    // table id
    //filterEqual[0] = 0xff;

    appSfalOpenParams.version         = SFAL_VERSION;
    //appSfalOpenParams.pid             = FAPEX_TSD_PAT_SECTION_PID;
    appSfalOpenParams.pid             = 0x385;
    appSfalOpenParams.tsdInput        = FAPEX_TSD3;
    appSfalOpenParams.filterDepth     = filter_depth;
    appSfalOpenParams.matchPtr        = filterData;
    appSfalOpenParams.equalMaskPtr    = filterEqual;
    appSfalOpenParams.notEqualMaskPtr = filterNonEq;


    appSfalHandle =  SFAL_Open(&appSfalOpenParams, &retVal);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Open Failed! retVal: %d\n", retVal);
	    return retVal;
    }

    retVal = SFAL_Start( appSfalHandle, 0, SFAL_LOOP_CONTINUOUS, SFAL_CRC_CHECK, NULL, NULL,
			 sectNotify, NULL);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Start Failed! retVal: %d\n",retVal);
        return retVal;
    }
    
    /* start test */ 
    count = 2000;
    while(count--)
    {
        SFAL_ThreadFunction();
        //sleep(1);
    }

    retVal = SFAL_Stop(appSfalHandle);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
        return retVal;
    }

    retVal = SFAL_Close(appSfalHandle);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Close Failed! retVal: %d\n",retVal);
        return retVal;
    }

    FAPI_SYS_PRINT_MSG("\n------------------ CAT filtering -----------------------\n");
    
    filterData[0] = FAPEX_TSD_CAT_TABLE;    // table id
    filterEqual[0] = 0xff;

    appSfalOpenParams.version         = SFAL_VERSION;
    appSfalOpenParams.pid             = FAPEX_TSD_CAT_SECTION_PID;
    appSfalOpenParams.tsdInput        = FAPEX_TSD3;
    appSfalOpenParams.filterDepth     = filter_depth;
    appSfalOpenParams.matchPtr        = filterData;
    appSfalOpenParams.equalMaskPtr    = filterEqual;
    appSfalOpenParams.notEqualMaskPtr = filterNonEq;

    appSfalHandle =  SFAL_Open(&appSfalOpenParams, &retVal);
    if(retVal != FAPI_OK)
    {
            FAPI_SYS_PRINT_MSG("[main] SFAL_Open Failed! retVal: %d\n", retVal);
            return retVal;
    }

    retVal = SFAL_Start( appSfalHandle, 0, SFAL_LOOP_CONTINUOUS, SFAL_CRC_CHECK, NULL, NULL,
                         sectNotify, NULL);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[main] SFAL_Start Failed! retVal: %d\n",retVal);
        return retVal;
    }

    /* start test */
    count = 100;
    while(count--)
    {
        SFAL_ThreadFunction();
        //sleep(1);
    }

    retVal = SFAL_Stop(appSfalHandle);
    if(retVal != FAPI_OK)
    {
	    FAPI_SYS_PRINT_MSG("[main] SFAL_Stop Failed! retVal: %d\n",retVal);
	    return retVal;
    }

    FAPI_SYS_PRINT_MSG("\n---------------- End of Test -----------------------\n");

    SFAL_Close( appSfalHandle );
    SFAL_Exit();

    return retVal;
}

