/*!
*****************************************************************************
**
** \file        ./subsystems/audio/aal/src/aal.h
**
** \version     $Id: aal.h 5874 2010-03-02 14:42:55Z wlaris $
**                                          2010-03-10  17:00:00   joy
**
** \brief       Audio abstraction layer (AAL)
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef AAL_H
#define AAL_H

#include "fapex/abst_types.h"
#include "fapex/abst_libauout.h"
#include "fapex/abst_libaudec.h"


    //*****************************************************************************
    //*****************************************************************************
    //** Defines and Macros
    //*****************************************************************************
    //*****************************************************************************

#ifdef _LINUX_FAPI

#define AAL_Assert(a)                                                \
        do                                                           \
        {                                                            \
            if ( !(a) )                                              \
            {                                                        \
                FAPI_SYS_PRINT_MSG("ASSERTION FAILED at %s, l.%i\n", \
                                  __FILE__ ,__LINE__ );              \
                FAPEX_SYS_ExitApplication(-1);                       \
            }                                                        \
        } while (0)                                                  \
        /*lint -restore */

#else

#define AAL_Assert(a)                                                \
        do                                                           \
        {                                                            \
            if ( !(a) )                                              \
            {                                                        \
                FAPI_SYS_PRINT_MSG("ASSERTION FAILED at %s, l.%i\n", \
                                  __FILE__ ,__LINE__ );              \
                RTOS_ExitApplication(-1);                            \
            }                                                        \
        } while (0)                                                  \

#endif

    /*!
    ********************************************************************************
    ** Standard error codes for the module
    ********************************************************************************
    */

    /*! Bad parameter passed. */
    #define SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER         (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_BAD_PARAMETER)

    /*! Memory allocation failed. */
    #define SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY         (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_OUT_OF_MEMORY)

    /*! Module already initialised. */
    #define SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED   (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_ALREADY_INITIALIZED)

    /*! Module not initialised. */
    #define SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED       (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_NOT_INITIALIZED)

    /*! Feature or function is not available. */
    #define SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_FEATURE_NOT_SUPPORTED)

    /*! Timeout occured. */
    #define SUBSYS_AUDIO_AAL_ERR_TIMEOUT               (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_TIMEOUT)

    /*! The device is busy, try again later. */
    #define SUBSYS_AUDIO_AAL_ERR_DEVICE_BUSY           (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_DEVICE_BUSY)

    /*! Invalid handle was passed. */
    #define SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE        (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_INVALID_HANDLE)

    /*! Semaphore could not be created. */
    #define SUBSYS_AUDIO_AAL_ERR_SEMAPHORE_CREATE      (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_SEMAPHORE_CREATE)

    /*! The driver's used version is not supported. */
    #define SUBSYS_AUDIO_AAL_ERR_UNSUPPORTED_VERSION   (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_UNSUPPORTED_VERSION)

    /*! Module has not been opened. */
    #define SUBSYS_AUDIO_AAL_ERR_NOT_OPEN              (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_NOT_OPEN)

    /*! Module is already open. */
    #define SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN          (SUBSYS_AUDIO_AAL_MODULE_BASE +\
                                                               FAPI_ERR_ALREADY_OPEN)

    /*!
    ********************************************************************************
    ** Additional driver error codes (reserved from -100 to -999)
    **
    ** Each error code shall be decreased in steps of 1.
    ********************************************************************************
    */

    /*! AAL specific error code base. */
    #define SUBSYS_AUDIO_AAL_ERR_BASE   (SUBSYS_AUDIO_AAL_MODULE_BASE - 100)

    /*! AAL specific errors. */
    #define SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE      (SUBSYS_AUDIO_AAL_ERR_BASE - 0)
    #define SUBSYS_AUDIO_AAL_ALREADY_STARTED         (SUBSYS_AUDIO_AAL_ERR_BASE - 1)

    #define AAL_VOLUMELEVEL_MAX     0xFE    //!< NOTE: Avoid setting of level 0xFF!

    //*****************************************************************************
    //*****************************************************************************
    //** Data types
    //*****************************************************************************
    //*****************************************************************************

#ifdef _LINUX_FAPI
    typedef enum {
           AAL_AUOUT_VERSION = (int32_t)0x00030000   //!< The current driver version
       } AAL_AUOUT_VersionEnumT;

    typedef enum {
           AAL_AUDEC_VERSION = (int32_t)0x00020000   //!< The current driver version

       } AAL_AUDEC_VersionEnumT;

    typedef enum {
            AAL_AUDEC_SAMPLE_16_BIT  = (int32_t)16, //!< 16 bit sample width.
            AAL_AUDEC_SAMPLE_24_BIT  = (int32_t)24 //!< 24 bit sample width.
        } AAL_AUDEC_SampleBitsEnumT;

    typedef enum
    {
        AAL_AUDEC_DECODER_UNDEF = (int32_t)-1, //!< Audio decoder undefined.
        AAL_AUDEC_DECODER_AC3 = 0,             //!< \b Dolby Digital 5.1 (AC-3) audio decoder.
        AAL_AUDEC_DECODER_AC3SPDIF,            //!< \b Dolby Digital 5.1 (AC-3) to SPDIF copier/synchronizer.
        AAL_AUDEC_DECODER_DD,                  //!< \b Dolby Digital 5.1 (AC-3) audio decoder.
        AAL_AUDEC_DECODER_DDPDCV,              //!< \b Dolby Digital Plus 5.1 (E-/AC-3) decoder/converter.
        AAL_AUDEC_DECODER_HEAAC,               //!< \b HE-AAC 5.1 audio decoder.
        AAL_AUDEC_DECODER_MIXER,               //!< \b PCM Mixer utility
        AAL_AUDEC_DECODER_MPEG,                //!< \b MPEG stereo audio decoder.
        AAL_AUDEC_DECODER_PCMOUTPUT,           //!< \b PCM/WAV Copier.
        AAL_AUDEC_DECODER_WMA,                 //!< \b WMA stereo audio decoder.
        AAL_AUDEC_DECODER_CUSTOMER = 100       //!< \b CUSTOMER specific audio decoder.
    } AAL_AUDEC_DecoderEnumT;

    typedef enum
    {
        AAL_AUOUT_SPEAKER_UNDEF      = (int32_t)-1,
        AAL_AUOUT_SPEAKER_LEFT       = 0,
        AAL_AUOUT_SPEAKER_CENTER     = 1,
        AAL_AUOUT_SPEAKER_RIGHT      = 2,
        AAL_AUOUT_SPEAKER_SUR_LEFT   = 3,
        AAL_AUOUT_SPEAKER_SUR_RIGHT  = 4,
        AAL_AUOUT_SPEAKER_BACK_LEFT  = 5,
        AAL_AUOUT_SPEAKER_BACK_RIGHT = 6,
        AAL_AUOUT_SPEAKER_LEF        = 7,
        AAL_AUOUT_SPEAKER_ALL        = 8
    }AAL_AUOUT_SpeakerEnumT;

    typedef enum
    {
        AAL_AUDEC_STATUS_RUNNING    = 0,
        AAL_AUDEC_STATUS_IDLE,
        AAL_AUDEC_STATUS_WAIT_INPUT,
        AAL_AUDEC_STATUS_WAIT_OUTPUT,
        AAL_AUDEC_STATUS_ABORT,
        AAL_AUDEC_STATUS_NOT_RESPOND,
        AAL_AUDEC_STATUS_RETRY

    } AAL_AUDEC_StatusEnumT;;


    typedef enum
    {
        AAL_AUOUT_STREAM_UNDEF       = (int32_t)-1,
        AAL_AUOUT_STREAM_AAC         = 0,
        AAL_AUOUT_STREAM_AC3         = 1,
        AAL_AUOUT_STREAM_MPEG_L1     = 2,
        AAL_AUOUT_STREAM_MPEG_L2     = 3,
        AAL_AUOUT_STREAM_MPEG_L3     = 4,
        AAL_AUOUT_STREAM_WMA         = 5,
        AAL_AUOUT_STREAM_EC3         = 6,
        AAL_AUOUT_STREAM_HEAAC       = 7,
        AAL_AUOUT_STREAM_PCM            = 8
    }AAL_AUOUT_StreamEnumT;


    typedef enum
    {
        AAL_AUOUT_UNDEF_HZ  = -1,
        AAL_AUOUT_8000_HZ    = 8000,
        AAL_AUOUT_11025_HZ   = 11025,
        AAL_AUOUT_12000_HZ   = 12000,
        AAL_AUOUT_16000_HZ   = 16000,
        AAL_AUOUT_22050_HZ   = 22050,
        AAL_AUOUT_24000_HZ   = 24000,
        AAL_AUOUT_32000_HZ   = 32000,
        AAL_AUOUT_44100_HZ   = 44100,
        AAL_AUOUT_48000_HZ   = 48000,
        AAL_AUOUT_96000_HZ   = 96000,
        AAL_AUOUT_176400_HZ  = 176400,
        AAL_AUOUT_192000_HZ  = 192000

    }AAL_AUOUT_SampleFreqEnumT;


    typedef enum
    {
        AAL_AUOUT_CHANNEL_UNDEF       = (int32_t)-1,
        AAL_AUOUT_CHANNEL_MONO        = 1,
        AAL_AUOUT_CHANNEL_STEREO      = 2,
        AAL_AUOUT_CHANNEL_DUALMONO    = 3,
        AAL_AUOUT_CHANNEL_C           = 4,
        AAL_AUOUT_CHANNEL_L_C_R       = 5,
        AAL_AUOUT_CHANNEL_L_R_S       = 6,
        AAL_AUOUT_CHANNEL_L_C_R_S     = 7,
        AAL_AUOUT_CHANNEL_L_R_SL_SR   = 8,
        AAL_AUOUT_CHANNEL_L_C_R_SL_SR = 9

    }AAL_AUOUT_ChannelEnumT;

    typedef struct
    {
        AAL_AUOUT_StreamEnumT type;
        AAL_AUOUT_SampleFreqEnumT sampleFreq;
        uint32_t lfeOn;
        AAL_AUOUT_ChannelEnumT channels;
        int32_t sampleBits;

    }AAL_AUOUT_StreamInfoT;


    typedef struct
    {
        AAL_AUOUT_VersionEnumT version;
        uint32_t enableSPDIF;
        AAL_AUOUT_SpeakerEnumT speakerArr[AAL_AUOUT_SPEAKER_ALL+1];
        FAPI_SYS_HandleT pcmOutBuffer;

    } AAL_AUOUT_OpenParamsT;




    typedef struct
    {
        uint8_t targetVolume[AAL_AUOUT_SPEAKER_ALL];
        uint8_t currentVolume[AAL_AUOUT_SPEAKER_ALL];

    } AAL_AUOUT_AptVolParamsT;


    typedef struct
    {
        AAL_AUOUT_SampleFreqEnumT minTargetFreq;
        AAL_AUOUT_SampleFreqEnumT maxTargetFreq;
        void* privatePtr;

    } AAL_AUOUT_AptSsrcParamsT;


    typedef struct
    {
        AAL_AUDEC_VersionEnumT version;
        AAL_AUDEC_DecoderEnumT decoder;
        AAL_AUDEC_SampleBitsEnumT sampleBits;
        FAPI_SYS_HandleT inputStreamHandle;
        FAPI_SYS_HandleT audioOutHandle;

    } AAL_AUDEC_OpenParamsT;
#endif

     /*!
    *************************************************************************
    ** \brief Audio Synchronization State.
    **
    ** This enumeration contains the various synchronization states of the
    ** audio output module.
    *************************************************************************
    */
    typedef enum {
        AAL_SYNC_UNDEF           = 0, //!< SYNC state is undefined.
        AAL_SYNCHRONISING,            //!< Synchronization is ongoing.
        AAL_SYNCHRONIZED              //!< Synchronization achieved.
    } AAL_SyncStateT;

    /*!
    ****************************************************************************
    ** \brief HDMI audio source
    **
    ** This enumeration contains the audio sources which can be selected for HDMI.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_HDMISOURCE_I2S = 1, //!< I2S (PCM) will output through HDMI.
        AAL_HDMISOURCE_SPDIF    //!< SPDIF (digital or PCM) will output through HDMI.
    } AAL_HDMISourceT;

    /*!
    ****************************************************************************
    ** \brief Supported Audio Formats
    **
    ** This enumeration contains the audio formats supported by
    ** Audio Abstraction Layer.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_AUDIOFORMAT_NONE = -1,      //!< Audio format: none (internally used).
        AAL_AUDIOFORMAT_MPEG = 0,       //!< Audio format: MPEG (1/2, layer 1...3).
        AAL_AUDIOFORMAT_AC3,            //!< Audio format: Dolby Digital/AC-3.
        AAL_AUDIOFORMAT_AAC,            //!< Audio format: AAC.
        AAL_AUDIOFORMAT_HEAAC,          //!< Audio format: HE-AAC.
        AAL_AUDIOFORMAT_DDP,            //!< Audio format: DD+/EC3.
        AAL_AUDIOFORMAT_WMA,            //!< Audio format: WMA.
        AAL_AUDIOFORMAT_WAV,            //!< Audio format: WAV.
        AAL_AUDIOFORMAT_PCM,            //!< Audio format: PCM. data is provided by given buffer
        AAL_AUDIOFORMAT_DD,             //!< Audio format:DD/AC3
        AAL_AUDIOFORMAT_NUMS            //!< Number of available audio formats.

    } AAL_AudioFormatT;

    /*!
    ****************************************************************************
    ** \brief Stereo Format
    **
    ** This enumeration contains the stereo mode supported by
    ** Audio Abstraction Layer.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_STEREO_MODE_STEREO = 0,     //!< Stereo.
        AAL_STEREO_MODE_MONO_LEFT,      //!< Mono from left channel on both ch.
        AAL_STEREO_MODE_MONO_RIGHT,     //!< Mono from right channel on both ch.
        AAL_STEREO_MODE_MONO_LEFT_ONLY, //!< Mono from left channel on left ch.
                                        //!< only.
        AAL_STEREO_MODE_MONO_RIGHT_ONLY,//!< Mono from right channel on right ch.
                                        //!< only.
        AAL_STEREO_MODE_STEREO_SWAPPED, //!< Stereo mode but left and right channels are swapped.
        AAL_STEREO_MODE_NUMS            //!< Number of available stereo modes.

    } AAL_StereoModeT;

    /*!
    ****************************************************************************
    ** \brief SPDIF output mode
    **
    ** This enumeration contains the Spdif output mode supported by
    ** Audio Abstraction Layer.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_SPDIFMODE_OFF = 0,          //!< SPDIF output is disabled.
        AAL_SPDIFMODE_PASSTHROUGH,      //!< SPDIF output is in passthrough mode,
                                        //!< stream specified by further argument.
        AAL_SPDIFMODE_PCM               //!< SPDIF outputs PCM samples of currently
                                        //!< active main audio stream.
    } AAL_SPDIFModeT;

    /*!
    ****************************************************************************
    ** \brief Audio Feed Mode
    **
    ** This enumeration contains the audio feed mode supported by
    ** Audio Abstraction Layer.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_FEEDMODE_TSD = 0,           //!< Input data is filtered from TSD.
        AAL_FEEDMODE_BM,                //!< Input data is manually input via BM
                                        //!< (see function AAL_FeedBuffer).
        AAL_FEEDMODE_BM_STREAMMEDIA     //!< Input data is manually input via BM in streaming mode.
    } AAL_FeedModeT;

   /*!
    ****************************************************************************
    ** \brief Audio spdif pcm set lock
    **
    ** This enumeration contains spdif pcm output lock supported by
    ** Audio Abstraction Layer.
    **
    ****************************************************************************
    */
    typedef enum
    {
        AAL_SPDIFPCM_UNLOCK = 0,        //!<  spdif pcm output unlock
        AAL_SPDIFPCM_LOCK,              //!<  spdif pcm output lock
    } AAL_SPDIFPCMSetLock;


    /*!
    *************************************************************************
    ** \brief Audio SPDIF mode IDs.
    **
    ** This enumeration contains the IDs of supported SPDIF modes.
    **
    ** \Audio Abstraction Layer
    *************************************************************************
    */
    typedef enum {
        AAL_AUOUT_SPDIF_DEFAULT = (int32_t)0,//!< The default mode according to AUOUT open parameters.
        AAL_AUOUT_SPDIF_PCM,                 //!< PCM output mode.
        AAL_AUOUT_SPDIF_HW_PASSTHROUGH,      //!< Digital passthrough (by hardware) output mode.
        AAL_AUOUT_SPDIF_PASSTHROUGH          //!< Digital passthrough (by software) output mode.
    }AAL_AUOUT_SpdifModeEnumT;



    /*!
    *************************************************************************
    ** \brief Audio processing Tool IDs.
    **
    ** This enumeration contains the IDs of supported audio processing tools.
    **
    ** \sa AAL_AUOUT_SetApt
    ** \sa AAL_AUOUT_AddApt
    *************************************************************************
    */
    typedef enum {
        AAL_AUOUT_APT_UNDEF           = (int32_t)0, //!< APT is undefined.
        AAL_AUOUT_APT_VOLUME_CONTROL,               //!< Software volume control (e.g. for SPDIF).
        AAL_AUOUT_APT_11DB_ATTENUATION,             //!< 11db output attenuation.
        AAL_AUOUT_APT_SIMPLE_SRC,                   //!< Simple Sample rate converter.
        AAL_AUOUT_APT_CUST1           = 8,          //!< Customer specific  audio processing tool 1.
        AAL_AUOUT_APT_CUST2,                        //!< Customer specific  audio processing tool 2.
        AAL_AUOUT_APT_CUST3,                        //!< Customer specific  audio processing tool 3.
        AAL_AUOUT_APT_CUST4,                        //!< Customer specific  audio processing tool 4.
        AAL_AUOUT_APT_CUST5,                        //!< Customer specific  audio processing tool 5.
        AAL_AUOUT_APT_CUST6,                        //!< Customer specific  audio processing tool 6.
        AAL_AUOUT_APT_CUST7,                        //!< Customer specific  audio processing tool 7.
        AAL_AUOUT_APT_CUST8,                        //!< Customer specific  audio processing tool 8.
        AAL_AUOUT_APT_ALL             = 16          //!< Total number of audio tools.
    } AAL_AUOUT_AptEnumT;

typedef enum
{
    AAL_PATH_MAIN = 0,
    AAL_PATH_SPDIF,
    AAL_PATH_MIX0,
    AAL_PATH_MIX1,
    AAL_PATH_MIX2,
//    AAL_PATH_MIX3,
//     AAL_PATH_MIX4,   // because only 8 PCM AUOUT device id, each mix need two, so only use 4 mix
    AAL_PATH_NUMS
} AAL_AudioPathT;

    /*!
    ****************************************************************************
    **
    ** \brief Notification function called before starting and after stopping
    **        TSD channel. This can be used for setting up descramblers for
    **        this particular channel.
    **
    ** \param start     1: TSD channel is about to be started, 0: TSD channel
    **                  has been stopped.
    ** \param tsdHandle Corresponding TSD handle.
    ** \param pid       Audio stream PID.
    ** \param optData   Optional pointer registered by the user together with
    **                  this callback function.
    **
    ** \sa    PVR_RecordOpenT
    **
    ***************************************************************************/
    typedef void (*AAL_DescrCtrlCb)(uint8_t start, int32_t tsdHandle,
                                    uint16_t pid, void* optData);


    /*!
    ****************************************************************************
    ** \brief Add Audio Processing Tools
    **
    ** This function is needed to add needed APTs to the audio output path.
    ** It will be called just after the audio output driver has been opened.
    **
    ** \param audioOutHandle Valid handle of the audio output where the APTs
    **                       should be added.
    ** \param format         The actual audio format which should be decoded.
    **
    ** \return #FAPI_OK if successul or specific error code.
    **
    ****************************************************************************
    */
    typedef int32_t (*AAL_AddAptsCbT) (int32_t audioOutHandle, AAL_AudioFormatT format);


        /*!
    ****************************************************************************
    ** \brief Removing Audio Processing Tools
    **
    ** This function is needed to remove previously added APTs from the
    ** audio output path. It will be called just before the audio output driver
    ** will be closed.
    **
    ** \param audioOutHandle Valid handle of the audio output from what the
    **                       APTs should be removed.
    **
    ** \return #FAPI_OK if successul or specific error code otherwise.
    **
    ****************************************************************************
    */
    typedef int32_t (*AAL_RemoveAptsCbT) (int32_t audioOutHandle);

    /*!
    ****************************************************************************
    ** \brief Open parameters for the AAL subsystem.
    **
    ** This data structure covers all parameters which need to be specified
    ** when an instance of the subsystem is opened.
    **
    ****************************************************************************
    */
    typedef struct
    {
        AAL_FeedModeT           feedMode;//!< Requested audio feed mode.

        AAL_AUOUT_SpeakerEnumT  speaker[AAL_AUOUT_SPEAKER_ALL];
                                        //!< Speaker arrangement
        uint8_t                 numSpeakers;
                                        //!< Number of speakers in speaker array.
        uint8_t                 suppressErrorRecognition;
                                        //!< Enter 0 in case of potentially
                                        //!< errorneous audio data (e.g. from
                                        //!< live broadcast). In this case, the
                                        //!< first few audio frames are lost! Enter
                                        //!< 1 for data where errors are unlikely,
                                        //!< e.g. playback of music files from HDD.
        AAL_AddAptsCbT          addAptsCb;
                                        //!< Callback function to add the required APTs.
                                        //!< If NULL the default APTs will be added.
                                        //!< If set also removeAptsCb must be set.
        AAL_RemoveAptsCbT       removeAptsCb;
                                        //!< Callback function to remove all current APTs.
                                        //!< if NULL the default APTs will be removed.
                                        //!< If set also addAptsCb must be set.

    } AAL_OpenParamsT;

    /*!
    ****************************************************************************
    ** \brief Init parameters for the AAL subsystem.
    **
    ** This data structure covers all parameters which need to be specified
    ** when the subsystem is initialized.
    **
    ****************************************************************************
    */
    typedef struct
    {
        uint32_t blockIndex;            //!< BM/TSD block to be used for filtering
                                        //!< input data.

    } AAL_InitParamsT;

/*!
    ****************************************************************************
    ** \brief AAL mixer path information
    **
    ****************************************************************************
    */
    typedef struct
    {
        uint32_t       mixerOpenNum;        //can use NUM,( means mix path can be used)
        uint32_t       mixerUsedNum;        // used NUM ,( means mix path is working )
        FAPI_SYS_HandleT*   openPtr;
    } AAL_MixInfoT;
    
/*!
    ****************************************************************************
    ** \brief AAL mixer path volume
    **
    ****************************************************************************
    */
    typedef struct
    {
        uint8_t  mixerPathVolume[AAL_AUOUT_SPEAKER_ALL];        //path's each speaker volume.
        uint8_t  pathPan ;       //path's pan
    } AAL_MixPathVolumeT;

/*!
    ****************************************************************************
    ** \brief AAL mixer path start params
    **
    ****************************************************************************
    */
    typedef struct
    {
        AAL_AudioFormatT format;
        uint32_t pid;
        AAL_AUOUT_SampleFreqEnumT sampleFreq;
        AAL_MixPathVolumeT targetPathVolume ;
        
    } AAL_MixPathPramsT;

    //*****************************************************************************
    //*****************************************************************************
    //** Global Data
    //*****************************************************************************
    //*****************************************************************************

    //*****************************************************************************
    //*****************************************************************************
    //** API Functions
    //*****************************************************************************
    //*****************************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    /* general interface */
    int32_t          AAL_Init               (const AAL_InitParamsT* paramsPtr);
    int32_t          AAL_Exit               (void);
    int32_t          AAL_FormatIsDecodable  (AAL_AudioFormatT format,
                                             uint8_t* resultPtr);
    FAPI_SYS_HandleT AAL_Open               (const AAL_OpenParamsT* paramsPtr,
                                             int32_t* errCodePtr);
    int32_t          AAL_Close              (FAPI_SYS_HandleT handle);
    int32_t          AAL_Start              (FAPI_SYS_HandleT handle,
                                             AAL_AudioFormatT format, uint32_t pid);
    int32_t          AAL_StartMixPath     (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                                             AAL_MixPathPramsT *mixerPathParams);
    int32_t          AAL_Stop               (FAPI_SYS_HandleT handle);
    int32_t          AAL_StopMixPath      (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID );   
    int32_t          AAL_StopAllMixPath   ( FAPI_SYS_HandleT handle );
    int32_t          AAL_SetSPDIFMode       (FAPI_SYS_HandleT handle,
                                             AAL_SPDIFModeT mode,
                                             AAL_AudioFormatT format, uint32_t pid);
    int32_t          AAL_SetStereoMode      (AAL_StereoModeT mode);
    int32_t          AAL_SwitchStereoMode (void);
    int32_t          AAL_SetMixer           (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                                             uint8_t pan ,uint8_t fade );
    int32_t          AAL_SetMixPathVol       ( FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID,
                                             AAL_MixPathVolumeT* targetPathVolume );
    int32_t          AAL_SetMixPathFreq   ( FAPI_SYS_HandleT handle , AAL_AudioPathT mixerID ,
                                             AAL_AUOUT_SampleFreqEnumT sampleFreq );
#if 0
    int32_t          AAL_FeedBuffer         (FAPI_SYS_HandleT handle,
                                             const void* bufferPtr, int32_t bytes);
#else
    int32_t          AAL_FeedBuffer         (FAPI_SYS_HandleT handle,
                                              const void* bufferPtr,
                                              int32_t bytes);
#endif
    int32_t          AAL_FeedMixPathBuffer    (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                                             const void* bufferPtr, int32_t bytes, uint32_t timeout);
    int32_t          AAL_ClearBuffer        (FAPI_SYS_HandleT handle);
    int32_t          AAL_ClearMixPathBuffer   (FAPI_SYS_HandleT handle , AAL_AudioPathT mixerID); 
    int32_t          AAL_GetDecStatus       (FAPI_SYS_HandleT handle,
                                             AAL_AUDEC_StatusEnumT* statusPtr);
    int32_t          AAL_GetMixPathDecStatus  (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                                             AAL_AUDEC_StatusEnumT* statusPtr);
    int32_t          AAL_GetStreamInfo      (FAPI_SYS_HandleT handle,
                                             AAL_AUOUT_StreamInfoT* infoPtr);
    int32_t          AAL_GetMixPathStreamInfo (FAPI_SYS_HandleT handle,  AAL_AudioPathT mixerID , 
                                             AAL_AUOUT_StreamInfoT* infoPtr); 
    int32_t          AAL_GetMixPathInfo        ( AAL_MixInfoT* INFO );
    int32_t          AAL_Task               (FAPI_SYS_HandleT handle);
    int32_t          AAL_Pause              (FAPI_SYS_HandleT handle,
                                             uint8_t enterOrExit);
    int32_t          AAL_Mute               (uint8_t mute);
    int32_t          AAL_EnableHdmi         (uint8_t enable);
    int32_t          AAL_SetVolume          (uint8_t lvlLeft, uint8_t lvlRight);
    uint32_t         AAL_GetBufferSize      (FAPI_SYS_HandleT handle);
    uint32_t         AAL_GetMixPathBufferSize (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID);
    uint32_t         AAL_GetBufferLevel     (FAPI_SYS_HandleT handle);
    uint32_t         AAL_GetMixPathBufferLevel (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID);
    uint32_t         AAL_GetSPDIFBufferSize (FAPI_SYS_HandleT handle);
    uint32_t         AAL_GetSPDIFBufferLevel(FAPI_SYS_HandleT handle);
    int32_t          AAL_SetOutputDelay     (FAPI_SYS_HandleT handle,
                                             int32_t outputDelayMsec);
    int32_t          AAL_RebootAudec        (void);
    int32_t          AAL_FeedPts            (FAPI_SYS_HandleT handle, uint64_t pts);

    int32_t          AAL_BufMmap            (FAPI_SYS_HandleT handle);
    int32_t          AAL_MixPathBufMmap(FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID);
    int32_t          AAL_BufMumap           (void);
    int32_t          AAL_MixPathBufMumap(AAL_AudioPathT mixerID);
    int32_t          AAL_SetHDMISource (FAPI_SYS_HandleT handle,
                                               AAL_HDMISourceT source);

    #ifdef __cplusplus
    }
    #endif

#endif /* AAL_H */
