/*
*****************************************************************************
**
** \file        ./subsystems/closedCaption/src/cc.c
**
** \brief       Closed Caption PES Decord driver.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Semiconductor Limited.
*****************************************************************************
*/

#include <stdint.h>
#include <fapex/abst_types.h>
#include <cc.h>

#include <draw/draw.h>
#include <osdhandler/osdhandler.h>

//***************************************************************************
//** Local Data
//***************************************************************************

static char_t pes_buf[32 * 1024];    /* MAX CC PES SIZE */
static char_t read_buf[16];

typedef struct {

    uint8_t  isPAPF;   /* Parameterized active position forward */
    uint8_t  isAPS;    /* Active position set                   */
    uint8_t  isCOL;    /* Colour Controls                       */
    uint8_t  isSZX;    /* Character Size Controls               */
    uint8_t  isFLC;    /* Flashing control                      */
    uint8_t  isPOL;    /* Pattern Polarity Controls             */
    uint8_t  isHLC;    /* HIGHLIGHTING CHARACTER BLOCK          */
    uint8_t  isRPC;    /* Repeat Character                      */
    uint8_t  isTIME;   /* Time Controls                         */
    uint8_t  isCSI;    /* Control Sequence Introducer           */

    uint8_t  isSpecial;

    uint8_t  isGetChar;
    uint8_t  fNewControlCode;

    uint8_t  paramCSI[15];
    uint32_t byteNumCSI;

    CC_DataInfo *tmpDataInfo;

} CC_TmpInfo;

/* character data */
static char_t char1[]   = "\x0020";
static char_t char2[]   = "\x0021";
static char_t char3[]   = "\x0022";
static char_t char4[]   = "\x0023";
static char_t char5[]   = "\x0024";
static char_t char6[]   = "\x0025";
static char_t char7[]   = "\x0026";
static char_t char8[]   = "\x0027";
static char_t char9[]   = "\x0028";
static char_t char10[]  = "\x0029";
static char_t char11[]  = "\x002a";
static char_t char12[]  = "\x002b";
static char_t char13[]  = "\x002c";
static char_t char14[]  = "\x002d";
static char_t char15[]  = "\x002e";
static char_t char16[]  = "\x002f";
static char_t char17[]  = "\x0030";
static char_t char18[]  = "\x0031";
static char_t char19[]  = "\x0032";
static char_t char20[]  = "\x0033";
static char_t char21[]  = "\x0034";
static char_t char22[]  = "\x0035";
static char_t char23[]  = "\x0036";
static char_t char24[]  = "\x0037";
static char_t char25[]  = "\x0038";
static char_t char26[]  = "\x0039";
static char_t char27[]  = "\x003a";
static char_t char28[]  = "\x003b";
static char_t char29[]  = "\x003c";
static char_t char30[]  = "\x003d";
static char_t char31[]  = "\x003e";
static char_t char32[]  = "\x003f";
static char_t char33[]  = "\x0040";
static char_t char34[]  = "\x0041";
static char_t char35[]  = "\x0042";
static char_t char36[]  = "\x0043";
static char_t char37[]  = "\x0044";
static char_t char38[]  = "\x0045";
static char_t char39[]  = "\x0046";
static char_t char40[]  = "\x0047";
static char_t char41[]  = "\x0048";
static char_t char42[]  = "\x0049";
static char_t char43[]  = "\x004a";
static char_t char44[]  = "\x004b";
static char_t char45[]  = "\x004c";
static char_t char46[]  = "\x004d";
static char_t char47[]  = "\x004e";
static char_t char48[]  = "\x004f";
static char_t char49[]  = "\x0050";
static char_t char50[]  = "\x0051";
static char_t char51[]  = "\x0052";
static char_t char52[]  = "\x0053";
static char_t char53[]  = "\x0054";
static char_t char54[]  = "\x0055";
static char_t char55[]  = "\x0056";
static char_t char56[]  = "\x0057";
static char_t char57[]  = "\x0058";
static char_t char58[]  = "\x0059";
static char_t char59[]  = "\x005a";
static char_t char60[]  = "\x005b";
static char_t char61[]  = "\x005c";
static char_t char62[]  = "\x005d";
static char_t char63[]  = "\x005e";
static char_t char64[]  = "\x005f";
static char_t char65[]  = "\x0060";
static char_t char66[]  = "\x0061";
static char_t char67[]  = "\x0062";
static char_t char68[]  = "\x0063";
static char_t char69[]  = "\x0064";
static char_t char70[]  = "\x0065";
static char_t char71[]  = "\x0066";
static char_t char72[]  = "\x0067";
static char_t char73[]  = "\x0068";
static char_t char74[]  = "\x0069";
static char_t char75[]  = "\x006a";
static char_t char76[]  = "\x006b";
static char_t char77[]  = "\x006c";
static char_t char78[]  = "\x006d";
static char_t char79[]  = "\x006e";
static char_t char80[]  = "\x006f";
static char_t char81[]  = "\x0070";
static char_t char82[]  = "\x0071";
static char_t char83[]  = "\x0072";
static char_t char84[]  = "\x0073";
static char_t char85[]  = "\x0074";
static char_t char86[]  = "\x0075";
static char_t char87[]  = "\x0076";
static char_t char88[]  = "\x0077";
static char_t char89[]  = "\x0078";
static char_t char90[]  = "\x0079";
static char_t char91[]  = "\x007a";
static char_t char92[]  = "\x007b";
static char_t char93[]  = "\x007c";
static char_t char94[]  = "\x007d";
static char_t char95[]  = "\x007e";
static char_t char96[]  = "\x00a1";
static char_t char97[]  = "\x00a2";
static char_t char98[]  = "\x00a3";
static char_t char99[]  = "\x20\xac";
static char_t char100[] = "\x00a5";
static char_t char101[] = "\x01\x60";
static char_t char102[] = "\x00a7";
static char_t char103[] = "\x01\x61";
static char_t char104[] = "\x00a9";
static char_t char105[] = "\x00aa";
static char_t char106[] = "\x00ab";
static char_t char107[] = "\x00ac";
static char_t char108[] = "\x00ff";
static char_t char109[] = "\x00ae";
static char_t char110[] = "\x00af";
static char_t char111[] = "\x00b0";
static char_t char112[] = "\x00b1";
static char_t char113[] = "\x00b2";
static char_t char114[] = "\x00b3";
static char_t char115[] = "\x01\x7d";
static char_t char116[] = "\x00b5";
static char_t char117[] = "\x00b6";
static char_t char118[] = "\x00b7";
static char_t char119[] = "\x01\x7e";
static char_t char120[] = "\x00b9";
static char_t char121[] = "\x00ba";
static char_t char122[] = "\x00bb";
static char_t char123[] = "\x01\x52";
static char_t char124[] = "\x01\x53";
static char_t char125[] = "\x01\x78";
static char_t char126[] = "\x00bf";
static char_t char127[] = "\x00c0";
static char_t char128[] = "\x00c1";
static char_t char129[] = "\x00c2";
static char_t char130[] = "\x00c3";
static char_t char131[] = "\x00c4";
static char_t char132[] = "\x00c5";
static char_t char133[] = "\x00c6";
static char_t char134[] = "\x00c7";
static char_t char135[] = "\x00c8";
static char_t char136[] = "\x00c9";
static char_t char137[] = "\x00ca";
static char_t char138[] = "\x00cb";
static char_t char139[] = "\x00cc";
static char_t char140[] = "\x00cd";
static char_t char141[] = "\x00ce";
static char_t char142[] = "\x00cf";
static char_t char143[] = "\x00d0";
static char_t char144[] = "\x00d1";
static char_t char145[] = "\x00d2";
static char_t char146[] = "\x00d3";
static char_t char147[] = "\x00d4";
static char_t char148[] = "\x00d5";
static char_t char149[] = "\x00d6";
static char_t char150[] = "\x00d7";
static char_t char151[] = "\x00d8";
static char_t char152[] = "\x00d9";
static char_t char153[] = "\x00da";
static char_t char154[] = "\x00db";
static char_t char155[] = "\x00dc";
static char_t char156[] = "\x00dd";
static char_t char157[] = "\x00de";
static char_t char158[] = "\x00df";
static char_t char159[] = "\x00e0";
static char_t char160[] = "\x00e1";
static char_t char161[] = "\x00e2";
static char_t char162[] = "\x00e3";
static char_t char163[] = "\x00e4";
static char_t char164[] = "\x00e5";
static char_t char165[] = "\x00e6";
static char_t char166[] = "\x00e7";
static char_t char167[] = "\x00e8";
static char_t char168[] = "\x00e9";
static char_t char169[] = "\x00ea";
static char_t char170[] = "\x00eb";
static char_t char171[] = "\x00ec";
static char_t char172[] = "\x00ed";
static char_t char173[] = "\x00ee";
static char_t char174[] = "\x00ef";
static char_t char175[] = "\x00f5";
static char_t char176[] = "\x00f1";
static char_t char177[] = "\x00f2";
static char_t char178[] = "\x00f3";
static char_t char179[] = "\x00f4";
static char_t char180[] = "\x00f5";
static char_t char181[] = "\x00f6";
static char_t char182[] = "\x00f7";
static char_t char183[] = "\x00f8";
static char_t char184[] = "\x00f9";
static char_t char185[] = "\x00fa";
static char_t char186[] = "\x00fb";
static char_t char187[] = "\x00fc";
static char_t char188[] = "\x00fd";
static char_t char189[] = "\x00fe";
static char_t char190[] = "\x26\x6a";
static char_t char191[] = "\x00a4";
static char_t char192[] = "\x00a6";
static char_t char193[] = "\x00a8";
static char_t char194[] = "\x00b4";
static char_t char195[] = "\x00b8";
static char_t char196[] = "\x00bc";
static char_t char197[] = "\x00bd";
static char_t char198[] = "\x00be";
static char_t char199[] = "\x20\x26";
static char_t char200[] = "\x20\x18";
static char_t char201[] = "\x20\x19";
static char_t char202[] = "\x20\x1c";
static char_t char203[] = "\x20\x1d";
static char_t char204[] = "\x20\x22";
static char_t char205[] = "\x21\x22";
static char_t char206[] = "\x21\x5b";
static char_t char207[] = "\x21\x5c";
static char_t char208[] = "\x21\x5d";
static char_t char209[] = "\x21\x5e";

static CC_TmpInfo *tmpInfo;

//***************************************************************************
//** local Functions
//***************************************************************************
static void CC_SetDrawChar(char_t * charData);
static void CC_GetCSI1(uint8_t *CSI_param, uint8_t *param1, uint32_t *num1);
static void CC_GetCSI2(uint8_t *CSI_param, uint8_t *param1, uint8_t *param2, uint32_t *num1, uint32_t *num2);
static void CC_ControlCodeChange(uint8_t *flag, uint8_t *isFlag, uint8_t *mem, uint8_t value);

/*!
*******************************************************************************
** \brief  Initialize the CC library.
**
** \return One of the following status codes:
**       - \c #FAPI_OK if successful
**       - \c #CC_ERR_ALREADY_INITIALIZED if the driver is already initialized
**       - \c #CC_ERR_OUT_OF_MEMORY if the system runs out of memory
**
** \sa CC_Exit
******************************************************************************/
int32_t CC_Init (void)
{

    if(tmpInfo != NULL) return CC_ERR_ALREADY_INITIALIZED;

    tmpInfo = FAPI_SYS_MALLOC(sizeof(CC_TmpInfo));
    if(tmpInfo == NULL) return CC_ERR_OUT_OF_MEMORY;

    memset(tmpInfo,0,sizeof(CC_TmpInfo));

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief  Exit the CC library.
**
** \return One of the following status codes:
**         - \c #FAPI_OK if successful
**         - \c #CC_ERR_NOT_INITIALIZED if driver is not initialized
**
** \sa VOC_Init
******************************************************************************/
int32_t CC_Exit (void)
{
    if(tmpInfo == NULL)
    {
        return CC_ERR_NOT_INITIALIZED;
    }
    else
    {
        FAPI_SYS_FREE(tmpInfo);
        tmpInfo = NULL;

        return FAPI_OK;
    }
}

/*!
*******************************************************************************
** \brief  Analyze the Closed Caption PES
**
** \return One of the following status codes:
**       - \c #FAPI_OK if successful
**       - \c #CC_ERR_BAD_PARAMETER  if parameter is bad
**
** \sa
******************************************************************************/
int32_t CC_AnalyzePes(char_t* pesPacket,
                      uint32_t pesPacketSize,
                      CC_HeaderInfo* headerInfo,
                      char_t* dataUnit)
{
    char_t* pRead;
    uint32_t i;

    uint8_t stream_id;
    uint16_t PES_packet_length;
    uint8_t PES_scrambling_control;
    uint8_t PES_priority;
    uint8_t data_alignment_indicator;
    uint8_t copyright;
    uint8_t original_or_copy;
    uint8_t PTS_DTS_flag;
    uint8_t ESCR_flag;
    uint8_t ES_rate_flag;
    uint8_t DSM_trick_mode_flag;
    uint8_t additional_copy_info_flag;
    uint8_t PES_CRC_flag;
    uint8_t PES_extension_flag;
    uint16_t PES_header_data_length;
    uint8_t pts33;
    uint8_t escr33;
    uint32_t pts_tmp;
    uint8_t PES_private_data_flag;
    uint8_t packet_header_field_flag;
    uint8_t program_packet_sequence_counter_flag;
    uint8_t P_STD_buffer_flag;
    uint8_t PES_extension_flag_2;

    uint8_t data_identifier;
    uint8_t private_stream_id;
    uint8_t PES_data_packet_header_length;
    uint8_t PES_data_private_data_byte[15];
    uint16_t data_group_size;
    uint32_t data_unit_loop_length;
    uint8_t unit_separator;

    /* argument NULL check */
    if (pesPacket == NULL || headerInfo == NULL) {
        FAPI_SYS_PRINT_DEBUG(1,"NULL ERROR\n");
        return CC_ERR_BAD_PARAMETER;
    }

    if (pesPacketSize > 32 * 1024) {
        FAPI_SYS_PRINT_DEBUG(1,"SIZE ERROR\n");
        return CC_ERR_BAD_PARAMETER;
    }

    memcpy(pes_buf, pesPacket, pesPacketSize);
    pRead = pes_buf;

#if 0
    FAPI_SYS_PRINT_DEBUG(1,"\n");
    for (h = 0; h < pesPacketSize; h++) {
        if (h%16 == 0) {
            FAPI_SYS_PRINT_DEBUG(1,"\n%x ",pes_buf[h]);
        }
        else {
            FAPI_SYS_PRINT_DEBUG(1,"%x ",pes_buf[h]);
        }
    }
    FAPI_SYS_PRINT_DEBUG(1,"\n");
#endif


    /***************************************/
    /* CC PES check                        */
    /***************************************/
    /* packet_start_code_prefix check */
    memcpy(read_buf, pRead, 3);
    if (read_buf[0] != 0x00 || read_buf[1] != 0x00 || read_buf[2] != 0x01) {
        FAPI_SYS_PRINT_DEBUG(1,"packet_start_code_prefix[0] = %x\n", read_buf[0]);
        FAPI_SYS_PRINT_DEBUG(1,"packet_start_code_prefix[1] = %x\n", read_buf[1]);
        FAPI_SYS_PRINT_DEBUG(1,"packet_start_code_prefix[2] = %x\n", read_buf[2]);
    }
    pRead += 3;

    /* Stream id check */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    stream_id = *read_buf;
    if (stream_id != 0xbd) {
        FAPI_SYS_PRINT_DEBUG(1,"stream_id = %x\n", stream_id);
    }

    /* get PES_packet_length    */
    memcpy(read_buf, pRead, 2);
    pRead += 2;
    PES_packet_length = (*read_buf)*256+*(read_buf+1);

    memcpy(read_buf, pRead, 2);
    pRead += 2;
        /* PES_scrambling_control check */
        PES_scrambling_control = (*read_buf & 0x30) >> 4;
        if (PES_scrambling_control != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"PES_scrambling_control = %x\n", PES_scrambling_control);
        }
        /* PES_priority check */
        PES_priority = (*read_buf & 0x08) >> 3;
        if (PES_priority != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"PES_priority = %x\n", PES_priority);
        }
        /* data_alignment_indicator check */
        data_alignment_indicator = (*read_buf & 0x04) >> 2;
        if (data_alignment_indicator != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"data_alignment_indicator = %x\n", data_alignment_indicator);
        }
        /* copyright check */
        copyright = (*read_buf & 0x02) >> 1;
        if (copyright != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"copyright = %x\n", copyright);
        }
        /* original_or_copy check */
        original_or_copy = (*read_buf & 0x01);
        if (original_or_copy != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"original_or_copy = %x\n", original_or_copy);
        }
        /* PTS_DTS_flag check */
        PTS_DTS_flag = *(read_buf+1) >> 6;
        if (PTS_DTS_flag != 2) {
            FAPI_SYS_PRINT_DEBUG(1,"PTS_DTS_flag = %x\n", PTS_DTS_flag);
            if (PTS_DTS_flag == 0) {
                headerInfo->fNotPTS = 1;
            }
        }
        /* ESCR_flag check */
        ESCR_flag = (*(read_buf+1) & 0x20) >> 5;
        if (ESCR_flag != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"ESCR_flag = %x\n", ESCR_flag);
        }
        /* ES_rate_flag check */
        ES_rate_flag = (*(read_buf+1) & 0x10) >> 4;
        if (ES_rate_flag != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"ES_rate_flag = %x\n", ES_rate_flag);
        }
        /* DSM_trick_mode_flag check */
        DSM_trick_mode_flag = (*(read_buf+1) & 0x08) >> 3;
        if (DSM_trick_mode_flag != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"DSM_trick_mode_flag = %x\n", DSM_trick_mode_flag);
        }
        /* additional_copy_info_flag check */
        additional_copy_info_flag = (*(read_buf+1) & 0x04) >> 2;
        if (additional_copy_info_flag != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"additional_copy_info_flag = %x\n", additional_copy_info_flag);
        }
        /* PES_CRC_flag check */
        PES_CRC_flag = (*(read_buf+1) & 0x02) >> 1;
        if (PES_CRC_flag != 0) {
            FAPI_SYS_PRINT_DEBUG(1,"PES_CRC_flag = %x\n", PES_CRC_flag);
        }
        /* PES_extension_flag check */
        PES_extension_flag = (*(read_buf+1) & 0x01);
        if (PES_extension_flag != 1) {
            FAPI_SYS_PRINT_DEBUG(1,"PES_extension_flag = %x\n", PES_extension_flag);
        }

    /* get PES_header_data_length    */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    PES_header_data_length = *read_buf;

    if (PTS_DTS_flag==2) {
        memcpy(read_buf, pRead, 5);
        pRead += 5;

        if ((*read_buf & 0x08)==0x08) {
            pts33 = 0x01;
        }
        else {
            pts33 = 0x00;
        }
        pts_tmp = ((*read_buf) >> 1) & 3;
        pts_tmp = (pts_tmp<<8) + *(read_buf+1);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+2) >> 1);
        pts_tmp = (pts_tmp<<8) + *(read_buf+3);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+4) >> 1);
        headerInfo->PTS = pts_tmp;
    }
    if (PTS_DTS_flag==3) {
        memcpy(read_buf, pRead, 10);
        pRead += 10;

        if ((*read_buf & 0x08)==0x08) {
            pts33 = 0x01;
        }
        else {
            pts33 = 0x00;
        }
        pts_tmp = ((*read_buf) >> 1) & 3;
        pts_tmp = (pts_tmp<<8) + *(read_buf+1);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+2) >> 1);
        pts_tmp = (pts_tmp<<8) + *(read_buf+3);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+4) >> 1);
        headerInfo->PTS = pts_tmp;

        if ((*(read_buf+5) & 0x08)==0x08) {
            pts33 = 0x01;
        }
        else {
            pts33 = 0x00;
        }
        pts_tmp = ((*(read_buf+5)) >> 1) & 3;
        pts_tmp = (pts_tmp<<8) + *(read_buf+6);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+7) >> 1);
        pts_tmp = (pts_tmp<<8) + *(read_buf+8);
        pts_tmp = (pts_tmp<<7) + (*(read_buf+9) >> 1);
        headerInfo->PTS = pts_tmp;
    }

    if (ESCR_flag==1) {
        memcpy(read_buf, pRead, 6);
        pRead += 6;

        if ((*read_buf & 0x20)==0x20) {
            escr33 = 0x01;
        }
        else {
            escr33 = 0x00;
        }
        pts_tmp = ((*read_buf) >> 3) & 3;
        pts_tmp = (pts_tmp<<2) + (*(read_buf)&3);
        pts_tmp = (pts_tmp<<8) + *(read_buf+1);
        pts_tmp = (pts_tmp<<5) + (*(read_buf+2) >> 3);
        pts_tmp = (pts_tmp<<2) + (*(read_buf+2)&3);
        pts_tmp = (pts_tmp<<8) + *(read_buf+3);
        pts_tmp = (pts_tmp<<5) + (*(read_buf+4) >> 3);
        headerInfo->PTS = pts_tmp;
    }


    if (ES_rate_flag==1) {
        memcpy(read_buf, pRead, 3);
        pRead += 3;
    }

    if(DSM_trick_mode_flag==1) {
        memcpy(read_buf, pRead, 1);
        pRead += 1;
    }

    if(additional_copy_info_flag==1) {
        memcpy(read_buf, pRead, 1);
        pRead += 1;
    }

    if(PES_CRC_flag==1) {
        memcpy(read_buf, pRead, 2);
        pRead += 2;
    }

    if(PES_extension_flag==1) {
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        PES_private_data_flag=*read_buf>>7;
        packet_header_field_flag=(*read_buf&0x40)>>6;
        program_packet_sequence_counter_flag=(*read_buf&0x20)>>5;
        P_STD_buffer_flag=(*read_buf&0x10)>>4;
        PES_extension_flag_2=*read_buf&0x01;

        if(PES_private_data_flag==1)
        {
            memcpy(read_buf, pRead, 16);
            pRead += 16;
        }

        if(packet_header_field_flag==1)
        {
            memcpy(read_buf, pRead, 1);
            pRead += 1;
        }

        if(program_packet_sequence_counter_flag==1)
        {
            memcpy(read_buf, pRead, 2);
            pRead += 2;
        }

        if(P_STD_buffer_flag==1)
        {
            memcpy(read_buf, pRead, 2);
            pRead += 2;
        }

        if(PES_extension_flag_2==1)
        {
            memcpy(read_buf, pRead, 1);
            pRead += 1;
        }
    }

    /* Stuffing_byte check */
    memcpy(read_buf, pRead, 1);
    pRead += 1;

    /***************************************/
    /* CC DATA check                       */
    /***************************************/

    /* data_identifier check */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    data_identifier = *read_buf;
    if (data_identifier != 0x80) {
        FAPI_SYS_PRINT_DEBUG(1,"data_identifier = %x\n", data_identifier);
    }

    /* private_stream_id check */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    private_stream_id = *read_buf;
    if (private_stream_id != 0xFF) {
        FAPI_SYS_PRINT_DEBUG(1,"private_stream_id = %x\n", private_stream_id);
    }

    memcpy(read_buf, pRead, 1);
    pRead += 1;

        /* get PES_data_packet_header_length */
        PES_data_packet_header_length = *read_buf & 0x0f;

        /* get PES_data_private_data_byte */
        if (PES_data_packet_header_length != 0) {
            memcpy(read_buf, pRead, PES_data_packet_header_length);
            pRead += PES_data_packet_header_length;
        }

        for (i = 0; i<PES_data_packet_header_length; i++) {
            PES_data_private_data_byte[i] = *read_buf+i;
        }

    memcpy(read_buf, pRead, 1);
    pRead += 1;

        /* get data_group_id */

        //                         DGI
        //                    A           B
        //management         0x00         0x20
        //data(1 language)   0x01         0x21
        //data(2 language)   0x02         0x22
        //data(3 language)   0x03         0x23
        //data(4 language)   0x04         0x24
        //data(5 language)   0x05         0x25
        //data(6 language)   0x06         0x26
        //data(7 language)   0x07         0x27
        //data(8 language)   0x08         0x28

        headerInfo->data_group_id = (*read_buf & 0xfc) >> 2;

        /* get data_group_version */
        headerInfo->data_group_version = *read_buf & 0x03;

    /* get data_group_link_number */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    headerInfo->data_group_link_number = *read_buf;
    if (headerInfo->data_group_link_number != 0x00) {
        FAPI_SYS_PRINT_DEBUG(1,"data_group_link_number = %x\n", headerInfo->data_group_link_number);
    }

    /* last_data_group_link_number */
    memcpy(read_buf, pRead, 1);
    pRead += 1;
    headerInfo->last_data_group_link_number = *read_buf;
    if (headerInfo->last_data_group_link_number != 0x00) {
        FAPI_SYS_PRINT_DEBUG(1,"last_data_group_link_number = %x\n", headerInfo->last_data_group_link_number);
    }

    memcpy(read_buf, pRead, 2);
    pRead += 2;
    data_group_size = (*read_buf)*256+*(read_buf+1);

    ///////////////////////
    /// management data ///
    ///////////////////////
    if (headerInfo->data_group_id == 0x0 || headerInfo->data_group_id == 0x20) {

        headerInfo->is_management = 1;

        /* get TMD */
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        headerInfo->TMD = (*read_buf & 0xc0) >> 6;
        if (headerInfo->TMD != 0x00) {
            FAPI_SYS_PRINT_DEBUG(1,"management TMD = %x\n", headerInfo->TMD);
        }

        /* get num_languages */
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        headerInfo->num_languages = *read_buf;
        if (headerInfo->num_languages > 2) {
            FAPI_SYS_PRINT_DEBUG(1,"management num_languages = %x\n", headerInfo->num_languages);
        }

        for (i = 0; i < headerInfo->num_languages; i++) {
            memcpy(read_buf, pRead, 1);
            pRead += 1;

                /* get language_tag */
                headerInfo->language_tag = (*read_buf & 0xe0) >> 5;

                /* get DMF */
                headerInfo->DMF = (*read_buf & 0x0f);
                if (headerInfo->DMF != 0x02 && headerInfo->DMF == 0x0a) {
                    //FAPI_SYS_PRINT_DEBUG(1,"management DMF = %x\n", headerInfo->DMF);
                }

            /* get ISO_639_language_code */
            memcpy(read_buf, pRead, 3);
            pRead += 3;
            memcpy(headerInfo->ISO_639_language_code, read_buf, 3);

            memcpy(read_buf, pRead, 1);
            pRead += 1;

                /* Format */
                headerInfo->Format = (*read_buf & 0xf0) >> 4;
                if (headerInfo->Format != 0x08 && headerInfo->Format != 0x09 &&headerInfo->Format != 0x0A &&headerInfo->Format != 0x0B) {
                    FAPI_SYS_PRINT_DEBUG(1,"management Format = %x\n", headerInfo->Format);
                }

                /* TCS */
                headerInfo->TCS = (*read_buf & 0x0b) >> 2;
                if (headerInfo->TCS != 0x00) {
                    FAPI_SYS_PRINT_DEBUG(1,"management TCS = %x\n", headerInfo->TCS);
                }

                /* rollup_mode */
                headerInfo->rollup_mode = (*read_buf & 0x03);
        }

    }
    else {
        /////////////////////
        ///  caption data ///
        /////////////////////
        headerInfo->is_management = 0;

        /* get TMD */
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        headerInfo->TMD = (*read_buf & 0xc0) >> 6;
        if (headerInfo->TMD != 0x00) {
            FAPI_SYS_PRINT_DEBUG(1,"TMD = %x\n", headerInfo->TMD);
        }

    }

    /* get data_unit_loop_length */
    memcpy(read_buf, pRead, 3);
    pRead += 3;
    data_unit_loop_length = (*read_buf)*65536+*(read_buf+1)*256+*(read_buf+2);

    if (data_unit_loop_length) {
        /* unit_separator check */
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        unit_separator = *read_buf;
        if (unit_separator != 0x1f) {
            FAPI_SYS_PRINT_DEBUG(1,"unit_separator = %x\n", unit_separator);
        }

        /* get data_unit_parameter */
        memcpy(read_buf, pRead, 1);
        pRead += 1;
        headerInfo->data_unit_parameter = *read_buf;
        if (headerInfo->data_unit_parameter != 0x20 && headerInfo->data_unit_parameter != 0x35
            &&headerInfo->data_unit_parameter != 0x30 &&headerInfo->data_unit_parameter != 0x31) {
            FAPI_SYS_PRINT_DEBUG(1,"data_unit_parameter = %x\n", headerInfo->data_unit_parameter);
        }

        /* get data_unit_size */
        memcpy(read_buf, pRead, 3);
        pRead += 3;
        headerInfo->data_unit_size = (*read_buf)*65536+*(read_buf+1)*256+*(read_buf+2);

        /***************************/
        /* get data_unit_data_byte */
        /***************************/

        memcpy(dataUnit, pRead, headerInfo->data_unit_size);
    }

    return FAPI_OK;

}

/*!
*******************************************************************************
** \brief  Decord the Closed Caption Data
**
** \return One of the following status codes:
**       - \c #FAPI_OK if successful
**       - \c #CC_ERR_BAD_PARAMETER
**       - \c #
**
** \sa
******************************************************************************/
int32_t CC_DecordDataUnit(char_t* dataUnit,
                          CC_DataInfo* dataInfo,
                          uint32_t *leftDataUnitSize)
{

    uint32_t i;
    uint32_t leftSize;

    tmpInfo->tmpDataInfo = dataInfo;

    tmpInfo->tmpDataInfo->charcount = 0;
    tmpInfo->fNewControlCode = 0;
    tmpInfo->tmpDataInfo->APB_num = 0;
    tmpInfo->tmpDataInfo->APF_num = 0;
    tmpInfo->tmpDataInfo->APD_num = 0;
    tmpInfo->tmpDataInfo->APU_num = 0;
    tmpInfo->tmpDataInfo->APR_num = 0;

    leftSize = *leftDataUnitSize;

    /***************************/
    /* decode character code   */
    /***************************/
    for (i = 0; i < leftSize; i++) {
            if (tmpInfo->fNewControlCode) {
                return FAPI_OK;
            }

            /* Get Active position parameter */
            if (tmpInfo->isPAPF) {
                tmpInfo->tmpDataInfo->paramPAPF[0] = dataUnit[i];
                tmpInfo->isPAPF = 0;
                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

            if (tmpInfo->isAPS) {
                tmpInfo->tmpDataInfo->paramAPS[0] = dataUnit[i];
                tmpInfo->tmpDataInfo->paramAPS[1] = dataUnit[i+1];
                tmpInfo->isAPS = 0;
                i++;
                *leftDataUnitSize = *leftDataUnitSize - 2;

                continue;
            }

            /* Get Colour Control parameter */
            if (tmpInfo->isCOL) {
                  tmpInfo->tmpDataInfo->paramCOL[0] = dataUnit[i];
                if ((tmpInfo->tmpDataInfo->paramCOL[0] & 0xf0) == 0x20) {
                    tmpInfo->tmpDataInfo->paramCOL[1] = dataUnit[i+1];
                    i++;
                }
                tmpInfo->isCOL = 0;
                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

            /* Get Character Size Control parameter */
            if (tmpInfo->isSZX) {
                tmpInfo->tmpDataInfo->paramSZX[0] = dataUnit[i];
                tmpInfo->isSZX = 0;
                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

            /* Get Flashing control parameter */
            if (tmpInfo->isFLC) {
                tmpInfo->tmpDataInfo->paramFLC[0] = dataUnit[i];
                if (tmpInfo->tmpDataInfo->paramFLC[0] != 0x40 && tmpInfo->tmpDataInfo->paramFLC[0] != 0x4F) {
                    return -1;
                }
                *leftDataUnitSize = *leftDataUnitSize - 1;
                tmpInfo->isFLC = 0;
                continue;
            }

            /* Get Pattern Polarity Control parameter */
            if (tmpInfo->isPOL) {
                tmpInfo->tmpDataInfo->paramPOL[0] = dataUnit[i];
                if (tmpInfo->tmpDataInfo->paramPOL[0] != 0x40 && tmpInfo->tmpDataInfo->paramPOL[0] != 0x41) {
                    return CC_ERR_BAD_PARAMETER;
                }
                *leftDataUnitSize = *leftDataUnitSize - 1;
                tmpInfo->isPOL = 0;
                continue;
            }

            /* Get HIGHLIGHTING CHARACTER BLOCK parameter */
            if (tmpInfo->isHLC) {
                tmpInfo->tmpDataInfo->paramHLC[0] = dataUnit[i];
                tmpInfo->isHLC = 0;
                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

            /* Get Repeat Character parameter */
            if (tmpInfo->isRPC) {
                tmpInfo->tmpDataInfo->paramRPC[0] = dataUnit[i];
                tmpInfo->isRPC = 0;
                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

            /* Get Time Control parameter */
            if (tmpInfo->isTIME) {
                tmpInfo->tmpDataInfo->paramTIME[0] = dataUnit[i];
                if (tmpInfo->tmpDataInfo->paramTIME[0] != 0x20) {
                    return CC_ERR_BAD_PARAMETER;
                }
                tmpInfo->tmpDataInfo->paramTIME[1] = dataUnit[i+1];
                tmpInfo->isTIME = 0;
                i++;
                *leftDataUnitSize = *leftDataUnitSize - 2;
                continue;
            }

            /* Get Control Sequence Introducer parameter */
            if (tmpInfo->isCSI) {
                tmpInfo->paramCSI[tmpInfo->byteNumCSI] = dataUnit[i];
                switch (tmpInfo->paramCSI[tmpInfo->byteNumCSI]) {
                    case 0x53: /* SWF  */
                        CC_GetCSI1(tmpInfo->paramCSI, tmpInfo->tmpDataInfo->paramSWF, &tmpInfo->tmpDataInfo->num1_SWF);
                        if (tmpInfo->tmpDataInfo->paramSWF[0] != 0x37 && tmpInfo->tmpDataInfo->paramSWF[0] != 0x38
                            && tmpInfo->tmpDataInfo->paramSWF[0] != 0x39 && tmpInfo->tmpDataInfo->paramSWF[0] != 0x3A) {
                            return CC_ERR_BAD_PARAMETER;
                        }
                        tmpInfo->tmpDataInfo->fSWF = 1;
                        break;
                    case 0x6E: /* RCS  */
                        CC_GetCSI1(tmpInfo->paramCSI, tmpInfo->tmpDataInfo->paramRCS, &tmpInfo->tmpDataInfo->num1_RCS);
                        tmpInfo->tmpDataInfo->fRCS = 1;
                        break;
                    case 0x61: /* ACPS */
                        CC_GetCSI2(tmpInfo->paramCSI, tmpInfo->tmpDataInfo->paramACPS_H, tmpInfo->tmpDataInfo->paramACPS_V,
                        &tmpInfo->tmpDataInfo->num1_ACPS, &tmpInfo->tmpDataInfo->num2_ACPS);
                        tmpInfo->tmpDataInfo->fACPS = 1;
                        break;
                    case 0x56: /* SDF  */
                        CC_GetCSI2(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramSDF_H,  tmpInfo->tmpDataInfo->paramSDF_V,
                        &tmpInfo->tmpDataInfo->num1_SDF, &tmpInfo->tmpDataInfo->num2_SDF);
                        tmpInfo->tmpDataInfo->fSDF = 1;
                        break;
                    case 0x5F: /* SDP  */
                        CC_GetCSI2(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramSDP_H,  tmpInfo->tmpDataInfo->paramSDP_V,
                        &tmpInfo->tmpDataInfo->num1_SDP, &tmpInfo->tmpDataInfo->num2_SDP);
                        tmpInfo->tmpDataInfo->fSDP = 1;
                        break;
                    case 0x57: /* SSM  */
                        CC_GetCSI2(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramSSM_H,  tmpInfo->tmpDataInfo->paramSSM_V,
                        &tmpInfo->tmpDataInfo->num1_SSM, &tmpInfo->tmpDataInfo->num2_SSM);
                        tmpInfo->tmpDataInfo->fSSM = 1;
                        break;
                    case 0x58: /* SHS  */
                        CC_GetCSI1(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramSHS, &tmpInfo->tmpDataInfo->num1_SHS);
                        tmpInfo->tmpDataInfo->fSHS = 1;
                        break;
                    case 0x59: /* SVS  */
                        CC_GetCSI1(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramSVS, &tmpInfo->tmpDataInfo->num1_SVS);
                        tmpInfo->tmpDataInfo->fSVS = 1;
                        break;
                    case 0x63: /* ORN  */
                        CC_GetCSI2(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramORN,  tmpInfo->tmpDataInfo->paramORN_C,
                        &tmpInfo->tmpDataInfo->num1_ORN, &tmpInfo->tmpDataInfo->num2_ORN);
                        tmpInfo->tmpDataInfo->fORN = 1;
                        break;
                    case 0x68: /* PRA  */
                        CC_GetCSI1(tmpInfo->paramCSI,  tmpInfo->tmpDataInfo->paramPRA, &tmpInfo->tmpDataInfo->num1_PRA);
                        tmpInfo->tmpDataInfo->fPRA = 1;
                        break;
                    case 0x6F: /* SCR?  */
                        break;
                    default:
                        tmpInfo->byteNumCSI ++;
                        break;
                }

                *leftDataUnitSize = *leftDataUnitSize - 1;
                continue;
            }

        switch (dataUnit[i]) {

            /******************/
            /*   Not Accept   */
            /******************/
            case 0x00:                                                                                                        break;   /* NUL     */
            case 0x01:                                                                                                        break;   /* NOT USE */
            case 0x02:                                                                                                        break;   /* NOT USE */
            case 0x03:                                                                                                        break;   /* NOT USE */
            case 0x04:                                                                                                        break;   /* NOT USE */
            case 0x05:                                                                                                        break;   /* NOT USE */
            case 0x06:                                                                                                        break;   /* NOT USE */
            case 0x10:                                                                                                        break;   /* NOT USE */
            case 0x11:                                                                                                        break;   /* NOT USE */
            case 0x12:                                                                                                        break;   /* NOT USE */
            case 0x13:                                                                                                        break;   /* NOT USE */
            case 0x14:                                                                                                        break;   /* NOT USE */
            case 0x15:                                                                                                        break;   /* NOT USE */
            case 0x17:                                                                                                        break;   /* NOT USE */
            case 0x1a:                                                                                                        break;   /* NOT USE */
            case 0x8c:                                                                                                        break;   /* NOT USE */
            case 0x8d:                                                                                                        break;   /* NOT USE */
            case 0x8e:                                                                                                        break;   /* NOT USE */
            case 0x8f:                                                                                                        break;   /* NOT USE */
            case 0x96:                                                                                                        break;   /* NOT USE */
            case 0x9c:                                                                                                        break;   /* NOT USE */
            case 0x9e:                                                                                                        break;   /* NOT USE */
            case 0x9f:                                                                                                        break;   /* NOT USE */
            case 0xa0:                                                                                                        break;   /*   10/0  */
            case 0xff:                                                                                                        break;   /*   15/15 */
            case 0x07:                                                                                                        break;   /*   BEL   */
            case 0x18:                                                                                                        break;   /*   CAN   */
            case 0x1e:                                                                                                        break;   /*   RS    */
            case 0x92:                                                                                                        break;   /*   CDC   */
            case 0x94:                                                                                                        break;   /*   WMM   */
            case 0x95:                                                                                                        break;   /*  MACRO  */
            case 0x7f:                                                                                                        break;   /*   DEL   */
            case 0x0e:                                                                                                        break;   /*   LS1   */
            case 0x0f:                                                                                                        break;   /*   LS0   */
            case 0x19:                                                                                                        break;   /*   SS2   */
            case 0x1f:                                                                                                        break;   /*   US    */
            case 0x1b:                                                                                                        break;   /*   ESC   */

            /******************/
            /*  Control Code  */
            /******************/
            /*  Active position  */
            case 0x08: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPB, NULL, NULL, 0);
                       if (!tmpInfo->fNewControlCode) {
                           tmpInfo->tmpDataInfo->APB_num ++;
                       }                                                                                                      break;    /* APB */
            case 0x09: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPF, NULL, NULL, 0);
                       if (!tmpInfo->fNewControlCode) {
                           tmpInfo->tmpDataInfo->APF_num ++;
                       }                                                                                                      break;    /* APF */
            case 0x0a: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPD, NULL, NULL, 0);
                       if (!tmpInfo->fNewControlCode) {
                           tmpInfo->tmpDataInfo->APD_num ++;
                       }                                                                                                      break;    /* APD */
            case 0x0b: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPU, NULL, NULL, 0);
                       if (!tmpInfo->fNewControlCode) {
                           tmpInfo->tmpDataInfo->APU_num ++;
                       }                                                                                                      break;    /* APU */
            case 0x0d: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPR, NULL, NULL, 0);
                       if (!tmpInfo->fNewControlCode) {
                           tmpInfo->tmpDataInfo->APR_num ++;
                       }                                                                                                      break;    /* APR */
            case 0x16: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fPAPF, &tmpInfo->isPAPF, NULL, 0);                         break;    /* PAPF */
            case 0x1c: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fAPS, &tmpInfo->isAPS, NULL, 0);                           break;    /* APS */

            /* Colour */
            case 0x80: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fBKF, NULL, tmpInfo->tmpDataInfo->charColor, 0x80);        break;    /* BKF */
            case 0x81: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fRDF, NULL, tmpInfo->tmpDataInfo->charColor, 0x81);        break;    /* RDF */
            case 0x82: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fGRF, NULL, tmpInfo->tmpDataInfo->charColor, 0x82);        break;    /* GRF */
            case 0x83: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fYLF, NULL, tmpInfo->tmpDataInfo->charColor, 0x83);        break;    /* YLF */
            case 0x84: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fBLF, NULL, tmpInfo->tmpDataInfo->charColor, 0x84);        break;    /* BLF */
            case 0x85: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fMGF, NULL, tmpInfo->tmpDataInfo->charColor, 0x85);        break;    /* MGF */
            case 0x86: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fCNF, NULL, tmpInfo->tmpDataInfo->charColor, 0x86);        break;    /* CNF */
            case 0x87: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fWHF, NULL, tmpInfo->tmpDataInfo->charColor, 0x87);        break;    /* WHF */
            case 0x90: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fCOL, &tmpInfo->isCOL, NULL, 0);                           break;    /* COL */

            /* Character Size */
            case 0x88: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fSSZ, NULL, tmpInfo->tmpDataInfo->charSize, 0x88);         break;    /* SSZ */
            case 0x89: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fMSZ, NULL, tmpInfo->tmpDataInfo->charSize, 0x89);         break;    /* MSZ */
            case 0x8a: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fNSZ, NULL, tmpInfo->tmpDataInfo->charSize, 0x8a);         break;    /* NSZ */
            case 0x8b: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fSZX, &tmpInfo->isSZX, NULL, 0);                           break;    /* SZX */

            /* Control Sequence Introducer */
            case 0x9b: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fCSI, &tmpInfo->isCSI, NULL, 0);                           break;    /* CSI */

            /* Flashing */
            case 0x91: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fFLC, &tmpInfo->isFLC, NULL, 0);                           break;    /* FLC */

            /* Pattern Polarity */
            case 0x93: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fPOL, &tmpInfo->isPOL, NULL, 0);                           break;    /* POL */

            /* HIGHLIGHTING CHARACTER BLOCK */
            case 0x97: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fHLC, &tmpInfo->isHLC, NULL, 0);                           break;    /* HLC */

            /* Repeat Character */
            case 0x98: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fRPC, &tmpInfo->isRPC, NULL, 0);                           break;    /* RPC */

            /* Stop Lining */
            case 0x99: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fSPL, NULL, NULL, 0);                                      break;    /* SPL */

            /* Start Lining */
            case 0x9a: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fSTL, NULL, NULL, 0);                                      break;    /* STL */

            /* Time */
            case 0x9d: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fTIME, &tmpInfo->isTIME, NULL, 0);                         break;    /* TIME */

            /* Clear screen */
            case 0x0c: CC_ControlCodeChange(&tmpInfo->tmpDataInfo->fCS, NULL, NULL, 0);                                       break;    /* CS  */

            /* Single shift 3 */
            case 0x1d: tmpInfo->isSpecial = 1;                                                                                break;    /* SS3 */

            /******************/
            /* Character Code */
            /******************/
            case 0x20: CC_SetDrawChar(char1);                                                                                 break;    /* SP  */
            case 0x21: (tmpInfo->isSpecial) ? CC_SetDrawChar(char190) : CC_SetDrawChar(char2);                                break;
            case 0x22: CC_SetDrawChar(char3);                                                                                 break;
            case 0x23: CC_SetDrawChar(char4);                                                                                 break;
            case 0x24: CC_SetDrawChar(char5);                                                                                 break;
            case 0x25: CC_SetDrawChar(char6);                                                                                 break;
            case 0x26: CC_SetDrawChar(char7);                                                                                 break;
            case 0x27: CC_SetDrawChar(char8);                                                                                 break;
            case 0x28: CC_SetDrawChar(char9);                                                                                 break;
            case 0x29: CC_SetDrawChar(char10);                                                                                break;
            case 0x2a: CC_SetDrawChar(char11);                                                                                break;
            case 0x2b: CC_SetDrawChar(char12);                                                                                break;
            case 0x2c: CC_SetDrawChar(char13);                                                                                break;
            case 0x2d: CC_SetDrawChar(char14);                                                                                break;
            case 0x2e: CC_SetDrawChar(char15);                                                                                break;
            case 0x2f: CC_SetDrawChar(char16);                                                                                break;
            case 0x30: (tmpInfo->isSpecial) ? CC_SetDrawChar(char191) : CC_SetDrawChar(char17);                               break;
            case 0x31: (tmpInfo->isSpecial) ? CC_SetDrawChar(char192) : CC_SetDrawChar(char18);                               break;
            case 0x32: (tmpInfo->isSpecial) ? CC_SetDrawChar(char193) : CC_SetDrawChar(char19);                               break;
            case 0x33: (tmpInfo->isSpecial) ? CC_SetDrawChar(char194) : CC_SetDrawChar(char20);                               break;
            case 0x34: (tmpInfo->isSpecial) ? CC_SetDrawChar(char195) : CC_SetDrawChar(char21);                               break;
            case 0x35: (tmpInfo->isSpecial) ? CC_SetDrawChar(char196) : CC_SetDrawChar(char22);                               break;
            case 0x36: (tmpInfo->isSpecial) ? CC_SetDrawChar(char197) : CC_SetDrawChar(char23);                               break;
            case 0x37: (tmpInfo->isSpecial) ? CC_SetDrawChar(char198) : CC_SetDrawChar(char24);                               break;
            case 0x38: CC_SetDrawChar(char25);                                                                                break;
            case 0x39: CC_SetDrawChar(char26);                                                                                break;
            case 0x3a: CC_SetDrawChar(char27);                                                                                break;
            case 0x3b: CC_SetDrawChar(char28);                                                                                break;
            case 0x3c: CC_SetDrawChar(char29);                                                                                break;
            case 0x3d: CC_SetDrawChar(char30);                                                                                break;
            case 0x3e: CC_SetDrawChar(char31);                                                                                break;
            case 0x3f: CC_SetDrawChar(char32);                                                                                break;
            case 0x40: (tmpInfo->isSpecial) ? CC_SetDrawChar(char199) : CC_SetDrawChar(char33);                               break;
            case 0x41: CC_SetDrawChar(char34);                                                                                break;
            case 0x42: (tmpInfo->isSpecial) ? CC_SetDrawChar(char200) : CC_SetDrawChar(char35);                               break;
            case 0x43: (tmpInfo->isSpecial) ? CC_SetDrawChar(char201) : CC_SetDrawChar(char36);                               break;
            case 0x44: (tmpInfo->isSpecial) ? CC_SetDrawChar(char202) : CC_SetDrawChar(char37);                               break;
            case 0x45: (tmpInfo->isSpecial) ? CC_SetDrawChar(char203) : CC_SetDrawChar(char38);                               break;
            case 0x46: (tmpInfo->isSpecial) ? CC_SetDrawChar(char204) : CC_SetDrawChar(char39);                               break;
            case 0x47: (tmpInfo->isSpecial) ? CC_SetDrawChar(char205) : CC_SetDrawChar(char40);                               break;
            case 0x48: (tmpInfo->isSpecial) ? CC_SetDrawChar(char206) : CC_SetDrawChar(char41);                               break;
            case 0x49: (tmpInfo->isSpecial) ? CC_SetDrawChar(char207) : CC_SetDrawChar(char42);                               break;
            case 0x4a: (tmpInfo->isSpecial) ? CC_SetDrawChar(char208) : CC_SetDrawChar(char43);                               break;
            case 0x4b: (tmpInfo->isSpecial) ? CC_SetDrawChar(char209) : CC_SetDrawChar(char44);                               break;
            case 0x4c: CC_SetDrawChar(char45);                                                                                break;
            case 0x4d: CC_SetDrawChar(char46);                                                                                break;
            case 0x4e: CC_SetDrawChar(char47);                                                                                break;
            case 0x4f: CC_SetDrawChar(char48);                                                                                break;
            case 0x50: CC_SetDrawChar(char49);                                                                                break;
            case 0x51: CC_SetDrawChar(char50);                                                                                break;
            case 0x52: CC_SetDrawChar(char51);                                                                                break;
            case 0x53: CC_SetDrawChar(char52);                                                                                break;
            case 0x54: CC_SetDrawChar(char53);                                                                                break;
            case 0x55: CC_SetDrawChar(char54);                                                                                break;
            case 0x56: CC_SetDrawChar(char55);                                                                                break;
            case 0x57: CC_SetDrawChar(char56);                                                                                break;
            case 0x58: CC_SetDrawChar(char57);                                                                                break;
            case 0x59: CC_SetDrawChar(char58);                                                                                break;
            case 0x5a: CC_SetDrawChar(char59);                                                                                break;
            case 0x5b: CC_SetDrawChar(char60);                                                                                break;
            case 0x5c: CC_SetDrawChar(char61);                                                                                break;
            case 0x5d: CC_SetDrawChar(char62);                                                                                break;
            case 0x5e: CC_SetDrawChar(char63);                                                                                break;
            case 0x5f: CC_SetDrawChar(char64);                                                                                break;
            case 0x60: CC_SetDrawChar(char65);                                                                                break;
            case 0x61: CC_SetDrawChar(char66);                                                                                break;
            case 0x62: CC_SetDrawChar(char67);                                                                                break;
            case 0x63: CC_SetDrawChar(char68);                                                                                break;
            case 0x64: CC_SetDrawChar(char69);                                                                                break;
            case 0x65: CC_SetDrawChar(char70);                                                                                break;
            case 0x66: CC_SetDrawChar(char71);                                                                                break;
            case 0x67: CC_SetDrawChar(char72);                                                                                break;
            case 0x68: CC_SetDrawChar(char73);                                                                                break;
            case 0x69: CC_SetDrawChar(char74);                                                                                break;
            case 0x6a: CC_SetDrawChar(char75);                                                                                break;
            case 0x6b: CC_SetDrawChar(char76);                                                                                break;
            case 0x6c: CC_SetDrawChar(char77);                                                                                break;
            case 0x6d: CC_SetDrawChar(char78);                                                                                break;
            case 0x6e: CC_SetDrawChar(char79);                                                                                break;
            case 0x6f: CC_SetDrawChar(char80);                                                                                break;
            case 0x70: CC_SetDrawChar(char81);                                                                                break;
            case 0x71: CC_SetDrawChar(char82);                                                                                break;
            case 0x72: CC_SetDrawChar(char83);                                                                                break;
            case 0x73: CC_SetDrawChar(char84);                                                                                break;
            case 0x74: CC_SetDrawChar(char85);                                                                                break;
            case 0x75: CC_SetDrawChar(char86);                                                                                break;
            case 0x76: CC_SetDrawChar(char87);                                                                                break;
            case 0x77: CC_SetDrawChar(char88);                                                                                break;
            case 0x78: CC_SetDrawChar(char89);                                                                                break;
            case 0x79: CC_SetDrawChar(char90);                                                                                break;
            case 0x7a: CC_SetDrawChar(char91);                                                                                break;
            case 0x7b: CC_SetDrawChar(char92);                                                                                break;
            case 0x7c: CC_SetDrawChar(char93);                                                                                break;
            case 0x7d: CC_SetDrawChar(char94);                                                                                break;
            case 0x7e: CC_SetDrawChar(char95);                                                                                break;
            case 0xa1: CC_SetDrawChar(char96);                                                                                break;
            case 0xa2: CC_SetDrawChar(char97);                                                                                break;
            case 0xa3: CC_SetDrawChar(char98);                                                                                break;
            case 0xa4: CC_SetDrawChar(char99);                                                                                break;
            case 0xa5: CC_SetDrawChar(char100);                                                                               break;
            case 0xa6: CC_SetDrawChar(char101);                                                                               break;
            case 0xa7: CC_SetDrawChar(char102);                                                                               break;
            case 0xa8: CC_SetDrawChar(char103);                                                                               break;
            case 0xa9: CC_SetDrawChar(char104);                                                                               break;
            case 0xaa: CC_SetDrawChar(char105);                                                                               break;
            case 0xab: CC_SetDrawChar(char106);                                                                               break;
            case 0xac: CC_SetDrawChar(char107);                                                                               break;
            case 0xad: CC_SetDrawChar(char108);                                                                               break;
            case 0xae: CC_SetDrawChar(char109);                                                                               break;
            case 0xaf: CC_SetDrawChar(char110);                                                                               break;
            case 0xb0: CC_SetDrawChar(char111);                                                                               break;
            case 0xb1: CC_SetDrawChar(char112);                                                                               break;
            case 0xb2: CC_SetDrawChar(char113);                                                                               break;
            case 0xb3: CC_SetDrawChar(char114);                                                                               break;
            case 0xb4: CC_SetDrawChar(char115);                                                                               break;
            case 0xb5: CC_SetDrawChar(char116);                                                                               break;
            case 0xb6: CC_SetDrawChar(char117);                                                                               break;
            case 0xb7: CC_SetDrawChar(char118);                                                                               break;
            case 0xb8: CC_SetDrawChar(char119);                                                                               break;
            case 0xb9: CC_SetDrawChar(char120);                                                                               break;
            case 0xba: CC_SetDrawChar(char121);                                                                               break;
            case 0xbb: CC_SetDrawChar(char122);                                                                               break;
            case 0xbc: CC_SetDrawChar(char123);                                                                               break;
            case 0xbd: CC_SetDrawChar(char124);                                                                               break;
            case 0xbe: CC_SetDrawChar(char125);                                                                               break;
            case 0xbf: CC_SetDrawChar(char126);                                                                               break;
            case 0xc0: CC_SetDrawChar(char127);                                                                               break;
            case 0xc1: CC_SetDrawChar(char128);                                                                               break;
            case 0xc2: CC_SetDrawChar(char129);                                                                               break;
            case 0xc3: CC_SetDrawChar(char130);                                                                               break;
            case 0xc4: CC_SetDrawChar(char131);                                                                               break;
            case 0xc5: CC_SetDrawChar(char132);                                                                               break;
            case 0xc6: CC_SetDrawChar(char133);                                                                               break;
            case 0xc7: CC_SetDrawChar(char134);                                                                               break;
            case 0xc8: CC_SetDrawChar(char135);                                                                               break;
            case 0xc9: CC_SetDrawChar(char136);                                                                               break;
            case 0xca: CC_SetDrawChar(char137);                                                                               break;
            case 0xcb: CC_SetDrawChar(char138);                                                                               break;
            case 0xcc: CC_SetDrawChar(char139);                                                                               break;
            case 0xcd: CC_SetDrawChar(char140);                                                                               break;
            case 0xce: CC_SetDrawChar(char141);                                                                               break;
            case 0xcf: CC_SetDrawChar(char142);                                                                               break;
            case 0xd0: CC_SetDrawChar(char143);                                                                               break;
            case 0xd1: CC_SetDrawChar(char144);                                                                               break;
            case 0xd2: CC_SetDrawChar(char145);                                                                               break;
            case 0xd3: CC_SetDrawChar(char146);                                                                               break;
            case 0xd4: CC_SetDrawChar(char147);                                                                               break;
            case 0xd5: CC_SetDrawChar(char148);                                                                               break;
            case 0xd6: CC_SetDrawChar(char149);                                                                               break;
            case 0xd7: CC_SetDrawChar(char150);                                                                               break;
            case 0xd8: CC_SetDrawChar(char151);                                                                               break;
            case 0xd9: CC_SetDrawChar(char152);                                                                               break;
            case 0xda: CC_SetDrawChar(char153);                                                                               break;
            case 0xdb: CC_SetDrawChar(char154);                                                                               break;
            case 0xdc: CC_SetDrawChar(char155);                                                                               break;
            case 0xdd: CC_SetDrawChar(char156);                                                                               break;
            case 0xde: CC_SetDrawChar(char157);                                                                               break;
            case 0xdf: CC_SetDrawChar(char158);                                                                               break;
            case 0xe0: CC_SetDrawChar(char159);                                                                               break;
            case 0xe1: CC_SetDrawChar(char160);                                                                               break;
            case 0xe2: CC_SetDrawChar(char161);                                                                               break;
            case 0xe3: CC_SetDrawChar(char162);                                                                               break;
            case 0xe4: CC_SetDrawChar(char163);                                                                               break;
            case 0xe5: CC_SetDrawChar(char164);                                                                               break;
            case 0xe6: CC_SetDrawChar(char165);                                                                               break;
            case 0xe7: CC_SetDrawChar(char166);                                                                               break;
            case 0xe8: CC_SetDrawChar(char167);                                                                               break;
            case 0xe9: CC_SetDrawChar(char168);                                                                               break;
            case 0xea: CC_SetDrawChar(char169);                                                                               break;
            case 0xeb: CC_SetDrawChar(char170);                                                                               break;
            case 0xec: CC_SetDrawChar(char171);                                                                               break;
            case 0xed: CC_SetDrawChar(char172);                                                                               break;
            case 0xee: CC_SetDrawChar(char173);                                                                               break;
            case 0xef: CC_SetDrawChar(char174);                                                                               break;
            case 0xf0: CC_SetDrawChar(char175);                                                                               break;
            case 0xf1: CC_SetDrawChar(char176);                                                                               break;
            case 0xf2: CC_SetDrawChar(char177);                                                                               break;
            case 0xf3: CC_SetDrawChar(char178);                                                                               break;
            case 0xf4: CC_SetDrawChar(char179);                                                                               break;
            case 0xf5: CC_SetDrawChar(char180);                                                                               break;
            case 0xf6: CC_SetDrawChar(char181);                                                                               break;
            case 0xf7: CC_SetDrawChar(char182);                                                                               break;
            case 0xf8: CC_SetDrawChar(char183);                                                                               break;
            case 0xf9: CC_SetDrawChar(char184);                                                                               break;
            case 0xfa: CC_SetDrawChar(char185);                                                                               break;
            case 0xfb: CC_SetDrawChar(char186);                                                                               break;
            case 0xfc: CC_SetDrawChar(char187);                                                                               break;
            case 0xfd: CC_SetDrawChar(char188);                                                                               break;
            case 0xfe: CC_SetDrawChar(char189);                                                                               break;
            default :                                                                                                         break;

        }

        if (!tmpInfo->fNewControlCode) {
            *leftDataUnitSize = *leftDataUnitSize - 1;
        }

    }

    return FAPI_OK;

}

/*!
*******************************************************************************
** \brief  Set draw character.
**
** \return Nothing
**
** \sa
******************************************************************************/
static void CC_SetDrawChar(char_t * charData)
{

    if (!tmpInfo->tmpDataInfo->charcount) {
        strcpy((char *)tmpInfo->tmpDataInfo->decordedData, charData);
    }
    else {
        strcat((char *)tmpInfo->tmpDataInfo->decordedData, charData);
    }

    tmpInfo->tmpDataInfo->charcount ++;
    tmpInfo->isSpecial = 0;
    tmpInfo->isGetChar = 1;
}


/*!
*******************************************************************************
** \brief Get Control Sequence Introducer parameter.
**        When intermediate character is 0x20 only.
**
** \return Nothing
**
** \sa
******************************************************************************/
static void CC_GetCSI1(uint8_t *CSI_param, uint8_t *param1, uint32_t *num1)
{
    uint32_t i;

    for (i = 0; i < tmpInfo->byteNumCSI; i++) {
        if (CSI_param[i] == 0x20) {
            *num1 = i;
        }
    }
    for (i = 0; i < *num1; i++) {
        param1[i] = CSI_param[i];
    }

    tmpInfo->byteNumCSI = 0;
    tmpInfo->isCSI = 0;
}

/*!
*******************************************************************************
** \brief Get Control Sequence Introducer parameter.
**        When intermediate character is 0x3B and 0x20.
**
** \return Nothing
**
** \sa
******************************************************************************/
static void CC_GetCSI2(uint8_t *CSI_param, uint8_t *param1, uint8_t *param2, uint32_t *num1, uint32_t *num2)
{
    uint32_t i;

    for (i = 0; i < tmpInfo->byteNumCSI; i++) {
        if (CSI_param[i] == 0x3B) {
            *num1 = i;
        }
        else if (CSI_param[i] == 0x20) {
            *num2 = i;
        }
    }

    memcpy(param1, CSI_param, *num1);
    memcpy(param2, CSI_param+(*num1)+1, (*num2) -(*num1)-1);
    *num2 = (*num2) -(*num1)-1;

    tmpInfo->byteNumCSI = 0;
    tmpInfo->isCSI = 0;

}

/*!
*******************************************************************************
** \brief  Detect control code.
**
** \return Nothing
**
** \sa
******************************************************************************/
static void CC_ControlCodeChange(uint8_t *flag, uint8_t *isFlag, uint8_t *mem, uint8_t value)
{


    if (!tmpInfo->isGetChar) {
        if (flag != NULL) {
            *flag = 1;
        }
        if (isFlag != NULL) {
            *isFlag = 1;
        }
        if (mem != NULL && value != 0) {
            *mem = value;
        }
    }
    else {
        tmpInfo->isGetChar = 0;
        tmpInfo->fNewControlCode = 1;
    }

}

