/******************************************************************************
**
** \file      M88DS3K.c  (version matching to V 2.08.07)
**
** \brief     Front-end device driver for M88DS3K frontend.
**
**            (C) Fujitsu Microelectronics Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   $Id: m88ds3k.c 6060 2011-04-06 12:29:48Z ebende $
**
******************************************************************************/
/*lint -save -e713 -e732 */
#include "../src/fe_priv.h"

#ifdef _LINUX_FAPI
#include <fapex/abst_types.h>
#else
#include <fapi/drv_gpio.h>
#include <fapi/sys_services.h>
#endif
/* FE specific includes */
#include "mt_fe_def.h"
#include "m88ds3k.h"

#if 0
 #include <boards/boards_common.h>
 #include "btestlib/btest.h"
 #define LOG_DIAG(a,b,c)  BTLIB_DIA_LogDiagnosticInfo(a,b,c)
 #define NO_LF  (FAPI_BOARD_DIAG_FORM_NO_LF<<FAPI_BOARD_DIAG_FORM_N_SHIFTL)
#else
 #define LOG_DIAG(a,b,c)
#endif

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
static feHandleT *pData[FE_MAX_NUM];
extern int32_t i2cHandle;
extern uint8_t   i2cAddr;
extern uint8_t   feIndex;
#define SET_I2C  i2cHandle = pData[index]->i2cId; \
                 i2cAddr   = pData[index]->i2cAddrDemod; \
                 feIndex   = index

static MT_FE_BOOL        curr22kHz[FE_MAX_NUM]    = {MtFe_True,MtFe_True,MtFe_True,MtFe_True};
static U8                envelop_mode[FE_MAX_NUM] = {0,0,0,0};
static MT_FE_LNB_VOLTAGE currMtFeLNB[FE_MAX_NUM]  = {MtFeLNB_13V,MtFeLNB_13V,MtFeLNB_13V,MtFeLNB_13V};
static MT_FE_BOOL        currMtFeLNBenabled[FE_MAX_NUM] = {MtFe_True, MtFe_True, MtFe_True,MtFe_True};
//FE_FeHwSpecificCfgDataT *hwSpecFlgasPtr;

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
static int32_t M88DS3K_Open(uint8_t index, FE_HANDLE *pDeviceData);
static int32_t M88DS3K_Close(uint8_t index);
static int32_t M88DS3K_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
static int32_t M88DS3K_GetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
static int32_t M88DS3K_Reset(uint8_t index);
static int32_t M88DS3K_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
static int32_t M88DS3K_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
static int32_t M88DS3K_GetSpecialInfos(uint8_t index, uint32_t whichInfoCmd, void *resultDstPtr);
static int32_t M88DS3K_GetAFC(uint8_t index, int32_t* pAFC);
static int32_t M88DS3K_GetBER(uint8_t index, uint32_t* pBER);
static int32_t M88DS3K_GetSignalQuality(uint8_t index, uint16_t* pQuality);
static int32_t M88DS3K_GetSignalStrength(uint8_t index, uint16_t* pStrength);
static int32_t M88DS3K_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);
static int32_t M88DS3K_SendDiseqcCommand(uint8_t index,uint8_t len,uint8_t *pCommand,
                                         uint8_t *pRetLen,uint8_t *pRetBuf);
static int32_t M88DS3K_Control22kHz(uint8_t index, FE_22KHZ_E mode);
static int32_t M88DS3K_SetAntennaPower(uint8_t index, uint8_t antPower);
static int32_t M88DS3K_ControlLnbVoltage(uint8_t index, FE_LNB_VOLTAGE_E mode);

static int32_t m88ds3kCheckChipId(feHandleT *hp, uint8_t i2cAddress);
static void    m88ds3kSetFunctions(feHandleT *hp);

#ifdef  ALLOW_SCAN
  static void    m88ds3k_callback(MT_FE_MSG msg, void *p_tp_info);
  static int32_t M88DS3K_Scan(uint8_t index, FE_SCAN_PARAMS_S *pScanParams);
#endif

/************************
****external function****
*************************/
//extern void _mt_sleep(U32 ms);



/*!
*******************************************************************************
**
** \brief Check chip ID.
** \param hp - FE handle for current FE front-end.
** \param i2cAddress  - i2c address for demod.
**
*******************************************************************************
*/
static int32_t m88ds3kCheckChipId(feHandleT *hp, uint8_t i2cAddress)
{
    uint8_t i2cBuffer[2];
    int32_t error;
    
    i2cBuffer[0] = 00;
    i2cBuffer[1] = 00;

    error = FE_I2C_Read(hp->i2cId, i2cAddress+1, &(i2cBuffer[0]), 1,
                                                    &(i2cBuffer[1]), 1);
    FAPI_SYS_PRINT_MSG("[M88DS3K] Checking for M88DS3K v2.03 on address %02x error %x read %x\n", i2cAddress, error, i2cBuffer[1]);

    if(error != FE_OK)
    {
        return error;
    }
    
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Set callback function for the current FE handle.
** \param hp - FE handle
**
*******************************************************************************
*/
static void m88ds3kSetFunctions(feHandleT *hp)
{
    hp->systemType             = FE_SYSTEM_TYPE_SATELLITE;
    hp->feOpenFct              = M88DS3K_Open;
    hp->feCloseFct             = M88DS3K_Close;
    hp->feSetSystemParamsFct   = M88DS3K_SetSystemParams;
    hp->feGetSystemParamsFct   = M88DS3K_GetSystemParams;
    hp->feResetFct             = M88DS3K_Reset;
    hp->feTuneFct              = M88DS3K_Tune;
#ifdef ALLOW_SCAN
    hp->feScanFct              = M88DS3K_Scan;
#else
    hp->feScanFct              = NULL;
#endif
    hp->feSendDiseqcCommandFct = M88DS3K_SendDiseqcCommand;
    hp->feGetStatusFct         = M88DS3K_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = M88DS3K_GetAFC;
    hp->feGetBERFct            = M88DS3K_GetBER;
    hp->feGetSignalQualityFct  = M88DS3K_GetSignalQuality;
    hp->feGetSignalStrengthFct = M88DS3K_GetSignalStrength;
    hp->feGetTuneParamsFct     = M88DS3K_GetTuneParams;
    hp->feSetAntennaPowerFct   = M88DS3K_SetAntennaPower;
    hp->feControl22kHz         = M88DS3K_Control22kHz;
    hp->feControlLnbVoltage    = M88DS3K_ControlLnbVoltage;
    hp->isPresent              = 1;
    //hp->feGetSpecificStatusFct = M88DS3K_GetSpecialInfos;
}

/*!
*******************************************************************************
**
** \brief Detect demod.
**
** \param slot  slot number
** \param hp    FE handle
** \note  up to now, this function does not really 'auto-detect' the 'tuner'
**        -> future enhancement
*******************************************************************************
*/
int32_t M88DS3K_Detect( uint8_t slot, feHandleT *hp)
{
/* from M88DS3000 Datasheet:
    ADDR_SEL[1:0]   I2C Addr
       00            0xD0
       01            0xD2
       10            0xD4
       11            0xD6 */
    if (slot == 0)
    {
        hp->i2cAddrDemod  = 0xD0;
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod  = 0xD4;
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }
    
    if(m88ds3kCheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    m88ds3kSetFunctions(hp);
    hp->i2cAddrTuner = 0xC0;
    hp->slotNum  = slot;
    hp->tunerType    = 0;

	actMntTunTpy = MT_FE_TN_SUPPORT_SHARP7306;

    return FE_OK;
}
int32_t M88DS3K_TS2020_Detect( uint8_t slot, feHandleT *hp)
{
    return M88DS3K_Detect(slot, hp);
}

int32_t M88DS3K_HE6306_Detect( uint8_t slot, feHandleT *hp)
{
    int32_t ret;
    
    ret = M88DS3K_Detect(slot, hp);
    if(ret != FE_OK)
    {
        return ret;
    }
    hp->tunerType = 1;
    
    return ret;
}
/*!
*******************************************************************************
**
** \brief Open M88DS3K demod.
** \param index - slot number
** \param pDeviceData  - An pointer point to the current FE handle.
**
*******************************************************************************
*/
static int32_t M88DS3K_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    MT_FE_RET error;

    if ( ( index >= FE_MAX_NUM ) ||
         ( pDeviceData == NULL ) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    pData[index]=&feHandleArray[index];
    *pDeviceData=(FE_HANDLE)(&feHandleArray[index]);

    SET_I2C;

    (void)M88DS3K_Reset(index);

    error = mt_fe_dmd_ds3k_init();
    if(error != MtFeErr_Ok)
    {
        return FE_ERR_BAD_PARAMETER;
    }

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Close demod.
**
** \param index     slot number.
**
*******************************************************************************
*/
static int32_t M88DS3K_Close(uint8_t index)
{
    if( index >= FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    // TODO
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Set systemParams to the FE handle defined by index..
**
** \param index             index of FE
** \param pSystemParams     pointer to system params.
**
*******************************************************************************
*/
static int32_t M88DS3K_SetSystemParams(uint8_t index,
                                FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if( (index >= FE_MAX_NUM) ||
        (pSystemParams == NULL) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    pData[index]->pSystemParams = *pSystemParams;

    return FE_OK;
}


/*!
*******************************************************************************
**
** \brief Get systemParams from the FE defined by index.
**
** \param index             index of FE
** \param pSystemParams     pointer to write system params to.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetSystemParams(uint8_t index,
                                FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if( (index >= FE_MAX_NUM) ||
        (pSystemParams == NULL) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    *pSystemParams = pData[index]->pSystemParams;

    return FE_OK;
}


/*!
*******************************************************************************
**
** \brief Reset demod.
**
** \param index     index of FE
**
*******************************************************************************
*/
static int32_t M88DS3K_Reset(uint8_t index)
{
    int32_t error;

    if (index >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }

    error = FE_ControlFeReset(index, FE_RESET_PULSE);
    if (error != FE_OK)
    {
        return error;
    }

    /* restore the previousely use output setting */
    (void) mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                  curr22kHz[index],
                                  currMtFeLNB[index],
                                  envelop_mode[index]);

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Tune to TP given by pTuneParams.
**        1. Initialize demod .
**        2. Set RF parameter.
**        3. Set Bandwidth.
**        4. channel setting
**        5. Demod lock and MPEG lock.
**
** \param index         index of FE
** \param pTuneParams   tune parameter.
**
*******************************************************************************
*/
static int32_t M88DS3K_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    FE_SYSTEM_PARAMS_S*     inst;
    uint32_t                ulIFFreq;
    uint32_t                LNBlowfreq;
    uint32_t                LNBhighfreq;
    MT_FE_RET               error;
//    uint64_t                uiTimeth;
    uint32_t                freq_MHz;
    uint32_t                sym_rate_KSs;
    MT_FE_TYPE              dvbs_type;
    int32_t                 fapiErr;

    if( ( index >= FE_MAX_NUM ) ||
        ( pData[index] == NULL ) ||
        ( pTuneParams  == NULL ) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    LOG_DIAG(0, "M88DS3K_Tune  TP: %06X ", pTuneParams->sat.frequency);
    if (pTuneParams->sat.pol == FE_POL_HOR) {
        LOG_DIAG(NO_LF, "H", 0); }
    else {
        LOG_DIAG(NO_LF, "V", 0); }
    LOG_DIAG(NO_LF, "%06X <<", pTuneParams->sat.symbolRate);
    if (pTuneParams->sat.modulation == FE_MOD_QPSK_S2) {
        LOG_DIAG(NO_LF, " S2", 0); }
    else {
        LOG_DIAG(NO_LF, " S", 0); }

#if 0
    BTLIB_IO_PrStrDezStr("\nSet TP ",(int32_t) pTuneParams->sat.frequency, 0);
    if (pTuneParams->sat.pol == FE_POL_HOR) BTLIB_IO_PrStr(" H ");
    else                                    BTLIB_IO_PrStr(" V ");
    BTLIB_IO_PrStrDezStr(0,(int32_t) pTuneParams->sat.symbolRate, 0);
    if (pTuneParams->sat.modulation == FE_MOD_QPSK_S2) BTLIB_IO_PrStr(" S2 ");
    else                                               BTLIB_IO_PrStr(" S ");
    BTLIB_IO_PrStrDezStr(" id: ",(int32_t) pTuneParams->sat.satID, " ");
#endif

    SET_I2C;

    inst        = &pData[index]->pSystemParams;
    LNBlowfreq  = inst->sat.lowFrequency;
    LNBhighfreq = inst->sat.highFrequency;

    if (LNBlowfreq  == 0)
    {
        LNBlowfreq  = LNBhighfreq;
    }
    if (LNBhighfreq == 0)
    {
        LNBhighfreq = LNBlowfreq;
    }
    fapiErr = FE_SetSignalRoute(pData[index], pTuneParams);
    if(fapiErr!=FE_OK)
    {
        LOG_DIAG(NO_LF, "{SigRouteErr}", 0);
        return FE_ERR_FE_TUNE;
    }

    if( inst->sat.diseqc != FE_DISEQC_1_1_SMATV)
    {
        if(labs(pTuneParams->sat.frequency - LNBlowfreq) < FE_BAND_SW)
        {
            /* low band ok */
            ulIFFreq = labs(pTuneParams->sat.frequency - LNBlowfreq);
        }
        else
        {
            /* using high band */
            ulIFFreq = labs(pTuneParams->sat.frequency - LNBhighfreq);
        }
    }
    else
    {
        ulIFFreq = labs((pTuneParams->sat.ifFrequency&0x0000FFFF) * 100);
    }

    if ( (ulIFFreq < FE_MIN_FREQ) ||
         (ulIFFreq > FE_MAX_FREQ) )
    {
        return(FE_ERR_BAD_PARAMETER);
    }
    if ( (pTuneParams->sat.symbolRate < FE_MIN_SYM) ||
         (pTuneParams->sat.symbolRate > FE_MAX_SYM) )
    {
        LOG_DIAG(NO_LF, "{BadParam}", 0);
        return(FE_ERR_BAD_PARAMETER);
    }
	
	
    freq_MHz     = ulIFFreq/1000;
//    BTLIB_IO_PrStrDezStr(" (o)IFF:",(int32_t) freq_MHz, 0);
    LOG_DIAG(0, ">> Tune IF:%08X <<", ulIFFreq);

    sym_rate_KSs = pTuneParams->sat.symbolRate;
    if(pTuneParams->sat.modulation == FE_MOD_QPSK)
    {
        dvbs_type = MtFeType_DvbS;
    }
    else
    {
        dvbs_type = MtFeType_DvbS2;
    }
	printf("freq %dmhz\n",freq_MHz);
    error = mt_fe_dmd_ds3k_connect(freq_MHz, sym_rate_KSs, dvbs_type);
    if(error != MtFeErr_Ok)
    {
        return FE_ERR_FE_TUNE;
    }
    if( inst->sat.diseqc == FE_DISEQC_1_1_SMATV)
    {
    //    FE_Delay(150);
         _mt_sleep(150);
        if(pData[index]->feControlLnbVoltage != NULL)
        {
            fapiErr = pData[index]->feControlLnbVoltage(pData[index]->slotNum,FE_LNB_13V);
            if(fapiErr != FE_OK)
            {
                return fapiErr;
            }
        }
        FE_SetTuneTimeout(index, 0); // set max tune timeout
    }
    else
    {
        FE_SetTuneTimeout(index, pTuneParams->sat.symbolRate);
    }

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Get Demod status.
**
** \param index     index of FE
** \param pStatus   store the status value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    MT_FE_LOCK_STATE lock_state = MtFeLockState_Undef;
    
    if( (index >= FE_MAX_NUM) ||
        (pStatus == NULL) )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    SET_I2C;
    
    (void)mt_fe_dmd_ds3k_get_lock_state(&lock_state);
    if(lock_state == MtFeLockState_Locked)
    {
        *pStatus = FE_STATUS_LOCKED;
    }
    else
    {
        if( FE_IsTuneTimeout(index) == 0 )
        {
            *pStatus = FE_STATUS_TUNING;
        }
        else
        {
            *pStatus = FE_STATUS_NOT_LOCKED;
        }
    }
    return FE_OK;
}
/*!
*******************************************************************************
**
** \brief This function get current AFC value
**
** \param index     index of FE
** \param pAFC      pointer to write AFC value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetAFC(uint8_t index, int32_t* pAFC)
{
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief This function get current BER value
**
** \param index     index of FE
** \param pBER      pointer to write BER value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetBER(uint8_t index, uint32_t* pBER)
{
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Get signal Quality
**
** \param index     index of FE
** \param pQuality  pointer to write quality value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    int8_t p_Snr;
    MT_FE_LOCK_STATE lock_state = MtFeLockState_Undef;


    if( (index >= FE_MAX_NUM) ||
        (pQuality == NULL) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    SET_I2C;

    *pQuality = 0;
    (void)mt_fe_dmd_ds3k_get_lock_state(&lock_state);
    if(lock_state == MtFeLockState_Locked)
    {
        /* returns -19..19 for S2 and 0..13 for QPSK */
        (void)mt_fe_dmd_ds3k_get_snr(&p_Snr);
        *pQuality = (p_Snr+19)*1724;
    }

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Get signale strength value.
**
** \param index     index of FE
** \param pStrength pointer to write strength value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    MT_FE_LOCK_STATE lock_state = MtFeLockState_Undef;
    int8_t p_Str;

    if(index >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    if(pStrength == NULL)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    *pStrength = 0;

    SET_I2C;

    (void)mt_fe_dmd_ds3k_get_lock_state(&lock_state);
    if(lock_state != MtFeLockState_Locked)
    {
        return FE_ERR_FE_NOT_LOCKED;
    }

    (void)mt_fe_dmd_ds3k_get_strength(&p_Str);
    *pStrength = p_Str*655;

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Get tune parameters.
**
** \param index         index of FE
** \param pTuneParams   pointer to write TuneParams value.
**
*******************************************************************************
*/
static int32_t M88DS3K_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{
    if(index >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    pTuneParams = pTuneParams;
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief Sending a DiSEqC command
**
** \param index         index of FE
** \param len           command lenght
** \param pCommand      command buffer
** \param pRetLen       receive lenght (optional)
** \param pRetBuf       receive buffer (optional)
**
*******************************************************************************
*/
static int32_t M88DS3K_SendDiseqcCommand(uint8_t index,uint8_t len,uint8_t*pCommand,
                                         uint8_t *pRetLen,uint8_t *pRetBuf)
{
    int32_t error = FE_OK;
    MT_FE_DiSEqC_MSG    msg;
    int32_t             i;

    if( (index >= FE_MAX_NUM) ||
        (pCommand == NULL) )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    SET_I2C;

    msg.size_send = (len > 8) ? 8 : len;
    for(i=0;i<msg.size_send;i++)
    {
        msg.data_send[i] = pCommand[i];
    }
    msg.is_enable_receive = pRetLen ? 1 : 0;
    (void)mt_fe_dmd_ds3k_DiSEqC_send_receive_msg( &msg);
    if(pRetLen && pRetBuf)
    {
        *pRetLen = msg.size_receive;
        for(i=0;i<*pRetLen;i++)
        {
            pRetBuf[i] = msg.data_receive[i];
        }
    }

    /* restore the previousely use output setting */
    error = mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                   curr22kHz[index],
                                   currMtFeLNB[index],
                                  envelop_mode[index]);

    return error;
}

int32_t M88DS3K_Control22kHz(uint8_t index, FE_22KHZ_E mode)
{
    int32_t error = FE_OK;

    SET_I2C;

    switch(mode)
    {
        case FE_22KHZ_OFF:
            curr22kHz[index] = MtFe_False;
            error = mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                           curr22kHz[index],
                                           currMtFeLNB[index],
                                           envelop_mode[index]);
            break;
        case FE_22KHZ_ON:
            curr22kHz[index] = MtFe_True;
            error = mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                           curr22kHz[index],
                                           currMtFeLNB[index],
                                           envelop_mode[index]);
            break;
        case FE_22KHZ_TONE_0:
            error = mt_fe_dmd_ds3k_DiSEqC_send_tone_burst(
                                                MtFeDiSEqCToneBurst_Unmoulated, envelop_mode[index]);
            break;
        case FE_22KHZ_TONE_1:
            error = mt_fe_dmd_ds3k_DiSEqC_send_tone_burst(
                                                MtFeDiSEqCToneBurst_Moulated, envelop_mode[index]);
            break;
        default:
            error = FE_ERR_FEATURE_NOT_SUPPORTED;
    }
    return error;
}

int32_t M88DS3K_SetAntennaPower(uint8_t index, uint8_t antPower)
{
    SET_I2C;

//    BTLIB_IO_PrStrHexStr(5, "\n[M88DS3K_SetAntennaPower] set LNB.Enable to :", antPower, 0);

    if( 0 == antPower)
    {
        currMtFeLNBenabled[index] = MtFe_False;
    }
    else
    {
        currMtFeLNBenabled[index] = MtFe_True;
    }

//  FE_ControlAntPow(index, antPower);  // call this 'as well' in case HW-Platform uses 'other solution'

    return mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                  curr22kHz[index],
                                  currMtFeLNB[index],
                                  envelop_mode[index]);
}

int32_t M88DS3K_ControlLnbVoltage(uint8_t index, FE_LNB_VOLTAGE_E mode)
{
    SET_I2C;

    switch(mode)
    {
        case FE_LNB_13V:
            currMtFeLNB[index] = MtFeLNB_13V;
            break;
        case FE_LNB_18V:
            currMtFeLNB[index] = MtFeLNB_18V;
            break;
        default:
            return FE_ERR_FEATURE_NOT_SUPPORTED;
    }
    return mt_fe_dmd_ds3k_set_LNB(currMtFeLNBenabled[index],
                                  curr22kHz[index],
                                  currMtFeLNB[index],
                                  envelop_mode[index]);
}


#ifdef ALLOW_SCAN
static FE_SCAN_PARAMS_S blindScanParams;
static void m88ds3k_callback(MT_FE_MSG msg, void *p_tp_info)
{
    FE_TUNE_PARAMS_S    foundTuneParams;
    MT_FE_TP_INFO       *tp_info;

    if(blindScanParams.feScanAbortFct != NULL)
    {
        if(blindScanParams.feScanAbortFct() == 1)
        {
            mt_fe_dmd_ds3k_blindscan_abort(MtFe_True);
            return;
        }
    }
    switch( msg )
    {
        case MtFeMsg_BSAbort:
            break;
        case MtFeMsg_BSTpLocked:
        case MtFeMsg_BSTpUnlock:
        case MtFeMsg_BSStart:
        case MtFeMsg_BSFinish:
        case MtFeMsg_BSOneWinFinish:
            if(blindScanParams.feScanProgressFct != NULL)
            {
                blindScanParams.feScanProgressFct(2);
            }
            break;
        case MtFeMsg_BSTpFind:
            if( p_tp_info != NULL )
            {
                tp_info = (MT_FE_TP_INFO*)p_tp_info;
                if(blindScanParams.feScanNotifyFct != NULL)
                {
                    memset(&foundTuneParams, 0,sizeof(FE_TUNE_PARAMS_S));
                    foundTuneParams.sat.frequency   = tp_info->freq_KHz;
                    foundTuneParams.sat.frequency +=  500;
                    foundTuneParams.sat.frequency /= 1000;
                    foundTuneParams.sat.frequency *= 1000;
                    foundTuneParams.sat.symbolRate  = tp_info->sym_rate_KSs;
                    if(foundTuneParams.sat.symbolRate > 10000)
                    {
                        foundTuneParams.sat.symbolRate += 50;
                        foundTuneParams.sat.symbolRate /= 100;
                        foundTuneParams.sat.symbolRate *= 100;
                    }
                    foundTuneParams.sat.pol         = blindScanParams.pol;
                    foundTuneParams.sat.satID       = blindScanParams.satID;
                    switch(tp_info->dvb_type)
                    {
                        case MtFeType_DvbS:
                            foundTuneParams.sat.modulation = FE_MOD_QPSK;
                            break;
                        case MtFeType_DvbS2:
                            foundTuneParams.sat.modulation = FE_MOD_QPSK_S2;
                            break;
                        default:
                            foundTuneParams.sat.modulation = FE_MOD_NONE;
                            break;
                    }
                    foundTuneParams.sat.iq = FE_IQ_AUTO;
                    FAPI_SYS_PRINT_MSG("to appl: Freq=%5d MHz Sym=%5d k\n",
                                                    foundTuneParams.sat.frequency/1000,
                                                    foundTuneParams.sat.symbolRate);

                    blindScanParams.feScanNotifyFct(&foundTuneParams);
                }
            }
        break;
    }
}


/*
*******************************************************************************
**
** \brief   BlindScan
**
** \note    Function is NOT tested !!!
**
** \param   index           Unused here.
** \param   pScanParams     Scan params.
**
*******************************************************************************
*/
static int32_t M88DS3K_Scan(uint8_t index, FE_SCAN_PARAMS_S *pScanParams)
{
    MT_FE_BS_TP_INFO            bsPara;
    int32_t                     error = FE_OK;
    FE_TUNE_PARAMS_S            tuneParams;
    FE_SYSTEM_PARAMS_S*         inst;
    uint32_t                    startFrequency;
    uint32_t                    stopFrequency;

    SET_I2C;

    if( (&pData[index] == NULL) ||
        (pScanParams   == NULL) ||
        (pScanParams->startSymbolRate <  BLINDSCAN_MIN_SYMB) ||
        (pScanParams->stopSymbolRate  >  BLINDSCAN_MAX_SYMB)
      )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    blindScanParams = *pScanParams;

    inst = &pData[index]->pSystemParams;
    startFrequency  = pScanParams->startFrequency - inst->sat.lowFrequency;
    startFrequency /= 1000;
    stopFrequency   = pScanParams->stopFrequency  - inst->sat.highFrequency;
    stopFrequency  /= 1000;

    if( (startFrequency  <  (BLINDSCAN_MIN_FREQ/1000)) ||
        (stopFrequency   >  (BLINDSCAN_MAX_FREQ/1000)) )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    memset(&tuneParams, 0 ,sizeof(tuneParams));
    tuneParams.sat.pol       = pScanParams->pol;
    tuneParams.sat.satID     = pScanParams->satID;

    /*Detail Scan: times = 3; Normal Scan: times = 1;*/
    /*Segment Lock:    algorithm = MT_FE_BS_ALGORITHM_A;
      Whole Lock Mode: algorithm = MT_FE_BS_ALGORITHM_B;*/
    bsPara.bs_times     = 1;
    bsPara.bs_algorithm = MT_FE_BS_ALGORITHM_B;

    FAPI_SYS_PRINT_MSG("Start to scan \n");
    /*register callback function to do something(parse TS stream, eg.)*/
    mt_fe_dmd_ds3k_register_notify(m88ds3k_callback);

    mt_fe_dmd_ds3k_blindscan(startFrequency, 1950, &bsPara);

    tuneParams.sat.frequency = pScanParams->stopFrequency;
    error = FE_SetSignalRoute(pData[index],&tuneParams);
    if(error != FE_OK)
    {
        return error;
    }
    if(pScanParams->feScanProgressFct != NULL)
    {
        pScanParams->feScanProgressFct(500);
    }
    mt_fe_dmd_ds3k_blindscan(1100, stopFrequency, &bsPara);

    return FE_OK;
}
#endif
