/****************************************************************************/
/*                   MONTAGE PROPRIETARY AND CONFIDENTIAL                   */
/*                   Montage Technology (Shanghai) Inc.                     */
/*                          All Rights Reserved                             */
/****************************************************************************/
/*
 * Filename:      mt_fe_tn_montage_he6306.c
 *
****************************************************************************/


#include "mt_fe_def.h"
#include "../src/fe_priv.h"
#if (MT_FE_TN_SUPPORT_HE6306 > 0)

#include "mt_fe_i2c.h"


extern void _mt_sleep(U32 ms);

static unsigned char init_data1[] = { 0x44, 0x7e, 0xe1, 0x42 };
static unsigned char init_data2[] = { 0xe5 };
static unsigned char init_data3[] = { 0xfd, 0x0d };




/*****************************************************************
** Function: mt_fe_tn_init_ts2020
**
**
** Discription:    This function is used to initial he6306. 
**
**
** Inputs:   None
**
** Outputs:  None
**
******************************************************************/
MT_FE_RET mt_fe_tn_init_he6306(void)
{ 
	U8  buf;
	MT_FE_RET err=0;
	printf("## mt_fe_tn_init_he6306() ##\n");
	err =  _mt_fe_tn_write_regs(sizeof(init_data1), init_data1);
	if(err){
		FE_DPrintf("nim_hz6306_init: init_data1 I2C write error\n");
	}
	
	err =  _mt_fe_tn_write_regs(sizeof(init_data2), init_data2);
	if(err){
		FE_DPrintf("nim_hz6306_init: init_data2 I2C write error\n");
	}
	FE_Wait(10);
	err =  _mt_fe_tn_write_regs(sizeof(init_data3), init_data3);
	if(err){
		FE_DPrintf("nim_hz6306_init: I2C write error\n");
	}

	return err;
}




/*************************************************************************
** Function: mt_fe_tn_get_tuner_freq_offset_ts2020
**
**
** Discription:	return the tuner freq. offset, unit: KHz
**
**
** Inputs: none
**
**
** Outputs:	none
**
**
*************************************************************************/
S32 mt_fe_tn_get_tuner_freq_offset_he6306(void)
{
	return 0;
}



/*****************************************************************
** Function: mt_fe_tn_get_strength_ts2020
**
**
** Discription:    This function is used to get signal strength. 
**
**
** Inputs:   None
**
** Outputs:
**
**	 Parametre		type			discription
**	----------------------------------------------------------
**	 p_strength    	S8*				strength value pointer
**
**
******************************************************************/
MT_FE_RET mt_fe_tn_get_strength_he6306(S8 *p_strength)
{ 
	#if 1
	*p_strength =13;
	#else
	U8 gain1, gain2;
	S8 strength;
	U16 gain;
	MT_FE_LOCK_STATE lockstate;
	
	
	_mt_fe_tn_get_reg(0x3d, &gain1);
	_mt_fe_tn_get_reg(0x21, &gain2);

	gain1 &= 0x1f;
	gain2 &= 0x1f;
	
	if((S8)gain1<0)     gain1 = 0;
	if(gain1>15)    gain1 = 15;
	if((S8)gain2<0 )	gain2 = 0;
	if(gain2>13)	gain2 = 13;

	gain = gain1*2 + gain2*3;

	if(gain>64)		gain = 64;
	if((S16)gain<0)      gain = 0;

	strength = 40 + (64 - gain) * 50 / 64;   // Strength: 40% -90% 
	// Cable unconnected
	mt_fe_dmd_ds3k_get_lock_state(&lockstate);
	if(((lockstate == MtFeLockState_Waiting)||(lockstate == MtFeLockState_Unlocked))&& (gain > 50))
		 strength = 0; 

	
	if(strength>100)  strength = 100;
	if(strength<0)    strength = 0;

	
	*p_strength = strength;
	#endif
	return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_init_ts2020
**
**
** Discription:    This function is used to initial ts2020. 
**
**
** Input:
**
**	Parametre		type			discription
**	----------------------------------------------------------
**	 freq_MHz    	U32				from 950 to 2150
**	 sym_rate_KSs   U32       		from 1000 to 45000
**
**
** Outputs:  	None
**	
**
**
******************************************************************/
MT_FE_RET mt_fe_tn_set_freq_he6306(U32 freq_MHz, U32 sym_rate_KSs, U32 lpf_offset_MHz)
{
	
	
	U8 data[4];
	U16 Npro,tmp;
	U32 Rs, BW;
	U8 Nswa;
	U8 LPF = 15;
	U8 BA = 1;
	U8 DIV = 0;
	S32 result;
	U32 freq = freq_MHz;
	//struct QPSK_TUNER_CONFIG_EXT * hz6306_ptr = NULL;
	//if(tuner_id>=hz6306_tuner_cnt||tuner_id>=MAX_TUNER_SUPPORT_NUM)
	//	return ERR_FAILUE;
	//hz6306_ptr = hz6306_dev_id[tuner_id];
#if 1
	U32 ratio = 135;	// 125, 120
#endif
	
	/* LPF cut_off */
	Rs = sym_rate_KSs;
	if (Rs == 0)
		Rs = 45000;
#if 1
	BW = Rs*135/200;                // rolloff factor is 35%
	BW = BW*130/100;                // 0.1dB to 3dB need about 30% transition band for 7th order LPF
	if (Rs < 6500)
		BW = BW + 3000;				// add 3M when Rs<5M, since we need shift 3M to avoid DC
	BW = BW + 2000;                 // add 2M for LNB frequency shifting
	BW = BW*108/100;                // add 8% margin since fc is not very accurate
#else
	if (ratio == 0)
		BW = 34000;
	else
		BW = Rs * ratio / 100;		// ???
#endif
	if (BW < 10000)
		BW = 10000;					// Sharp6036 LPF can be tuned form 10M to 34M, step is 2.0M
	if (BW > 34000)
		BW = 34000;
	
	if (BW <= 10000)
		LPF = 3;
	else if (BW <= 12000)
		LPF = 4;
	else if (BW <= 14000)
		LPF = 5;
	else if (BW <= 16000)
		LPF = 6;
	else if (BW <= 18000)
		LPF = 7;
	else if (BW <= 20000)
		LPF = 8;
	else if (BW <= 22000)
		LPF = 9;
	else if (BW <= 24000)
		LPF = 10;
	else if (BW <= 26000)
		LPF = 11;
	else if (BW <= 28000)
		LPF = 12;
	else if (BW <= 30000)
		LPF = 13;
	else if (BW <= 32000)
		LPF = 14;
	else
		LPF = 15;

	FE_DPrintf ("nim_hz6306_control: sym = %d, BW  = %d, LPF = %d\n", sym_rate_KSs, BW, LPF);
	
	/* local oscillator select */

	if (freq <= 1154)
		DIV = 1;
	else
		DIV = 0;

	if (freq <= 986)
		BA = 5;
	else if (freq <= 1073)
		BA = 6;
	else if (freq <= 1154)
		BA = 7;
	else if (freq <= 1291)
		BA = 1;
	else if (freq <= 1447)
		BA = 2;
	else if (freq <= 1615)
		BA = 3;
	else if (freq <= 1791)
		BA= 4;
	else if (freq <= 1972)
		BA= 5;
	else //if (freq <= 2150)
		BA = 6;

	FE_DPrintf ("nim_hz6306_control: freq = %d, BA  = %d, DIV = %d\n", freq, BA, DIV);

	tmp = freq * 1000 * 8 / REF_OSC_FREQ;	// REF = 1
	Nswa = tmp % 32;						// PSC = 0
	Npro = tmp / 32;
	FE_DPrintf ("nim_hz6306_control: Npro = %d, Nswa = %d\n", Npro, Nswa);
	
	data[0] = (U8) ((Npro >> 3) & 0x7F);		// BG1:0 = 00
	data[1] = Nswa | (((U8)Npro & 0x07) << 5);
	data[2] = 0xE1;								// C1:0 = 11, TM = 0
	data[3] = (BA<<5) | (DIV<<1);
	if ((result = _mt_fe_tn_write_regs(4,data+0)) != MtFeErr_Ok)
	{
		FE_DPrintf("nim_hz6306_control: I2C write error\n");
		return result;
	}
	
	data[2] = 0xE5;								// TM = 1
	if ((result = _mt_fe_tn_write_regs(1, data+2)) != MtFeErr_Ok)
		return result;
	FE_Wait(10);
	
	data[2] |= ((LPF & 0x01) << 4) | ((LPF & 0x02) << 2);
	data[3] |= ((LPF & 0x04) << 1) | ((LPF & 0x08) >> 1);
	if ((result = _mt_fe_tn_write_regs(2, data+2)) != MtFeErr_Ok)
		return result;

	
	//FE_DPrintf("[m88ds3k] mt_fe_tn_set_freq_ts2020 err(%d)\n",err);
	return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_sleep_ts2020
**
**
** Discription:    This function is used to set tuner in sleep mode. 
**
**
** Inputs:   None
**
** Outputs:	 None
**
**
******************************************************************/
MT_FE_RET mt_fe_tn_sleep_he6306(void)
{
	#if 1
	
	#else
	_mt_fe_tn_set_reg(0x00, 0x00); 
	#endif
	
	return MtFeErr_Ok;
}



#endif/*#if (MT_FE_TN_SUPPORT_TS2020 > 0)*/

