/******************************************************************
*	Copyright (c) 2008, MONTAGE PROPRIETARY AND CONFIDENTIAL
*	Montage Technology (Shanghai) Inc.
*	All Rights Reserved
*	------------------------------------
*
*	FILE:				mt_fe_tn_serit_sp761vb.c
*
*	VERSION:			2.02.00
*
*	DESCRIPTION:
*			Define the interfaces to set Serit sp761vb tuner.
*
* 	History:
*	Description     Version   	Date 	       Author
*------------------------------------------------------------------
*   Create			2.02.00		2009.05.20		
*******************************************************************/

#include "mt_fe_def.h"
#include "mt_fe_i2c.h"

extern void _mt_sleep(U32 ms);


#if (MT_FE_TN_SUPPORT_SERIT_SP761VB > 0)
	#define MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB 16000 /* 16 MHz reference (KHz) */

	#define LSB(X) ((X & 0xFF))
	#define MSB(Y) ((Y>>8) & 0xFF)
	#define ABS(X) ((X<0) ? (-X) : (X))
	#define BYTES2WORD(X,Y) ((X<<8)+(Y))


	/* STV6110 definition */
	/*  CTRL1        */
	#define RSTV6110_CTRL1		0
	#define FSTV6110_K			0
	#define FSTV6110_LPT		1
	#define FSTV6110_RX			2
	#define FSTV6110_SYN		3
	
	/*  CTRL2        */
	#define RSTV6110_CTRL2		1
	#define FSTV6110_CODIV		4
	#define FSTV6110_REFOUTSEL	5	
	#define FSTV6110_BBGAIN		6	
	
	/*  TUNING0      */
	#define RSTV6110_TUNING0	2	
	#define FSTV6110_NDIV_LSB	7
	
	/*  TUNING1      */
	#define RSTV6110_TUNING1	3	
	#define FSTV6110_RDIV		8
	#define FSTV6110_PRESC32ON	9
	#define FSTV6110_DIV4SEL	10
	#define FSTV6110_NDIV_MSB	11
	
	/*  CTRL3        */
	#define RSTV6110_CTRL3		4	
	#define FSTV6110_DCLOOP_OFF	12
	#define FSTV6110_RCCLKOFF   13
	#define FSTV6110_ICP		14
	#define FSTV6110_CF			15
	
	
	/*  STAT1        */
	#define RSTV6110_STAT1		5	
	#define FSTV6110_TEST1		16
	#define FSTV6110_CALVCOSTRT	17
	#define FSTV6110_CALRCSTRT	18
	#define FSTV6110_LOCKPLL	19

	/*  STAT2        */
	#define RSTV6110_STAT2		6	
	#define FSTV6110_TEST2		20

	/*  STAT3        */
	#define RSTV6110_STAT3		7	
	#define FSTV6110_TEST3		21

	/*Total Register Numbers and Field Numbers*/
	#define	STV6110_NBREGS 		8
	#define STV6110_NBFIELDS 	22
		
	/* structures */
	typedef enum
	{
		TUNER_NULL = 0,		/* unknown tuner */
		TUNER_STV6110,		/* ST STV6110 tuner */
		TUNER_HZ1184,		/* sharp HZ1184 nim module */
		TUNER_MAX2116,		/* maxim 2116 tuner */
		TUNER_SU1200,		/* philips SU1200 nim module */  
		TUNER_TBMU301,		/* samsung TBMU301 nim module */
		TUNER_IX2410,		/* Sharp IX2410 tuner */    
		TUNER_STB6000,		/* RF magic STB6000 tuner */
		TUNER_STB6100,		/* RF magic STB6100 tuner */
		TUNER_IX2476		/* sharp IX2476 tuner */
	}TUNER_Model_t;
	
	typedef enum
	{
		TUNER_IQ_NORMAL = 1,
		TUNER_IQ_INVERT = -1
	}TUNER_IQ_t;

    typedef struct
    {
    	TUNER_Model_t	Model;    	/* Tuner model */
        U32				Reference;	/* reference frequency (Hz) */ 
        U32				IF;       	/* Intermediate frequency (KHz) */
        TUNER_IQ_t		IQ_Wiring; 	/* hardware I/Q invertion */
    }TUNER_InitParams_t, *TUNER_Params_t;

	/* access modes for fields and registers */
	typedef enum
	{
		STCHIP_ACCESS_WR,  /* can be read and written */
		STCHIP_ACCESS_R,   /* only be read from */
		STCHIP_ACCESS_W,   /* only be written to */
		STCHIP_ACCESS_NON  /* cannot be read or written (guarded register, e.g. register skipped by ChipApplyDefaultValues() etc.) */
	}STCHIP_Access_t;
	
	/* register field type */
	typedef enum
	{
		CHIP_UNSIGNED,
		CHIP_SIGNED
	}STCHIP_FieldType_t;

	typedef struct
	{
	    U8             		Addr;  		/* Address */
	    U32   				Default;	/* Default value */
	    U8 					Value;		/* Current value */
	    char            	Name[30];	/* Name */
	    STCHIP_Access_t 	Access;		/* access mode */
	}Register_t;


	/* register field information */
	typedef struct
	{
	    U8 				   Reg;
	    unsigned char      Pos;      /* Bit position */
	    unsigned char      Bits;     /* Bit width */
	    U32				   Mask;	 /* Mask compute with width and position */
	    STCHIP_FieldType_t Type;     /* Signed or unsigned */
	    char               Name[30]; /* Name */
	}Field_t;



	/* Default values for tuner sat */
	Register_t STV6110_RegMap[STV6110_NBREGS];
	Field_t	STV6110_FieldMap[STV6110_NBFIELDS];

/*#define USE_ST 1

#if USE_ST
	U8 DefSTV6110Val[STV6110_NBREGS] = {0x07,0x11,0xdc,0x85,0x17,0x01,0xe6,0x1e}; //ST
#else
	U8 DefSTV6110Val[STV6110_NBREGS] = {0x07,0x93,0x0e,0x86,0x04,0x01,0xe2,0x0f}; //Serit
#endif*/


	U8 DefSTV6110Val[STV6110_NBREGS] = {0x07,0x13,0xdc,0x85,0x17,0x01,0xe6,0x1e};

	static S32	g_tuner_freq_input_KHz = 0;




/*****************************************************
**FUNCTION	::	absolute
**ACTION	::	Compute  abs(x) (where n is an integer) 
**PARAMS IN	::	number -> x
**PARAMS OUT::	NONE
**RETURN	::	-x(x<0), x(x>=0)
*****************************************************/
static U32 absolute(S32 x)
{
	if(x>0)
		return x;
	else
		return (-x);
}



/*****************************************************
**FUNCTION	::	powof2
**ACTION	::	Compute  2^n (where n is an integer) 
**PARAMS IN	::	number -> n
**PARAMS OUT::	NONE
**RETURN	::	2^n
*****************************************************/
static U32 powof2(U32 number)
{
	U32 i;
	U32 result=1;
	
	for(i=0;i<number;i++)
		result*=2;
		
	return result;
}


/****************************************************
**												   **
** ST6110 Registers and Field Write/Read Operation **
**												   **
*****************************************************/
static U32 Log2N(int number)
{
	int i=0;
	U32	powof2=1;

	if(number<1)
		return 0;
	
	while(powof2 <= (U32)(ABS(number)))
	{
		powof2 *= 2;
		i++;
	}
	
	return i-1;
}


static U32 CreateMask(char nbits, char pos)
{
	U32	val;

	val = (U32)((powof2(nbits)-1)<<pos);

	return val;
}


static void STV6110AddReg(U32 Id,char * Name,U8 Address,U8 Default,STCHIP_Access_t Access)
{
	STV6110_RegMap[Id].Addr	    = Address;
	STV6110_RegMap[Id].Default	= Default;
	STV6110_RegMap[Id].Value	= Default;
	STV6110_RegMap[Id].Access   = Access;
}


static void STV6110AddField(U8 RegId, U32 FieldId,char * Name, char Pos, char NbBits, STCHIP_FieldType_t Type)
{
	STV6110_FieldMap[FieldId].Reg = RegId;
	STV6110_FieldMap[FieldId].Pos = Pos;
	STV6110_FieldMap[FieldId].Bits = NbBits;
	STV6110_FieldMap[FieldId].Type = Type;
	if(NbBits)
		STV6110_FieldMap[FieldId].Mask = CreateMask(NbBits,Pos);
}


static void STV6110SetFieldImage(U32 FieldId, int Value)
{
	if(FieldId < STV6110_NBFIELDS)
	{
		if(STV6110_FieldMap[FieldId].Type == CHIP_SIGNED)
			Value = (Value > 0 ) ? Value : Value + (1<<STV6110_FieldMap[FieldId].Bits);	/*	compute signed fieldval	*/

		Value = STV6110_FieldMap[FieldId].Mask & (Value << STV6110_FieldMap[FieldId].Pos);	/*	Shift and mask value	*/
		STV6110_RegMap[STV6110_FieldMap[FieldId].Reg].Value = (STV6110_RegMap[STV6110_FieldMap[FieldId].Reg].Value & (~STV6110_FieldMap[FieldId].Mask)) + Value;	/*	Concat register value and fieldval	*/
	}
}

static int STV6110GetOneRegister(U8 RegId)
{
	MT_FE_RET ret = MtFeErr_Ok;
	U8	data, reg_index;

	reg_index = STV6110_RegMap[RegId].Addr;
	
	ret = _mt_fe_tn_get_reg(reg_index, &data);
	if(ret != MtFeErr_I2cErr)
		STV6110_RegMap[RegId].Value = data;

	return	data;
}


static MT_FE_RET STV6110SetOneRegister(U8 RegId, U8 value)
{
	MT_FE_RET ret = MtFeErr_Ok;
	U8  val, reg_index;

	reg_index = STV6110_RegMap[RegId].Addr;
	val		  = value;

	ret = _mt_fe_tn_set_reg(reg_index, val);	
	//if(ret != MtFeErr_Ok)
	//	mt_fe_print(("WriteReg  ERROR\n"));

	return ret;
}


static int STV6110GetFieldImage(U32 FieldId)
{
	int value = 0xFF;
		
	if(FieldId < STV6110_NBFIELDS)
	{
		value = STV6110_RegMap[STV6110_FieldMap[FieldId].Reg].Value;
					
		value=(value & STV6110_FieldMap[FieldId].Mask) >> STV6110_FieldMap[FieldId].Pos;	/*	Extract field	*/
		if((STV6110_FieldMap[FieldId].Type == CHIP_SIGNED)&&(value & (1<<(STV6110_FieldMap[FieldId].Bits-1))))
			value = value - (1<<STV6110_FieldMap[FieldId].Bits);			/*	Compute signed value	*/
	}
	return value;
}

static int STV6110GetField(U32 FieldId)
{
	int value = 0xFF;   
	
	if(FieldId < STV6110_NBFIELDS)
	{
		/* I2C Read : register address set-up */
		STV6110GetOneRegister(STV6110_FieldMap[FieldId].Reg);				/*	Read the register	*/     
		value = STV6110GetFieldImage(FieldId);
	}
	return value;
}


static void STV6110SetField(U32 FieldId,int Value)
{
	if(FieldId < STV6110_NBFIELDS)
	{
		if(STV6110_RegMap[STV6110_FieldMap[FieldId].Reg].Access != STCHIP_ACCESS_R)
		{
			STV6110GetOneRegister(STV6110_FieldMap[FieldId].Reg);	/*	Read the register	*/
			STV6110SetFieldImage(FieldId,Value);					/*	Compute new RegMap value */
			STV6110SetOneRegister(STV6110_FieldMap[FieldId].Reg,STV6110_RegMap[STV6110_FieldMap[FieldId].Reg].Value);		/*	Write the register */
		}
	}
}


static void STV6110SetRegisters(int FirstReg, int NbRegs)
{
	unsigned char nbdata = 0;
	int index;
	
	 for(index = FirstReg; index < (FirstReg + NbRegs); index++)
	 {
		//printf("Reg = 0x%02x Value = 0x%02x \n", STV6110_RegMap[index].Addr, STV6110_RegMap[index].Value);
		STV6110SetOneRegister(STV6110_RegMap[index].Addr, STV6110_RegMap[index].Value);
	 }
}


static void STV6110GetRegisters(int FirstReg, int NbRegs)
{
    int index;
    for(index = FirstReg; index < (FirstReg + NbRegs); index++)
    {
		//printf("FirstReg=%d, hChip->pRegMap[FirstReg].Value=%x \n", FirstReg, hChip->pRegMap[FirstReg].Value);
		STV6110_RegMap[index].Value=STV6110GetOneRegister(STV6110_RegMap[index].Addr);
    }
}



/*****************************************
**
** Montage external APIs for Serit761vb **
**
******************************************/

/*****************************************************************
** Function: mt_fe_tn_init_serit761vb
**
**
** Discription:    This function is used to initial Serit761vb. 
**
**
** Inputs:   None
**
** Outputs:  None
**
******************************************************************/
MT_FE_RET mt_fe_tn_init_serit761vb(void)
{
	U32 index=0;

	/*	REGISTER INITIALIZATION */
	/*	CTRL1	*/
	STV6110AddReg(RSTV6110_CTRL1,"CTRL1",0x00,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_CTRL1,FSTV6110_K,"K",3,5,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL1,FSTV6110_LPT,"LPT",2,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL1,FSTV6110_RX,"RX",1,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL1,FSTV6110_SYN,"SYN",0,1,CHIP_UNSIGNED);

	/*	CTRL2	*/
	STV6110AddReg(RSTV6110_CTRL2,"CTRL2",0x01,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_CTRL2,FSTV6110_CODIV,"ODIV",6,2,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL2,FSTV6110_REFOUTSEL,"REFOUTSEL",4,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL2,FSTV6110_BBGAIN,"BBGAIN",0,4,CHIP_UNSIGNED);  

	/*	TUNING0 */
	STV6110AddReg(RSTV6110_TUNING0,"TUNING0",0x02,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_TUNING0,FSTV6110_NDIV_LSB,"NDIV_LSB",0,8,CHIP_UNSIGNED);

	/*	TUNING1 */
	STV6110AddReg(RSTV6110_TUNING1,"TUNING1",0x03,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_TUNING1,FSTV6110_RDIV,"RDIV",6,2,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_TUNING1,FSTV6110_PRESC32ON,"PRESC32ON",5,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_TUNING1,FSTV6110_DIV4SEL,"DIV4SEL",4,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_TUNING1,FSTV6110_NDIV_MSB,"NDIV_MSB",0,4,CHIP_UNSIGNED);

	/*	CTRL3	*/
	STV6110AddReg(RSTV6110_CTRL3,"CTRL3",0x04,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_CTRL3,FSTV6110_DCLOOP_OFF,"DCLOOP_OFF",7,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL3,FSTV6110_RCCLKOFF,"RCCLK_OFF",6,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL3,FSTV6110_ICP,"ICP",5,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_CTRL3,FSTV6110_CF,"CF",0,5,CHIP_UNSIGNED); 		

	/*	STAT1	*/
	STV6110AddReg(RSTV6110_STAT1,"STAT1",0x05,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_STAT1,FSTV6110_TEST1,"TEST1",3,5,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_STAT1,FSTV6110_CALVCOSTRT,"CALVCOSTRT",2,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_STAT1,FSTV6110_CALRCSTRT,"CALRCSTRT",1,1,CHIP_UNSIGNED);
	STV6110AddField(RSTV6110_STAT1,FSTV6110_LOCKPLL,"LOCKPLL",0,1,CHIP_UNSIGNED);

	/*	STAT2	*/
	STV6110AddReg(RSTV6110_STAT2,"STAT2",0x06,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_STAT2,FSTV6110_TEST2,"TEST2",0,8,CHIP_UNSIGNED);

	/*	STAT3	*/
	STV6110AddReg(RSTV6110_STAT3,"STAT3",0x07,DefSTV6110Val[index++],STCHIP_ACCESS_WR);
	STV6110AddField(RSTV6110_STAT3,FSTV6110_TEST3,"TEST3",0,8,CHIP_UNSIGNED);

	STV6110SetRegisters(RSTV6110_CTRL1,8);

	return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_set_output_clock
**
**
** Discription: This function is used to set output clock. 
**
** Input:	
**
**	 Parameter		    	Type			Description
**	--------------------------------------------------------------
**	 divider    			U32				divider for clock output		
**
** Outputs: None
**
******************************************************************/
MT_FE_RET mt_fe_tn_set_output_clock(U8 divider)
{

	/*Allowed values 1,2,4 and 8*/
	STV6110SetField(FSTV6110_CODIV,(divider >> 1));
	
	return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_set_bandwidth_serit761vb
**
**
** Discription: This function is used to set bandwidth to Serit761vb. 
**
** Input:	
**
**	 Parameter		    	Type			Description
**	--------------------------------------------------------------
**	 symbol_rate_KSs    	U32				KSs		
**
** Outputs: None
**
******************************************************************/
MT_FE_RET mt_fe_tn_set_bandwidth_serit761vb(U32 symbol_rate_KSs)
{
	U8  u8;
	S32 i;
	U32 bandwidth;
	

	bandwidth = symbol_rate_KSs*135/100 + 10000;

	if((bandwidth/2)>36000) 			/*F[4:0] BW/2 max =31+5=36 mhz for F=31*/
		u8 = 31;
	else if((bandwidth/2)<5000)			/* BW/2 min = 5Mhz for F=0 */
		u8 = 0;
	else								/*if 5 < BW/2 < 36*/
		u8=(bandwidth/2000)-5;

	STV6110SetField(FSTV6110_RCCLKOFF,0);	 /* Activate the calibration Clock */
	STV6110SetField(FSTV6110_CF,u8);		 /* Set the LPF value */
	STV6110SetField(FSTV6110_CALRCSTRT,1);   /* Start LPF auto calibration*/

	i=0;
	while((i<10) && (STV6110GetField(FSTV6110_CALRCSTRT)!=0))
	{
		_mt_sleep(1);/* wait for LPF auto calibration */
		i++;
	}
	
	STV6110SetField(FSTV6110_RCCLKOFF,1);	 /* calibration dobe, desactivate the calibration Clock */  

	return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_get_bandwidth_serit761vb
**
**
** Discription: This function is used to get bandwidth to Serit761vb. 
**
** Input:	
**
**	 Parameter		    Type			Description
**	--------------------------------------------------------------
**	 lpf_MSs    		U32*			unit: KSs
**
** Outputs: None
**
******************************************************************/
MT_FE_RET mt_fe_tn_get_bandwidth_serit761vb(U32 *lpf_KSs)
{	
	*lpf_KSs = (STV6110GetField(FSTV6110_CF)+5)*2000; /* x2 for ZIF tuner */  	
	
	return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_get_lock_serit761vb
**
**
** Discription: This function is used to get bandwidth to Serit761vb. 
**
** Input:	None
**
** Outputs: 
**
**	 Parameter		    Type					Description
**	--------------------------------------------------------------
**	 p_state    		MT_FE_LOCK_STATE*		lock status
**
******************************************************************/
MT_FE_RET mt_fe_tn_get_lock_serit761vb(MT_FE_LOCK_STATE *p_state)
{
	U8 locked;
	
	locked = STV6110GetField(FSTV6110_LOCKPLL);

	if(locked)	*p_state = MtFeLockState_TunerLocked;
	else		*p_state = MtFeLockState_TunerLocked;
	
	return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_get_gain_serit761vb
**
**
** Discription: This function is used to get gain. 
**
** Input:	None
**
** Outputs: 
**
**	 Parameter		    Type				Description
**	--------------------------------------------------------------
**	 p_gain    		    U32 *				gain value
**
******************************************************************/
MT_FE_RET mt_fe_tn_get_gain_serit761vb(U32 *p_gain)
{
	U8 reg_0x3f, reg_0x41;

	_mt_fe_dmd_get_reg(0x3f, &reg_0x3f);
	_mt_fe_dmd_get_reg(0x41, &reg_0x41);

	if(reg_0x3f < 0x7f)
		reg_0x41 = 255;


	*p_gain = 125*reg_0x3f + 21*(255 - reg_0x41);  /*125*PWM RATE + 21*(255 - clipping)*/

	return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_get_strength_serit761vb
**
**
** Discription: This function is used to get signal strength. 
**
** Input:	None
**
** Outputs: 
**
**	 Parameter		    Type				Description
**	--------------------------------------------------------------
**	 p_strength    		S8 *				signal strength
**
******************************************************************/
#define STRENGTH_VALUE_C  750
#define STRENGTH_RATIO    100

MT_FE_RET mt_fe_tn_get_strength_serit761vb(S8 *p_strength)
{
	U8  ii=0;
	U32 gain_val, strength;
	MT_FE_LOCK_STATE lock_state;

#if 1
	
	mt_fe_tn_get_gain_serit761vb(&gain_val);
	
    #if MT_FE_TN_SUPPORT_ST6110_ON_BOARD
		gain_val -= STRENGTH_VALUE_C;
    #endif
	
	if(gain_val <= 7700)		strength = 99;							  //about -20dBm    99%
#if MT_FE_TN_SUPPORT_ST6110_ON_BOARD
	else if(gain_val <= 12750)  strength = 60 + (12750 - gain_val)/121;   //about -20dBm ~ -60dBm   60%~99%
	else if(gain_val <= 18770)  strength = 0 + (18770 - gain_val)/99;     //about -60dBm ~ -105dBm  0 ~ 60%
#else//SERIT761VB
	else if(gain_val <= 11925)  strength = 60 + (11925 - gain_val)/98;    //about -20dBm ~ -60dBm    60%~99%
	else if(gain_val <= 17000)  strength = 0 + (17000 - gain_val)/85;     //about -60dBm ~ -105dBm   0 ~ 60%
#endif
	else					    strength = 0;


	mt_fe_dmd_ds3k_get_lock_state(&lock_state);
	if((lock_state == MtFeLockState_Locked)&&(strength < 40))
		strength = 20 + strength/2;

	strength = strength*STRENGTH_RATIO/100;


	if(strength >= 100) strength = 99;

	*p_strength = strength;

	return MtFeErr_Ok;

#else

	int nSignalLevel_1[][2] = 
	{
		//RAGC  strength
		{49, 10},
		{69, 28},
		{82, 29},
		{100, 30},
		{110, 31},
		{133, 32},
		{154, 33},
		{163, 34},
		{175, 38},
		{181, 41},
		{189, 43},
		{202, 45},
		{215, 46},
		{224, 48},
		{231, 50},
		{237, 53}
	};
	
	int nSignalLevel_2[][2] = 
	{
		//RAGC strength
		{62, 95},
		{67, 92},
		{71, 90},
		{65, 89},
		{67, 88},
		{68, 86},
		{69, 85},
		{70, 84},
		{72, 83},
		{73, 82},
		{74, 81},
		{75, 80},      		
		{76, 78},      		
		{77, 77},				
		{79, 75},    		
		{80, 73},
		{82, 72},
		{84, 71},				
		{86, 70},
		{88, 69},
		{90, 67},
		{93, 64},
		{96, 63},
		{99, 62},
		{103, 61},
		{107, 60},
		{110, 59},
		{114, 57},      		
		{119, 55},
		{122, 53},				
		{125, 51},				
		{127, 50},
	};
	
	_mt_fe_dmd_get_reg(0x3f, &tmp);
	
	if((tmp&0x7f) < 0x7f)
	{
		for(ii=0; ii<(sizeof(nSignalLevel_2)/(sizeof(int)*2)); ii++)
		{
			if(nSignalLevel_2[ii][0] < tmp)
			{
				continue;
			}
			*p_strength = nSignalLevel_2[ii][1];
			break;
		}
	}
	else
	{
		_mt_fe_dmd_get_reg(0x41, &tmp);
		
		for(ii=0; ii<(sizeof(nSignalLevel_1)/(sizeof(int)*2)); ii++)
		{
			if(nSignalLevel_1[ii][0] < tmp)
			{
				continue;
			}		
			*p_strength = nSignalLevel_1[ii][1];
			break;
		}

		if (ii >= sizeof (nSignalLevel_1)/(sizeof(int)*2))
		{
			*p_strength = 53;
		}
	}
	
	return MtFeErr_Ok;
#endif
}





/*****************************************************************
** Function: mt_fe_tn_set_freq_serit761vb
**
**
** Discription: This function is used to set freq to Serit761vb. 
**
**
** Input:
**
**	 Parameter		    Type		Description
**	----------------------------------------------------------
**	 freq_KHz    		U32			from 950000 to 2150000
**	 sym_rate_KSs   	U32       	from 1000 to 45000
**	 lpf_offset_KHz 	S16			low pass filter offset 
**
** Outputs:  	None
**
******************************************************************/
MT_FE_RET mt_fe_tn_set_freq_serit761vb(U32 freq_KHz, U32 sym_rate_KSs, S16 lpf_offset_KHz)
{
	MT_FE_RET ret = MtFeErr_Ok;
	TUNER_Params_t hTunerParams = NULL;
	U32 divider;
	U8  p,presc,r,i;
	S32 pCalc, pCalcOpt=1000, rDiv=0, rDivOpt=0, pVal;

	g_tuner_freq_input_KHz = freq_KHz;

	mt_fe_tn_init_serit761vb();

	STV6110SetField(FSTV6110_K,((MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB/1000)-16));
	if (freq_KHz<=1023000) 
	{
		p=1;
		presc=0;
	}
	else if (freq_KHz<=1300000)
	{
		p=1;
		presc=1;
	}
	else if (freq_KHz<= 2046000)
	{
		p=0;
		presc=0;
	}
	else
	{
		p=0;
		presc=1;
	}
	STV6110SetField(FSTV6110_DIV4SEL,p);
	STV6110SetField(FSTV6110_PRESC32ON,presc);

	pVal=(int)powof2(p+1)*10;
	for(rDiv=0;rDiv<=3;rDiv++)
	{
		pCalc= (MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB/100)/(powof2(rDiv+1));

		if((absolute(pCalc-pVal)) < (absolute(pCalcOpt-pVal)))
			rDivOpt = rDiv;

		pCalcOpt= (MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB/100)/(powof2(rDivOpt+1));
	}
	
	r=powof2(rDivOpt+1);
	divider= (freq_KHz * r * powof2(p+1)*10)/MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB;  
	divider=(divider+5)/10;

	STV6110SetField(FSTV6110_RDIV,rDivOpt);
	STV6110SetField(FSTV6110_NDIV_MSB,MSB(divider));
	STV6110SetField(FSTV6110_NDIV_LSB,LSB(divider));
	
	STV6110SetField(FSTV6110_CALVCOSTRT,1);		/* VCO Auto Calibration */

	i=0;
	while((i<10) && (STV6110GetField(FSTV6110_CALVCOSTRT)!=0))
	{
		/* wait for VCO auto calibration */
		_mt_sleep(1);
		i++;
	}


	mt_fe_tn_set_bandwidth_serit761vb(sym_rate_KSs);

	
	return ret;
}



/*****************************************************************
** Function: mt_fe_tn_set_freq_serit761vb
**
**
** Discription: This function is used to set freq to Serit761vb. 
**
** Input:	None
**
** Outputs: 
**
**
**	 Parameter		    Type		Description
**	----------------------------------------------------------
**	 freq_KHz    		U32			from 950000 to 2150000
**
******************************************************************/
MT_FE_RET mt_fe_tn_get_freq_serit761vb(U32 *freq_KHz)
{
	U32 nbsteps, frequency, psd2, divider;

	divider = 0;

	STV6110GetRegisters(RSTV6110_TUNING0,2);
	divider = BYTES2WORD(STV6110GetFieldImage(FSTV6110_NDIV_MSB),STV6110GetFieldImage(FSTV6110_NDIV_LSB));	/*N*/
	nbsteps = STV6110GetFieldImage(FSTV6110_RDIV);	/*Rdiv*/
	psd2 = STV6110GetFieldImage(FSTV6110_DIV4SEL);	/*p*/
				
	frequency = divider * MT_FE_CRYSTAL_FREQ_KHZ_SERIT761VB;
	frequency = frequency/(powof2(nbsteps+psd2));
	frequency /= 4;

	*freq_KHz = frequency;

	return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_get_tuner_freq_offset_serit761vb
**
**
** Discription: This function is used to get frequency offset. 
**
** Input:	None
**
** Outputs: 
**
**	 Parameter		    Type					Description
**	--------------------------------------------------------------
**	 p_state    		MT_FE_LOCK_STATE*		lock status
**
******************************************************************/
S32	mt_fe_tn_get_tuner_freq_offset_serit761vb(void)
{
	U32 true_freq_KHz;
	S32 freq_offset_KHz;

	mt_fe_tn_get_freq_serit761vb(&true_freq_KHz);

	freq_offset_KHz = true_freq_KHz - g_tuner_freq_input_KHz;
	
	return freq_offset_KHz;
}



#endif/*#if (MT_FE_TN_SUPPORT_SERIT_SP761VB > 0)*/

