/****************************************************************************
*	Copyright (c) 2008, MONTAGE PROPRIETARY AND CONFIDENTIAL
*	Montage Technology (Shanghai) Inc.
*	All Rights Reserved
*	------------------------------------
*
*	FILE:				mt_fe_tn_sharp6306.c
*
*	VERSION:			2.01.00
*
*	DESCRIPTION:
*			Define the interfaces to initialize sharp6306 tuner
*		and set the control bytes of sharp6306 tuner when tuning
*		to a specific frequency.
*
*	FUNCTION:
*		    mt_fe_tn_set_freq_sharp6306();
*			mt_fe_tn_init_sharp6306();
*
* 	History:
*			Description     Version	  	Date 	        Author 			
*---------------------------------------------------------------------
*			File Create	    1.10.00		2008.06.11		ChenXiaopeng	
*			Modify          2.00.00		2009.03.18		Rick			
*			Modify          2.01.00		2009.04.29		Rick			
****************************************************************************/
#include "mt_fe_def.h"

#include "mt_fe_i2c.h"


#if (MT_FE_TN_SUPPORT_SHARP6306 > 0)

//static U32 freq_real_KHz  			= 0;
static S32 g_tuner_offset 			= 0;
static U32 g_true_tuner_freq_MHz 	= 0;

typedef enum _Tuner_BBGain_Sharp6306
{
	Bbg_0_Sharp,
	Bbg_1_Sharp,
	Bbg_2_Sharp,
	Bbg_4_Sharp   
}Tuner_BBGain_Sharp6306;


typedef enum _Tuner_PumpCurrent_Sharp6306
{
	/**
	 * min +/- 78 uA; typical +/- 120 uA; Max +/- 150 uA
	 */
	PC_78_150_Sharp,
	/**
	 * min +/- 169 uA; typical +/- 260 uA; Max +/- 325 uA
	 */
	PC_169_325_Sharp,
	/**
	 * min +/- 360 uA; typical +/- 555 uA; Max +/- 694 uA
	 */
	PC_360_694_Sharp,
	/**
	 * min +/- 780 uA; typical +/- 1200 uA; Max +/- 1500 uA
	 */
	PC_780_1500_Sharp
}Tuner_PumpCurrent_Sharp6306;




//static U8  tuner_reg0[4];
static U8  ucRegData[4];

U16 ucLPF;



extern void _mt_sleep(U32 ms);



MT_FE_RET mt_fe_tn_set_BBgain_sharp6306(U8 BBGain)
{
	ucRegData[0] &=  ~(0x3<<5);
	ucRegData[0] |= (BBGain<<5);
	return(MtFeErr_Ok);
}


MT_FE_RET mt_fe_tn_set_chargepump_sharp6306(U8 current)
{
	ucRegData[2] &= ~(0x3<<5);
	ucRegData[2] |= (current<<5);
	return(MtFeErr_Ok);
}


MT_FE_RET mt_fe_tn_set_frequency_sharp6306(U16 frequency_100kHz)
{
	U16 P, N, A, DIV;

	if(frequency_100kHz < 9500) 
	{
		return(MtFeErr_Param);
	}
	else if(frequency_100kHz <= 9860)
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x5<<5);
		P = 16;
		DIV = 1;
	}
	else if( frequency_100kHz <= 10730 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x6<<5);
		P = 16;
		DIV = 1;
	}
	else if( frequency_100kHz <= 11540 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x7<<5);
		P = 32;
		DIV = 1;
	}
	else if( frequency_100kHz <= 12910 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x1<<5);
		P = 32;
		DIV = 0;
	}
	else if( frequency_100kHz <= 14470 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x2<<5);
		P = 32;
		DIV = 0;
	}
	else if( frequency_100kHz <= 16150 ) 
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x3<<5);
		P = 32;
		DIV = 0;
	}
	else if( frequency_100kHz <= 17910 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x4<<5);
		P = 32;
		DIV = 0;
	}
	else if( frequency_100kHz <= 19720 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x5<<5);
		P = 32;
		DIV = 0;
	}
	else if( frequency_100kHz <= 21550 )
	{
		ucRegData[3] &= ~(0x7<<5);
		ucRegData[3] |= (0x6<<5);
		P = 32;
		DIV = 0;
	}
	else
	{
		return(MtFeErr_Param);
	}

	A = (frequency_100kHz/10)%P;
	N = (frequency_100kHz/10)/P;

	g_true_tuner_freq_MHz = (P*N)+A;

	ucRegData[3] &= ~(0x1<<4);
	if( P==16 )
	{
		ucRegData[3] |= (0x1<<4);
	}

	ucRegData[3] &= ~(0x1<<1);
	ucRegData[3] |= (DIV<<1);

	ucRegData[1] &= ~(0x1f<<0);
	ucRegData[1] |= (A<<0);

	ucRegData[1] &= ~(0x7<<5);
	ucRegData[1] |= (N<<5);
	ucRegData[0] &= ~(0x1f<<0);
	ucRegData[0] |= ((N>>3)<<0);


	return(MtFeErr_Ok);
}


MT_FE_RET mt_fe_tn_set_LPF_sharp6306(U16 LPF_10kHz)
{
	LPF_10kHz /=100;
	
	if( LPF_10kHz < 10)
	{
		LPF_10kHz = 10;
	}
	
	if( LPF_10kHz>34 )
	{
		LPF_10kHz = 34;
	}
	
	ucLPF = (LPF_10kHz-10)/2+3;

	
	return(MtFeErr_Ok);
}



U16 mt_fe_tn_calculate_LPF_sharp6306(U16 symbolrate_10kHz)
{
	U32 lpf = symbolrate_10kHz;
	
	lpf *= 80;
	lpf /= 100;
	lpf += 200;
	
	return((U16)lpf);
}


MT_FE_RET mt_fe_tn_commitsetting_sharp6306()
{
	MT_FE_RET	ret;
	
	ucRegData[0] &= 0x7f;
	ucRegData[2] |= 0x80;

	ucRegData[2] &= ~(0x7<<2);
	ucRegData[3] &= ~(0x3<<2);


	ret = _mt_fe_tn_write(ucRegData, sizeof(ucRegData));
	if( ret != MtFeErr_Ok )
	{
		return(ret);
	}
	
	ucRegData[2] |= (0x1<<2);

	ret = _mt_fe_tn_write(ucRegData+2, 1);
	if( ret != MtFeErr_Ok )
	{
		return(ret);
	}

	_mt_sleep(12);

	ucRegData[2] |= ((((ucLPF)>>1)&0x1)<<3); /* PD4 */
	ucRegData[2] |= ((((ucLPF)>>0)&0x1)<<4); /* PD5 */
	ucRegData[3] |= ((((ucLPF)>>3)&0x1)<<2); /* PD2 */
	ucRegData[3] |= ((((ucLPF)>>2)&0x1)<<3); /* PD3 */

	ret = _mt_fe_tn_write(ucRegData+2, 2);


	return(ret);
}



MT_FE_RET mt_fe_tn_lock_sharp6306(U16 frequency_100kHz, U16 LPF_10kHz)
{
	ucRegData[0] = ucRegData[1] = ucRegData[2] = ucRegData[3] = 0;
	
	mt_fe_tn_set_LPF_sharp6306(LPF_10kHz);

	mt_fe_tn_set_frequency_sharp6306(frequency_100kHz);
	
	//use default values
	mt_fe_tn_set_chargepump_sharp6306(PC_780_1500_Sharp);
	mt_fe_tn_set_BBgain_sharp6306(Bbg_4_Sharp);


	mt_fe_tn_commitsetting_sharp6306();
	
	return(MtFeErr_Ok);
}


MT_FE_RET mt_fe_tn_get_gain_sharp6306(U32 *p_gain)
{
	U8 reg_0x3f, reg_0x40, reg_0x41;
	U32 tmp;

	_mt_fe_dmd_get_reg(0x3f, &reg_0x3f);
	_mt_fe_dmd_get_reg(0x40, &reg_0x40);
	_mt_fe_dmd_get_reg(0x41, &reg_0x41);

	tmp = (reg_0x3f<<8)|reg_0x40;

	if(tmp < 32512)
		reg_0x41 = 255;

	*p_gain = 23*tmp + 101*(255 - reg_0x41);  /*23*PWM RATE + 101*(255 - clipping)*/

	return MtFeErr_Ok;
}

#define STRENGTH_RATIO 100

static MT_FE_RET mt_fe_tn_get_strength_xx(U32 tuneType, S8 *p_strength)
{
//	U8  ii=0;
	U32 gain_val, strength;
	MT_FE_LOCK_STATE lock_state;
	
	mt_fe_tn_get_gain_sharp6306(&gain_val);

//#if MT_FE_TN_SUPPORT_SHARP7306
    if (tuneType == MT_FE_TN_SUPPORT_SHARP7306) 
    {
	    if(gain_val <= 677741)		strength = 99;										  //about -30dBm    99%
	    else if(gain_val <= 718550) strength = 60 + ((718550 - gain_val)/102+5)/10;   //1020 //about -30dBm ~ -60dBm   60%~99%
	    else if(gain_val <= 772824) strength = 0 + ((772824 - gain_val)/91 + 5)/10;   //906 //about -60dBm ~ -105dBm  0 ~ 60%
	    else 					    strength =  0;          
    }
//#else//SHARP6306
    else 
    {
	    if(gain_val <= 677258)		 strength = 99;										  //about -25dBm    99%
	    else if(gain_val <= 712770) strength = 60 + ((712770 - gain_val)/89+5)/10;  //887 //about -25dBm ~ -60dBm   60%~99%
	    else if(gain_val <= 771208) strength = 0 + ((771208 - gain_val)/97 + 5)/10;              //973 //about -60dBm ~ -105dBm  0 ~ 40%
	    else					     strength = 0;
    }
//#endif

	mt_fe_dmd_ds3k_get_lock_state(&lock_state);
	if((lock_state == MtFeLockState_Locked)&&(strength < 40))
		strength = 20 + strength/2;

	strength = strength*STRENGTH_RATIO/100;
	
	if(strength >= 100) strength = 99;
	
	*p_strength = strength;

	return MtFeErr_Ok;
}


MT_FE_RET mt_fe_tn_get_strength_sharp6306(S8 *p_strength)
{
    MT_FE_RET St;
    
    St = mt_fe_tn_get_strength_xx(actMntTunTpy, p_strength);
    return St;
}

MT_FE_RET mt_fe_tn_get_strength_sharp7306(S8 *p_strength)
{
    MT_FE_RET St;

    St = mt_fe_tn_get_strength_xx(actMntTunTpy, p_strength);
    return St;
}


/*	FUNCTIN:
**		mt_fe_tn_set_freq_sharp6306
**
**	DESCRIPTION:
**		set the frquency of tuner
**
**	IN:
**
**	OUT:
**
**	RETURN:
*/
MT_FE_RET mt_fe_tn_set_freq_sharp6306(U32 freq_KHz, U32 sym_rate_KSs, S16 lpf_offset_KHz)
{
	U16 LPF_10kHz, freq_100KHz;

	freq_100KHz = freq_KHz/100;
	
	LPF_10kHz = mt_fe_tn_calculate_LPF_sharp6306((U16)((sym_rate_KSs)/10));

	mt_fe_tn_lock_sharp6306(freq_100KHz, LPF_10kHz);

	g_tuner_offset = freq_KHz - g_true_tuner_freq_MHz*1000;

	return MtFeErr_Ok;
}

MT_FE_RET mt_fe_tn_set_freq_sharp7306(U32 freq_KHz, U32 sym_rate_KSs, S16 lpf_offset_KHz)
{
    MT_FE_RET St;
    
    St = mt_fe_tn_set_freq_sharp6306( freq_KHz, sym_rate_KSs, lpf_offset_KHz);
    return St;
}


/*	FUNCTIN:
**		mt_fe_tn_init_sharp6306
**
**	DESCRIPTION:
**
**	IN:
**
**	OUT:
**
**	RETURN:
*/
MT_FE_RET mt_fe_tn_init_sharp6306(void)
{
	return MtFeErr_Ok;
}


S32 mt_fe_tn_get_tuner_freq_offset_sharp6306(void)
{
	return g_tuner_offset;
}

S32 mt_fe_tn_get_tuner_freq_offset_sharp7306(void)
{
	return g_tuner_offset;
}

#endif /* MT_FE_TN_SUPPORT_SHARP6306 */


