/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Customise.c
**
** \brief     application dependent functions for MB86667 driver
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Customise.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
/* MB86667 related includes */
#include "MB86667_Api.h"
#include "MB86667_Customise.h" /* Customise must be first because it holds all defines */
#include "MB86667_Regs.h"
#include "MB86667_Priv.h"

#include "MB86667_Demod.h"

/* these include files are made for use with Fujitsu FAPI and have to be replaced
   by the appropriate functions in the customers environment */
#ifdef USE_WITHIN_FAPI
//    #include "../drv_fe.h"
    #include "fapex/abst_types.h"
    #include "fapex/abst_libi2c.h"
#endif

/* for debug only */
#if defined (FAPI_FE_MB86667_DEBUG01) || defined (FAPI_FE_MB86667_DEBUG02)
   #include <stdio.h>
   char PrintHelp[128];  /* temporary string used for printing messages */
#endif

/*---------------------------------------------------------------------------*/
/* global data                                                                */
/*---------------------------------------------------------------------------*/
FE_MB86667_RF_INI_FCT_T mb86667_Rf_Val_Set_Ini = NULL;
FE_MB86667_RF_SET_FCT_T mb86667_Rf_Val_Set     = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/* buffers to support multiple instances */
static FAPI_MB86667_DIS_FLAGS_E  bufFlags[FE_MAX_NUM];
static uint8_t                      bufComRep[FE_MAX_NUM];
static uint8_t                      bufUncRep[FE_MAX_NUM];
static uint8_t                      bufClockFac[FE_MAX_NUM];
static uint16_t                     bufSymRate[FE_MAX_NUM];
static FAPI_MB86667_TUNE_STATE_E bufTuneState[FE_MAX_NUM];
static uint32_t                     bufLockTime[FE_MAX_NUM];
static uint32_t                     bufCsTime[FE_MAX_NUM];
static uint32_t                     bufStartTimes[FE_MAX_NUM][MB86667_MAX_USER_TIMERS];

static FAPI_SYS_HandleT        resetHandle;
static int32_t                 i2cId;

static uint8_t                      i2cDevAddr;
static uint8_t                      i2cTunAddr;
static uint8_t                      actIndex;

/* other statics */
static uint32_t       StartTimes[MB86667_MAX_USER_TIMERS];


#ifdef  __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------------*/

void MB86667_SetContext(uint8_t index, FAPI_SYS_HandleT resetFct, int32_t _i2cId, uint8_t _i2cDevAddr, uint8_t _i2cTunAddr, FE_MB86667_RF_INI_FCT_T iniFct, FE_MB86667_RF_SET_FCT_T setFct)
{
    uint16_t i;

    if (index < FE_MAX_NUM)
    {
        actIndex=index;
        resetHandle = (FAPI_SYS_HandleT) resetFct;
        i2cId  = _i2cId;
        i2cDevAddr = _i2cDevAddr;
        i2cTunAddr = _i2cTunAddr;

        mb86667_flags(SET, bufFlags[index]);
        mb86667_com_sw(SET, bufComRep[index]);
        mb86667_unc_sw(SET, bufUncRep[index]);
        mb86667_clock_fac(SET, bufClockFac[index]);
        mb86667_sym_rate(SET, bufSymRate[index]);
        mb86667_tune_states(SET, bufTuneState[index]);
        mb86667_lock_time(SET, bufLockTime[index]);
        mb86667_cs_time(SET, bufCsTime[index]);

        mb86667_Rf_Val_Set_Ini = iniFct;
        mb86667_Rf_Val_Set     = setFct;

        for (i=0; i<MB86667_MAX_USER_TIMERS; i++)
           StartTimes[i] = bufStartTimes[index][i];
    }
}

void MB86667_SaveContext(uint8_t index)
{
    uint16_t i;

    if (index < FE_MAX_NUM)
    {
        bufFlags[index]     = mb86667_flags(GET, NO_DIS_FLAG);
        bufComRep[index]    = mb86667_com_sw(GET, 0);
        bufUncRep[index]    = mb86667_unc_sw(GET, 0);
        bufClockFac[index]  = mb86667_clock_fac(GET, 0);
        bufSymRate[index]   = mb86667_sym_rate(GET, 0);
        bufTuneState[index] = mb86667_tune_states(GET, TF_ACQ_END);
        bufLockTime[index]  = mb86667_lock_time(GET, 0);
        bufCsTime[index]    = mb86667_cs_time(GET, 0);
        for (i=0; i<MB86667_MAX_USER_TIMERS; i++)
           bufStartTimes[index][i] = StartTimes[i];
    }
}

void MB86667_ChangeFlags(uint8_t index, FAPI_MB86667_DIS_FLAGS_E flags, uint8_t comRep, uint8_t uncRep)
{
    if (index < FE_MAX_NUM)
    {
        bufFlags[index]  = flags;
        bufComRep[index] = comRep;
        bufUncRep[index] = uncRep;
    }
}

/**************************************************
  tuner-register read via the bypass I2C-bus interface

  Input:
    reg    : tuners sub address as byte
    rvalue : return value
  Output:

  Returns:
    FE_OK
    FE_ERR_FE_I2C
***************************************************/
int32_t MB86667_I2C_Bypass_Read(uint8_t reg, uint8_t* rvalue)
{
  uint8_t data = 0x02;

  if(MB86667_I2C_Write(MB86667_REG_I2C2,1,&data)!=FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG02
    MB86667_User_Print("\nMB86667 I2C BYPASS ERROR");
    #endif
    return FE_ERR_FE_I2C;
  }

  if(TUNER_I2C_Read(reg, 1 , rvalue)!=FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG02
    MB86667_User_Print("\nMB86667 BYPASS READ ERROR");
    #endif
    return FE_ERR_FE_I2C;
  }
  return FE_OK;
}

/**************************************************
  Tuner-register write via the bypass I2C-bus interface

  Input:
    value  : value to write
    length : number of bytes to write

  Returns:
    FE_OK
    FE_ERR_FE_I2C
***************************************************/ 
int32_t MB86667_I2C_Bypass_Write(uint8_t* value, uint8_t length)
{
  uint8_t   data=0x02;

  if(MB86667_I2C_Write(MB86667_REG_I2C2,1,&data)!=FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG02
    MB86667_User_Print("\nMB86667 I2C BYPASS ERROR");
    #endif
    return FE_ERR_FE_I2C;
  }
  
  if(TUNER_I2C_Write(length, value)!=FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG02
    MB86667_User_Print("\nMB86667 I2C BYPASS WRITE ERROR");
    #endif
    return FE_ERR_FE_I2C;
  }
  return FE_OK;
}


/**************************************************
  Layer between MB86667 API and customer specific I2C routines
***************************************************/

/**************************************************
  MB86667-register read via the users I2C-bus interface

  Input:
    ucSubAddress : MB86667 sub address as byte
    ucNrOfBytes  : number of bytes to read

  Output:
    pucData      : Pointer to the location where the data
                   should be stored

  Returns:
    FE_OK
    FE_ERR_FE_I2C
***************************************************/
int32_t MB86667_I2C_Read(uint8_t ucSubAddress, uint8_t ucNrOfBytes, uint8_t *pucData)
{
    /* define your own I2C read routine here using i2cDevAddr in "#else" */
    uint8_t trData[2];

    trData[0] = ucSubAddress;    /* !sub-address byte */
    if (FE_I2C_Read(i2cId, i2cDevAddr, trData, 1, pucData , ucNrOfBytes) != FE_OK)
    {
        #ifdef FAPI_FE_MB86667_DEBUG01
            sprintf(PrintHelp, "\nMB86667_I2C_Read failed!");
            MB86667_User_Print(PrintHelp);
        #endif
        return FE_ERR_FE_I2C;
    }

    return FE_OK;
}

/**************************************************
  MB86667-register write via the users I2C-bus interface

  Input:
    ucSubAddress  : MB86667 sub address as byte
    ucNrOfBytes   : number of Bytes excluding sub address, limited to 10
    pucData       : Pointer to the location from where the data
                    are taken
  Output:

  Returns:
    FE_OK
    FE_ERR_FE_I2C
    FD_ERR_BAD_PARAMETER
***************************************************/
int32_t MB86667_I2C_Write(uint8_t ucSubAddress, uint8_t ucNrOfBytes, uint8_t *pucData)
{
    /* define your own I2C write routine here using i2cDevAddr in "#else" */
    uint8_t ucCounter;
    uint8_t ucRegAdrAndData[11];

    if (ucNrOfBytes > 10)
    {
        #ifdef FAPI_FE_MB86667_DEBUG01
            sprintf(PrintHelp, "MB86667_I2C_Write bad params!");
            MB86667_User_Print(PrintHelp);
        #endif
        return FE_ERR_BAD_PARAMETER;
    }

    /* treat sub address as data byte, there is no difference for write calls */
    ucRegAdrAndData[0] = ucSubAddress;
    #ifdef FAPI_FE_MB86667_DEBUG01
        sprintf(PrintHelp, "\nSub address: %x", ucSubAddress);
        MB86667_User_Print(PrintHelp);
    #endif

    for (ucCounter = 0; ucCounter < ucNrOfBytes; ucCounter++)
    {
        ucRegAdrAndData[ucCounter+1] = pucData[ucCounter];
        #ifdef FAPI_FE_MB86667_DEBUG01
            sprintf(PrintHelp, " %x", pucData[ucCounter]);
            MB86667_User_Print(PrintHelp);
        #endif
    }

    if (FE_I2C_Write(i2cId, i2cDevAddr, ucRegAdrAndData, ucNrOfBytes+1) != FE_OK)
    {
        #ifdef FAPI_FE_MB86667_DEBUG01
            sprintf(PrintHelp, "MB86667_I2C_Write failed!");
            MB86667_User_Print(PrintHelp);
        #endif
        return FE_ERR_FE_I2C;
    }
  return FE_OK;
}

/**************************************************
  tuner read via the users I2C-bus interface

  Input:
    ucReg        : tuner sub address as byte or 255 if not needed
    ucNrOfBytes  : number of bytes to read

  Output:
    pucData      : Pointer to the location where the data
                   should be stored

  Returns:
    FE_OK
    FE_ERR_FE_I2C
***************************************************/
int32_t TUNER_I2C_Read(uint8_t ucReg, uint8_t ucNrOfBytes, uint8_t *pucData)
{
   /* define your own I2C read routine here using i2cTunAddr in "#else" */
   uint8_t trData[2];
   uint32_t ulLen;

   if(ucReg==255)  // == -1  so read access goes directly to device
      ulLen=0;
   else
      ulLen=1;
   trData[0]=ucReg;

   if (FE_I2C_Read(i2cId, i2cTunAddr, trData, ulLen, pucData , ucNrOfBytes) != FE_OK)
   {
      #ifdef FAPI_FE_MB86667_DEBUG01
         sprintf(PrintHelp, "\nTUNER_I2C_Read failed!");
         MB86667_User_Print(PrintHelp);
      #endif
      return FE_ERR_FE_I2C;
   }

   return FE_OK;
}

/**************************************************
  tuner write via the users I2C-bus interface

  Input:
    ucNrOfBytes   : number of Bytes including slave and sub address
                    (tuner special)
    pucData       : Pointer to the location from where the data
                    are taken

  Output:

  Returns:
    FE_OK
    FE_ERR_FE_I2C
    FD_ERR_BAD_PARAMETER
***************************************************/
int32_t TUNER_I2C_Write(uint8_t ucNrOfBytes, uint8_t *pucData)
{
    /* define your own I2C write routine here using i2cTunAddr in "#else" */
    if (FE_I2C_Write(i2cId, i2cTunAddr, &pucData[1],ucNrOfBytes-1) != FE_OK)
    {
        #ifdef FAPI_FE_MB86667_DEBUG01
           sprintf(PrintHelp, "\nTUNER_I2C_Write failed!");
           MB86667_User_Print(PrintHelp);
        #endif
        return FE_ERR_FE_I2C;
    }

    return FE_OK;
}

/**************************************************
* User defined timer routines for time critical operations
***************************************************/

/**************************************************
*
*
* Inputs:
*   ucFlag :
*               TIM_SYN_TIM
*                 This flag causes the routine to wait for a transition of the system tick.
*                 If you use TIM_RES_TIM and TIM_GET_TIM and the routine was not
*                 synchronised before then you have to add one tick time
*                 to your timeout in order to guarantee at least the timeout length.
*                 ucIndex is ignored if only this flag is set.
*               TIM_RES_TIM Resets the timer counter with Index ucIndex.
*                 The reset is done immediately without synchronisation to the system
*                 timer. If your timer tick is long (like the 40ms in our example) then it is
*                 recommended to call User_Timers with TIM_SYN_TIM before.
*                 This gives the maximum accuracy but requires in worst case 40 ms for
*                 synchronisation.
*               TIM_GET_TIM requests the elapsed time of Index ucIndex since its last reset
*   ucIndex:    Timer index (currently from 0 to MAX_INDEX)
*   ulDiffTime: holds the elapsed time of a request in ms.
*               The precision is depending on the system clock.
*               Here we have a resolution of 40 resp. 33 ms
*               This gives in total a max time of more than 1000 hours.
*
* Returns:
*   FE_OK or FD_ERR_BAD_PARAMETER, defined in MB86667_API.h
***************************************************/
int32_t MB86667_User_Timers(uint8_t ucFlag, uint8_t ucIndex, uint32_t *pulDiffTime)
{
    uint32_t ulTempTime;

    *pulDiffTime = 0;

    if (ucFlag > TIM_GET_TIM) return(FE_ERR_BAD_PARAMETER);
    if (ucIndex > MB86667_MAX_USER_TIMERS) return(FE_ERR_BAD_PARAMETER);

    /* sync */
    if ((ucFlag & TIM_SYN_TIM) == TIM_SYN_TIM)
    {
        /* wait here until timer changes */
        ulTempTime = (uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        while ((uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) == ulTempTime); /* wait */
    }

    /* reset */
    if ((ucFlag & TIM_RES_TIM) == TIM_RES_TIM)
    {
        /* set start time to current time */
        StartTimes[ucIndex] = (uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        *pulDiffTime = 0;
    }

    /* get */
    if ((ucFlag & TIM_GET_TIM) == TIM_GET_TIM)
    {
        /* return current time */
        ulTempTime = (uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        if (ulTempTime >= StartTimes[ucIndex])
        {
           *pulDiffTime = (uint32_t)(ulTempTime - StartTimes[ucIndex]);
        }
        else
        {
           /* in case of timer overflow reset start time */
            StartTimes[ucIndex] = (uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
            *pulDiffTime = 0;
        }
    }

    /* Mp_VoIntTimeClock increments every 1/25 sec (1/30 for NTSC) */
    /* Other systems may have better reference and need to correct the pulDiffTime in a different way */
    *pulDiffTime *= TIM_RESOLUTION; /* with this multiplication we get an overflow if time is longer than 1000 hours => shouldn't be a problem */

    return FE_OK;
}

/**************************************************
* MB86667_HardReset
* This function performs a hardware reset of the MB86667.
* It uses a GPIO port which is connected to the MB86667
* /RESET input.
* The function is called from MB86667_Init.
* It is strongly recommended to do a hardware reset of
* the MB86667 after power up. However, there is no need
* for a GPIO if the MB86667 /RESET-pin is connected
* to a reset generator which monitors the power supply.
*
* Inputs:
*   none
*
* Returns:
*   FE_OK, defined in MB86667_API.h
***************************************************/
int32_t MB86667_HardReset(void)
{
    FAPI_FE_RESET_FCT_T pReset;

    /* enter your code here to hard-reset the MB86667 */
    pReset= (FAPI_FE_RESET_FCT_T)resetHandle;
    pReset(actIndex);

    return FE_OK;
}

/**************************************************
* MB86667_User_Sleep
* This function currently justs wastes the time in a loop.
* It is recommended that the user implements it in such a
* way that the processor uses the time for other functions,
* For example in an RTOS-environment the control can be given
* back to the scheduler to allow other tasks to execute within
* that time.
* The function is used in all files of the MB86667_Api only as a
* minimum wait time: That means implementations that "wait" longer
* than specified by ulTime are allowed. But please keep in mind
* that this may increase the locking time and reduce your zapping speed.
*
* Inputs:
*   ulTime: the min. time to sleep in ms.
*
* Returns:
*   FE_OK or FD_ERR_BAD_PARAMETER, defined in MB86667_API.h
***************************************************/
int32_t MB86667_User_Sleep(uint32_t ulTime)
{
    /* enter your code here to realise a "wait" function */
      uint64_t t1, t2;

      t1 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
      do
      {
        t2 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        if (t2<t1) /* in case of timer overflow */
        {
            t1 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
            t2 = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        }
      } while ((t2-t1) < (uint32_t)ulTime);

    return FE_OK;
}

#if defined (FAPI_FE_MB86667_DEBUG01) || defined (FAPI_FE_MB86667_DEBUG02)
/**************************************************
* User defined print routines for debug outputs
*
* Inputs:
*   pString  : pointer to the string that should be printed
*
* Returns   : FE_OK, defined in MB86667_API.h
***************************************************/
int32_t MB86667_User_Print(char *pString)
{
    FAPI_SYS_PRINT_MSG(pString);
    return(FE_OK);
}
#endif

#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Customise.c,v $
Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.2  2007/11/19 09:11:32  kraft
fuj sat fe with blind scan

Revision 1.8  2007/08/29 11:47:13  skau
Supporting confidential files and front end detection now.

Revision 1.7  2007/07/04 10:40:45  skau
Finally made compatible to the windows monitor software.

Revision 1.6  2007/06/25 12:26:49  skau
New file structure.

Revision 1.3  2007/01/24 10:08:00  skau
Compiles without warnings, all CVS headers updated and
ready for first customer release.

Revision 1.2  2006/11/30 09:50:19  skau
First working version.

Revision 1.1  2006/11/03 15:11:02  skau
Driver for MB86667.This is the first (non-working) version.


*****************************************************************************/
