/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Demod.c
**
** \brief     Demod Drivers for MB86667.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   $Id: MB86667_Demod.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
#include "MB86667_Customise.h"

#include "MB86667_Demod.h"
#include "MB86667_Tuner.h"
#include "MB86667_Priv.h"
#include "MB86667_Regs.h"


/*** defines for BER counter ***/
  /* sampling time for post viterbi BER monitor
     0x00 = 12500 ksamples, 0x08 = 25000 ksamples,
     0x10 = 50000 ksamples, 0x18 = 100000 ksamples */
  #define BERRST 0x00
/*---------------------------------------------------------------------------*/
/* global data                                                                */
/*---------------------------------------------------------------------------*/
extern FE_MB86667_RF_INI_FCT_T mb86667_Rf_Val_Set_Ini;
extern FE_MB86667_RF_SET_FCT_T mb86667_Rf_Val_Set;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
#ifdef  __cplusplus
extern "C" {
#endif

/*----------------------------------------------------------------------------
Name:        MB86667_Initial_All
-------------------------------------------------------------------------------*/
int32_t MB86667_Initial_All(void)
{
    int32_t error;

    /* SLOCK0=0 */
    error = MB86667_Seq_Set(0);
    if(error != FE_OK)
        return error;

    error = MB86667_Iq_Vt_Set(0);
    if(error != FE_OK)
        return error;

    error = MB86667_AGC_SET(4);
    if(error != FE_OK)
        return error;

    error = MB86667_Initial_Set();
    if(error != FE_OK)
        return error;

    error = MB86667_Stlp_Set2(2,3);
    if(error != FE_OK)
        return error;

    error = MB86667_Eq_Set(1,3);
    if(error != FE_OK)
        return error;

    error = MB86667_Dagc_Data_Set(3,2);
    if(error != FE_OK)
        return error;

    return MB86667_En_Set(0,0);
}

/*----------------------------------------------------------------------------
Name:        MB86667_I2C_WRITE
-------------------------------------------------------------------------------*/
int32_t MB86667_I2C_WRITE(uint8_t reg, uint8_t value)
{
  int32_t ret;

  ret = MB86667_I2C_Write(reg, 1, &value);
  if(ret != FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG01
       sprintf(PrintHelp, "\nFD_ERR_FE_I2C during write");
       MB86667_User_Print(PrintHelp);
    #endif
    return FE_ERR_FE_I2C;
  }
  return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_I2C_READ
-------------------------------------------------------------------------------*/
int32_t MB86667_I2C_READ(uint8_t reg, uint8_t *rvalue)
{
  int32_t ret;

  ret = MB86667_I2C_Read(reg, 1, rvalue);
  if(ret != FE_OK)
  {
    #ifdef FAPI_FE_MB86667_DEBUG02
        sprintf(PrintHelp, "\nFD_ERR_FE_I2C during read");
        MB86667_User_Print(PrintHelp);
    #endif
    return FE_ERR_FE_I2C;
  }
  return FE_OK;
}


/*----------------------------------------------------------------------------
Name:        MB86667_Initial_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Initial_Set(void)
{
    int32_t error;

    error = MB86667_Stlp_Set(5,7);
    if(error!=FE_OK)
        return error;

    error = MB86667_Acfex_Data_Set(0,0);
    if(error!=FE_OK)
        return error;

    error = MB86667_Afcofs_Data_Set(0,0);
    if(error!=FE_OK)
        return error;

    /* CRBS=2, CRAS=6 */
    error = MB86667_I2C_WRITE(MB86667_REG_CR_L_FLT1, 0x16);
    if(error!=FE_OK)
        return error;

    /* CRWS=1 */
    error = MB86667_I2C_WRITE(MB86667_REG_CR_WIN1_2, 0x21);
    if(error!=FE_OK)
        return error;

    /* S4T=7 */
    error = MB86667_I2C_WRITE(MB86667_REG_S4_S5_TIM, 0xc7);
    if(error!=FE_OK)
        return error;


    /* VMAG=7 */
    error = MB86667_I2C_WRITE(MB86667_REG_VIT_GAIN, 0x38);
    if(error!=FE_OK)
        return error;

    /* ETH=4 */
    /* VIA=5 */
    error = MB86667_Vi_Set(4, 5);
    if(error!=FE_OK)
        return error;

    /* Front AGC Setting 1 */
    error = MB86667_I2C_WRITE(MB86667_REG_FRONTAGC1, 0x00);
    if(error!=FE_OK)
        return error;

    /* Front AGC Setting 2(4 to 3)*/
    error = MB86667_I2C_WRITE(MB86667_REG_FRONTAGC2, 0x1c);
    if(error!=FE_OK)
        return error;

    /* Front AGC Setting 3(3 to 2) */
    error = MB86667_I2C_WRITE(MB86667_REG_FRONTAGC3, 0x20);
    if(error!=FE_OK)
        return error;

    /* Front AGC Setting 4(3 to 4) */
    error = MB86667_I2C_WRITE(MB86667_REG_FRONTAGC4, 0x1e);
    if(error!=FE_OK)
        return error;

    /* Front AGC Setting 5(2 to 3) */
    error = MB86667_I2C_WRITE(MB86667_REG_FRONTAGC5, 0x23);
    if(error!=FE_OK)
        return error;

    error = MB86667_I2C_WRITE(MB86667_REG_AFCTH_1_L, 0xff);
    if(error!=FE_OK)
        return error;

    return MB86667_I2C_WRITE(MB86667_REG_AFC_3_0, 0x0a);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Iq_Vt_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Iq_Vt_Set(uint8_t iqinv)
{
  return MB86667_I2C_WRITE(MB86667_REG_IQ_AUTODET, \
         (uint8_t)(IQRES|IQAT|VIR4|VIR3|VIR2|VIR1|VIR0)|(uint8_t)((iqinv&0x01) << SHIFT_IQINV));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Fec_Srst
-------------------------------------------------------------------------------*/
int32_t MB86667_Fec_Srst(void)
{
  return MB86667_I2C_WRITE(MB86667_REG_RESET, MB86667_BIT_RESET_FEC);

}

/*----------------------------------------------------------------------------
Name:        MB86667_AGC_SET
-------------------------------------------------------------------------------*/
int32_t MB86667_AGC_SET(uint8_t agcw)
{
#if 0
    //ADI-PATCH: out AGC need positive polarity and slow speed, but AGC automatic
    return MB86667_I2C_WRITE(MB86667_REG_FAGC, (uint8_t)AGMD|(uint8_t)((agcw&0x0f)<<SHIFT_AGCW_0_3)); //|(uint8_t)AGCP|(uint8_t)AGCR;
#endif
   return MB86667_I2C_WRITE(MB86667_REG_FAGC, (uint8_t)AGCP|(uint8_t)(agcw<<SHIFT_AGCW_0_3));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Stlp_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Stlp_Set(uint8_t stras,uint8_t strbs)
{
  return MB86667_I2C_WRITE(MB86667_REG_STRF1, (uint8_t)(strbs<<SHIFT_STRBS_0_2)|(stras));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Stlp_Set2
-------------------------------------------------------------------------------*/
int32_t MB86667_Stlp_Set2(uint8_t stra,uint8_t strb)
{
  return MB86667_I2C_WRITE(MB86667_REG_STRF2, (uint8_t)(strb<<SHIFT_STRBS_0_2)|(stra));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Acfex_Data_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Acfex_Data_Set(uint8_t afcex_l,uint8_t afcex_h)
{
  int32_t error;

  error = MB86667_I2C_WRITE(MB86667_REG_AFCIL,afcex_l);
  if(error!=FE_OK)
    return error;

  return MB86667_I2C_WRITE(MB86667_REG_AFCIH,afcex_h);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Afcofs_Data_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Afcofs_Data_Set(uint8_t afcex_l,uint8_t afcex_h)
{
  int32_t error;

  error = MB86667_I2C_WRITE(MB86667_REG_AFCCOFS_L,afcex_l);
  if(error!=FE_OK)
    return error;

  return MB86667_I2C_WRITE(MB86667_REG_AFCCOFS_H,afcex_h);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Seq_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Seq_Set(uint8_t loop)
{
  return MB86667_I2C_WRITE(MB86667_REG_SEQ_MODE_SET, (uint8_t)SLOCK1|(uint8_t)(loop<<SHIFT_SLOCK1)|IQ_DET_BY_FRAME_SYNC);
//  return MB86667_I2C_WRITE(MB86667_REG_SEQ_MODE_SET, (uint8_t)SLOCK1|(uint8_t)(loop<<SHIFT_SLOCK1));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Eq_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Eq_Set(uint8_t en,uint8_t gain)
{
  return MB86667_I2C_WRITE(MB86667_REG_EQL,(uint8_t)(gain<<SHIFT_ESHIFT_0_3)|en);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Dgac_Data_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Dagc_Data_Set(uint8_t dagca, uint8_t dagcw)
{
  return MB86667_I2C_WRITE(MB86667_REG_DIGITAL_AGC, (uint8_t)(dagca<<SHIFT_DAGCA_0_1)|dagcw);
}

/*----------------------------------------------------------------------------
Name:        MB86667_En_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_En_Set(uint8_t cren, uint8_t afcen)
{
  uint8_t val;
  val = (uint8_t)(MB86667_BIT_NFEN|MB86667_BIT_DAGCEN|MB86667_BIT_DECIEN|MB86667_BIT_INTPEN|MB86667_BIT_DRCEN) | \
        (uint8_t)(cren<<MB86667_BIT_SHIFT_CREN)|(uint8_t)(afcen<<MB86667_BIT_SHIFT_AFCEN);
  return MB86667_I2C_WRITE(MB86667_REG_THROUGH_MODE, val);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Afcexen_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Afcexen_Set(uint8_t afcexen, uint16_t smrt)
{
  uint8_t afca;

  if     ( smrt>18875 ) afca = 4 ;
  else if( smrt> 9375 ) afca = 3 ;
  else if( smrt> 2250 ) afca = 2 ;
  else                  afca = 1 ;

  return MB86667_I2C_WRITE(MB86667_REG_AFC_SET, (uint8_t)0x02|(uint8_t)(afcexen<<SHIFT_AFCEXEN)|(uint8_t)(afca<<SHIFT_AFCA_0_2));
}

/*----------------------------------------------------------------------------
Name:        MB86667_Cntm_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Cntm_Set(uint8_t timint1, uint8_t timint2,uint8_t cnext)
{
  uint8_t val ;

  val = (uint8_t)(timint1<<SHIFT_TIMINT1_0_1)|(uint8_t)(timint2<<SHIFT_TIMINT2_0_1)|cnext;
  return MB86667_I2C_WRITE(MB86667_REG_CN_TIMER, val);
}

/*----------------------------------------------------------------------------
Name:        MB86667_S01t_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_S01t_Set(uint8_t s1t, uint8_t s0t )
{
  return MB86667_I2C_WRITE(MB86667_REG_S0_S1_TIM, (uint8_t)(s1t << SHIFT_S1T_0_3)|s0t);
}

/*----------------------------------------------------------------------------
Name:        MB86667_S2t_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_S2t_Set(uint8_t s2t )
{
  return MB86667_I2C_WRITE(MB86667_REG_S2_S3_TIM, (uint8_t)0x70|s2t);

}

/*----------------------------------------------------------------------------
Name:        MB86667_S45t_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_S45t_Set(uint8_t s4t, uint8_t s5t )
{
  return MB86667_I2C_WRITE(MB86667_REG_S4_S5_TIM, (uint8_t)(s5t << SHIFT_S5T_0_4)|s4t);

}

/*----------------------------------------------------------------------------
Name:        MB86667_Sync_Chk2
-------------------------------------------------------------------------------*/
int32_t MB86667_Sync_Chk2(uint8_t *virm, uint8_t *sync)
{
  uint8_t    val;
  int32_t  error;

  error = MB86667_I2C_READ(MB86667_REG_STAT, &val);
  if(error != FE_OK)
    return error;

  *sync = val & (uint8_t)0x01 ;
  *virm = (uint8_t)((val& (uint8_t)0x1c)>>2) ;
  return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Sync_Chk1
-------------------------------------------------------------------------------*/
int32_t MB86667_Sync_Chk1(uint8_t *sync)
{
  uint8_t    val;
  int32_t  error;

  error = MB86667_I2C_READ(MB86667_REG_STAT, &val);
  if(error != FE_OK)
    return error;
  *sync = val & (uint8_t)0x01 ;
  return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Smrt_Info_Get
-------------------------------------------------------------------------------*/
void MB86667_Smrt_Info_Get(uint16_t sr, uint8_t *deci, uint8_t *csel, uint8_t *rsel, int32_t *clkmst )
{
   if(      sr>=37501 ) { *deci=0 ; *csel=0 ; *rsel=0 ; }
   else if( sr>=30001 ) { *deci=0 ; *csel=0 ; *rsel=1 ; }
   else if( sr>=26251 ) { *deci=0 ; *csel=1 ; *rsel=0 ; }
   else if( sr>=22501 ) { *deci=0 ; *csel=1 ; *rsel=1 ; }
   else if( sr>=18751 ) { *deci=1 ; *csel=0 ; *rsel=0 ; }
   else if( sr>=15001 ) { *deci=1 ; *csel=0 ; *rsel=1 ; }
   else if( sr>=13126 ) { *deci=1 ; *csel=1 ; *rsel=0 ; }
   else if( sr>=11251 ) { *deci=1 ; *csel=1 ; *rsel=1 ; }
   else if( sr>= 9376 ) { *deci=2 ; *csel=0 ; *rsel=0 ; }
   else if( sr>= 7501 ) { *deci=2 ; *csel=0 ; *rsel=1 ; }
   else if( sr>= 6563 ) { *deci=2 ; *csel=1 ; *rsel=0 ; }
   else if( sr>= 5626 ) { *deci=2 ; *csel=1 ; *rsel=1 ; }
   else if( sr>= 4688 ) { *deci=3 ; *csel=0 ; *rsel=0 ; }
   else if( sr>= 3751 ) { *deci=3 ; *csel=0 ; *rsel=1 ; }
   else if( sr>= 3282 ) { *deci=3 ; *csel=1 ; *rsel=0 ; }
   else if( sr>= 2814 ) { *deci=3 ; *csel=1 ; *rsel=1 ; }
   else if( sr>= 2344 ) { *deci=4 ; *csel=0 ; *rsel=0 ; }
   else if( sr>= 1876 ) { *deci=4 ; *csel=0 ; *rsel=1 ; }
   else if( sr>= 1641 ) { *deci=4 ; *csel=1 ; *rsel=0 ; }
   else if( sr>= 1407 ) { *deci=4 ; *csel=1 ; *rsel=1 ; }
   else if( sr>= 1172 ) { *deci=5 ; *csel=0 ; *rsel=0 ; }
   else if( sr>=  939 ) { *deci=5 ; *csel=0 ; *rsel=1 ; }
   else if( sr>=  821 ) { *deci=5 ; *csel=1 ; *rsel=0 ; }
   else                 { *deci=5 ; *csel=1 ; *rsel=1 ; }

  if( *csel==0 ) *clkmst = 92000 ;
  else           *clkmst = 61333 ;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Smrt_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Smrt_Set(uint16_t sr, uint8_t deci, uint8_t csel, uint8_t rsel, int32_t clkmst)
{
  int32_t   tmp, m;
  uint8_t    stofs0, stofs1 ;
  int32_t  error;

  m = 1 << deci;

  tmp = (int32_t)(8192*clkmst - 2*m*sr*8192 + clkmst/2)/clkmst ;

  stofs0 =  (uint8_t)(tmp & 0x0FF);
  stofs1 = (uint8_t)((tmp & 0xF00) >> 8);

  error = MB86667_I2C_WRITE(MB86667_REG_RATE_ROUGH, \
             (uint8_t)(deci << MB86667_BIT_RATE_ROUGH_SHIFT_DECI2)|(uint8_t)(csel << MB86667_BIT_RATE_ROUGH_SHIFT_CSEL)|rsel);
  if(error != FE_OK)
       return error;
  /* save symbol rate and clockmaster for context switch */
  mb86667_sym_rate(SET, sr);
  mb86667_clock_fac(SET, csel);

  error = MB86667_I2C_WRITE(MB86667_REG_RATE_ACCUR1, stofs0);
  if(error != FE_OK)
       return error;

  return MB86667_I2C_WRITE(MB86667_REG_RATE_ACCUR2, stofs1);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Swp_Info_Get
-------------------------------------------------------------------------------*/
void MB86667_Swp_Info_Get(int32_t fOSC_start, int32_t clkmst, int8_t v,
                           int16_t rf_freq_step, int32_t swp_ofs, int32_t *fOSC_10,
                           int32_t *afcex_freq, uint8_t *afcex_l, uint8_t *afcex_h )
{
  int32_t afcex ;
  int32_t crnt_swp_freq ;

  crnt_swp_freq = fOSC_start*1000 + v*swp_ofs ;

  if     ( rf_freq_step==2000 )
    *fOSC_10 = (crnt_swp_freq  +1000)/2000*20 ;
  else if( rf_freq_step==1000 )
    *fOSC_10 = (crnt_swp_freq  + 500)/1000*10 ;
  else if( rf_freq_step== 500 )
    *fOSC_10 = (crnt_swp_freq*2+ 500)/1000*10/2 ;

  *afcex_freq = crnt_swp_freq - *fOSC_10*100 ;

  #if( TUNER_POLARITY == 1 )
    afcex =  *afcex_freq * 8192/clkmst;
  #else
    afcex = -*afcex_freq * 8192/clkmst;
  #endif
  *afcex_l = (uint8_t) (afcex & 0x00ff);
  *afcex_h = (uint8_t)((afcex & 0x0f00) >> 8);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Swp_Info_Get2
-------------------------------------------------------------------------------*/
void MB86667_Swp_Info_Get2(int16_t rf_freq_step,int32_t swp_freq, int32_t clkmst, int32_t *afcex_freq,
              int32_t *fOSC_10, uint8_t *afcex_l, uint8_t *afcex_h )
{
  int32_t afcex ;

  if(      rf_freq_step==2000 ) *fOSC_10 = (swp_freq  +1000)/2000*20 ;
  else if( rf_freq_step==1000 ) *fOSC_10 = (swp_freq  + 500)/1000*10 ;
  else if( rf_freq_step== 500 ) *fOSC_10 = (swp_freq*2+ 500)/1000*10/2 ;

  *afcex_freq = swp_freq - *fOSC_10*100;

  #if( TUNER_POLARITY == 1 )
    afcex =  *afcex_freq*8192/clkmst;
  #else
    afcex = -*afcex_freq*8192/clkmst;
  #endif
  *afcex_l =  (uint8_t)(afcex & 0x00ff) ;
  *afcex_h = (uint8_t)((afcex & 0x0f00)>>8) ;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Srst
-------------------------------------------------------------------------------*/
int32_t MB86667_Srst(void)
{
  return MB86667_I2C_WRITE(MB86667_REG_RESET, MB86667_BIT_RESET_QP);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Vi_Set
-------------------------------------------------------------------------------*/
int32_t MB86667_Vi_Set(uint8_t eth, uint8_t via)
{
    int32_t  error;

    error = MB86667_I2C_WRITE(MB86667_REG_VIT_ERR_OFFS, eth);
    if(error != FE_OK)
        return error;

    error = MB86667_I2C_WRITE(MB86667_REG_VIT_PROT_VAL, (uint8_t)0xF0|via);
    if(error != FE_OK)
        return error;

    return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_WAIT
-------------------------------------------------------------------------------*/
void MB86667_WAIT(int32_t time)
{
  MB86667_User_Sleep((uint32_t) time);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Swp_Freq_Calculation
-------------------------------------------------------------------------------*/
int32_t MB86667_Swp_Freq_Calculation( int32_t i, int8_t v, int16_t *va,  int8_t vmax, int8_t vmin,
              uint8_t sigmin, int32_t fOSC_10, int32_t afcex_freq, int32_t swp_ofs, uint8_t *sig1)
{
    int32_t swp_freq ;

    if( (i%2==1)&&(v<=vmax))
    {   /* positive v */
        if( (v-1==vmin) && (va[30+v]>=0) && (va[29+v]>=0) && (va[29+v] > va[30+v]) \
            && (va[29+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 + afcex_freq - swp_ofs;
            *sig1 = va[29+v];
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "<<pos 1>>\n");
                MB86667_User_Print(PrintHelp);
            #endif
        }
        else if((v==vmax) && (va[30+v]>=0) && (va[29+v]>=0) && (va[30+v] > va[29+v]) && (va[30+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 + afcex_freq;
            *sig1 = va[30+v];
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "<<pos 2>>\n");
                MB86667_User_Print(PrintHelp);
            #endif
        }
        else if( (va[30+v] > 0) && (va[29+v] > 0) && (va[28+v] > 0) && (va[27+v] > 0) \
                 && (va[29+v] > va[30+v]) && (va[28+v] > va[27+v]) \
                 && ((va[29+v] > (int16_t)sigmin) || (va[28+v] > (int16_t)sigmin)) )
        {
            if(va[29+v]>=va[28+v])
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs ;
                *sig1 = va[29+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<pos 3_1>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs*2 ;
                *sig1 = va[28+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<pos 3_2>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
        }
        else if( (v==vmax) && (va[30+v] >= 0) && (va[29+v] >= 0) && (va[28+v] >= 0) \
                 && (va[30+v] > va[28+v]) && (va[29+v] > va[28+v]) \
                 && ( (va[30+v] > (int16_t)sigmin) || (va[29+v] > (int16_t)sigmin)) )
        {
            if(va[30+v]>=va[29+v])
            {
                swp_freq = fOSC_10*100 + afcex_freq ;
                *sig1 = va[30+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<pos 4_1>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs ;
                *sig1 = va[29+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<pos 4_2>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
        }
        else swp_freq = -1 ;
    }
    else if((i%2==0)&&(v>=vmin))
    {   /* negative v */
        if( (va[30+v] > 0) && (va[31+v] > 0) && (va[32+v] > 0) && (va[31+v] > va[30+v]) && (va[31+v] > va[32+v])
            && (va[31+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 + afcex_freq + swp_ofs ;
            *sig1 = va[31+v];
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "<<neg 1>>\n");
                MB86667_User_Print(PrintHelp);
            #endif
        }
        else if((v+1==vmax) && (va[30+v] >= 0) && (va[31+v] >= 0) && (va[31+v] > va[30+v]) && (va[31+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 + afcex_freq + swp_ofs ;
            *sig1 = va[30+v];
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "<<neg 2>>\n");
                MB86667_User_Print(PrintHelp);
            #endif
        }
        else if( (v==vmin) && (va[30+v] > 0) && (va[31+v] > 0) && (va[32+v] > 0) \
                 && (va[30+v] > va[31+v]) && (va[30+v] > va[32+v]) && (va[30+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 + afcex_freq ;
            *sig1 = va[30+v];
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "<<neg 3>>\n");
                MB86667_User_Print(PrintHelp);
            #endif
        }
        else if( (va[30+v]>=0) && (va[31+v]>=0) && (va[32+v]>=0) && (va[33+v]>=0)
                 && (va[31+v] > va[30+v]) && (va[32+v] > va[33+v])
                 && ((va[31+v] > (int16_t)sigmin) || (va[32+v] > (int16_t)sigmin)) )
        {
            if(va[31+v] >= va[32+v])
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs ;
                *sig1 = va[31+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 4_1>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs*2 ;
                *sig1 = va[32+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 4_2>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
        }
        else if( (va[30+v]>=0) && (va[31+v]>=0) && (va[32+v]>=0) && (va[33+v]>=0) && (va[30+v] > va[32+v]) \
                  && (va[31+v] > va[32+v]) && (va[30+v] > va[33+v]) && (va[31+v] > va[33+v]) \
                  && ((va[30+v] > (int16_t)sigmin) || (va[31+v] > (int16_t)sigmin)) )
        {
            if(va[30+v]>=va[31+v])
            {
                swp_freq = fOSC_10*100 + afcex_freq ;
                *sig1 = va[30+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 5_1>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs ;
                *sig1 = va[31+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 5_2>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
        }
        else if( (v+2==vmin) && (va[30+v] >= 0) && (va[31+v] >= 0) && (va[32+v] >= 0) \
                  && (va[31+v] > va[30+v]) && (va[32+v] > va[30+v]) \
                  && ((va[31+v] > (int16_t)sigmin) || (va[32+v] > (int16_t)sigmin)) )
        {
            if(va[31+v]>=va[32+v])
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs ;
                *sig1 = va[31+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 6_1>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs*2 ;
                *sig1 = va[32+v];
                #ifdef FAPI_FE_MB86667_DEBUG02
                    sprintf(PrintHelp, "<<neg 6_2>>\n");
                    MB86667_User_Print(PrintHelp);
                #endif
            }
        }
        else if((vmax==0) && (vmin==0) && (va[30+v] > (int16_t)sigmin))
        {
            swp_freq = fOSC_10*100 ;
            *sig1 = va[30+v];
        }
        else swp_freq = -1 ;
    }
    else swp_freq = -1 ;

    return swp_freq ;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Abs
-------------------------------------------------------------------------------*/
int32_t MB86667_Abs(int32_t a)
{
    if(a>=0)
        return a;
    else
        return (0-a);
}

/*----------------------------------------------------------------------------
Name:        MB86667_Signal_Det
-------------------------------------------------------------------------------*/
int32_t MB86667_Signal_Det(uint16_t smrt, uint8_t *deci, uint8_t *csel, uint8_t *rsel, int32_t *clkmst, uint8_t *sig, FBOOL *ret )
{
    uint16_t         smrtd ;
    int32_t         wait_sym,wait_t ;
    uint8_t          si[3],i ;
    int32_t        error;

    if( *sig > 45 )
    {
        error = MB86667_Cntm_Set( 2, 1, 2 );
        if(error!= FE_OK)
            return error;
        wait_sym = 40000 ;
    }
    else
    {
        error = MB86667_Cntm_Set( 3, 1, 2 );
        if(error!= FE_OK)
            return error;
        wait_sym = 80000 ;
    }

    for( i=0;i<3;i++)
    {
        if(      i==0 ) smrtd = (uint16_t)((uint32_t)(smrt*98)/100);
        else if( i==1 ) smrtd = smrt;
        else            smrtd = (uint16_t)((uint32_t)(smrt*102)/100);
        MB86667_Smrt_Info_Get( smrtd, deci, csel, rsel, clkmst);
        error = MB86667_Smrt_Set(smrtd, *deci, *csel, *rsel, *clkmst );
        if(error!= FE_OK)
            return error;
        error = MB86667_Srst();
        if(error!= FE_OK)
            return error;

        wait_t = (int32_t)(wait_sym+(int32_t)(99*(int32_t)smrtd/100))/(int32_t)smrtd ;
        if( wait_t==0 )
            wait_t = 1 ;
        MB86667_WAIT(wait_t);

        error = MB86667_I2C_READ(MB86667_REG_SIG_1, &(si[i]));
        if(error!= FE_OK)
            return error;

        #ifdef FAPI_FE_MB86667_DEBUG02
            sprintf(PrintHelp, "\nsmrt =%d, sig1 =%d",smrtd,si[i]);
            MB86667_User_Print(PrintHelp);
        #endif
    }
    if( (si[1] > (uint8_t)((uint16_t)(si[0]*112)/100)) &&
        (si[1] > (uint8_t)((uint16_t)(si[2]*112)/100)) &&
        (si[1] > 10) )
    {
        *ret = FTRUE ;
        #ifdef FAPI_FE_MB86667_DEBUG02
            sprintf(PrintHelp, "si[1] OK\n");
            MB86667_User_Print(PrintHelp);
        #endif
    }
    else
    {
        *ret = FFALSE ;
        #ifdef FAPI_FE_MB86667_DEBUG02
            sprintf(PrintHelp, "si[1] NG:Too small\n");
            MB86667_User_Print(PrintHelp);
        #endif
    }
    *sig = si[1] ;

    error = MB86667_Cntm_Set( 0, 1, 2 );

    return error ;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Afcex_Info_Get
-------------------------------------------------------------------------------*/
void MB86667_Afcex_Info_Get(int32_t clkmst, int32_t afcex_freq, uint8_t *afcex_l,uint8_t *afcex_h)
{
    int32_t afcex ;

#if( TUNER_POLARITY == 1 )
    afcex =  afcex_freq * 8192/clkmst;
#else
    afcex = -afcex_freq * 8192/clkmst;
#endif
    *afcex_l =  (uint8_t)(afcex&0x00ff) ;
    *afcex_h = (uint8_t)((afcex&0x0f00)>>8) ;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Freqerr_Chk
-------------------------------------------------------------------------------*/
int32_t MB86667_Freqerr_Chk( uint32_t fTP, int32_t fOSC_10, uint16_t smrt, uint8_t unit, int32_t *freqerr)
{
    uint8_t        crm2, afcml, afcmh ;
    uint8_t        temp1 ;
    uint8_t        deci, csel, rsel ;
    int32_t       clkmst ;
    int32_t       crm, afcm1, afcm2 ;
    int32_t       crrerr, afcerr ;
    int32_t       afcen, afcexen ;
    int32_t      error;

    error = MB86667_I2C_READ(MB86667_REG_CRM, &crm2);
    if(error!= FE_OK)
        return error;

    if( crm2 > 127 ) crm = crm2 - 256 ;
    else             crm = crm2 ;
    crrerr = (int32_t)(TUNER_POLARITY*smrt*crm)/256 ;
    #ifdef FAPI_FE_MB86667_DEBUG02
        sprintf(PrintHelp, "\ncrrerr[kHz]= %d",crrerr);
        MB86667_User_Print(PrintHelp);
    #endif

    error = MB86667_I2C_READ(MB86667_REG_AFC_SET ,&temp1);
    if(error!= FE_OK)
        return error;
    afcexen = (temp1 & 0x20) >> 5;
    if( afcexen==0 )
    {
        error = MB86667_I2C_READ(MB86667_REG_THROUGH_MODE, &temp1);
        if(error!= FE_OK)
            return error;
        afcen = (temp1 & 0x04) >> 2;
    }
    else
        afcen = 0 ;

    if( afcexen == 1 )
    {
        error = MB86667_I2C_READ(MB86667_REG_AFCIL, &afcml);
        if(error!= FE_OK)
            return error;
        error = MB86667_I2C_READ(MB86667_REG_AFCIH, &afcmh);
        if(error!= FE_OK)
            return error;
    }
    else if( afcen == 1 )
    {
        error = MB86667_I2C_READ(MB86667_REG_AFCML, &afcml);
        if(error!= FE_OK)
            return error;
        error = MB86667_I2C_READ(MB86667_REG_AFCMH, &afcmh);
        if(error!= FE_OK)
            return error;
    }

    if( (afcen == 1) || (afcexen == 1) )
    {
        MB86667_Smrt_Info_Get( smrt, &deci, &csel, &rsel, &clkmst );
        afcm2 = ((afcmh&0x0f) << 8) + afcml ;
        if( afcm2 > 2047 )
            afcm1 = afcm2 - 4096 ;
        else
            afcm1 = afcm2 ;
        afcerr = TUNER_POLARITY * (afcm1 * clkmst / 8192) ;
        #ifdef FAPI_FE_MB86667_DEBUG02
            sprintf(PrintHelp, "\nafcex[kHz]= %d",afcerr);
            MB86667_User_Print(PrintHelp);
        #endif
    }
    else
        afcerr = 0;

    if( (afcen == 0) && (afcexen == 0) )
    {
        error = MB86667_I2C_READ(MB86667_REG_AFCCOFS_L, &afcml);
        if(error!= FE_OK)
            return error;
        error = MB86667_I2C_READ(MB86667_REG_AFCCOFS_H, &afcmh);
        if(error!= FE_OK)
            return error;
        afcm2 = ((afcmh&0x0f) << 8) + afcml ;
        if( afcm2 > 1023 )
            afcm1 = afcm2 - 2048 ;
        else
            afcm1 = afcm2 ;
        afcerr = TUNER_POLARITY * (afcm1 * clkmst / 8192) ;
        #ifdef FAPI_FE_MB86667_DEBUG02
            sprintf(PrintHelp, "\nafcex[kHz]= %d",afcerr);
            MB86667_User_Print(PrintHelp);
        #endif
    }
    #ifdef FAPI_FE_MB86667_DEBUG02
        sprintf(PrintHelp, "\nfTP[MHz] = %d",fTP);
        MB86667_User_Print(PrintHelp);
    #endif

    if(unit==0)
    {   /* MHz */
        if( (int16_t)crrerr + (int16_t)afcerr - (int16_t)fTP*1000 + (int16_t)fOSC_10*100 >= 0 )
            *freqerr = (crrerr + afcerr - fTP*1000 + fOSC_10*100 + 500)/1000 ;
        else
            *freqerr = (crrerr + afcerr - fTP*1000 + fOSC_10*100 - 500)/1000 ;
    }
    else
    {   /* KHz */
        *freqerr = crrerr + afcerr - fTP*1000 + fOSC_10*100;
    }
    #ifdef FAPI_FE_MB86667_DEBUG02
        sprintf(PrintHelp, "\ncrrerr= %d, afcerr = %d", crrerr, afcerr);
        MB86667_User_Print(PrintHelp);
        sprintf(PrintHelp, "\nfreqerr[kHz] = %d", *freqerr);
        MB86667_User_Print(PrintHelp);
    #endif

    return FE_OK;
}

/*----------------------------------------------------------*/

/*----------------------------------------------------------------------------
Name:        MB86667_Tune
Description: lock carrier
-------------------------------------------------------------------------------*/
int32_t MB86667_DoTune(MB86667_SET_PARAM* tp_param, uint8_t mode, FBOOL quicklock)
{
   uint8_t   ucReg;
   uint8_t   deci, csel, rsel;
   uint32_t  frequency;
   uint16_t  symbolrate;
   int32_t  clkmst;
   int32_t error;

   /* for quicklock option */
   //int32_t  tmp, m;
   uint8_t   dagca, dagcw, agcw, vmag, agcr;

   /* for normal lock */
   uint8_t   loop, virm;
   uint8_t   afcex_l, afcex_h, sig1, signal_dupl, sync, sync2;
   uint8_t   s2t, s4t, s5t, eth, via;
   int8_t   vmax, vmin, vmax_his, vmin_his, v;
   int16_t  va[60];
   int32_t  swp_ofs, fOSC_start, ftemp, wait_t, prev_freq_num, fOSC_10;
   int32_t  i, j, swp_freq, afcex_freq, prev_swp_freq[20];
   FBOOL signal;

   /* check if function pointers are set */
   if (mb86667_Rf_Val_Set_Ini == NULL)
      return(FE_ERR_FE_OTHER);
   if (mb86667_Rf_Val_Set == NULL)
      return(FE_ERR_FE_OTHER);

   frequency  = tp_param -> frequency / 1000;
   symbolrate = tp_param -> symbolrate;
   if ((frequency < 950) || (frequency > 2150) || (symbolrate < 1000) || (symbolrate > 45000))
      return FE_ERR_BAD_PARAMETER;
   tp_param->freqerr = 0; /* preset frequency error */

   if(quicklock == FTRUE)
   {
      /* Simple control routines, similiar to the MB86A15
         but not optimal under difficult reception conditions */

      /* tuner settings */
      uint8_t tuner_tries=4;
      error=FE_ERR_FE_OTHER;
      while(((tuner_tries--)>0) && ((error = mb86667_Rf_Val_Set_Ini(tp_param->frequency / 100, symbolrate))!=FE_OK))
          MB86667_WAIT( 1000 ) ;
      if (error != FE_OK)
         return(FE_ERR_FE_OTHER);

////NEW_API
      error = MB86667_Initial_Set();
      if (error != FE_OK)
         return(FE_ERR_FE_I2C);
      //error = MB86667_Initial_All();
      //if(error!=FE_OK)
      //   return error;

         //ADI: notwendig ????
         //    error = MB86667_Cntm_Set(0,1,2);
         //    if(error!=FE_OK)
         //       return error;

         //ADI: notwendig ????
         error = MB86667_Eq_Set( 1, 0 ) ;
         if (error!=FE_OK)
            return error;
                         
           {
             uint8_t time_factor = 2;

             //if     ( smrt>18875 ) time_factor = 2 ;
             //else if( smrt> 9375 ) time_factor = 2 ;
             //else if( smrt> 2250 ) time_factor = 2 ;
             //else                  time_factor = 2 ;
            
             error = MB86667_S01t_Set(time_factor,time_factor);
             if(error!=FE_OK)
                return error;
             error = MB86667_S2t_Set(time_factor);
             if(error!=FE_OK)
                return error;
             error = MB86667_S45t_Set(time_factor,time_factor);
             if(error!=FE_OK)
                return error;

             error = MB86667_Seq_Set( 1 ) ;
             if(error!=FE_OK)
                return error;
         }

//      /* initial settings */
//      ucReg = SHIFT_S1T_0_3;
//      error = MB86667_I2C_Write(MB86667_REG_S0_S1_TIM, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//      ucReg = 0;
//      error = MB86667_I2C_Write(MB86667_REG_CN_TIMER, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//      ucReg = 0x35;
//      error = MB86667_I2C_Write(MB86667_REG_SIG_1, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//      ucReg = 0;
//      error = MB86667_I2C_Write(MB86667_REG_AFCIH, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//      ucReg = 0x02;
//      error = MB86667_I2C_Write(MB86667_REG_SIG_2, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//
//      ucReg = 0x04;
//      error = MB86667_I2C_Write(MB86667_REG_VIT_ERR_OFFS, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);
//      ucReg = 0x0a;
//      error = MB86667_I2C_Write(MB86667_REG_AFC_3_0, 1, &ucReg);
//      if (error != FE_OK)
//         return(FE_ERR_FE_I2C);

      /* EN_set */
      //ucReg = MB86667_BIT_DRCEN | MB86667_BIT_INTPEN | MB86667_BIT_DECIEN | MB86667_BIT_DAGCEN |
      //        MB86667_BIT_NFEN  | MB86667_BIT_CREN   | MB86667_BIT_AFCEN;
      //error = MB86667_I2C_Write(MB86667_REG_THROUGH_MODE, 1, &ucReg);
      error = MB86667_En_Set(1,1);
      if (error != FE_OK)
         return(FE_ERR_FE_I2C);

      /* set symbol rate */

      //if(      symbolrate>37750 ) { deci=0 ; csel=0 ; rsel=0 ; }
      //else if( symbolrate>30500 ) { deci=0 ; csel=0 ; rsel=1 ; }
      //else if( symbolrate>26625 ) { deci=0 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate>22750 ) { deci=0 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate>18875 ) { deci=1 ; csel=0 ; rsel=0 ; }
      //else if( symbolrate>15000 ) { deci=1 ; csel=0 ; rsel=1 ; }
      //else if( symbolrate>13125 ) { deci=1 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate>11250 ) { deci=1 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate> 9375 ) { deci=2 ; csel=0 ; rsel=0 ; }
      //else if( symbolrate> 7500 ) { deci=2 ; csel=0 ; rsel=1 ; }
      //else if( symbolrate> 6500 ) { deci=2 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate> 5500 ) { deci=2 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate> 4625 ) { deci=3 ; csel=0 ; rsel=0 ; }
      //else if( symbolrate> 3750 ) { deci=3 ; csel=0 ; rsel=1 ; }
      //else if( symbolrate> 3250 ) { deci=3 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate> 2750 ) { deci=3 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate> 2250 ) { deci=4 ; csel=0 ; rsel=0 ; }
      //else if( symbolrate> 1850 ) { deci=4 ; csel=0 ; rsel=1 ; }
      //else if( symbolrate> 1500 ) { deci=4 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate> 1250 ) { deci=4 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate> 1125 ) { deci=4 ; csel=1 ; rsel=0 ; }
      //else if( symbolrate>=1000 ) { deci=4 ; csel=1 ; rsel=1 ; }
      //else if( symbolrate>  875 ) { deci=5 ; csel=0 ; rsel=0 ; }
      //else                       { deci=5 ; csel=1 ; rsel=1 ; }
      //
      //if( csel==0 )
      //   clkmst = 92000 ;
      //else
      //   clkmst = 61333 ;
      
      //m = 1<<deci ;
      //tmp = ((int32_t)(8192 * clkmst - m*symbolrate*2*8192 + clkmst/2)/ clkmst);
      //
      //ucReg = (uint8_t)((deci<<2)|(csel<<1)|rsel);
      //error = MB86667_I2C_Write(MB86667_REG_RATE_ROUGH, 1, &ucReg);
      //if (error != FE_OK)
      //   return(FE_ERR_FE_I2C);
      //ucReg = (uint8_t)(tmp & 0x0ff); /* STOFS0 */
      //error = MB86667_I2C_Write(MB86667_REG_RATE_ACCUR1, 1, &ucReg);
      //if (error != FE_OK)
      //   return(FE_ERR_FE_I2C);
      //ucReg = (uint8_t)((tmp & 0xf00) >> 8); /* STOFS1 */
      //error = MB86667_I2C_Write(MB86667_REG_RATE_ACCUR2, 1, &ucReg);
      //if (error != FE_OK)
      //   return(FE_ERR_FE_I2C);

      // call to save values for later parameter calculations
      MB86667_Smrt_Info_Get(symbolrate, &deci, &csel, &rsel, &clkmst);
      error = MB86667_Smrt_Set(symbolrate, deci, csel, rsel, clkmst);
      if (error != FE_OK)
         return error;


      /* DAGC set */
      agcr = 0;
      /* use different AGC settings for low and high baud rates */
      if( symbolrate >= 8000 )
      {
         dagca = 0 ;
         dagcw = 2 ;
         agcw  = 5 ;
         vmag  = 6 ;
      }
      else
      {
         dagca = 0 ;
         dagcw = 1 ;
         agcw  = 6 ;
         vmag  = 7 ;
      }
      //ucReg = (uint8_t)((dagca << 3) | dagcw);
      //if( MB86667_I2C_Write(MB86667_REG_DIGITAL_AGC, 1, &ucReg) != FE_OK )
      if( MB86667_Dagc_Data_Set(dagca, dagcw) != FE_OK )
      {   /* DIGITAL AGCA */
         return(FE_ERR_FE_I2C);
      }
      //ucReg = (uint8_t)((agcw << 2) | AGCP | agcr);
      //if( MB86667_I2C_Write(MB86667_REG_FAGC, 1, &ucReg) != FE_OK )
      if (MB86667_AGC_SET(agcw) != FE_OK )
      {   /* AGC */
         return(FE_ERR_FE_I2C);
      }
      
      ucReg = (uint8_t)(vmag << 3);
      if( MB86667_I2C_Write(MB86667_REG_VIT_GAIN, 1, &ucReg) != FE_OK )
      {   /* vmag, QMAG */
         return(FE_ERR_FE_I2C);
      }

      /* reset */
      //ucReg = MB86667_BIT_RESET_QP;
      //if( MB86667_I2C_Write(MB86667_REG_RESET, 1, &ucReg) != FE_OK )
      if( MB86667_Srst() != FE_OK )
          return(FE_ERR_FE_I2C);
      if( MB86667_Fec_Srst() != FE_OK )
          return(FE_ERR_FE_I2C);

      /* iq_vt_set */
      //ucReg = IQRES | IQAT | VIR4 | VIR3 | VIR2 | VIR1 | VIR0;
      //if( MB86667_I2C_Write(MB86667_REG_IQ_AUTODET, 1, &ucReg) != FE_OK )
      if( MB86667_Iq_Vt_Set(1) != FE_OK ) 
         return(FE_ERR_FE_I2C);

      //error = MB86667_Freqerr_Chk( frequency, 0, symbolrate, 1, &tp_param->freqerr);
      //if (error != FE_OK)
      //   return(error);
   } /* if(quicklock == FTRUE) */
   else
   {
      /* Optimised for difficult reception conditions */
      swp_ofs = symbolrate/4;
      for(i=0;i<60;i++)
        va[i] = -1;
      for(i=0;i<20;i++)
         prev_swp_freq[i] = 0;
  
      error = MB86667_Initial_All();
      if(error!=FE_OK)
         return error;
  
      error = MB86667_Afcexen_Set(1,symbolrate);
      if(error!=FE_OK)
         return error;
  
      error = MB86667_Cntm_Set(0,1,2);
      if(error!=FE_OK)
         return error;
  
      error = MB86667_S01t_Set(0,0);
      if(error!=FE_OK)
         return error;
  
      MB86667_Smrt_Info_Get(symbolrate, &deci, &csel, &rsel, &clkmst);
  
      error = MB86667_Smrt_Set(symbolrate, deci, csel, rsel, clkmst);
      if (error != FE_OK)
         return error;
  
      #if (RF_FRE_STEP == 2000)
          if( frequency%2==0 )
              fOSC_start = frequency ;
          else
          {
              fOSC_start = frequency+1 ;
              if( fOSC_start>2150 ) fOSC_start = frequency-1 ;
          }
      #else
          fOSC_start = frequency;
      #endif
      loop = 1 ;
      ftemp = fOSC_start*1000 ;
      vmax = 0 ;
      while( loop == 1 )
      {
          ftemp = ftemp + swp_ofs ;
          vmax++ ;
          if( ftemp>2150000 )
          {
              loop = 0 ;
              vmax-- ;
          }
          else if ((int32_t)(symbolrate/9) > FCP )
          {
              loop = 0 ;
              vmax = 0 ;
          }
          else if ( (ftemp == 2150000) || ((int32_t)(ftemp-frequency*1000) >= (int32_t)(FCP-symbolrate/9)) )
              loop = 0 ;
      }
      loop = 1 ;
      ftemp = fOSC_start*1000 ;
      vmin = 0 ;
      while( loop == 1 )
      {
          ftemp = ftemp - swp_ofs ;
          vmin-- ;
          if( ftemp < 950000 )
          {
              loop = 0 ;
              vmin++ ;
          }
          else if( (uint16_t)(symbolrate/9) > FCP )
          {
              loop = 0 ;
              vmin = 0 ;
          }
          else if( (ftemp == 950000) || ( (int32_t)(frequency*1000-ftemp) >= (int32_t)(FCP-symbolrate/9)) )
              loop = 0;
      }
      if (mode == 1)
      {   /* for blind scan mode */
          vmin = -2;
          vmax =  2;
      }

      #ifdef FAPI_FE_MB86667_DEBUG02
          sprintf(PrintHelp, "\n vmax = %d\n vmin = %d\n",vmax,vmin);
          MB86667_User_Print(PrintHelp);
      #endif

      wait_t = (8000 + (int32_t)(symbolrate/2))/(int32_t)symbolrate ;
      if( wait_t == 0 )
          wait_t = 1 ;

      i = 0 ;
      prev_freq_num = 0 ;
      loop = 1 ;
      signal = FFALSE ;
      vmax_his = 0 ;
      vmin_his = 0 ;
      v = 0 ;
      while( loop == 1 )
      {
          MB86667_Swp_Info_Get(fOSC_start, clkmst, v, RF_FRE_STEP,
                               swp_ofs, &fOSC_10, &afcex_freq, &afcex_l, &afcex_h );
          if( i == 0 )
          {
              uint8_t tuner_tries=4;
              error=FE_ERR_FE_OTHER;
              while(((tuner_tries--)>0) && ((error = mb86667_Rf_Val_Set_Ini(fOSC_10, symbolrate))!=FE_OK))
                  MB86667_WAIT( 1000 ) ;
              if(error!=FE_OK)
                  return error;
          }
          else
          {
              uint8_t tuner_tries=4;
              error=FE_ERR_FE_OTHER;
              while(((tuner_tries--)>0) && ((error = mb86667_Rf_Val_Set(fOSC_10, symbolrate ))!=FE_OK))
                  MB86667_WAIT( 1000 ) ;
              if(error!=FE_OK)
                  return error;
          }

          error = MB86667_Acfex_Data_Set( afcex_l, afcex_h );
          if(error!=FE_OK)
              return error;

          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\ni =%d, v =%d, fOSC_10 =%d, afcex_freq =%d\n",i,v,fOSC_10,afcex_freq);
              MB86667_User_Print(PrintHelp);
          #endif

          error = MB86667_Srst();
          if(error!=FE_OK)
              return error;

          MB86667_WAIT(wait_t);

          error = MB86667_I2C_READ(MB86667_REG_SIG_1, &sig1);
          if(error!=FE_OK)
              return error;

          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\nsig1 = %d\n",sig1);
              MB86667_User_Print(PrintHelp);
          #endif

          va[30+v] = sig1 ;
          swp_freq = MB86667_Swp_Freq_Calculation( i, v, va, vmax, vmin, SIG1MIN, fOSC_10, afcex_freq, swp_ofs, &sig1 );
          signal_dupl = 0 ;
          for( j = 0 ; j < prev_freq_num ; j++ )
          {
              if( MB86667_Abs(prev_swp_freq[j]-swp_freq) < swp_ofs*3/2)
                  signal_dupl = 1 ;
          }
          if((signal_dupl == 0) && (swp_freq > 0))
          {
              #ifdef FAPI_FE_MB86667_DEBUG02
                  sprintf(PrintHelp, "\nSignal Found: swp_freq = %d, smrt =  %d\n",swp_freq,symbolrate);
                  MB86667_User_Print(PrintHelp);
              #endif

              prev_swp_freq[prev_freq_num] = swp_freq ;
              prev_freq_num++ ;
              MB86667_Swp_Info_Get2(RF_FRE_STEP, swp_freq, clkmst, &afcex_freq, &fOSC_10, &afcex_l, &afcex_h ) ;
              {
                  uint8_t tuner_tries=4;
                  error=FE_ERR_FE_OTHER;
                  while(((tuner_tries--)>0) && ((error = mb86667_Rf_Val_Set(fOSC_10, symbolrate ))!=FE_OK))
                      MB86667_WAIT( 1000 ) ;
              }
              if(error!=FE_OK)
                  return error;

              error = MB86667_Acfex_Data_Set( afcex_l, afcex_h );
              if(error!=FE_OK)
                  return error;

              error = MB86667_Signal_Det(symbolrate, &deci, &csel, &rsel, &clkmst, &sig1, &signal );
              if(error!=FE_OK)
                  return error;
              #ifdef FAPI_FE_MB86667_DEBUG01
                  sprintf(PrintHelp, "\nSignal =  %d\n",signal);
                  MB86667_User_Print(PrintHelp);
              #endif
              if( signal == FTRUE )
                  loop = 0 ;
              else
              {
                  MB86667_Smrt_Info_Get( symbolrate, &deci, &csel, &rsel, &clkmst );
                  error = MB86667_Smrt_Set(symbolrate, deci, csel, rsel, clkmst);
                  if (error != FE_OK)
                  {
                     #ifdef FAPI_FE_MB86667_DEBUG01
                        sprintf(PrintHelp, "\nMB86667_Smrt_Set() failed\n");
                        MB86667_User_Print(PrintHelp);
                     #endif
                     return error;
                  }
              }
          } /* if((signal_dupl == 0) ... */
          else
          {
            #ifdef FAPI_FE_MB86667_DEBUG02
                sprintf(PrintHelp, "\nsingal_found, but duplicated\n");
                MB86667_User_Print(PrintHelp);
            #endif
          } /* if((signal_dupl == 0) ... else */
          if( v >= vmax ) vmax_his = 1 ;
          if( v <= vmin ) vmin_his = 1 ;
          i++ ;
          if( (i % 2 == 1) && (vmax_his == 1) )
              i++ ;
          if( (i % 2 == 0) && (vmin_his == 1) )
              i++ ;
  
          if( i % 2 == 1 )
              v = (int8_t)((i+1)/2) ;
          else
              v = (int8_t)(-i/2) ;
  
          if( (vmax_his==1)&&(vmin_his==1) ) loop = 0;
      } /* while( loop == 1 ) */
  
      #ifdef FAPI_FE_MB86667_DEBUG01
          sprintf(PrintHelp, "\nSignal =  %d\n",signal);
          MB86667_User_Print(PrintHelp);
      #endif
      if( signal == FTRUE )
      {
          error = MB86667_AGC_SET( 2 );
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_S01t_Set( 4, 0 ) ;
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_Dagc_Data_Set( 0, 0 );
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_Eq_Set( 1, 0 ) ;
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          MB86667_Smrt_Info_Get( symbolrate, &deci, &csel, &rsel, &clkmst );
          error = MB86667_Smrt_Set( symbolrate, deci, csel, rsel, clkmst );
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_En_Set(1,1);
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_Afcexen_Set(0, symbolrate);
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
  
          MB86667_Afcex_Info_Get( clkmst, afcex_freq, &afcex_l, &afcex_h );
          error = MB86667_Afcofs_Data_Set( afcex_l, afcex_h );
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
          error = MB86667_Srst() ;
          if(error!=FE_OK)
              return FE_ERR_FE_OTHER;
  
          wait_t = 7 + (int32_t)(10000+symbolrate/2)/(int32_t)symbolrate;
          if( wait_t==0 )
              wait_t = 1 ;
          MB86667_WAIT( wait_t ) ;
  
          error = MB86667_I2C_READ(MB86667_REG_SIG_1,&sig1);
          if(error!=FE_OK)
              return FE_ERR_FE_I2C;
  
          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\nSIG1_center = %d\n",sig1);
              MB86667_User_Print(PrintHelp);
          #endif
  
          if( sig1 > 105 )
          {
              s2t=4 ; s4t=8 ; s5t=8 ; eth=7 ; via=2 ;
              wait_t = 7 + (1245184+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
          }
          else if( sig1 > 85 )
          {
              s2t=5 ; s4t=8 ; s5t=8 ; eth=7 ; via=2 ;
              wait_t = 7 + (1507328+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
          }
          else if( sig1 > 65 )
          {
              s2t=6 ; s4t=8 ; s5t=8 ; eth=7 ; via=2 ;
              wait_t = 7 + (1769472+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
          }
          else
          {
              s2t=7 ; s4t=8 ; s5t=8 ; eth=7 ; via=2 ;
              wait_t = 7 + (2424832+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
          }
  
          error = MB86667_S2t_Set( s2t );
          if(error != FE_OK)
                 return error;
          error = MB86667_S45t_Set( s4t, s5t );
          if(error != FE_OK)
                 return error;
          error = MB86667_Vi_Set( eth, via );
          if(error != FE_OK)
                 return error;
          error = MB86667_Srst();
          if(error != FE_OK)
                 return error;
  
          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\nwait time %d [ms]\n",wait_t);
              MB86667_User_Print(PrintHelp);
          #endif
  
          MB86667_WAIT( wait_t ) ;
  
          error = MB86667_Sync_Chk2(&virm, &sync);
          if(error != FE_OK)
              return FE_ERR_FE_I2C;
  
          if( virm > 0 )
          {
              if( virm == 4 )
              {
                  if( sig1 > 110 )
                      wait_t = ( 786432+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  else
                      wait_t = (1572864+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  #ifdef FAPI_FE_MB86667_DEBUG02
                      sprintf(PrintHelp, "\nwait time %d [ms]\n",wait_t);
                      MB86667_User_Print(PrintHelp);
                  #endif
                  if( symbolrate < 5000 )
                      MB86667_WAIT( 2*wait_t );
                  else
                      MB86667_WAIT( wait_t );
                  error = MB86667_Sync_Chk1(&sync2);
                  if(error!=FE_OK)
                      return FE_ERR_FE_I2C;
  
                  if( sync2 == 0 )
                  {
                      error = MB86667_Iq_Vt_Set(1);
                      if(error!=FE_OK)
                          return error;
                      error = MB86667_Fec_Srst() ;
                      if(error!=FE_OK)
                          return error;
                  }
                  if( sig1 > 110 )
                      wait_t = (786432+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  else
                      wait_t = (1572864+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  #ifdef FAPI_FE_MB86667_DEBUG02
                      sprintf(PrintHelp, "\nwait time %d [ms]\n",wait_t);
                      MB86667_User_Print(PrintHelp);
                  #endif
                  MB86667_WAIT( wait_t ) ;
                  MB86667_Seq_Set( 1 ) ;
              } /* if( virm == 4 ) */
              else
              {
                  if( sig1 > 110 )
                      wait_t = (786432+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  else
                      wait_t = (1572864+(int32_t)(99*symbolrate)/100)/(int32_t)symbolrate ;
                  #ifdef FAPI_FE_MB86667_DEBUG02
                      sprintf(PrintHelp, "\nwait time %d [ms]\n",wait_t);
                      MB86667_User_Print(PrintHelp);
                  #endif
                  MB86667_WAIT( wait_t ) ;
                  MB86667_Seq_Set( 1 ) ;
              } /* if( virm == 4 ) ... else */
          } /* if( virm > 0 ) */
          else
          {
              #ifdef FAPI_FE_MB86667_DEBUG01
                  sprintf(PrintHelp, "\nNo Signal\n");
                  MB86667_User_Print(PrintHelp);
              #endif
              MB86667_Seq_Set( 1 ) ;
          } /* if( virm > 0 ) ... else */
      } /* if (signal == FTRUE) */
      if (mode == 0)
      {
          error = MB86667_Sync_Chk1(&sync);
          if(error != FE_OK)
              return error;
          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\n<<SYNC=0x%02X>>\n",sync);
              MB86667_User_Print(PrintHelp);
          #endif
          error = MB86667_Freqerr_Chk( frequency, fOSC_10, symbolrate, 1, &tp_param->freqerr);
          if(error != FE_OK)
              return error;
          #ifdef FAPI_FE_MB86667_DEBUG02
              sprintf(PrintHelp, "\n<<Freq error=%d>>\n", tp_param->freqerr);
              MB86667_User_Print(PrintHelp);
          #endif
      }
   } /* if(quicklock == FTRUE) ... else */

   return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Status
Description: Monitor signal quality, strength and Lock status
-------------------------------------------------------------------------------*/
uint32_t MB86667_GetStatus(void)
{
  int32_t error;
  uint8_t temp_status;
  uint32_t mb86667_status = 0;

  error = MB86667_I2C_READ(MB86667_REG_STAT, &temp_status);
  if(error!=FE_OK)
    return (uint32_t)error;
  mb86667_status |= temp_status;

  error = MB86667_I2C_READ(MB86667_REG_AGC_MONITOR, &temp_status);
  if(error!=FE_OK)
    return (uint32_t)error;
  mb86667_status |= ((temp_status&0x00FF)<<8);

  error = MB86667_I2C_READ(MB86667_REG_CN_MONITOR, &temp_status);
  if(error!=FE_OK)
    return (uint32_t)error;
  mb86667_status |= ((temp_status&0x000000FF)<<16);

  return mb86667_status;
}

#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Demod.c,v $
Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.4  2008/02/26 03:55:17  kraft
optimizations for quick lock added by Andi

Revision 1.3  2008/02/24 21:11:52  kraft
version Fuj 4.3 implemented

Revision 1.4  2007/12/17 15:12:01  skau
Register changes for fast auto IQ detection made.

Revision 1.3  2007/08/29 11:47:13  skau
Supporting confidential files and front end detection now.

Revision 1.2  2007/07/04 10:40:45  skau
Finally made compatible to the windows monitor software.

Revision 1.1  2007/06/25 12:26:49  skau
New file structure.

Revision 1.4  2007/01/24 12:11:32  skau
Static variables removed to support more than one instance.

Revision 1.3  2007/01/24 10:08:00  skau
Compiles without warnings, all CVS headers updated and
ready for first customer release.

Revision 1.2  2006/11/30 09:50:19  skau
First working version.

Revision 1.1  2006/11/03 15:11:02  skau
Driver for MB86667.This is the first (non-working) version.


*****************************************************************************/
