/*****************************************************************************

File Name:   MB86667_Scan.c

Copyright:   (C) Fujitsu Semiconductor Europe 2007

             THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
             ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
             OMMISSIONS.

Description:
   Scan routines for Fujitsu MB86667.
   The scan routines can be used in addition to the normal MB86667 API.
   All previous API functions will continue to work without the MB86667_Scan files.

$Log: MB86667_Scan.c,v $
Revision 1.2  2008/07/22 09:03:00  sgehre
fix feNotifyFct prototype to enable compilation with Metaware

Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.2  2008/02/24 21:11:52  kraft
version Fuj 4.3 implemented

Revision 1.3  2007/12/17 15:12:17  skau
Blind scan improved.

Revision 1.2  2007/10/12 08:55:08  skau
FM_PrStr replaced by SDEBUG().

Revision 1.1  2007/10/10 13:10:46  skau
Supports blind scan now.


Revision 1.0  2007/03/23 09:37:16  skau
first release

Copyright:
   $(c) 2007 Fujitsu Semiconductor Europe GmbH

******************************************************************************/
/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdlib.h>

#include "MB86667_Api.h"
#ifdef USE_WITHIN_FAPI
//   #include "fapex/abst_types.h"
#endif

#include "MB86667_Regs.h"
#include "MB86667_Customise.h"
#include "MB86667_Demod.h"
#include "MB86667_Tuner.h"
#include "MB86667_Priv.h"
#include "MB86667_Scan.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MB86667_MAX_TUNER_TRIES  50
#define MB86667_TUNER_PAUSE       1  /* pause for the tuner after not locked in ms */

#ifdef  __cplusplus
extern "C" {
#endif

//#define SDEBUG_0(x) x // error debug messages
#define SDEBUG_0(x) // error debug messages
//#define SDEBUG_1(x) x // scan loop debug messages
#define SDEBUG_1(x) // scan loop debug messages
//#define SDEBUG(x) x
#define SDEBUG(x)
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static uint16_t TP_smrt_min, TP_smrt_max;
static uint32_t ulLoFreq;      /* current LO frequency in KHz */
//static MB86667_TRANSPONDER_T  foundScanParams; /* handed over in callback function when a transponder was found */
static FE_TUNE_PARAMS_S  foundScanParams;
uint16_t                 oldParams;
/*---------------------------------------------------------------------------*/
/* call back functions                                                       */
/*---------------------------------------------------------------------------*/
FBOOL  (*MB86667_ScanAbortFunction)();
void   (*MB86667_ScanNotifyFunction)(FE_TUNE_PARAMS_S* pFoundTuneParams);

/*---------------------------------------------------------------------------*/
/* prototypes of local  functions                                            */
/*---------------------------------------------------------------------------*/
static int32_t MB86667_Reception( int32_t fTP, int32_t smrt, int32_t mode, int32_t *retval );
static int32_t  MB86667_TP_Search1( int32_t i, int32_t v, int32_t vmax, int32_t *V, int32_t swp_freq, int32_t swp_ofs, int32_t *prev_f, int32_t smrtd );
static int32_t MB86667_TP_Search2( int32_t i, int32_t *SIGMAX, int32_t temp_freq,
                int32_t *prev_f_L, int32_t *prev_f_H,
                int32_t *prev_SR_L, int32_t *prev_SR_H,
                int32_t *sch_ptn, int32_t *tmp_smrt,
                int32_t *delta_SIG, int32_t RF_freq_step, int32_t *sch_smrt_min, int32_t *sch_smrt_max,
                int32_t swp_ofs, int32_t *retval );
static int32_t MB86667_SR_Scan( int32_t i, int32_t SIGMAX, int32_t swp_freq, int32_t sch_ptn, int32_t SR_min,
                 int32_t sch_smrt_min, int32_t sch_smrt_max, int32_t RF_freq_step, int32_t *retval );
static int32_t MB86667_SR_Chk( int32_t i, int32_t swp_freq, int32_t smrt, int32_t RF_freq_step, int32_t *retval );
static int32_t MB86667_Afcerr_Chk(int32_t clkmst, int32_t *afcerr);
static void MB86667_Smrt_Info_Get2( int32_t sr, uint8_t *DECI, uint8_t *CSEL, uint8_t *RSEL, int32_t *clkmst);
static void MB86667_S_Swp_Info_Get( int32_t fOSC_start, int32_t smrt, int32_t clkmst, int32_t v,
                int32_t RF_freq_step, int32_t swp_ofs,
                int32_t *fOSC_10, int32_t *afcex_freq,
                uint8_t *AFCEX_L, uint8_t *AFCEX_H );
static void MB86667_S_Swp_Info_Get2( int32_t smrt, int32_t RF_freq_step, int32_t swp_freq, int32_t clkmst, int32_t *afcex_freq,
                int32_t *fOSC_10, uint8_t *AFCEX_L, uint8_t *AFCEX_H );
static int32_t  MB86667_S_Swp_Freq_Calculation( int32_t i, int32_t v, int32_t *V,  int32_t vmax, int32_t vmin,
                int32_t SIGMIN, int32_t fOSC_10, int32_t afcex_freq,
                int32_t swp_ofs, uint8_t *SIG1 );
static int32_t  MB86667_S_Swp_Freq_Calculation( int32_t i, int32_t v, int32_t *V,  int32_t vmax, int32_t vmin,
                int32_t SIGMIN, int32_t fOSC_10, int32_t afcex_freq,
                int32_t swp_ofs, uint8_t *SIG1 );

/*---------------------------------------------------------------------------*/
/* global data                                                                */
/*---------------------------------------------------------------------------*/
extern FE_MB86667_RF_INI_FCT_T mb86667_Rf_Val_Set_Ini;
extern FE_MB86667_RF_SET_FCT_T mb86667_Rf_Val_Set;

/*
   Progress calculation for Scan

   in case of symbol rate range we have more runs
   max range = 3
   tp means SAT frequencies

   actrun: 0-2
   runs: 1-3
*/
static uint32_t mb86667_progress(int32_t tpMinFreq, int32_t tpMaxFreq, int32_t tpSwitchFreq,
                int32_t runs, int32_t tpActFreq, int32_t actRun)
{
   int32_t base=tpMaxFreq-tpMinFreq;
   int32_t curr;

   if((tpMinFreq <= tpSwitchFreq) && (tpMaxFreq > tpSwitchFreq)) /* 2 region problem */
   {
       /* here are 2 cases: in lower or higher band */
       if(tpActFreq <= tpSwitchFreq) /* now in lower band */
       {
           curr = actRun*(tpSwitchFreq-tpMinFreq) + tpActFreq - tpMinFreq;
       }
       else
       {
           curr = actRun*(tpMaxFreq-tpSwitchFreq) + tpActFreq - tpSwitchFreq +
                  runs*(tpSwitchFreq-tpMinFreq);
       }
   }
   else /* 1 region problem */
   {
       curr = base*actRun + tpActFreq-tpMinFreq;
   }
   return((uint32_t)((1000*curr)/(base*runs)));
}


/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*************************************************
Description:
  function to start the scan
Inputs
  pScanParams:
    poS32er to scan parameters, see MB86667_Api.h
  pSatType:
    poS32er to scan parameters, see MB86667_Api.h
Outputs
  none
Return
    RET_OK
    FE_ERR_BAD_PARAMETER
    RET_ERROR
**************************************************/
int32_t MB86667_Scan(MB86667_SCAN_PARAMS_T *pScanParams, MB86667_SATELLITE_T* pSatType)
{
    int32_t iError;
    uint8_t rest=0;
    /* variables for scan */
    uint32_t ulScanF, ulStartF, ulStopF, ulIFFreq; /* current scan frequency (RF and IF) in KHz */
    FAPI_MB86667_BAND_E bUsedBand; /* indicating HIGH or LOW band */
    uint32_t ulProgress;

    int32_t tpActFreq;
    int32_t runs=0,actrun=0;

    int32_t i, i_start, i_stop;
    int32_t j;
    int32_t fOSC_10;
    int32_t fOSC_start;
    int32_t clkmst;    /* int32_t external clock freq[kHz] */
    int32_t smrt, smrtd;

    int32_t wait_t;
    int32_t swp_ofs;
    int32_t SIGMAX;
    int32_t v, vmax=0,vfirst=0;
    int32_t V[6];

    uint8_t DECI, CSEL, RSEL;
    uint8_t CREN, AFCEN, AFCEXEN;
    uint8_t SIG1;
    uint8_t TIMINT1, TIMINT2, TIMEXT;
    uint8_t S0T, S1T;
    uint8_t AFCEX_L, AFCEX_H;
    uint8_t VIRM;
    int32_t RF_freq_step;

    int32_t afcex_freq;
    int32_t swp_freq;
    int32_t prev_freq[6];
    int32_t prev_f_L[6], prev_f_H[6];
    int32_t prev_SR_L[6], prev_SR_H[6];
    int32_t temp_freq, temp_freq2;
    int32_t tmp_smrt, tmp_smrt2;
    int32_t sch_ptn;

    int32_t TP_freq[60];
    int32_t TP_smrt[60];
    int32_t TP_num;
    int32_t TP_edge_L[60], TP_edge_H[60];
    int32_t freq_err;
    int32_t delta_SIG[3];
    int32_t sch_smrt_min, sch_smrt_max;
    int32_t tmp_fTP;
    uint8_t iq_fec, iqat, iqinv;
    int32_t fTP_min, fTP_max, smrt_min, smrt_max;
    uint32_t tunerTries;

    /* check if function pointers are set */
    if (mb86667_Rf_Val_Set_Ini == NULL)
        return(FE_ERR_FE_OTHER);
    if (mb86667_Rf_Val_Set == NULL)
        return(FE_ERR_FE_OTHER);
    if (pScanParams->feScanNotifyFct == NULL)
        return(FE_ERR_FE_OTHER);
    MB86667_ScanAbortFunction  = pScanParams->feScanAbortFct;
    MB86667_ScanNotifyFunction = pScanParams->feScanNotifyFct;

    iError = FE_OK;
    fTP_min  = pScanParams->startFrequency / 1000;
    fTP_max  = pScanParams->stopFrequency / 1000;
    smrt_min = pScanParams->startSymbolRate;
    smrt_max = pScanParams->stopSymbolRate;

    if(smrt_min>smrt_max)
        /* smrt_min - smrt_max in wrong direction */
        return(FE_ERR_BAD_PARAMETER);

    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** fTP_min=%d   fTP_max=%d \n", fTP_min, fTP_max ));
    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** smrt_min=%d   smrt_max=%d \n", smrt_min, smrt_max ));

    for( i=0; i<60; i++ )
    {
        TP_freq[i]   =       0;
        TP_smrt[i]   =       0;
        TP_edge_L[i] = 3000000;
        TP_edge_H[i] =       0;
    }
    TP_num = 0;

    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Blind_scan \n" ));

    ulStartF    = pScanParams->startFrequency;
    ulStopF     = pScanParams->stopFrequency;
    TP_smrt_min = pScanParams->startSymbolRate;
    TP_smrt_max = pScanParams->stopSymbolRate;
    /* set poS32er to callback functions
       These two functions are called outside MB86667_Scan whereas pScanParams->feScanProgressFct is
       only used inside MB86667_Scan */

    if (ulStartF > ulStopF) /* start freq. bigger than stop freq */
        return(FE_ERR_BAD_PARAMETER);
    if ((ulStartF > MB86667_MAX_FREQ + pSatType->LNBhighfreq) || (ulStartF < MB86667_MIN_FREQ + pSatType->LNBlowfreq))
        /* start freq. too high */
        return(FE_ERR_BAD_PARAMETER);
    if ((ulStopF < MB86667_MIN_FREQ  + pSatType->LNBlowfreq) || (ulStopF > MB86667_MAX_FREQ + pSatType->LNBhighfreq))
        /* stop freq. too low */
        return(FE_ERR_BAD_PARAMETER);

    /* select tuning band */
    ulScanF = ulStartF;
    if (labs(ulScanF - pSatType->LNBlowfreq) <= MB86667_BAND_SW)
    { /* using low band */
        bUsedBand = LOW;
        ulLoFreq = pSatType->LNBlowfreq;
    }
    else
    {   /* using high band */
        bUsedBand = HIGH;
        ulLoFreq = pSatType->LNBhighfreq;
    }

    /* check IF frequency range */
    ulIFFreq = labs(ulScanF - ulLoFreq);
    if ((ulIFFreq < MB86667_MIN_FREQ) || (ulIFFreq > MB86667_MAX_FREQ))
        return(FE_ERR_BAD_PARAMETER);

    if ( (TP_smrt_min < MB86667_MIN_SYM) || (TP_smrt_max > MB86667_MAX_SYM) )
        return(FE_ERR_BAD_PARAMETER);
    if ( TP_smrt_min > TP_smrt_max )
        return(FE_ERR_BAD_PARAMETER);

    /* control LNB */
    if (pScanParams->pol > CIRC_RIGHT)
        return(FE_ERR_BAD_PARAMETER);

    /* start of scan algorithms */
    /* set LNB control */
    if ((iError=mb86667_lnb_control(ulScanF, pScanParams->pol, pSatType->satID, bUsedBand)) != FE_OK)
        return(iError);
    /* copy parameters that are fixed during the scan */
    oldParams   &= 0xFFFC; /* mask polarisation */
    oldParams   |= (CIRC_RIGHT & pScanParams->pol);    /* add new polarisation */

    if(      smrt_max>=12702 )
        i_start = 0;
    else if( smrt_max>= 3564 )
        i_start = 1;
    else if( smrt_max>= 1000 )
        i_start = 2;
    else
    {
        SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Invalid smrt_max\n" ));
        return FE_ERR_BAD_PARAMETER;
    }
    if(      smrt_min>=12702 )
        i_stop = 0;
    else if( smrt_min>= 3564 )
        i_stop = 1;
    else if( smrt_min>= 1000 )
        i_stop = 2;
    else
    {
        SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Invalid smrt_min\n" ));
        return FE_ERR_BAD_PARAMETER;
    }

    for( j=0; j<6; j++ )
    {
        V[j] = -1;
    }

    /* calculate if freq fOSC_start from sat fequency in units[MHz]*/
    if(bUsedBand==LOW)
        fOSC_start = (labs(pScanParams->startFrequency - pSatType->LNBlowfreq))/1000;
    else
        fOSC_start = (labs(pScanParams->startFrequency - pSatType->LNBhighfreq))/1000;

    /* DAGCA=3,DAGCW=2 */
    if((iError=MB86667_Dagc_Data_Set( 3, 2 ))!=FE_OK)
        return iError;

    RF_freq_step = 500;
    runs=i_stop-i_start+1;

    do
    {
        actrun=0;
        for( i = i_start; i <= i_stop; i++ )
        {
            if( i==0 )
            {
                smrt=27945;
                smrtd=12702;
                SDEBUG_1(FAPI_SYS_PRINT_MSG("\n--- Checking symbol rates >= 12702\n" ));
            }
            else if( i==1 )
            {
                smrt= 7841;
                smrtd= 3564;
                SDEBUG_1(FAPI_SYS_PRINT_MSG("\n--- Checking symbol rates from 3564 to 12702\n" ));
            }
            else
            {
                smrt= 2200;
                smrtd= 1000;
                SDEBUG_1(FAPI_SYS_PRINT_MSG("\n--- Checking symbol rates from 1000 to 3564\n" ));
            }

            swp_ofs = smrtd/3;    /* value=4234(i=0),1188(i=1),333(i=2) */
            if(bUsedBand == LOW)
            {
                if (labs((fTP_max*1000) - pSatType->LNBlowfreq) > MB86667_BAND_SW)
                {
                    rest=1;
                    vmax = (((pSatType->LNBlowfreq+MB86667_BAND_SW)/1000)-fTP_min)*1000/swp_ofs;
                    SDEBUG_1(FAPI_SYS_PRINT_MSG("--- Checking both bands -> low band first\n" ));
                }
                else
                {   /* only low bands need check */
                    rest=0;
                    vmax = (fTP_max-fTP_min)*1000/swp_ofs;
                    SDEBUG_1(FAPI_SYS_PRINT_MSG("--- Checking low bands\n" ));
                }
            }
            else
            {
                rest=0;
                vmax = (fTP_max-fOSC_start-(pSatType->LNBhighfreq/1000))*1000/swp_ofs;
                SDEBUG_1(FAPI_SYS_PRINT_MSG("--- Checking high band\n" ));
            }

            CREN = 0; AFCEN = 0; AFCEXEN = 1;
            TIMINT1 = 0; TIMINT2 = 1; TIMEXT = 2;
            S1T = 0; S0T = 0;
            if((iError=MB86667_Initial_Set())!=FE_OK)
                return iError;
            if((iError=MB86667_AGC_SET( 4 ))!=FE_OK)
                return iError;
            if((iError=MB86667_Eq_Set( 0, 3 ))!=FE_OK)
                return iError;
            if((iError=MB86667_En_Set( CREN, AFCEN ))!=FE_OK)
                return iError;
            if((iError=MB86667_Afcexen_Set( AFCEXEN, smrt ))!=FE_OK)
                return iError;
            if((iError=MB86667_Cntm_Set( TIMINT1, TIMINT2, TIMEXT ))!=FE_OK)
                return iError;
            if((iError=MB86667_S01t_Set( S1T, S0T ))!=FE_OK)
                return iError;
            MB86667_Smrt_Info_Get( smrt, &DECI, &CSEL, &RSEL, &clkmst );
            if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!=FE_OK)
                return iError;
            if((iError=MB86667_Seq_Set( 0 ))!=FE_OK)
                return iError;    /* added for v2200 */
            if((iError=MB86667_Stlp_Set( 5, 7 ))!=FE_OK)
                return iError;
            if((iError=MB86667_Stlp_Set2( 5, 7 ))!=FE_OK)
                return iError;
            wait_t = (9000+smrtd/2)/smrtd;
            if( wait_t==0 ) wait_t = 1;

            SDEBUG(FAPI_SYS_PRINT_MSG("F01 *** smrt=%05d[kbaud] smrtd=%05d[kbaud] ", smrt, smrtd ));
            SDEBUG(FAPI_SYS_PRINT_MSG("F01 *** wait_t[ms]=%d clkmst[kHz]=%d\n", wait_t, clkmst ));

            for( j=0; j<6; j++ )
            {
                prev_freq[j] = 0;
                prev_f_L[j] = 0;
                prev_f_H[j] = 0;
                prev_SR_L[j] = 0;
                prev_SR_H[j] = 0;
            }
            for( v=0; v<=vmax; v++ )  /* this is the real running loop */
            {
                MB86667_S_Swp_Info_Get( fOSC_start, smrt, clkmst, v, RF_freq_step,
                                        swp_ofs, &fOSC_10, &afcex_freq,
                                        &AFCEX_L, &AFCEX_H );
                SDEBUG(FAPI_SYS_PRINT_MSG("S01 *** fOSC_start=%d smrt=%d ", fOSC_start, smrt ));
                SDEBUG(FAPI_SYS_PRINT_MSG("clkmst=%d v=%d\n", clkmst, v ));
                SDEBUG(FAPI_SYS_PRINT_MSG("S01 *** RF_freq_step=%d swp_ofs=%d\n", RF_freq_step, swp_ofs ));
                SDEBUG(FAPI_SYS_PRINT_MSG("S01 *** fOSC_10=%d afcex_freq=%d\n", fOSC_10, afcex_freq ));
                SDEBUG(FAPI_SYS_PRINT_MSG("S01 *** AFCEX_L=%d AFCEX_H=%d\n", AFCEX_L, AFCEX_H ));

                if( v==0 )
                {
                    tunerTries = MB86667_MAX_TUNER_TRIES;
                    do
                    {
                        iError = mb86667_Rf_Val_Set_Ini( fOSC_10, smrt );
                        tunerTries--;
                        if (iError != FE_OK)
                            MB86667_User_Sleep( MB86667_TUNER_PAUSE );
                    } while ((iError != FE_OK) && (tunerTries > 0));
                    if(tunerTries < MB86667_MAX_TUNER_TRIES-1)
                       SDEBUG_0(FAPI_SYS_PRINT_MSG("\nScan: mb86667_Rf_Val_Set_Ini() needed %d tries\n", MB86667_MAX_TUNER_TRIES - tunerTries) );
                    if(iError!=FE_OK)
                    {
                       SDEBUG_0(FAPI_SYS_PRINT_MSG("\nScan: Failed in mb86667_Rf_Val_Set_Ini()\n"));
                       return(FE_ERR_FE_OTHER);
                    }
                }
                else
                {
                    tunerTries = MB86667_MAX_TUNER_TRIES;
                    do
                    {
                        iError = mb86667_Rf_Val_Set( fOSC_10, smrt );
                        tunerTries--;
                        if (iError != FE_OK)
                            MB86667_User_Sleep( MB86667_TUNER_PAUSE );
                    } while ((iError != FE_OK) && (tunerTries > 0));
                    if(tunerTries < MB86667_MAX_TUNER_TRIES-1)
                       SDEBUG_0(FAPI_SYS_PRINT_MSG("\nScan: mb86667_Rf_Val_Set() needed %d tries\n", MB86667_MAX_TUNER_TRIES - tunerTries) );
                    if(iError!=FE_OK)
                    {
                       SDEBUG_0(FAPI_SYS_PRINT_MSG("\nScan: Failed in mb86667_Rf_Val_Set()\n"));
                       return(FE_ERR_FE_OTHER);
                    }
                }
                if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H ))!=FE_OK)
                    return iError;
                swp_freq = fOSC_10*100 + afcex_freq;
                if((iError=MB86667_Srst())!=FE_OK)
                    return iError;
                MB86667_WAIT( wait_t );
                if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1, &SIG1))!=FE_OK)
                    return iError;
                for( j=5; j>0; j-- )
                    V[j] = V[j-1];
                V[0] = SIG1;
                temp_freq = MB86667_TP_Search1( i, v, vmax, V, swp_freq, swp_ofs, prev_freq, smrtd );

                for( j=0; j<TP_num; j++ )
                {
                    if( (TP_edge_L[j]<=temp_freq)&&(temp_freq<=TP_edge_H[j]) )
                    {
                        SDEBUG(FAPI_SYS_PRINT_MSG("F14 *** i=%d,temp_freq=%07d,tmp_smrt=%05d\n", i, temp_freq, tmp_smrt ));
                        temp_freq = 0;
                    }
                }

                if( temp_freq>0 )
                {
                    SDEBUG(FAPI_SYS_PRINT_MSG("F02 *** i=%d,temp_freq=%d\n", i, temp_freq ));
                    if((iError=MB86667_TP_Search2( i, &SIGMAX, temp_freq,
                                                prev_f_L, prev_f_H,
                                                prev_SR_L, prev_SR_H,
                                                &sch_ptn, &tmp_smrt,
                                                delta_SIG, RF_freq_step, &sch_smrt_min, &sch_smrt_max,
                                                swp_ofs, &temp_freq2 ))!=FE_OK)
                        return iError;
                    if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!=FE_OK)
                        return iError;


                    for( j=0; j<TP_num; j++ )
                    {
                        if( (TP_edge_L[j]<=temp_freq2)&&(temp_freq2<=TP_edge_H[j]) )
                        {
                            SDEBUG(FAPI_SYS_PRINT_MSG("F14 *** i=%d,temp_freq2=%07d,tmp_smrt=%05d\n", i, temp_freq, tmp_smrt ));
                            temp_freq2 = 0;
                        }
                    }

                    if( (temp_freq2>0)&&(tmp_smrt==0) )
                    {
                        if((iError=MB86667_SR_Scan( i, SIGMAX, temp_freq2, sch_ptn, smrt_min,
                                            sch_smrt_min, sch_smrt_max, RF_freq_step, &tmp_smrt ))!=FE_OK)
                            return iError;
                        if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!=FE_OK)
                            return iError;
                    }
                    if( (temp_freq2>0)&&(tmp_smrt>0) )
                    {
                        tmp_smrt2 = tmp_smrt;
                        if( tmp_smrt2>0 )
                        {
                            uint8_t sync2;
                            SDEBUG(FAPI_SYS_PRINT_MSG("F07 *** i=%d,swp_freq=%07d,smrt=%05d\n", i, temp_freq2, tmp_smrt2 ));
                            tmp_fTP = (temp_freq2+500)/1000;
                            if((iError=MB86667_Reception( tmp_fTP, tmp_smrt2, 1, &fOSC_10 ))!=FE_OK)
                                return iError;
                            if((iError=MB86667_Sync_Chk2(&VIRM, &sync2))!=FE_OK)
                                return iError;
                            if( sync2==1 )
                            {
                                SDEBUG(FAPI_SYS_PRINT_MSG("F12 *** FREQ=%04d,SR=%05d <<LOCK>>\n", (temp_freq2+500)/1000, tmp_smrt2 ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("F12 *** fOSC_10=%d\n", fOSC_10 ));
                                if((iError=MB86667_Freqerr_Chk( tmp_fTP, fOSC_10, tmp_smrt2, 1, &freq_err ))!=FE_OK)
                                    return iError;
                                TP_freq[TP_num] = (tmp_fTP*1000 + freq_err + 500)/1000;
                                TP_smrt[TP_num] = tmp_smrt2;
                                TP_edge_L[TP_num] = tmp_fTP*1000 + freq_err - TP_smrt[TP_num]/2*65/100;
                                TP_edge_H[TP_num] = tmp_fTP*1000 + freq_err + TP_smrt[TP_num]/2*65/100;
                                SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** FREQ=%04d,SR=%05d,", TP_freq[TP_num], TP_smrt[TP_num] ));

                                if(      VIRM==1 )
                                    SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** PR=1/2\n" ));
                                else if( VIRM==2 )
                                    SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** PR=2/3\n" ));
                                else if( VIRM==3 )
                                    SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** PR=3/4\n" ));
                                else if( VIRM==4 )
                                    SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** PR=5/6\n" ));
                                else if( VIRM==5 )
                                    SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** PR=7/8\n" ));

                                if((iError=MB86667_I2C_READ(MB86667_REG_IQ_AUTODET, &iq_fec)) != FE_OK )
                                    return iError;
                                iqat  = (0x40&iq_fec)>>6;
                                iqinv = (0x20&iq_fec)>>5;

                                foundScanParams.sat.symbolRate = tmp_smrt2;
                                /* osc feq in MHz freqerr in kHz */
                                if(bUsedBand==LOW)
                                    foundScanParams.sat.frequency = tmp_fTP*1000 + freq_err + pSatType->LNBlowfreq;
                                else
                                    foundScanParams.sat.frequency = tmp_fTP*1000 + freq_err + pSatType->LNBhighfreq;
                                foundScanParams.sat.fec = VIRM;

                                oldParams |= (iqinv<<2);
                                oldParams |= (iqat<<3);

                                /* add call back function here for FAPI */
                                MB86667_ScanNotifyFunction(&foundScanParams);

                                TP_num++;
                            }
                            else
                            {
                                SDEBUG(FAPI_SYS_PRINT_MSG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n" ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Can not lock to the signal\n" ));
                                if(bUsedBand==LOW)
                                   SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Sat-Freq: %d\n",     tmp_fTP*1000 + freq_err + pSatType->LNBlowfreq ));
                                else
                                   SDEBUG(FAPI_SYS_PRINT_MSG("--- *** Sat-Freq: %d\n",     tmp_fTP*1000 + freq_err + pSatType->LNBhighfreq ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** i=%d FREQ=%04d,SR=%05d,", i, temp_freq2/1000, tmp_smrt2 ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("F09 *** Unlock\n" ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("F12 *** FREQ=%04d,SR=%05d <<UNLOCK>>\n", temp_freq2/1000, tmp_smrt2 ));
                                SDEBUG(FAPI_SYS_PRINT_MSG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n" ));
                            }
                            if((iError=MB86667_Dagc_Data_Set( 3, 2 )) != FE_OK )
                                return iError;
                            if((iError=MB86667_AGC_SET( 4 )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Eq_Set( 0, 3 )) != FE_OK )
                                return iError;
                            if((iError=MB86667_En_Set( CREN, AFCEN )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Afcexen_Set( AFCEXEN, smrt )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Cntm_Set( TIMINT1, TIMINT2, TIMEXT )) != FE_OK )
                                return iError;
                            if((iError=MB86667_S01t_Set( S1T, S0T )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Seq_Set( 0 )) != FE_OK )
                                return iError;    /* added for v2200 */
                            if((iError=MB86667_Stlp_Set( 5, 7 )) != FE_OK )
                                return iError;
                            if((iError=MB86667_Stlp_Set2( 5, 7 )) != FE_OK )
                                return iError;
                        }
                    }
                }
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** i=%d,swpfreq=%07d TP_num=%d\n", i, swp_freq, TP_num ));
                SDEBUG(FAPI_SYS_PRINT_MSG("F01 *** %d    %03d\n", swp_freq, SIG1 ));

                /* some statistics based on tuner if */
                if(bUsedBand==LOW)
                    tpActFreq = fOSC_10*100 + pSatType->LNBlowfreq;
                else
                    tpActFreq = fOSC_10*100 + pSatType->LNBhighfreq;
                ulProgress = mb86667_progress(pScanParams->startFrequency/1000,
                                      pScanParams->stopFrequency/1000,
                                      (pSatType->LNBlowfreq + MB86667_BAND_SW)/1000,
                                      runs,tpActFreq/1000,actrun);
                if (ulProgress > 1000)
                {
                    ulProgress = 1000;
                    SDEBUG(FAPI_SYS_PRINT_MSG("\n limit progress"));
                }
                if(pScanParams->feScanProgressFct != 0)
                   pScanParams->feScanProgressFct((uint16_t) ulProgress);
                /* if we want to abort */
                if(MB86667_ScanAbortFunction!=0)
                {
                    if (MB86667_ScanAbortFunction() != FFALSE)
                    {
                        rest=0;
                        break;
                    }
                }
            }
            actrun++;
        }
        /* we have to check wether there is a bit way left*/
        if(rest!=0)
        {
            vfirst = vmax * runs;
            /* we calculate the first possible start freq in higher band */
            fOSC_start = ((pSatType->LNBlowfreq+MB86667_BAND_SW)/1000)-(pSatType->LNBhighfreq/1000);
            bUsedBand = HIGH;
            iError = mb86667_lnb_control((pSatType->LNBlowfreq+MB86667_BAND_SW), pScanParams->pol, pSatType->satID, bUsedBand);
            if (iError != FE_OK)
                return iError;
        }
    }
    while(rest!=0);

    for( j=0; j<TP_num; j++ )
    {
        SDEBUG(FAPI_SYS_PRINT_MSG("F20 *** TP_edge_L[%02d]=%07d TP_edge_H[%02d]=%07d\n",
                        j, TP_edge_L[j], j, TP_edge_H[j] ));
    }

    return(iError); /* return preset error */
} /* MB86667_Scan */


/*----------------------------------------------------------------------------
Name:        MB86667_Reception
Description: lock carrier
-------------------------------------------------------------------------------*/
int32_t MB86667_Reception( int32_t fTP, int32_t smrt, int32_t mode, int32_t *retval )
{
    int32_t iError;
    uint32_t tunerTries;
    int32_t i, j;
    int32_t fOSC_10;         /* [100kHz] */
    int32_t fOSC_start=fTP;  /* [MHz] */
    int32_t clkmst;

    int32_t wait_t;
    int32_t fcp;        /* [kHz] */
    int32_t swp_ofs;    /* [kHz] */
    int32_t V[60];

    uint8_t DECI, CSEL, RSEL;
    uint8_t CREN, AFCEN, AFCEXEN;
    uint8_t SIG1;
    uint8_t TIMINT1, TIMINT2, TIMEXT;
    uint8_t S0T, S1T;
    uint8_t S2T, S4T, S5T;
    uint8_t AFCEX_L, AFCEX_H;
    uint8_t VIRM;
    uint8_t ETH, VIA;
    int32_t RF_freq_step;

    int32_t loop;
    int32_t ftemp;
    int32_t v, vmax, vmin;
    int32_t vmax_his, vmin_his;
    int32_t swp_freq, prev_swp_freq[20];
    int32_t prev_freq_num;
    int32_t signal_dupl;
    int32_t afcex_freq;
    FBOOL signal;
    uint8_t sync;

    SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** fTP=%04d,smrt=%05d\n", fTP, smrt ));

    fcp = 3000;    /* capture range of carrier recovery[kHz] */
    RF_freq_step = 500;    /* freq_step=500kHz */

    swp_ofs = smrt/4;
    for( i=0; i<60; i++ ) V[i] = -1;
    for( i=0; i<20; i++ ) prev_swp_freq[i] = 0;

    if((iError=MB86667_Seq_Set( 0 ))!=FE_OK) return iError;
    if((iError=MB86667_Iq_Vt_Set( 0 ))!=FE_OK) return iError;
    CREN = 0;
    AFCEN = 0;
    AFCEXEN = 1;
    TIMINT1 = 0;
    TIMINT2 = 1;
    TIMEXT = 2;
    S1T = 0;
    S0T = 0;
    if((iError=MB86667_AGC_SET( 4 ))!=FE_OK)
        return iError;
    if((iError=MB86667_Initial_Set())!=FE_OK)
        return iError;
    if((iError=MB86667_Stlp_Set2( 2, 3 ))!=FE_OK)
        return iError;
    if((iError=MB86667_Eq_Set( 1, 3 ))!=FE_OK)
        return iError;
    if((iError=MB86667_Dagc_Data_Set( 3, 2 ))!=FE_OK)
        return iError;
    if((iError=MB86667_En_Set( CREN, AFCEN ))!=FE_OK)
        return iError;
    if((iError=MB86667_Afcexen_Set( AFCEXEN, smrt ))!=FE_OK)
        return iError;
    if((iError=MB86667_Cntm_Set( TIMINT1, TIMINT2, TIMEXT ))!=FE_OK)
        return iError;
    if((iError=MB86667_S01t_Set( S1T, S0T ))!=FE_OK)
        return iError;
    MB86667_Smrt_Info_Get( smrt, &DECI, &CSEL, &RSEL, &clkmst );
    if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!=FE_OK)
        return iError;

    if( (RF_freq_step==1000)||(RF_freq_step==500) )
        fOSC_start = fTP;
    else if( RF_freq_step==2000 )
    {
        if( fTP%2==0 )
            fOSC_start = fTP;
        else
        {
            fOSC_start = fTP+1;
            if( fOSC_start>2150 )
                fOSC_start = fTP-1;
        }
    }

    loop = 1;
    ftemp = fOSC_start*1000;
    vmax = 0;
    while( loop==1 )
    {
        ftemp = ftemp + swp_ofs;
        vmax++;
        if( ftemp>2150000 )
        {
            loop = 0;
            vmax--;
        }
        else if( smrt/9>fcp )
        {
            loop = 0; vmax = 0;
        }
        else if( (ftemp==2150000)||(ftemp-fTP*1000>=fcp-smrt/9) ) loop = 0;
    }
    loop = 1;
    ftemp = fOSC_start*1000;
    vmin = 0;
    while( loop==1 )
    {
        ftemp = ftemp - swp_ofs;
        vmin--;
        if( ftemp<950000 )
        {
            loop = 0;
            vmin++;
        }
        else if( smrt/9>fcp )
        {
            loop = 0;
            vmin = 0;
        }
        else if( (ftemp==950000)||(fTP*1000-ftemp>=fcp-smrt/9) )
            loop = 0;
    }
    if( mode==1 )
    {
        vmin = -2;    /* for blind scan mode */
        vmax =  2;    /* for blind scan mode */
    }

    SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** vmax=%03d,vmin=%03d\n", vmax, vmin ));

    wait_t = (8000+smrt/2)/smrt;

    if( wait_t==0 ) wait_t = 1;

    i = 0;
    j = 0;
    prev_freq_num = 0;
    loop = 1;
    signal = FFALSE;
    vmax_his = 0;
    vmin_his = 0;
    v = 0;
    while( loop==1 )
    {
        MB86667_S_Swp_Info_Get( fOSC_start, smrt, clkmst, v, RF_freq_step, swp_ofs, &fOSC_10, &afcex_freq, &AFCEX_L, &AFCEX_H );
        if( i==0 )
        {
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set_Ini( fOSC_10, smrt );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("Reception: Failed in mb86667_Rf_Val_Set_Ini()\n"));
               return(FE_ERR_FE_OTHER);
            }
        }
        else
        {
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set( fOSC_10, smrt );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("Reception: Failed in mb86667_Rf_Val_Set()\n"));
               return(FE_ERR_FE_OTHER);
            }
        }
        if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H ))!=FE_OK)
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** i=%02d,v=%+03d,fOSC_10=%05d,afcex_freq=%+04d,", i, v, fOSC_10, afcex_freq ));
        if((iError=MB86667_Srst())!=FE_OK)
            return iError;
        MB86667_WAIT( wait_t );
        if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1, &SIG1))!= FE_OK )
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** SIG1=%d\n", SIG1 ));
        V[30+v] = SIG1;
        swp_freq = MB86667_S_Swp_Freq_Calculation( i, v, V, vmax, vmin, SIG1MIN, fOSC_10, afcex_freq, swp_ofs, &SIG1 );
        signal_dupl = 0;
        for( j=0; j<prev_freq_num; j++ )
        {
            if( abs(prev_swp_freq[j]-swp_freq)<swp_ofs*3/2 )
                signal_dupl = 1;
        }
        if( (signal_dupl==0)&&(swp_freq>0) )
        {
            SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** signal found swp_freq=%07d,SR=%05d\n", swp_freq, smrt ));
            prev_swp_freq[prev_freq_num] = swp_freq;
            prev_freq_num++;
            MB86667_S_Swp_Info_Get2( smrt, RF_freq_step, swp_freq, clkmst, &afcex_freq, &fOSC_10, &AFCEX_L, &AFCEX_H );
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set( fOSC_10, smrt );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("Reception: Failed in mb86667_Rf_Val_Set()\n"));
               return(FE_ERR_FE_OTHER);
            }
            if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H ))!= FE_OK )
                return iError;
            if((iError=MB86667_Signal_Det((uint16_t) smrt, &DECI, &CSEL, &RSEL, &clkmst, &SIG1, &signal ))!= FE_OK )
                return iError;
            if( signal==FTRUE )
                loop = 0;
            else
            {
                MB86667_Smrt_Info_Get( smrt, &DECI, &CSEL, &RSEL, &clkmst );
                if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!= FE_OK )  return iError;
            }
        }
        if( v>=vmax )
            vmax_his = 1;
        if( v<=vmin )
            vmin_his = 1;
        i++;
        if( (i%2==1)&&(vmax_his==1) )
            i++;
        if( (i%2==0)&&(vmin_his==1) )
            i++;

        if( i%2==1 )
            v = (i+1)/2;
        else
            v = -i/2;

        if( (vmax_his==1)&&(vmin_his==1) )
            loop = 0;
    }

    if( signal==FTRUE )
    {
        S1T = 4;
        S0T = 0;
        CREN = 1;
        AFCEN = 1;
        AFCEXEN = 0;
        if((iError=MB86667_AGC_SET( 2 ))!= FE_OK )
            return iError;
        if((iError=MB86667_S01t_Set( S1T, S0T ))!= FE_OK )
            return iError;
        if((iError=MB86667_Dagc_Data_Set( 0, 0 ))!= FE_OK )
            return iError;
        if((iError=MB86667_Eq_Set( 1, 0 ))!= FE_OK )
            return iError;
        MB86667_Smrt_Info_Get( smrt, &DECI, &CSEL, &RSEL, &clkmst );
        if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst ))!= FE_OK )
            return iError;
        if((iError=MB86667_En_Set( CREN, AFCEN ))!= FE_OK )
            return iError;
        if((iError=MB86667_Afcexen_Set( AFCEXEN, smrt ))!= FE_OK )
            return iError;
        MB86667_Afcex_Info_Get( clkmst, afcex_freq, &AFCEX_L, &AFCEX_H );
        if((iError=MB86667_Afcofs_Data_Set( AFCEX_L, AFCEX_H ))!= FE_OK )
            return iError;
        if((iError=MB86667_Srst())!= FE_OK )
            return iError;
        wait_t = 7 + (10000+smrt/2)/smrt;
        if( wait_t==0 )
            wait_t = 1;
        MB86667_WAIT( wait_t );
        if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1, &SIG1))!= FE_OK )
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** SIG1_center=%d\n", SIG1 ));

        if( SIG1>105 )
        {
            S2T=4; S4T=8; S5T=8; ETH=7; VIA=2; wait_t = 7 + (1245184+99*smrt/100)/smrt;
        }
        else if( SIG1> 85 )
        {
            S2T=5; S4T=8; S5T=8; ETH=7; VIA=2; wait_t = 7 + (1507328+99*smrt/100)/smrt;
        }
        else if( SIG1> 65 )
        {
            S2T=6; S4T=8; S5T=8; ETH=7; VIA=2; wait_t = 7 + (1769472+99*smrt/100)/smrt;
        }
        else
        {
            S2T=7; S4T=8; S5T=8; ETH=7; VIA=2; wait_t = 7 + (2424832+99*smrt/100)/smrt;
        }

        if((iError=MB86667_S2t_Set( S2T ))!= FE_OK )
            return iError;
        if((iError=MB86667_S45t_Set( S4T, S5T ))!= FE_OK )
            return iError;
        if((iError=MB86667_Vi_Set( ETH, VIA ))!= FE_OK )
            return iError;
        if((iError=MB86667_Srst())!= FE_OK )
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** wait %d[s]\n", wait_t ));
        MB86667_WAIT( wait_t );
        if((iError=MB86667_Sync_Chk2( &VIRM, &sync ))!= FE_OK )
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** VIRM=%d\n", VIRM ));
        if( VIRM>0 )
        {
            if( VIRM==4 )
            {
                uint8_t sync1;
                if( SIG1>110 )
                    wait_t = ( 786432+99*smrt/100)/smrt;
                else
                    wait_t = (1572864+99*smrt/100)/smrt;
                SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** wait_t=%d\n", wait_t ));

                MB86667_WAIT( wait_t );
                if((iError=MB86667_Sync_Chk1(&sync1))!= FE_OK )
                    return iError;
                if( sync1==0 )
                {
                    if((iError=MB86667_Iq_Vt_Set( 1 ))!= FE_OK )
                        return iError;
                    if((iError=MB86667_Fec_Srst())!= FE_OK )
                        return iError;
                }
                if( SIG1>110 )
                    wait_t = ( 786432+99*smrt/100)/smrt;
                else
                    wait_t = (1572864+99*smrt/100)/smrt;
                SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** wait_t=%d\n", wait_t ));
                MB86667_WAIT( wait_t );
                if((iError=MB86667_Seq_Set( 1 ))!= FE_OK )
                    return iError;
            }
            else
            {
                if( SIG1>110 )
                    wait_t = ( 786432+99*smrt/100)/smrt;
                else
                    wait_t = (1572864+99*smrt/100)/smrt;
                SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** wait_t=%d\n", wait_t ));
                MB86667_WAIT( wait_t );
                if((iError=MB86667_Seq_Set( 1 ))!= FE_OK )
                    return iError;
            }
        }
        else
        {
            SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** No signal\n" ));
            if((iError=MB86667_Seq_Set( 1 ))!= FE_OK )
                return iError;
        }
    }

    if( mode==0 )
    {
        int32_t freqerr;
        if((iError=MB86667_Sync_Chk1((uint8_t*)&sync))!= FE_OK )
            return iError;
        SDEBUG(FAPI_SYS_PRINT_MSG("F15 *** <<SYNC=%d>>\n", sync ));
        if((iError=MB86667_Freqerr_Chk( fTP, fOSC_10, smrt, 1, &freqerr ))!= FE_OK )
            return iError;
    }

    *retval = fOSC_10;
    return iError;
}

/*----------------------------------------------------------------------------
Name:        MB86667_S_Swp_Freq_Calculation
Description: calculates sweep frequency
-------------------------------------------------------------------------------*/
int32_t MB86667_S_Swp_Freq_Calculation( int32_t i, int32_t v, int32_t *V,  int32_t vmax, int32_t vmin,
                                  int32_t SIGMIN, int32_t fOSC_10, int32_t afcex_freq,
                                  int32_t swp_ofs, uint8_t *SIG1 )
{
    int32_t swp_freq;

    if( (i%2==1)&&(v<=vmax) )
    {    /* positive v */
        if( (v-1==vmin)&&(*(V+30+v)>=0)&&(*(V+30+v-1)>=0)    //positive case 1
            &&(*(V+30+v-1)>*(V+30+v))&&(*(V+30+v-1)>SIGMIN) )
        {
            swp_freq = fOSC_10*100 + afcex_freq -swp_ofs;
            *SIG1 = *(V+30+v-1);
            SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 1>>\n" ));
        }
        else if( (v==vmax)&&(*(V+30+v)>=0)&&(*(V+30+v-1)>=0)    //positive case 2
            &&(*(V+30+v)>*(V+30+v-1))&&(*(V+30+v)>SIGMIN) )
        {
            swp_freq = fOSC_10*100 + afcex_freq;
            *SIG1 = *(V+30+v);
            SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 2>>\n" ));
        }
        else if( (*(V+30+v)>0)&&(*(V+30+v-1)>0)&&(*(V+30+v-2)>0)&&(*(V+30+v-3)>0)    //positive case 3
            &&(*(V+30+v-1)>*(V+30+v))&&(*(V+30+v-2)>*(V+30+v-3))
            &&((*(V+30+v-1)>SIGMIN)||(*(V+30+v-2)>SIGMIN)) )
        {
            if( *(V+30+v-1)>=*(V+30+v-2) )
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs;
                *SIG1 = *(V+30+v-1);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 3_1>>\n" ));
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs*2;
                *SIG1 = *(V+30+v-2);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 3_2>>\n" ));
            }
        }
        else if( (v==vmax)&&(*(V+30+v)>=0)&&(*(V+30+v-1)>=0)&&(*(V+30+v-2)>=0)    //positive case 4
            &&(*(V+30+v)>*(V+30+v-2))&&(*(V+30+v-1)>*(V+30+v-2))
            &&((*(V+30+v)>SIGMIN)||(*(V+30+v-1)>SIGMIN)) )
        {
            if( *(V+30+v)>=*(V+30+v-1) )
            {
                swp_freq = fOSC_10*100 + afcex_freq;
                *SIG1 = *(V+30+v);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 4_1>>\n" ));
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq - swp_ofs;
                *SIG1 = *(V+30+v-1);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<pos 4_2>>\n" ));
            }
        }
        else swp_freq = -1;
    }
    else if( (i%2==0)&&(v>=vmin) )
    {    //negative v
        if( (*(V+30+v)>0)&&(*(V+30+v+1)>0)&&(*(V+30+v+2)>0)    //negative case 1
            &&(*(V+30+v+1)>*(V+30+v))&&(*(V+30+v+1)>*(V+30+v+2))
            &&(*(V+30+v+1)>SIGMIN) )
        {
            swp_freq = fOSC_10*100 + afcex_freq + swp_ofs;
            *SIG1 = *(V+30+v+1);
            SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 1>>\n" ));
        }
        else if( (v+1==vmax)&&(*(V+30+v)>=0)&&(*(V+30+v+1)>=0)    //negative case 2
            &&(*(V+30+v+1)>*(V+30+v))
            &&(*(V+30+v+1)>SIGMIN) )
        {
            swp_freq = fOSC_10*100 + afcex_freq + swp_ofs;
            *SIG1 = *(V+30+v);
            SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 2>>\n" ));
        }
        else if( (v==vmin)&&(*(V+30+v)>0)&&(*(V+30+v+1)>0)&&(*(V+30+v+2)>0)    //negative case 3
            &&(*(V+30+v)>*(V+30+v+1))&&(*(V+30+v)>*(V+30+v+2))
            &&(*(V+30+v)>SIGMIN) )
        {
            swp_freq = fOSC_10*100 + afcex_freq;
            *SIG1 = *(V+30+v);
            SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 3>>\n" ));
        }
        else if( (*(V+30+v)>=0)&&(*(V+30+v+1)>=0)&&(*(V+30+v+2)>=0)&&(*(V+30+v+3)>=0)    //negative case 4
            &&(*(V+30+v+1)>*(V+30+v))&&(*(V+30+v+2)>*(V+30+v+3))
            &&((*(V+30+v+1)>SIGMIN)||(*(V+30+v+2)>SIGMIN)) )
        {
            if( *(V+30+v+1)>=*(V+30+v+2) )
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs;
                *SIG1 = *(V+30+v+1);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 4_1>>\n" ));
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs*2;
                *SIG1 = *(V+30+v+2);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 4_2>>\n" ));
            }
        }
        else if( (*(V+30+v)>=0)&&(*(V+30+v+1)>=0)&&(*(V+30+v+2)>=0)&&(*(V+30+v+3)>=0)    //negative case 5
            &&(*(V+30+v)>*(V+30+v+2))&&(*(V+30+v+1)>*(V+30+v+2))
            &&(*(V+30+v)>*(V+30+v+3))&&(*(V+30+v+1)>*(V+30+v+3))
            &&((*(V+30+v)>SIGMIN)||(*(V+30+v+1)>SIGMIN)) )
        {
            if( *(V+30+v)>=*(V+30+v+1) )
            {
                swp_freq = fOSC_10*100 + afcex_freq;
                *SIG1 = *(V+30+v);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 5_1>>\n" ));
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs;
                *SIG1 = *(V+30+v+1);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 5_2>>\n" ));
            }
        }
        else if( (v+2==vmin)&&(*(V+30+v)>=0)&&(*(V+30+v+1)>=0)&&(*(V+30+v+2)>=0)    //negative case 6
            &&(*(V+30+v+1)>*(V+30+v))&&(*(V+30+v+2)>*(V+30+v))
            &&((*(V+30+v+1)>SIGMIN)||(*(V+30+v+2)>SIGMIN)) )
        {
            if( *(V+30+v+1)>=*(V+30+v+2) )
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs;
                *SIG1 = *(V+30+v+1);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 6_1>>\n" ));
            }
            else
            {
                swp_freq = fOSC_10*100 + afcex_freq + swp_ofs*2;
                *SIG1 = *(V+30+v+2);
                SDEBUG(FAPI_SYS_PRINT_MSG("--- *** <<neg 6_2>>\n" ));
            }
        }
        else if( (vmax==0)&&(vmin==0)&&(*(V+30+v)>SIGMIN) )
        {
            swp_freq = fOSC_10*100;
            *SIG1 = *(V+30+v);
        }
        else swp_freq = -1;
    }
    else swp_freq = -1;

    return swp_freq;
}

/*----------------------------------------------------------------------------
Name:        MB86667_TP_Search1
Description: tp search step 1
-------------------------------------------------------------------------------*/
int32_t MB86667_TP_Search1( int32_t i, int32_t v, int32_t vmax, int32_t *V, int32_t swp_freq, int32_t swp_ofs, int32_t *prev_f, int32_t smrtd )
{
    int32_t ret;
    int32_t k, dupl;
    uint32_t SIGMIN;

    SIGMIN = 20;

    if( (v==1)&&(*(V+1)>*V)&&(*(V+1)>SIGMIN) )
        ret = swp_freq - swp_ofs;
    else if( (v==2)&&(*(V+2)>*V)&&(*(V+1)>*V)&&((*(V+2)>SIGMIN)||(*(V+1)>SIGMIN)) )
    {
        if( *(V+2)>=*(V+1) )
            ret = swp_freq - 2*swp_ofs;
        else
            ret = swp_freq - swp_ofs;
    }
    else if( (v==3)&&(*(V+3)>*V)&&(*(V+2)>*V)&&(*(V+1)>*V)&&((*(V+3)>SIGMIN)||(*(V+2)>SIGMIN)) )
    {
        if( *(V+3)>=*(V+2) )
            ret = swp_freq - 3*swp_ofs;
        else
            ret = swp_freq - 2*swp_ofs;
    }
    else if( (v==vmax)&&(*V>*(V+1))&&(*V>SIGMIN) )
        ret = swp_freq - swp_ofs;
    else if( (v==vmax)&&(*V>*(V+2))&&(*(V+1)>*(V+2))&&((*V>SIGMIN)||(*(V+1)>SIGMIN)) )
    {
        if( *(V+1)>=*V )
            ret = swp_freq - swp_ofs;
        else
            ret = swp_freq;
    }
    else if( (v==vmax)&&(*V>*(V+3))&&(*(V+1)>*(V+3))&&(*(V+2)>*(V+3))&&((*V>SIGMIN)||(*(V+1)>SIGMIN)) )
    {
        if( *(V+1)>=*V )
            ret = swp_freq - swp_ofs;
        else
            ret = swp_freq;
    }
    else if( (*(V+2)>0)&&(*(V+1)>*V)&&(*(V+1)>*(V+2))&&(*(V+1)>SIGMIN) )
    {
        ret = swp_freq - swp_ofs;
    }
    else if( (*(V+3)>0)&&(*(V+1)>*V)&&(*(V+2)>*(V+3))&&((*(V+1)>SIGMIN)||(*(V+2)>SIGMIN)) )
    {
        if( *(V+2)>=*(V+1) )
            ret = swp_freq - 2*swp_ofs;
        else
            ret = swp_freq -   swp_ofs;
    }
    else if( (*(V+4)>0)&&(*(V+1)>*V)&&(*(V+2)>*V)&&(*(V+3)>*(V+4))&&(*(V+2)>*(V+4)) &&
           ( (*(V+1)>SIGMIN) || (*(V+2)>SIGMIN) || (*(V+3)>SIGMIN)) )
    {
        if(      (*(V+3)>=*(V+2))&&(*(V+3)>=*(V+1)) )
            ret = swp_freq - 3*swp_ofs;
        else if( (*(V+2)>=*(V+3))&&(*(V+2)>=*(V+1)) )
            ret = swp_freq - 2*swp_ofs;
        else
            ret = swp_freq -   swp_ofs;
    }
    else if( (*(V+5)>0) && (*(V+1)>*V) && (*(V+2)>*V) && (*(V+4)>*(V+5)) && (*(V+3)>*(V+5)) &&
           ( (*(V+1)>SIGMIN) || (*(V+2)>SIGMIN) || (*(V+3)>SIGMIN) || (*(V+4)>SIGMIN)) )
    {
        if(      (*(V+4)>=*(V+3))&&(*(V+4)>=*(V+2))&&(*(V+4)>=*(V+1)) )
            ret = swp_freq - 4*swp_ofs;
        else if( (*(V+3)>=*(V+4))&&(*(V+3)>=*(V+2))&&(*(V+3)>=*(V+1)) )
            ret = swp_freq - 3*swp_ofs;
        else if( (*(V+2)>=*(V+4))&&(*(V+2)>=*(V+3))&&(*(V+2)>=*(V+1)) )
            ret = swp_freq - 2*swp_ofs;
        else
            ret = swp_freq -   swp_ofs;
    }
    else ret = 0;

    if( ret>0 )
    {
        SDEBUG(FAPI_SYS_PRINT_MSG("F03 *** i=%d ret=%d\n", i, ret ));

        dupl = 0;
        for( k=0; k<6; k++ )
        {
            if( abs(ret-*(prev_f+k))<smrtd/3 ) dupl++;
        }
        if( dupl>0 )
            ret = 0;
        else
        {
            for( k=5; k>=1; k-- )
            {
                *(prev_f+k) = *(prev_f+k-1);
            }
            *prev_f = ret;
        }
        if( dupl==0 )
            SDEBUG(FAPI_SYS_PRINT_MSG("F03 *** Duplication chk OK\n" ));
        else
        {
            SDEBUG(FAPI_SYS_PRINT_MSG("F03 *** Duplication chk NG\n" ));
            for( k=0; k<6; k++ )
                SDEBUG(FAPI_SYS_PRINT_MSG("F03 *** prev_f=%d\n", *(prev_f+k) ));
        }
    }

    return ret;
}

/*----------------------------------------------------------------------------
Name:        MB86667_TP_Search2
Description: tp search step 2
-------------------------------------------------------------------------------*/
int32_t MB86667_TP_Search2( int32_t i, int32_t *SIGMAX, int32_t temp_freq,
                         int32_t *prev_f_L, int32_t *prev_f_H,
                         int32_t *prev_SR_L, int32_t *prev_SR_H,
                         int32_t *sch_ptn, int32_t *tmp_smrt, int32_t *delta_SIG,
                         int32_t RF_freq_step,
                         int32_t *sch_smrt_min, int32_t *sch_smrt_max,
                         int32_t swp_ofs,
                         int32_t *retval )
{
    int32_t iError;
    uint32_t tunerTries;
    int32_t smrt[9];
    int32_t j, k;
    uint8_t  DECI, CSEL, RSEL;
    int32_t clkmst;
    int32_t swp_freq;          /* [kHz] */
    int32_t afcex_freq;        /* [kHz] */
    int32_t fOSC_10;           /* [100kHz] */
    int32_t fOSC_temp_10[10];  /* [100kHz] */
    int32_t afcerr[10];        /* [kHz] */
    uint8_t  AFCEX_L, AFCEX_H;
    int32_t wait_t;
    uint8_t  SIG[9];
    int32_t dupl;
    int32_t ret, ret_bef=0;
    int32_t ref_SMRT=0;

    uint8_t  SIGMIN;
    uint8_t  SIG_L, SIG_L_MAX;
    int32_t peak_num1, peak_num3;
    int32_t pos, neg;
    int32_t peak_pos[8];
    int32_t peak_dupl[8];
    int32_t dupl_case;
    int32_t freq1, freq2;

    if( i==0 )
    {    /* high SR */
        smrt[0]= 5925;
        smrt[1]= 7640;
        smrt[2]= 9851;
        smrt[3]=12702;
        smrt[4]=16358;
        smrt[5]=21067;
        smrt[6]=27132;
        smrt[7]=34942;
        smrt[8]=45000;
    }
    else if( i==1 )
    {
        smrt[0]= 1663;
        smrt[1]= 2144;
        smrt[2]= 2764;
        smrt[3]= 3564;
        smrt[4]= 4595;
        smrt[5]= 5925;
        smrt[6]= 7640;
        smrt[7]= 9851;
        smrt[8]=12702;
    }
    else
    {
        smrt[0]=  500;
        smrt[1]=  630;
        smrt[2]=  794;
        smrt[3]= 1000;
        smrt[4]= 1289;
        smrt[5]= 1663;
        smrt[6]= 2144;
        smrt[7]= 2764;
        smrt[8]= 3564;
    }

    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** [TP_search2] called\n" ));    //for bug test

    if((iError=MB86667_AGC_SET( 2 ))!=FE_OK)
        return iError;
    if((iError=MB86667_Cntm_Set( 2, 0, 2 ))!=FE_OK)    //CNTM1=2,CNTM2=0,CMTM=2
        return iError;
    if((iError=MB86667_Dagc_Data_Set( 2, 0 ))!=FE_OK)    //DAGCA=2, DAGCW=0
        return iError;
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** i=%d,temp_freq=%07d\n", i, temp_freq ));
    if((iError=MB86667_S01t_Set( 7, 1 ))!=FE_OK)    //S1T=7, S0T=1
        return iError;
    if((iError=MB86667_En_Set( 0, 1 ))!=FE_OK)    //CREN=0, AFCEN=1
        return iError;
    if((iError=MB86667_Eq_Set( 1, 2 ))!=FE_OK)    //EQFEN=1, ESHIFT=2
        return iError;

    for( k=3; k<=8; k++ )
    {
        swp_freq = temp_freq;
        MB86667_Smrt_Info_Get2( smrt[k], &DECI, &CSEL, &RSEL, &clkmst );
        if((iError=MB86667_Smrt_Set( smrt[k], DECI, CSEL, RSEL, clkmst ))!=FE_OK)
           return iError;
        MB86667_S_Swp_Info_Get2( smrt[k], RF_freq_step, swp_freq, clkmst, &afcex_freq, &(fOSC_temp_10[k]), &AFCEX_L, &AFCEX_H );
        if((iError=MB86667_Afcofs_Data_Set( AFCEX_L, AFCEX_H ))!=FE_OK)
           return iError;
        if((iError=MB86667_Afcexen_Set( 0, smrt[k] ))!=FE_OK)
           return iError;
        tunerTries = MB86667_MAX_TUNER_TRIES;
        do
        {
            iError = mb86667_Rf_Val_Set_Ini( fOSC_temp_10[k], smrt[k] );
            tunerTries--;
            if (iError != FE_OK)
                MB86667_User_Sleep( MB86667_TUNER_PAUSE );
        } while ((iError != FE_OK) && (tunerTries > 0));
        if (iError != FE_OK)
        {
           SDEBUG_0(FAPI_SYS_PRINT_MSG("TP_Search2: Failed in mb86667_Rf_Val_Set_Ini()\n"));
           return(FE_ERR_FE_OTHER);
        }
        if((iError=MB86667_Srst())!=FE_OK)
           return iError;

        if( CSEL==0 ) wait_t = 3+(90000+999*smrt[k]/1000)/smrt[k];
        else          wait_t = 4+(90000+999*smrt[k]/1000)/smrt[k];

        MB86667_WAIT( wait_t );
        if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1,&(SIG[k])))!=FE_OK)
            return iError;
        if((iError=MB86667_Afcerr_Chk(clkmst,&(afcerr[k]))) != FE_OK )
           return iError;
        if( abs(afcerr[k])>(swp_ofs*2) ) SIG[k] = 0;

        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** swp=%07d fOSC=%07d afcofs=%07d,", swp_freq, fOSC_temp_10[k]*100, afcex_freq ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** freq=%07d,", fOSC_temp_10[k]*100+afcerr[k] ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** smrt[%d]=%05d SIG[%d]=%03d afcerr[%d]=%+05d\n", k, smrt[k], k, SIG[k], k, afcerr[k] ));
    }

    *SIGMAX = 0;
    SIGMIN  = 28;
    SIG_L_MAX = 50;

    for( j=0; j<8; j++ )
    {
        peak_pos[j] = 8;
    }

    peak_num1 = 0;
    pos = 1;
    neg = 0;
    for( j=4; j<=8; j++ )
    {
        if( SIG[j-1]<SIG[j] )
        {
            pos = 1;
            neg = 0;
        }
        if( (SIG[j-1]>SIG[j])&&(SIG[j-1]>SIGMIN) )
        {
            if( pos==1 )
            {
                peak_pos[peak_num1] = j-1;
                peak_num1++;
            }
            neg = 1;
            pos = 0;
        }
        if( (j==8) && (pos==1) && (SIG[8] > SIGMIN) )
        {
            peak_pos[peak_num1] = j;
            peak_num1++;
        }
    }
    if( peak_num1==0 )
        ret = 0;
    else
        ret = 1;

    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** [peak_num1=%d]\n", peak_num1 ));
    for( j=0; j<peak_num1; j++ )
    {
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_pos[%d]=%d\n", j, peak_pos[j] ));
    }
    peak_num3 = 0;
    if( ret>0 )
    {
        for( j=0; j<peak_num1; j++ )
        {
            if( peak_pos[j] < 6 )
            {
                if((iError=MB86667_En_Set( 0, 0 ))!=FE_OK)        //CREN=0, AFCEN=0
                    return iError;
                swp_freq = fOSC_temp_10[peak_pos[j]]*100+afcerr[peak_pos[j]];
                MB86667_Smrt_Info_Get2( smrt[peak_pos[j]-3], &DECI, &CSEL, &RSEL, &clkmst );
                if((iError=MB86667_Smrt_Set( smrt[peak_pos[j]-3], DECI, CSEL, RSEL, clkmst ))!=FE_OK)
                    return iError;
                MB86667_S_Swp_Info_Get2( smrt[peak_pos[j]-3], RF_freq_step, swp_freq, clkmst, &afcex_freq, &(fOSC_temp_10[peak_pos[j]-3]), &AFCEX_L, &AFCEX_H );
                if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H ))!=FE_OK)
                    return iError;
                if((iError=MB86667_Afcexen_Set( 1, smrt[peak_pos[j]-3] ))!=FE_OK)
                    return iError;
                tunerTries = MB86667_MAX_TUNER_TRIES;
                do
                {
                    iError = mb86667_Rf_Val_Set( fOSC_temp_10[peak_pos[j]-3], smrt[peak_pos[j]-3] );
                    tunerTries--;
                    if (iError != FE_OK)
                        MB86667_User_Sleep( MB86667_TUNER_PAUSE );
                } while ((iError != FE_OK) && (tunerTries > 0));
                if (iError != FE_OK)
                {
                   SDEBUG_0(FAPI_SYS_PRINT_MSG("TP_Search2: Failed in mb86667_Rf_Val_Set()\n"));
                   return(FE_ERR_FE_OTHER);
                }
                if((iError=MB86667_Srst())!=FE_OK)
                    return iError;
                if( CSEL==0 )
                    wait_t = 3+(70000+999*smrt[peak_pos[j]-3]/1000) / smrt[peak_pos[j]-3];
                else
                    wait_t = 4+(70000+999*smrt[peak_pos[j]-3]/1000) / smrt[peak_pos[j]-3];
                MB86667_WAIT( wait_t );
                if( (iError = MB86667_I2C_READ(MB86667_REG_SIG_1, &SIG_L)) != FE_OK)
                    return iError;
                SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** SIG_L newly read freq=%07d sr=%05d value=%03d\n", swp_freq, smrt[peak_pos[j]-3], SIG_L ));
            }
            else
            {
                SIG_L = SIG[peak_pos[j]-3];
                SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** SIG[%d] was used for SIG_L value=%03d\n", peak_pos[j]-3, SIG_L ));
            }

            if( (SIG_L < SIG[peak_pos[j]]) && (SIG_L < SIG_L_MAX) &&
                (((int32_t)(SIG[peak_pos[j]]-SIG_L)*100)/(int32_t)SIG[peak_pos[j]] > (int32_t)(SIG[peak_pos[j]]*20-300)/(int32_t)(SIG[peak_pos[j]]+3)) )
            {
                peak_num3++;
                SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_pos[%d]=%d delta_pos2[%d]=%d SIG_L=%03d SIG[%d]=%03d SIG_L=OK peak_num3=%d\n",
                                j, peak_pos[j], j, delta_pos2[j], SIG_L, peak_pos2[j], SIG[peak_pos2[j]], peak_num3 ));
            }
            else
            {
                SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_pos2[%d]=%d delta_pos2[%d]=%d SIG_L=%03d SIG[%d]=%03d SIG_L=NG peak_num3=%d\n",
                                j, peak_pos2[j], j, delta_pos2[j], SIG_L, peak_pos2[j], SIG[peak_pos2[j]], peak_num3 ));
            }
        }
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** [peak_num3=%d]\n", peak_num3 ));
    }

    if( peak_num3==0 ) ret = 0;
    else               ret = 1;

    *sch_smrt_min = 0;
    *sch_smrt_max = 0;
    *sch_ptn = 9;

    for (j=0; j<8; j++)
        peak_dupl[j] = 0;

    if( ret==1 )
    {
        for( j=0; j<peak_num3; j++ )
        {
            for( k=peak_pos[j]+1 ; k<peak_pos[j+1] ; k++ )
            {
                if( SIG[peak_pos[j]] == SIG[k] )
                {
                    peak_dupl[j]++ ;
                }
            }
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_dupl[%d]=%d\n", j, peak_dupl[j] ));
        }

        if( peak_num3==1 )
        {
            if( peak_dupl[0]==0 )
            {
                if( peak_pos[0]==3 )
                {
                    *sch_smrt_min = smrt[3];
                    *sch_smrt_max = (smrt[4] + smrt[5]) / 2;
                    ret = fOSC_temp_10[3]*100 + afcerr[3];
                    *sch_ptn = 2;    //scan start from slowest SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=1\n" ));

                }
                else if(peak_pos[0]==4 )
                {
                    *sch_smrt_min = smrt[3] ;
                    *sch_smrt_max = (smrt[5]+smrt[6])/2 ;
                    ret = fOSC_temp_10[3]*100 + afcerr[3] ;
                    *sch_ptn = 2 ;  //scan start from slowest SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=1\n" ));
                }
                else if(peak_pos[0]==8 )
                {
                    *sch_smrt_min = (smrt[6] + smrt[7]) / 2;
                    *sch_smrt_max = smrt[8];
                    ret = fOSC_temp_10[8]*100 + afcerr[8];
                    *sch_ptn = 0;    //scan start from highest SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=2\n" ));
                }
                else
                {
                    if( (int32_t)(SIG[peak_pos[0]] - SIG[peak_pos[0]-1]) > ((int32_t)(SIG[peak_pos[0]]*8)/10) )
                    {
                        *sch_smrt_min = smrt[peak_pos[0]-1] ;
                    }
                    else if( (int32_t)(SIG[peak_pos[0]] - SIG[peak_pos[0]-1])>((int32_t)(SIG[peak_pos[0]]*9)/10) )
                    {
                        *sch_smrt_min = (smrt[peak_pos[0]-1]+smrt[peak_pos[0]-2])/2 ;
                    }
                    else
                    {
                        *sch_smrt_min = smrt[peak_pos[0]-2] ;
                    }
                    if( ((int32_t)(SIG[peak_pos[0]]-SIG[peak_pos[0]+1]) < 3) && (peak_pos[0] < 7) )
                    {
                        *sch_smrt_max = (smrt[peak_pos[0]+1]+smrt[peak_pos[0]+2])/2 ;
                    }
                    else
                    {
                        *sch_smrt_max = smrt[peak_pos[0]+1] ;
                    }
                    ret = fOSC_temp_10[peak_pos[0]]*100 + afcerr[peak_pos[0]] ;
                    *sch_ptn = 1 ;    //scan start from middle SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=3\n" ));
                }
            }
            else if( peak_dupl[0] > 0 )
            {
                if( peak_pos[0] == 3 )
                {
                    *sch_smrt_min = smrt[3];
                    *sch_smrt_max = (smrt[4]+smrt[4+peak_dupl[0]])/2 ;
                    ret = fOSC_temp_10[3]*100 + afcerr[3];
                    *sch_ptn = 1;    //scan start from middle SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=4\n" ));
                }
                else if( peak_pos[0] == 8-peak_dupl[0] )
                {
                    *sch_smrt_min = (smrt[7-peak_dupl[0]]+smrt[7])/2 ;
                    *sch_smrt_max = smrt[8];
                    ret = fOSC_temp_10[7]*100 + afcerr[7];
                    *sch_ptn = 1;    //scan start from middle SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=5\n" ));
                }
                else
                {
                    *sch_smrt_min = (smrt[peak_pos[0]]  +smrt[peak_pos[0]-peak_dupl[0]])/2 ;
                    *sch_smrt_max = (smrt[peak_pos[0]+peak_dupl[0]]+smrt[peak_pos[0]+peak_dupl[0]+1])/2 ;
                    ret = fOSC_temp_10[peak_pos[0]]*100 + afcerr[peak_pos[0]] ;
                    *sch_ptn = 1;    //scan start from middle SR
                    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=6\n" ));
                }
            }
            else
            {
                *sch_smrt_min = 0;
                *sch_smrt_max = 0;
                ret = 0;
            }
        }
        else if( peak_num3>=2 )
        {
            *sch_smrt_min = smrt[peak_pos[0]-1] ;
            if( peak_pos[peak_num3-1]==8 ) *sch_smrt_max = smrt[peak_pos[peak_num3-1]] ;
            else                           *sch_smrt_max = smrt[peak_pos[peak_num3-1]+1] ;
            freq1 = fOSC_temp_10[peak_pos[0]          ]*100 + afcerr[peak_pos[0]] ;
            freq2 = fOSC_temp_10[peak_pos[peak_num3-1]]*100 + afcerr[peak_pos[peak_num3-1]] ;
            ret = (freq1+freq2)/2 ;
            *sch_ptn = 1 ;
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** peak_num=1:case=7\n" ));
        }
        else
        {
            *sch_smrt_min = 0;
            *sch_smrt_max = 0;
            ret = 0;
        }
    }
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_min_first=%05d\n", *sch_smrt_min ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_max_first=%05d\n", *sch_smrt_max ));

    /* duplication check */
    dupl = 0;
    if( ret>0 )
    {
        for( k=0; k<6; k++ )
        {
            if( (*(prev_f_L+k)<ret)&&(ret<*(prev_f_H+k)) )
            {
                if( (*(prev_SR_H+k)<=*(sch_smrt_min))||(*sch_smrt_max<=*(prev_SR_L+k)) )
                {
                    *sch_smrt_min = *sch_smrt_min;
                    *sch_smrt_max = *sch_smrt_max;
                    dupl_case = 1;
                }
                else if( (*sch_smrt_min<*(prev_SR_H+k))&&(*(prev_SR_H+k)<*sch_smrt_max) )
                {
                    *sch_smrt_min = *(prev_SR_H+k);
                    *sch_smrt_max = *sch_smrt_max;
                    dupl_case = 2;
                }
                else if( (*(prev_SR_L+k)<=*sch_smrt_min)&&(*sch_smrt_max<=*(prev_SR_H+k)) )
                {
                    *sch_smrt_min = 0;
                    *sch_smrt_max = 0;
                    dupl++;
                    dupl_case = 3;
                }
                else if( (*sch_smrt_min<*(prev_SR_L+k))&&(*(prev_SR_H+k)<*sch_smrt_max) )
                {
                    if( *(prev_SR_L+k)-*sch_smrt_min>*sch_smrt_max-*(prev_SR_H+k) )
                    {
                        *sch_smrt_min = *sch_smrt_min;
                        *sch_smrt_max = *(prev_SR_L+k);
                        dupl_case = 4;
                    }
                    else
                    {
                        *sch_smrt_min = *(prev_SR_H+k);
                        *sch_smrt_max = *sch_smrt_max;
                        dupl_case = 5;
                    }
                }
                else if( (*sch_smrt_min<*(prev_SR_L+k))&&(*sch_smrt_max<*(prev_SR_H+k)) )
                {
                    *sch_smrt_min = *sch_smrt_min;
                    *sch_smrt_max = *(prev_SR_L+k);
                    dupl_case = 6;
                }
            }
            else dupl_case = 0;
        }

        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_min_second=%05d\n", *sch_smrt_min ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_max_second=%05d\n", *sch_smrt_max ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** dupl_case=%d\n", dupl_case ));
        ref_SMRT = (*sch_smrt_min*6+*sch_smrt_max*4)/10;
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** ref_SMRT=%05d\n", ref_SMRT ));
        if( dupl>0 )
        {
            ret = 0;
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** Duplication chk=NG\n" ));
        }
        else
        {
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** Duplication chk=OK\n" ));
        }
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** prev_f_L=%07d,%07d,%07d,%07d,%07d,%07d\n",
                            *prev_f_L,     *(prev_f_L+1), *(prev_f_L+2),
                            *(prev_f_L+3), *(prev_f_L+4), *(prev_f_L+5) ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** prev_f_H=%07d,%07d,%07d,%07d,%07d,%07d\n",
                            *prev_f_H,     *(prev_f_H+1), *(prev_f_H+2),
                            *(prev_f_H+3), *(prev_f_H+4), *(prev_f_H+5) ));

        ret_bef = ret;
    }

    if( ret>0 )
    {
        if((iError=MB86667_AGC_SET( 2 ))!=FE_OK)    //AGCW=2
            return iError;
        if((iError=MB86667_Dagc_Data_Set( 3, 0 ))!=FE_OK)    //DAGCA=3, DAGCW=0
            return iError;
        if((iError=MB86667_S01t_Set( 0, 0 ))!=FE_OK)    //S1T=0, S0T=0
            return iError;
        if((iError=MB86667_Eq_Set( 1, 3 ))!=FE_OK)
            return iError;
        if((iError=MB86667_Cntm_Set( 2, 0, 2 ))!=FE_OK)    //test
            return iError;
        if((iError=MB86667_En_Set( 0, 0 ))!=FE_OK)    //CREN=0, AFCEN=0
            return iError;
        for( k=0; k<3; k++ )
        {
            MB86667_Smrt_Info_Get( ref_SMRT, &DECI, &CSEL, &RSEL, &clkmst );
            if((iError=MB86667_Smrt_Set( ref_SMRT, DECI, CSEL, RSEL, clkmst ))!=FE_OK)
                return iError;
            swp_freq = ret + (k-1) * 6 * ref_SMRT/10;
            MB86667_S_Swp_Info_Get2( ref_SMRT, RF_freq_step, swp_freq, clkmst, &afcex_freq, &fOSC_10, &AFCEX_L, &AFCEX_H );
            if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H ))!=FE_OK)
                return iError;
            if((iError=MB86667_Afcexen_Set( 1, ref_SMRT ))!=FE_OK)
                return iError;
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set( fOSC_10, ref_SMRT );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if (iError != FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("TP_Search2: Failed in mb86667_Rf_Val_Set()\n"));
               return(FE_ERR_FE_OTHER);
            }
            if((iError=MB86667_Srst())!=FE_OK)
                return iError;
            if( CSEL==0 )
                wait_t = (66000+999*ref_SMRT/1000)/ref_SMRT;
            else
                wait_t = (66000+999*ref_SMRT/1000)/ref_SMRT;

            if( wait_t==1 )
                wait_t = 2;
            MB86667_WAIT( wait_t );
            if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1, &(SIG[k]))) != FE_OK )
                return iError;
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** swp_freq=%07d SIG[%d]=%03d\n", swp_freq, k, SIG[k] ));
        }
        if( (int32_t)(SIG[1]-SIG[0]) > (int32_t)(SIG[1]-SIG[2]) )
            *delta_SIG = SIG[1] - SIG[2];
        else
            *delta_SIG = SIG[1] - SIG[0];
        if( SIG[1]==0 )
            ret = 0;
        else if( ( (*delta_SIG*100)/(int32_t)SIG[1] < ( ((int32_t)(SIG[1]/4) - 3) * 100 / (int32_t)(SIG[1]+3) )) &&
                 (*delta_SIG < 10) )
        {
            ret = 0;
            *SIGMAX = 0;
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** delta_SIG=%03d : delta_SIG NG\n", *delta_SIG ));
        }
        else
        {
            *SIGMAX = SIG[1];
            SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** delta_SIG=%03d : delta_SIG OK\n", *delta_SIG ));
        }

        for( k=5; k>=1; k-- )
        {
            *(prev_f_L+k) = *(prev_f_L+k-1);
            *(prev_f_H+k) = *(prev_f_H+k-1);
            *(prev_SR_L+k) = *(prev_SR_L+k-1);
            *(prev_SR_H+k) = *(prev_SR_H+k-1);
        }
        *prev_f_L = ret_bef - ref_SMRT/2*65/100;
        *prev_f_H = ret_bef + ref_SMRT/2*65/100;
        *prev_SR_L = *sch_smrt_min;
        *prev_SR_H = *sch_smrt_max;
    }

    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_min=%05d\n", *sch_smrt_min ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_smrt_max=%05d\n", *sch_smrt_max ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** sch_ptn=%d\n", *sch_ptn ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F06 *** ret=%07d\n\n", ret ));

    *tmp_smrt = 0;    /* this is required by Blind_scan */

    if((iError=MB86667_AGC_SET( 4 ))!=FE_OK)    /* AGCW=4 */
        return iError;
    if((iError=MB86667_Cntm_Set( 0, 1, 2 ))!=FE_OK)
        return iError;
    if((iError=MB86667_Dagc_Data_Set( 3, 2 ))!=FE_OK)    /* DAGCA=3, DAGCW=2 */
        return iError;
    if((iError=MB86667_Afcexen_Set( 1, *tmp_smrt ))!=FE_OK)
        return iError;
    if((iError=MB86667_S01t_Set( 0, 0 ))!=FE_OK)    /* S1T=0, S0T=0(set to defalut value) */
        return iError;
    if((iError=MB86667_En_Set( 0, 0 ))!=FE_OK)      /* CREN=0, AFCEN=0 */
        return iError;
    if((iError=MB86667_Eq_Set( 0, 3 ))!=FE_OK)      /* EQFEN=0, ESHIFT=3 */
        return iError;

    *retval = ret;
    return iError;
}

/*----------------------------------------------------------------------------
Name:        MB86667_SR_Scan
Description: scan symbol rate
-------------------------------------------------------------------------------*/
int32_t MB86667_SR_Scan( int32_t i, int32_t SIGMAX, int32_t swp_freq, int32_t sch_ptn, int32_t SR_min,
             int32_t sch_smrt_min, int32_t sch_smrt_max, int32_t RF_freq_step, int32_t *retval)
{
    int32_t iError;
    uint32_t tunerTries;
    int32_t smrt_min, smrt_max;
    int32_t smrt;
    int32_t j, k;
    uint8_t DECI, CSEL, RSEL;
    int32_t clkmst;
    int32_t afcex_freq, fOSC;
    uint8_t AFCEX_L, AFCEX_H;
    int32_t wait_t;
    int32_t P[5], M[5];
    uint8_t SIG;
    int32_t smrt_p, smrt_m;
    int32_t loop;
    int32_t smrt_fin=0;
    int32_t p_fin, m_fin;
    int32_t wait_sym;
    int32_t step;
    int32_t smrt_temp;
    uint8_t SIGMIN2;
    uint8_t byte;

    SIGMIN2 = 53;
    step = 1006;

    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** [SR_scan] called\n" ));

    smrt_max = sch_smrt_max*step/1000*step/1000*step/1000*step/1000;
    smrt_min = sch_smrt_min*1000/step*1000/step*1000/step*1000/step;
    if( smrt_min<SR_min )
        smrt_min = SR_min*1000/step*1000/step*1000/step*1000/step;

    if( sch_ptn==0 )
    {
        smrt_p = 100000;
        smrt_m = smrt_max;
        p_fin = 1;
        m_fin = 0;
    }
    else if( sch_ptn==1 )
    {
        smrt_p = (smrt_max+smrt_min)/2*1000/step*1000/step*1000/step*1000/step;
        smrt_m = (smrt_max+smrt_min)/2*step/1000*step/1000*step/1000*step/1000;
        p_fin = 0;
        m_fin = 0;
    }
    else
    {
        smrt_p = smrt_min;
        smrt_m = 0;
        p_fin = 0;
        m_fin = 1;
    }

    if((iError=MB86667_AGC_SET( 2 )) != FE_OK ) return iError;                //AGCW=2
    if((iError=MB86667_Dagc_Data_Set( 2, 0 )) != FE_OK ) return iError;       //DAGCA=2, DAGCW=0
    if((iError=MB86667_Stlp_Set( 7, 7 )) != FE_OK ) return iError;            //STRAS=7,STRBS=7

    byte = 18;
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN1_2, byte)) != FE_OK ) return iError;    //STRWS=18
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN2_2, byte)) != FE_OK ) return iError;    //STRWS=18

    if((iError=MB86667_Cntm_Set( 0, 1, 2 )) != FE_OK ) return iError;
    if((iError=MB86667_En_Set( 0, 0 )) != FE_OK ) return iError;    //CREN=0, AFCEN=0
    if((iError=MB86667_S01t_Set( 0, 0 )) != FE_OK ) return iError;  //S1T=0, S0T=0
    if((iError=MB86667_Eq_Set( 0, 2 )) != FE_OK ) return iError;    //changed

    wait_sym = 16000;

    loop = 1;
    for( j=0; j<5; j++ )
    {
        P[j] = 255;
        M[j] = 255;
    }

    j = 0;
    if( sch_ptn==0 ) j++;

    SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** [i=%d swp_freq=%d]\n", i, swp_freq ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** sch_ptn=%d\n", sch_ptn ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** smrt_max=%05d,smrt_min=%05d\n", smrt_max, smrt_min ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** SIGMAX=%03d\n", SIGMAX ));

    SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** [i=%d swp_freq=%d]\n", i, swp_freq ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** sch_ptn=%d\n", sch_ptn ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** smrt_max=%05d,smrt_min=%05d\n", smrt_max, smrt_min ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** SIGMAX=%03d\n", SIGMAX ));

    while( loop==1 )
    {
        if( j%2==0 ) smrt = smrt_p;
        else         smrt = smrt_m;
        MB86667_Smrt_Info_Get( smrt, &DECI, &CSEL, &RSEL, &clkmst );
        if((iError=MB86667_Smrt_Set( smrt, DECI, CSEL, RSEL, clkmst )) != FE_OK ) return iError;
        MB86667_S_Swp_Info_Get2( smrt, RF_freq_step, swp_freq, clkmst, &afcex_freq, &fOSC, &AFCEX_L, &AFCEX_H );
        if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H )) != FE_OK ) return iError;
        if((iError=MB86667_Afcexen_Set( 1, smrt )) != FE_OK ) return iError;
        if( i==0 )
        {
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set_Ini( fOSC, smrt );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("SR_Scan: Failed in mb86667_Rf_Val_Set_Ini()\n"));
               return(FE_ERR_FE_OTHER);
            }
        }
        else
        {
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set( fOSC, smrt );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("SR_Scan: Failed in mb86667_Rf_Val_Set()\n"));
               return(FE_ERR_FE_OTHER);
            }
        }
        if((iError=MB86667_Srst()) != FE_OK )
            return iError;

        if( i==0 )
        {
            if( CSEL==0 ) wait_t = 3+(wait_sym+999*smrt/1000)/smrt ;
            else          wait_t = 4+(wait_sym+999*smrt/1000)/smrt ;
        }
        else
        {
            wait_t = (wait_sym+999*smrt/1000)/smrt ;
        }

        MB86667_WAIT( wait_t );
        if((iError=MB86667_I2C_READ(MB86667_REG_SIG_1, &SIG)) != FE_OK )
            return iError;

        SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** j=%03d,smrt=%05d,smrt_p=%05d,smrt_m=%05d,SIG=%03d,", j, smrt, smrt_p, smrt_m, SIG ));

        if( j%2==0 )
        {
            for( k=4; k>0; k-- )
                P[k] = P[k-1];
            P[0] = SIG;
        }
        else
        {
            for( k=4; k>0; k-- )
                M[k] = M[k-1];
            M[0] = SIG;
        }

        if( (j%2==0)&&(smrt_p>=smrt_max) )
        {
            p_fin = 1;
        }
        if( (j%2==1)&&(smrt_m<=smrt_min) )
        {
            m_fin = 1;
        }
        if( j%2==0 )
            SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** P[4-0]=%03d,%03d,%03d,%03d%,%03d ", P[4], P[3], P[2], P[1], P[0] ));
        else
            SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** M[4-0]=%03d,%03d,%03d,%03d%,%03d ", M[4], M[3], M[2], M[1], M[0] ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** p_fin=%d m_fin=%d\n", p_fin, m_fin ));

        if( ( (P[2]-P[0])*10 > (P[2]*10/7)) && 
            ( (P[2]-P[4])*10 > (P[2]*10/7)) &&
              (P[2]          > (int32_t)SIGMIN2) && (j%2==0) )
        {
            smrt_fin = smrt_p*1000/step*1000/step;
            loop = 0;
            if((iError=MB86667_SR_Chk( i, swp_freq, smrt_fin, RF_freq_step, &smrt_temp)) != FE_OK )
                return iError;
            smrt_fin = smrt_temp;
            SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** P[4-0]=%03d,%03d,%03d,%03d%,%03d\n", P[4], P[3], P[2], P[1], P[0] ));
        }
        else if( ( ((P[2]-P[0])*10 > (P[2]*10/13)) || ((P[2]-P[1])*10>(P[2]*10/13))) &&
                 ( ((P[2]-P[4])*10 > (P[2]*10/13)) || ((P[2]-P[3])*10>(P[2]*10/13))) &&
                 (  P[2]           > (int32_t)SIGMIN2) && (j%2==0) )
        {
            smrt_fin = smrt_p*1000/step*1000/step;
            if((iError=MB86667_SR_Chk( i, swp_freq, smrt_fin, RF_freq_step, &smrt_temp )) != FE_OK )
                return iError;
            if( smrt_temp>0 )
            {
                loop = 0;
                smrt_fin = smrt_temp;
                SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** P[4-0]=%03d,%03d,%03d,%03d%,%03d\n", P[4], P[3], P[2], P[1], P[0] ));
            }
        }
        else if( ( (M[2]-M[0])*10 > (M[2]*10/7)) &&
                 ( (M[2]-M[4])*10 > (M[2]*10/7)) &&
                 (  M[2]          > (int32_t)SIGMIN2) && (j%2==1) )
        {
            smrt_fin = smrt_m*step/1000*step/1000;
            loop = 0;
            if((iError=MB86667_SR_Chk( i, swp_freq, smrt_fin, RF_freq_step, &smrt_temp )) != FE_OK )
                return iError;
            smrt_fin = smrt_temp;
            SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** M[4-0]=%03d,%03d,%03d,%03d%,%03d\n", M[4], M[3], M[2], M[1], M[0] ));
        }
        else if( ( ((M[2]-M[0])*10 > (M[2]*10/13)) || ((M[2]-M[1])*10 > (M[2]*10/13))) &&
                 ( ((M[2]-M[4])*10 > (M[2]*10/13)) || ((M[2]-M[3])*10 > (M[2]*10/13))) &&
                 (   M[2]          > (int32_t)SIGMIN2) && (j%2==1) )
        {
            smrt_fin = smrt_m*step/1000*step/1000;
            if((iError=MB86667_SR_Chk( i, swp_freq, smrt_fin, RF_freq_step, &smrt_temp )) != FE_OK )
                return iError;
            if( smrt_temp>0 )
            {
                loop = 0;
                smrt_fin = smrt_temp;
                SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** M[4-0]=%03d,%03d,%03d,%03d%,%03d\n", M[4], M[3], M[2], M[1], M[0] ));
            }
        }
        else if( (p_fin==1)&&(m_fin==1) )
        {
            loop = 0;
            smrt_fin = 0;
        }

        if( j%2==0 ) smrt_p = smrt_p*step/1000;
        else         smrt_m = smrt_m*1000/step;

        if( (j%2==0)&&(smrt_p>=smrt_max) )
            SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** smrt_p>=smrt_max : p_fin=1 j=%d\n", j ));
        if( (j%2==1)&&(smrt_m<=smrt_min) )
            SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** smrt_m<=smrt_min : m_fin=1 j=%d\n", j ));

        j++;
        if( sch_ptn==1 )
        {
            if( (p_fin==1)&&(j%2==0) ) j++;
            if( (m_fin==1)&&(j%2==1) ) j++;
        }
        if( !(sch_ptn==1) ) j++;

    }

    SDEBUG(FAPI_SYS_PRINT_MSG("F05 *** \n" ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F11 *** \n" ));

    SDEBUG(FAPI_SYS_PRINT_MSG("F04 *** i=%d,swp_freq=%07d,sch_ptn=%d,", i, swp_freq, sch_ptn ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F04 *** smrt_fin=%05d,", smrt_fin ));
    if( sch_ptn==1 ) SDEBUG(FAPI_SYS_PRINT_MSG("F04 *** j=%03d ", j ));
    else             SDEBUG(FAPI_SYS_PRINT_MSG("F04 *** j=%03d ", j/2 ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F04 *** sch_smrt_min,max=%05d,%05d ", sch_smrt_min, sch_smrt_max ));

    /* setting for "Blind_scan */
    if((iError=MB86667_S01t_Set( 0, 0 )) != FE_OK ) return iError;
    if((iError=MB86667_AGC_SET( 4 )) != FE_OK ) return iError;                //AGCW=4
    if((iError=MB86667_Dagc_Data_Set( 3, 2 )) != FE_OK ) return iError;       //DAGCA=3, DAGCW=2
    if((iError=MB86667_Stlp_Set( 5, 7 )) != FE_OK ) return iError;            //STRAS=5,STRBS=7

    byte = 10;
    if((iError=MB86667_I2C_WRITE( MB86667_REG_STR_WIN1_2, byte )) != FE_OK ) return iError;    //STRWS=10
    if((iError=MB86667_I2C_WRITE( MB86667_REG_STR_WIN2_2, byte )) != FE_OK ) return iError;    //STRW=10
    if((iError=MB86667_Cntm_Set( 0, 1, 2 )) != FE_OK ) return iError;

    *retval = smrt_fin;
    return iError;
}

/*----------------------------------------------------------------------------
Name:        MB86667_SR_Chk
Description: check symbol rate
-------------------------------------------------------------------------------*/
int32_t MB86667_SR_Chk( int32_t i, int32_t swp_freq, int32_t smrt, int32_t RF_freq_step, int32_t *retval )
{
    int32_t iError;
    uint32_t tunerTries;
    uint8_t DECI, CSEL, RSEL;
    uint8_t AFCEX_L, AFCEX_H;
    uint8_t STRMIN, STRMAX;
    int32_t j, k;
    int32_t clkmst;
    int32_t afcex_freq;
    int32_t fOSC;
    int32_t wait_t;
    int32_t strmin=0, strmax=0;
    int32_t loop, loop2;
    int32_t delta_str=0;
    int32_t ret;
    int32_t smrt_test=0;
    int32_t delta_str_max;
    uint8_t byte;

    delta_str_max = 50;

    SDEBUG(FAPI_SYS_PRINT_MSG("--- *** [SR_chk] called\n" ));
    SDEBUG(FAPI_SYS_PRINT_MSG("F16 *** swp_freq=%07d smrt=%05d\n", swp_freq, smrt ));

    if((iError=MB86667_AGC_SET( 2 )) != FE_OK )
        return iError;                //AGCW=2
    if((iError=MB86667_Dagc_Data_Set( 2, 0 )) != FE_OK )
        return iError;       //DAGCA=2, DAGCW=0
    if((iError=MB86667_Stlp_Set( 7, 7 )) != FE_OK )
        return iError;            //STRAS=7,STRBS=7
    byte = 18;
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN1_2, byte)) != FE_OK )
        return iError;    //STRWS=18
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN2_2, byte)) != FE_OK )
        return iError;    //STRWS=18

    if((iError=MB86667_En_Set( 0, 0 )) != FE_OK )
        return iError;    //CREN=0, AFCEN=0
    if((iError=MB86667_S01t_Set( 0, 1 )) != FE_OK )
        return iError;  //S1T=0, S0T=1
    if((iError=MB86667_Eq_Set( 0, 2 )) != FE_OK )
        return iError;    //changed
    if((iError=MB86667_Afcexen_Set( 1, (uint16_t) smrt )) != FE_OK )
        return iError;

    loop2 = 1;
    j = 0;
    while( loop2==1 )
    {
        if( j==0 )
            smrt_test = smrt;
        else if( j==1 )
            smrt_test = smrt + smrt*6/1000;
        else
            smrt_test = smrt - smrt*6/1000;
        MB86667_Smrt_Info_Get( smrt_test, &DECI, &CSEL, &RSEL, &clkmst );
        if((iError=MB86667_Smrt_Set( smrt_test, DECI, CSEL, RSEL, clkmst )) != FE_OK )
            return iError;
        MB86667_S_Swp_Info_Get2( smrt_test, RF_freq_step, swp_freq, clkmst, &afcex_freq, &fOSC, &AFCEX_L, &AFCEX_H );
        if((iError=MB86667_Acfex_Data_Set( AFCEX_L, AFCEX_H )) != FE_OK )
            return iError;

        if( j==0 )
        {
            tunerTries = MB86667_MAX_TUNER_TRIES;
            do
            {
                iError = mb86667_Rf_Val_Set_Ini( fOSC, smrt_test );
                tunerTries--;
                if (iError != FE_OK)
                    MB86667_User_Sleep( MB86667_TUNER_PAUSE );
            } while ((iError != FE_OK) && (tunerTries > 0));
            if(iError!=FE_OK)
            {
               SDEBUG_0(FAPI_SYS_PRINT_MSG("SR_Chk: Failed in mb86667_Rf_Val_Set_Ini()\n"));
               return(FE_ERR_FE_OTHER);
            }
        }
        tunerTries = MB86667_MAX_TUNER_TRIES;
        do
        {
            iError = mb86667_Rf_Val_Set( fOSC, smrt_test );
            tunerTries--;
            if (iError != FE_OK)
                MB86667_User_Sleep( MB86667_TUNER_PAUSE );
        } while ((iError != FE_OK) && (tunerTries > 0));
        if(iError!=FE_OK)
        {
           SDEBUG_0(FAPI_SYS_PRINT_MSG("SR_Chk: Failed in mb86667_Rf_Val_Set()\n"));
           return(FE_ERR_FE_OTHER);
        }


        if((iError=MB86667_Srst()) != FE_OK )
            return iError;
        if( CSEL==0 )
            wait_t = 3;
        else
            wait_t = 4;
        MB86667_WAIT( wait_t );

        loop = 1;
        k = 0;
        wait_t = 5000/smrt_test;
        if( wait_t==0 )
            wait_t = 1;
        while( loop==1 )
        {
            MB86667_WAIT( wait_t );
            if((iError=MB86667_I2C_READ(MB86667_REG_STR_F_MIN, &STRMIN)) != FE_OK )
                return iError;
            if((iError=MB86667_I2C_READ(MB86667_REG_STR_F_MAX, &STRMAX)) != FE_OK )
                return iError;
            if( STRMIN>127 )
                strmin = STRMIN - 256;
            else
                strmin = STRMIN;
            if( STRMAX>127 )
                strmax = STRMAX - 256;
            else
                strmax = STRMAX;
            delta_str = strmax - strmin;
            if( delta_str>=delta_str_max )
                loop = 0;
            k++;
            if( k>=20 )
                loop = 0;
        }
        SDEBUG(FAPI_SYS_PRINT_MSG("F16 *** i=%d, j=%d, swp_freq=%07d, smrt_test=%05d\n", i, j, swp_freq, smrt_test ));
        SDEBUG(FAPI_SYS_PRINT_MSG("F16 *** strmin=%+04d,strmax=%+04d\n", strmin, strmax ));

        j++;
        if( (j>2)||(delta_str<delta_str_max ) )
            loop2 = 0;
    }

    if( delta_str>delta_str_max )
    {
        ret = 0;
        SDEBUG(FAPI_SYS_PRINT_MSG("F16 *** delta STRM too large\n\n" ));
    }
    else
    {
        ret = smrt_test - (strmax+strmin)/2*clkmst/(1<<DECI)/65536;
        SDEBUG(FAPI_SYS_PRINT_MSG("F16 *** delta STRM small enough smrt=%05d\n\n", ret ));
    }

    //setting for SR_scan
    if((iError=MB86667_AGC_SET( 2 )) != FE_OK )
        return iError;                //AGCW=2
    if((iError=MB86667_Dagc_Data_Set( 2, 0 )) != FE_OK )
        return iError;        //DAGCA=2, DAGCW=0
    if((iError=MB86667_Stlp_Set( 7, 7 )) != FE_OK )
        return iError;            //STRAS=7,STRBS=7
    byte = 18;
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN1_2, byte)) != FE_OK )
        return iError;    //STRWS=18
    if((iError=MB86667_I2C_WRITE(MB86667_REG_STR_WIN2_2, byte)) != FE_OK )
        return iError;    //STRWS=18

    if((iError=MB86667_Cntm_Set( 0, 1, 2 )) != FE_OK )
        return iError;
    if((iError=MB86667_En_Set( 0, 0 )) != FE_OK )
        return iError;    //CREN=0, AFCEN=0
    if((iError=MB86667_S01t_Set( 0, 1 )) != FE_OK )
        return iError;    //S1T=0, S0T=1
    if((iError=MB86667_Eq_Set( 0, 2 )) != FE_OK )
        return iError;    //changed

    *retval =ret;
    return iError;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Afcerr_Chk
-------------------------------------------------------------------------------*/
int32_t MB86667_Afcerr_Chk(int32_t clkmst, int32_t *afcerr)
{
    int32_t iError;
    uint8_t AFCM_L, AFCM_H;
    int32_t AFCM;
    int32_t afcm;

    if((iError=MB86667_I2C_READ(MB86667_REG_AFCML,&AFCM_L))!=FE_OK)
        return iError;
    if((iError=MB86667_I2C_READ(MB86667_REG_AFCMH,&AFCM_H))!=FE_OK)
        return iError;
    AFCM = (AFCM_H<<8) + AFCM_L;

    if( AFCM>2048 )
        afcm = AFCM- 4096;
    else
        afcm = AFCM;
    *afcerr = afcm*clkmst/8192;
    if( TUNER_POLARITY==-1)
        *afcerr = - (*afcerr);
    return iError;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Smrt_Info_Get2
-------------------------------------------------------------------------------*/
void MB86667_Smrt_Info_Get2( int32_t sr, uint8_t *DECI, uint8_t *CSEL, uint8_t *RSEL, int32_t *clkmst)
{

    if(      sr>=37501 ) { *DECI=0; *CSEL=0; *RSEL=0; }
    else if( sr>=23001 ) { *DECI=0; *CSEL=0; *RSEL=1; }
    else if( sr>=18751 ) { *DECI=1; *CSEL=0; *RSEL=0; }
    else if( sr>=11501 ) { *DECI=1; *CSEL=0; *RSEL=1; }
    else if( sr>= 9376 ) { *DECI=2; *CSEL=0; *RSEL=0; }
    else if( sr>= 5751 ) { *DECI=2; *CSEL=0; *RSEL=1; }
    else if( sr>= 4688 ) { *DECI=3; *CSEL=0; *RSEL=0; }
    else if( sr>= 2875 ) { *DECI=3; *CSEL=0; *RSEL=1; }
    else if( sr>= 2344 ) { *DECI=4; *CSEL=0; *RSEL=0; }
    else if( sr>= 1438 ) { *DECI=4; *CSEL=0; *RSEL=1; }
    else if( sr>= 1172 ) { *DECI=5; *CSEL=0; *RSEL=0; }
    else if( sr>=  719 ) { *DECI=5; *CSEL=0; *RSEL=1; }

    *clkmst = 92000;

}

/*----------------------------------------------------------------------------
Name:        MB86667_S_Swp_Info_Get
-------------------------------------------------------------------------------*/
void MB86667_S_Swp_Info_Get( int32_t fOSC_start, int32_t smrt, int32_t clkmst, int32_t v,
                             int32_t RF_freq_step, int32_t swp_ofs,
                             int32_t *fOSC_10, int32_t *afcex_freq,
                             uint8_t *AFCEX_L, uint8_t *AFCEX_H )
{
    int32_t AFCEX;
    int32_t crnt_swp_freq;

    crnt_swp_freq = fOSC_start*1000 + v*swp_ofs;

    if(      RF_freq_step==2000 ) *fOSC_10 = (crnt_swp_freq  +1000)/2000*20;
    else if( RF_freq_step==1000 ) *fOSC_10 = (crnt_swp_freq  + 500)/1000*10;
    else if( RF_freq_step== 500 ) *fOSC_10 = (crnt_swp_freq*2+ 500)/1000*10/2;

    *afcex_freq = crnt_swp_freq - *fOSC_10*100;

    if( TUNER_POLARITY==1 ) AFCEX =  *afcex_freq*8192/clkmst;
    else                    AFCEX = -*afcex_freq*8192/clkmst;
    *AFCEX_L =  AFCEX&0x00ff;
    *AFCEX_H = (AFCEX&0x0f00)>>8;
}


/*----------------------------------------------------------------------------
Name:        MB86667_S_Swp_Info_Get2
-------------------------------------------------------------------------------*/
void MB86667_S_Swp_Info_Get2( int32_t smrt, int32_t RF_freq_step, int32_t swp_freq, int32_t clkmst, int32_t *afcex_freq,
                    int32_t *fOSC_10, uint8_t *AFCEX_L, uint8_t *AFCEX_H )
{
    int32_t AFCEX;

    if(      RF_freq_step==2000 ) *fOSC_10 = (swp_freq  +1000)/2000*20;
    else if( RF_freq_step==1000 ) *fOSC_10 = (swp_freq  + 500)/1000*10;
    else if( RF_freq_step== 500 ) *fOSC_10 = (swp_freq*2+ 500)/1000*10/2;

    *afcex_freq = swp_freq - *fOSC_10*100;

    if( TUNER_POLARITY==1 ) AFCEX =  *afcex_freq*8192/clkmst;
    else                    AFCEX = -*afcex_freq*8192/clkmst;
    *AFCEX_L =  AFCEX&0x00ff;
    *AFCEX_H = (AFCEX&0x0f00)>>8;

}
#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Scan.c,v $
Revision 1.2  2008/07/22 09:03:00  sgehre
fix feNotifyFct prototype to enable compilation with Metaware

Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.2  2008/02/24 21:11:52  kraft
version Fuj 4.3 implemented

Revision 1.3  2007/12/17 15:12:17  skau
Blind scan improved.

Revision 1.2  2007/10/12 08:55:08  skau
FM_PrStr replaced by SDEBUG().

Revision 1.1  2007/10/10 13:10:46  skau
Supports blind scan now.

Revision 1.1  2007/09/12 15:11:02  skau
Driver for MB86667 BlindScan.This is the first (non-working) version.


*****************************************************************************/

