/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_fe_tools.c
**
** \brief     Front-end device driver for Fujitsu MB86668 cable frontend.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_fe_tools.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "../src/fe_priv.h"

#include "mb86668_fe_types.h"
#include "mb86668_fe_tools.h"

#define FeOk 0

/*-----------------------------------------------------------------------------
   function:    int32_t TOOL_resetFe(uint32_t resetIndex)
   description: does a reset cycle to FE device
   input:       index for ghw driver
   output:      nothing
   return:      FeErr
-----------------------------------------------------------------------------*/
int32_t TOOL_resetFe(uint32_t resetIndex)
{
   int32_t error;

   if(resetIndex != 0)
   {
      error = FE_ControlFeReset(resetIndex, FE_RESET_PULSE);
      if (error != FE_OK)
         return error;
      FE_Wait(50);
   }
   return FE_OK;
}

/*-----------------------------------------------------------------------------
   function:    FeErr TOOL_i2cWrite(uint8_t i2c_address,uint8_t *data, uint32_t length)
   description: i2c write access
   input:       uint8_t i2c_address,uint8_t *data, uint32_t length
   output:      nothing
   return:      FeErr
-----------------------------------------------------------------------------*/
FeErr TOOL_i2cWrite(uint8_t i2c_address,uint8_t *data, uint32_t length)
{
   if(FE_I2C_Write(feHandleArray[0].i2cId,i2c_address,data,length)!=FE_OK)
   {
      return -2;
   }
   return FeOk;
}

/*-----------------------------------------------------------------------------
   function:    FeErr TOOL_i2cRead(uint8_t i2c_address,uint8_t *data, uint32_t length)
   description: i2c read access
   input:       uint8_t i2c_address, uint32_t length
   output:      uint8_t *data
   return:      FeErr
-----------------------------------------------------------------------------*/
FeErr TOOL_i2cRead(uint8_t i2c_address,uint8_t *data, uint32_t length)
{
   uint8_t tmp;

   if(FE_I2C_Read(feHandleArray[0].i2cId,i2c_address,&tmp,0,data,length)!=FE_OK)
   {
      return -2;
   }
   return FeOk;
}

/* Version History */
/******************************************************************************

$Log: mb86668_fe_tools.c,v $
Revision 1.2  2008/07/09 15:08:38  sgehre
- set I2C to hardware/400kbps
- fix some MISRA warnings

Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.


******************************************************************************/

