/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_tuner_c.h
**
** \brief     cable tuner driver functions for Philips.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_tuner_c.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

#ifndef _FD_MB86668_TUNER_C_H_
#define _FD_MB86668_TUNER_C_H_

#define USED_IF_FREQUENCY 36125 // in kHz

#define BW7MHZ 7000       // in kHz
#define BW8MHZ 8000       // in kHz

#define STEPWIDTH62500  0x06

#define AGC_HIGH 0   (0<<3)  // recommended for digital use
#define AGC_MEDIUM_2 (2<<3)  // recommended for digital use
#define AGC_MEDIUM_1 (1<<3)
#define AGC_LOW      (3<<3)

// accessable tuner types
typedef enum
{
   NONE_TUN=0,
   CD1316L_3,
   CD1600L_3,
   DCT70602,
   DCT70701,
} TUN_types;

typedef struct
{
   TUN_types Type;
   uint8_t I2cAddress;
   uint8_t WrSize;
   uint8_t WrBuffer[8];
   uint8_t RdSize;
   uint8_t RdBuffer[8];

// tuner private data
   int32_t StepSize;
   int32_t IF;
   int32_t I_Q;
   int32_t BandWidth;
   int16_t IfAgcLimiterLow;
   int16_t IfAgcLimiterHigh;
   int16_t RfAgcLimiterLow;
   int16_t RfAgcLimiterHigh;
   uint16_t IfAgcRef;
} TUN_device;

// error coding int tuner
#define TUN_ERR_BASE -1000

typedef enum
{
  TUN_I2C_ERR         =  TUN_ERR_BASE -1,
  TUN_WRONG_PARAMETER =  TUN_ERR_BASE -2,
  TUN_NOT_LOCKED      =  TUN_ERR_BASE -3,
  TUN_NOT_FOUND       =  TUN_ERR_BASE -4,
} TUN_err;


#endif // _FD_MB86668_TUNER_C_H_
