/****************************************************************************
*
*  Front-end device driver for Fujitsu MB86A21 ISDB-T frontend.
*
*  Copyright (c) 2009 Fujitsu Microelectronics 
*  All rights reserved. 
*
*  Created 02/02/2009
*
****************************************************************************/
#include "../src/fe_priv.h"
#include "MB86A21_API_defs.h"
#include "MB86A21_user_define.h"
#ifndef _LINUX_FAPI
#include <fapi/drv_timer.h>
#include <fapi/sys_services.h>
#endif
/****************************************************************************
*   MB86A21 Timer function (Get GetTickPeriod)
*   Unit = 1
****************************************************************************/
UINT32 MB86A2x_GetTickPeriod()
{
#ifndef _LINUX_FAPI
    return FAPI_TIMER_RESOLUTION_1_MSEC/1000;
#else
    return FAPEX_TIMER_RESOLUTION_1_MSEC/1000;	
#endif
}

/****************************************************************************
*   MB86A21 Timer function (Get Tick)
*   Unit = ms
****************************************************************************/
UINT32 MB86A2x_GetTickTime()
{
#ifndef _LINUX_FAPI
    return FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC);
#else
    return FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
#endif
}

/****************************************************************************
*   MB86A21 Timer function
*   Unit = ms
****************************************************************************/
UINT32 MB86A2x_wait( SINT32 wait_time )
{
    FAPI_SYS_SLEEP(wait_time);
    return 1;
}

/****************************************************************************
*   MB86A21 I2C Write Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A2x_write( UINT8 OFDM_adr, UINT8 address , UINT8 value )
{
    feHandleT *hp;
    uint8_t regBuffer[2], index = 0;
    int32_t ferr;
    
    regBuffer[0] = (uint8_t)(address);
    regBuffer[1] = (uint8_t)(value & 0xff);
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( OFDM_adr == feHandleArray[index].i2cAddrDemod )
            break;
    }
    
    hp = &feHandleArray[index];
	#ifndef _LINUX_FAPI
    ferr = FE_I2C_Write( hp->i2cHandle, OFDM_adr, &regBuffer[0], 2 );
    #else
	ferr = FE_I2C_Write( hp->i2cId, OFDM_adr, &regBuffer[0], 2 );
	#endif
    if(ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Write Demod I2C addr:0x%02x sub-address:0x%02x value:0x%02x error:0x%02x\n", OFDM_adr, address, value, ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   MB86A21 I2C Read Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A2x_read( UINT8 OFDM_adr, UINT8 address , UINT8 *value )
{
    feHandleT *hp;
    uint8_t index = 0, temp;
    int32_t ferr;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( OFDM_adr == feHandleArray[index].i2cAddrDemod )
            break;
    }
    
    hp = &feHandleArray[index];
    temp = (uint8_t)(address&0xff);
	#ifndef _LINUX_FAPI
    ferr = FE_I2C_Read( hp->i2cHandle, OFDM_adr, &temp, 1, value, 1 );
    #else
	ferr = FE_I2C_Read( hp->i2cId, OFDM_adr, &temp, 1, value, 1 );
	#endif
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Read Demod I2C addr:0x%02x sub-address:0x%02x value:0x%02x error:0x%02x\n", OFDM_adr, address, value, ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   Tuner Write Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A2x_tuner_write( UINT8 Tu_adr, UINT8 bytes , UINT8 *value )
{
    int32_t ferr = 0;
    feHandleT *hp;
    uint8_t index = 0;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( Tu_adr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    
    hp = &feHandleArray[index];
    if( MB86A2x_write( hp->i2cAddrDemod, 0xFE, 0x00 ) == I2C_OK ) // I2C Repeater
    {
		#ifndef _LINUX_FAPI
        ferr = FE_I2C_Write( hp->i2cHandle, Tu_adr, value, bytes );
    	#else
        ferr = FE_I2C_Write( hp->i2cId, Tu_adr, value, bytes );	
		#endif
    }
    else
        return I2C_NG;
    
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Write Tuner I2C error addr:0x%02x sub-address:0x%02x error:0x%02x\n", Tu_adr, value[0], ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   Tuner Read Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A2x_tuner_read( UINT8 Tu_adr, UINT8 bytes , UINT8 *value )
{
    int32_t ferr = 0;
    feHandleT *hp;
    uint8_t index = 0;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( Tu_adr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    
    hp = &feHandleArray[index];
    if( MB86A2x_write( hp->i2cAddrDemod, 0xFE, 0x00 ) == I2C_OK ) // I2C Repeater
    {
		#ifndef _LINUX_FAPI
        ferr = FE_I2C_Read( hp->i2cHandle, Tu_adr+1, &value[0], 1, &value[1], 1 );
   		#else
		ferr = FE_I2C_Read( hp->i2cId, Tu_adr+1, &value[0], 1, &value[1], 1 );
		#endif
    }
    else
        return I2C_NG;
    
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Read Tuner I2C error addr:0x%02x sub-address:0x%02x error:0x%02x\n", Tu_adr, value[0], ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

SINT32 MB86A2x_silicon_tuner_write_in_dword( UINT32 tuner_w_adr, UINT32 start_adr, UINT32 write_num, UINT32 *write_data)
{
    UINT8 i;
    UINT8 tu_w_data[40] ; // TDA18211/18271 has 39-bytes register map while MAX3580 has 16-bytes
    int32_t err = 0;
    UINT8   RF_adr ;

    tu_w_data[0] = start_adr ;
    RF_adr = (UINT8)(0xFF&tuner_w_adr) ;
    
    for( i=1 ; i<=write_num ; i++ ) tu_w_data[i] = *(write_data+i-1) ;
    if( MB86A2x_tuner_write( RF_adr, write_num+1, tu_w_data ) != I2C_OK ) err=1 ;

    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 MB86A2x_silicon_tuner_write_in_bytes( UINT32 tuner_w_adr, UINT32 start_adr, UINT32 write_num, UINT8 *write_data)
{
    UINT8 i;
    UINT8 tu_w_data[40] ; // TDA18211/18271 has 39-bytes register map while MAX3580 has 16-bytes
    int32_t err = 0;
    UINT8   RF_adr ;

    tu_w_data[0] = start_adr ;
    RF_adr = (UINT8)(0xFF&tuner_w_adr) ;
    
    for( i=1 ; i<=write_num ; i++ ) tu_w_data[i] = write_data[i-1];
    if( MB86A2x_tuner_write( RF_adr, write_num+1, tu_w_data ) != I2C_OK ) err=1 ;

    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 MB86A2x_silicon_tuner_read( unsigned int tuner_r_adr, unsigned int start_adr, unsigned int read_num, unsigned int *read_data) 
{
    int err = 0;
    unsigned char ReadBuffer[40] = {0};
    unsigned int  uCounter;
    unsigned int*   pReadBuffer32;
    uint8_t tunerAddr,regBuffer;
    uint8_t index = 0;
    
    feHandleT *hp;
    tunerAddr = (uint8_t)(tuner_r_adr&0xff);
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( tunerAddr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    hp = &feHandleArray[index];
    
    if( MB86A2x_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    regBuffer = (uint8_t)(start_adr&0xFF);
	#ifndef _LINUX_FAPI
    err = FE_I2C_Write(hp->i2cHandle, tunerAddr,&regBuffer,1);
    #else
	err = FE_I2C_Write(hp->i2cId, tunerAddr,&regBuffer,1);
	#endif
    if( MB86A2x_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    #ifndef _LINUX_FAPI
    err = FE_I2C_Read(hp->i2cHandle, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
    #else
	err = FE_I2C_Read(hp->i2cId, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
	#endif
    // copy from ReadBuffer32 -> read_data
    pReadBuffer32 = read_data;
    for (uCounter = 0; uCounter < read_num; uCounter++)
    {
        *pReadBuffer32 = (unsigned int)ReadBuffer[uCounter];
        pReadBuffer32 ++;
    }
        
    if( err==0 ) return 1 ;
    else         return 0 ;
}
