/*

 Driver APIs for MxL135RF Tuner
 
 Copyright, Maxlinear, Inc.
 All Rights Reserved
 
 File Name:      MxL135RF_API.c
 
 Version:    1.1.0
*/


//#include "StdAfx.h"
#include "MxL135RF_API.h"
#include "MxL_User_Define.h"



//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//																		   //
//							Tuner Functions								   //
//																		   //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
MxL_ERR_MSG MxL_Set_Register(MxL135RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char RegData)
{
	unsigned int Status=0;
	unsigned char pArray[2];
	pArray[0] = RegAddr;
	pArray[1] = RegData;
	Status = MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2);
	if(Status) return MxL_ERR_SET_REG;

	return MxL_OK;

}

MxL_ERR_MSG MxL_Get_Register(MxL135RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char *RegData)
{
	if(MxL_I2C_Read((unsigned char)myTuner->I2C_Addr, RegAddr, RegData))
		return MxL_ERR_GET_REG;
	return MxL_OK;

}

MxL_ERR_MSG MxL_Soft_Reset(MxL135RF_TunerConfigS* myTuner)
{
	unsigned char reg_reset;
	reg_reset = 0xFF;
	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, &reg_reset, 1))
		return MxL_ERR_OTHERS;

	return MxL_OK;
}

MxL_ERR_MSG MxL_Loop_Through_On(MxL135RF_TunerConfigS* myTuner, MxL135RF_LoopThru isOn)
{	
	unsigned char pArray[2];	// a array pointer that store the addr and data pairs for I2C write	
	
	pArray[0]=0x04;
	if(isOn)
	 pArray[1]= 0x01;
	else
	 pArray[1]= 0x0;

	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
		return MxL_ERR_OTHERS;

	return MxL_OK;
}

MxL_ERR_MSG MxL_Stand_By(MxL135RF_TunerConfigS* myTuner)
{
	unsigned char pArray[4];	// a array pointer that store the addr and data pairs for I2C write	
	
	pArray[0] = 0x01;
	pArray[1] = 0x0;
	pArray[2] = 0x0F;
	pArray[3] = 0x0;

	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 4))
		return MxL_ERR_OTHERS;

	return MxL_OK;
}

MxL_ERR_MSG MxL_Wake_Up(MxL135RF_TunerConfigS* myTuner)
{
	unsigned char pArray[2];	// a array pointer that store the addr and data pairs for I2C write	
	
	pArray[0] = 0x01;
	pArray[1] = 0x01;

	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
		return MxL_ERR_OTHERS;

	if(MxL_Tuner_RFTune(myTuner, myTuner->RF_Freq_Hz, myTuner->BW_MHz))
		return MxL_ERR_RFTUNE;

	return MxL_OK;
}

MxL_ERR_MSG MxL_Tuner_Init(MxL135RF_TunerConfigS* myTuner)
{	
	unsigned char pArray[MAX_ARRAY_SIZE];	// a array pointer that store the addr and data pairs for I2C write
	unsigned int Array_Size;							// a integer pointer that store the number of element in above array

	//Soft reset tuner
	if(MxL_Soft_Reset(myTuner))
		return MxL_ERR_INIT;

	//perform initialization calculation
	if(MxL135RF_Init(pArray, &Array_Size, (unsigned char)myTuner->Mode, (unsigned int)myTuner->Xtal_Freq, (unsigned int)myTuner->IF_Freq, 
		(unsigned char)myTuner->IF_Spectrum, (unsigned char)myTuner->ClkOut_Setting, (unsigned char)myTuner->ClkOut_Amp))
	return MxL_ERR_INIT;

	//perform I2C write here
	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, Array_Size))
		return MxL_ERR_INIT;

	return MxL_OK;
}


MxL_ERR_MSG MxL_Tuner_RFTune(MxL135RF_TunerConfigS* myTuner, unsigned int RF_Freq_Hz, MxL135RF_BW_MHz BWMHz)
{
	unsigned char pArray[MAX_ARRAY_SIZE];	// a array pointer that store the addr and data pairs for I2C write
	unsigned int Array_Size;							// a integer pointer that store the number of element in above array

	//Store information into struc
	myTuner->RF_Freq_Hz = RF_Freq_Hz;
	myTuner->BW_MHz = BWMHz;

	//perform Channel Change calculation
	if(MxL135RF_RFTune(pArray,&Array_Size,RF_Freq_Hz,BWMHz))
		return MxL_ERR_RFTUNE;

	//perform I2C write here
	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, Array_Size))
		return MxL_ERR_RFTUNE;

	//wait for 3ms
	MxL_Delay(3); 

	return MxL_OK;
}

MxL135RF_ChipVersion MxL_Check_ChipVersion(MxL135RF_TunerConfigS* myTuner)
{	
	unsigned char Data;
	MxL135RF_ChipVersion version;
	
	if(MxL_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD9, &Data))
		return MxL_GET_ID_FAIL;
	switch(Data)
	{
//		case 0x14:
		case 0x0: 
			version = MxL_135RF_ID; 
			break;
		default: 
			version = MxL_UNKNOWN_ID;
	}

	return version;
}

MxL_ERR_MSG MxL_RFSynth_Lock_Status(MxL135RF_TunerConfigS* myTuner, BOOL* isLock)
{	
	unsigned char Data;
	*isLock = FALSE; 
	if(MxL_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD8, &Data))
		return MxL_ERR_OTHERS;
	Data &= 0x0C;
	if (Data == 0x0C)
		*isLock = TRUE;  //RF Synthesizer is Lock	
	return MxL_OK;
}

MxL_ERR_MSG MxL_REFSynth_Lock_Status(MxL135RF_TunerConfigS* myTuner, BOOL* isLock)
{
	unsigned char Data;
	*isLock = FALSE; 
	if(MxL_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD8, &Data))
		return MxL_ERR_OTHERS;
	Data &= 0x03;
	if (Data == 0x03)
		*isLock = TRUE;   //REF Synthesizer is Lock
	return MxL_OK;
}

MxL_ERR_MSG MxL_Set_IF_Output_Gain(MxL135RF_TunerConfigS* myTuner, unsigned int IF_Gain)
{
    unsigned char pArray[2];	// a array pointer that store the addr and data pairs for I2C write	
	unsigned char tempData;
	
	if(IF_Gain < 56 || IF_Gain > 63)
		return MxL_ERR_OTHERS;

	if(MxL_I2C_Read((unsigned char)myTuner->I2C_Addr, 0x00, &tempData))
		return MxL_ERR_OTHERS;

	pArray[0]=0x00;
	pArray[1]= ((unsigned char)IF_Gain | (0xE0 & tempData));
	
	if(MxL_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
		return MxL_ERR_OTHERS;

	return MxL_OK;
}

