/*******************************************************************************
 *
 * FILE NAME          : MxL_Common.h
 * 
 * AUTHOR             : Kyle Huang
 * DATE CREATED       : May 05, 2008
 *
 * DESCRIPTION        : 
 *
 *******************************************************************************
 *                Copyright (c) 2006, MaxLinear, Inc.
 ******************************************************************************/

#ifndef __MXL135RF_COMMON_H
#define __MXL135RF_COMMON_H



#include "MxL_User_Define.h"



/******************************/
/*	MxL135RF Err message  	  */
/******************************/
typedef enum{
	MxL_OK				=   0,
	MxL_ERR_INIT		=   1,
	MxL_ERR_RFTUNE		=   2,
	MxL_ERR_SET_REG		=   3,
	MxL_ERR_GET_REG		=	4,
	MxL_ERR_OTHERS		=   10
}MxL_ERR_MSG;

/******************************/
/*	MxL135RF Chip verstion     */
/******************************/
typedef enum{
	MxL_UNKNOWN_ID		= 0x00,
	MxL_135RF_ID		= 0x14,
	MxL_GET_ID_FAIL		= 0xFF
}MxL135RF_ChipVersion;


/******************************************************************************
    CONSTANTS
******************************************************************************/

#ifndef MHz
	#define MHz 1000000
#endif

#define MAX_ARRAY_SIZE 100


// Enumeration of Mode
// Enumeration of Mode
typedef enum 
{
	MxL_MODE_ISDBT = 0
} MxL135RF_Mode ;

typedef enum
{
	MxL_IF_4_MHZ	  = 4000000,
	MxL_IF_4_5_MHZ	  = 4500000,
	MxL_IF_4_57_MHZ	  =	4570000,
	MxL_IF_5_MHZ	  =	5000000,
	MxL_IF_5_38_MHZ	  =	5380000,
	MxL_IF_6_MHZ	  =	6000000,
	MxL_IF_6_28_MHZ	  =	6280000,
	MxL_IF_9_1915_MHZ =	9191500,
	MxL_IF_35_25_MHZ  = 35250000,
	MxL_IF_36_15_MHZ  = 36150000,
	MxL_IF_44_MHZ	  = 44000000
} MxL135RF_IF_Freq ;

typedef enum
{
	MxL_XTAL_16_MHZ		= 16000000,
	MxL_XTAL_20_MHZ		= 20000000,
	MxL_XTAL_20_25_MHZ	= 20250000,
	MxL_XTAL_20_48_MHZ	= 20480000,
	MxL_XTAL_24_MHZ		= 24000000,
	MxL_XTAL_25_MHZ		= 25000000,
	MxL_XTAL_25_14_MHZ	= 25140000,
	MxL_XTAL_27_MHZ		= 27000000,
	MxL_XTAL_28_8_MHZ	= 28800000,
	MxL_XTAL_32_MHZ		= 32000000,
	MxL_XTAL_40_MHZ		= 40000000,
	MxL_XTAL_44_MHZ		= 44000000,
	MxL_XTAL_48_MHZ		= 48000000,
	MxL_XTAL_49_3811_MHZ = 49381100	
} MxL135RF_Xtal_Freq ;

typedef enum
{
	MxL_BW_6MHz = 6
} MxL135RF_BW_MHz;

typedef enum
{
	MxL_NORMAL_IF = 0 ,
	MxL_INVERT_IF

} MxL135RF_IF_Spectrum ;

typedef enum
{
	MxL_LT_DISABLE = 0 ,
	MxL_LT_ENABLE

} MxL135RF_LoopThru ;

typedef enum
{
	MxL_CLKOUT_DISABLE = 0 ,
	MxL_CLKOUT_ENABLE

} MxL135RF_ClkOut;

typedef enum
{
	MxL_CLKOUT_AMP_0 = 0 ,
	MxL_CLKOUT_AMP_1,
	MxL_CLKOUT_AMP_2,
	MxL_CLKOUT_AMP_3,
	MxL_CLKOUT_AMP_4,
	MxL_CLKOUT_AMP_5,
	MxL_CLKOUT_AMP_6,
	MxL_CLKOUT_AMP_7
} MxL135RF_ClkOut_Amp;

typedef enum
{
	MxL_I2C_ADDR_96 = 96 ,
	MxL_I2C_ADDR_97 = 97 ,
	MxL_I2C_ADDR_98 = 98 ,
	MxL_I2C_ADDR_99 = 99 	
} MxL135RF_I2CAddr ;

//
// MxL135RF TunerConfig Struct
//
typedef struct _MxL135RF_TunerConfigS
{
	MxL135RF_I2CAddr	I2C_Addr;
	MxL135RF_Mode		Mode;
	MxL135RF_Xtal_Freq	Xtal_Freq;
	MxL135RF_IF_Freq	    IF_Freq;
	MxL135RF_IF_Spectrum IF_Spectrum;
	MxL135RF_ClkOut		ClkOut_Setting;
    MxL135RF_ClkOut_Amp	ClkOut_Amp;
	MxL135RF_BW_MHz		BW_MHz;
	unsigned int				RF_Freq_Hz;
} MxL135RF_TunerConfigS;




#endif /* __MXL135RF_COMMON_H__*/

