/* -------------------------------------------------------------------------
Max3580 Example programming code

Copyright (C) 2006 Maxim Integrated Products
Maxim Integrated Products Confidential

File Name: Max3580Driver.c
    Date: 1/30/06
 Version: 1.1
 Comment: Partially rewritten for better readability
----------------------------------------------------------------------------
File Name: Max3580Driver.c
    Date: 4/11/06
 Version: 1.0.1
 Comment: Updated Tracking Filter ROM Table redefinitions in Max3580_read_rom
----------------------------------------------------------------------------
   Date: 9/20/06
 Version: 1.0.3
 Comment: Fixed a bug in max3580_CalcTF that would only show up upon initialization.
New   Line 228: TFP_Caps = (TFP_Caps & 0xF0 )  | tf_sht ;
New   Line 230: Baseband = ( Baseband & 0xF ) | (bbfilt<<4);
---------------------------------------------------------------------------- */

#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "mxmdef.h"
#include "MB86A21_API_defs.h"
#include "Tuner.h"
#include "MB86A21_API_structs.h"
//#define WG27DLL_EXPORTS
#include "Max3580Driver.h"
//#include <fapi/sys_services.h>


/* LPT_Functions.h Header file that relates to printer port driver.
  Replace by microprocessor specific I2C access routines */
//#include "LPT_Functions.h"

/* global variables, initialized by Max3580Init() code */
UINT8 MAX3580_ADDR;    /*I2C device address */
UINT8 registers[16];                          /* local copy of values to be written to registers */
UINT8 tf_ser_200, tf_par_200, tf_sht_200;     /* calibration constants for RF tracking filter */
UINT8 tf_ser_470, tf_par_470, tf_sht_470;
UINT8 tf_ser_860, tf_par_860, tf_sht_860;
UINT8 bb_filt8;	              /* calibration constant for baseband filter */

  /* Set Max3580 for receiving DVB-T on a given frequency.
     Changes PLL, VCO, RF & Baseband Registers, using
     calibration data stored in global variables. */


//wg27_API int __stdcall Max3580Init()
UINT8 Max3580Init()
{
	UINT8 ret;

    registers[MAX3580_NDIV]         = 0x2E;
    registers[MAX3580_FRAC2]        = 0x19;
    registers[MAX3580_FRAC1]        = 0xFA;
    registers[MAX3580_FRAC0]        = 0xBC;

    registers[MAX3580_TRK_FILT2]    = 0x60;
    registers[MAX3580_PLL]          = 0x0A;
    registers[MAX3580_TEST]         = 0x08;
    registers[MAX3580_SHUTDOWN]     = 0x00;
    registers[MAX3580_VCO]          = 0xc4;
    registers[MAX3580_BASEBAND]     = 0xC0;
    registers[MAX3580_DCOFFSET]     = 0x74;
    registers[MAX3580_DCOFFSET_DAC] = 0x00; // DcOffsetDAC register
    registers[MAX3580_ROM_ADDR]     = 0x00; // ROM_Address register. Set to zero at all times for normal operation
    registers[0x0e]                 = 0x00;
    registers[0x0f]                 = 0x00;

    max3580_set_pll(474.0,20.0);
    //max3580_set_pll(474.0,16.0);

    // First read calibration constants from Max3580 and store in global
    //   variables for use when setting RF tracking filter 
    ret = max3580_read_rom();
	if (ret == 0) return(0);

    max3580_CalcTF(474.0);

    //  Maxim evkit I2c communication... Replace by microprocessor specific code 
    I2C_IWriteBlock(MAX3580_ADDR,0,registers,16);
	//I2C_IWriteBlock(MAX3580_ADDR,5,registers,16);

    return(1);
}



//wg27_API int __stdcall Max3580SetFrequency(double frequency, double xtalref,
//						double bbfilter)
UINT8 Max3580SetFrequency(double frequency, double xtalref/*, double bbfilter*/)

{ 
  UINT8 ret;
  /* all frequencies in MHz */
  /* set up PLL divider ratios */
  max3580_set_pll(frequency,xtalref);
  /* Program tracking filters and other frequency dependent registers */
  
  max3580_CalcTF(frequency);
  /*  Maxim evkit I2c communication... Replace by microprocessor specific code */
  ret = I2C_IWriteBlock(MAX3580_ADDR,MAX3580_NDIV,registers,6); /* Only need to send first 6 registers */
  if (ret == 0) return(0);
  
  /* If operation with different baseband filter bandwidth is desired, choose this
     one now. Examples suggest 7MHz wide VHF and 8.0 MHz wide UHF channels. */
  /*
  if (frequency > 300.0)
    max3580_set_bb_filter(4); // request filter cutoff frequency set to 4MHz
  else
    max3580_set_bb_filter(3.5);
  */
  max3580_set_bb_filter(3);

  ret = I2C_IWriteBlock(MAX3580_ADDR,MAX3580_BASEBAND,registers,1);
  if (ret == 0) return(0);
  
  return(1);
}

void max3580_set_pll(double freq, double xtal)
{ /* set Max3580 PLL divider ratio's for a given RF frequency and
     given XTal frequency, both in MHz */
  SINT32 div_b, vco_div;
  UINT8 a,b,c,d;

  //div_b = (int) (pow(2,20)*(freq / xtal));
  div_b = (int) (pow((double)2,20)*(freq / xtal));
  a = (unsigned char) (div_b >> 20);
  b = (unsigned char) (div_b >> 16);
  c = (unsigned char) (div_b >> 8);
  d = (unsigned char) div_b;

  
  registers[MAX3580_NDIV]= a;
  registers[MAX3580_FRAC2] = ( registers[MAX3580_FRAC2] & 0xf0 ) | ( b & 0x0F);
  registers[MAX3580_FRAC1] = c;
  registers[MAX3580_FRAC0] = d;

  if (freq > MAX3580_MINVCO)
    vco_div = 0x00;                  /* dont divide at all */
  else if (freq > MAX3580_MINVCO/2)
    vco_div = 0x40;                  /* div-by-2 */
  else if (freq > MAX3580_MINVCO/4)
    vco_div = 0x80;                  /* div-by-4 */
  else
    vco_div = 0xC0;                  /* div-by-8, but never used since RF frequency always >173 MHz */
  registers[MAX3580_TRK_FILT] = vco_div | (registers[MAX3580_TRK_FILT] & 0x0F);
}

UINT8 max3580_read_rom()
{ /* Read out Max3580 ROM table containing calibration constants
     and setup global variables with that data. */
  SINT32 i,b[7];
  UINT8 arr[1], ret; 

  for (i=1; i<=6; i++)
    { /*  Maxim evkit I2c communication... Replace by microprocessor specific code */
	  arr[0] = i;
      ret = I2C_IWriteBlock2(MAX3580_ADDR,MAX3580_ROM_ADDR,arr,1);     /* Set index for ROM entry to read */
      if (ret == 0)
      {
	      printf("Failed at Write No %d\n", i);
	      return(0); 
      }
	  ret = I2C_IReadBlock(MAX3580_ADDR,MAX3580_ROMD_READ,&b[i],1);  /* Read back from table. */
      if (ret == 0)
      {
	      printf("Failed at Read No %d\n", i);
	      return(0); 
      }
    }
  arr[0] = 0;
  ret = I2C_IWriteBlock2(MAX3580_ADDR,MAX3580_ROM_ADDR,arr,1);   /* Reset Index in ROM in table to 0 */
      if (ret == 0)
      {
	      printf("Failed \n");
	      return(0); 
      }
  
  /*  changed 7/9/06  to new ROM Table */
  tf_ser_200 = b[1] & 0xF;
  tf_sht_200 = b[2] & 0xF;
  tf_par_200 = b[1] >> 4;
  tf_ser_470 = b[3] & 0xF;
  tf_sht_470 = b[4] & 0xF;
  tf_par_470 = b[3] >> 4;
  tf_ser_860 = b[5] & 0xF;
  tf_sht_860 = b[4] >> 4;
  tf_par_860 = b[5] >> 4;
  bb_filt8 = b[6] >> 4;

  return(1);
}

void max3580_CalcTF( double freq )
{  /*  Calculate RF tracking filter register for a given receive frequency in MHz */
  UINT8 tf_ser, tf_par, tf_sht;
//  unsigned char bb_filt;

  /* RF tracking filter. Note different interpolation algorithms
     used for VHF III vs. UHF band. No range checking done. */

  if (freq < 300)
  { /* use calibration constants and substract 1 count per 10 MHz */
    tf_ser  = (unsigned char) (tf_ser_200 - (freq-200.0)/10.0);
    tf_sht = (unsigned char) (tf_sht_200 - (freq-200.0)/10.0);
    tf_par  = (unsigned char) (tf_par_200 - (freq-200.0)/10.0);
  }
  else
  { /* plain linear interpolation between 470 and 860 MHz,
       correctly round to nearest integer */
    tf_ser = (unsigned char) (0.5+tf_ser_470+(freq-470.0)/390.0*(tf_ser_860-tf_ser_470));
    tf_sht = (unsigned char) (0.5+tf_sht_470+(freq-470.0)/390.0*(tf_sht_860-tf_sht_470));
    tf_par = (unsigned char) (0.5+tf_par_470+(freq-470.0)/390.0*(tf_par_860-tf_par_470));
  }

  /* just in case, limit to 4 bits to prevent rounding problems near 470 MHz */
  if (tf_ser > 15) tf_ser = 15;
  if (tf_sht > 15) tf_sht = 15;
  if (tf_par > 15) tf_par = 15;
  registers[MAX3580_TRK_FILT] = ( tf_par << 4 ) | tf_ser ;
  registers[MAX3580_TRK_FILT2] = ( registers[MAX3580_TRK_FILT2] & 0xF0 )  | tf_sht ;
}

void max3580_set_bb_filter(double frequency)
{ /* Set baseband filter cutoff frequency in MHz */

  SINT32 tf_filter;

  if (frequency < 3.14)
    tf_filter=bb_filt8-6;
  else if (frequency < 3.25)
    tf_filter=bb_filt8-5;
  else if (frequency < 3.37)
    tf_filter=bb_filt8-4;
  else if (frequency < 3.50)
    tf_filter=bb_filt8-3;
  else if (frequency < 3.65)
    tf_filter=bb_filt8-2;
  else if (frequency < 3.78)
    tf_filter=bb_filt8-1;
  else if (frequency < 4.0)
    tf_filter=bb_filt8;
  else if (frequency < 4.17)
    tf_filter=bb_filt8+1;
  else
    tf_filter=bb_filt8+2;

  /* limit offset in case user requested unsupported wide or narrow setting */
  if (tf_filter <0) tf_filter=0;
  if (tf_filter >15) tf_filter=15;
  registers[MAX3580_BASEBAND]= (registers[MAX3580_BASEBAND] & 0x0F) | (tf_filter<<4);

}

void max3580_set_device_adr(unsigned char adr)
{
  MAX3580_ADDR = adr;
}
