/* -------------------------------------------------------------------------
Max3580 Example programming code

Copyright (C) 2006 Maxim Integrated Products
Maxim Integrated Products Confidential

File Name: Max3580Driver.h
    Date: 1/30/06
 Version: 1.1
 Comment: Partially rewritten for better readability
----------------------------------------------------------------------------
    Date: 4/11/06
 Version: 1.0.1
---------------------------------------------------------------------------- */

#ifndef MAX3540_Driver_H
#define MAX3540_Driver_H

/*
#ifdef WG27DLL_EXPORTS
    #define wg27_API __declspec(dllexport)
#else
    #define wg27_API __declspec(dllimport)
#endif
*/

/******* Max3580 constants. Do not change. *******/
//#define MAX3580_ADDR 0xC2      /*I2C device address */

#define MAX3580_NDIV 0         /*Register 0 - Ndivider*/
#define MAX3580_FRAC2 0x1      /*Register 1 - Highest byte of Fractional N Diveder*/
#define MAX3580_FRAC1 0x2      /*Register 2 - Med byte of Fractional N Divider*/
#define MAX3580_FRAC0 0x3      /*Register 3 - Lowest byte of Fractional N Divider*/
#define MAX3580_TRK_FILT 0x4   /*Register 4 - Tracking Filter  Caps*/
#define MAX3580_TRK_FILT2 0x5  /*Register 5 - Tracking Filter 2 Caps*/
#define MAX3580_PLL 0x6        /*Register 6 - Pll control  */
#define MAX3580_TEST 0x7       /*Register 7 - Test Functions*/
#define MAX3580_SHUTDOWN 0x8   /*Register 8 - Shutdown individual Functional Blocks*/
#define MAX3580_VCO 0x9        /*Register 9 - VCO Control */
#define MAX3580_BASEBAND 0xa   /*Register 10 - BaseBand Control */
#define MAX3580_DCOFFSET 0xb   /*Register 11 - DC Offset Control*/
#define MAX3580_DCOFFSET_DAC 0xc    /*Register 12 - DC Offset DAC*/
#define MAX3580_ROM_ADDR 0xd   /*Register 13 - Rom Address*/
#define MAX3580_ROMD_READ 0x10 /*Register 14- Read-Only Rom Data Read*/
#define MAX3580_STATUS  0x11   /*Register 15 - Read-Only Status */
#define MAX3580_AUTOTUNER  0x12   /* Register 16 - Read-Only Autotuner */

#define MAX3580_MINVCO 540    /* RF frequency in MHz below which the RF VCO
                                 needs to be divided down */

/******* Max3580 public functions *******/
/* Initialize Max3580 for DVB-H reception. Read out
calibration data ROM & setup all global variables
Also tunes the chip to 474MHz, but only if a 20 Mhz crystal is used */
//wg27_API int __stdcall Max3580Init();
UINT8 Max3580Init(void);
/* Recalculate & change all Max3580 register needed to tune
   to a given receive frequency. All inputs in MHz */
//wg27_API int __stdcall Max3580SetFrequency(double frequency, double xtalref,
//				double bbfilter);
UINT8 Max3580SetFrequency(double frequency, double xtalref/*, double bbfilter*/);
/* Other functions that calculate Max3580 register values. These ones are
   called by the above two toplevel functions */
UINT8 max3580_read_rom(void);
void max3580_CalcTF( double freq);
void max3580_set_pll(double freq, double xtal);
void max3580_set_vco_trackfilt(double freq);
void max3580_set_bb_filter(double freq);
void max3580_set_device_adr(unsigned char adr);

/******* Max3580 porting interface *******/
UINT32 I2C_IWriteBlock(UINT8 TuAddr, UINT8 StartAddr, UINT8* pArray, UINT32 count);
UINT32 I2C_IWriteBlock2(UINT8 TuAddr, UINT8 WriteAddr, UINT8* pArray, UINT32 count);
UINT32 I2C_IReadBlock(UINT8 TuAddr, UINT8 ReadAddr, SINT32* pArray, UINT32 count);

#endif
