/* Driver APIs for Max3580 Tuner */
#include "MB86A21_API_defs.h"
#include "MB86A21_user_define.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                                                                         //
//                  I2C Functions (implement by customer)                  //
//                                                                         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
/******************************************************************************
**
**  Name: I2C_IWriteBlock
**
**  Description:    I2C write operations
**
**  Parameters:     
**                  TuAddr      - Max3580 Device address
**                  StartAddr   - Write data array's start address
**                  pArray      - Write data array pointer
**                  count       - total number of array
**
**  Returns:        void
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   
**
******************************************************************************/
UINT32 I2C_IWriteBlock(UINT8 TuAddr, UINT8 StartAddr, UINT8* pArray, UINT32 count)
{
    UINT8 pArray_tmp[16], i;
    
    for(i = 0; i < count; i++)
        pArray_tmp[i] = pArray[StartAddr + i];

    return MB86A2x_silicon_tuner_write_in_bytes((UINT8)TuAddr, StartAddr, count, pArray_tmp) ;
}

/******************************************************************************
**
**  Name: I2C_IWriteBlock2
**
**  Description:    I2C write operations
**
**  Parameters:     
**                  TuAddr  - Max3580 Device address
**                  WriteAddr   - Write address
**                  pArray      - Write data array pointer
**                  count       - total number of array
**
**  Returns:        void
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   
**
******************************************************************************/
UINT32 I2C_IWriteBlock2(UINT8 TuAddr, UINT8 WriteAddr, UINT8* pArray, UINT32 count)
{
    return MB86A2x_silicon_tuner_write_in_bytes((UINT8)TuAddr, WriteAddr, count, pArray) ;
}

/******************************************************************************
**
**  Name: I2C_IReadBlock
**
**  Description:    I2C write operations
**
**  Parameters:     
**                  TuAddr  - Max3580 Device address
**                  ReadAddr    - Read address
**                  pArray      - Read data
**                  count       - total number of array
**
**  Returns:        void
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   
**
******************************************************************************/
UINT32 I2C_IReadBlock(UINT8 TuAddr, UINT8 ReadAddr, SINT32* pArray, UINT32 count)
{
    return MB86A2x_silicon_tuner_read((UINT8)(TuAddr), ReadAddr, count, (unsigned int*)pArray) ;
}
