/******************************************************************************
**
** \file      mb86a21.c
**
** \brief     Front-end device driver for Fujitsu MB86A21 ISDB-T frontend.
**
**            (C) FMC 2003 - 2010
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86a21.c,v 1.3 2009/11/17 10:33:25 PeterQ Exp $
**
******************************************************************************/
//#define DEBUG_FE  1


///added for gnuarm compiler, to be confirmed for RVDS
#include "../src/fe_priv.h"

#ifdef TMBSL_TDA18271

#include "TDA18271\tmtypes.h"
#include "TDA18271\SystemFuncURT.h"
#include "TDA18271\tmbsltda182i1local.h"
#include "TDA18271\tmbslTDA182I1StaticInstance.h"
#include "TDA18271\tmbslTuner.h"
#include "TDA18271\tmbslTDA18271.h"
#include "TDA18271\tmbslTDA182I1.c"

#include "MB86A21_TDA18271_default_setting.h"

#endif

#ifdef NI386X

#include "MB86A21_NI386X_default_setting.h"

#endif

#ifdef MAX3580

#include "MB86A21_Max3580_default_setting.h"

#endif

#ifdef MXL135RF

#include "MB86A21_MxL_default_setting.h"
#include "Mxl135RF/MxL135RF_API.h"
#include "Mxl135RF/MxL135RF_Common.h"

#endif

#include "MB86A21_API_version.h"
#include "MB86A21_API_defs.h"
#include "Tuner.h"
#include "MB86A21_API_structs.h"
#include "MB86A21_API_version.h"
#include "MB86A21_user_define.h"
#include "MB86A21_API.h"
#include "MB86A21_print.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_services.h>
#include <fapi/drv_gpio.h>
#else
#include "fapex/abst_libgpio.h"
#endif
/*---------------------------------------------------------------------------*/
/* local variables                                                           */
/*---------------------------------------------------------------------------*/
static feHandleT *pData[FE_MAX_NUM];
static MB86A2x_VERSION_INFO  version_info[FE_MAX_NUM];     // prepare pointer(1/8)
static MB86A2x_PARAM         feDat[FE_MAX_NUM];            // prepare pointer(2/8)
static MB86A2x_LOCKNG_INFO   locking_info[FE_MAX_NUM];     // prepare pointer(3/8)
static MB86A2x_AGC_INFO      agc_info[FE_MAX_NUM];         // prepare pointer(4/8)
//static MB86A2x_FREQ_ERR_INFO freq_err_info[FE_MAX_NUM];    // prepare pointer(5/8)
static MB86A2x_TMCC_INFO     tmcc_info[FE_MAX_NUM];        // prepare pointer(6/8)
static MB86A2x_SIG_QUAL_INFO sign_qual_info[FE_MAX_NUM];   // prepare pointer(7/8)
static MB86A2x_CH_SCH_INFO   ch_sch_info[FE_MAX_NUM];      // prepare pointer(8/8)

tuner_type_t mb86a21TunerType[FE_MAX_NUM] = { NO_TUNER };

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t MB86A21_Open(uint8_t index, FE_HANDLE *pDeviceData);
int32_t MB86A21_Close(uint8_t index);
int32_t MB86A21_GetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t MB86A21_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t MB86A21_Reset(uint8_t index);
int32_t MB86A21_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t MB86A21_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t MB86A21_GetAFC(uint8_t index, int32_t* pAFC);
int32_t MB86A21_GetBER(uint8_t index, uint32_t* pBER);
int32_t MB86A21_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t MB86A21_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t MB86A21_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);

void mb86a21SetFunctions(feHandleT *hp)
{
    // register APIs to handle
    hp->systemType             = FE_SYSTEM_TYPE_TERRESTRIAL;
    hp->feOpenFct              = MB86A21_Open;
    hp->feCloseFct             = MB86A21_Close;
    hp->feGetSystemParamsFct   = MB86A21_GetSystemParams;
    hp->feSetSystemParamsFct   = MB86A21_SetSystemParams;
    hp->feResetFct             = MB86A21_Reset;
    hp->feTuneFct              = MB86A21_Tune;
    hp->feScanFct              = NULL;
    hp->feSendDiseqcCommandFct = NULL;
    hp->feGetStatusFct         = MB86A21_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = MB86A21_GetAFC;
    hp->feGetBERFct            = MB86A21_GetBER;
    hp->feGetSignalQualityFct  = MB86A21_GetSignalQuality;
    hp->feGetSignalStrengthFct = MB86A21_GetSignalStrength;
    hp->feGetTuneParamsFct     = MB86A21_GetTuneParams;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
}

#ifdef TDA18211
/* Digibest silicon tuner */
int32_t MB86A21_TDA18211_Detect( uint8_t slot, feHandleT* hp )
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;
    
    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }
    
    // hardware reset
    MB86A21_Reset(slot);
    
    // read chip ID
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    if( ferr == FE_OK )
    {
         FAPI_SYS_PRINT_MSG("Chip ID:0x%x\n",i2cBuffer[1]);
        /* the Driver suppose to be compatible for MB86A21 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A21 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        FAPI_SYS_PRINT_MSG("FE_I2C_Read error\n");
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    // set RF I2C address by pinging different addresses
    for( i = 0; i < 5; i++, rf_addr += 2 )
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        ferr = FE_I2C_Write( hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2 );
        if( ferr != FE_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            break;
        }
        i2cBuffer[0] = 0x55;
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2 );
        if( ferr == FE_OK )
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator with RF addr 0x%2x\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    mb86a21SetFunctions(hp);
    
    hp->slotNum  = slot;
    mb86a21TunerType[slot] = ISDBT_TDA18211;
    return FE_OK;
}
#endif

#ifdef NI386X
/* Nutune Full-NIM CAN tuner */
int32_t MB86A21_NI386x_Detect( uint8_t slot, feHandleT* hp )
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;
    
    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    (void) MB86A21_Reset(slot);
    
    // read chip ID
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    if( ferr == FE_OK )
    {
        /* the Driver suppose to be compatible for MB86A21 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A21 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    // set RF I2C address by pinging different addresses
    for( i = 0; i < 5; i++, rf_addr += 2 )
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        ferr = FE_I2C_Write( hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2 );
        if( ferr != FE_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            break;
        }
        
        i2cBuffer[0] = 0x55;
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2 );
        if( ferr == FE_OK )
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator with RF addr 0x%2x\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    mb86a21SetFunctions(hp);
    
    hp->slotNum  = slot;
    mb86a21TunerType[slot] = ISDBT_NI386X;
    return FE_OK;
}
#endif

#ifdef MAX3580
/* Gospell silicon tuner */
int32_t MB86A21_MAX3580_Detect( uint8_t slot, feHandleT* hp )
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;
    
    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }
    
    // hardware reset
    MB86A21_Reset(slot);
   
    // read chip ID
    #ifndef _LINUX_FAPI
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #else
    ferr = FE_I2C_Read(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #endif

    if( ferr == FE_OK )
    {
        /* the Driver suppose to be compatible for MB86A21 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A21 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    // set RF I2C address by pinging different addresses
    for( i = 0; i < 5; i++, rf_addr += 2 )
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
       #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write( hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2 );
       #else
        ferr = FE_I2C_Write( hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 2 );
       #endif 
       if( ferr != FE_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            break;
        }
        i2cBuffer[0] = 0x55;
        #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2 );
        #else
        ferr = FE_I2C_Write(hp->i2cId, rf_addr, &(i2cBuffer[0]), 2 );
        #endif
        if( ferr == FE_OK )
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator with RF addr 0x%2x\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    mb86a21SetFunctions(hp);
    
    hp->slotNum  = slot;
    mb86a21TunerType[slot] = ISDBT_MAX3580;
    return FE_OK;
}
#endif

#ifdef MXL135RF
/* Maxlinear silicon tuner */
int32_t MB86A21_MXL135RF_Detect( uint8_t slot, feHandleT* hp )
{
    uint8_t i;
    uint8_t i2cBuffer[2] = { 0x00, 0xFF };
    int32_t ferr;
    MxL135RF_TunerConfigS myTuner;
    MxL135RF_ChipVersion myChipVersion;
    
    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }
    
    // hardware reset
    MB86A21_Reset(slot);
    
    // read chip ID
    #ifndef _LINUX_FAPI
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #else
    ferr = FE_I2C_Read(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #endif
    if( ferr == FE_OK )
    {
         FAPI_SYS_PRINT_MSG("MB86A21 Chip ID:0x%x\n",i2cBuffer[1]);

        /* the Driver suppose to be compatible for MB86A21 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A21 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
    	#ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
        FAPI_SYS_PRINT_MSG("FE_I2C_Read error:%d\n",ferr);
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    
    // check RF chip id
    hp->i2cAddrTuner = MXL135RF_Tuner_Adr_DEFAULT;
    myTuner.I2C_Addr = (MxL135RF_I2CAddr) MXL135RF_Tuner_Adr_DEFAULT;
    for( i = 0; i < 5; i++, myTuner.I2C_Addr += 1 )
    {
        myChipVersion = MxL_Check_ChipVersion( &myTuner );
        if(myChipVersion == MxL_135RF_ID)
        {
FAPI_SYS_PRINT_MSG("Detect version=%d\n",myChipVersion);
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Maxlinear RF device detected with chipID 0x%x\n", myChipVersion);
        #endif
            hp->i2cAddrTuner = myTuner.I2C_Addr;
            break;
        }
    }
    if(myChipVersion != MxL_135RF_ID)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to get Maxlinear RF chip ID\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }

    mb86a21SetFunctions(hp);
    
    hp->slotNum  = slot;
    mb86a21TunerType[slot] = ISDBT_MXL135RF;
    return FE_OK;
}
#endif

int32_t MB86A21_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    MB86A2x_ERR_MSG ERR_MSG;
    
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    pData[index] = &feHandleArray[index];
    *pDeviceData = (FE_HANDLE)(&feHandleArray[index]);
    
    //Enter device address to the pointer.
    MB86A2x_Init_Adr( &feDat[index], mb86a21TunerType[index] );

    //Check version information.
    ERR_MSG = MB86A2x_Get_Version( &feDat[index].nim_param, &version_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_Version ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
#ifdef DEBUG_FE
    FUJITSU_printf_version( &version_info[index] ) ;
#endif

    //Enter initialization parameters after power on to the pointer.
    MB86A2x_Init_Param( &feDat[index], mb86a21TunerType[index] );
    return FE_OK;
}

int32_t MB86A21_Close(uint8_t index)
{
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
        
    // TODO: For TDA18211/18271, power down mode could be set here
    return FE_OK;
}

int32_t MB86A21_GetSystemParams(uint8_t index,FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    memcpy( pSystemParams, &(pData[index]->pSystemParams), sizeof(FE_SYSTEM_PARAMS_S) );
    return FE_OK;
}

int32_t MB86A21_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    memcpy( &(pData[index]->pSystemParams), pSystemParams, sizeof(FE_SYSTEM_PARAMS_S) );
    return FE_OK;
}

int32_t MB86A21_Reset(uint8_t index)
{
    int32_t error;
    
    if(index > FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }

    error = FE_ControlFeReset(index,FE_RESET_PULSE);
    if(error != FE_OK)
    {  
        return error; 
    }
 
    FAPI_SYS_SLEEP(40);
}

int32_t MB86A21_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    MB86A2x_ERR_MSG       ERR_MSG ;
    UINT8 IFlevel_value = 0x07; // IF_Level -12 dB
    UINT32 freq ;            //channel frequency[kHz]
    #ifdef TMBSL_TDA18271
    // tuner declaration & init
    int32_t i2cerr = 0;
    tmUnitSelect_t TunerUnit = index;
    tmbslTuParam_t Param;
    tmhalFEBslSystemFunc_t thesystemFunc;
    #else
    UINT32 IFLEVEL = 362;
    #endif
    
    MB86A2x_RECEP_LAYER recep_layer ;    //Recption layer
                                        //This value should be one of below values.
                                        //MB86A2x_RECEP_LAYER_A : layer-A reception
                                        //MB86A2x_RECEP_LAYER_B : layer-B reception
                                        //MB86A2x_RECEP_LAYER_C : layer-C reception
                                        //MB86A2x_RECEP_HIGHEST : highest layer reception

    //Enter device address to the pointer.
    ERR_MSG = MB86A2x_Init_Adr( &feDat[index], mb86a21TunerType[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Init_Adr ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }

    //Check version information.
    ERR_MSG = MB86A2x_Get_Version( &feDat[index].nim_param, &version_info[index] ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_Version ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    else
    {
    #ifdef DEBUG_FE
        FUJITSU_printf_version( &version_info[index] ) ;
    #endif
    }
    
    #ifdef TMBSL_TDA18271
    if( mb86a21TunerType[index] == ISDBT_TDA18211 )
    {
        //Initialize the tuner system function 
        thesystemFunc.SY_GetTickTime = &MB86A2x_GetTickTime ;
        thesystemFunc.SY_GetTickPeriod = &MB86A2x_GetTickPeriod ;
        thesystemFunc.SY_GetCommErrors = NULL ;
        thesystemFunc.SY_Write = &MB86A2x_silicon_tuner_write_in_dword ;
        thesystemFunc.SY_WriteBit = NULL ;
        thesystemFunc.SY_Read = &MB86A2x_silicon_tuner_read ;
        thesystemFunc.SY_TimerWait = &MB86A2x_wait ;
        
        Param.uHwAddress = (feDat[index].nim_param.MB86A2x_Adr<<8) | 0xc0 ;    //TDA18271 Device address=0xc0
        Param.systemFunc = thesystemFunc;
        i2cerr = tmbslTDA18271Init(TunerUnit, Param);
        i2cerr = tmbslTDA18271SetConfig(TunerUnit, FEINIT, 0);
    }
    #endif
    
    //Enter initialization parameters after power on to the pointer.
    ERR_MSG = MB86A2x_Init_Param( &feDat[index],mb86a21TunerType[index] ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Init_Param ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    
#ifdef TMBSL_TDA18271

    i2cerr = tmbslTDA18271SetStandardMode(TunerUnit, tmDigital_TV_ISDBT_6MHz); // IF : 4.0 MHz ; IF_level : 0dB
    if( i2cerr>=1 )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "I2C ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
        
#endif
    
    //Write initialization parameters after power on.
    ERR_MSG = MB86A2x_Set_Init( &feDat[index] ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Set_Init ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    
    //Set reception frequency here.
    freq = pTuneParams->terr.frequency / 1000 ;
    //Set reception segment here.
    recep_layer = MB86A2x_RECEP_HIGHEST ;

    //MB86A21 reception procedure(1/2)
    ERR_MSG = Reception_before_Channel_Search_Fixed_Timer_Register_Values_1_freq(
        &feDat[index], &locking_info[index],
        &sign_qual_info[index], &version_info[index],
        index, IFLEVEL, IFlevel_value,
        freq ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "Reception_before_Channel_Search_Fixed_Timer_Register_Values_1_freq ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    
    //MB86A21 reception procedure(2/2)
    ERR_MSG = Reception_before_Channel_Search_Fixed_Timer_Register_Values_2(
            &feDat[index], &locking_info[index], &sign_qual_info[index], &tmcc_info[index], &ch_sch_info[index], &version_info[index], recep_layer ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "Reception_before_Channel_Search_Fixed_Timer_Register_Values_2 ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    
    //set the timeout waiting for tuner locking
    FE_SetTuneTimeout(index, 0xFFFFFFFF);
    
    return FE_OK;
}

int32_t MB86A21_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    MB86A2x_ERR_MSG ERR_MSG;
    
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    ERR_MSG = MB86A2x_Get_SEQ_value( &feDat[index].nim_param, &locking_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_SEQ_value ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    else
    {
		#ifdef DEBUG_FE
        	FAPI_SYS_PRINT_MSG( "locking_info.STATE: %d\n",locking_info[index].STATE) ;
    	#endif
        if( locking_info[index].STATE >= 8 )
        {
            *pStatus = FE_STATUS_LOCKED;
        }
        else
        {
            if( FE_IsTuneTimeout(index) == 0 )
            {
                *pStatus = FE_STATUS_TUNING;
            }
            else
            {
                *pStatus = FE_STATUS_NOT_LOCKED;
            }
        }
    }
    return FE_OK;
}

int32_t MB86A21_GetAFC(uint8_t index, int32_t* pAFC)
{
    if (index >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    if (pAFC == NULL)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    *pAFC = 0;
    
    // still open

    return FE_OK;
}

int32_t MB86A21_GetBER(uint8_t index, uint32_t* pBER)
{
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    // TODO
    return FE_OK;
}

int32_t MB86A21_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    MB86A2x_ERR_MSG ERR_MSG;
    
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }

    ERR_MSG = MB86A2x_Get_CN_Flag( &feDat[index].nim_param, &sign_qual_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_CN_Flag ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    else
    {    
        if( sign_qual_info[index].CN_update == 1 ) 
        {   
            //Check whether whether C/N measurement is finished or not by checking the value of CN_update.
            //Read register values to know C/N value and calculate estimated C/N.
            ERR_MSG = MB86A2x_Get_CN_Value( &feDat[index].nim_param, &sign_qual_info[index] );
            if( ERR_MSG != MB86A2x_OK )
            {
            #ifdef DEBUG_FE
                FAPI_SYS_PRINT_MSG( "MB86A2x_Get_CN_Value ERR\n" ) ;
            #endif
                return FE_ERR_FE_I2C;
            }
    
            //Reset C/N counter
            ERR_MSG = MB86A2x_Reset_CN_Counter( &feDat[index].nim_param, &feDat[index].mon_tim_param );
            if( ERR_MSG != MB86A2x_OK )
            {
            #ifdef DEBUG_FE
                FAPI_SYS_PRINT_MSG( "MB86A2x_Reset_CN_Counter ERR\n" ) ;
            #endif
                return FE_ERR_FE_I2C;
            }
        }
        *pQuality = (int16_t)(sign_qual_info[index].CN_current) * 2184;
    }
    return FE_OK;
}

int32_t MB86A21_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    MB86A2x_ERR_MSG ERR_MSG;
    
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
    
    ERR_MSG = MB86A2x_Get_IF_level( &feDat[index].nim_param, &agc_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_IF_level ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    *pStrength = agc_info[index].IF_INPUT_LEVEL * 256;
    return FE_OK;
}

int32_t MB86A21_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{ 
    FE_STATUS_E lockStatus = FE_STATUS_NOT_LOCKED;
    
    if( index > FE_MAX_NUM )
    {
        return FE_ERR_BAD_PARAMETER;
    }
        
    /* check if is locked */
    MB86A21_GetStatus(index, &lockStatus);
    if(lockStatus == FE_STATUS_LOCKED)
    {
        /* get ISDB-T configuration and map into DVB structure */
        pTuneParams->terr.frequency = feDat[index].nim_param.tuner_param.RF_freq_kHz * 1000 ;
        
        /* currently driver only support 6Mhz, so fixed here */
        pTuneParams->terr.bandwidth = FE_BANDWIDTH_6_MHZ ;
        
        /* Because ISDB-T could have multi-layers in one frequency, below fixed as auto */
        pTuneParams->terr.codeRateHP    = FE_FEC_AUTO ;
        pTuneParams->terr.codeRateLP    = FE_FEC_AUTO ;
        pTuneParams->terr.modulation    = FE_MOD_NONE ;
        pTuneParams->terr.transmission  = FE_TRANS_MODE_AUTO ;
        pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_AUTO ;
        pTuneParams->terr.hierarchy     = FE_HIERARCHY_AUTO ;
        pTuneParams->terr.iq            = FE_IQ_AUTO ;
        
        return FE_OK;
    }
    else
    {
        return FE_STATUS_NOT_LOCKED;
    }
}

