/*==============================================================================
    FILE NAME   : FC2581_driver_v1411.h  
    
    VERSION     : 1.411
    
    UPDATE      : October 11. 2010
    
==============================================================================*/ 

/*==============================================================================

  Chip ID of fc2581 is 0x56 or 0xAC(including I2C write bit)

==============================================================================*/


#define FREQ_XTAL	16000	//16MHz
#define	BORDER_FREQ	2600000	//2.6GHz : The border frequency which determines whether Low VCO or High VCO is used
#define USE_EXT_CLK	0	//0 : Use internal XTAL Oscillator / 1 : Use External Clock input
#define OFS_RSSI 57

typedef enum {
	UHF_BAND,
	L_BAND,
	VHF_BAND,
	NO_BAND
} fc2581_band_type;

/*==============================================================================
		i2c command write							EXTERNAL FUNCTION

  This function is a generic function which write a byte into fc2581's
 specific address.

  <input parameter>

  slave_id
	i2c id of slave chip
	type : byte
	
  addr
	memory address of slave chip
	type : byte

  data
	target data
	type : byte

==============================================================================*/
extern void i2c_write( unsigned char slave_id, unsigned char addr, unsigned char data );

/*==============================================================================
		i2c command write							EXTERNAL FUNCTION

  This function is a generic function which gets called to read data from
 slave chip's target memory address.

  <input parameter>

  slave_id
	i2c id of slave chip
	type : byte
	
  addr
	memory address of slave chip
	type : byte

  <return value>
  data
	a byte of data read out of target address 'addr' of slave chip
	type : byte

==============================================================================*/
extern unsigned char i2c_read( unsigned char slave_id, unsigned char addr );

/*==============================================================================
		milisecond delay function					EXTERNAL FUNCTION

  This function is a generic function which write a byte into fc2581's
 specific address.

  <input parameter>

  a
	length of wanted delay in milisecond unit

==============================================================================*/
extern void wait_msec(int a);



/*==============================================================================
       fc2581 i2c command write

  This function is a generic function which write a byte into fc2581's
 specific address.

  <input parameter>

  addr
	fc2581's memory address
	type : byte

  data
	target data
	type : byte

==============================================================================*/
void fc2581_i2c_write(unsigned char tunerAddr, unsigned char addr, unsigned char data);

/*==============================================================================
       fc2581 i2c data read

  This function is a generic function which gets called to read data from
 fc2581's target memory address.

  <input parameter>

  addr
	fc2581's memory address
	type : byte


  <return value>
  data
	a byte of data read out of target address 'addr'
	type : byte

==============================================================================*/
unsigned char fc2581_i2c_read(unsigned char tunerAddr, unsigned char addr);

/*==============================================================================
       fc2581 initial setting

  This function is a generic function which gets called to initialize

  fc2581 in DVB-H mode or L-Band TDMB mode

  <input parameter>

  ifagc_mode
    type : integer
	1 : Internal AGC
	2 : Voltage Control Mode

==============================================================================*/
void fc2581_set_init(unsigned char tunerAddr, int ifagc_mode);

/*==============================================================================
       fc2581 frequency setting

  This function is a generic function which gets called to change LO Frequency

  of fc2581 in DVB-H mode or L-Band TDMB mode

  <input parameter>

  f_lo
	Value of target LO Frequency in 'kHz' unit
	ex) 2.6GHz = 2600000

==============================================================================*/
void fc2581_set_freq(unsigned char tunerAddr, unsigned int f_lo);


/*==============================================================================
       fc2581 filter BW setting

  This function is a generic function which gets called to change Bandwidth

  frequency of fc2581's channel selection filter

  <input parameter>

  filter_bw
    1 : 1.53MHz(TDMB)
	5 : 5MHz
	6 : 6MHz
	7 : 7MHz
	8 : 7.8MHz
	9 : 8MHz

==============================================================================*/
void fc2581_set_filter(unsigned char tunerAddr, unsigned char filter_bw);

/*==============================================================================
       fc2581 RSSI function

  This function is a generic function which returns fc2581's
  
  current RSSI value.

  <input parameter>
	none

  <return value>
  int
  	rssi : estimated input power.

==============================================================================*/
int fc2581_get_rssi(unsigned char tunerAddr);

