/****************************************************************************
*
*  Functions to calculate Tuner Parameter
*
*  Copyright (c) 2009 Fujitsu Microelectronics 
*  All rights reserved. 
*
*  Created 02/02/2009
*
****************************************************************************/
#include "../src/fe_priv.h" //for gnuarm compiler
#include "MB86A22_API_defs.h"
#include "MB86A22_Tuner.h"

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
#include "MAX3580/Max3580Driver.h"
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
#include "Mxl136RF/MxL136RF_API.h"
#include "Mxl136RF/MxL136RF_Common.h"
#endif

TUNER_MSG MB86A22_SHARP_TUNER_INIT( TUNER_PARAM *mytuner )
{
    UINT8  CP = 0;
    UINT8  BS = 0;
    UINT32 N = 0;
    UINT8  AL = 0;
    UINT8  ATC = 0;

    if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

    if(      mytuner->RF_freq_kHz<140000 ) CP = 4 ;
    else if( mytuner->RF_freq_kHz<163000 ) CP = 5 ;
    else if( mytuner->RF_freq_kHz<220000 ) CP = 3 ;
    else if( mytuner->RF_freq_kHz<330000 ) CP = 5 ;
    else if( mytuner->RF_freq_kHz<354000 ) CP = 7 ;
    else if( mytuner->RF_freq_kHz<400000 ) CP = 2 ;
    else if( mytuner->RF_freq_kHz<450000 ) CP = 3 ;
    else if( mytuner->RF_freq_kHz<550000 ) CP = 4 ;
    else if( mytuner->RF_freq_kHz<600000 ) CP = 5 ;
    else if( mytuner->RF_freq_kHz<700000 ) CP = 6 ;
    else                                   CP = 7 ;

    if(      mytuner->RF_freq_kHz<163000 ) BS = 1 ;
    else if( mytuner->RF_freq_kHz<354000 ) BS = 2 ;
    else                                   BS = 4 ;

    N = (mytuner->RF_freq_kHz + mytuner->IF_freq_kHz) * 7 / 1000 ;

    AL = 0 ;
    ATC = 0 ;

    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = 0xc1 ;
    mytuner->Tuner_Data[3] = (CP<<5) | BS ;
    mytuner->Tuner_Data[4] = 0x80 | (ATC<<3) | AL ;

    return TUNER_OK ;
}


TUNER_MSG MB86A22_ALPS_TDAL_TUNER_INIT( TUNER_PARAM *mytuner )
{
    UINT8 CP2, CP1, CP0 ;
    UINT8 CP = 0;
    UINT8 AGST = 0;
    UINT8 BS = 0;
    UINT8 R ;
    UINT32 M ;
    UINT32 N ;

    if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

    R = 1;        // Reference Freq: 50KHz

    if(      (mytuner->RF_freq_kHz>=174000) && (mytuner->RF_freq_kHz<=216000) ) AGST = 3 ;
    else if( (mytuner->RF_freq_kHz>=470000) && (mytuner->RF_freq_kHz<=806000) ) AGST = 4 ;

    if(      (mytuner->RF_freq_kHz>=174000) && (mytuner->RF_freq_kHz<=216000) ) { CP=4 ; BS=2 ; }
    else if( (mytuner->RF_freq_kHz>=470000) && (mytuner->RF_freq_kHz<=750000) ) { CP=6 ; BS=8 ; }
    else if( (mytuner->RF_freq_kHz >750000) && (mytuner->RF_freq_kHz<=806000) ) { CP=0 ; BS=8 ; }
 
    CP1 = ( CP & 0x04 ) >> 2 ;
    CP0 = ( CP & 0x02 ) >> 1 ;
    CP2 = ( CP & 0x01 ) ;

    if(      R==0 ) M =  24 ;
    else if( R==1 ) M =  28 ;
    else if( R==2 ) M =  80 ;
    else if( R==3 ) M =  64 ;
    else            M = 128 ;

    N = ((mytuner->RF_freq_kHz*10 + mytuner->IF_freq_kHz*10)*M+20000) / 40000 ;

    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = 0x80 | (AGST<<3) | R ;
    mytuner->Tuner_Data[3] = 0x00 | (CP1<<7) | (CP0<<6) | BS ;
    mytuner->Tuner_Data[4] = 0xc2 | (CP2<<3) ;

    return TUNER_OK ;
}


TUNER_MSG MB86A22_THOMSON_TUNER_INIT_1( TUNER_PARAM *mytuner )
{
    UINT8 ATP ;
    UINT8 RS ;
    UINT8 CP = 0;
    UINT8 BS= 0, BS3 = 0 ;
    UINT8 ATC ;
    UINT8 STBY ;
    UINT8 XTO ;
    UINT32 N ;

    if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

    ATP = 1 ;
    RS = 0 ;    //for Digital reception
    BS3 = 0 ;   //GPIO = Off
    ATC = 1 ;
    STBY = 0 ;
    XTO = 1 ;   //XTAL Output = off

    if(      mytuner->RF_freq_kHz<=140000 ) { CP=0 ; BS=1 ; }
    else if( mytuner->RF_freq_kHz<=172000 ) { CP=1 ; BS=1 ; }
    else if( mytuner->RF_freq_kHz<=188000 ) { CP=2 ; BS=1 ; }

    else if( mytuner->RF_freq_kHz<=344000 ) { CP=0 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=424000 ) { CP=1 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=460000 ) { CP=2 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=470000 ) { CP=3 ; BS=2 ; }

    else if( mytuner->RF_freq_kHz<=514000 ) { CP=0 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=814000 ) { CP=1 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=874000 ) { CP=2 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=904000 ) { CP=3 ; BS=4 ; }


    N = ( (mytuner->RF_freq_kHz + mytuner->IF_freq_kHz) * 6 + 500 ) / 1000 ;    //166.67kHz

    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = 0x80 | (ATP<<3) | RS ;
    mytuner->Tuner_Data[3] = 0x00 | (CP<<6)  | ((BS&0x04)<<1) | (BS3<<2) | (BS&0x03) ;
    mytuner->Tuner_Data[4] = 0xc2 | (ATC<<5) | (STBY<<4) | XTO ;

    return TUNER_OK ;
}


TUNER_MSG MB86A22_THOMSON_TUNER_INIT_2( TUNER_PARAM *mytuner )
{
    UINT8 ATP ;
    UINT8 RS ;
    UINT8 CP = 0;
    UINT8 BS = 0, BS3 = 0;
    UINT8 ATC ;
    UINT8 STBY ;
    UINT8 XTO ;
    UINT32 N ;

    ATP = 1 ;
    RS = 0 ;    //for Digital reception
    BS3 = 0 ;   //GPIO = Off
    ATC = 0 ;
    STBY = 0 ;
    XTO = 1 ;   //XTAL Output = off

    if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

    if(      mytuner->RF_freq_kHz<=140000 ) { CP=0 ; BS=1 ; }
    else if( mytuner->RF_freq_kHz<=172000 ) { CP=1 ; BS=1 ; }
    else if( mytuner->RF_freq_kHz<=188000 ) { CP=2 ; BS=1 ; }

    else if( mytuner->RF_freq_kHz<=344000 ) { CP=0 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=424000 ) { CP=1 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=460000 ) { CP=2 ; BS=2 ; }
    else if( mytuner->RF_freq_kHz<=470000 ) { CP=3 ; BS=2 ; }

    else if( mytuner->RF_freq_kHz<=514000 ) { CP=0 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=814000 ) { CP=1 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=874000 ) { CP=2 ; BS=4 ; }
    else if( mytuner->RF_freq_kHz<=904000 ) { CP=3 ; BS=4 ; }


    N = ( (mytuner->RF_freq_kHz + mytuner->IF_freq_kHz) * 6 + 500 ) / 1000 ;    //166.67kHz

    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = 0x80 | (ATP<<3) | RS ;
    mytuner->Tuner_Data[3] = 0x00 | (CP<<6)  | ((BS&0x04)<<1) | (BS3<<2) | (BS&0x03) ;
    mytuner->Tuner_Data[4] = 0xc2 | (ATC<<5) | (STBY<<4) | XTO ;

    return TUNER_OK ;
}

TUNER_MSG MB86A22_SAMSUNG_TUNER_INIT( TUNER_PARAM *mytuner )
{
/*

//  int TuAdr = 0xc2 ;
//  int tu_val[6] ;
//  unsigned char CP ;
//  unsigned char BS ;
//  unsigned char AL ;
//  unsigned char ATC ;
//  int N ;
    int freq1000000 ;

    //tuner bypass setting
    I2C->I2Cwrite( 0xfe, 0x00 ) ;

    CP = 6 ;

    if(      freq<110.0 ) BS = 1 ;
    else if( freq<225.0 ) BS = 2 ;
    else                  BS = 8 ;

    AL = 2 ;
    ATC = 0 ;

    freq1000000 = (int)((freq)*1000000.0) ;
//  N = (freq1000000+44000000)*0.00002 ;
    N = (freq1000000+44000000)*2/100000 ;
    TuAdr = 0xc2 ;
    tu_val[0] = N >> 8 ;
    tu_val[1] = N & 0x00ff ;
    tu_val[2] = 0xc3 ;
    tu_val[3] = (CP<<5) | BS ;
    tu_val[4] = 0x80 | (ATC<<3) | AL ;

    I2C->TUNER_write( TuAdr, 5, tu_val ) ;
*/

// sharp to samusung

    UINT8  CP ;
    UINT8  BS ;
    UINT32 N ;
    UINT8  AL ;
    UINT8  ATC ;

    if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

//  if(      mytuner->RF_freq_kHz<140000 ) CP = 4 ;
//  else if( mytuner->RF_freq_kHz<163000 ) CP = 5 ;
//  else if( mytuner->RF_freq_kHz<220000 ) CP = 3 ;
//  else if( mytuner->RF_freq_kHz<330000 ) CP = 5 ;
//  else if( mytuner->RF_freq_kHz<354000 ) CP = 7 ;
//  else if( mytuner->RF_freq_kHz<400000 ) CP = 2 ;
//  else if( mytuner->RF_freq_kHz<450000 ) CP = 3 ;
//  else if( mytuner->RF_freq_kHz<550000 ) CP = 4 ;
//  else if( mytuner->RF_freq_kHz<600000 ) CP = 5 ;
//  else if( mytuner->RF_freq_kHz<700000 ) CP = 6 ;
//  else                                   CP = 7 ;

    CP = 6 ;

//  if(      mytuner->RF_freq_kHz<163000 ) BS = 1 ;
//  else if( mytuner->RF_freq_kHz<354000 ) BS = 2 ;
//  else                                   BS = 4 ;

    if      (mytuner->RF_freq_kHz<110000 ) BS = 1 ;
    else if( mytuner->RF_freq_kHz<225000 ) BS = 2 ;
    else                  BS = 8 ;

//  N = (mytuner->RF_freq_kHz + mytuner->IF_freq_kHz) * 7 / 1000 ;
//  N = (freq1000000+44000000)*2/100000 ;

    N = (mytuner->RF_freq_kHz*1000+44000000)*2/100000 ;

//  AL = 0 ;
//  ATC = 0 ;

    AL = 2 ;
    ATC = 0 ;

//  mytuner->Tuner_Data[0] = N >> 8 ;
//  mytuner->Tuner_Data[1] = N & 0x00ff ;
//  mytuner->Tuner_Data[2] = 0xc1 ;
//  mytuner->Tuner_Data[3] = (CP<<5) | BS ;
//  mytuner->Tuner_Data[4] = 0x80 | (ATC<<3) | AL ;

    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = 0xc3 ;
    mytuner->Tuner_Data[3] = (CP<<5) | BS ;
    mytuner->Tuner_Data[4] = 0x80 | (ATC<<3) | AL ;

    return TUNER_OK ;
}




static TUNER_MSG CH_to_freq( TUNER_PARAM *mytuner )
{

    if( mytuner->CH_UNIT==MB86A2x_CH_UNIT_JP ) {
        if(      mytuner->CH== 1 ) mytuner->RF_freq_kHz = 93143 ;
        else if( mytuner->CH== 2 ) mytuner->RF_freq_kHz = 99143 ;
        else if( mytuner->CH== 3 ) mytuner->RF_freq_kHz = 105143 ;
        else if( mytuner->CH== 4 ) mytuner->RF_freq_kHz = 173143 ;
        else if( mytuner->CH== 5 ) mytuner->RF_freq_kHz = 179143 ;
        else if( mytuner->CH== 6 ) mytuner->RF_freq_kHz = 185143 ;
        else if( mytuner->CH== 7 ) mytuner->RF_freq_kHz = 191143 ;
        else if( mytuner->CH== 8 ) mytuner->RF_freq_kHz = 195143 ;
        else if( mytuner->CH== 9 ) mytuner->RF_freq_kHz = 201143 ;
        else if( mytuner->CH==10 ) mytuner->RF_freq_kHz = 207143 ;
        else if( mytuner->CH==11 ) mytuner->RF_freq_kHz = 213143 ;
        else if( mytuner->CH==12 ) mytuner->RF_freq_kHz = 219143 ;
        else if( mytuner->CH==13 ) mytuner->RF_freq_kHz = 473143 ;
        else                       mytuner->RF_freq_kHz = 6000*((mytuner->CH)-13)+ 473143 ;
    }
    else {
        if( mytuner->CH<=13 ) mytuner->RF_freq_kHz = 177143 + 6000*((mytuner->CH)-7) ;
        else                  mytuner->RF_freq_kHz = 473143 + 6000*((mytuner->CH)-14) ;
    }

    return TUNER_OK ;
}

#if 0
TUNER_MSG CH_to_freq_4_MxL( MxL136RF_TunerConfigS *mytuner )
{

    if( mytuner->CH_UNIT==MB86A2x_CH_UNIT_JP ) {
        if(      mytuner->CH== 1 ) mytuner->RF_Freq_Hz = 93143000 ;
        else if( mytuner->CH== 2 ) mytuner->RF_Freq_Hz = 99143000 ;
        else if( mytuner->CH== 3 ) mytuner->RF_Freq_Hz = 105143000 ;
        else if( mytuner->CH== 4 ) mytuner->RF_Freq_Hz = 173143000 ;
        else if( mytuner->CH== 5 ) mytuner->RF_Freq_Hz = 179143000 ;
        else if( mytuner->CH== 6 ) mytuner->RF_Freq_Hz = 185143000 ;
        else if( mytuner->CH== 7 ) mytuner->RF_Freq_Hz = 191143000 ;
        else if( mytuner->CH== 8 ) mytuner->RF_Freq_Hz = 195143000 ;
        else if( mytuner->CH== 9 ) mytuner->RF_Freq_Hz = 201143000 ;
        else if( mytuner->CH==10 ) mytuner->RF_Freq_Hz = 207143000 ;
        else if( mytuner->CH==11 ) mytuner->RF_Freq_Hz = 213143000 ;
        else if( mytuner->CH==12 ) mytuner->RF_Freq_Hz = 219143000 ;
        else if( mytuner->CH==13 ) mytuner->RF_Freq_Hz = 473143000 ;
        else                       mytuner->RF_Freq_Hz = 6000000*((mytuner->CH)-13)+ 473143000 ;
    }
    else {
        if( mytuner->CH<=13 ) mytuner->RF_Freq_Hz = 177143000 + 6000000*((mytuner->CH)-7) ;
        else                  mytuner->RF_Freq_Hz = 473143000 + 6000000*((mytuner->CH)-14) ;
    }

    return TUNER_OK ;
}
#endif
TUNER_MSG MB86A22_NXP_TUNER_INIT_2( TUNER_PARAM *mytuner )
{
    UINT8  CP= 0 ,RS= 0,P= 0;//,ATC;
    int N;

    /* remove it to enable scan based on frequency user input */
    ///if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;

    if(      mytuner->RF_freq_kHz<150000 ) {CP = 0x7 ;P=2;}
    else if( mytuner->RF_freq_kHz<180000 ) {CP = 0xe ;P=2;}
    else if( mytuner->RF_freq_kHz<188000 ) {CP = 0xf ;P=2;}
    else if( mytuner->RF_freq_kHz<320000 ) {CP = 0x6 ;P=2;}
    else if( mytuner->RF_freq_kHz<420000 ) {CP = 0x7 ;P=2;}
    else if( mytuner->RF_freq_kHz<460000 ) {CP = 0xe ;P=2;}
    else if( mytuner->RF_freq_kHz<470000 ) {CP = 0xf ;P=2;}
    else if( mytuner->RF_freq_kHz<514000 ) {CP = 0x6 ;P=4;}
    else if( mytuner->RF_freq_kHz<745000 ) {CP = 0x7 ;P=4;}
    else if( mytuner->RF_freq_kHz<845000 ) {CP = 0xe ;P=4;}
    else if( mytuner->RF_freq_kHz<905000 ) {CP = 0xf ;P=4;}
        
    N = (mytuner->RF_freq_kHz + mytuner->IF_freq_kHz) * 7 / 1000 ;
#ifdef DEBUG_FE
    FE_DPrintf("NXP freq %dkhz N=%x IF_freq_kHz %d\n",mytuner->RF_freq_kHz,N,mytuner->IF_freq_kHz);
#endif
    RS  = 1;
    //ATC = 0x20 ;
    //mytuner->Tuner_Data[0] = 0xc2;
    mytuner->Tuner_Data[0] = N >> 8 ;
    mytuner->Tuner_Data[1] = N & 0x00ff ;
    mytuner->Tuner_Data[2] = (CP<<3)|0x80 |(RS<<1);//control bytes
    mytuner->Tuner_Data[3] = 0x80|P;               //band switch bytes
    return TUNER_OK ;
}

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
TUNER_MSG MB86A22_Max3580_TUNER_INIT( UINT8 OFDM_addr, UINT8 Tu_addr, TUNER_PARAM *mytuner )
{
    double f_Freq, xtalref;
    UINT8 value;
    
    max3580_set_device_adr(Tu_addr) ;

    /* remove it to enable scan based on frequency user input */
    ///if( mytuner->CH_UNIT<2 ) CH_to_freq( mytuner ) ;
    
    // Tuner Initialization
    value = Max3580Init();
    if (value == 0)
    {
        return TUNER_NG ;
    }
   
    // Tuner Parameter Setting
    f_Freq = (double)(mytuner->RF_freq_kHz) / 1000.0;
    xtalref = 16.0;
    value = Max3580SetFrequency(f_Freq, xtalref);
    if(value == 0)
    {
        return TUNER_NG;
    }
    return TUNER_OK ;
}
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
TUNER_MSG MB86A22_MxL_TUNER_INIT( TUNER_PARAM *tuner_param, MB86A2x_BANDSEL BANDSEL )
{
    MxL_ERR_MSG Status ;
    MxL136RF_LoopThru LoopThru_OnOff;
    MxL136RF_TunerConfigS mytuner;
    //UINT8 MxL_RegData;
    //MxL136RF_ChipVersion version = 0;
    //unsigned char MxL_RegData = 0;
    
    /* remove it to enable scan based on frequency user input */
    ///if( mytuner->CH_UNIT<2 ) CH_to_freq_4_MxL( mytuner ) ;
//  if( mytuner->CH_UNIT<2 ) CH_to_freq_4_MxL( mytuner ) ;
    
    if (BANDSEL == 0){
        mytuner.Mode = MxL_MODE_ISDBT ;
        mytuner.BW_MHz = MxL_BW_6MHz ;
    }
    else if (BANDSEL == 1){
        mytuner.Mode = MxL_MODE_DVBT ;
        mytuner.BW_MHz = MxL_BW_6MHz ;
    }
    else if (BANDSEL == 2){
        mytuner.Mode = MxL_MODE_DVBT ;
        mytuner.BW_MHz = MxL_BW_7MHz ;
    }
    else if (BANDSEL == 3){
        mytuner.Mode = MxL_MODE_DVBT ;
        mytuner.BW_MHz = MxL_BW_8MHz ;
    }
    
    mytuner.I2C_Addr = (MxL136RF_I2CAddr)(tuner_param->Tuner_Adr);
    mytuner.RF_Freq_Hz = (unsigned int)(tuner_param->RF_freq_kHz * 1000);   
    //mytuner.Mode = MxL_MODE_ISDBT ;
    mytuner.Xtal_Freq = MxL_XTAL_32_MHZ ;
    //mytuner.BW_MHz = MxL_BW_6MHz ;
    mytuner.ClkOut_Setting = MxL_CLKOUT_ENABLE ;
    mytuner.ClkOut_Amp = MxL_CLKOUT_AMP_0 ;
    mytuner.IF_Diff_Out_Level = -8 ;
    mytuner.IF_Spectrum = MxL_NORMAL_IF ;
    mytuner.IF_Freq = (MxL136RF_IF_Freq)(tuner_param->IF_freq_kHz * 1000);
    
  //  MxL_OFMD_adr_init( OFDM_adr );  

    LoopThru_OnOff = MxL_LT_DISABLE;
    Status = MxL136RF_Loop_Through_On( &mytuner, LoopThru_OnOff );
    if (Status != MxL_OK) return TUNER_NG;
    
    Status = MxL136RF_Tuner_Init( &mytuner ) ;
    if (Status != MxL_OK) return TUNER_NG;
        
    Status = MxL136RF_Tuner_RFTune( &mytuner, mytuner.RF_Freq_Hz, mytuner.BW_MHz ) ;
    if (Status != MxL_OK) return TUNER_NG;

    // IF Output Gain Setting: IF Gain = 63(Default) -> 61
    //Status = MxL_Get_Register(&mytuner, 0x00, &MxL_RegData);
    //if (Status != MxL_OK) return TUNER_NG;
    Status = MxL136RF_Set_Register(&mytuner, 0x00, 61);
    if (Status != MxL_OK) return TUNER_NG;
    //Status = MxL_Get_Register(&mytuner, 0x00, &MxL_RegData);
    //if (Status != MxL_OK) return TUNER_NG;
    return TUNER_OK ;
}
#endif

