/****************************************************************************
*
*  Functions to calculate Tuner Parameter
*
*  Copyright (c) 2009 Fujitsu Microelectronics 
*  All rights reserved. 
*
*  Created 03/24/2009
*
****************************************************************************/
#ifndef _MB86A22_TUNER_H_
#define _MB86A22_TUNER_H_
/****************************************************************************
*   Tuner Type
****************************************************************************/
typedef enum
{
   MB86A22_NO_TUNER = 0,
   MB86A22_ISDBT_TDA18211,  // NXP Silicon Tuner
   MB86A22_ISDBT_TDA18212,
   MB86A22_ISDBT_NI386X,    // Nutune Full-NIM
   MB86A22_ISDBT_MAX3580,   // Maxim Silcon Tuner done by Gospell
   MB86A22_ISDBT_MXL136RF,   // Maxlinear
   MB86A22_ISDBT_FC2581,   
   MB86A22_LAST_TUNER,
} MB86A22_tuner_type_t;

/* TUNER SUPPORTED TYPE, Please Select The Tuner You Used. */
#define MB86A22_TUNER_SUPPORT_TDA18211             0   //now it must be unused, reserved.
#define MB86A22_TUNER_SUPPORT_TDA18212              0  //now it must be unused, reserved.
#define MB86A22_TUNER_SUPPORT_MAX3580             0   //now it must be unused, reserved.
#define MB86A22_TUNER_SUPPORT_MXL136RF            1   
#define MB86A22_TUNER_SUPPORT_FC2581                1   
#define MB86A22_TUNER_SUPPORT_NI386X                    0  //now it must be unused, reserved.

/****************************************************************************
*   Tuner Configuration
****************************************************************************/
typedef struct 
{
    UINT8           Tuner_Adr ;
    UINT8           Tuner_Data[5] ;
    UINT32          IF_freq_kHz ;
    MB86A2x_CH_UNIT CH_UNIT ;
    UINT32          RF_freq_kHz ;
    UINT32          CH ;
} TUNER_PARAM ;


/****************************************************************************
*   Tuner Error Message
****************************************************************************/

typedef enum {
    TUNER_NG       = 0,
    TUNER_OK       = 1,
} TUNER_MSG ;


/****************************************************************************
*  Functions to calculate Tuner Parameter
****************************************************************************/

TUNER_MSG MB86A22_SHARP_TUNER_INIT( TUNER_PARAM *mytuner ) ;
TUNER_MSG MB86A22_ALPS_TDAL_TUNER_INIT( TUNER_PARAM *mytuner ) ;
TUNER_MSG MB86A22_THOMSON_TUNER_INIT_1( TUNER_PARAM *mytuner ) ;
TUNER_MSG MB86A22_THOMSON_TUNER_INIT_2( TUNER_PARAM *mytuner ) ;
TUNER_MSG MB86A22_SAMSUNG_TUNER_INIT( TUNER_PARAM *mytuner ) ;
TUNER_MSG MB86A22_NXP_TUNER_INIT_2( TUNER_PARAM *mytuner );
TUNER_MSG MB86A22_Max3580_TUNER_INIT(UINT8 OFDM_addr, UINT8 Tu_addr, TUNER_PARAM *mytuner);
TUNER_MSG MB86A22_MxL_TUNER_INIT( TUNER_PARAM *tuner_param, MB86A2x_BANDSEL BANDSEL );
static TUNER_MSG CH_to_freq( TUNER_PARAM *mytuner ) ;

#endif //_MB86A22_TUNER_H_

