/****************************************************************************
*
*  Front-end device driver for Fujitsu MB86A22 ISDB-T frontend.
*
*  Copyright (c) 2009 Fujitsu Microelectronics 
*  All rights reserved. 
*
*  Created 02/02/2009
*
****************************************************************************/
#define DEBUG 1

#include "../src/fe_priv.h"
#include "MB86A22_API_defs.h"
#include "MB86A22_user_define.h"
#ifndef _LINUX_FAPI
#include <fapi/drv_timer.h>
#include <fapi/sys_services.h>
#endif
#if defined (TMBSL_TDA182I2)

#include "TDA182I2\tmbslTDA18212.h"

#endif

/****************************************************************************
*   MB86A22 Timer function (Get GetTickPeriod)
*   Unit = 1
****************************************************************************/
UINT32 MB86A22_GetTickPeriod()
{
#ifndef _LINUX_FAPI
    return FAPI_TIMER_RESOLUTION_1_MSEC/1000;
#else
    return FAPEX_TIMER_RESOLUTION_1_MSEC/1000;	
#endif
}

/****************************************************************************
*   MB86A22 Timer function (Get Tick)
*   Unit = ms
****************************************************************************/
UINT32 MB86A22_GetTickTime()
{
#ifndef _LINUX_FAPI
    return FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC);
#else
    return FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
#endif
}

/****************************************************************************
*   MB86A22 Timer function
*   Unit = ms
****************************************************************************/
UINT32 MB86A22_wait( SINT32 wait_time )
{
    FAPI_SYS_SLEEP(wait_time);
    return 1;
}

/****************************************************************************
*   MB86A22 I2C Write Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A22_write( UINT8 OFDM_adr, UINT8 address , UINT8 value )
{
    feHandleT *hp;
    uint8_t regBuffer[2], index = 0;
    int32_t ferr;
    
    regBuffer[0] = (uint8_t)(address);
    regBuffer[1] = (uint8_t)(value & 0xff);
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( OFDM_adr == feHandleArray[index].i2cAddrDemod )
            break;
    }

    hp = &feHandleArray[index];
	  #ifndef _LINUX_FAPI    
    ferr = FE_I2C_Write( hp->i2cHandle, OFDM_adr, &regBuffer[0], 2 );
    #else
	  ferr = FE_I2C_Write( hp->i2cId, OFDM_adr, &regBuffer[0], 2 );
	  #endif    
    if(ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Write Demod I2C addr:0x%02x sub-address:0x%02x value:0x%02x error:0x%02x\n", OFDM_adr, address, value, ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   MB86A22 I2C Read Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A22_read( UINT8 OFDM_adr, UINT8 address , UINT8 *value )
{
    feHandleT *hp;
    uint8_t index = 0, temp;
    int32_t ferr;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( OFDM_adr == feHandleArray[index].i2cAddrDemod )
            break;
    }
    
    hp = &feHandleArray[index];
    
    temp = (uint8_t)(address&0xff);
	  #ifndef _LINUX_FAPI    
    ferr = FE_I2C_Read( hp->i2cHandle, OFDM_adr, &temp, 1, value, 1 );
    #else
	  ferr = FE_I2C_Read( hp->i2cId, OFDM_adr, &temp, 1, value, 1 );
	  #endif    
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Read Demod I2C addr:0x%02x sub-address:0x%02x value:0x%02x error:0x%02x\n", OFDM_adr, address, value, ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   Tuner Read Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A22_tuner_read( UINT8 Tu_adr, UINT8 bytes , UINT8 *value )
{
    int32_t ferr = 0;
    feHandleT *hp;
    uint8_t index = 0;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( Tu_adr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    
    hp = &feHandleArray[index];
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) == I2C_OK ) // I2C Repeater
    {
    	  #ifndef _LINUX_FAPI
        ferr = FE_I2C_Read( hp->i2cHandle, Tu_adr+1, &value[0], 1, &value[1], 1 );
	   		#else
			  ferr = FE_I2C_Read( hp->i2cId, Tu_adr+1, &value[0], 1, &value[1], 1 );
			  #endif        
    }
    else
        return I2C_NG;
    
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Read Tuner I2C error addr:0x%02x sub-address:0x%02x error:0x%02x\n", Tu_adr, value[0], ferr);
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}

/****************************************************************************
*   Tuner Write Access
*   Return value  OK:1, NG;0
****************************************************************************/
UINT32 MB86A22_tuner_write( UINT8 Tu_adr, UINT8 bytes , UINT8 *value )
{
    int32_t ferr = 0;
    feHandleT *hp;
    uint8_t index = 0;
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( Tu_adr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    
    hp = &feHandleArray[index];
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) == I2C_OK ) // I2C Repeater
    {
		    #ifndef _LINUX_FAPI    	
        ferr = FE_I2C_Write( hp->i2cHandle, Tu_adr, value, bytes );
    	  #else
        ferr = FE_I2C_Write( hp->i2cId, Tu_adr, value, bytes );	
		    #endif        
    }
    else
        return I2C_NG;
        
    if( ferr != FE_OK )
    {
    #ifdef DEBUG_FE
        switch(ferr)
        {
            case FAPI_I2C_ERR_BUSY:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_BUSY\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_BUS_OCCUP:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_BUS_OCCUP\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_SL_STOP_REC:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_SL_STOP_REC\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_RC_NACK:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_RC_NACK\n", Tu_adr, value[0]);
                break;
        
            case FAPI_I2C_ERR_BE_STOP:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_BE_STOP\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_BE_START:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_BE_START\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_AL_ADRSTA:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_AL_ADRSTA\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_AL_DATA:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_AL_DATA\n", Tu_adr, value[0]);
                break;
        
            case FAPI_I2C_ERR_SL_NACK:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_SL_NACK\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_MS_STOP_REC:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_MS_STOP_REC\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_AMDMA:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_AMDMA\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_UNSUPPORTED:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_UNSUPPORTED\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_GPIO_FAILED:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_GPIO_FAILED\n", Tu_adr, value[0]);
                break;
            
            case FAPI_I2C_ERR_TIMER_FAILED:
                FAPI_SYS_PRINT_MSG("Write Tuner I2C addr:0x%02x sub-address:0x%02x err = FAPI_I2C_ERR_TIMER_FAILED\n", Tu_adr, value[0]);
                break;
        
        }
        
    #endif
    }
    return ferr==FE_OK?I2C_OK:I2C_NG;
}
//#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
#ifdef TDA18211
/* below are only for NXP silicon tuner TDA18211/18271 */
SINT32 TDA182I1_write( UINT32 tuner_w_adr, UINT32 start_adr, UINT32 write_num, UINT32 *write_data)
{
    UINT8 i, tu_w_data[40] ; // here TDA18211/18271 has only 39 bytes register map
    int32_t err = 0;
    UINT8   RF_adr ;

    tu_w_data[0] = start_adr ;
    RF_adr = (UINT8)(0xFF&tuner_w_adr) ;
    
    for( i=1 ; i<=write_num ; i++ ) tu_w_data[i] = *(write_data+i-1) ;//*(write_data+i+start_adr-1) ;
    if( MB86A22_tuner_write( RF_adr, write_num+1, tu_w_data ) != I2C_OK ) err=1 ;

    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 TDA182I1_read( unsigned int tuner_r_adr, unsigned int start_adr, unsigned int read_num, unsigned int *read_data) 
{
    int err = 0;
    unsigned char ReadBuffer[40] = {0};
    unsigned int  uCounter;
    unsigned int*   pReadBuffer32;
    uint8_t tunerAddr,regBuffer;
    uint8_t index = 0;
    
    feHandleT *hp;
    tunerAddr = (uint8_t)(tuner_r_adr&0xff);
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( tunerAddr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    hp = &feHandleArray[index];
    
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    regBuffer = (uint8_t)(start_adr&0xFF);
    #ifndef _LINUX_FAPI
    err = FE_I2C_Write(hp->i2cHandle, tunerAddr,&regBuffer,1);
    #else
	  err = FE_I2C_Write(hp->i2cId, tunerAddr,&regBuffer,1);
	  #endif    
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    #ifndef _LINUX_FAPI
    err = FE_I2C_Read(hp->i2cHandle, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
    #else
	  err = FE_I2C_Read(hp->i2cId, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
	  #endif
    // copy from ReadBuffer32 -> read_data
    pReadBuffer32 = read_data;
    for (uCounter = 0; uCounter < read_num; uCounter++)
    {
        *pReadBuffer32 = (unsigned int)ReadBuffer[uCounter];
        pReadBuffer32 ++;
    }
        
    if( err==0 ) return 1 ;
    else         return 0 ;
}
#endif

SINT32 MB86A22_silicon_tuner_write_in_dword(UINT32 tuner_w_adr, UINT32 start_adr, UINT32 write_num, UINT32 *write_data)
{
    UINT8 i;
    // TDA182I1: 39-bytes register map
    // TDA182I2: 68-bytes register map
    // MAX3580:  16-bytes register map
    UINT8 tu_w_data[80];
    int32_t err = 0;
    UINT8   RF_adr ;

    tu_w_data[0] = start_adr ;
    RF_adr = (UINT8)(0xFF&tuner_w_adr) ;
    
    for( i=1 ; i<=write_num ; i++ ) tu_w_data[i] = *(write_data+i-1) ;
    if( MB86A22_tuner_write( RF_adr, write_num+1, tu_w_data ) != I2C_OK ) err=1 ;

    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 MB86A22_silicon_tuner_write_in_bytes(UINT32 tuner_w_adr, UINT32 start_adr, UINT32 write_num, UINT8 *write_data)
{
    UINT8 i;
    // TDA182I1: 39-bytes register map
    // TDA182I2: 68-bytes register map
    // MAX3580:  16-bytes register map
    UINT8 tu_w_data[80];
    int32_t err = 0;
    UINT8   RF_adr ;

    tu_w_data[0] = start_adr ;
    RF_adr = (UINT8)(0xFF&tuner_w_adr) ;
    
    for( i=1 ; i<=write_num ; i++ ) tu_w_data[i] = write_data[i-1];
    if( MB86A22_tuner_write( RF_adr, write_num+1, tu_w_data ) != I2C_OK ) err=1 ;

    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 MB86A22_silicon_tuner_read_in_dword(UINT32 tuner_r_adr, UINT32 start_adr, UINT32 read_num, UINT32 *read_data)
{
    int err = 0;
    // TDA182I1: 39-bytes register map
    // TDA182I2: 68-bytes register map
    // MAX3580:  16-bytes register map
    unsigned char ReadBuffer[80] = {0};
    unsigned int  uCounter;
    unsigned int*   pReadBuffer32;
    uint8_t tunerAddr,regBuffer;
    uint8_t index = 0;
    
    feHandleT *hp;
    tunerAddr = (uint8_t)(tuner_r_adr&0xff);
    
    /* get index by comparing demod I2C address */
    for( index = 0; index < FE_MAX_NUM; index++ )
    {
        if( tunerAddr == feHandleArray[index].i2cAddrTuner )
            break;
    }
    hp = &feHandleArray[index];
    
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    regBuffer = (uint8_t)(start_adr&0xFF);
    #ifndef _LINUX_FAPI
    err = FE_I2C_Write(hp->i2cHandle, tunerAddr,&regBuffer,1);
    #else
	  err = FE_I2C_Write(hp->i2cId, tunerAddr,&regBuffer,1);
	  #endif    
    if( MB86A22_write( hp->i2cAddrDemod, 0xFE, 0x00 ) != I2C_OK ){ err=1 ; } ;  //Open SCL2, SDA2
    #ifndef _LINUX_FAPI	
    err = FE_I2C_Read(hp->i2cHandle, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
    #else
	  err = FE_I2C_Read(hp->i2cId, tunerAddr+1,&regBuffer,1,(uint8_t*)ReadBuffer,read_num);
	  #endif    
    // copy from ReadBuffer32 -> read_data
    pReadBuffer32 = read_data;
    for (uCounter = 0; uCounter < read_num; uCounter++)
    {
        *pReadBuffer32 = (unsigned int)ReadBuffer[uCounter];
        pReadBuffer32 ++;
    }
        
    if( err==0 ) return 1 ;
    else         return 0 ;
}

SINT32 MB86A22_silicon_tuner_read_in_bytes(UINT32 tuner_r_adr, UINT32 start_adr, UINT32 read_num, UINT8 *read_data)
{
    int err = 0;
    uint8_t tunerAddr, regBuffer;
    uint8_t index = 0;
    
    feHandleT *hp;
    tunerAddr = (uint8_t)(tuner_r_adr&0xff);
    
    /* get index by comparing demod I2C address */
    for(index = 0; index < FE_MAX_NUM; index++)
    {
        if(tunerAddr == feHandleArray[index].i2cAddrTuner)
            break;
    }
    hp = &feHandleArray[index];
    
    //Open SCL2, SDA2
    if(MB86A22_write(hp->i2cAddrDemod, 0xFE, 0x00) != I2C_OK) err = 1;
    regBuffer = (uint8_t)(start_adr&0xFF);
    #ifndef _LINUX_FAPI
    err = FE_I2C_Write(hp->i2cHandle, tunerAddr, &regBuffer, 1);
    #else
	  err = FE_I2C_Write(hp->i2cId, tunerAddr,&regBuffer,1);
	  #endif    
    //Open SCL2, SDA2
    if(MB86A22_write(hp->i2cAddrDemod, 0xFE, 0x00) != I2C_OK) err = 1;
    #ifndef _LINUX_FAPI 	
    err = FE_I2C_Read(hp->i2cHandle, tunerAddr+1, &regBuffer, 1, read_data, read_num);
    #else
	  err = FE_I2C_Read(hp->i2cId, tunerAddr+1,&regBuffer,1,(uint8_t*)read_data,read_num);
	  #endif        
    if(err == 0)
        return 1;
    else
        return 0;
}

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_read
//* Object              : 
//* Input Parameters    :   tmUnitSelect_t tUnit
//*                         UInt32 AddrSize,
//*                         UInt8* pAddr,
//*                         UInt32 ReadLen,
//*                         UInt8* pData
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t TDA182I2_read(tmUnitSelect_t tUnit, UInt32 AddrSize, UInt8* pAddr,
UInt32 ReadLen, UInt8* pData)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/
    
    UINT32 start_adr;
    SINT32 ret;
    
    start_adr = (*pAddr)&0xFF;
    ret = MB86A22_silicon_tuner_read_in_bytes(feHandleArray[tUnit].i2cAddrTuner, 
                                              start_adr,
                                              ReadLen,
                                              pData);
    if(ret == 0)
        err = TDA182I2_ERR_HW_FAILED;
    
/* ...*/
/* End of Customer code here */

    return err;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_write
//* Object              : 
//* Input Parameters    :   tmUnitSelect_t tUnit
//*                         UInt32 AddrSize,
//*                         UInt8* pAddr,
//*                         UInt32 WriteLen,
//*                         UInt8* pData
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t TDA182I2_write (tmUnitSelect_t tUnit,     UInt32 AddrSize, UInt8* pAddr,
UInt32 WriteLen, UInt8* pData)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/
    UINT32 start_adr;
    SINT32 ret;
    ///int i;
    
    start_adr = (*pAddr)&0xFF;
    ret = MB86A22_silicon_tuner_write_in_bytes(feHandleArray[tUnit].i2cAddrTuner,
                                               start_adr,
                                               WriteLen,
                                               pData);
    
    ///#ifdef DEBUG_FE
    ///FAPI_SYS_PRINT_MSG("TDA182I2_write: tUnit=0x%02x AddrSize=0x%02x pAddr=0x%02x, WriteLen=0x%02x\n",tUnit,AddrSize,*pAddr,WriteLen);
    ///for(i=0;i<WriteLen;i++)
    ///{
    ///    FAPI_SYS_PRINT_MSG("pData[%d] = 0x%02x\n",i, pData[i]);
    ///}    
    ///#endif
    
    if(ret == 0)
        err = TDA182I2_ERR_HW_FAILED;
        
/* ...*/
/* End of Customer code here */

    return err;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_wait
//* Object              : 
//* Input Parameters    :   tmUnitSelect_t tUnit
//*                         UInt32 tms
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t TDA182I2_wait(tmUnitSelect_t tUnit, UInt32 tms)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/
    MB86A22_wait(tms);
/* ...*/
/* End of Customer code here */

    return err;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_mutexInit
//* Object              : 
//* Input Parameters    :   ptmbslFrontEndMutexHandle *ppMutexHandle
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t  TDA182I2_mutexInit(ptmbslFrontEndMutexHandle *ppMutexHandle)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/

/* ...*/
/* End of Customer code here */

    return err;
}


//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_mutexDeinit
//* Object              : 
//* Input Parameters    :    ptmbslFrontEndMutexHandle pMutex
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t  TDA182I2_mutexDeinit( ptmbslFrontEndMutexHandle pMutex)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/

/* ...*/
/* End of Customer code here */

    return err;
}



//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_mutexAcquire
//* Object              : 
//* Input Parameters    :   ptmbslFrontEndMutexHandle pMutex, UInt32 timeOut
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t  TDA182I2_mutexAcquire(ptmbslFrontEndMutexHandle pMutex, UInt32 timeOut)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/

/* ...*/
/* End of Customer code here */

    return err;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_mutexRelease
//* Object              : 
//* Input Parameters    :   ptmbslFrontEndMutexHandle pMutex
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t  TDA182I2_mutexRelease(ptmbslFrontEndMutexHandle pMutex)
{
    /* Variable declarations */
    tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/

/* ...*/
/* End of Customer code here */

    return err;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : TDA182I2_Print
//* Object              : 
//* Input Parameters    :   UInt32 level, const char* format, ...
//*                         
//* Output Parameters   : tmErrorCode_t.
//*--------------------------------------------------------------------------------------
tmErrorCode_t           TDA182I2_Print(UInt32 level, const char* format, ...)
{
   /* Variable declarations */
   tmErrorCode_t err = TM_OK;

/* Customer code here */
/* ...*/
    FAPI_SYS_PRINT_DEBUG(0, format);
/* ...*/
/* End of Customer code here */

   return err;
}

#endif
