/*

 Driver APIs for MxL136RF Tuner
 
 Copyright, Maxlinear, Inc.
 All Rights Reserved
 
 File Name:      MxL136RF_API.c
 
 Version:    1.1.0
*/


//#include "StdAfx.h"
#include "MxL136RF_API.h"
#include "MxL136RF_User_Define.h"
#include "stdio.h"



//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                                                                         //
//                          Tuner Functions                                //
//                                                                         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
MxL_ERR_MSG MxL136RF_Set_Register(MxL136RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char RegData)
{
    unsigned int Status=0;
    unsigned char pArray[2];
    pArray[0] = RegAddr;
    pArray[1] = RegData;
    Status = MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2);
    if(Status) return MxL_ERR_SET_REG;

    return MxL_OK;

}

MxL_ERR_MSG MxL136RF_Get_Register(MxL136RF_TunerConfigS* myTuner, unsigned char RegAddr, unsigned char *RegData)
{
    if(MxL136RF_I2C_Read((unsigned char)myTuner->I2C_Addr, RegAddr, RegData))
        return MxL_ERR_GET_REG;

    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Soft_Reset(MxL136RF_TunerConfigS* myTuner)
{
    unsigned char reg_reset;
    reg_reset = 0xFF;
    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, &reg_reset, 1))
        return MxL_ERR_OTHERS;

    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Loop_Through_On(MxL136RF_TunerConfigS* myTuner, MxL136RF_LoopThru isOn)
{   
    unsigned char pArray[2];    // a array pointer that store the addr and data pairs for I2C write 
    
    pArray[0]=0x04;
    if(isOn)
     pArray[1]= 0x01;
    else
     pArray[1]= 0x0;

    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
        return MxL_ERR_OTHERS;
    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Stand_By(MxL136RF_TunerConfigS* myTuner)
{
    unsigned char pArray[4];    // a array pointer that store the addr and data pairs for I2C write 
    
    pArray[0] = 0x01;
    pArray[1] = 0x0;
    pArray[2] = 0x0F;
    pArray[3] = 0x0;

    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 4))
        return MxL_ERR_OTHERS;

    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Wake_Up(MxL136RF_TunerConfigS* myTuner)
{
    unsigned char pArray[2];    // a array pointer that store the addr and data pairs for I2C write 
    
    pArray[0] = 0x01;
    pArray[1] = 0x01;

    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
        return MxL_ERR_OTHERS;

    if(MxL_Tuner_RFTune(myTuner, myTuner->RF_Freq_Hz, myTuner->BW_MHz))
        return MxL_ERR_RFTUNE;

    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Tuner_Init(MxL136RF_TunerConfigS* myTuner)
{   
    unsigned char pArray[MAX_ARRAY_SIZE];   // a array pointer that store the addr and data pairs for I2C write
    unsigned int Array_Size;                            // a integer pointer that store the number of element in above array

    //Soft reset tuner
    if(MxL_Soft_Reset(myTuner))
        return MxL_ERR_INIT;

    //perform initialization calculation
    if(MxL136RF_Init(pArray, &Array_Size, (unsigned char)myTuner->Mode, myTuner->IF_Diff_Out_Level, (unsigned int)myTuner->Xtal_Freq, (unsigned int)myTuner->IF_Freq, 
        (unsigned char)myTuner->IF_Spectrum, (unsigned char)myTuner->ClkOut_Setting, (unsigned char)myTuner->ClkOut_Amp))
    return MxL_ERR_INIT;
    //perform I2C write here
    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, Array_Size))
        return MxL_ERR_INIT;

    return MxL_OK;
}


MxL_ERR_MSG MxL136RF_Tuner_RFTune(MxL136RF_TunerConfigS* myTuner, unsigned int RF_Freq_Hz, MxL136RF_BW_MHz BWMHz)
{
    unsigned char pArray[MAX_ARRAY_SIZE];   // a array pointer that store the addr and data pairs for I2C write
    unsigned int Array_Size;                            // a integer pointer that store the number of element in above array

    //Store information into struc
    myTuner->RF_Freq_Hz = RF_Freq_Hz;
    myTuner->BW_MHz = BWMHz;

    //perform Channel Change calculation
    if(MxL136RF_RFTune(pArray,&Array_Size,RF_Freq_Hz,BWMHz))
        return MxL_ERR_RFTUNE;
    //perform I2C write here
    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, Array_Size))
        return MxL_ERR_RFTUNE;

    //wait for 3ms
    MxL136RF_Delay(3); 

    return MxL_OK;
}

MxL136RF_ChipVersion MxL136RF_Check_ChipVersion(MxL136RF_TunerConfigS* myTuner)
{   
    unsigned char Data;
    MxL136RF_ChipVersion version;
    
    if(MxL136RF_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD9, &Data))
        return MxL_GET_ID_FAIL;
    switch(Data)
    {
        case 0x14: 
            version = MxL_136RF_ID; 
            break;
        default: 
            version = MxL_UNKNOWN_ID;
    }

    return version;
}

MxL_ERR_MSG MxL136RF_RFSynth_Lock_Status(MxL136RF_TunerConfigS* myTuner, BOOL* isLock)
{   
    unsigned char Data;
    *isLock = FALSE; 
    if(MxL136RF_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD8, &Data))
        return MxL_ERR_OTHERS;
    Data &= 0x0C;
    if (Data == 0x0C)
        *isLock = TRUE;  //RF Synthesizer is Lock   
    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_REFSynth_Lock_Status(MxL136RF_TunerConfigS* myTuner, BOOL* isLock)
{
    unsigned char Data;
    *isLock = FALSE; 
    if(MxL136RF_I2C_Read((unsigned char)myTuner->I2C_Addr, 0xD8, &Data))
        return MxL_ERR_OTHERS;
    Data &= 0x03;
    if (Data == 0x03)
        *isLock = TRUE;   //REF Synthesizer is Lock
    return MxL_OK;
}

MxL_ERR_MSG MxL136RF_Set_IF_Output_Gain(MxL136RF_TunerConfigS* myTuner, unsigned int IF_Gain)
{
    unsigned char pArray[2];    // a array pointer that store the addr and data pairs for I2C write 
    unsigned char tempData;
    
    if(IF_Gain < 56 || IF_Gain > 63)
        return MxL_ERR_OTHERS;

    if(MxL136RF_I2C_Read((unsigned char)myTuner->I2C_Addr, 0x00, &tempData))
        return MxL_ERR_OTHERS;

    pArray[0]=0x00;
    pArray[1]= ((unsigned char)IF_Gain | (0xE0 & tempData));
    
    if(MxL136RF_I2C_Write((unsigned char)myTuner->I2C_Addr, pArray, 2))
        return MxL_ERR_OTHERS;

    return MxL_OK;
}

/*void MxL_OFMD_adr_init(unsigned char OFDM_addr)
{
     OFDM_adr = OFDM_addr;
}*/

