/*
 
 Driver APIs for MxL136RF Tuner
 
 Copyright, Maxlinear, Inc.
 All Rights Reserved
 
 File Name:      MxL_User_Define.c

 */

#include "MxL136RF_User_Define.h"
#include "MxL136RF_Common.h"

#include "../MB86A22_user_define.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                                                                         //
//                  I2C Functions (implement by customer)                  //
//                                                                         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
#include "../../src/fe_priv.h" //for gnuarm compiler

/******************************************************************************
**
**  Name: MxL_I2C_Write
**
**  Description:    I2C write operations
**
**  Parameters:     
**                  DeviceAddr  - MxL136RF Device address
**                  pArray      - Write data array pointer
**                  count       - total number of array
**
**  Returns:        0 if success
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   N/A   12-16-2007   khuang initial release.
**
******************************************************************************/
unsigned int MxL136RF_I2C_Write(unsigned char DeviceAddr, unsigned char* pArray, unsigned int count)
{
    unsigned char ret = 0;

    ret = (unsigned char)MB86A22_tuner_write(DeviceAddr,count,pArray);

    return ret == 1?0:1;
}

/******************************************************************************
**
**  Name: MxL_I2C_Read
**
**  Description:    I2C read operations
**
**  Parameters:     
**                  DeviceAddr  - MxL136RF Device address
**                  Addr        - register address for read
**                  *Data       - data return
**
**  Returns:        0 if success
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   N/A   12-16-2007   khuang initial release.
**
******************************************************************************/
unsigned int MxL136RF_I2C_Read(unsigned char DeviceAddr, unsigned char Addr, unsigned char* mData)
{
    unsigned char ret = 0;
    unsigned char i2cBuffer[2] = { 0x0 };
    
    i2cBuffer[0] = Addr;
    ret = MB86A22_tuner_read(DeviceAddr, 1+1, &i2cBuffer[0]);
    *mData = i2cBuffer[1];
    
    return ret == 1?0:1;
}

/******************************************************************************
**
**  Name: MxL_Delay
**
**  Description:    Delay function in milli-second
**
**  Parameters:     
**                  mSec        - milli-second to delay
**
**  Returns:        0
**
**  Revision History:
**
**   SCR      Date      Author  Description
**  -------------------------------------------------------------------------
**   N/A   12-16-2007   khuang initial release.
**
******************************************************************************/
void MxL136RF_Delay(unsigned int mSec)
{
    FE_Wait(mSec);
}
