/******************************************************************************
**
** \file      mb86a22.c
**
** \brief     Front-end device driver for Fujitsu MB86A22 ISDB-T frontend.
**
**            (C) FMC 2003 - 2010
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86a22.c 5874 2010-03-02 14:42:55Z wlaris $
**
******************************************************************************/
//#define DEBUG_FE 

///added for gnu arm compiler, to be confirmed for RVDS
#include "../src/fe_priv.h"
#include "stdio.h"

#include "MB86A22_API_version.h"
#include "MB86A22_API_defs.h"
#include "MB86A22_Tuner.h"

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
#include "TDA182I1\tmtypes.h"
#include "TDA182I1\SystemFuncURT.h"
#include "TDA182I1\tmbsltda182i1local.h"
#include "TDA182I1\tmbslTDA182I1StaticInstance.h"
#include "TDA182I1\tmbslTuner.h"
#include "TDA182I1\tmbslTDA18271.h"
#include "TDA182I1\tmbslTDA182I1.c"
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
#include "TDA182I2\tmNxTypes.h"
#include "TDA182I2\tmbslFrontEndTypes.h"
#include "TDA182I2\tmbsltda182i2local.h"
#include "TDA182I2\tmbslTDA182I2Instance.h"
#include "TDA182I2\tmbslTDA18212.h"
#include "TDA182I2\tmbslTDA182I2.c"
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
#include "Mxl136RF/MxL136RF_API.h"
#include "Mxl136RF/MxL136RF_Common.h"
#endif

#include "MB86A22_API_structs.h"
#include "MB86A22_Tuner_default_setting.h"
#include "MB86A22_API_version.h"
#include "MB86A22_user_define.h"
#include "MB86A22_API.h"
#include "MB86A22_print.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_services.h>
#include <fapi/drv_gpio.h>
#else
#include "fapex/abst_libgpio.h"
#endif

/*---------------------------------------------------------------------------*/
/* local variables                                                           */
/*---------------------------------------------------------------------------*/

static feHandleT *pData[FE_MAX_NUM];
static MB86A2x_VERSION_INFO  version_info[FE_MAX_NUM];     // prepare pointer(1/8)
static MB86A2x_PARAM         feDat[FE_MAX_NUM];            // prepare pointer(2/8)
static MB86A2x_LOCKNG_INFO   locking_info[FE_MAX_NUM];     // prepare pointer(3/8)
static MB86A2x_AGC_INFO      agc_info[FE_MAX_NUM];         // prepare pointer(4/8)
//static MB86A2x_FREQ_ERR_INFO freq_err_info[FE_MAX_NUM];    // prepare pointer(5/8)
static MB86A2x_TMCC_INFO     tmcc_info[FE_MAX_NUM];        // prepare pointer(6/8)
static MB86A2x_SIG_QUAL_INFO sign_qual_info[FE_MAX_NUM];   // prepare pointer(7/8)
static MB86A2x_CH_SCH_INFO   ch_sch_info[FE_MAX_NUM];      // prepare pointer(8/8)
static MB86A22_tuner_type_t mb86a22TunerType[FE_MAX_NUM] = { MB86A22_NO_TUNER };

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
static tmbslTuParam_t Param;
static tmhalFEBslSystemFunc_t thesystemFunc;
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
static tmbslFrontEndDependency_t sSrvFunc;   /* setup parameters */
#endif

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t MB86A22_Open(uint8_t index, FE_HANDLE *pDeviceData);
int32_t MB86A22_Close(uint8_t index);
int32_t MB86A22_GetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t MB86A22_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t MB86A22_Reset(uint8_t index);
int32_t MB86A22_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t MB86A22_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t MB86A22_GetAFC(uint8_t index, int32_t* pAFC);
int32_t MB86A22_GetBER(uint8_t index, uint32_t* pBER);
int32_t MB86A22_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t MB86A22_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t MB86A22_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);

void mb86a22SetFunctions(feHandleT *hp)
{
    // register APIs to handle
    hp->systemType             = FE_SYSTEM_TYPE_TERRESTRIAL;
    hp->feOpenFct              = MB86A22_Open;
    hp->feCloseFct             = MB86A22_Close;
    hp->feGetSystemParamsFct   = MB86A22_GetSystemParams;
    hp->feSetSystemParamsFct   = MB86A22_SetSystemParams;
    hp->feResetFct             = MB86A22_Reset;
    hp->feTuneFct              = MB86A22_Tune;
    hp->feScanFct              = NULL;
    hp->feSendDiseqcCommandFct = NULL;
    hp->feGetStatusFct         = MB86A22_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = MB86A22_GetAFC;
    hp->feGetBERFct            = MB86A22_GetBER;
    hp->feGetSignalQualityFct  = MB86A22_GetSignalQuality;
    hp->feGetSignalStrengthFct = MB86A22_GetSignalStrength;
    hp->feGetTuneParamsFct     = MB86A22_GetTuneParams;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
}

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
/* TDA182I1 silicon tuner */
int32_t MB86A22_TDA182I1_Detect( uint8_t slot, feHandleT* hp )
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;

    // set Demodulator I2C address
    if(slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if(slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    MB86A22_Reset(slot);

    // read chip ID
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    if( ferr == FE_OK )
    {
    		FAPI_SYS_PRINT_MSG("Chip ID:0x%x\n",i2cBuffer[1]);
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if(i2cBuffer[1] == 0x23)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if(i2cBuffer[1] == 0x13)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
    		FAPI_SYS_PRINT_MSG("FE_I2C_Read error\n");
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }

    // set RF I2C address by pinging different addresses
    for(i = 0; i < 5; i++, rf_addr += 2)
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        ferr = FE_I2C_Write(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
        if(ferr != FE_OK)
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %d",__FILE__,__LINE__,ferr);
            break;
        }
        i2cBuffer[0] = 0x55;
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2);
        if(ferr == FE_OK)
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("TDA18211 with RF addr 0x%2x detected\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if(ferr != FE_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_TDA18211;

    return FE_OK;
}
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
int32_t MB86A22_TDA182I2_Detect(uint8_t slot, feHandleT* hp)
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;

    // set Demodulator I2C address
    if(slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if(slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    MB86A22_Reset(slot);

    // read chip ID
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    if(ferr == FE_OK)
    {
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if(i2cBuffer[1] == 0x23)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if(i2cBuffer[1] == 0x13)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }

    // set RF I2C address by pinging different addresses
    for(i = 0; i < 5; i++, rf_addr += 2)
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        ferr = FE_I2C_Write(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
        if(ferr != FE_OK)
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %d",__FILE__,__LINE__,ferr);
            break;
        }
        i2cBuffer[0] = 0x55;
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2);
        if(ferr == FE_OK)
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("TDA182I2 with RF addr 0x%2x detected\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if(ferr != FE_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_TDA18212;

    return FE_OK;
}
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
int32_t MB86A22_TDA182I2_Detect(uint8_t slot, feHandleT* hp)
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;

    // set Demodulator I2C address
    if(slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if(slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }
    // hardware reset
    (void) MB86A22_Reset(slot);

    // read chip ID
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    if(ferr == FE_OK)
    {
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if(i2cBuffer[1] == 0x23)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if(i2cBuffer[1] == 0x13)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }

    // set RF I2C address by pinging different addresses
    for(i = 0; i < 5; i++, rf_addr += 2)
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        ferr = FE_I2C_Write( hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
        if(ferr != FE_OK)
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %d",__FILE__,__LINE__,ferr);
            break;
        }

        i2cBuffer[0] = 0x55;
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2);
        if(ferr == FE_OK)
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("NI386x with RF addr 0x%2x detected\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if(ferr != FE_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_NI386X;

    return FE_OK;
}
#endif

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
/* Gospell silicon tuner */
int32_t MB86A22_MAX3580_Detect(uint8_t slot, feHandleT* hp)
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xC0, i;
    int32_t ferr;

    // set Demodulator I2C address
    if(slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if(slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    MB86A22_Reset(slot);

    // read chip ID
    #ifndef _LINUX_FAPI
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #else
    ferr = FE_I2C_Read(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #endif
    if(ferr == FE_OK)
    {
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }

    // set RF I2C address by pinging different addresses
    for(i = 0; i < 5; i++, rf_addr += 2)
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
       #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
       #else
        ferr = FE_I2C_Write( hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 2 );
       #endif
        if( ferr != FE_OK )
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %d",__FILE__,__LINE__,ferr);
            break;
        }
        i2cBuffer[0] = 0x55;
        #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2);
        #else
        ferr = FE_I2C_Write(hp->i2cId, rf_addr, &(i2cBuffer[0]), 2 );
        #endif
        if(ferr == FE_OK)
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("MAX3580 with RF addr 0x%2x detected\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if(ferr != FE_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_MAX3580;


    return FE_OK;
}
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
/* Maxlinear silicon tuner */
int32_t MB86A22_MXL136RF_Detect( uint8_t slot, feHandleT* hp )
{
    //uint8_t i;
    uint8_t i2cBuffer[2] = { 0x00, 0xFF };
    int32_t ferr;

    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    MB86A22_Reset(slot);

    // read chip ID
    #ifndef _LINUX_FAPI
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #else
    ferr = FE_I2C_Read(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #endif
    if( ferr == FE_OK )
    {
    		FAPI_SYS_PRINT_MSG("MB86A22 Chip ID:0x%x\n",i2cBuffer[1]);
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if( i2cBuffer[1] == 0x53 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    // check RF chip id
    hp->i2cAddrTuner = MB86A22_MXL136RF_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_MXL136RF;

    return FE_OK;
}
#endif

#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
/* FCI silicon tuner */
int32_t MB86A22_FC2581_Detect(uint8_t slot, feHandleT* hp)
{
    uint8_t i2cBuffer[2] = { 0x00, 0xFF }, rf_addr = 0xA8, i;
    int32_t ferr;

    // set Demodulator I2C address
    if (slot == 0)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_24; //ADR1=0, ADR0=0
    }
    else if (slot == 1)
    {
        hp->i2cAddrDemod = MB86A2x_DEV_ADR_20; //ADR1=1, ADR0=0
    }
    else
    {
        return (FE_ERR_BAD_PARAMETER);
    }

    // hardware reset
    MB86A22_Reset(slot);

    // read chip ID
    #ifndef _LINUX_FAPI
    ferr = FE_I2C_Read(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #else
    ferr = FE_I2C_Read(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 1, &(i2cBuffer[1]), 1);
    #endif
    if( ferr == FE_OK )
    {
        /* the Driver suppose to be compatible for MB86A22 and MB86A20S */
        if( i2cBuffer[1] == 0x23 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A22 Detected\n");
        #endif
        }
        else if( i2cBuffer[1] == 0x13 )
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: MB86A20S Detected\n");
        #endif
        }
        else
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
            return FE_ERR_FE_UNKNOWN_DEVICE;
        }
    }
    else
    {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("ISDB-T Demodulator: Unknown Device\n");
        #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    // set RF I2C address by pinging different addresses
    for(i = 0; i < 5; i++, rf_addr += 2)
    {
        // I2C Repeater
        i2cBuffer[0] = 0xFE;
        i2cBuffer[1] = 0x00;
        #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write(hp->i2cHandle, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
        #else
        ferr = FE_I2C_Write(hp->i2cId, hp->i2cAddrDemod, &(i2cBuffer[0]), 2);
        #endif
        if(ferr != FE_OK)
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %s",__FILE__,__LINE__,FAPI_SYS_GetErrorString(ferr));
            FAPI_SYS_PRINT_DEBUG(1,"[%s l:%i] FE_I2C_Write() failed: %d",__FILE__,__LINE__,ferr);
            break;
        }
        i2cBuffer[0] = 0x55;
        #ifndef _LINUX_FAPI
        ferr = FE_I2C_Write(hp->i2cHandle, rf_addr, &(i2cBuffer[0]), 2);
        #else
        ferr = FE_I2C_Write(hp->i2cId, rf_addr, &(i2cBuffer[0]), 2);
        #endif
        if(ferr == FE_OK)
        {
            hp->i2cAddrTuner = rf_addr;
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG("FC2581 with RF addr 0x%2x detected\n", hp->i2cAddrTuner);
        #endif
            break;
        }
    }
    if(ferr != FE_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG("Fujitsu ISDB-T Demodulator: Failed to Ping RF\n");
    #endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    mb86a22SetFunctions(hp);

    hp->slotNum  = slot;
    mb86a22TunerType[slot] = MB86A22_ISDBT_FC2581;
    return FE_OK;
}
#endif

int32_t MB86A22_Detect( uint8_t slot, feHandleT* hp )
{
    hp = hp;

    //-- check supported indices --
    if(slot > 1)
        return FE_ERR_FE_UNKNOWN_DEVICE;

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
    if(MB86A22_MXL136RF_Detect(slot, hp) == FAPI_OK)        
        return FE_OK;   
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    if(MB86A22_TDA182I1_Detect(slot, hp) == FAPI_OK)
        return FE_OK;
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    if(MB86A22_TDA182I2_Detect(slot, hp) == FAPI_OK)
        return FE_OK;
#endif

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
    if(MB86A22_MAX3580_Detect(slot, hp) == FAPI_OK)
        return FE_OK;
#endif  
    
#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
    if(MB86A22_FC2581_Detect(slot, hp) == FAPI_OK)
        return FE_OK;
#endif
        return FE_ERR_FE_UNKNOWN_DEVICE;
}

int32_t MB86A22_Open(uint8_t index, FE_HANDLE* pDeviceData)
{    
    #if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    // tuner declaration & init
    int32_t i2cerr = 0;
    tmUnitSelect_t TunerUnit = index;
    #endif
    #if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    tmUnitSelect_t TunerUnit = index;
    tmErrorCode_t err = TM_OK;
    #endif

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    pData[index] = &feHandleArray[index];
    *pDeviceData = (FE_HANDLE)(&feHandleArray[index]);
    feDat[index].nim_param.TUNER_TYPE = mb86a22TunerType[index];

    //Enter device address to the pointer.
    MB86A22_Init_Adr(&feDat[index]);

    //Check version information.
    MB86A22_Get_Version(&feDat[index].nim_param, &version_info[index]);
#ifdef DEBUG_FE
    MB86A22_printf_version(&version_info[index]) ;
#endif

    //Enter initialization parameters after power on to the pointer.
    MB86A22_Init_Param(&feDat[index]);
    feDat[index].nim_param.TUNER_TYPE = mb86a22TunerType[index];

    //Initialize RF specially for silicon tuner
    #if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    
    //Initialize the tuner system function 
    thesystemFunc.SY_GetTickTime    = &MB86A22_GetTickTime ;
    thesystemFunc.SY_GetTickPeriod  = &MB86A22_GetTickPeriod ;
    thesystemFunc.SY_GetCommErrors  = NULL ;
    thesystemFunc.SY_Write          = &MB86A22_silicon_tuner_write_in_dword ;
    thesystemFunc.SY_WriteBit       = NULL ;
    thesystemFunc.SY_Read           = &MB86A22_silicon_tuner_read_in_dword ;
    thesystemFunc.SY_TimerWait      = &MB86A22_wait ;

    Param.uHwAddress = (feDat[index].nim_param.MB86A2x_Adr<<8) | 0xc0 ;    //TDA18271 Device address=0xc0
    Param.systemFunc = thesystemFunc;
    i2cerr = tmbslTDA18271Init(TunerUnit, Param);
    if(i2cerr >= 1)
        return FE_ERR_FE_I2C;

    i2cerr = tmbslTDA18271SetConfig(TunerUnit, FEINIT, 0);
    if(i2cerr >= 1)
        return FE_ERR_FE_I2C;
    #endif
    
    #if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    sSrvFunc.sIo.Write              = TDA182I2_write;
    sSrvFunc.sIo.Read               = TDA182I2_read;
    sSrvFunc.sTime.Get              = NULL;
    sSrvFunc.sTime.Wait             = TDA182I2_wait;
    sSrvFunc.sDebug.Print           = NULL;
    sSrvFunc.sMutex.Init            = TDA182I2_mutexInit;
    sSrvFunc.sMutex.DeInit          = TDA182I2_mutexDeinit;
    sSrvFunc.sMutex.Acquire         = TDA182I2_mutexAcquire;
    sSrvFunc.sMutex.Release         = TDA182I2_mutexRelease;
    sSrvFunc.dwAdditionalDataSize   = 0;
    sSrvFunc.pAdditionalData        = NULL;

    // Master TDA182I2 + MB86A22
    err = tmbslTDA182I2Init(TunerUnit, &sSrvFunc);
    if(err != TM_OK)
        return FE_ERR_FE_OPEN;

    // Hardware power state
    err = tmbslTDA182I2SetPowerState(TunerUnit, tmTDA182I2_PowerNormalMode);
    if(err != TM_OK)
        return FE_ERR_FE_OPEN;

    // Hardware initialization
    /* Determine if we need to wait in Reset function */
    err = tmbslTDA182I2SetIRQWait(TunerUnit, True);
    if(err == TM_OK)
    {
        err = tmbslTDA182I2SetPowerState(TunerUnit, tmTDA182I2_PowerNormalMode);
    }
    if(err == TM_OK)
    {
        err = tmbslTDA182I2Reset(TunerUnit);
    }
    if(err == TM_OK)
    {
        /* Set TDA18212 power state to standby */
        err = tmbslTDA182I2SetPowerState(TunerUnit, tmTDA182I2_PowerStandbyWithLNAOnAndWithXtalOn);
    }
    if(err != TM_OK)
        return FE_ERR_FE_OPEN;

    #endif

    return FE_OK;
}

int32_t MB86A22_Close(uint8_t index)
{
    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    // TODO: For TDA1821I1/TDA182I2, power down mode could be set here
    return FE_OK;
}

int32_t MB86A22_GetSystemParams(uint8_t index,FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    memcpy(pSystemParams, &(pData[index]->pSystemParams), sizeof(FE_SYSTEM_PARAMS_S));

    return FE_OK;
}

int32_t MB86A22_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    memcpy(&(pData[index]->pSystemParams), pSystemParams, sizeof(FE_SYSTEM_PARAMS_S));

    return FE_OK;
}

int32_t MB86A22_Reset(uint8_t index)
{
    int32_t error;

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    error = FE_ControlFeReset(index, FE_RESET_PULSE);
    if(error != FE_OK)
    {
        return error;
    }
    FAPI_SYS_SLEEP(40);
}

int32_t MB86A22_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    MB86A2x_ERR_MSG       ERR_MSG ;
    UINT8 IFlevel_value = 0x07; // IF_Level -12 dB
    UINT32 freq ;            //channel frequency[kHz]
    UINT32 IFLEVEL      = 362;
    
    #if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    // tuner declaration & init
    int32_t i2cerr = 0;
    tmUnitSelect_t TunerUnit = index;
    #endif
    #if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    tmUnitSelect_t TunerUnit = index;
    tmErrorCode_t err = TM_OK;
    #endif
	
	//MB86A22_Reset(index);
	//FAPI_SYS_PRINT_MSG("!!!!!!!!!!reset\n");
    MB86A2x_RECEP_LAYER recep_layer ;   //Recption layer
                                        //This value should be one of below values.
                                        //MB86A2x_RECEP_LAYER_A : layer-A reception
                                        //MB86A2x_RECEP_LAYER_B : layer-B reception
                                        //MB86A2x_RECEP_LAYER_C : layer-C reception
                                        //MB86A2x_RECEP_HIGHEST : highest layer reception

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18211)
    {
        i2cerr = tmbslTDA18271SetStandardMode(TunerUnit, tmDigital_TV_ISDBT_6MHz); // IF : 4.0 MHz ; IF_level : 0dB
        if(i2cerr >= 1)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG( "I2C ERR\n" ) ;
        #endif
            return FE_ERR_FE_I2C;
         }
        feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_TDA182I1_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif
#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18212)
    {
        // Hardware power state
        err = tmbslTDA182I2SetPowerState(TunerUnit, tmTDA182I2_PowerNormalMode);
        if(err != TM_OK)
            return FE_ERR_FE_OPEN;

        //Now only 6MHz for ISDB-T is supported
        err = tmbslTDA182I2SetStandardMode(TunerUnit, tmTDA182I2_ISDBT_6MHz);
        if(err != TM_OK)
        {
        #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG( "I2C ERR\n" ) ;
        #endif
            return FE_ERR_FE_I2C;
      }
      feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_TDA182I2_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif
#if (MB86A22_TUNER_SUPPORT_NI386X > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_NI386X)
    {
        feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_NI386X_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_MAX3580)
    {
        feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_Max3580_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_MXL136RF)
    {
        feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_MXL136RF_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
    if(feDat[index].nim_param.TUNER_TYPE == MB86A22_ISDBT_FC2581)
    {
        feDat[index].nim_param.tuner_param.IF_freq_kHz = MB86A22_FC2581_PARAM_TABLE.nim_param.tuner_param.IF_freq_kHz;
    }
#endif

    /*  prepare some other initial values for bandwidth control       */
    switch(pTuneParams->terr.bandwidth)
    {
            FAPI_SYS_PRINT_MSG( "bandwidth ERR:%d\n" , pTuneParams->terr.bandwidth);
        case FE_BANDWIDTH_6_MHZ:
            //feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_DVBT_6MHz;
            feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_ISDBT_6MHz;
            break;
        case FE_BANDWIDTH_7_MHZ:
            feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_DVBT_7MHz;
            break;
        case FE_BANDWIDTH_8_MHZ:
            feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_DVBT_8MHz;
            break;
        case FE_BANDWIDTH_ISDB_6_MHZ:
            feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_ISDBT_6MHz;
            break;
        default:
    #ifdef DEBUG_FE
            FAPI_SYS_PRINT_MSG( "bandwidth ERR:%d\n" , pTuneParams->terr.bandwidth);
            feDat[index].pll_param.BANDSEL = MB86A2x_BANDSEL_ISDBT_6MHz;
    #endif
            break;
    } /* end of switch(pTuneParams->terr.bandwidth) */
    #ifdef DEBUG_FE
    FAPI_SYS_PRINT_MSG( "FE-band:%d, A2x-BANDSEL:%d,freq=%d\n" ,pTuneParams->terr.bandwidth , feDat[index].pll_param.BANDSEL,pTuneParams->terr.frequency);
    #endif

    //Write initialization parameters after power on.
    ERR_MSG = MB86A22_Set_Init( &feDat[index] ,1) ;
    if(ERR_MSG != MB86A2x_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Set_Init ERR\n" );
    #endif
        return FE_ERR_FE_I2C;
    }

    //Set reception frequency here.
    freq = pTuneParams->terr.frequency / 1000;
    //Set reception segment here.
    recep_layer = MB86A2x_RECEP_HIGHEST;

    //MB86A22 reception procedure(1/2)
    ERR_MSG = MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_1_freq(
        &feDat[index], &locking_info[index],
        &sign_qual_info[index], &version_info[index],
        index, IFLEVEL, IFlevel_value,
        freq ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_1_freq ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }

    //MB86A22 reception procedure(2/2)
    ERR_MSG = MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_2(
            &feDat[index], &locking_info[index], &sign_qual_info[index], &tmcc_info[index], &ch_sch_info[index], &version_info[index], recep_layer ) ;
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_2 ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }

    //set the timeout waiting for tuner locking
    FE_SetTuneTimeout(index, 0xFFFFFFFF);

    return FE_OK;
}

int32_t MB86A22_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    MB86A2x_ERR_MSG ERR_MSG;

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    if(FE_IsTuneTimeout(index) == 0)
    {
        *pStatus = FE_STATUS_TUNING;
        return FE_OK;
    }

    // for debug only, to be removed later
    ///#if defined (FC2581)
    ///FAPI_SYS_PRINT_MSG("FX2581 addr 0x71 = 0x%08x\n", fc2581_i2c_read(0xac, 0x71));
    ///FAPI_SYS_PRINT_MSG("FX2581 addr 0x72 = 0x%08x\n", fc2581_i2c_read(0xac, 0x72));
    ///FAPI_SYS_PRINT_MSG("FX2581 addr 0x73 = 0x%08x\n", fc2581_i2c_read(0xac, 0x73));
    ///FAPI_SYS_PRINT_MSG("FX2581 addr 0x74 = 0x%08x\n", fc2581_i2c_read(0xac, 0x74));
    ///#endif

    ERR_MSG = MB86A22_Get_SEQ_value( &feDat[index].nim_param, &locking_info[index] );
    if(ERR_MSG != MB86A2x_OK)
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_SEQ_value ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    else
    {
		#ifdef DEBUG_FE
        	FAPI_SYS_PRINT_MSG( "locking_info.STATE: %d\n",locking_info[index].STATE) ;
    	#endif
        if(locking_info[index].STATE >= 8)
        {
            *pStatus = FE_STATUS_LOCKED;
        }
        else
        {
            if(FE_IsTuneTimeout(index) == 0)
            {
                *pStatus = FE_STATUS_TUNING;
            }
            else
            {
                *pStatus = FE_STATUS_NOT_LOCKED;
            }
        }
    }
    return FE_OK;
}

int32_t MB86A22_GetAFC(uint8_t index, int32_t* pAFC)
{
    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    if(pAFC == NULL) return FE_ERR_BAD_PARAMETER;

    *pAFC = 0;

    // still open
    return FE_OK;
}

int32_t MB86A22_GetBER(uint8_t index, uint32_t* pBER)
{
    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    // TODO
    return FE_OK;
}

int32_t MB86A22_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    MB86A2x_ERR_MSG ERR_MSG;

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    ERR_MSG = MB86A22_Get_CN_Flag( &feDat[index].nim_param, &sign_qual_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_CN_Flag ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    else
    {
        if( sign_qual_info[index].CN_update == 1 )
        {
            //Check whether whether C/N measurement is finished or not by checking the value of CN_update.
            //Read register values to know C/N value and calculate estimated C/N.
            ERR_MSG = MB86A22_Get_CN_Value( &feDat[index].nim_param, &sign_qual_info[index] );
            if( ERR_MSG != MB86A2x_OK )
            {
            #ifdef DEBUG_FE
                FAPI_SYS_PRINT_MSG( "MB86A2x_Get_CN_Value ERR\n" ) ;
            #endif
                return FE_ERR_FE_I2C;
            }

            //Reset C/N counter
            ERR_MSG = MB86A22_Reset_CN_Counter( &feDat[index].nim_param, &feDat[index].mon_tim_param );
            if( ERR_MSG != MB86A2x_OK )
            {
            #ifdef DEBUG_FE
                FAPI_SYS_PRINT_MSG( "MB86A2x_Reset_CN_Counter ERR\n" ) ;
            #endif
                return FE_ERR_FE_I2C;
            }
        }
        *pQuality = (int16_t)(sign_qual_info[index].CN_current) * 2184;
    }
    return FE_OK;
}

int32_t MB86A22_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    MB86A2x_ERR_MSG ERR_MSG;

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    ERR_MSG = MB86A22_Get_IF_level( &feDat[index].nim_param, &agc_info[index] );
    if( ERR_MSG != MB86A2x_OK )
    {
    #ifdef DEBUG_FE
        FAPI_SYS_PRINT_MSG( "MB86A2x_Get_IF_level ERR\n" ) ;
    #endif
        return FE_ERR_FE_I2C;
    }
    *pStrength = agc_info[index].IF_INPUT_LEVEL * 256;
    return FE_OK;
}

int32_t MB86A22_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{
    FE_STATUS_E lockStatus = FE_STATUS_NOT_LOCKED;

    if(index > FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    /* check if is locked */
    MB86A22_GetStatus(index, &lockStatus);
    if(lockStatus == FE_STATUS_LOCKED)
    {
        /* get ISDB-T configuration and map into DVB structure */
        pTuneParams->terr.frequency = feDat[index].nim_param.tuner_param.RF_freq_kHz * 1000 ;

        /* currently driver only support 6Mhz, so fixed here */
        pTuneParams->terr.bandwidth = FE_BANDWIDTH_6_MHZ ;

        /* Because ISDB-T could have multi-layers in one frequency, below fixed as auto */
        pTuneParams->terr.codeRateHP    = FE_FEC_AUTO ;
        pTuneParams->terr.codeRateLP    = FE_FEC_AUTO ;
        pTuneParams->terr.modulation    = FE_MOD_NONE ;
        pTuneParams->terr.transmission  = FE_TRANS_MODE_AUTO ;
        pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_AUTO ;
        pTuneParams->terr.hierarchy     = FE_HIERARCHY_AUTO ;
        pTuneParams->terr.iq            = FE_IQ_AUTO ;

        return FE_OK;
    }
    else
    {
        return FE_STATUS_NOT_LOCKED;
    }
}
