    AREA ARM1176CODE, CODE, READONLY
    PRESERVE8

OFFSET_X EQU 0
OFFSET_Y EQU 2
OFFSET_W EQU 4
OFFSET_H EQU 6
OFFSET_C EQU 8

;*******************************************************************************
; void OSD_Fill8bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst, uint32_t layerStride);
;*******************************************************************************
    EXPORT OSD_Fill8bpp
OSD_Fill8bpp PROC
    ldrh    r12,[r1,#OFFSET_X]     ; load x
    ldrb    r3, [r1,#OFFSET_C]     ; load c
    ands    r12,r12,#3             ; is destination word-aligned?
    orr     r3,r3,r3,lsl#8         ; duplicate c
    orr     r3,r3,r3,lsl#16        ; quadruplicate c
    beq     OSD_Fill_w

    ;; fill left stripe first if the destination
    ;; is not 32-bit aligned
    cmp     r12,#2
    push    {r4,r5}
    ldrh    r4, [r1,#OFFSET_Y]     ; load y
    ldrh    r5, [r1,#OFFSET_H]     ; load h
    mla     r12,r4,r2,r0           ; r12 = line start address
    ldrh    r4, [r1,#OFFSET_X]     ; load x
    add     r12,r4                 ; r12 = start address
    beq     byte_fill2             ; take the loop for (dst&3) == 2
    bhi     byte_fill3             ; take the loop for (dst&3) == 3
byte_fill1
    pld     [r12,+r2,lsl#2]        ; cache hint 4 lines in advance
    strh    r3,[r12,#-1]
    strb    r3,[r12,#+1]
    subs    r5,r5,#1
    add     r12,r2
    bhi     byte_fill1
    mov     r5, #3
    b       setAlignB

byte_fill2
    pld     [r12,+r2,lsl#2]        ; cache hint 4 lines in advance
    strh    r3,[r12,#-2]
    subs    r5,r5,#1
    add     r12,r2
    bhi     byte_fill2
    mov     r5, #2
    b       setAlignB

byte_fill3
    pld     [r12,+r2,lsl#2]        ; cache hint 4 lines in advance
    strb    r3,[r12,#-3]
    subs    r5,r5,#1
    add     r12,r2
    bhi     byte_fill3
    mov     r5, #1

    ;; adjust width and x to get a new rectangle
    ;; with the left border 32-bit aligned
setAlignB
    ldrh    r12,[r1,#OFFSET_W]     ; load width
    add     r4, r5                 ; adjust x to aligned position
    strh    r4, [r1,#OFFSET_X]     ; store aligned x
    sub     r12,r5                 ; adjust width
    strh    r12,[r1,#OFFSET_W]     ; store adjusted width
    pop     {r4,r5}
    b       OSD_Fill_w

    ENDP

;*******************************************************************************
; void OSD_Fill16bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                    uint32_t layerStride);
;*******************************************************************************
    EXPORT OSD_Fill16bpp
OSD_Fill16bpp PROC
    ldrh    r12,[r1,#OFFSET_X]     ; load x
    ldrh    r3, [r1,#OFFSET_C]     ; load c
    mov     r12,r12, lsl #1
    strh    r12,[r1,#OFFSET_X]     ; store x*2
    ands    r12,r12,#2             ; is destination word-aligned?
    ldrh    r12,[r1,#OFFSET_W]     ; load w
    orr     r3,r3,r3,lsl#16        ; duplicate c
    str     r3, [r1,#OFFSET_C]     ; store c
    mov     r12,r12, lsl #1
    strh    r12,[r1,#OFFSET_W]     ; store w*2
    beq     OSD_Fill_w

    ;; fill left stripe first if the destination
    ;; is not 32-bit aligned
    push    {r4,r5}
    ldrh    r4, [r1,#OFFSET_Y]     ; load y
    ldrh    r5, [r1,#OFFSET_H]     ; load h
    mla     r12,r4,r2,r0           ; r12 = line start address
    ldrh    r4, [r1,#OFFSET_X]     ; load x
    add     r12,r4                 ; r12 = start address
half_fill
    pld     [r12,+r2,lsl#2]        ; cache hint 4 lines in advance
    strh    r3,[r12]
    subs    r5,r5,#1
    add     r12,r2
    bhi     half_fill

    ;; adjust width and x to get a new rectangle
    ;; with the left border 32-bit aligned
    ldrh    r12,[r1,#OFFSET_W]     ; load width
    add     r4, #2                 ; adjust x to aligned position
    strh    r4, [r1,#OFFSET_X]     ; store aligned x
    sub     r12,#2                 ; adjust width
    strh    r12,[r1,#OFFSET_W]     ; store adjusted width
    pop     {r4,r5}
    b       OSD_Fill_w

    ENDP

;*******************************************************************************
; void OSD_Fill32bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                    uint32_t layerStride);
;*******************************************************************************
    EXPORT OSD_Fill32bpp
OSD_Fill32bpp PROC
    ldrh    r3, [r1,#OFFSET_X]     ; load x
    ldrh    r12,[r1,#OFFSET_W]     ; load w
    mov     r3, r3, lsl #2
    mov     r12,r12,lsl #2
    strh    r3, [r1,#OFFSET_X]     ; store x*4
    strh    r12,[r1,#OFFSET_W]     ; store w*4
    ldr     r3, [r1,#OFFSET_C]     ; load c

    ENDP

;*******************************************************************************
; void OSD_Fill_w(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                 uint32_t layerStride, uint32_t c);
;*******************************************************************************
OSD_Fill_w PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r8, [r1,#OFFSET_W]     ; load w
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r12,r12,r2,r0          ; r12 = line start address
    mov     r4,r3
    mov     r5,r3
    mov     r6,r3
    add     r12,r7                 ; r12 = start address
vloop
    mov     r0,r12
    mov     r7,r8                  ; restore width
    subs    r7,r7,#0x20            ; count -= 32
hloop32
    pld     [r0,+r2]               ; cache hint to next line
    stmcs   r0!,{r3,r4,r5,r6}
    stmcs   r0!,{r3,r4,r5,r6}
    subcss  r7,r7,#0x20
    bcs     hloop32
    pld     [r0,+r2]               ; cache hint to next line
    lsls    r7,r7,#28
    stmcs   r0!,{r3,r4,r5,r6}
    stmmi   r0!,{r3,r4}
    lsls    r7,r7,#2
    strcs   r3,[r0],#4
    beq     endaligned
    strmih  r3,[r0,#2]
    submi   r0,#2
    tst     r7,#0x40000000
    strneb  r3,[r0,#3]
endaligned
    add     r12,r2
    subs    r9,#1
    bhi     vloop
    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppBEto8bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                           uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppBEto8bpp
OSD_Blit1bppBEto8bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7              ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
    rev     r5,r5                  ; byteswap BE to LE
vloopBE
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopBE
    lsls    r5,#1                  ; shift bit into overflow/carry
    eorcs   r8,r0,#3               ; toggle lowest two address bits as we have
                                   ; a little-endian OSD memory layout
    strcsb  r4,[r8]                ; set byte if bit was set
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count if empty
    add     r0,#1                  ; dst++
    revcc   r5,r5                  ; byteswap BE to LE
    subs    r7,#1                  ; reduce width
    bhi     hloopBE

    add     r12,r2
    subs    r9,#1
    bhi     vloopBE

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppLEto8bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                           uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppLEto8bpp
OSD_Blit1bppLEto8bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7              ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
vloopLE
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopLE
    lsls    r5,#1                  ; shift bit into overflow/carry
    eorcs   r8,r0,#3               ; toggle lowest two address bits as we have
                                   ; a little-endian OSD memory layout
    strcsb  r4,[r8]                ; store byte if overflow/carry
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count on empty bit count
    subs    r7,#1                  ; reduce width
    add     r0,#1                  ; dst++
    bhi     hloopLE

    add     r12,r2
    subs    r9,#1
    bhi     vloopLE

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppBEto16bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                            uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppBEto16bpp
OSD_Blit1bppBEto16bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7,lsl#1        ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
    rev     r5,r5                  ; byteswap BE to LE
vloopBE16
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopBE16
    lsls    r5,#1                  ; shift bit into overflow/carry
    eorcs   r8,r0,#2               ; toggle second-lowest address bit as we have
                                   ; a little-endian OSD memory layout
    strcsh  r4,[r8]                ; set 16bit value if bit was set
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count if empty
    add     r0,#2                  ; dst++
    revcc   r5,r5                  ; byteswap BE to LE
    subs    r7,#1                  ; reduce width
    bhi     hloopBE16

    add     r12,r2
    subs    r9,#1
    bhi     vloopBE16

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppLEto16bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                            uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppLEto16bpp
OSD_Blit1bppLEto16bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7,lsl#1        ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
vloopLE16
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopLE16
    lsls    r5,#1                  ; shift bit into overflow/carry
    eorcs   r8,r0,#2               ; toggle second-lowest address bit as we have
                                   ; a little-endian OSD memory layout
    strcsh  r4,[r8]                ; set 16bit value if bit was set
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count if empty
    add     r0,#2                  ; dst++
    subs    r7,#1                  ; reduce width
    bhi     hloopLE16

    add     r12,r2
    subs    r9,#1
    bhi     vloopLE16

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppBEto32bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                            uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppBEto32bpp
OSD_Blit1bppBEto32bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7,lsl#2        ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
    rev     r5,r5                  ; byteswap BE to LE
vloopBE32
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopBE32
    lsls    r5,#1                  ; shift bit into overflow/carry
    strcs   r4,[r0]                ; set 32bit value if bit was set
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count if empty
    add     r0,#4                  ; dst++
    revcc   r5,r5                  ; byteswap BE to LE
    subs    r7,#1                  ; reduce width
    bhi     hloopBE32

    add     r12,r2
    subs    r9,#1
    bhi     vloopBE32

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

;*******************************************************************************
; void OSD_Blit1bppLEto32bpp(uint32_t *layerStart, OSDHANDLER_RectT *dst,
;                            uint32_t layerStride, uint32_t *src);
;*******************************************************************************
    EXPORT OSD_Blit1bppLEto32bpp
OSD_Blit1bppLEto32bpp PROC
    push    {r4,r5,r6,r7,r8,r9}
    ldrh    r12,[r1,#OFFSET_Y]     ; load y
    ldrh    r7, [r1,#OFFSET_X]     ; load x
    ldrh    r9, [r1,#OFFSET_H]     ; load h
    mla     r0,r12,r2,r0           ; r0 = line start address
    ldr     r4, [r1,#OFFSET_C]     ; load c
    add     r12,r0,r7,lsl#2        ; r12 = start address
    ldr     r5,[r3],#4             ; load bit src
    mov     r6,#31                 ; set bit count
vloopLE32
    pld     [r12,#+32]             ; cache hint into next cache-line
    pld     [r12,+r2,lsl#1]        ; cache hint 2 screenlines in advance
    mov     r0,r12
    ldrh    r7, [r1,#OFFSET_W]     ; restore width
hloopLE32
    lsls    r5,#1                  ; shift bit into overflow/carry
    strcs   r4,[r0]                ; set 32bit value if bit was set
    subs    r6,#1                  ; reduce bit count
    ldrcc   r5,[r3],#4             ; load bit src on empty bit count
    movcc   r6,#31                 ; reset bit count if empty
    add     r0,#4                  ; dst++
    subs    r7,#1                  ; reduce width
    bhi     hloopLE32

    add     r12,r2
    subs    r9,#1
    bhi     vloopLE32

    pop     {r4,r5,r6,r7,r8,r9}
    bx  lr

    ENDP

    END
