/*!
*****************************************************************************
**
** \file        ./middleware/pvr/src/pvr_priv.h
** 
** \version     $Id: pvr_priv.h 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       PVR middleware private functions.
**
** This file contains the declaration of the PVR private API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef PVR_PRIV_H
#define PVR_PRIV_H

//#include <fapi/sys_driver.h>
//#include <fapi/drv_dma.h>
//#include <rtos/rtos.h>

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libdma.h"
#include "fapex/abst_libtsd.h"

#include "pvr.h"



//***********************************************************************
//***********************************************************************
//** Defines and Macros
//***********************************************************************
//***********************************************************************

#define PVR_USING_TSD_BLOCK     FAPEX_TSD3

#define PVR_FULL_FILEPATH_LENGTH   (PVR_PATH_NAME_LENGTH \
                                  + PVR_FILE_NAME_LENGTH \
                                  + PVR_FILE_SEP_LENGTH  \
                                  + PVR_FILE_EXT_LENGTH)

#define PVR_HEADER_MAGIC        0x10233203  /* for checking the data type */

#define PVR_TSPKT_SIZE_LW       47          //!< TS packet size in LW.

#define PVR_HEADERAREA_SIZE     512         //!< Area reserved for headers at
                                            //!< beginning of each chunk

/* Below structures rely on PVR_MAX_TRACKS being a fixed value */
#if ( PVR_MAX_TRACKS != 32 )
    #error Invalid assignment of pvr_max_tracks!
#endif

//***********************************************************************
//***********************************************************************
//** Data types
//***********************************************************************
//***********************************************************************

#define PVR_HEADER_V2          0x00000002  /* supported version */
#define PVR_HEADER_VERSION     0x00000003  /* current version */

/* header packet structure of current version (v3): */
/* note that the compiler will align to 16bit by default */
/* for bytes take care of this alignment */
typedef struct
{
    /* --- first packet --- */

    /* TS packet header fields (4 bytes) */
    uint32_t  header;

    /* identifier and control fields (8 bytes) */
    uint32_t  magic;                          /* magic word                       */
    uint32_t  version;                        /* version of this structure        */

    /* fields used only in first physical header packet (12 bytes) */
    uint32_t  offset;                         /* position offset [chunks]         */
    uint32_t  size;                           /* total size [chunks]              */
    uint32_t  duration;                       /* duration [ms]                    */
                                         /* in case of looping for last loop */

    /* track/pid data, first part (68 bytes) */
    /* note that PVR_MAX_TRACKS must be a multiple of 4 */
    uint32_t  numberOfTracks;                 /* number of tracks                 */
    uint16_t  trackPid[PVR_MAX_TRACKS];       /* track PIDs                       */

    /* other data (76 bytes) */
    uint32_t  time;                           /* time stamp [ms]                  */
    uint32_t  dataRate;                       /* data rate                        */
    uint32_t  tag;                            /* tag                              */
    char_t    name[64];                       /* tag name (length = 64)           */

    /* for detecting events (8 bytes) */
    uint16_t  nodataCounter;                  /* data interruptions happened      */
    uint16_t  tagCounter;                     /* tag was set                      */
    uint16_t  nameCounter;                    /* service name changed             */
    uint16_t  dummyCounter;                   /* not used                         */

    /* index of video packet containing a sequence header code (4 bytes) */
    uint8_t   seqHeaderPacket[4];

    /* suppport of file splitting (6 bytes) */
    uint32_t  splitSize;                      /* split size [chunks]              */
    uint16_t  numberOfSplits;                 /* valid only in 1st split/header   */

    /* fill up to 188 bytes (2 bytes) */
    uint8_t   dummy[2];

    /* --- second packet --- */

    /* 2nd TS packet header fields (4 bytes) */
    uint32_t  header2;

    /* track/pid data, second part(160 bytes) */
    uint8_t   trackType[PVR_MAX_TRACKS];      /* track types PVR_TRACK_TYPE_E  */
    uint32_t  trackInfo[PVR_MAX_TRACKS];      /* track info (user defined)     */
    
    uint8_t   trackFormat[PVR_MAX_TRACKS / 2];/* track format (2 formats/byte) */
    
    /* fill up to 188 bytes (8 bytes) */
    uint8_t   dummy2[8];

} PVR_HeaderPacketT;

/* deprecated but still supported structure of version V2 */
typedef struct
{
    /* --- first packet --- */

    /* TS packet header fields (4 bytes) */
    uint32_t  header;

    /* identifier and control fields (8 bytes) */
    uint32_t  magic;                          /* magic word                       */
    uint32_t  version;                        /* version of this structure        */

    /* fields used only in first physical header packet (12 bytes) */
    uint32_t  offset;                         /* position offset [chunks]         */
    uint32_t  size;                           /* total size [chunks]              */
    uint32_t  duration;                       /* duration [ms]                    */
                                              /* in case of looping for last loop */

    /* track/pid data, first part (68 bytes) */
    /* note that PVR_MAX_TRACKS must be a multiple of 4 */
    uint32_t  numberOfTracks;                 /* number of tracks                 */
    uint16_t  trackPid[PVR_MAX_TRACKS];       /* track PIDs                       */

    /* other data (76 bytes) */
    uint32_t  time;                           /* time stamp [ms]                  */
    uint32_t  dataRate;                       /* data rate                        */
    uint32_t  tag;                            /* tag                              */
    char_t    name[64];                       /* tag name (length = 64)           */

    /* for detecting events (8 bytes) */
    uint16_t  nodataCounter;                  /* data interruptions happened      */
    uint16_t  tagCounter;                     /* tag was set                      */
    uint16_t  nameCounter;                    /* service name changed             */
    uint16_t  dummyCounter;                   /* not used                         */

    /* index of video packet containing a sequence header code (4 bytes) */
    uint8_t   seqHeaderPacket[4];

    /* suppport of file splitting (6 bytes) */
    uint32_t  splitSize;                      /* split size [chunks]              */
    uint16_t  numberOfSplits;                 /* valid only in 1st split/header   */

    /* fill up to 188 bytes (2 bytes) */
    uint8_t   dummy[2];

    /* --- second packet --- */

    /* 2nd TS packet header fields (4 bytes) */
    uint32_t  header2;

    /* track/pid data, second part(160 bytes) */
    uint8_t   trackType[PVR_MAX_TRACKS];      /* track types PVR_TRACK_TYPE_E    */
    uint32_t  trackInfo[PVR_MAX_TRACKS];      /* track info (user defined)        */
    
    /* fill up to 188 bytes (24 bytes) */
    uint8_t   dummy2[24];

} PVR_HeaderPacket_V2T;

//***********************************************************************
//***********************************************************************
//** Global Data
//***********************************************************************
//***********************************************************************


//***********************************************************************
//***********************************************************************
//** API Functions
//***********************************************************************
//***********************************************************************

#ifdef __cplusplus
extern "C" {
#endif

/* in pvr_private.c: */
int32_t          pvrPrivInit          (void);

int32_t          pvrSetMountPath      (const char_t* pathPtr);
const char_t*    pvrGetMountPath      (void);
int32_t          pvrSetFileExtension  (const char_t* extPtr);
const char_t*    pvrGetFileExtension  (void);
int32_t          pvrSetFileSplitParams(uint8_t numDigits,
                                       uint8_t decimalNotation);
uint16_t         pvrGetMaxSplitNumber (void);

uint64_t         pvrGetTimeStamp      (void);
int32_t          pvrGetFileInfo       (const char_t* fileNamePtr,
                                       PVR_FileInfoT* fileInfoPtr,
                                       PVR_HeaderPacketT* headerPtr);
int32_t          pvrImportHeader      (PVR_HeaderPacketT* headerPtr,
                                       void* srcDataPtr, uint32_t dataBytes);
uint8_t          pvrFeedTsd           (uint8_t tsd, const uint32_t* dataPtr,
                                       uint32_t packets,
                                       uint8_t discardStuffing,
                                       uint64_t* msNeededPtr,
                                       int32_t   tsd_block_fd,
                                       uint8_t* input_port_base);
void             pvrGetFilename       (char_t* subFileNamePtr,
                                       const char_t* fileNamePtr,
                                       uint16_t split);
void             pvrDeleteFiles       (const char_t* fileNamePtr);

int32_t          pvrSetTrackFormat    (PVR_HeaderPacketT* headerPtr,
                                       uint8_t trackIndex, uint8_t format);
uint8_t          pvrGetTrackFormat    (const PVR_HeaderPacketT* headerPtr,
                                       uint8_t trackIndex);
void             pvrSetCryptoKeys     (const PVR_CryptoParamsT* paramsPtr,
                                       FAPEX_DMA_CryptoKeysT* keysPtr);
int32_t          pvrPerformCryptoOperation (void* dstBufPtr,
                                       const void* srcBufPtr,
                                       const FAPEX_DMA_CryptoKeysT* keysPtr,
                                       uint8_t decrypt);
/* in pvr.c: */
FAPEX_SYS_HandleT pvrGetPlaybackHandleByTsd   (uint8_t tsd);
FAPEX_SYS_HandleT pvrGetRecordingHandleByTsd  (uint8_t tsd);
uint32_t         pvrGetPlaybackChunkDuration (uint8_t tsd);

int32_t         pvrGetFileSize(const char* fpath);


#ifdef __cplusplus
}
#endif

#endif /* PVR_PRIV_H */
