/****************************************************************************
 * MODULE     : TT_Queue
 * FILE       : TT_Queue.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   Teletext message queue used to send messages to the decoder.
 ****************************************************************************/

#ifndef TT_QUEUE_H
#define TT_QUEUE_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/

#include <teletext/TT_Stdinc.h>

/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/

/****************************************************************************
 * TYPE: TT_QUEUE_TYPE
 *
 * DESCRIPTION:
 *   Defines the type of queue.
 ****************************************************************************/
typedef enum
{
  TT_QUEUE_DSP,
  TT_QUEUE_KEY,
  TT_QUEUE_MAX

} TT_QUEUE_TYPE;

/****************************************************************************
 * TYPE: TT_QUEUE_MESSAGE
 *
 * DESCRIPTION:
 *   A TT_QUEUE_MESSAGE is used to transfer commands between tasks. The
 *   command itself is an unsigned short value and is defined by the task.
 *   Each command can optionally contain up to two unsigned short parameters.
 *
 * COMPONENTS:
 *   Info   - The message value
 *   Param1 - Optional parameter 1
 *   Param2 - Optional parameter 2
 ****************************************************************************/
typedef struct
{
  U_INT16 
    Info,
    Param1,
    Param2;

} TT_QUEUE_MESSAGE;


#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_QueueSet
 *
 * DESCRIPTION:
 *   Write message into a message queue
 *
 * PARAMETERS:
 *   aQueue_id - queue ID
 *   aMessage  - pointer to message
 *
 * RETURN:
 *   success
 ****************************************************************************/

PUBLIC TT_BOOL
TT_QueueSet(
  U_INT8             aQueue_id, 
  TT_QUEUE_MESSAGE * aMessage );


/****************************************************************************
 * FUNCTION: TT_QueueGetNoOfMessages
 *
 * DESCRIPTION:
 *   Return current the number of messages in the queue.
 *
 * PARAMETERS:
 *   aQueue_id - queue ID
 *
 * RETURN:
 *   The current the number of messages in the queue. -1 on error.
 ****************************************************************************/

PUBLIC INT16
TT_QueueGetNoOfMessages(
  U_INT8             aQueue_id );



#ifdef __cplusplus
  }
#endif

#endif /* TT_QUEUE_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
