/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_pvr_demo/src/main.c
**
** \version     $Id$		Aug 5@2010	Martin.H@FSDC
**
** \brief       Demo Application for PVR & Audio/Video decoding.
**
** This application implements a test program for PVR and Live and File playback
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
********************************************************************************
********************************************************************************
*/
/*
********************************************************************************
**  Change Log  [Internal Only]
********************************************************************************
** 20100805 -- Martin.H
********************************************************************************
********************************************************************************
*/
//#define DEBUG 1

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libir.h>
#include <fapex/abst_libfpc.h>
#include <fapex/abst_libclkpwr.h>
#include <fapex/abst_libtsd.h>
#include <fapex/abst_libgpio.h>
#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */
#include <val/val.h>
#include "demo_app.h"
#include "tsd.h"
#include "ttx.h"
#include "audio.h"
#include "servicelist.h"
#include "osd.h"
#include "sidebug.h"
#include "scan.h"
#include "hdmi_cec.h"
#if ENABLE_GUI_WINDOW == 1
#include "gui.h"
#endif

#include "pvr/pvr.h"
#include "frontend.h"
#include "pvr_dec.h"
#include "pvr_m.h"


//******************************************************************************
//******************************************************************************
//** Defines and Macros
//******************************************************************************
//******************************************************************************

#if defined(__arm__)
    #define APP_CPU      "arm1176"
#elif defined(_ARC)
    #define APP_CPU      "arc725d"
#else
    #define APP_CPU      "unknown"
#endif

#if defined(MB86H60)
    #define APP_TARGET   "MB86H60"
#else
    #define APP_TARGET   "unknown"
#endif

#define APP_VERSION      "2.0.6"
#define APP_PREFIX       "[SH  ]"
#define APP_SHELL_PROMPT "[SH=>] \n"

#define APP_CLEARSCREEN "" // "\033[2J\033[0;0H"
#define APP_SEPARATOR   "################################################################"
#define APP_DESCRIPTION "PVR Demo Application (TS file playback support version)"
#define APP_COPYRIGHT   "(C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH"
#define APP_COPYRIGHT2  "(C) Copyright 2008-2010 by Fujitsu Semiconductor Limited"

#define APP_CMD_STR_LENGTH  128
#define APP_REMOTE_MODE     FAPEX_IR_MODE_NEC   // FAPEX_IR_MODE_NEC or FAPEX_IR_MODE_RC5

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_START_THREAD_STACKSIZE         256
#define APP_START_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGHEST
#define APP_START_THREAD_NAME              "app:start"
#define APP_SHELL_THREAD_STACKSIZE         2048
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"
#define APP_SCAN_THREAD_STACKSIZE          1024
#define APP_SCAN_THREAD_PRIORITY           FAPEX_SYS_PRIORITY_HIGH
#define APP_SCAN_THREAD_NAME               "app:scan"
#define APP_VIDEO_THREAD_STACKSIZE         1024
#define APP_VIDEO_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGH
#define APP_VIDEO_THREAD_NAME              "app:video"
#define APP_AUDIO_THREAD_STACKSIZE         1024
#define APP_AUDIO_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGH
#define APP_AUDIO_THREAD_NAME              "app:audio"
#define APP_TTX_THREAD_STACKSIZE           1024
#define APP_TTX_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_TTX_THREAD_NAME                "app:ttx"
#define APP_OSD_THREAD_STACKSIZE           1024
#define APP_OSD_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_OSD_THREAD_NAME                "app:osd"
#define APP_WATCHDOG_THREAD_STACKSIZE      256
#define APP_WATCHDOG_THREAD_PRIORITY       (FAPEX_SYS_PRIORITY_HIGHEST - 1)
#define APP_WATCHDOG_THREAD_NAME           "app:watchdog"
#define APP_IR_THREAD_STACKSIZE            1024
#define APP_IR_THREAD_PRIORITY             FAPEX_SYS_PRIORITY_MEDIUM
#define APP_IR_THREAD_NAME                 "app:ir"
#define APP_FPC_THREAD_STACKSIZE           1024
#define APP_FPC_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_FPC_THREAD_NAME                "app:fpc"
#define APP_HDMI_THREAD_STACKSIZE          1024
#define APP_HDMI_THREAD_PRIORITY           FAPEX_SYS_PRIORITY_MEDIUM
#define APP_HDMI_THREAD_NAME               "app:hdmi"
#if ENABLE_GUI_WINDOW == 1
#define APP_GUI_THREAD_STACKSIZE           4096
#define APP_GUI_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_GUI_THREAD_NAME                "app:gui"
#endif
#define APP_PVR_THREAD_STACKSIZE           4096
#define APP_PVR_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_PVR_THREAD_NAME                "app:pvr"

//******************************************************************************
//******************************************************************************
//** Local Types
//******************************************************************************
//******************************************************************************
#define DRV_ID_IR0      (FAPEXDEV_IR_MINOR_FIRST)
#define DRV_ID_FPC0     (FAPEXDEV_FPC_MINOR_FIRST)
#define DRV_ID_CLKPWR   (FAPEXDEV_CLKPWR_MINOR_FIRST)

#define PCR_TSD_CHANNEL     7
#define DRV_ID_TSD0_PCR_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + PCR_TSD_CHANNEL)

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************
static int32_t fd_IR0      = -1;
static int32_t fd_FPC0     = -1;
static int32_t fd_CLKPWR   = -1;
static int32_t fd_TSD0_PCR = -1;

static int32_t           appStandbyMode  = 0;
static VAL_HdEncoderT    appHdSystemType = VAL_1920x1080x50I;
static VAL_SdEncoderT    appSdSystemType = VAL_SD_PAL;
static int32_t           trpListIndex = 0;
#if 0 /* now not supported */
static int32_t           tsScan = 0;
#endif
static VAL_EnvironmentT  appPlaybackMode = VAL_ENV_LIVE;

#ifndef _LINUX_FAPI
static FAPI_SYS_ShellInputHandler* appShellInputHandler;
#endif
static uint32_t                    appServiceCount = 0;
static int32_t                     appCurrentServiceIndex = -1;
static SERVICELIST_ServiceInfo_T*  appServicePtr = NULL;
static FAPEX_TSD_InputParamStrT    appTsInputParams;
static char_t                      appTsInputFileName[FAPEX_SYS_FMAXPATH];
static uint32_t                    appTsInputStartOffset;

static FAPEX_SYS_ThreadT    appStartThreadHandle;
static FAPEX_SYS_ThreadT    appShellThreadHandle;
static FAPEX_SYS_ThreadT    appScanThreadHandle;
static FAPEX_SYS_ThreadT    appVideoThreadHandle;
static FAPEX_SYS_ThreadT    appAudioThreadHandle;
static FAPEX_SYS_ThreadT    appTtxThreadHandle;
static FAPEX_SYS_ThreadT    appPvrThreadHandle;
#if ENABLE_GUI_WINDOW == 0
static FAPEX_SYS_ThreadT    appOsdThreadHandle;
#else
static FAPEX_SYS_ThreadT    appGuiThreadHandle;
static int32_t              appOsd1_fd = -1;
#endif
static FAPEX_SYS_ThreadT    appIrThreadHandle;
static FAPEX_SYS_ThreadT    appFpcThreadHandle;
static FAPEX_SYS_ThreadT    appHdmiThreadHandle;
static int32_t              appHdmiEnable = -1;
#ifdef USE_WATCHDOG
static FAPEX_SYS_ThreadT    appWatchdogThreadHandle;
#endif

/* thread status ... APP_THREAD_STATUS_xxx */
static APP_ThreadStatus  appVideoThreadStatus;
static APP_ThreadStatus  appAudioThreadStatus;
static APP_ThreadStatus  appTtxThreadStatus;
static APP_ThreadStatus appPvrThreadStatus;

#if ENABLE_GUI_WINDOW == 0
static APP_ThreadStatus  appOsdThreadStatus;
#endif

#ifdef _LINUX_FAPI
#define APP_ColorFgBlack     "\033[30m"
#define APP_ColorFgGreen     "\033[32m"
#define APP_ColorFgBlue      "\033[34m"
#define APP_ColorBgWhite     "\033[47m"
#define APP_CommandTableSize  64
#define APP_CommandStringSize 80
#define APP_HistoryTableSize  16
#define APP_ArgvSize          16

typedef struct APP_CommandStruct {
    char_t*    command;
    char_t*    brief;
    char_t*    description;
    uint32_t   (*function)(int32_t, char_t**);
} APP_CommandStr;

APP_CommandStr  commandTable[APP_CommandTableSize];
char_t          inputCommand[APP_CommandStringSize];
char_t          historyTable[APP_HistoryTableSize][APP_CommandStringSize];
int32_t         historyIndex;
int32_t         updownIndex;

#endif /* !_LINUX_FAPI */

typedef enum
{
    /* add your command here */
    PVRCALLBACK_CMD_NONE                    = -1,
    PVRCALLBACK_CMD_PLAYBACK_STOP           = 0,
    PVRCALLBACK_CMD_GET_PLAYBACK_INFO,
    PVRCALLBACK_CMD_PLAYBACK_RESUME,
    PVRCALLBACK_CMD_LOOP
}PVR_CALLBACK_CmdT;

//******************************************************************************
//******************************************************************************
//** Global Data
//******************************************************************************
//******************************************************************************
VAL_DataT*     vidDatPtr  = NULL;
VAL_OpenT*     vidOpenPtr = NULL;
extern int32_t fd_TSD[];

static FAPEX_SYS_HandleT  pbHandle = NULL;
static FAPEX_SYS_HandleT  recHandle = NULL;     /*current op recHandle*/
static FAPEX_SYS_HandleT  recHandle1 = NULL;    /*using TSD_2*/
static FAPEX_SYS_HandleT  recHandle2 = NULL;  	/*using TSD_3*/
static char  pvr_mnt_path[50] = "/mnt/tmp/";
static char  pvr_timeshift_file_name[50] = "ts.ts";

static FAPEX_SYS_SemaphoreT pvr_callback_cmd_sem = NULL;
static PVR_CALLBACK_CmdT pvrCallbackCmd;

static FAPEX_SYS_ThreadT	pvrTimeDispThread = NULL;

uint64_t pvr_begin_ts;
uint64_t pvr_end_ts;

/*
 * brief: Test this global and appPlaybackMode to determine whether in playback 
 *        pause or in live pause (timeshift entry).
 * 0: Not in pause status
 * 1: In pause status
 */
static uint8_t pvrlive_pause_status = 0;

/*
 * Note: take care that the IFRAME_TSD_CHANNEL must not confilict with TSD channel used
 *          by other fucntion module.
 */
#define IFRAME_TSD_CHANNEL 30

//******************************************************************************
//******************************************************************************
//** Local Functions
//******************************************************************************
//******************************************************************************

static void appStandby( uint32_t autoStart );
#if 0 /* now not supported */
static void appFpcChannelUp(void* optDataPtr);
static void appFpcChannelDown(void* optDataPtr);
#endif
static void appChangeService( SERVICELIST_ServiceInfo_T* servicePtr, uint32_t showProgramFlag );
static SERVICELIST_ServiceInfo_T* appGetNextService( void );
static SERVICELIST_ServiceInfo_T* appGetPrevService( void );
static void appHdmiAutoSync(void);


#if ENABLE_GUI_WINDOW == 1
static void appGuiThreadFunction(void* optArg);
#endif

static int32_t 
PVRSYS_TestFile(const char* file_path);

static int32_t
PVRSYS_PlayChangeMode (FAPI_SYS_HandleT handlePlay,
                                PVR_PlayModeT mode, uint8_t modeIndex);
static int32_t 
PVRSYS_PlaybackStart(const char *file_name, PVR_PlayModeT init_mode);

static int32_t 
PVRSYS_TimeshiftPlaybackStart(const char *file_name, PVR_PlayModeT init_mode);

static int32_t 
PVRSYS_PlaybackStop(void);

static int32_t 
PVRSYS_PlaybackResume(void);

static int32_t 
PVRSYS_RecordResume(void);

static int32_t 
PVRSYS_PlayWaitForTransitionEnd (FAPEX_SYS_HandleT handle);

static int32_t
PVRSYS_RecordStart(const char* file_name);

static int32_t 
PVRSYS_RecWaitForStart (FAPEX_SYS_HandleT rec_handle);

FAPEX_SYS_HandleT
PVRSYS_GetCurrentTimeshiftRecording(void);

FAPEX_SYS_HandleT
PVRSYS_FindCurrentRecordingByFileName(const char *file_name);


/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Init(APP_ThreadStatus *status)
{
    status->value = APP_THREAD_STATUS_DISABLE;
    status->sem   = FAPEX_SYS_CreateSemaphore(1);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Exit(APP_ThreadStatus *status)
{
    FAPEX_SYS_DestroySemaphore(status->sem);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Lock(APP_ThreadStatus *status)
{
    FAPEX_SYS_GetSemaphore(status->sem, FAPEX_SYS_SUSPEND);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_Unlock(APP_ThreadStatus *status)
{
    FAPEX_SYS_SetSemaphore(status->sem, FAPEX_SYS_NO_SUSPEND);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
void APP_ThreadStatus_SetValue(APP_ThreadStatus *status, int32_t value)
{
    // lock
    FAPEX_SYS_GetSemaphore(status->sem, FAPEX_SYS_SUSPEND);

    status->value = value;

    // unlock
    FAPEX_SYS_SetSemaphore(status->sem, FAPEX_SYS_SUSPEND);
}

/*
*****************************************************************************
** \brief
*****************************************************************************
*/
static PVR_CALLBACK_CmdT pvrGetCallbackCmd(void)
{
    PVR_CALLBACK_CmdT   cmd;

    FAPEX_SYS_GetSemaphore(pvr_callback_cmd_sem, FAPEX_SYS_SUSPEND);

    cmd = pvrCallbackCmd;

    FAPEX_SYS_SetSemaphore(pvr_callback_cmd_sem, FAPEX_SYS_NO_SUSPEND);

    return cmd;
}

static void pvrSetCallbackCmd(PVR_CALLBACK_CmdT cmd)
{

    FAPEX_SYS_GetSemaphore(pvr_callback_cmd_sem, FAPEX_SYS_SUSPEND);

    pvrCallbackCmd = cmd;

    FAPEX_SYS_SetSemaphore(pvr_callback_cmd_sem, FAPEX_SYS_NO_SUSPEND);

    return;
}

/*
*****************************************************************************
** \brief Video initial setup
*****************************************************************************
*/
static int32_t appVideoInit()
{
    VAL_UpParamsT  vidParams;
    int32_t        result;

    // VAL Goup
    vidParams.hdSystemType                    = appHdSystemType;
    vidParams.sdSystemType                    = appSdSystemType;
    vidParams.displayAspectRatio[VAL_VOUT_SD] = FAPEX_VIOUT_DISPLAY_AR_NONE;
    vidParams.displayAspectRatio[VAL_VOUT_HD] = FAPEX_VIOUT_DISPLAY_AR_4_3;
    vidParams.scalingMethod                   = FAPEX_VIOUT_SCALER_MODE_FSZOOM;

    vidDatPtr = VAL_GoUp (&vidParams, &result);
    if(vidDatPtr == NULL) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] VAL_GoUp failed. %d\n", result);
    }

#if ENABLE_GUI_WINDOW == 1
    appGuiThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_GUI_THREAD_STACKSIZE,
            APP_GUI_THREAD_PRIORITY,
            appGuiThreadFunction,
            0, 0, APP_GUI_THREAD_NAME );
    if( appGuiThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_GUI_THREAD_NAME );
        exit(1);
    }
#endif
    return result;
}

/*
*****************************************************************************
** \brief Video exit
*****************************************************************************
*/
static int32_t appVideoExit()
{
    int32_t  result;

#if ENABLE_GUI_WINDOW == 1
    if(appOsd1_fd >= 0) {
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)appOsd1_fd,
                                    FAPEX_IOCCOM_CLOSE_SUBID,
                                    (FAPEXIOCARG3_T)NULL );
        if( result < 0 ) {
            FAPI_SYS_PRINT_DEBUG(2, "ERROR:[main.c] OSD1 close failed. %d\n", result);
        }
        appOsd1_fd = -1;
    }

    GUI_Close();
#endif

    result = VAL_GoDown(vidDatPtr);
    if( result < 0 ) {
        FAPI_SYS_PRINT_DEBUG(2, "ERROR:[main.c] VAL_GoDown failed. %d\n", result);
    }

    return result;
}

/*
********************************************************************************
** Initialize PCR TSD channel.
********************************************************************************
*/
static int32_t pcrTsdOpen(void)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    int32_t result;

    if(fd_TSD0_PCR != -1) {
        return 0;
    }

    pidChParam.channelNo = PCR_TSD_CHANNEL;
    pidChParam.sub_id    = DRV_ID_TSD0_PCR_CH;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                              FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Open PCR TSD channel. %d\n", result);
    }
    else {
        fd_TSD0_PCR = pidChParam.sub_id;
    }
    return result;
}

/*
********************************************************************************
** Initialize PCR TSD channel.
********************************************************************************
*/
static int32_t pcrTsdClose(void)
{
    int32_t  result;

    if(fd_TSD0_PCR == -1) {
        return 0;
    }

    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                              FAPEX_IOCCOM_CLOSE_SUBID,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Close PCR TSD channel. %d\n", result);
    }
    else {
        fd_TSD0_PCR = -1;
    }
    return result;
}

/*
********************************************************************************
** Start PCR TSD channel.
********************************************************************************
*/
static int32_t pcrTsdStart(uint32_t pid)
{
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t result;

    if(fd_TSD0_PCR == -1) {
        return 0;
    }

    /* set PCR filtering */
    pidParam.pid        = pid;
    pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
    pidParam.bufferSize = 0;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                              FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                              (FAPEXIOCARG3_T)&pidParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Set PCR Pid. %d\n", result);
        return result;
    }

    /* start PCR TSD channel */
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Start PCR TSD channel. %d\n", result);
    }
    return result;
}

/*
********************************************************************************
** Stop PCR TSD channel.
********************************************************************************
*/
static int32_t pcrTsdStop(void)
{
    int32_t  result;

    if(fd_TSD0_PCR == -1) {
        return 0;
    }

    /* stop PCR TSD channel */
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                              FAPEX_IOCCOM_STOP,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Stop PCR TSD channel. %d\n", result);
    }
    return result;
}

/*
********************************************************************************
** Open Service
**
** \params openLevel
**             - APP_THREAD_STATUS_OPENED  : all thread open for standby
**             - APP_THREAD_STATUS_READY   : TSD block open only
********************************************************************************
*/
static int32_t appOpenService(int32_t openLevel)
{
    int32_t  opened_cnt;
    int32_t  result;

    if(openLevel == APP_THREAD_STATUS_OPENED) {
        /* for standby mode */
        result = appVideoInit();
        if( result < 0 ) {
            return result;
        }
    }

    if(openLevel < APP_THREAD_STATUS_READY) {
        // open VIDEO/AUDIO/TTX module
        while(1)
        {
            opened_cnt = 0;

            APP_ThreadStatus_Lock( &appVideoThreadStatus );
            if( (appVideoThreadStatus.value >= APP_THREAD_STATUS_CREATED)
             && (appVideoThreadStatus.value < openLevel) ) {
                if(appVideoThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                    APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                    FAPEX_SYS_WakeupThread( appVideoThreadHandle );
                }
                else {
                    APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                opened_cnt ++;
            }

            APP_ThreadStatus_Lock( &appAudioThreadStatus );
            if( (appAudioThreadStatus.value >= APP_THREAD_STATUS_CREATED)
             && (appAudioThreadStatus.value < openLevel) ) {
                if(appAudioThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                    APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                    FAPEX_SYS_WakeupThread( appAudioThreadHandle );
                }
                else {
                    APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                opened_cnt ++;
            }

            APP_ThreadStatus_Lock( &appTtxThreadStatus );
            if( (appTtxThreadStatus.value >= APP_THREAD_STATUS_CREATED)
             && (appTtxThreadStatus.value < openLevel) ) {
                if(appTtxThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                    APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                    FAPEX_SYS_WakeupThread( appTtxThreadHandle );
                }
                else {
                    APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                opened_cnt ++;
            }

#if ENABLE_GUI_WINDOW == 0
            APP_ThreadStatus_Lock( &appOsdThreadStatus );
            if( (appOsdThreadStatus.value >= APP_THREAD_STATUS_CREATED)
             && (appOsdThreadStatus.value < openLevel) ) {
                if(appOsdThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                    APP_ThreadStatus_Unlock( &appOsdThreadStatus );
                    FAPEX_SYS_WakeupThread( appOsdThreadHandle );
                }
                else {
                    APP_ThreadStatus_Unlock( &appOsdThreadStatus );
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );
                opened_cnt ++;
            }

            if(opened_cnt >= 4) {
                break;
            }
#else
            if(opened_cnt >= 3) {
                break;
            }
#endif
            FAPI_SYS_SLEEP(10);
        }
    }
    return 0;
}

/*
********************************************************************************
** Close Service
**
** \params closeLevel
**             - APP_THREAD_STATUS_CREATED : end each thread
**             - APP_THREAD_STATUS_OPENED  : suspend each thread for standby
**             - APP_THREAD_STATUS_READY   : suspend each thread
********************************************************************************
*/
static int32_t appCloseService(int32_t closeLevel)
{
    int32_t  closed_cnt;
    int32_t  result = 0;

    TSD_Stop();

    // close VIDEO/AUDIO/TTX module
    appServicePtr = NULL;
    if(closeLevel == APP_THREAD_STATUS_CREATED) {
        appStandbyMode = -1;
    }

    while(1)
    {
        closed_cnt = 0;

        APP_ThreadStatus_Lock( &appVideoThreadStatus );
        if(appVideoThreadStatus.value >= closeLevel) {
            if(appVideoThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                FAPEX_SYS_WakeupThread( appVideoThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appVideoThreadStatus );
            closed_cnt ++;
        }

        APP_ThreadStatus_Lock( &appAudioThreadStatus );
        if(appAudioThreadStatus.value >= closeLevel) {
            if(appAudioThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                FAPEX_SYS_WakeupThread( appAudioThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appAudioThreadStatus );
            closed_cnt ++;
        }

        APP_ThreadStatus_Lock( &appTtxThreadStatus );
        if(appTtxThreadStatus.value >= closeLevel) {
            if(appTtxThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                FAPEX_SYS_WakeupThread( appTtxThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appTtxThreadStatus );
            closed_cnt ++;
        }

#if ENABLE_GUI_WINDOW == 0
        APP_ThreadStatus_Lock( &appOsdThreadStatus );
        if(appOsdThreadStatus.value >= closeLevel) {
            if(appOsdThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );
                FAPEX_SYS_WakeupThread( appOsdThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appOsdThreadStatus );
            closed_cnt ++;
        }

        if(closed_cnt >= 4) {
            break;
        }
#else
        if(closed_cnt >= 3) {
            break;
        }
#endif
        FAPI_SYS_SLEEP(10);
    }

    if(closeLevel == APP_THREAD_STATUS_CREATED) {
        FAPEX_SYS_WaitEndThread( appVideoThreadHandle );
        APP_ThreadStatus_Exit( &appVideoThreadStatus );
        result = appVideoExit();

        FAPEX_SYS_WaitEndThread( appAudioThreadHandle );
        APP_ThreadStatus_Exit( &appAudioThreadStatus );

        FAPEX_SYS_WaitEndThread( appTtxThreadHandle );
        APP_ThreadStatus_Exit( &appTtxThreadStatus );

#if ENABLE_GUI_WINDOW == 0
        FAPEX_SYS_WaitEndThread( appOsdThreadHandle );
        APP_ThreadStatus_Exit( &appOsdThreadStatus );
#endif
    }
    else if(closeLevel == APP_THREAD_STATUS_OPENED) {
        /* for standby mode */
        result = appVideoExit();
    }
    if( result < 0 ) {
        return result;
    }

    result = pcrTsdClose();
    if( result < 0 ) {
        return result;
    }

    if(closeLevel == APP_THREAD_STATUS_CREATED) {
        result = TSD_Close();
    }
    return result;
}

/*
********************************************************************************
** Start Service
********************************************************************************
*/
static int32_t appStartService(SERVICELIST_ServiceInfo_T *servicePtr, uint32_t showProgramFlag)
{
    int32_t  result = 0;

    if(servicePtr == NULL) {
        return 0;
    }
    appServicePtr = servicePtr;

    TSD_Restart();

    // start VIDEO/AUDIO/TTX module
    FAPEX_SYS_WakeupThread( appVideoThreadHandle );
    FAPEX_SYS_WakeupThread( appAudioThreadHandle );
    FAPEX_SYS_WakeupThread( appTtxThreadHandle );

    if(servicePtr->pcrPid != 0) {
        if( (servicePtr->pcrPid != servicePtr->videoPid)
         && (servicePtr->pcrPid != servicePtr->audioPid[0]) ) {
            result = pcrTsdOpen();
            if(result == 0) {
                result = pcrTsdStart(servicePtr->pcrPid);
            }
        }
    }

    if(appHdmiEnable == 0) {
        /* wait for Video start */
        while(1) {
            if(appServicePtr->videoPid == 0) {
                break;
            }
            if(appVideoThreadStatus.value >= APP_THREAD_STATUS_READY) {
                if(appVideoThreadStatus.value >= APP_THREAD_STATUS_STARTED) {
                    appHdmiAutoSync();
                }
                break;
            }
            FAPI_SYS_SLEEP(100);
        }
    }

    if(showProgramFlag != 0) {
        /* show service name on OSD */
#if ENABLE_GUI_WINDOW == 0
        FAPEX_SYS_WakeupThread( appOsdThreadHandle );
#else
        GUI_SetServiceInfo(appServicePtr);
#endif
    }
    return result;
}

/*
********************************************************************************
** Stop Service
********************************************************************************
*/
static int32_t appStopService(void)
{
    static SERVICELIST_ServiceInfo_T  service_info;
    int32_t  stoped_cnt;
    int32_t  result;

    // stop VIDEO/AUDIO/TTX module
    service_info.videoPid    = 0;
    service_info.audioPid[0] = 0;
    service_info.teletextPid = 0;
    appServicePtr = &service_info;

    while(1) {
        stoped_cnt = 0;

        APP_ThreadStatus_Lock( &appVideoThreadStatus );
        if(appVideoThreadStatus.value >= APP_THREAD_STATUS_READY) {
            if(appVideoThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                FAPEX_SYS_WakeupThread( appVideoThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appVideoThreadStatus );
            stoped_cnt ++;
        }

        APP_ThreadStatus_Lock( &appAudioThreadStatus );
        if(appAudioThreadStatus.value >= APP_THREAD_STATUS_READY) {
            if(appAudioThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                FAPEX_SYS_WakeupThread( appAudioThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appAudioThreadStatus );
            stoped_cnt ++;
        }

        APP_ThreadStatus_Lock( &appTtxThreadStatus );
        if(appTtxThreadStatus.value >= APP_THREAD_STATUS_READY) {
            if(appTtxThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                FAPEX_SYS_WakeupThread( appTtxThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
            }
        }
        else {
            APP_ThreadStatus_Unlock( &appTtxThreadStatus );
            stoped_cnt ++;
        }

        if(stoped_cnt >= 3) {
            break;
        }
        FAPI_SYS_SLEEP(10);
    }
    result = pcrTsdStop();

    return result;
}

/*
********************************************************************************
** \brief Change service.
**
** \param servicePtr      Pointer of the new service.
** \param showProgramFlag Flag for show program
**
********************************************************************************
*/
static void appChangeService( SERVICELIST_ServiceInfo_T* servicePtr, uint32_t showProgramFlag )
{
    if(servicePtr->videoPid != 0) {
        if( (servicePtr->videoType != VID_MPEG2)
         && (servicePtr->videoType != VID_H264) ) {
            FAPI_SYS_PRINT_MSG("ERROR:[main.c] Unsupported video type.\n");
            return;
        }
    }

    if(showProgramFlag != 0) {
        FAPI_SYS_PRINT_MSG( "[main.c]: Change to service: %s\n",
                            servicePtr->name);
    }

    // Stop current service
    pcrTsdStop();

#if 0 /* now not supported */
    if(servicePtr->trpListIndex != trpListIndex)
    {
        (void)FRONTEND_SetDefaultTuneParmeters(servicePtr->trpListIndex, &tune_params);
        /* tune to new transponder */
        if(FRONTEND_Tune(&tune_params) == FAPI_OK)
            FAPI_SYS_PRINT_MSG( "[main.c] FE locked!\n" );
        trpListIndex = servicePtr->trpListIndex;
    }
#endif

    appStartService( servicePtr, showProgramFlag );

    if(fd_FPC0 > 0) {
        int32_t  result;

        result = FAPEX_FPC_Ioctl( fd_FPC0,
                                  FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE,
                                  (FAPEXIOCARG3_T)(appCurrentServiceIndex + 1) );
        if( result != 0 ) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] error FPC set value (code: %d).\n", result);
        }
    }
}

/*
********************************************************************************
** \brief Get next service.
**
** This function returns the next available service from the actual service list.
**
** \return Valid pointer to the next service or NULL if no service available.
**
********************************************************************************
*/
static SERVICELIST_ServiceInfo_T* appGetNextService(void)
{
    SERVICELIST_ServiceInfo_T*  service_ptr;

    if ( !appServiceCount ) {
        appCurrentServiceIndex = -1;
        appServiceCount = SERVICELIST_GetSize();
        if ( !appServiceCount ) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] No service list available.\n" );
#if ENABLE_GUI_WINDOW == 1
            GUI_SetRunningCancel();
#endif
            return(NULL);
        }
    }

    if ( appCurrentServiceIndex == (int32_t)(appServiceCount-1) )
        appCurrentServiceIndex = -1;

    service_ptr = SERVICELIST_GetService((uint32_t)(appCurrentServiceIndex+1));
    if ( !service_ptr ) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Next service not found.\n" );
        return(NULL);
    }
    appCurrentServiceIndex++;

    return( service_ptr );
}

/*
********************************************************************************
** \brief Get next service.
**
** This function returns the next available service from the actual service list.
**
** \return Valid pointer to the next service or NULL if no service available.
**
********************************************************************************
*/
static SERVICELIST_ServiceInfo_T* appGetServiceByPid( int32_t argc, char_t* argv[] )
{
    static SERVICELIST_ServiceInfo_T  service_info;
    SERVICELIST_ServiceInfo_T*  service_ptr   = NULL;
    uint32_t                    service_index = 0;
    uint32_t                    track         = 0;
    uint32_t                    audio_pid     = 0;
    uint32_t                    video_pid     = 0;
    uint32_t                    ttx_pid       = 0;

    if( (argc != 2) && ((argc < 3 ) || (argc > 4 ))) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] wrong number of arguments for command \"%s\"\n", argv[0] );
        FAPI_SYS_PRINT_MSG( "               expected %d or %d but got %d, type \"help\" for details\n", 3, 4, argc );
        return( NULL );
    }

    /* atoi is deprecated by MISRA */
    audio_pid = (uint32_t)atoi(argv[1]);
    video_pid = (uint32_t)atoi(argv[2]);
    if(argc == 4) {
        ttx_pid = (uint32_t)atoi(argv[3]);
    }

    if ( !appServiceCount ) {
        appCurrentServiceIndex = -1;
        appServiceCount = SERVICELIST_GetSize();
        if ( !appServiceCount ) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] No service list available.\n" );
#if ENABLE_GUI_WINDOW == 1
            GUI_SetRunningCancel();
#endif
            return(NULL);
        }
    }

    if ( appCurrentServiceIndex == (int32_t)(appServiceCount-1) )
        appCurrentServiceIndex = -1;

    for(service_index = 0; service_index < appServiceCount; service_index++) {
        service_ptr = SERVICELIST_GetService((service_index));
        if ( !service_ptr ) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c]  Next service not found.\n" );
            return(NULL);
        }

        if((video_pid != 0) && (service_ptr->videoPid == video_pid)) {
            break;
        }

        for(track=0; track<service_ptr->audioNumberOfTracks; track++)
        {
            if((audio_pid != 0) && (service_ptr->audioPid[track] == audio_pid)) {
                break;
            }
        }

        if((ttx_pid != 0) && (service_ptr->teletextPid == ttx_pid)) {
            break;
        }
    }

    /* no service found */
    if(service_index == appServiceCount) {
        return(NULL);
    }

    memcpy(&service_info, service_ptr, sizeof(SERVICELIST_ServiceInfo_T));

    if(video_pid != 0) {
        service_info.videoPid = video_pid;
    }
    if(audio_pid != 0) {
        service_info.audioPid[0] = audio_pid;
    }
    if(ttx_pid != 0) {
        service_info.teletextPid = ttx_pid;
    }

    return( &service_info );
}

/*
********************************************************************************
** \brief Get previous service.
**
** This function returns the previous available service from the actual service list.
**
** \return Valid pointer to the previous service or NULL if no service available.
**
********************************************************************************
*/
static SERVICELIST_ServiceInfo_T* appGetPrevService(void)
{
    SERVICELIST_ServiceInfo_T*  service_ptr;

    if ( !appServiceCount ) {
        appCurrentServiceIndex = -1;
        appServiceCount = SERVICELIST_GetSize();
        if ( !appServiceCount ) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] No service list available.\n" );
#if ENABLE_GUI_WINDOW == 1
            GUI_SetRunningCancel();
#endif
            return(NULL);
        }
    }

    if ( appCurrentServiceIndex <= 0 )
        appCurrentServiceIndex = (int32_t)appServiceCount;

    service_ptr = SERVICELIST_GetService((uint32_t)(appCurrentServiceIndex-1));
    if ( !service_ptr ) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Previous service not found.\n" );
        return(NULL);
    }
    appCurrentServiceIndex--;

    return( service_ptr );
}

/*
*****************************************************************************
**
** \brief HDMI auto sync
**
** This function handles the HDMI auto sync.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appHdmiAutoSync(void)
{
    int32_t                  err_code              = FAPI_OK;
    int32_t                  hdmi_delay            = 0;
    int32_t                  hdmi_interlaced_delay = 0;
    int32_t                  delay                 = 0;
    int32_t                  frame_time            = 0;
    FAPEX_VIDEC_VESParamStrT ves_params;

    memset(&ves_params, 0, sizeof(ves_params));

    HDMI_GetDelay(&hdmi_delay, &hdmi_interlaced_delay);
    /* delay information is invalid */
    if(hdmi_delay == 0xDEAD) {
        return;
    }

    err_code = FAPEX_VIDEC_Ioctl( VAL_GetIdVidec(vidOpenPtr),
                                  FAPEX_IOCCOM_VIDEC_GET_VES_PARAMS,
                                  (FAPEXIOCARG3_T)&ves_params );
    if(err_code == FAPI_OK)
    {
        /* determine progressive or interlaced */
        if(ves_params.frameField == 0)
            delay = hdmi_interlaced_delay;
        else
            delay = hdmi_delay;

        FAPI_SYS_PRINT_MSG( "[HDMI] %d ms %s delay (auto sync).\n", delay,
                            ( ves_params.frameField == 0 ) ? "interlaced" : "progressive" );

        /* set delay */
        if( delay >= 0)
        {
            (void)VAL_SetOutputDelay( vidDatPtr, 0 );
            AUDIO_SetOutputDelay( delay );
        }
        else
        {
            // in case video has to be delayed, we can divide the delay value
            // by the framrate/100 (frameRate) and then try to speed up audio
            // by the remainder
            frame_time = (int32_t)( 100000 / ves_params.frameRate );
            AUDIO_SetOutputDelay( ( ( delay * 100 ) / frame_time) % frame_time);
            err_code = VAL_SetOutputDelay( vidDatPtr, abs(delay) );
            if(err_code != FAPI_OK)
            {
                FAPI_SYS_PRINT_MSG("[VIDEO] Error upon setting delay (code: %d).\n", err_code);
            }
        }
        err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                                    FAPEX_IOCCOM_RESET,
                                    (FAPEXIOCARG3_T)NULL );
        if(err_code < 0) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] reset STC regulation. %d\n", err_code);
        }
    }
}

/*
********************************************************************************
** \brief Standby handler.
**
** This function is called when the system is set to standby or wakeup.
**
********************************************************************************
*/
static void appStandby(uint32_t autoStart)
{
    int32_t  result;
    SERVICELIST_ServiceInfo_T* service_ptr = NULL;

    /* system is in stand by */
    if(appStandbyMode)
    {
        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_START,
                                     (FAPEXIOCARG3_T)appStandbyMode );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] error on wakeup (code: %d).\n", result);
        }

        result = appOpenService(APP_THREAD_STATUS_OPENED);
        if( result < 0 ) {
            return;
        }

        result = TSD_Start(&appTsInputParams, appTsInputFileName, appTsInputStartOffset);
        if( result < 0 ) {
#if ENABLE_GUI_WINDOW == 1
            GUI_SetRunningCancel();
#endif
            return;
        }

        if(autoStart) {
            service_ptr = SERVICELIST_GetService( (uint32_t)(appCurrentServiceIndex) );
            if(service_ptr) {
                appChangeService( service_ptr, 1 );
            }
        }
        appStandbyMode = 0;
    }
    /* put system in stand by */
    else
    {
        FAPEX_VIOUT_FadingParamStrT  param;

        param.mode     = FAPEX_VIOUT_VIDEO_FADE_OUT;
        param.stepSize = 2;

        result = FAPEX_VIOUT_Ioctl( VAL_GetIdViscaleHd(vidDatPtr),
                                    FAPEX_IOCCOM_VIOUT_SET_FADING,
                                    (FAPEXIOCARG3_T)&param );

        result = OSD_DrawCurtain(1000);

        result = appCloseService(APP_THREAD_STATUS_OPENED);

        appStandbyMode = FAPEX_CLKPWR_BASIC_POWER_DOWN;

        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_STOP,
                                     (FAPEXIOCARG3_T)appStandbyMode );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] error on standby (code: %d).\n", result);
        }
    }
}

/*
********************************************************************************
** \brief set TS input
********************************************************************************
*/
static int32_t appSetTsInput (uint32_t tsSelect)
{
    appTsInputParams.useStc           = 1;
    appTsInputParams.tsSelect         = tsSelect;
    appTsInputParams.syncByteDistance = 0;
    appTsInputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    appTsInputParams.oflags           = 0;

    if(tsSelect < FAPEX_TSD_INPUT_DIRECT_MODE) {
        appTsInputStartOffset = 0;
        appPlaybackMode = VAL_ENV_LIVE;
    }
    else {
        appPlaybackMode = VAL_ENV_PLAY;
    }
#if ENABLE_GUI_WINDOW == 1
    GUI_SetPlaybackMode(appPlaybackMode);
#endif

    return 0;
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "start" command
**
** This function will be called by the shell thread if the "start" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_StartCommand( int32_t argc, char_t* argv[] )
{
    SERVICELIST_ServiceInfo_T *service_ptr;

    if( (argc != 2) && ((argc < 3 ) || (argc > 4 ))) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Wrong number of arguments for command \"%s\"\n", argv[0] );
        FAPI_SYS_PRINT_MSG( "               expected %d, %d or %d but got %d, type \"help\" for details\n",
                            2, 3, 4, argc );
        return( 0 );
    }
    if(appStandbyMode) {
        appStandby(0);
    }

    /* atoi is deprecated by MISRA */
    if(argc == 2)
    {
        /* service list number provided */
        appServicePtr = SERVICELIST_GetService((uint32_t)atoi(argv[1]));
        if(appServicePtr) {
            appCurrentServiceIndex = (int32_t)atoi(argv[1]);
        }
        else {
            FAPI_SYS_PRINT_MSG("ERROR:[main.c] Servide not available.\n");
            return( 0 );
        }
        appChangeService( appServicePtr, 1 );
        return( 1 );
    }

    // Stop current service
    pcrTsdStop();
    TSD_Stop();

    /* get service information */
    service_ptr = appGetServiceByPid(argc, argv);
    if(service_ptr == NULL) {
        return( 0 );
    }
    appStartService(service_ptr , 1);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "stop" command
**
** This function will be called by the shell thread if the "stop" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_StopCommand( int32_t argc, char_t* argv[] )
{
    //MISRA
    argc = argc;
    argv = argv;

    if(appStandbyMode)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    if(appPlaybackMode == VAL_ENV_LIVE) {
        appStopService();
    }
    else {
        appCloseService(APP_THREAD_STATUS_READY);
    }

    return(1);
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "scan" command
**
** This function will be called by the shell thread if the "scan" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_Scan( int32_t argc, char_t* argv[] )
{
    // MISRA
    argc = argc;
    argv = argv;

    if(appStandbyMode)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    if (appPlaybackMode == VAL_ENV_PLAY)
    {
    	FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in PVR mode."
    										"Quit PVR mode first\n" );
        return( 1 );
    }

    appStopService();

    SERVICELIST_ClearList();
    appServiceCount = 0;
    SCAN_Stop();
    FAPI_SYS_SLEEP(1000);
    SCAN_Start(trpListIndex);

    FAPEX_SYS_WakeupThread( appScanThreadHandle );

    return(1);
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "play" command
**
** This function will be called by the shell thread if the "play" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_PlaybackStart( int32_t argc, char_t* argv[] )
{
    SERVICELIST_ServiceInfo_T* service_ptr;
    int32_t  result;

    if(appStandbyMode) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    if ( argc < 2 )
    {
        if(appTsInputFileName[0] == '\0') {
            FAPI_SYS_PRINT_MSG("Filename argument missing.\n");
            return 0;
        }
        appTsInputStartOffset = 0;

        if(appPlaybackMode == VAL_ENV_LIVE) {
            goto APP_PlaybackStart_reset;
        }
        appStopService();

        service_ptr = SERVICELIST_GetService( (uint32_t)(appCurrentServiceIndex) );
        appChangeService( service_ptr, 1 );

        FAPI_SYS_PRINT_MSG("Playback restart\n");

        return 1;
    }
    else if ( strlen(argv[1]) > FAPEX_SYS_FMAXPATH )
    {
        FAPI_SYS_PRINT_MSG("Filename too long.\n");
        return 1;
    }

    // copy file name
    strncpy(appTsInputFileName, argv[1], FAPEX_SYS_FMAXPATH);

    // setup file offset
    if ( argc >= 3 ) {
        appTsInputStartOffset = atoi(argv[2]);
    }
    else {
        appTsInputStartOffset = 0;
    }

APP_PlaybackStart_reset:

    appCloseService(APP_THREAD_STATUS_READY);

    appSetTsInput(FAPEX_TSD_INPUT_LINEAR_AUTO_MODE); // reconfigure TSD block input settings

    result = appOpenService(APP_THREAD_STATUS_READY);
    if( result < 0 ) {
#if ENABLE_GUI_WINDOW == 1
        GUI_SetRunningCancel();
#endif
        appCloseService(APP_THREAD_STATUS_READY);
        return 0;
    }

    result = TSD_Start(&appTsInputParams, appTsInputFileName, appTsInputStartOffset);
    if( result < 0 ) {
#if ENABLE_GUI_WINDOW == 1
        GUI_SetRunningCancel();
#endif
        return 0;
    }

    APP_Scan(0, NULL);

    FAPI_SYS_PRINT_MSG("Playback start\n");

    return 1;
}

/*
*******************************************************************************
**
** \brief playback start
**
** \param none
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_KeyPlay(void)
{
    if(appTsInputFileName[0] != '\0') {
        APP_PlaybackStart( 1, NULL );
        return 1;
    }
    else {
        return 0;
    }
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "live" command
**
** This function will be called by the shell thread if the "live" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_KeyStop( int32_t argc, char_t* argv[] )
{
    int32_t  result;

    if(appStandbyMode) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    if(appPlaybackMode == VAL_ENV_LIVE) {
        return 1;
    }

    appCloseService(APP_THREAD_STATUS_READY);

    FAPI_SYS_PRINT_MSG("Playback stop\n");

    appSetTsInput(FAPEX_TSD_INPUT_CH_A); // reconfigure TSD block input settings

    result = appOpenService(APP_THREAD_STATUS_READY);
    if( result < 0 ) {
#if ENABLE_GUI_WINDOW == 1
        GUI_SetRunningCancel();
#endif
        appCloseService(APP_THREAD_STATUS_READY);
        return 0;
    }

    result = TSD_Start(&appTsInputParams, appTsInputFileName, appTsInputStartOffset);
    if( result < 0 ) {
#if ENABLE_GUI_WINDOW == 1
        GUI_SetRunningCancel();
#endif
        return 0;
    }

    APP_Scan(0, NULL);

    return 1;
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "p" command
**
** This function will be called by the shell thread if the "p" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_KeyPause ( int32_t argc, char_t* argv[] )
{
    static int32_t  pause_status = 0;
    int32_t  play_mode;
    int32_t  result;

    if(pause_status == 0) {
        TSD_Pause();
        play_mode    = FAPEX_VIDEC_PLAYBACK_MODE_PAUSE;
        pause_status = 1;
    }
    else {
        TSD_Restart();
        play_mode    = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
        pause_status = 0;
    }

    result = FAPEX_VIDEC_Ioctl( VAL_GetIdVidec(vidOpenPtr),
                                FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_MODE,
                                (FAPEXIOCARG3_T)play_mode );
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] set playback mode failed: %d\n",
                           result);
    }
    return 1;
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "dir" command
**
** This function will be called by the shell thread if the "dir" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_Dir( int32_t argc, char_t* argv[] )
{
#if 1
/* shared code for RTOS and Linux */
    FAPEX_SYS_FFindStrT  entry;
    char_t   path_name[APP_CMD_STR_LENGTH];
    uint32_t kbytes;
    uint32_t bytes;
    uint8_t  day, month;
    uint16_t year;
    uint8_t  sec, min, hour;

    if(argc > 1) {
        strncpy(path_name, argv[1], APP_CMD_STR_LENGTH-1);
    }
    else {
        strcpy(path_name, ".");
    }

    if ( !FAPEX_SYS_FFindFirst(path_name, &entry) ) {
        FAPI_SYS_PRINT_MSG( "Directory contents: \"%s\"\n", path_name );
        FAPI_SYS_PRINT_MSG( "Type Attr          Filename             kbytes.bytes     Date & Time\n" );
        FAPI_SYS_PRINT_MSG( "---- ---- ------------------------- --------------------------------------\n" );
        do
        {
            year   = 1980;
            kbytes = (entry.filesize) / 1024;
            bytes  = (entry.filesize) % 1024;
            day    = (entry.cdate & FAPEX_SYS_FCDATE_DAY_MASK)   >> FAPEX_SYS_FCDATE_DAY_SHIFT;
            month  = (entry.cdate & FAPEX_SYS_FCDATE_MONTH_MASK) >> FAPEX_SYS_FCDATE_MONTH_SHIFT;
            year  += (entry.cdate & FAPEX_SYS_FCDATE_YEAR_MASK)  >> FAPEX_SYS_FCDATE_YEAR_SHIFT;
            sec    = (entry.ctime & FAPEX_SYS_FCTIME_SEC_MASK)   >> FAPEX_SYS_FCTIME_SEC_SHIFT;
            min    = (entry.ctime & FAPEX_SYS_FCTIME_MIN_MASK)   >> FAPEX_SYS_FCTIME_MIN_SHIFT;
            hour   = (entry.ctime & FAPEX_SYS_FCTIME_HOUR_MASK)  >> FAPEX_SYS_FCTIME_HOUR_SHIFT;

            FAPI_SYS_PRINT_MSG( "  %c  0x%02X %25s %10u.%04u    %.2d.%.2d.%.4d %.2d:%.2d:%.2d\n",
                        (entry.attr & FAPEX_SYS_FATTR_DIR) ? 'D' : 'F',
                        entry.attr,
                        entry.filename,
                        kbytes,
                        bytes,
                        day,
                        month,
                        year,
                        hour,
                        min,
                        sec );
        } while (!FAPEX_SYS_FFindNext(&entry));

        FAPI_SYS_PRINT_MSG( "\n" );
    }
#else
/* depended code on Linux */
    char_t  command[APP_CMD_STR_LENGTH];

    if(argc > 1) {
        snprintf(command, APP_CMD_STR_LENGTH, "ls -l %s", argv[1]);
    }
    else {
        snprintf(command, APP_CMD_STR_LENGTH, "ls -l .");
    }
    system(command);
#endif
    return( 1 );
}

#ifdef _LINUX_FAPI  /* depended code on Linux */
/*
*******************************************************************************
**
** \brief Callback function to handle the "sys" command
**
** This function will be called by the shell thread if the "sys" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_Sys( int32_t argc, char_t* argv[] )
{
    char_t  command[APP_CMD_STR_LENGTH];
    int32_t i, cnt;

    if(argc <= 1) {
        return 0;
    }
    cnt = snprintf(command, APP_CMD_STR_LENGTH, "%s", argv[1]);

    if(cnt > 0) {
        for(i=2; i<argc; i++) {
            cnt += strlen(argv[i]) + 1;
            if(cnt > (APP_CMD_STR_LENGTH - 1)) {
                break;
            }
            strcat(command, " ");
            strcat(command, argv[i]);
        }
    }
    FAPI_SYS_PRINT_MSG("shell command : \"%s\"\n", command);

    system(command);

    return 1;
}
#endif /* _LINUX_FAPI */



#if 0 /* now not supported */
/*
********************************************************************************
**
** \brief Callback function to handle the "tune" command
**
** This function will be called by the shell thread if the "tune" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
***********************************************************************8********
*/
static uint32_t APP_Tune( int32_t argc, char_t* argv[] )
{
    FE_TUNE_PARAMS_S    tune_params;
    int32_t             index = 0;

    if(FRONTEND_GetDeliveryType() == FE_SYSTEM_TYPE_DIRECT)
    {   /* for baseband interface */
        FAPI_SYS_PRINT_MSG( "[main.c] 'tune' not supported with baseband IF\n");
        return( 1 );
    }

    if(appStandbyMode)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    /* stop everything */
    AUDIO_Stop();
    (void)OSD_Stop();
    (void)VAL_Stop(vidOpenPtr, 0);
    TTXVBI_Stop();

    if((argc==2) && (FRONTEND_GetDeliveryType() != FE_SYSTEM_TYPE_TERRESTRIAL) )
    {   /* select one of the in-built tune parameters */
        /* atoi is deprecated by MISRA */
        /*lint -save -e586 */
        index = (int32_t)atoi(argv[1]);
        /*lint -restore */
        if(FRONTEND_SetDefaultTuneParmeters(index, &tune_params) != FAPI_OK)
            return(0);
    }
    else
    {
        /* we only modify basic parameters as frequency etc so load full set first */
        if(FRONTEND_SetDefaultTuneParmeters(0, &tune_params) != FAPI_OK)
            return(0);

        switch(FRONTEND_GetDeliveryType())
        {
            case FE_SYSTEM_TYPE_DIRECT: return( 1 ); // just exit

            case FE_SYSTEM_TYPE_SATELLITE:
                /* scan mode sat */
                if( (argc<4) || (argc>5) )
                {
                    FAPI_SYS_PRINT_MSG( "\nuse: tune <transponderSelect>\n");
                    FAPI_SYS_PRINT_MSG( "     tune <freq MHz> <symbolrate KS/s> <horizontal|vertical>\n");
                    return( 1 );
                }
                /* atoi is deprecated by MISRA */
                /*lint -save -e586 */
                tune_params.sat.frequency  = (uint32_t)atoi(argv[1])*1000;
                tune_params.sat.symbolRate = (uint32_t)atoi(argv[2]);
                /*lint -restore */
                tune_params.sat.fec        = FE_FEC_AUTO;
                if(argv[3][0] == 'h')
                    tune_params.sat.pol = FE_POL_HOR;
                else
                    tune_params.sat.pol = FE_POL_VER;
                if( argc==5 )
                {
                    if( argv[4][0] == 's')
                        tune_params.sat.modulation = FE_MOD_QPSK_S2;
                    if( argv[4][0] == '8')
                        tune_params.sat.modulation = FE_MOD_8PSK;
                }
                break;

            case FE_SYSTEM_TYPE_TERRESTRIAL:
                /* scan mode for terr */
                if(argc!=2)
                {
                    FAPI_SYS_PRINT_MSG( "use: tune <freq MHz>\n");
                    return( 1 );
                }
                break;

            case FE_SYSTEM_TYPE_CABLE:
                /* scan mode cable */
                if(argc!=3)
                {
                    FAPI_SYS_PRINT_MSG( "\nuse: tune <transponderSelect>\n");
                    FAPI_SYS_PRINT_MSG( "     tune <freq MHz> <symbolrate KS/s>\n");
                    return( 1 );
                }
                break;

            default:
                FAPI_SYS_PRINT_MSG( "ERROR:[main.c] unknown NIM\n" );
                return( 1 );
        }
    }

    trpListIndex = index;

    /* tune to new transponder */
    if(FRONTEND_Tune(&tune_params) == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG( "[main.c] FE locked!\n" );
    }
    return(1);
}

/*
********************************************************************************
**
** \brief Callback function to handle the "tune" command
**
** This function will be called by the shell thread if the "tune" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
***********************************************************************8********
*/
static uint32_t APP_ConfigSat( int32_t argc, char_t* argv[] )
{
    FE_SYSTEM_PARAMS_S  sys_params;

    if(appStandbyMode)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    if((argc!=5) || (FRONTEND_GetDeliveryType() != FE_SYSTEM_TYPE_SATELLITE) )
    {
        return( 0 );
    }

    memset(&sys_params, 0, sizeof(sys_params));

    /* stop everything */
    AUDIO_Stop();
    (void)OSD_Stop();
    (void)VAL_Stop(vidOpenPtr, 1);
    TTXVBI_Stop();

    /* atoi is deprecated by MISRA */
    /*lint -save -e586 */
    sys_params.sat.lowFrequency  = 1000*(uint32_t)atoi(argv[1]);
    sys_params.sat.highFrequency = 1000*(uint32_t)atoi(argv[2]);
    sys_params.sat.toneburst = (uint8_t)atoi(argv[3]);
    sys_params.sat.diseqc    = (FE_DISEQC_E)atoi(argv[4]);
    /*lint -restore */
    sys_params.sat.classic       = 1;
    sys_params.sat.diseqc10Rep   = 0;
    sys_params.sat.diseqc11Rep   = 0;
    sys_params.sat.autoFEC       = 1;
    sys_params.sat.defaultIQ     = FE_IQ_AUTO;

    (void)FRONTEND_SetSatParmeters(&sys_params);

    return(1);
}
#endif

/*
********************************************************************************
**
** \brief Callback function to handle the "standby" command
**
** This function will be called by the shell thread if the "standby" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  upon completion.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
********************************************************************************
*/
static uint32_t APP_Standby( int32_t argc, char_t* argv[] )
{
    // MISRA
    argc = argc;
    argv = argv;

    appStandby(1);

    return(1);
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "list" command
**
** This function will be called by the shell thread if the "list" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1 if command could be handled successfully.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_ServiceList( int32_t argc, char_t* argv[] )
{
    uint32_t                    cnt = 0;
    SERVICELIST_ServiceInfo_T*  service;
    char_t                      service_name[31];

    //MISRA
    argc = argc;
    argv = argv;

    if(appStandbyMode)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] System is in standby mode.\n" );
        return( 1 );
    }

    cnt = SERVICELIST_GetSize();
    FAPI_SYS_PRINT_MSG("[main.c] number of services:%d\n", cnt);
    for(cnt=0; cnt<SERVICELIST_GetSize(); cnt++) {
        memset(service_name, 0, sizeof(service_name));
        service = SERVICELIST_GetService(cnt);

        if(!service)
            continue;

        service_name[0] = '\'';
        strcat(service_name, (char_t*)(service->name));
        strcat(service_name, "'\0");
        FAPI_SYS_PRINT_MSG("%2i. %s (ServiceID:%5i / TsIdx:%i)\n     ", cnt,
                                              service_name, service->serviceId,
                                              service->trpListIndex);
        if(service->videoPid != 0) {
            FAPI_SYS_PRINT_MSG("Video PID: %4i ", service->videoPid);
            SI_DEBUG_PsiStreamType(service->videoType);
        }
        if(service->audioPid[0] != 0) {
            if(service->videoPid != 0) {
                FAPI_SYS_PRINT_MSG("    ");
            }
            FAPI_SYS_PRINT_MSG("Audio PID: %4i ", service->audioPid[0]);
            SI_DEBUG_PsiStreamType(service->audioType[0]);
        }
        FAPI_SYS_PRINT_MSG("\n     PCR PID:   %4i ", service->pcrPid);
        if(service->teletextPid != 0) {
            FAPI_SYS_PRINT_MSG("              Teletext PID: %4i", service->teletextPid);
        }
        FAPI_SYS_PRINT_MSG("\n");
    }

    return(1);
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "up" command
**
** This function will be called by the shell thread if the "up" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_ServiceUp( int32_t argc, char_t* argv[] )
{
    SERVICELIST_ServiceInfo_T*  service_ptr;

    //MISRA
    argc = argc;
    argv = argv;

    if(appStandbyMode) {
        appStandby(1);
    }
    else {
        service_ptr = appGetNextService();
        if ( service_ptr ) {
            appChangeService( service_ptr, 1 );
        }
    }
    return(1);
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "down" command
**
** This function will be called by the shell thread if the "down" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_ServiceDown( int32_t argc, char_t* argv[] )
{
    SERVICELIST_ServiceInfo_T*  service_ptr;

    //MISRA
    argc = argc;
    argv = argv;

    if(appStandbyMode) {
        appStandby(1);
    }
    else {
        service_ptr = appGetPrevService();
        if ( service_ptr ) {
            appChangeService( service_ptr, 1 );
        }
    }
    return(1);
}

/*!
*******************************************************************************
**
** \brief Callback function to handle the "halt" command
**
** This function will be called by the shell thread if the "halt" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via APP_Printf.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via APP_Printf, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_HaltCommand( int32_t argc, char_t* argv[] )
{
    if( argc == 1 ) {
#ifdef USE_WATCHDOG
        {
            int32_t  result;

            result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                         FAPEX_IOCCOM_CLKPWR_WATCHDOG_STOP,
                                         (FAPEXIOCARG3_T)NULL );
            if(result < 0) {
                FAPI_SYS_PRINT_MSG("Watchdon Stop Error %d\n", result);
            }
        }
#endif

        if(appHdmiEnable == 0) {
            FAPEX_SYS_DestroyThread( appHdmiThreadHandle );
            HDMI_Close();
        }

        if(fd_IR0 > 0) {
            FAPEX_SYS_DestroyThread( appIrThreadHandle );
            FAPEX_IR_Close( fd_IR0 );
        }

        if(fd_FPC0 > 0) {
            FAPEX_SYS_DestroyThread( appFpcThreadHandle );
            FAPEX_FPC_Close( fd_FPC0 );
        }

        if(fd_CLKPWR > 0) {
            if(appStandbyMode) {
                appStandby(0);
            }
#ifdef USE_WATCHDOG
            FAPEX_SYS_DestroyThread( appWatchdogThreadHandle );
#endif /* USE_WATCHDOG */
            FAPEX_CLKPWR_Close( fd_CLKPWR );
        }

        appCloseService(APP_THREAD_STATUS_CREATED);

        FAPEX_SYS_DestroyThread( appScanThreadHandle );

        FAPI_SYS_PRINT_MSG( "%s Halting - Bye bye!!!\n\n", APP_PREFIX );
        exit( 1 );
    }
    return( 0 );
}

#ifdef USE_HDMICEC
/*
*******************************************************************************
**
** \brief Callback function to handle the "readedid" command
**
** This function will be called by the shell thread if the "readedid" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_HDMI_ReadEDID( int32_t argc, char_t* argv[] )
{
    int32_t status = FAPI_OK;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_ReadEDID();
    if(status != FAPI_OK) {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Read EDID failed. %d\n", status);
        return( 1 );
    }

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "checkhpd" command
**
** This function will be called by the shell thread if the "checkhpd" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_HDMI_CheckHPD( int32_t argc, char_t* argv[] )
{
    int32_t hpd_status = 0;

    //MISRA
    argc = argc;
    argv = argv;

    hpd_status = HDMI_CheckHPD();
    FAPI_SYS_PRINT_MSG("HPD status = %x\n", hpd_status);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecsendstatus" command
**
** This function will be called by the shell thread if the "cecsendstatus" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_CheckSendStatus( int32_t argc, char_t* argv[] )
{
    uint32_t status = 0;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_CEC_CheckSendStatus();
    FAPI_SYS_PRINT_MSG("Sending status = %x\n", status);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecrecstatus" command
**
** This function will be called by the shell thread if the "cecrecstatus" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_CheckReceiveStatus( int32_t argc, char_t* argv[] )
{
    uint32_t status = 0;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_CEC_CheckReceiveStatus();
    FAPI_SYS_PRINT_MSG("Receive status = %x\n", status);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cec" command
**
** This function will be called by the shell thread if the "cec" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_MessaageControlInManual( int32_t argc, char_t* argv[] )
{
    int32_t status = FAPI_OK;
    int32_t message = 0;
    int32_t send_flag = 0;
    int32_t rec_flag = 0;
    uint32_t InitiatorLogicalAddr = 0;
    uint32_t FollowerLogicalAddr = 0;
    uint8_t val1 = 0;
    uint8_t val2 = 0;

    if(argc != 6){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s [message] [send_flag] [rec_flag] [InitiatorLogicalAddr] [FollowerLogicalAddr]\n",argv[0]);
        return( 0 );
    }

    if((*(argv[1]) >= '0') && (*(argv[1]) <= '9')){
        val1 = (uint8_t)(*(argv[1]) - '0');
    }else if((*(argv[1]) >= 'a') && (*(argv[1]) <= 'f')){
        val1 = (uint8_t)(*(argv[1]) - ('a' - 10));
    }else if((*(argv[1]) >= 'A') && (*(argv[1]) <= 'F')){
        val1 = (uint8_t)(*(argv[1]) - ('A' - 10));
    }
    if((*(argv[1]+1) >= '0') && (*(argv[1]+1) <= '9')){
        val2 = (uint8_t)(*(argv[1]+1) - '0');
    }else if((*(argv[1]+1) >= 'a') && (*(argv[1]+1) <= 'f')){
        val2 = (uint8_t)(*(argv[1]+1) - ('a' - 10));
    }else if((*(argv[1]+1) >= 'A') && (*(argv[1]+1) <= 'F')){
        val2 = (uint8_t)(*(argv[1]+1) - ('A' - 10));
    }
    message = (val1 * 16) + val2;

    if((message > 0xff) && (message < 0xffff)){
        FAPI_SYS_PRINT_MSG("arg error(message)\n");
    }
    /* atoi is deprecated by MISRA */
    /*lint -save -e586 */
    send_flag = (int32_t)atoi(argv[2]);
    if((send_flag < 0) || (send_flag > 1)){
        FAPI_SYS_PRINT_MSG("arg error(send_flag)\n");
    }
    rec_flag = (int32_t)atoi(argv[3]);
    if((rec_flag < 0) || (rec_flag > 1)){
        FAPI_SYS_PRINT_MSG("arg error(rec_flag)\n");
    }
    InitiatorLogicalAddr = (uint32_t)atoi(argv[4]);
    if(InitiatorLogicalAddr > 15){
        FAPI_SYS_PRINT_MSG("arg error(InitiatorLogicalAddr)\n");
    }
    FollowerLogicalAddr = (uint32_t)atoi(argv[5]);
    if(FollowerLogicalAddr > 15){
        FAPI_SYS_PRINT_MSG("arg error(FollowerLogicalAddr)\n");
    }

    status = HDMI_CEC_SendOrReceiveMessage(message, send_flag, rec_flag, InitiatorLogicalAddr, FollowerLogicalAddr);
    if(status != FAPI_OK){
        FAPI_SYS_PRINT_MSG("HDMI_CEC_SendOrReceiveMessage status = %d\n",status);
    }

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "devinsaddr" command
**
** This function will be called by the shell thread if the "devinsaddr" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_DeviceInstrationAndAddressing( int32_t argc, char_t* argv[] )
{
#if 0
    int32_t status = FAPI_OK;

    if(argc != 1){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s\n",argv[0]);
        return( 0 );
    }

    status = HDMI_CEC_DeviceInstrationAndAddressing();
    if(status != FAPI_OK){
        FAPI_SYS_PRINT_MSG("HDMI_CEC_DeviceInstrationAndAddressing status = %d\n",status);
    }

    return( 1 );
#else
    int32_t status = FAPI_OK;
    int32_t count = 0;
    int32_t i = 0;

    if(argc != 2){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s [count]\n",argv[0]);
        return( 0 );
    }
    count = atoi(argv[1]);
    if((count < 0) || (count > 255)){
        FAPI_SYS_PRINT_MSG("arg error(count)\n");
        return( 0 );
    }

    for(i=0; i<count; i++){
        status = HDMI_CEC_DeviceInstrationAndAddressing();
        if(status != FAPI_OK){
            FAPI_SYS_PRINT_MSG("HDMI_CEC_DeviceInstrationAndAddressing status = %d\n",status);
            return( 1 );
        }
    }
    FAPI_SYS_PRINT_MSG("APP_CEC_DeviceInstrationAndAddressing %x\n", count);

    return( 1 );
#endif
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "allocatelogicaladdr" command
**
** This function will be called by the shell thread if the "allocatelogicaladdr" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_AllocateLogicalAddress( int32_t argc, char_t* argv[] )
{
    int32_t status = FAPI_OK;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_CEC_AllocateLogicalAddress();
    if(status != FAPI_OK){
        FAPI_SYS_PRINT_MSG("HDMI_CEC_AllocateLogicalAddress status = %d\n",status);
    }

    return( 0 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecgetmsg" command
**
** This function will be called by the shell thread if the "cecgetmsg" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_GetMsg( int32_t argc, char_t* argv[] )
{
    int32_t status = 0;
    uint32_t RecInitiator = 0;
    uint32_t RecMessage = 0;
    uint32_t RemainRecMsg = 0;
    uint32_t MessageType = 0;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_CEC_GetMsg(&RecInitiator, &RecMessage, &RemainRecMsg, &MessageType);
    if(status != FAPI_OK){
        FAPI_SYS_PRINT_MSG("HDMI_CEC_GetMsg status = %d\n",status);
    }

    return( 1 );

}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecrecmsg" command
**
** This function will be called by the shell thread if the "cecrecmsg" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_RecMsg( int32_t argc, char_t* argv[] )
{
    int32_t status = 0;

    //MISRA
    argc = argc;
    argv = argv;

    status = HDMI_CEC_ReceivingMessage();
    if(status != FAPI_OK){
        FAPI_SYS_PRINT_MSG("HDMI_CEC_ReceivingMessage status = %d\n",status);
    }

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecsetlogicaladdr" command
**
** This function will be called by the shell thread if the "cecsetlogicaladdr" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_SetLogicalAddress( int32_t argc, char_t* argv[] )
{
    int32_t MyDevLogicalAddr = 0;

    if(argc != 2){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s\n",argv[0]);
        return( 0 );
    }
    MyDevLogicalAddr = (int32_t)atoi(argv[1]);

    HDMI_CEC_SetLogicalAddress(MyDevLogicalAddr);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecgetlogicaladdr" command
**
** This function will be called by the shell thread if the "cecgetlogicaladdr" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_GetLogicalAddress( int32_t argc, char_t* argv[] )
{
    uint32_t MyDevLogicalAddr;

    if(argc != 1){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s\n",argv[0]);
        return( 0 );
    }
    MyDevLogicalAddr = HDMI_CEC_GetLogicalAddress();
    FAPI_SYS_PRINT_MSG("MyDevLogicalAddr = %d\n",MyDevLogicalAddr);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "cecsetparam" command
**
** This function will be called by the shell thread if the "cecsetparam" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_SetCecMessageParameters( int32_t argc, char_t* argv[] )
{
    uint32_t opcodeName;
    int32_t cnt = 0;
    uint8_t val1 = 0;
    uint8_t val2 = 0;
    uint8_t cecparambuf[14];

    if((argc <= 1) || (argc >= 16)){
        FAPI_SYS_PRINT_MSG("arg error.\n");
        FAPI_SYS_PRINT_MSG("usage : %s  [opcode] [cecparameter: Sets it with a hex data]\n",argv[0]);
        return( 0 );
    }

    if((*(argv[1]) >= '0') && (*(argv[1]) <= '9')){
        val1 = (uint8_t)(*(argv[1]) - '0');
    }else if((*(argv[1]) >= 'a') && (*(argv[1]) <= 'f')){
        val1 = (uint8_t)(*(argv[1]) - ('a' - 10));
    }else if((*(argv[1]) >= 'A') && (*(argv[1]) <= 'F')){
        val1 = (uint8_t)(*(argv[1]) - ('A' - 10));
    }
    if((*(argv[1]+1) >= '0') && (*(argv[1]+1) <= '9')){
        val2 = (uint8_t)(*(argv[1]+1) - '0');
    }else if((*(argv[1]+1) >= 'a') && (*(argv[1]+1) <= 'f')){
        val2 = (uint8_t)(*(argv[1]+1) - ('a' - 10));
    }else if((*(argv[1]+1) >= 'A') && (*(argv[1]+1) <= 'F')){
        val2 = (uint8_t)(*(argv[1]+1) - ('A' - 10));
    }
    opcodeName = (val1 * 16) + val2;

    if((opcodeName > 0xff) && (opcodeName < 0xffff)){
        FAPI_SYS_PRINT_MSG("arg error(count),Range 1..255\n");
        return( 0 );
    }

    for(cnt=2; cnt<=(argc-1); cnt++){
        if((*(argv[cnt]) >= '0') && (*(argv[cnt]) <= '9')){
            val1 = (uint8_t)(*(argv[cnt]) - '0');
        }else if((*(argv[cnt]) >= 'a') && (*(argv[cnt]) <= 'f')){
            val1 = (uint8_t)(*(argv[cnt]) - ('a' - 10));
        }else if((*(argv[cnt]) >= 'A') && (*(argv[cnt]) <= 'F')){
            val1 = (uint8_t)(*(argv[cnt]) - ('A' - 10));
        }
        if((*(argv[cnt]+1) >= '0') && (*(argv[cnt]+1) <= '9')){
            val2 = (uint8_t)(*(argv[cnt]+1) - '0');
        }else if((*(argv[cnt]+1) >= 'a') && (*(argv[cnt]+1) <= 'f')){
            val2 = (uint8_t)(*(argv[cnt]+1) - ('a' - 10));
        }else if((*(argv[cnt]+1) >= 'A') && (*(argv[cnt]+1) <= 'F')){
            val2 = (uint8_t)(*(argv[cnt]+1) - ('A' - 10));
        }
        cecparambuf[cnt-2] = (uint8_t)((val1 * 16) + val2);
    }

    HDMI_CEC_SetCecMessageParameters(opcodeName, cecparambuf);

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "sendosdstring" command
**
** This function will be called by the shell thread if the "sendosdstring" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_SendOSDStringMessage( int32_t argc, char_t* argv[] )
{
    uint32_t cnt;
    int32_t status = FAPI_OK;
    uint32_t initiatorLogicalAddr = 0;
    uint32_t followerLogicalAddr = 0;
    uint32_t FollowrLogicalAddr[2] = {0,14};
    uint32_t ack = 0;
    //MISRA
    argc = argc;
    argv = argv;

    initiatorLogicalAddr = HDMI_CEC_GetLogicalAddress();

    for(cnt=0; cnt<2; cnt++){
       followerLogicalAddr = FollowrLogicalAddr[cnt];
       status = HDMI_CEC_SendMessage_PollingMessage(1, 0, initiatorLogicalAddr, followerLogicalAddr, &ack);
       if(status != FAPI_OK){
           FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_PollingMessage status = %d\n",status);
            return( 0 );
       }
       if(ack == 1){
           status = HDMI_CEC_SendMessage_SetOSDString(1, 0, initiatorLogicalAddr, followerLogicalAddr);
            if(status != FAPI_OK){
                FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_SetOSDString status = %d\n",status);
                return( 0 );
            }
            return( 1 );
        }
    }/* for */

    return( 1 );
}

/*
*******************************************************************************
**
** \brief Callback function to handle the "onetouchplay" command
**
** This function will be called by the shell thread if the "onetouchplay" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
** - 1  if command could not be handled successfully (because of bad
**         parameters), but the handler already reported the error condition
**         via FAPI_SYS_PRINT_MSG.
** - 0 if command could not be handled successfully (because of bad
**         parameters) and the handler has not reported the error condition
**         via FAPI_SYS_PRINT_MSG, in that case a common error message will
**         be created automatically.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
static uint32_t APP_CEC_OneTouchPlay( int32_t argc, char_t* argv[] )
{
    uint32_t cnt;
    int32_t status = FAPI_OK;
    uint32_t initiatorLogicalAddr = 0;
    uint32_t followerLogicalAddr = 0;
    uint8_t UpperPhysAddr = 0;
    uint8_t LowerPhysAddr = 0;
    uint32_t FollowrLogicalAddr[2] = {0,14};
    uint32_t ack = 0;
    //MISRA
    argc = argc;
    argv = argv;

    initiatorLogicalAddr = HDMI_CEC_GetLogicalAddress();

    for(cnt=0; cnt<2; cnt++){
       followerLogicalAddr = FollowrLogicalAddr[cnt];
       status = HDMI_CEC_SendMessage_PollingMessage(1, 0, initiatorLogicalAddr, followerLogicalAddr, &ack);
       if(status != FAPI_OK){
           FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_PollingMessage status = %d\n",status);
           return( 0 );
       }
       if(ack == 1){
           status = HDMI_CEC_SendMessage_ImageViewOn(1, 0, initiatorLogicalAddr, followerLogicalAddr);
           if(status != FAPI_OK){
               FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_ImageViewOn status = %d\n",status);
               return( 0 );
           }

           status = HDMI_GetPhysicalAddressInEDID(&UpperPhysAddr, &LowerPhysAddr);
           if(status != FAPI_OK){
               FAPI_SYS_PRINT_MSG("HDMI_GetPhysicalAddressInEDID status = %d\n",status);
               return( 0 );
           }else{
               FAPI_SYS_PRINT_MSG("Physycal Address = %x\n",((UpperPhysAddr << 8) | LowerPhysAddr));
           }

           status = HDMI_CEC_SendMessage_ActiveSource(1, 0, initiatorLogicalAddr, followerLogicalAddr, UpperPhysAddr, LowerPhysAddr);
           if(status != FAPI_OK){
               FAPI_SYS_PRINT_MSG("HDMI_CEC_SendMessage_ImageViewOn status = %d\n",status);
               return( 0 );
           }

           return( 1 );
        }
    }/* for */

    return( 1 );
}
#endif /* USE_HDMICEC */



#ifdef _LINUX_FAPI
/**
*******************************************************************************
**
** \brief Print the shell prompt.
**
** This function prints the shell prompt in green color on white background
** and resets the color to blue which is used for echoing the command input.
**
*******************************************************************************
*/
static void APP_PrintPrompt(void)
{
    FAPI_SYS_PRINT_MSG( "\r%s%s%s%s%s",
                        APP_ColorBgWhite, APP_ColorFgGreen,
                        APP_SHELL_PROMPT,
                        APP_ColorBgWhite, APP_ColorFgBlack );
}
#endif /* _LINUX_FAPI */

static uint8_t PVRSYS_GetLiveVideoFormat(void)
{
    uint8_t pvr_track_format = PVR_TRACK_FORMAT_OTHER;

    switch (appServicePtr->videoType)
    {   
        case VID_MPEG1:
        case VID_MPEG2:
            pvr_track_format = PVR_TRACK_FORMAT_VIDEO_MPEG2;
            FAPI_SYS_PRINT_DEBUG(1, "[APP ] PVRSYS_GetLiveVideoFormat: Video = MPEG2 \n");
            break;

        case VID_H264:
            pvr_track_format = PVR_TRACK_FORMAT_VIDEO_H264;
            FAPI_SYS_PRINT_DEBUG(1, "[APP ] PVRSYS_GetLiveVideoFormat: Video = H264 \n");
            break;
        default:
            break;
    }

    return pvr_track_format;
}

static uint8_t PVRSYS_GetLiveAudioFormat(uint8_t track)
{
    uint8_t pvr_track_format = PVR_TRACK_FORMAT_OTHER;

    switch (appServicePtr->audioType[track])
    {
        case AUD_AC3_ATSC_A52:
        case USER_DEF_AC_3:
             pvr_track_format = PVR_TRACK_FORMAT_AUDIO_AC3;
             break;
        case AUD_AAC_MPEG2:
            pvr_track_format = PVR_TRACK_FORMAT_AUDIO_AAC_MPEG2;
            break;
        case AUD_AAC_MPEG4:
            pvr_track_format = PVR_TRACK_FORMAT_AUDIO_AAC_MPEG4;
            break;
        case USER_DEF_EAC_3:
            pvr_track_format = PVR_TRACK_FORMAT_AUDIO_DDPDCV;
            break;
        default:
            pvr_track_format = PVR_TRACK_FORMAT_AUDIO_MPEG2;
    }

    return pvr_track_format;
}

static int32_t PVRSYS_TestFile(const char* file_path)
{
    int32_t result;

    /* Using F_OK to test the existence of file */
    result = access(file_path, F_OK);

    if (result == 0)
    {
        return 0;
    }
    else 
    {
        return -1;
    }
}

/*
*****************************************************************************
* don't directly call PVR lib interface in the next two callback function
* do like that may be conflict with the PVR thread.
* Send a command and make non-pvr thread call PVR lib interface
*****************************************************************************
*/
    
static void 
PVRSYS_PlayCallback (FAPEX_SYS_HandleT playHandle,PVR_PlayNotifyT reason)
{
    int32_t result;

    result = FAPI_OK;
    FAPI_SYS_PRINT_DEBUG(1,">>> Playback Callback = ");

    switch (reason)
    {
        case PVR_PLAY_NOTIFY_FILE_ERROR:    
        	FAPI_SYS_PRINT_DEBUG(1,"FILE ERROR\n");    
        	pvrSetCallbackCmd(PVRCALLBACK_CMD_PLAYBACK_RESUME);
        	FAPEX_SYS_WakeupThread(appPvrThreadHandle);
        	break;
        case PVR_PLAY_NOTIFY_END_OF_DATA:
      
            FAPI_SYS_PRINT_DEBUG(1,"END OF DATA\n");
            pvrSetCallbackCmd(PVRCALLBACK_CMD_PLAYBACK_STOP);

            pvr_end_ts = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
            printf("[APP ] PVRSYS_PlayCallback: playend [%lld]\n", pvr_end_ts);
            printf("[APP ] PVRSYS_PlayCallback: playend, using [%lld] ms\n", (pvr_end_ts - pvr_begin_ts));
            FAPEX_SYS_WakeupThread(appPvrThreadHandle);
            break;
        case PVR_PLAY_NOTIFY_BEGIN_OF_DATA: 
        	FAPI_SYS_PRINT_DEBUG(1,"BEGIN OF DATA\n"); 
        	pvrSetCallbackCmd(PVRCALLBACK_CMD_GET_PLAYBACK_INFO);
        	FAPEX_SYS_WakeupThread(appPvrThreadHandle);
        	break;
        case PVR_PLAY_NOTIFY_GAP_IN_DATA:   FAPI_SYS_PRINT_DEBUG(1,"GAP IN DATA");     break;
        case PVR_PLAY_NOTIFY_TAG_SET:       FAPI_SYS_PRINT_DEBUG(1,"TAG SET");         break;
        case PVR_PLAY_NOTIFY_NAME_SET:      FAPI_SYS_PRINT_DEBUG(1,"NAME SET");        break;
        case PVR_PLAY_NOTIFY_IFRAME_LOCKED: FAPI_SYS_PRINT_DEBUG(1,"IFRAME FOUND AND LOCKED"); break;
        default:                            FAPI_SYS_PRINT_DEBUG(1,"%u", (uint32_t)reason); break;
    }

    FAPI_SYS_PRINT_DEBUG(1,"\n");

}

static 
void PVRSYS_RecCallback (FAPEX_SYS_HandleT handle,
                                 PVR_RecordNotifyT reason)
{
    uint8_t             recIdx = 0;

    printf("\n>>> REC Callback (idx %i) = ", recIdx);

    switch (reason)
    {
        case PVR_RECORD_NOTIFY_FILE_ERROR:        printf("FILE ERROR");
                                                  break;
        case PVR_RECORD_NOTIFY_DISK_FULL:         printf("DISK FULL");
                                                  break;
        case PVR_RECORD_NOTIFY_MAX_SIZE:          printf("MAX SIZE");
                                                  break;
        case PVR_RECORD_NOTIFY_STOP_POSITION:     printf("STOP POS");
                                                  break;
        case PVR_RECORD_NOTIFY_FILLING:           printf("FILLING");
                                                  break;
        case PVR_RECORD_NOTIFY_LOOPING:           printf("LOOPING");
                                                  break;
        case PVR_RECORD_NOTIFY_NO_DATA:           printf("NO DATA");
                                                  break;
        case PVR_RECORD_NOTIFY_DATA:              printf("DATA");
                                                  break;
        case PVR_RECORD_NOTIFY_PERFORMANCE_LIMIT: printf("PERFORMANCE LIMIT");
                                                  break;
        default:                                  printf("%u", (uint32_t)reason);
                                                  break;
    }

    FAPI_SYS_PRINT_DEBUG(0,"\n");

}

static int32_t
PVRSYS_PlayChangeMode (FAPI_SYS_HandleT handlePlay,
                                PVR_PlayModeT mode, uint8_t modeIndex)
{
    PVR_PlayModeT   playMode;
    uint8_t         playModeIndex;
    PVR_PlayModeT   lastPlayMode;
    int32_t         retVal;
    uint64_t        tStart = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    uint8_t         timeout = 0;

    retVal = PVR_GetPlaybackMode (handlePlay, &playMode, &playModeIndex);

    if ( retVal == FAPI_OK )
    {
        if ( (playMode      != mode)
          || (playModeIndex != modeIndex) )
        {
            lastPlayMode = playMode;

            (void)PVR_SetPlaybackMode (handlePlay, mode, modeIndex);

            timeout = 1;

            do
            {
                retVal = PVR_GetPlaybackMode (handlePlay, &playMode,
                                              &playModeIndex);

                if ( retVal != FAPI_OK )
                {
                    break;
                }
                else if ( (playMode      == mode)
                       && (playModeIndex == modeIndex) )
                {
                    /* requested mode successfully set */
                    timeout = 0;
                    break;
                }
                else if ( (playMode != lastPlayMode)
                       && (playMode != PVR_PLAY_MODE_CHANGING) )
                {
                    /* mode has been changed by pvr_lib, set requested mode again */
                    lastPlayMode = playMode;

                    (void)PVR_SetPlaybackMode (handlePlay, mode, modeIndex);
                }

                FAPI_SYS_SLEEP (50);
            }
            while ( FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) - tStart
                    < 10000 );

            if ( (retVal == FAPI_OK) && timeout )
            {
                retVal = SUBSYS_PVR_ERR_TIMEOUT;

                FAPI_SYS_PRINT_DEBUG(2,"[PVR] PVRSYS_PlayChangeMode %i: Timeout \n", mode);
            }
        }
    }

    FAPI_SYS_PRINT_DEBUG(4,"[PVR] Change to mode %i index %i: result %i\n", mode,
               modeIndex, retVal);
    return retVal;
}

static int32_t
PVRSYS_RecordChangeMode (FAPEX_SYS_HandleT handlePlay,
                                PVR_PlayModeT mode, uint8_t modeIndex)
{
    PVR_RecordModeT   recordMode;
    PVR_PlayModeT   lastRecordMode;
    int32_t         retVal;
    uint64_t        tStart = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    uint8_t         timeout = 0;

    retVal = PVR_GetRecordingMode(handlePlay, &recordMode);

    if ( retVal == FAPI_OK )
    {
        if ( recordMode != mode)
        {
            lastRecordMode = recordMode;

            (void)PVR_SetRecordingMode (handlePlay, mode);

            timeout = 1;

            do
            {
                retVal = PVR_GetRecordingMode (handlePlay, &recordMode);

                if ( retVal != FAPI_OK )
                {
                    break;
                }
                else if ( (recordMode == mode))
                {
                    /* requested mode successfully set */
                    timeout = 0;
                    break;
                }
                else if ( (recordMode != lastRecordMode)
                       && (recordMode != PVR_RECORD_MODE_CHANGING) )
                {
                    /* mode has been changed by pvr_lib, set requested mode
                       again */
                    lastRecordMode = recordMode;

                    (void)PVR_SetRecordingMode (handlePlay, mode);
                }

                FAPI_SYS_SLEEP (50);
            }
            while ( FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) - tStart
                    < 10000 );

            if ( (retVal == FAPI_OK) && timeout )
            {
                retVal = SUBSYS_PVR_ERR_TIMEOUT;

                FAPI_SYS_PRINT_DEBUG(2,"[PVR] PVRSYS_RecordChangeMode %i: Timeout \n", mode);
            }
        }
    }

    FAPI_SYS_PRINT_DEBUG(4,"[PVR] Change to mode %i index %i: result %i\n", mode,
               modeIndex, retVal);
    return retVal;
}

static void
PVRSYS_ThreadFunction(void* optArg)
{
    APP_ThreadStatus_SetValue( &appPvrThreadStatus, APP_THREAD_STATUS_CREATED );
    while (1)
    {
        // wait start trigger
        APP_ThreadStatus_SetValue( &appPvrThreadStatus, APP_THREAD_STATUS_OPENED);
        APP_ThreadStatus_Lock(&appPvrThreadStatus);
        appPvrThreadStatus.value |= APP_THREAD_STATUS_READY;
        appPvrThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
        APP_ThreadStatus_Unlock(&appPvrThreadStatus);
        FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread());

        APP_ThreadStatus_SetValue( &appPvrThreadStatus, APP_THREAD_STATUS_STARTED);

        switch (pvrGetCallbackCmd())
        {
            case PVRCALLBACK_CMD_PLAYBACK_STOP:
            	printf("[PBCB]: get playback stop command\n");
                PVRSYS_PlaybackStop();
                break;
                
            case PVRCALLBACK_CMD_PLAYBACK_RESUME:
            	printf("[PBCB]: get playback resume command\n");
                PVRSYS_PlaybackResume();
                break;

            case PVRCALLBACK_CMD_LOOP:
                {
                    PVR_PlayModeT playMode = -1;
                    uint8_t   modeIndex;
                    if (pbHandle != NULL)
                    {
                        PVR_GetPlaybackMode(pbHandle, &playMode, &modeIndex);
                        while(playMode != PVR_PLAY_MODE_PAUSE && playMode != PVR_PLAY_MODE_STOPPED)
                        {
                            if (playMode == PVR_PLAY_MODE_PAUSE || playMode == PVR_PLAY_MODE_STOPPED)
                            {
                                break;
                            }
                            PVR_GetPlaybackMode(pbHandle, &playMode, &modeIndex);
                        }
                        PVR_SeekPlayback(pbHandle, PVR_SEEK_MODE_CHUNKS, 0);
                        PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);
                    }
                }
                break;
                
            case PVRCALLBACK_CMD_GET_PLAYBACK_INFO:
            	printf("[PBCB]: get playback info command\n");
            	{
            		PVR_PlayInfoT pb_info;
            		int32_t ret;

            		memset(&pb_info, 0, sizeof(PVR_PlayInfoT));
            		if (pbHandle)
            		{
						ret = PVRSYS_PlayWaitForTransitionEnd(pbHandle);
            			if (ret == FAPI_OK)
            			{
	            			ret = PVR_GetPlaybackInfo(pbHandle, &pb_info);
	            			if (ret == FAPI_OK)
	            			{
	            				printf("[PBCB]: INFO: time[%d] pos[%d]\n",\
	            							pb_info.time, pb_info.position);
	            			}
	            			else
	            			{
	            				printf("[PBCB]: INFO: get playback info failed[%d]\n",\
	            							ret);
	            			}
            			}
            		}
            	}
            	break;
            default:
                break;
        }
    }
    
}

static void 
PVRSYS_TimeDispThreadFunction(void *optData)
{
	PVR_PlayInfoT pb_info;
	uint32_t 	pb_time_ms = 0; /*unit: ms*/
	uint32_t 	pb_time_s = 0;	/*unit: s*/
	int32_t	    ret;

	printf("[APP ] Play time: start\n");

	while (1)
	{
		if (pbHandle)
		{
			ret = PVRSYS_PlayWaitForTransitionEnd(pbHandle);
			if (ret == FAPI_OK)
			{
				ret = PVR_GetPlaybackInfo(pbHandle, &pb_info);
				if (ret == FAPI_OK)
				{
					if (pb_info.time > pb_time_ms)
					{
						if (pb_info.time /1000 > pb_time_s)
						{
							pb_time_s = pb_info.time / 1000;
							pb_time_ms = pb_info.time;
							printf("[APP ] Play time[%04d] Total time[%04d]\n",\
											pb_time_s, pb_info.duration / 1000);
						}
						else
						{
							/*  less sleep time, more accurate display time & more
							 *   system load
							 */
							FAPI_SYS_SLEEP(50);
						}
					}
					else if (pb_info.time < pb_time_ms)
					{	/*rewind happed, reset the pb time stamp (both ms & s)*/
						pb_time_ms = pb_info.time;
						pb_time_s = pb_info.time / 1000;
						printf("[APP ] Play time[%04d] Total time[%04d]\n",\
											pb_time_s, pb_info.duration / 1000);
					}
					else
					{
						/* do nothing but sleep*/
						FAPI_SYS_SLEEP(50);
					}
				}
				else
				{
					printf("[APP ] Play time: get playback info failed[%d]\n", ret);
				}
			}
		}
		else
		{
			FAPI_SYS_SLEEP(50);
			break;
		}
	}
	pvrTimeDispThread = NULL;
	printf("[APP ] Play time: end\n");
}

static int32_t 
PVRSYS_PlaybackStart(const char *file_name, PVR_PlayModeT init_mode)
{
    PVR_PlayOpenT      playOpen;    
    int32_t            err_code;
    int32_t            result;
    FAPEX_TSD_InputParamStrT inputParams;
    char              file_path[128];

    if (init_mode != PVR_PLAY_MODE_NORMAL && \
        init_mode != PVR_PLAY_MODE_PAUSE)
    {
        printf("[APP ] PVRSYS_PlaybackStart: Unsupported init_mode\n");
        return -1;
    }
        
    memset(file_path, 0, sizeof(file_path));
    sprintf(file_path, "%s%s", pvr_mnt_path, file_name);
    if (PVRSYS_TestFile(file_path) != 0)
    {
        printf("[APP ] PVRSYS_PlaybackStart: [%s] not exists\n", file_path);
        return -1;
    }

    if (appPlaybackMode == VAL_ENV_LIVE)
    {
        result = VAL_Stop(vidOpenPtr, 1);
        printf("[APP ] APP_PvrPlayback: VAL_Stop[%d]\n", result);
        result = VAL_Close(vidOpenPtr);
        printf("[APP ] APP_PvrPlayback: VAL_Close[%d]\n", result);
        vidOpenPtr = NULL;

        result = AUDIO_Stop();
        printf("[APP ] APP_PvrPlayback: AUDIO_Stop[%d]\n", result);
        result = AUDIO_Close();
        printf("[APP ] APP_PvrPlayback: AUDIO_Close[%d]\n", result);

    }

    PVR_SetMountPath(pvr_mnt_path);

    /*prepare, redefine the input*/
    inputParams.useStc           = 0;
    inputParams.tsSelect         = FAPEX_TSD_INPUT_LINEAR_AUTO_MODE;
    inputParams.syncByteDistance = 0;
    inputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    inputParams.oflags           = 0;
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)&inputParams );

    if(result < 0) {
        printf("[APP ] %s: set input error %d\n", __FUNCTION__, result );
        return result;
    }

    memset (&playOpen, 0, sizeof(playOpen));

    playOpen.version = PVR_VERSION;
    sprintf(playOpen.fileName, "%s", file_name);
    playOpen.readThreadPriority     = FAPEX_SYS_PRIORITY_HIGH;/*30 -> org*/
    playOpen.feedThreadPriority     = FAPEX_SYS_PRIORITY_MEDIUM;/*31 -> org*/
    playOpen.tsdBlockIndex          = FAPEX_TSD0;
    if (playOpen.tsdBlockIndex < 0){
        return 0;
    }
    
    playOpen.tsdBlockFd             = fd_TSD[playOpen.tsdBlockIndex];
    playOpen.iFrameBufferAddress    = 0;
    playOpen.iFrameBufferSize       = 8 * PVR_CHUNK_SIZE;
    playOpen.iFrameTsdChannelNo     = IFRAME_TSD_CHANNEL;
    playOpen.bufferToCheck          = 0;
    playOpen.initVideoDecoderAlways = 0;
    playOpen.callback = (PVR_PlayCallback)PVRSYS_PlayCallback;
    playOpen.startPosition = 0;

    /* attach decoder interface functions */
    PVRDEC_RegisterFunctions (&(playOpen.targetControl));
    
    pbHandle = PVR_OpenPlayback(&playOpen, &err_code);

    if (err_code != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[APP ] APP_PvrPlayback: PVR_OpenPlayback faile[%d]\n",\
                    err_code);
        return -1;
    }

    appPlaybackMode = VAL_ENV_PLAY;

    
    result = PVRSYS_PlayChangeMode(pbHandle, init_mode, 0);
    
    return result;
}


static int32_t 
PVRSYS_TimeshiftPlaybackStart(const char *file_name, PVR_PlayModeT init_mode)
{
    PVR_PlayOpenT	playOpen;    
    int32_t         err_code;
    int32_t         result;
    char            file_path[128];
    
    FAPEX_SYS_HandleT			ts_rec_handle = NULL;
    FAPEX_TSD_InputParamStrT 	inputParams;

    if (init_mode != PVR_PLAY_MODE_NORMAL && \
        init_mode != PVR_PLAY_MODE_PAUSE)
    {
        printf("[APP ] PVRSYS_TimeshiftPlaybackStart: Unsupported init_mode\n");
        return -1;
    }
        
    memset(file_path, 0, sizeof(file_path));
    sprintf(file_path, "%s%s", pvr_mnt_path, file_name);

    PVR_SetMountPath(pvr_mnt_path);

    ts_rec_handle = PVRSYS_GetCurrentTimeshiftRecording();
    if (NULL == ts_rec_handle)
    {
    	return -1;
    }
	/*wait until recording size > 0 to start playback successfully*/
    result = PVRSYS_RecWaitForStart(ts_rec_handle);
    if (result != FAPI_OK)
    {
    	printf("[APP ] PVRSYS_TimeshiftPlaybackStart: wait rec start failed[%d]\n", result);
    	return -1;
    }

    /*prepare, redefine the input*/
    inputParams.useStc           = 0;
    inputParams.tsSelect         = FAPEX_TSD_INPUT_LINEAR_AUTO_MODE;
    inputParams.syncByteDistance = 0;
    inputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    inputParams.oflags           = 0;
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)&inputParams );

    if(result < 0) {
        printf("[APP ] %s: set input error %d\n", __FUNCTION__, result );
        return result;
    }

    memset (&playOpen, 0, sizeof(playOpen));

    playOpen.version = PVR_VERSION;
    sprintf(playOpen.fileName, "%s", file_name);
    playOpen.readThreadPriority     = FAPEX_SYS_PRIORITY_HIGH;/*30 -> org*/
    playOpen.feedThreadPriority     = FAPEX_SYS_PRIORITY_MEDIUM;/*31 -> org*/
    playOpen.tsdBlockIndex          = FAPEX_TSD0;
    if (playOpen.tsdBlockIndex < 0){
        return -1;
    }
    
    playOpen.tsdBlockFd             = fd_TSD[playOpen.tsdBlockIndex];
    playOpen.iFrameBufferAddress    = 0;
    playOpen.iFrameBufferSize       = 8 * PVR_CHUNK_SIZE;
    playOpen.iFrameTsdChannelNo     = IFRAME_TSD_CHANNEL;
    playOpen.bufferToCheck          = 0;
    playOpen.initVideoDecoderAlways = 0;
    playOpen.callback = (PVR_PlayCallback)PVRSYS_PlayCallback;
    playOpen.startPosition = 0;

    /* attach decoder interface functions */
    PVRDEC_RegisterFunctions (&(playOpen.targetControl));
    
    pbHandle = PVR_OpenPlayback(&playOpen, &err_code);

    if (err_code != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[APP ] %s: PVR_OpenPlayback faile[%d]\n",\
                    									__FUNCTION__, err_code);
        return err_code;
    }

    appPlaybackMode = VAL_ENV_PLAY;

    err_code = PVRSYS_PlayChangeMode(pbHandle, init_mode, 0);
    
    return err_code;
}


static int32_t 
PVRSYS_PlaybackStop(void)
{
    int32_t result;
    VAL_OpenParamsT   vidOpenParams;
    int32_t           err_code;
    FAPEX_TSD_InputParamStrT inputParams;


    if (pbHandle == NULL)
    {
        printf("[APP ] PVRSYS_PlaybackStop: no playback running\n");
        return -1;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_STOPPED, 0);

    if (result != FAPI_OK)
    {
        printf("[APP ] PVRSYS_PlaybackStop: change to stop failed,(%d)\n", result);
        return result;
    }

    result = PVR_ClosePlayback(pbHandle);
    if (result != FAPI_OK)
    {
        printf("[APP ] PVRSYS_PlaybackStop: close play back failed(%d)\n",result);
    }
    else
    {
        pbHandle = NULL;
    }

    if (pvrTimeDispThread)
    {
    	FAPEX_SYS_WaitEndThread(pvrTimeDispThread);
    	FAPEX_SYS_DestroyThread(pvrTimeDispThread);
    	pvrTimeDispThread = NULL;
    }
    
    /*VAL was closed in PVR lib when playback shut down*/
    vidOpenPtr = NULL;


    ///////////////////////////////////////////////////////////////////////////
    // Restart the Live TV
    ///////////////////////////////////////////////////////////////////////////

    /*prepare, redefine the input*/
    inputParams.useStc           = 1;
    inputParams.tsSelect         = FAPEX_TSD_INPUT_CH_A;
    inputParams.syncByteDistance = 0;
    inputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    inputParams.oflags           = 0;
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)&inputParams );


    if (NULL != appServicePtr)
    {
        switch ( appServicePtr->videoType )
        {
            case VID_MPEG1:
            case VID_MPEG2:
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
                break;

            case VID_H264:
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_H264;
                break;

            default:
                APP_ThreadStatus_SetValue( &appVideoThreadStatus,\
                									  APP_THREAD_STATUS_READY );
                return -1;
        }
        vidOpenParams.environment =  VAL_ENV_LIVE;
        vidOpenParams.decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
        vidOpenParams.h264AudType = VAL_H264_AUD_TYPE_AUTO;
        vidOpenParams.enableAfd = 1;
        vidOpenPtr = VAL_Open(vidDatPtr, &vidOpenParams, &err_code);

        result = VAL_Start(vidOpenPtr, appServicePtr->videoPid,
        					VAL_PLAYMODE_NORMAL, VAL_STREAM_SEGMENT_FULL, 0);

		 #if 1
        ///////////////////////////////////////
        // Set PCR PID
        ///////////////////////////////////////
        if ( (appServicePtr->pcrPid != appServicePtr->videoPid) &&\
              (appServicePtr->pcrPid != appServicePtr->audioPid[0]))
        {
            FAPEX_TSD_PidParamStrT  pidParam;
            pidParam.pid        = (uint32_t)appServicePtr->pcrPid;
            pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
            pidParam.bufferSize = 0;
            result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                                       FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                                       (FAPEXIOCARG3_T)&pidParam );
        }
        ///////////////////////////////////////
        // Set STC REGULATION
        ///////////////////////////////////////
        (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)vidOpenPtr->videcId,
                            FAPEX_IOCCOM_SYNC_SET_STC_REGULATION,
                            1);
        #endif
		
        printf("[APP ] APP_PvrPlayStop: AUDIO_Close[%d]\n", AUDIO_Close());
        printf("[APP ] APP_PvrPlayStop: AUDIO_Open[%d]\n", AUDIO_Open());
        printf("[APP ] APP_PvrPlayStop: AUDIO_Start[%d]\n", \
                AUDIO_Start(appServicePtr->audioPid[0], 0, appServicePtr->audioType[0]));
        
        
    }
    appPlaybackMode = VAL_ENV_LIVE;
    return 0;
}

static int32_t 
PVRSYS_PlaybackResume(void)
{
    int32_t result;

    if (pbHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG("[APP ] PVRSYS_PlaybackResume: no playback running\n");
        return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[APP ] PVRSYS_PlaybackResume:"
        						   "change to normal failed(%d)\n", result);
        return result;
    }

    return result;
}

static int32_t 
PVRSYS_RecordResume(void)
{
    int32_t result;

    if (recHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG("[APP ] PVRSYS_RecordResume: no recording running\n");
        return SUBSYS_PVR_ERR_BAD_PARAMETER;
    }

    result = PVRSYS_RecordChangeMode(pbHandle, PVR_RECORD_MODE_NORMAL, 0);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[APP ] PVRSYS_RecordResume: change to normal failed(%d)\n", result);
        return result;
    }

    return FAPI_OK;
}

static int32_t 
PVRSYS_PlayWaitForTransitionEnd (FAPEX_SYS_HandleT handle)
{
    PVR_PlayModeT   playMode;
    int32_t         retVal;
    uint64_t        tStart = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);

    do
    {
        retVal = PVR_GetPlaybackMode (handle, &playMode, NULL);

        if ( retVal != FAPI_OK )
        {
            return retVal;
        }

        if ( playMode != PVR_PLAY_MODE_CHANGING ) return FAPI_OK;

        FAPI_SYS_SLEEP (50);
    }
    while ( FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) - tStart
            < 10000 );

    FAPI_SYS_PRINT_MSG("[APP ] PVRSYS_PlayWaitForTransitionEnd: Timeout \n");

    return SUBSYS_PVR_ERR_TIMEOUT;
}

static int32_t
PVRSYS_RecordStart(const char* file_name)
{
    PVR_RecordOpenT     rec_open;
    int32_t             result;
    int32_t 			fdTsdBlock;
    uint8_t 			indexTsdBlock;
    uint8_t				rec_index = 0;

	if (NULL == file_name)
	{
		printf("[APP ] PVRSYS_RecordStart: invalid file name\n");
		return -1;
	}

    if (recHandle1 != NULL && recHandle2 != NULL)
	{
		printf("[APP ] PVRSYS_RecordStart: no more record channel avaible\n");
		return -1;
	}
	
	if (recHandle1 == NULL)
	{
		fdTsdBlock = fd_TSD[2];
		indexTsdBlock = FAPEX_TSD2;
		rec_index = 1;

		printf("[APP ] PVRSYS_RecordStart: starting recording using record [1]\n");
	} 
	else if (recHandle2 == NULL)
	{
		fdTsdBlock = fd_TSD[3];
		indexTsdBlock = FAPEX_TSD3;
		rec_index = 2;
		printf("[APP ] PVRSYS_RecordStart: starting recording using record [2]\n");
	}
	else
	{
		printf("\n[APP ] PVRSYS_RecordStart: no more record channel avaible\n");
		return -1;
	}
    
    memset(&rec_open, 0, sizeof(PVR_RecordOpenT));
    rec_open.bufferAddress  = 0;             /* will be assign in hwOpen*/   
    rec_open.bufferSize     = 20 * 512 * 188;  /* 2M */
    rec_open.callback       = PVRSYS_RecCallback; /* called when rec evt happens*/
    rec_open.enableLooping  = 0;             /* no circular recording*/
    sprintf(rec_open.fileName, "%s", file_name);

    rec_open.fillingLimit   = 0;
    rec_open.tsdBlockFd     = fdTsdBlock;
    rec_open.fileSplitSize  = 0x7FFFFFFF/(PVR_CHUNK_SIZE);/* 2G / 1Chunk*/
    rec_open.maxSize        = 0;
    rec_open.tag            = 0;
    rec_open.overwrite      = 1;
    rec_open.tsdBlockIndex  = indexTsdBlock;

    rec_open.numberOfTracks = 2;

    /* assigning the pid */
    rec_open.trackPid[0] = appServicePtr->videoPid;//appServicePtr->videoPid;
    printf("[APP ] PVRSYS_RecordStart: video PID [%d]\n", rec_open.trackPid[0]);
    rec_open.trackType[0] = PVR_TRACK_TYPE_VIDEO;
    rec_open.trackFormat[0] = PVRSYS_GetLiveVideoFormat();

    rec_open.trackPid[1] = appServicePtr->audioPid[0];//appServicePtr->audioPid[0];
    printf("[APP ] PVRSYS_RecordStart: audio PID [%d]\n", rec_open.trackPid[1]);
    rec_open.trackType[1] = PVR_TRACK_TYPE_AUDIO;
    rec_open.trackFormat[1] = PVRSYS_GetLiveAudioFormat(0);;

    rec_open.pcrPid = appServicePtr->pcrPid;

    /* Open the Recording */
    recHandle = NULL;
    recHandle = PVR_OpenRecording(&rec_open, &result);
    if (recHandle == NULL || result != FAPI_OK)
    {
        printf("[APP ] PVRSYS_RecordStart: Open recording failed (%d)\n", result);
        return result;
    }
    else
    {
        printf("[APP ] PVRSYS_RecordStart: Open recording success (%d)\n", result);
    }
    
    /* Configuration Recording*/
    result = PVR_SetFlushingFrequency(recHandle, 256);

    /* ignore performance */
    result = PVR_SetRecordingPerfLimitTimeout (recHandle, 0);

    /* Start Recording */
    result = PVRSYS_RecordChangeMode(recHandle, PVR_RECORD_MODE_NORMAL, 0);


	if (rec_index == 1)
	{
		recHandle1 = recHandle;
	}
	else if (rec_index == 2)
	{
		recHandle2 = recHandle;
	}
	else
	{
		printf("\n[APP ] invalid record index\n");	
		return SUBSYS_PVR_ERR_BAD_PARAMETER;
	}

	return FAPI_OK;
}

static int32_t 
PVRSYS_RecWaitForStart (FAPEX_SYS_HandleT rec_handle)
{
    PVR_RecordInfoT recInfo;
    int32_t         retVal;
    uint64_t        tStart = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);

    if ( rec_handle == NULL )
    {
        return -1;
    }

    do
    {
        retVal = PVR_GetRecordingInfo (rec_handle, &recInfo);

        if ( retVal != FAPI_OK )
        {
            return retVal;
        }

        /* wait for first data chunk to be written to target device */
        if ( recInfo.size > 0 ) return FAPI_OK;

        FAPI_SYS_SLEEP (50);
    }
    while ( FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) - tStart
            < 10000 );

    FAPI_SYS_PRINT_DEBUG(1,"[APP ] PVRSYS_RecWaitForStart: Timeout \n");

    return -1;
}

FAPEX_SYS_HandleT
PVRSYS_GetCurrentTimeshiftRecording(void)
{
	PVR_RecordOpenT rec_open;
	int32_t ret;

	if (recHandle1 != NULL)
	{
		ret = PVR_GetRecordingOpen(recHandle1, &rec_open);
		if (ret == FAPI_OK)
		{
			printf("[APP ] PVRSYS_GetCurrentTimeshiftRecording: rec 1 [%s]\n", rec_open.fileName);
			if (strncmp(rec_open.fileName, pvr_timeshift_file_name, strlen(pvr_timeshift_file_name)) == 0)
			{
				printf("[APP ] PVRSYS_GetCurrentTimeshiftRecording: rec 1 found\n");
				return recHandle1;
			}
		}
		
	}
	
	if (recHandle2 != NULL)
	{
		ret = PVR_GetRecordingOpen(recHandle2, &rec_open);
		if (ret == FAPI_OK)
		{
			printf("[APP ] PVRSYS_GetCurrentTimeshiftRecording: rec 2 [%s]\n", rec_open.fileName);
			if (strncmp(rec_open.fileName, pvr_timeshift_file_name, strlen(pvr_timeshift_file_name)) == 0)
			{
				printf("[APP ] PVRSYS_GetCurrentTimeshiftRecording: rec 2 found\n");
				return recHandle2;
			}
		}
		
	}

	printf("[APP ] PVRSYS_GetCurrentTimeshiftRecording: no rec found\n");
	return NULL;
}

FAPEX_SYS_HandleT
PVRSYS_FindCurrentRecordingByFileName(const char *file_name)
{
	PVR_RecordOpenT rec_open;
	int32_t ret;

	if (NULL == file_name)
	{
		return NULL;
	}
	
	if (recHandle1 != NULL)
	{
		ret = PVR_GetRecordingOpen(recHandle1, &rec_open);
		if (ret == FAPI_OK)
		{
			printf("[APP ] PVRSYS_FindCurrentRecordingByFileName: rec 1 [%s]\n", rec_open.fileName);
			if (strncmp(rec_open.fileName, file_name, strlen(file_name)) == 0)
			{
				printf("[APP ] PVRSYS_PlayCurrentRecording: rec 1 found\n");
				return recHandle1;
			}
		}
		
	}
	
	if (recHandle2 != NULL)
	{
		ret = PVR_GetRecordingOpen(recHandle2, &rec_open);
		if (ret == FAPI_OK)
		{
			printf("[APP ] PVRSYS_FindCurrentRecordingByFileName: rec 2 [%s]\n", rec_open.fileName);
			if (strncmp(rec_open.fileName, file_name, strlen(file_name)) == 0)
			{
				printf("[APP ] PVRSYS_FindCurrentRecordingByFileName: rec 2 found\n");
				return recHandle2;
			}
		}
		
	}

	printf("[APP ] PVRSYS_FindCurrentRecordingByFileName: no rec found\n");
	return NULL;
}


static uint32_t 
APP_PvrRecord(int32_t argc, char_t* argv[])
{
    int32_t	result;

    printf("\n[APP ] ====== Enter Record(ASync) ======\n");
    if (argc <= 1)
    {
        return 0;
    }

    if (argc >= 3)
    {
        PVR_SetMountPath(argv[2]);
    }
    else
    {
        PVR_SetMountPath(pvr_mnt_path);
    }

	result = PVRSYS_RecordStart(argv[1]);
	if (result != FAPI_OK)
	{
		FAPI_SYS_PRINT_MSG("[APP ] APP_PvrRecord: start recording failed[%d]\n",\
																		result);
		return 0;
	}
    printf("\n[APP ] ====== Quit Record(ASync) ======\n\n");
    return 1;
}

static uint32_t 
APP_PvrPlayback(int32_t argc, char_t* argv[])
{
    PVR_PlayOpenT    playOpen;    
    int32_t          err_code;
    int32_t          result;
    char             file_path[128];
    uint8_t          modeIndex = 0;
    uint16_t		 fast_factor = 0; 

	FAPEX_TSD_InputParamStrT inputParams;
	FAPEX_SYS_HandleT		 cur_rec_handle = NULL;
    
    printf("\n[APP ] ====== Enter Playback ======\n");

    /* close last playback */
    if (pbHandle != NULL)
    {
        printf("[APP ] %s: playback already running, shut it down first\n",
                                                          __FUNCTION__);
        return 0;
    
    }
    
    if (argc >= 2){
        printf("[APP ] file name ==> [%s]\n", argv[1]);
    } else {
        printf ("Invalid file path,quit\n");
        return 0;
    }

    cur_rec_handle = PVRSYS_FindCurrentRecordingByFileName(argv[1]);
    if (NULL == cur_rec_handle)
    {
    	 /* test file */
	    memset(file_path, 0, sizeof(file_path));
	    sprintf(file_path, "%s%s", pvr_mnt_path, argv[1]);
	    if (0 != PVRSYS_TestFile(file_path))
	    {
	        printf("[APP ] APP_PvrPlayback: [%s] not exists\n", file_path);
	        return 0;
	    }
    }
    else
    {
    	/*wait until recording size > 0 to start playback successfully*/
	    result = PVRSYS_RecWaitForStart(cur_rec_handle);
	    if (result != FAPI_OK)
	    {
	    	printf("[APP ] PVRSYS_TimeshiftPlaybackStart: wait rec start failed[%d]\n", result);
	    	return 0;
	    }
    }
   
    if (appPlaybackMode == VAL_ENV_LIVE)
    {
        result = VAL_Stop(vidOpenPtr, 1);
        printf("[APP ] APP_PvrPlayback: VAL_Stop[%d]\n", result);

        //Debug return
        result = VAL_Close(vidOpenPtr);
        printf("[APP ] APP_PvrPlayback: VAL_Close[%d]\n", result);
        vidOpenPtr = NULL;

        result = AUDIO_Stop();
        printf("[APP ] APP_PvrPlayback: AUDIO_Stop[%d]\n", result);
        result = AUDIO_Close();
        printf("[APP ] APP_PvrPlayback: AUDIO_Close[%d]\n", result);

    }

    PVR_SetMountPath(pvr_mnt_path);

    

    /*
         *
	  * shortcut for start Fastfoward; Debug use only;
        */
    if (argc >= 3)
    {
        fast_factor  = atoi(argv[2]);
        switch (fast_factor)
        {
        	case 2:
        		modeIndex = 24;
        		break;
            case 4:
                modeIndex = 25;
                break;
            case 8:
                modeIndex = 26;
                break;
            case 16:
                modeIndex = 27;
                break;
            case 32:
                modeIndex = 28;
                break;
            case 64:
            	modeIndex = 29;
            	break;
            case 128:
            	modeIndex = 30;
            	break;
            case 256:
            	modeIndex = 31;
            	break;
            default:
            	FAPI_SYS_PRINT_DEBUG(1, "[APP ] APP_PvrPlayback:"
            						"error fast factor, enter normal play\n");
                modeIndex = 0;
        }
    }
    
    /*prepare, redefine the input*/
    inputParams.useStc           = 0;
    inputParams.tsSelect         = FAPEX_TSD_INPUT_LINEAR_AUTO_MODE;
    inputParams.syncByteDistance = 0;
    inputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    inputParams.oflags           = 0;
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)&inputParams );

    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[tsd.c] set input error %d\n",
                            result );
        return result;
    }
    
    memset (&playOpen, 0, sizeof(playOpen));

    playOpen.version = PVR_VERSION;
    sprintf(playOpen.fileName, "%s", argv[1]);
    playOpen.readThreadPriority     = 99;/*30 -> org*/
    playOpen.feedThreadPriority     = 99;/*31 -> org*/
    playOpen.tsdBlockIndex          = FAPEX_TSD0;
    if (playOpen.tsdBlockIndex < 0){
        return 0;
    }
    
    playOpen.tsdBlockFd             = fd_TSD[playOpen.tsdBlockIndex];
    playOpen.iFrameBufferAddress    = 0;


   	playOpen.iFrameBufferSize       = 8 * PVR_CHUNK_SIZE;

    playOpen.iFrameTsdChannelNo     = IFRAME_TSD_CHANNEL;
    playOpen.bufferToCheck          = 0;
    playOpen.initVideoDecoderAlways = 0;
    playOpen.callback = (PVR_PlayCallback)PVRSYS_PlayCallback;
    playOpen.startPosition = 0;

    /* attach decoder interface functions */
    PVRDEC_RegisterFunctions (&(playOpen.targetControl));
    
    pbHandle = PVR_OpenPlayback(&playOpen, &err_code);

    if (err_code != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[APP ] APP_PvrPlayback: PVR_OpenRecording faile[%d]\n",\
                    err_code);
        // TODO: restart live   if existed 
        return 0;
    }

    appPlaybackMode = VAL_ENV_PLAY;

    {
        PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);
    }

    if (modeIndex != 0)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[APP ] APP_PvrPlayback: start in fastmode [%d]\n", modeIndex);
        
        PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_FAST, modeIndex);
    }
    else
    {
    	if(pvrTimeDispThread == NULL) {
       		pvrTimeDispThread = FAPEX_SYS_CreateThread(
                                                NULL, 1024, 20,
                                                PVRSYS_TimeDispThreadFunction,
                                                NULL, NULL, "pvrTimeDispThread" );
        }
    }
    pvr_begin_ts = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    FAPI_SYS_PRINT_DEBUG(1, "[APP ] APP_PvrPlayback: play start[%lld]\n", \
                                                                 pvr_begin_ts);
    printf("[APP ] ====== Quit Playback ======\n\n");

    return 1;
}


static uint32_t 
APP_PvrPause(int32_t argc, char_t* argv[])
{
    int32_t result;
    int32_t ret = 1;
    char   opt_p[2] = "p";  /* pause playback */
    char   opt_r[2] = "r";  /* pause recording */
    char   opt_l[2] = "l";  /* pause live and enter timeshifting */

    printf("\n[APP ] ====== Enter Pause ======\n");


    if (argc < 2)
    {
        printf("[APP ] miss opt(%d)\n", -1);
        ret = 0;
        goto end;
    }

    if (strncmp(opt_p, argv[1], 1) == 0)
    {
        if (pbHandle == NULL)
        {
            FAPI_SYS_PRINT_MSG("[APP ] APP_PvrPause:"
            				    "change to pause failed, not started yet\n");
            ret = 0;
            goto end;
        }
        
        result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_PAUSE, 0);

        if (result != FAPI_OK)
        {
            printf("[APP ] change to pause playing failed(%d)\n", result);
            ret = 0;
            goto end;
        }
        pvrlive_pause_status = 1;
        goto end;
    }
    else if (strncmp(opt_r, argv[1], 1) == 0)
    {
        if (recHandle == NULL)
        {
            printf("[APP ] change to pause failed, not recording yet(%d)\n", -1);
            ret = 0;
            goto end;
        }
        
        result = PVRSYS_RecordChangeMode(recHandle, PVR_RECORD_MODE_PAUSE, 0);

        if (result != FAPI_OK)
        {
            printf("[APP ] change to pause recording failed(%d)\n", result);
            ret = 0;
            goto end;
        }
        goto end;
    }
    /* Pause the live decoding and enter background recording for timeshift */
    else if (strncmp(opt_l, argv[1], 1) == 0)
    {
        if (appPlaybackMode == VAL_ENV_PLAY)
        {
            printf("[APP ] APP_PvrPause: !!!Warning: You can't pause live in playback mode, try 'pa p'\n");
            ret = 0;
            goto end;
        }
    
        if (recHandle)
        {
            /*recoder already open*/
            if (appPlaybackMode == VAL_ENV_LIVE)
            {
                //VAL_Freeze(vidOpenPtr, 1);
                VAL_Stop(vidOpenPtr, 1);
                VAL_Close(vidOpenPtr);
                vidOpenPtr = NULL;

            }else if (appPlaybackMode == VAL_ENV_PLAY)
            {
                if (pbHandle)
                {
                    PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_PAUSE, 0);
                }
                else
                {
                    printf("[APP ] appPlayBackMode[%d], pbHandle[%d]\n", \
                            appPlaybackMode, (uint32_t)pbHandle);
                }
            }
            else
            {
                printf("[APP ] invalid appPlaybackMode\n");
                ret = 0;
                goto end;
            }
            
        }
        else
        {
            /* Freeze Live*/
            result = VAL_Stop(vidOpenPtr, 1);
            if (result != FAPI_OK)
            {
                printf("[APP ] APP_PvrPause: Freeze Picture failed[%d]\n", result);
                ret = 0;
                goto end;
            }
            result = AUDIO_Pause();

            //result = AUDIO_Close();
            if (result == FAPI_OK)
            {
                printf("[APP ] APP_PvrPause: LIVE ==> PAUSE\n");
                pvrlive_pause_status = 1;
            }
            else
            {
                printf("[APP ] APP_PvrPause: Disable AUOUT failed[%d]\n", result);
            }
        
            /* Start recorder background */
            PVR_SetMountPath(pvr_mnt_path);
            result = PVRSYS_RecordStart(pvr_timeshift_file_name);

            if (result == FAPI_OK)
            {
            	result = PVRSYS_TimeshiftPlaybackStart(pvr_timeshift_file_name,PVR_PLAY_MODE_PAUSE);
            	if (result != FAPI_OK)
            	{
            		printf("[APP ] APP_PvrPause: start timeshift playback [%d]\n", result);	
            	}
            }
            
            if (result == FAPI_OK)
            {
                ret = 1;
                goto end;
            }
        }
    }
    else
    {
        printf("[APP ] wrong opt(%d)\n", -1);
        ret = 0;
        goto end;
    }
    
end:
    printf("[APP ] ====== Quit Pause ======\n\n");
    return ret;
}

static uint32_t 
APP_PvrResume(int32_t argc, char_t* argv[])
{
    int32_t result;
    int32_t ret = 1;

    printf("\n[APP ] ====== Enter Resume ======\n");

    if (pvrlive_pause_status == 0)
    {
        printf("[APP ] APP_PvrResume: Not in any pause mode (Live Pause or Playback Pause), quit\n");
        ret = 0;
        goto end;
    }

    /* Paused in Playback Mode */ 
    if (appPlaybackMode == VAL_ENV_PLAY)
    {
        if (pbHandle == NULL)
        {
            printf("[APP ] APP_PvrResume: appPlaybackMode(VAL_ENV_PLAY) conflict with pbHandle(NULL)\n");
            ret = 0;
            goto end;
        }
        result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);

        if (result != FAPI_OK)
        {
            printf("[APP ] APP_PvrResume: change to normal playback failed(%d)\n", result);
            ret = 0;
            goto end;
        }
        pvrlive_pause_status = 0;
    }
    /* Live paused earlier, background recording is running */
    else if (appPlaybackMode == VAL_ENV_LIVE)
    {
        /* Check the existence background recording */
        if (recHandle == NULL)
        {
            printf("[APP ] APP_PvrResume: can't resume live pause, no related recording running\n");
            /*UnFreeze the picture and audio*/
            if (vidOpenPtr)
            {
                VAL_Freeze(vidOpenPtr, 0);
            }
            AUDIO_Resume();

            ret = 0;
            goto end;
        }

        /* Background recording running, start playback to resume the paused*/
        result = PVRSYS_PlaybackStart(pvr_timeshift_file_name, PVR_PLAY_MODE_PAUSE);

        if (result == FAPI_OK && pbHandle != NULL)
        {
            PVR_SeekPlayback(pbHandle, PVR_SEEK_MODE_CHUNKS, 1);
            PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);
        }
        
        // TODO: Check the result       
        if (result != FAPI_OK)
        {
            printf("[APP ] APP_PvrResume: start playback for timeshift failed\n");
            ret = 0;
            goto end;
        }
        pvrlive_pause_status = 0;
    }
    else
    {
        printf("[APP ] APP_PvrResume: Bad appPlaybackMode(%d), can't resume\n", appPlaybackMode);
        ret = 0;
        goto end;
    }
    
end:
    printf("[APP ] ====== Quit Resume ======\n\n");
    return ret;
}

static uint32_t 
APP_PvrFastFoward(int32_t argc, char_t* argv[])
{
    int32_t 		result;
    PVR_TrackInfoT	track_info;
    uint8_t			track   = 0xFF;
    uint8_t			mode_index;
    uint8_t			type_index; /*0: MPEG 1: H264*/
    uint16_t 		speed_factor = 2; /* default speed factor is 2*/
    uint8_t			speed_index = 0;  /*calculated by speed_factor */
    
    printf("\n[APP ] ====== Enter FF ======\n");
    if (pbHandle == NULL)
    {
        printf("[APP ] change to fastfoward failed, not started yet(%d)\n", -1);
        return 0;
    }
    // 25 --> H264 FF 4
    // 26 --> H264 FF 8
    // 10  --> MPEG I Frame FF
    // 8   --> MPEG I Frame FF 2


	result = PVR_GetPlaybackTrack(pbHandle, PVR_DECTYPE_VIDEO, &track);
	if (result != FAPI_OK)
	{
		printf("[APP ] get video track failed,(%d)\n", result);
		return 0;
	}
	else
	{
		printf("[APP ] get video track success,(%d)\n", track);
	}

	result = PVR_GetPlaybackTrackInfo(pbHandle, &track_info);
	if (result != FAPI_OK)
	{
		printf("[APP ] get track info failed,(%d)\n", result);
		return 0;
	}
	else
	{
		printf("[APP ] get track info success\n");
	}

	switch (track_info.trackFormat[track])
	{
		case PVR_TRACK_FORMAT_VIDEO_MPEG1:
		case PVR_TRACK_FORMAT_VIDEO_MPEG2:
			type_index = 0;
			break;

		case PVR_TRACK_FORMAT_VIDEO_H264:
			type_index = 1;
			break;

		default:
			printf("[APP ] invalid video type,(%d)\n",\
												track_info.trackFormat[track]);
			return 0;
	}

	if (argc >= 2)
	{
		speed_factor = (uint16_t)atoi(argv[1]);
		if (speed_factor != 2 && speed_factor != 4 && speed_factor != 8 &&\
			speed_factor != 16 && speed_factor != 32 && speed_factor != 64 &&\
			speed_factor != 128 && speed_factor != 256)
		{
			printf("[APP ] invalid speed factor,(%d)\n", speed_factor);
			return 0;
		}
	}
	printf("[APP ] speed factor[%d] ", speed_factor);
	/* speed factor to speed index */
	while (speed_factor > 2)
	{
		speed_index++;
		speed_factor /= 2;
	}
	
	/* speed factor to mode index */
	mode_index = 16 * type_index  + speed_index + 8; 
    printf("speed index[%d] mode index[%d]\n", speed_index, mode_index);
    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_FAST, mode_index);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to fastfoward failed,(%d)\n", result);
        return 0;
    }
    printf("[APP ] ====== Quit FF ======\n\n");
    return 1;
}

static uint32_t 
APP_PvrFastBackward(int32_t argc, char_t* argv[])
{

#if 0
    int32_t result;

    printf("\n[APP ] ====== Enter FB ======\n");
    if (pbHandle == NULL)
    {
        printf("[APP ] change to fastbackward failed, not started yet(%d)\n", -1);
        return 0;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_FAST, 22);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to fastbackward failed,(%d)\n", result);
        return 0;
    }
    printf("[APP ] ====== Quit FB ======\n\n");
    return 1;
#else
	int32_t 		result;
    PVR_TrackInfoT	track_info;
    uint8_t			track   = 0xFF;
    uint8_t			mode_index;
    uint8_t			type_index; /*0: MPEG 1: H264*/
    uint16_t 		speed_factor = 2; /* default speed factor is 2*/
    uint8_t			speed_index = 0;  /*calculated by speed_factor */
    
    printf("\n[APP ] ====== Enter FF ======\n");
    if (pbHandle == NULL)
    {
        printf("[APP ] change to fastfoward failed, not started yet(%d)\n", -1);
        return 0;
    }

	result = PVR_GetPlaybackTrack(pbHandle, PVR_DECTYPE_VIDEO, &track);
	if (result != FAPI_OK)
	{
		printf("[APP ] get video track failed,(%d)\n", result);
		return 0;
	}
	else
	{
		printf("[APP ] get video track success,(%d)\n", track);
	}

	result = PVR_GetPlaybackTrackInfo(pbHandle, &track_info);
	if (result != FAPI_OK)
	{
		printf("[APP ] get track info failed,(%d)\n", result);
		return 0;
	}
	else
	{
		printf("[APP ] get track info success\n");
	}

	switch (track_info.trackFormat[track])
	{
		case PVR_TRACK_FORMAT_VIDEO_MPEG1:
		case PVR_TRACK_FORMAT_VIDEO_MPEG2:
			type_index = 0;
			break;

		case PVR_TRACK_FORMAT_VIDEO_H264:
			type_index = 1;
			break;

		default:
			printf("[APP ] invalid video type,(%d)\n",\
												track_info.trackFormat[track]);
			return 0;
	}

	if (argc >= 2)
	{
		speed_factor = (uint16_t)atoi(argv[1]);
		if (speed_factor != 2 && speed_factor != 4 && speed_factor != 8 &&\
			speed_factor != 16 && speed_factor != 32 && speed_factor != 64 &&\
			speed_factor != 128 && speed_factor != 256)
		{
			printf("[APP ] invalid speed factor,(%d)\n", speed_factor);
			return 0;
		}
	}
	printf("[APP ] speed factor[%d] ", speed_factor);
	/* speed factor to speed index */
	while (speed_factor > 2)
	{
		speed_index++;
		speed_factor /= 2;
	}
	
	/* speed factor to mode index */
	mode_index = 16 * type_index  - (speed_index+1) + 8; 
    printf("speed index[%d] mode index[%d]\n", speed_index, mode_index);
    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_FAST, mode_index);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to fastfoward failed,(%d)\n", result);
        return 0;
    }
    printf("[APP ] ====== Quit FF ======\n\n");
    return 1;
#endif
}

static uint32_t 
APP_PvrFastToNormal(int32_t argc, char_t* argv[])
{
    int32_t result;

    printf("\n[APP ] ====== Enter Normal ======\n");
    if (pbHandle == NULL)
    {
        printf("[APP] change to stop failed, not started yet(%d)\n", -1);
        return 0;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_NORMAL, 0);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to normal failed,(%d)\n", result);
        return 0;
    }

    printf("[APP ] ====== Quit Normal ======\n\n");
    return 1;

}

static uint32_t 
APP_PvrPlayStop(int32_t argc, char_t* argv[])
{
    int32_t result;

     VAL_OpenParamsT   vidOpenParams;
    int32_t           err_code;

    FAPEX_TSD_InputParamStrT inputParams;


    printf("\n[APP ] ====== Enter PlayStop ======\n");
    if (pbHandle == NULL)
    {
        printf("[APP] change to stop failed, not started yet(%d)\n", -1);
        return 0;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_STOPPED, 0);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to stop failed,(%d)\n", result);
        return 0;
    }

    result = PVR_ClosePlayback(pbHandle);
    if (result != FAPI_OK)
    {
        printf("[APP ] close play back failed(%d)\n",result);
    }
    else
    {
        pbHandle = NULL;
    }
	
    if (pvrTimeDispThread)
    {
    	FAPEX_SYS_WaitEndThread(pvrTimeDispThread);
    	FAPEX_SYS_DestroyThread(pvrTimeDispThread);
    	pvrTimeDispThread = NULL;
    }
    
    /* VAL has been close in PVR_ClosePlayback */
    vidOpenPtr = NULL;

    ///////////////////////////////////////////////////////////////////////////
    // Restart the Live TV
    ///////////////////////////////////////////////////////////////////////////

    /*prepare, redefine the input*/
    inputParams.useStc           = 1;
    inputParams.tsSelect         = FAPEX_TSD_INPUT_CH_A;
    inputParams.syncByteDistance = 0;
    inputParams.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    inputParams.oflags           = 0;
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)&inputParams );

    if (appServicePtr != NULL)
    {
        switch ( appServicePtr->videoType )
        {
            case VID_MPEG1:
            case VID_MPEG2:
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
                FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] Video = MPEG2 \n");
                break;

            case VID_H264:
                vidOpenParams.decoder.system = FAPEX_VIDEC_CODEC_TYPE_H264;
                FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] Video = H264 \n");
                break;

            default:
                FAPI_SYS_PRINT_MSG("[Video Thread] Unsupported video type.\n");
                APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_READY );
                return -1;
        }
        vidOpenParams.environment =  VAL_ENV_LIVE;
        vidOpenParams.decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
        vidOpenParams.h264AudType = VAL_H264_AUD_TYPE_AUTO;
        vidOpenParams.enableAfd = 1;
        vidOpenPtr = VAL_Open(vidDatPtr, &vidOpenParams, &err_code);

        VAL_Start(vidOpenPtr, appServicePtr->videoPid, VAL_PLAYMODE_NORMAL, VAL_STREAM_SEGMENT_FULL, 0);

        #if 1
        ///////////////////////////////////////
        // Set PCR PID
        ///////////////////////////////////////
        if ( (appServicePtr->pcrPid != appServicePtr->videoPid) &&\
              (appServicePtr->pcrPid != appServicePtr->audioPid[0]))
        {

            printf("[Martin] PCR PID = %d, let's set\n",appServicePtr->pcrPid);
            FAPEX_TSD_PidParamStrT  pidParam;
            pidParam.pid        = (uint32_t)appServicePtr->pcrPid;
            pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
            pidParam.bufferSize = 0;
            result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                                       FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                                       (FAPEXIOCARG3_T)&pidParam );
            printf("\n[%s] set pcr pid return %d\n", __FUNCTION__, result);
        }
        ///////////////////////////////////////
        // Set STC REGULATION
        ///////////////////////////////////////
        (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)vidOpenPtr->videcId,
                            FAPEX_IOCCOM_SYNC_SET_STC_REGULATION,
                            1);
        #endif

        printf("[APP ] APP_PvrPlayStop: AUDIO_Close[%d]\n", AUDIO_Close());
        printf("[APP ] APP_PvrPlayStop: AUDIO_Open[%d]\n", AUDIO_Open());
        printf("[APP ] APP_PvrPlayStop: AUDIO_Start[%d]\n", \
                AUDIO_Start(appServicePtr->audioPid[0], 0, appServicePtr->audioType[0]));
                
    }
    appPlaybackMode = VAL_ENV_LIVE;
    printf("\n[APP ] ====== Quit PlayStop ======\n\n");
    return 1;
}

static uint32_t 
APP_PvrPlaySeek(int32_t argc, char_t* argv[])
{
    int32_t result;
    PVR_PlayModeT playMode;
    uint8_t modeIndex;
    uint32_t value;
    PVR_SeekModeT seekMode;
    PVR_FileInfoT file_info;
    PVR_PlayInfoT play_info;

    printf("\n[APP ] ====== Enter Seek ======\n");
    
    if (argc < 3)
    {
        printf("[APP ] seek failed: invalid seek mode or seek value\n");
        return 0;
    }

    printf("[APP ] command: %s %s %s\n", argv[0], argv[1], argv[2]);
    
    if (pbHandle == NULL)
    {
        printf("[APP ] change to stop failed, not started yet(%d)\n", -1);
        return 0;
    }

    result = PVR_GetPlaybackMode(pbHandle, &playMode, &modeIndex);

    if (result != FAPI_OK)
    {
        printf("[APP ] seek failed: get playback mode failed\n");
        
        return 0;
    }

    if (playMode != PVR_PLAY_MODE_PAUSE && playMode != PVR_PLAY_MODE_STOPPED)
    {
        printf("[APP ] seek failed, seek only valid in pause or stopped mode\n");
        return 0;
    }

    memset(&file_info, 0, sizeof(PVR_FileInfoT));
    memset(&play_info, 0, sizeof(PVR_PlayInfoT));
    PVR_GetPlaybackFileInfo(pbHandle, &file_info);
    PVR_GetPlaybackInfo(pbHandle, &play_info);
    switch (argv[1][0])
    {
        case 'a': /* seek absolutely chunk */
            value = atoi(argv[3]);
            if (value > file_info.size)
            {
                printf("[APP ] seek failed: larger than the size of file\n");
                return 0;
            }
            seekMode = PVR_SEEK_MODE_CHUNKS;
            break;

        case 'f':  /*seek foward*/
            value = atoi(argv[3]);
            if (value + play_info.position > file_info.size)
            {
                printf("[APP ] seek failed: over the bound\n");
                return 0;
            }
            seekMode = PVR_SEEK_MODE_CHUNKS_FORWARD;
            break;

        case 'b': /*seek backward*/
        	value = atoi(argv[3]);
            if (play_info.position < value)
            {
                printf("[APP ] seek failed: over the bound\n");
                return 0;
            }
            seekMode = PVR_SEEK_MODE_CHUNKS_BACKWARD;
            break;

        default:
            printf("[APP ] seek failed: unknown mode\n");
            return 0;
    }

    result = PVR_SeekPlayback(pbHandle, seekMode, value);
    if (result != FAPI_OK)
    {
        printf("[APP ] seek failed: result [%d]\n", result);
        return 0;
    }

    printf("\n[APP ] ====== Quit Seek ======\n\n");
    return 1;
}


static uint32_t 
APP_PvrRecordStop(int32_t argc, char_t* argv[])
{
    int32_t result;
    uint8_t rec_index;

    printf("\n[APP ] ====== Enter RecordStop ======\n");
    
    if (recHandle1 == NULL && recHandle2 == NULL)
    {
        printf("[APP] change to stop failed, not started yet(%d)\n", -1);
        return 0;
    }


	if (recHandle1 != NULL && recHandle2 != NULL)
	{
		if (argc < 2)
		{
			printf("[APP ] 2 recording are running, you must specify one\n");
			return 0;
		}
		else
		{
			rec_index = atoi(argv[1]);
			if (rec_index != 1 && rec_index != 2)
			{
				printf("[APP ] invalid rec index which must be 1 or 2\n");
				return 0;
			}

			if (rec_index == 1)
			{
				recHandle = recHandle1;
			}
			else
			{
				recHandle = recHandle2;
			}
		}
	}
	else if (recHandle1 != NULL)
	{
		recHandle = recHandle1;
		rec_index = 1;
	}
	else if (recHandle2 != NULL)
	{
		recHandle = recHandle2;
		rec_index = 2;
	}
	else
	{
		printf("[APP] change to stop failed, not started yet(%d)\n", -1);
        return 0;
	}

    result = PVRSYS_RecordChangeMode(recHandle, PVR_RECORD_MODE_STOPPED, 0);

    if (result != FAPI_OK)
    {
        printf("[APP ] change to stop failed,(%d)\n", result);
        return 0;
    }

    result = PVR_CloseRecording(recHandle);
    if (result != FAPI_OK)
    {
        printf("[APP ] close record failed(%d)\n",result);
    }
    else
    {
        if (rec_index == 1)
        {
        	recHandle1 = NULL;
	        recHandle = NULL;
        }
        else if (rec_index == 2)
        {
        	recHandle2 = NULL;
        	recHandle = NULL;
        }
    }
    printf("[APP ] ====== Quit RecordStop ======\n\n");
    return 1;
}


static uint32_t 
APP_PvrSetMntPath(int32_t argc, char_t* argv[])
{
    if (argc < 2){
        return 0;
    }

    printf("\n[APP ] ====== Enter PvrSetMntPath ======\n");
    memset(pvr_mnt_path, 0, sizeof(pvr_mnt_path));
    sprintf(pvr_mnt_path, "%s", argv[1]);
    printf("[APP ] set pvr new mnt path[%s]\n", pvr_mnt_path);
    printf("[APP ] ====== Quit PvrSetMntPath ======\n\n");
    return 1;
}

static uint32_t
APP_PvrSlowPlay(int32_t argc, char* argv[])
{
	int32_t		result = FAPI_OK;
	uint8_t 	mode_index = 0;  /*default mode index 0 : FAPI_VIDEC_SLOW_FORWARD_2X_PLAYBACK_MODE*/
	uint8_t 	speed_factor = 2;
	
	printf("\n[APP ] ====== Enter PvrSlow ======\n");
	
	if (pbHandle == NULL)
    {
        printf("[APP ] change to fastfoward failed, not started yet(%d)\n", -1);
        goto err;
    }

    if (argc >= 2)
    {
    	speed_factor = (uint8_t)atoi(argv[1]);
    	if (speed_factor != 2 && speed_factor != 4 && speed_factor != 8)
    	{
    		printf("[APP ] invalid speed factor (%d)\n", speed_factor);
    		goto err;
    	}
    }
	
    switch (speed_factor)
    {
    	case 2:
    		mode_index = 0;
    		break;

    	case 4:
    		mode_index = 1;
    		break;

    	case 8:
    		mode_index = 2;
    		break;

    	default:
    		printf("[APP ] invalid speed factor (%d)\n", speed_factor);
    		goto err;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_SLOW, mode_index);
    if (result != FAPI_OK)
    {
    	printf("[APP ] change to slow mode failed (%d)\n", result);
    }
	printf("[APP ] ====== Quit PvrSlow ======\n\n");
	goto done;
done:
	return 1;

err:
	return 0;
}

static uint32_t
APP_Magic(int32_t argc, char* argv[])
{
	int32_t		result = FAPI_OK;
	uint8_t 	mode_index = 0;  /*default mode index 0 : FAPI_VIDEC_SLOW_FORWARD_2X_PLAYBACK_MODE*/
	uint8_t 	speed_factor = 2;
	
	printf("\n[APP ] ====== Enter PvrSlow ======\n");
	
	if (pbHandle == NULL)
    {
        printf("[APP ] change to fastfoward failed, not started yet(%d)\n", -1);
        goto err;
    }

    if (argc >= 2)
    {
    	speed_factor = (uint8_t)atoi(argv[1]);
    	if (speed_factor != 2 && speed_factor != 4 && speed_factor != 8)
    	{
    		printf("[APP ] invalid speed factor (%d)\n", speed_factor);
    		goto err;
    	}
    }
	
    switch (speed_factor)
    {
    	case 2:
    		mode_index = 0;
    		break;

    	case 4:
    		mode_index = 1;
    		break;

    	case 8:
    		mode_index = 2;
    		break;

    	default:
    		printf("[APP ] invalid speed factor (%d)\n", speed_factor);
    		goto err;
    }

    result = PVRSYS_PlayChangeMode(pbHandle, PVR_PLAY_MODE_SLOW, mode_index);
    if (result != FAPI_OK)
    {
    	printf("[APP ] change to slow mode failed (%d)\n", result);
    }
	printf("[APP ] ====== Quit PvrSlow ======\n\n");
	goto done;
done:
	return 1;

err:
	return 0;
}

/**
*******************************************************************************
**
** \brief Register a new command.
**
** This function registers the given command to the internal command table.
** An already existing command will be unregistered first and replaced by the
** given parameters. The buffers for the brief and the description texts will
** be dynamically allocated to avoid buffer overflows.
**
** \param command     The command (string) to register.
** \param brief       A short (one line) help message.
** \param description The full command description, maybe NULL.
** \param function    The function to be called when the user enters the
**                    "command" (with optional parameters), this functions
**                    must accept the argc, argv argument pair for additional
**                    optional parameters [strings].
**
*******************************************************************************
*/
int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler,
                                         command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}

#if ENABLE_GUI_WINDOW == 1
/*
*****************************************************************************
**
** \brief check whether Video Frame is available
**
** This function controls the VIDEO module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appVideoFrameAvailable(void* optArg)
{
    FAPI_SYS_PRINT_DEBUG(0, "\n%lld, appVideoFrameAvailable()\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    GUI_SetPlaybackStart();
}

/*
*****************************************************************************
**
** \brief check whether Video parameter is changed
**
** This function controls the VIDEO module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appVideoParamChanged(FAPEX_VIDEC_DecFrameInfoStrT *pInfo, void* optArg)
{
    GUI_SetVideoFrameParameter(pInfo);
}
#endif /* ENABLE_GUI_WINDOW */

/*
*****************************************************************************
**
** \brief Video thread
**
** This function controls the VIDEO module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appVideoThreadFunction(void* optArg)
{
    VAL_OpenParamsT   vidOpenParams;
    VAL_DecoderTypeT  decoder;
    VAL_StatusT       vidStatus;
    uint32_t          video_pcr_flag;
    uint8_t           freeze_flag;
    int32_t           result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appVideoThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_CREATED );

    decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;

    while(appStandbyMode >= 0) {
        APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_OPENED );

        while(1) {
            APP_ThreadStatus_Lock( &appVideoThreadStatus );
            if( (appServicePtr == NULL) || (appServicePtr->videoPid == 0)
             || (appVideoThreadStatus.value >= APP_THREAD_STATUS_READY) ) {

                appVideoThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );

                // wait start trigger
                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

                APP_ThreadStatus_Lock( &appVideoThreadStatus );
                appVideoThreadStatus.value &= ~APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );

                FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] Wake UP \n");

                if(appServicePtr == NULL) {
                    break;
                }
                APP_ThreadStatus_Lock( &appVideoThreadStatus );
                if(appVideoThreadStatus.value == APP_THREAD_STATUS_STARTED) {
                    APP_ThreadStatus_Unlock( &appVideoThreadStatus );

                    freeze_flag = (appServicePtr->videoPid != 0) ? 0 : 1;
					/*Martin -- 20100728 -- force freeze */
                    //freeze_flag = 1;
                    result = VAL_Stop(vidOpenPtr, freeze_flag);
                    if( result < 0 ) {
                        FAPI_SYS_PRINT_MSG("[Video Thread] Stop Video failed. %d\n", result );
                    }
                    else
                    {
                        FAPI_SYS_PRINT_MSG("[Video Thread] Stop Video success.\n");
                    }

                    (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)vidOpenPtr->videcId,
                                    FAPEX_IOCCOM_SYNC_SET_STC_REGULATION,
                                    1);
                }
                else {
                    APP_ThreadStatus_Unlock( &appVideoThreadStatus );
                }
                if(appServicePtr->videoPid == 0) {
                    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_OPENED );
                    continue;
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appVideoThreadStatus );
            }

            if(appServicePtr->pcrPid == appServicePtr->videoPid) {
                video_pcr_flag = 1;
            }
            else {
                video_pcr_flag = 0;
            }
            
            FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] videoPid = [%u], audioPid=[%u], PCR = [%u].\n",
                                    appServicePtr->videoPid, appServicePtr->audioPid[0],appServicePtr->pcrPid);

            /* Start Video Decoding  */

            switch ( appServicePtr->videoType )
            {
                case VID_MPEG1:
                case VID_MPEG2:
                    decoder.system = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
                    FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] Video = MPEG2 \n");
                    break;

                case VID_H264:
                    decoder.system = FAPEX_VIDEC_CODEC_TYPE_H264;
                    FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] Video = H264 \n");
                    break;

                default:
                    FAPI_SYS_PRINT_MSG("[Video Thread] Unsupported video type.\n");
                    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_READY );
                    continue;
            }

            if(vidOpenPtr != NULL) {
                //if(vidOpenParams.decoder.system != decoder.system) {
                if (1){
                    result = VAL_Close(vidOpenPtr);
                    if( result < 0 ) {
                        FAPI_SYS_PRINT_MSG("[Video Thread] Close Video failed. %d\n", result );
                        return;
                    }
                    else
                    {
                        FAPI_SYS_PRINT_MSG("[Video Thread] Close Video success\n");
                    }
                    vidOpenPtr = NULL;
                }
            }
            if(vidOpenPtr == NULL) {
                vidOpenParams.environment = appPlaybackMode;
                vidOpenParams.decoder     = decoder;
                vidOpenParams.enableAfd   = 1;

                vidOpenPtr = VAL_Open(vidDatPtr, &vidOpenParams, &result);

                if ( vidOpenPtr == NULL )
                {
                    FAPI_SYS_PRINT_MSG("[Video Thread] VAL_Open failed: %i\n", result);
                    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_READY );
#if ENABLE_GUI_WINDOW == 1
                    GUI_SetWarning("Video Decoder Open failed");
#endif
                    continue;
                }
                FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] VAL_Open success \n");

#if ENABLE_GUI_WINDOW == 1
                result = VAL_SetFrameAvailableCb(vidOpenPtr,
                                                 appVideoFrameAvailable, NULL);
                if ( result != FAPI_OK )
                {
                    FAPI_SYS_PRINT_MSG("[Video Thread] VAL_SetFrameAvailableCb failed: %i\n", result);
                }
                result = VAL_SetParamChangeCb(vidOpenPtr,
                                              appVideoParamChanged, NULL);
                if ( result != FAPI_OK )
                {
                    FAPI_SYS_PRINT_MSG("[Video Thread] VAL_SetParamChangeCb failed: %i\n", result);
                }
#endif
             }

             {
                result = VAL_Start(vidOpenPtr, appServicePtr->videoPid,
                                   VAL_PLAYMODE_NORMAL, VAL_STREAM_SEGMENT_FULL, video_pcr_flag);
                if ( result != FAPI_OK )
                {
                    FAPI_SYS_PRINT_MSG("[Video Thread] VAL_Start failed: %i\n", result);
                    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_READY );
#if ENABLE_GUI_WINDOW == 1
                    GUI_SetWarning("Video Decoder Start failed");
#endif
                    continue;
                }
				if (vidOpenParams.environment == VAL_ENV_LIVE)
				{
	                #if 1
	                ///////////////////////////////////////
	                // Set PCR PID
	                ///////////////////////////////////////
	                if ( (appServicePtr->pcrPid != appServicePtr->videoPid) &&\
	                      (appServicePtr->pcrPid != appServicePtr->audioPid[0]))
	                {
                        FAPEX_TSD_PidParamStrT  pidParam;
                        pidParam.pid        = (uint32_t)appServicePtr->pcrPid;
                        pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
                        pidParam.bufferSize = 0;
                        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                                                   FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                                                   (FAPEXIOCARG3_T)&pidParam );

	                }
	                ///////////////////////////////////////
	                // Set STC REGULATION
	                ///////////////////////////////////////
	                (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)vidOpenPtr->videcId,
	                                    FAPEX_IOCCOM_SYNC_SET_STC_REGULATION,
	                                    1);
	                #endif
                }
                FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] VAL_Start success \n");

                if ( (VAL_GetStatus(vidOpenPtr, &vidStatus) == FAPI_OK)
                  && (vidStatus.decoderStatus == VAL_STATUS_RUNNING) )
                {
                    FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] VAL_Task success \n");
                }
#if ENABLE_GUI_WINDOW == 1
                GUI_SetVidecId(VAL_GetIdVidec(vidOpenPtr));
#endif
            }
            APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_STARTED );
        }
        APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_OPENED );

        if(vidOpenPtr != NULL) {
            result = VAL_Close(vidOpenPtr);
            if( result < 0 ) {
                FAPI_SYS_PRINT_MSG("[Video Thread] Close Video failed. %d\n", result );
                break;
            }
            FAPI_SYS_PRINT_DEBUG(3, "[Video Thread] VAL_Close success \n");
            vidOpenPtr = NULL;
        }

        if( appStandbyMode < 0 ) {
            break;
        }
        APP_ThreadStatus_SetValue( &appVideoThreadStatus,
                                  (APP_THREAD_STATUS_CREATED | APP_THREAD_STATUS_SUSPENDED) );

        // wait start trigger
        FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

        APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_CREATED );
    }
    APP_ThreadStatus_SetValue( &appVideoThreadStatus, APP_THREAD_STATUS_DISABLE );

    FAPI_SYS_PRINT_DEBUG(1, "appVideoThreadFunction() end\n");
}

/*
*****************************************************************************
**
** \brief Audio thread
**
** This function controls the AUDIO module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appAudioThreadFunction(void* optArg)
{
    uint32_t  audio_pcr_flag;
    int32_t   result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appAudioThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_CREATED );

    while(appStandbyMode >= 0) {
        // Open AUDIO
        result = AUDIO_Open();
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Audio Thread] Open Audio failed. %d\n", result );
#if ENABLE_GUI_WINDOW == 1
            GUI_SetWarning("Audio Decoder Open failed");
#endif
            break;
        }
        APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_OPENED );

        while(1) {
            APP_ThreadStatus_Lock( &appAudioThreadStatus );
            if( (appServicePtr == NULL) || (appServicePtr->audioPid[0] == 0)
             || (appAudioThreadStatus.value >= APP_THREAD_STATUS_READY) ) {

                appAudioThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );

                // wait start trigger
                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

                APP_ThreadStatus_Lock( &appAudioThreadStatus );
                appAudioThreadStatus.value &= ~APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );

                FAPI_SYS_PRINT_DEBUG(3, "[Audio Thread] Wake UP \n");

                if(appServicePtr == NULL) {
                    break;
                }
                APP_ThreadStatus_Lock( &appAudioThreadStatus );
                if(appAudioThreadStatus.value == APP_THREAD_STATUS_STARTED) {
                    APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                    result = AUDIO_Stop();
                    if( result < 0 ) {
                        FAPI_SYS_PRINT_MSG("[Audio Thread] Stop Audio failed. %d\n", result );
                    }
                }
                else {
                    APP_ThreadStatus_Unlock( &appAudioThreadStatus );
                }
                if(appServicePtr->audioPid[0] == 0) {
                    APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_OPENED );
                    continue;
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appAudioThreadStatus );
            }

            if(appServicePtr->pcrPid == appServicePtr->audioPid[0]) {
                audio_pcr_flag = 1;
            }
            else {
                audio_pcr_flag = 0;
            }
            result = AUDIO_Start( appServicePtr->audioPid[0],
                                  audio_pcr_flag,
                                  appServicePtr->audioType[0]);
            if( result == 0 ) {
                APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_STARTED );

#if ENABLE_GUI_WINDOW == 1
                if(appServicePtr->videoPid == 0) {
                    GUI_SetPlaybackStart();
                }
                GUI_SetAuoutId(AUDIO_GetIdAuout());
#endif
            }
            else {
                FAPI_SYS_PRINT_MSG("[Audio Thread] Starting Audio PID %d failed. %d\n",
                                   appServicePtr->audioPid[0], result );
                APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_READY );
#if ENABLE_GUI_WINDOW == 1
                GUI_SetWarning("Audio Decoder Start failed");
#endif
            }
        }
        APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_OPENED );

        // Close AUDIO
        result = AUDIO_Close();
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Audio Thread] Close Audio failed. %d\n", result );
            break;
        }

        if( appStandbyMode < 0 ) {
            break;
        }
        APP_ThreadStatus_SetValue( &appAudioThreadStatus,
                                  (APP_THREAD_STATUS_CREATED | APP_THREAD_STATUS_SUSPENDED) );

        // wait start trigger
        FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

        APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_CREATED );
    }
    APP_ThreadStatus_SetValue( &appAudioThreadStatus, APP_THREAD_STATUS_DISABLE );

    FAPI_SYS_PRINT_DEBUG(1, "appAudioThreadFunction() end\n");
}

/*
*****************************************************************************
**
** \brief TTX thread
**
** This function controls the TTX module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appTtxThreadFunction(void* optArg)
{
    int32_t   result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appTtxThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_CREATED );

    while(appStandbyMode >= 0) {
        // Open TTX
        result = TTXVBI_Open();
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Ttx Thread] Open Teletext failed. %d\n", result );
            break;
        }
        APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_OPENED );

        while(1) {
            APP_ThreadStatus_Lock( &appTtxThreadStatus );
            if( (appServicePtr == NULL) || (appServicePtr->teletextPid == 0)
             || (appTtxThreadStatus.value >= APP_THREAD_STATUS_READY) ) {

                appTtxThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );

                // wait start trigger
                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

                APP_ThreadStatus_Lock( &appTtxThreadStatus );
                appTtxThreadStatus.value &= ~APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );

                FAPI_SYS_PRINT_DEBUG(3, "[Ttx Thread] Wake UP \n");

                if(appServicePtr == NULL) {
                    break;
                }
                APP_ThreadStatus_Lock( &appTtxThreadStatus );
                if(appTtxThreadStatus.value == APP_THREAD_STATUS_STARTED) {
                    APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                    result = TTXVBI_Stop();
                    if( result < 0 ) {
                        FAPI_SYS_PRINT_MSG("[Ttx Thread] Stop Teletext failed. %d\n", result );
                    }
                }
                else {
                    APP_ThreadStatus_Unlock( &appTtxThreadStatus );
                }

                if(appServicePtr->teletextPid == 0) {
                    APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_OPENED );
                    continue;
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appTtxThreadStatus );
            }

            result = TTXVBI_Start( appServicePtr->teletextPid );
            if( result == 0 ) {
                APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_STARTED );
            }
            else {
                FAPI_SYS_PRINT_MSG("[Ttx Thread] Starting Teletext PID %d failed. %d\n",
                                   appServicePtr->teletextPid, result );
                APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_READY );
            }
        }
        APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_OPENED );

        // Close TTX
        result = TTXVBI_Close();
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Ttx Thread] Close Teletext failed. %d\n", result );
            break;
        }

        if( appStandbyMode < 0 ) {
            break;
        }
        APP_ThreadStatus_SetValue( &appTtxThreadStatus,
                                  (APP_THREAD_STATUS_CREATED | APP_THREAD_STATUS_SUSPENDED) );

        // wait start trigger
        FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

        APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_CREATED );
    }
    APP_ThreadStatus_SetValue( &appTtxThreadStatus, APP_THREAD_STATUS_DISABLE );

    FAPI_SYS_PRINT_DEBUG(1, "appTtxThreadFunction() end\n");
}

#if ENABLE_GUI_WINDOW == 0
/*
*****************************************************************************
**
** \brief OSD thread
**
** This function controls the OSD module.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appOsdThreadFunction(void* optArg)
{
    int32_t   result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appOsdThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_CREATED );

    while(appStandbyMode >= 0) {
        if(appStandbyMode > 0) {
            FAPEX_VIOUT_FadingParamStrT  param;

            param.mode     = FAPEX_VIOUT_VIDEO_FADE_IN;
            param.stepSize = 10;

            result = FAPEX_VIOUT_Ioctl( VAL_GetIdViscaleHd(vidDatPtr),
                                        FAPEX_IOCCOM_VIOUT_SET_FADING,
                                        (FAPEXIOCARG3_T)&param );
            if( result < 0 ) {
                return;
            }
        }
        // Open OSD
        result = OSD_Open(appHdSystemType);
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Osd Thread] Open OSD failed. %d\n", result );
            break;
        }
        APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_OPENED );

        while(1) {
            APP_ThreadStatus_Lock( &appOsdThreadStatus );
            if( (appServicePtr == NULL) || (appServicePtr->name[0] == '\0')
             || (strncmp((const char*)appServicePtr->name, "SVC ID:", 7) == 0)
             || (appOsdThreadStatus.value >= APP_THREAD_STATUS_READY) ) {

                appOsdThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );

                // wait start trigger
                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

                APP_ThreadStatus_Lock( &appOsdThreadStatus );
                appOsdThreadStatus.value &= ~APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );

                FAPI_SYS_PRINT_DEBUG(3, "[Osd Thread] Wake UP \n");

                if(appServicePtr == NULL) {
                    break;
                }

                APP_ThreadStatus_Lock( &appOsdThreadStatus );
                if(appOsdThreadStatus.value == APP_THREAD_STATUS_STARTED) {
                    APP_ThreadStatus_Unlock( &appOsdThreadStatus );

                    result = OSD_Stop();
                    if( result < 0 ) {
                        FAPI_SYS_PRINT_MSG("[Osd Thread] Stop OSD failed. %d\n", result );
                    }
                }
                else {
                    APP_ThreadStatus_Unlock( &appOsdThreadStatus );
                }

                if(appServicePtr->name == NULL) {
                    APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_OPENED );
                    continue;
                }
            }
            else {
                APP_ThreadStatus_Unlock( &appOsdThreadStatus );
            }

            /* show service name on OSD */
#if ENABLE_GUI_WINDOW == 0
            result = OSD_ShowProgram( appServicePtr->name );
            if( result == 0 ) {
                APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_STARTED );
            }
            else {
                FAPI_SYS_PRINT_MSG("[Osd Thread] OSD_ShowProgram failed. %d\n", result );
                APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_READY );
            }
#else
            GUI_SetServiceInfo(appServicePtr);
#endif
        }
        APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_OPENED );

        result = OSD_Close();
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG("[Osd Thread] Close OSD failed. %d\n", result );
            break;
        }

        if( appStandbyMode < 0 ) {
            break;
        }
        APP_ThreadStatus_SetValue( &appOsdThreadStatus,
                                  (APP_THREAD_STATUS_CREATED | APP_THREAD_STATUS_SUSPENDED) );

        // wait start trigger
        FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

        APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_CREATED );
    }
    APP_ThreadStatus_SetValue( &appOsdThreadStatus, APP_THREAD_STATUS_DISABLE );

    FAPI_SYS_PRINT_DEBUG(1, "appOsdThreadFunction() end\n");
}
#endif

/*
*****************************************************************************
**
** \brief Scan thread
**
** This function controls the TSD module for scanning the stream.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appScanThreadFunction(void* optArg)
{
    SERVICELIST_ServiceInfo_T*  service_ptr;
    int32_t  show_osd_flag = 0;
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appScanThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    /* reset Service Parameters */
    appServiceCount = 0;
    service_ptr     = NULL;

    while(1) {

        result = SCAN_Task();

        if(result == -1) {
            /* scan complete */

            /* reset Service Parameters */
            appServiceCount = 0;

            service_ptr = appGetNextService();
            if( service_ptr ) {
                /* show service name on OSD */
                appServicePtr = service_ptr;
                if(show_osd_flag == 0) {
#if ENABLE_GUI_WINDOW == 0
                    FAPEX_SYS_WakeupThread( appOsdThreadHandle );
#else
                    GUI_SetServiceInfo(appServicePtr);
#endif
                }
            }
            /* show service list on terminal */
            (void)APP_ServiceList( 1, NULL );

            FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appScanThreadFunction() sleep\n",
                                    FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

            // wait next scan trigger
            FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

            /* reset parameters */
            show_osd_flag = 0;
            service_ptr   = NULL;
        }
        else if(result == 2) {
            /* start first service */
            service_ptr = appGetNextService();
            if ( service_ptr ) {
                appChangeService( service_ptr, 0 );
            }
        }

        /* check service name */
        if((service_ptr != NULL) && (show_osd_flag == 0)) {
            if( (service_ptr->name[0] != '\0')
             && (strncmp((const char*)service_ptr->name, "SVC ID:", 7) != 0) ) {
                /* show service name on OSD */
#if ENABLE_GUI_WINDOW == 0
                FAPEX_SYS_WakeupThread( appOsdThreadHandle );
#else
                GUI_SetServiceInfo(service_ptr);
#endif
                show_osd_flag = 1;
            }
        }
    }
}

/*
*****************************************************************************
**
** \brief IR thread
**
** This function monitors the IR event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appIrThreadFunction(void* optArg)
{
    FAPEX_IR_KeyParamStrT       keyParams;
    SERVICELIST_ServiceInfo_T*  service_ptr;
    int32_t  result;
#if 0 /* now not supported */
    int32_t  argc = 2;
    char_t*  argv[3];

    argv[0] = "";
#endif

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appIrThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    while(1) {
        result = FAPEX_IR_Ioctl( fd_IR0,
                                 FAPEX_IOCCOM_IR_GET_KEY,
                                 (FAPEXIOCARG3_T)&keyParams );
        if( result != 0 ) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] Get IR Key. %d\n", result);
        }
        switch( keyParams.rcmCode ) {
            case FAPEX_IR_KEY_MUTE:
                if ( !keyParams.repeatFlag ) AUDIO_Mute();
                break;

            case FAPEX_IR_KEY_VOL_UP:
                AUDIO_Volume(2);
                break;

            case FAPEX_IR_KEY_VOL_DOWN:
                AUDIO_Volume(-2);
                break;

            case FAPEX_IR_KEY_CHANNEL_UP:
                if(appStandbyMode) {
                    appStandby(1);
                }
                else {
                    service_ptr = appGetNextService();
                    if ( service_ptr ) {
                        appChangeService( service_ptr, 1 );
                    }
                }
                break;

            case FAPEX_IR_KEY_CHANNEL_DOWN:
                if(appStandbyMode) {
                    appStandby(1);
                }
                else {
                    service_ptr = appGetPrevService();
                    if ( service_ptr ) {
                        appChangeService( service_ptr, 1 );
                    }
                }
                break;

            case FAPEX_IR_KEY_STANDBY:
                appStandby(1);
                break;

#if ENABLE_GUI_WINDOW == 0
            case FAPEX_IR_KEY_PLAY:
                APP_KeyPlay();
                break;

            case FAPEX_IR_KEY_STOP:
                APP_KeyStop( 0, NULL );
                break;

            case FAPEX_IR_KEY_PAUSE:
                APP_KeyPause( 0, NULL );
                break;
#else
            case FAPEX_IR_KEY_PLAY:
            case FAPEX_IR_KEY_STOP:
            case FAPEX_IR_KEY_PAUSE:    // now not supported
            case FAPEX_IR_KEY_RIGHT:
            case FAPEX_IR_KEY_LEFT:
            case FAPEX_IR_KEY_UP:
            case FAPEX_IR_KEY_DOWN:
            case FAPEX_IR_KEY_OK:
            case FAPEX_IR_KEY_EXIT:
            case FAPEX_IR_KEY_INFO:
            case FAPEX_IR_KEY_PIPSWAP:
                GUI_RemoteControl(keyParams.rcmCode);
                break;
#endif

#if 0 /* now not supported */
            case FAPEX_IR_KEY_1:
                argv[1] = "0";
                SERVICELIST_ClearList();
                (void)APP_Tune(argc, argv);
                trpListIndex = 0;
                appCurrentServiceIndex = -1;
                tsScan = 1;
                SCAN_Start(trpListIndex);
                break;

            case FAPEX_IR_KEY_2:
                argv[1] = "0";
                argv[2] = "0";
                FAPI_SYS_PRINT_MSG("Toneburst OFF, DiseqC OFF\n");
                (void)APP_ConfigSat( 3, argv );
                break;

            case FAPEX_IR_KEY_3:
                argv[1] = "0";
                argv[2] = "1";
                FAPI_SYS_PRINT_MSG("Toneburst OFF, DiseqC ON\n");
                (void)APP_ConfigSat( 3, argv );
                break;
#endif

#if ENABLE_GUI_WINDOW == 1
            case FAPEX_IR_KEY_PIPSELECT:
                {
                    int32_t  temp, order[4];

                    result = FAPEX_VIOUT_Ioctl(vidDatPtr->vioutId,
                                               FAPEX_IOCCOM_VIOUT_GET_PLANEORDER,
                                               (FAPEXIOCARG3_T)order );
                    if(result < 0) {
                        FAPI_SYS_PRINT_MSG("ERROR:[main.c] get plane order. %d\n", result );
                    }
                    /* video <-> osd1 */
                    temp     = order[0];
                    order[0] = order[1];
                    order[1] = temp;

                    result = FAPEX_VIOUT_Ioctl(vidDatPtr->vioutId,
                                               FAPEX_IOCCOM_VIOUT_SET_PLANEORDER,
                                               (FAPEXIOCARG3_T)order );
                    if(result < 0) {
                        FAPI_SYS_PRINT_MSG("ERROR:[main.c] set plane order. %d\n", result );
                    }
                }
                break;

            case FAPEX_IR_KEY_F1:
            case FAPEX_IR_KEY_F2:
                {
                    if(appOsd1_fd < 0) {
                        FAPEX_VIOUT_OpenSubParamStrT osd_open_params;

                        osd_open_params.sub_id     = FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 3;
                        osd_open_params.scalerType = FAPEX_VIOUT_SCALER_OSD1;

                        result = FAPEX_VIOUT_Ioctl(vidDatPtr->vioutId,
                                                   FAPEX_IOCCOM_VIOUT_OPEN_SUBID,
                                                   (FAPEXIOCARG3_T)&osd_open_params );
                        if(result == 0) {
                            appOsd1_fd = osd_open_params.sub_id;
                        }
                    }
                    if(appOsd1_fd >= 0) {
                        FAPEX_VIOUT_AlphaParamStrT  alpha_params;
                        int32_t  value;

                        alpha_params.mode = FAPEX_VIOUT_PARAMETER_GET;

                        result = FAPEX_VIOUT_Ioctl(appOsd1_fd,
                                                   FAPEX_IOCCOM_VIOUT_SET_ALPHA,
                                                   (FAPEXIOCARG3_T)&alpha_params );

                        if(result == 0) {
                            if(keyParams.rcmCode == FAPEX_IR_KEY_F1) {
                                value = alpha_params.alpha + 16;
                                if(value > 255) {
                                    value = 255;
                                }
                            }
                            else {
                                value = alpha_params.alpha - 16;
                                if(value < 0) {
                                    value = 0;
                                }
                            }
                            if(value != alpha_params.alpha) {
                                alpha_params.mode  = FAPEX_VIOUT_PARAMETER_SET;
                                alpha_params.alpha = value;

                                result = FAPEX_VIOUT_Ioctl(appOsd1_fd,
                                                           FAPEX_IOCCOM_VIOUT_SET_ALPHA,
                                                           (FAPEXIOCARG3_T)&alpha_params );
                            }
                        }
                    }
                    if(result < 0) {
                        FAPI_SYS_PRINT_MSG("ERROR:[main.c] get/set aplha. %d\n", result );
                    }
                }
                break;
#endif
            default:
                FAPI_SYS_PRINT_MSG( "Received key: 0x%x\n", keyParams.rcmCode );
        }
    }
}

/*
*****************************************************************************
**
** \brief FPC thread
**
** This function monitors the FPC's event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appFpcThreadFunction(void* optArg)
{
    SERVICELIST_ServiceInfo_T  *service_ptr;
    uint32_t buttonFlag;
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appFpcThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    while(1) {
        result = FAPEX_FPC_Ioctl( fd_FPC0,
                                  FAPEX_IOCCOM_FPC_GET_BUTTON_NOTIFY,
                                  (FAPEXIOCARG3_T)&buttonFlag );
        if( result != 0 ) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] error FPC get notify (code: %d).\n", result);
            break;
        }
        service_ptr = NULL;
        if( buttonFlag == FAPEX_FPC_EVENT_BUTTON3 ) {       // Up button Only
            service_ptr = appGetNextService();
        }
        else if( buttonFlag == FAPEX_FPC_EVENT_BUTTON8 ) {  // Down button Only
            service_ptr = appGetPrevService();
        }

        if( service_ptr ) {
            appChangeService( service_ptr, 1 );
        }
        FAPI_SYS_PRINT_MSG( "Received button: 0x%x\n", buttonFlag );
    }
}

/*
*****************************************************************************
**
** \brief HDMI thread
**
** This function monitors the HDMI-CEC's event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appHdmiThreadFunction(void* optArg)
{
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appHdmiThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    while(1) {
        result = HDMI_Task();
        if(result < 0) {
            FAPI_SYS_SLEEP(1000);
        }
        else {
            if(result == 1) {
                /* detect the HPD interrupt */
                appHdmiAutoSync();
            }
            FAPI_SYS_SLEEP(100);
        }
    }
}

#if ENABLE_GUI_WINDOW == 1
/*
*****************************************************************************
**
** \brief GUI thread
**
** This function monitors the GUI's event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appGuiThreadFunction(void* optArg)
{
    GUI_CallbackFuncs_T  control;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appGuiThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    control.playbackStart       = APP_PlaybackStart;
    control.playbackStop        = APP_StopCommand;
    control.getFileReadPosition = TSD_GetFileReadPosition;
    control.getTimeStamp        = TSD_GetTimeStamp;

    GUI_Open(&control, "/mnt/usb");
}
#endif

/*
*****************************************************************************
** \brief USB enable function
*****************************************************************************
*/

uint32_t usb_enable(int32_t argc, char **argv)
{
    int32_t                 GPIO_ID;

    FAPEX_GPIO_OpenParamStrT   paramsGPIO;
    paramsGPIO.pin = 63;

    paramsGPIO.function = FAPEX_GPIO_OUT_1;//0;



    GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);

    usleep(50000);

    FAPEX_GPIO_Close(GPIO_ID);

    paramsGPIO.function = FAPEX_GPIO_OUT_0;//0;

    GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);

    FAPEX_GPIO_Close(GPIO_ID);


    /* module install */
    system("insmod /home/usb_mod/usbcore.ko");
    system("insmod /home/usb_mod/nop-usb-xceiv.ko");
    system("insmod /home/usb_mod/musb_hdrc.ko");
    system("insmod /home/usb_mod/usb-storage.ko");
    system("insmod /home/usb_mod/usbmon.ko");
    printf("mount done\n");
    
    return 1;
}

/*
********************************************************************************
** The application's initialisation thread.
**
** This function is required to initialise the entire system.
** Within a non-RTOS environment is is called a a generic function,
** within an RTOS environment it is called within thread context.
********************************************************************************
*/
static void appInitialize( void )
{
    FAPEX_IR_OpenParamStrT irOpenParams;
	VAL_InitParamsT        initParams;
    int32_t  result;
#ifdef _LINUX_FAPI
    int32_t  index;
#endif
    // tuner lock 
   // (void)APP_Tune(NULL,NULL);

    // Open TSD
    TSD_Open();

    // Init Service list
    SERVICELIST_Init();

    #if 1
    if(appTsInputParams.tsSelect != FAPEX_TSD_INPUT_CH_A) {
        result = TSD_Start(&appTsInputParams, appTsInputFileName, appTsInputStartOffset);
        if( result < 0 ) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] TSD start \"%d\" failed\n", result );
        }
    }
    #endif

    SCAN_Start(trpListIndex);

    // Open SCAN thread
    #if 1
    appScanThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_SCAN_THREAD_STACKSIZE,
            APP_SCAN_THREAD_PRIORITY,
            appScanThreadFunction,
            0, 0, APP_SCAN_THREAD_NAME );
    if( appScanThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_SCAN_THREAD_NAME );
        exit(1);
    }
    #endif

    // Init VIDEO
    memset (&initParams, 0, sizeof(initParams));
    initParams.threadPriority = FAPEX_SYS_PRIORITY_MEDIUM;
    result = VAL_Init(&initParams);
    if( result < 0 ) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open Video failed. %d\n", result );
        exit(1);
    }
    appVideoInit();

    // Open VIDEO thread
    #if 1
    APP_ThreadStatus_Init( &appVideoThreadStatus );
    appVideoThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_VIDEO_THREAD_STACKSIZE,
            APP_VIDEO_THREAD_PRIORITY,
            appVideoThreadFunction,
            0, 0, APP_VIDEO_THREAD_NAME );
    if( appVideoThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appVideoThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_VIDEO_THREAD_NAME );
        exit(1);
    }
    #endif
    // Open AUDIO thread
    #if 1
    APP_ThreadStatus_Init( &appAudioThreadStatus );
    appAudioThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_AUDIO_THREAD_STACKSIZE,
            APP_AUDIO_THREAD_PRIORITY,
            appAudioThreadFunction,
            0, 0, APP_AUDIO_THREAD_NAME );
    if( appAudioThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appAudioThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_AUDIO_THREAD_NAME );
        exit(1);
    }
    #endif

    // Open TTX thread
    #if 1
    APP_ThreadStatus_Init( &appTtxThreadStatus );
    appTtxThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_TTX_THREAD_STACKSIZE,
            APP_TTX_THREAD_PRIORITY,
            appTtxThreadFunction,
            0, 0, APP_TTX_THREAD_NAME );
    if( appTtxThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appTtxThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_TTX_THREAD_NAME );
        exit(1);
    }
    #endif

#if ENABLE_GUI_WINDOW == 0
    // Open OSD thread
    APP_ThreadStatus_Init( &appOsdThreadStatus );
    appOsdThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_OSD_THREAD_STACKSIZE,
            APP_OSD_THREAD_PRIORITY,
            appOsdThreadFunction,
            0, 0, APP_OSD_THREAD_NAME );
    if( appOsdThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appOsdThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_OSD_THREAD_NAME );
        exit(1);
    }
#endif

    // Init PVR thread
    pvr_callback_cmd_sem = FAPEX_SYS_CreateSemaphore(1);
    APP_ThreadStatus_Init(&appPvrThreadStatus);
    appPvrThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_PVR_THREAD_STACKSIZE,
            APP_PVR_THREAD_PRIORITY,
            PVRSYS_ThreadFunction,
            0, 0, APP_PVR_THREAD_NAME );
    if (NULL == appPvrThreadHandle)
    {
        APP_ThreadStatus_Exit(&appPvrThreadStatus);
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_PVR_THREAD_NAME );
        exit(1);
    }

	#if 1
    // Open IR
    irOpenParams.mode = APP_REMOTE_MODE;
    fd_IR0 = FAPEX_IR_Open( DRV_ID_IR0,
                            O_RDWR,
                            (FAPEXOPENARG3_T)&irOpenParams );
    if(fd_IR0 < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open IR driver.\n");
    }
    else {
#if APP_REMOTE_MODE == FAPEX_IR_MODE_RC5
        FAPEX_IR_MapTableParamStrT  mapping_table;
        FAPEX_IR_KeycodeMapStrT     key_codes[] =
        {
            { 0x0C, FAPEX_IR_KEY_STANDBY },   { 0x57, FAPEX_IR_KEY_OK},
            { 0x50, FAPEX_IR_KEY_UP},         { 0x51, FAPEX_IR_KEY_DOWN},
            { 0x55, FAPEX_IR_KEY_LEFT},       { 0x56, FAPEX_IR_KEY_RIGHT},
            { 0x10, FAPEX_IR_KEY_VOL_UP},     { 0x11, FAPEX_IR_KEY_VOL_DOWN},
            { 0x20, FAPEX_IR_KEY_CHANNEL_UP}, { 0x21, FAPEX_IR_KEY_CHANNEL_DOWN},
            { 0x00, FAPEX_IR_KEY_0 },         { 0x01, FAPEX_IR_KEY_1 },
            { 0x02, FAPEX_IR_KEY_2 },         { 0x03, FAPEX_IR_KEY_3 },
            { 0x04, FAPEX_IR_KEY_4 },         { 0x05, FAPEX_IR_KEY_5 },
            { 0x06, FAPEX_IR_KEY_6 },         { 0x07, FAPEX_IR_KEY_7 },
            { 0x08, FAPEX_IR_KEY_8 },         { 0x09, FAPEX_IR_KEY_9 },
            { 0x0A, FAPEX_IR_KEY_MUTE },      { 0x52, FAPEX_IR_KEY_MENU},
            { 0x0F, FAPEX_IR_KEY_INFO},       { 0x6B, FAPEX_IR_KEY_RED },
            { 0x6C, FAPEX_IR_KEY_GREEN },     { 0x6D, FAPEX_IR_KEY_YELLOW },
            { 0x6E, FAPEX_IR_KEY_BLUE },      { 0x00, FAPEX_IR_KEY_UNDEFINED}
        };
        mapping_table.deviceId      = 0;
        mapping_table.pKeycodeTable = key_codes;
        mapping_table.keycodeNum    = 28;
        (void)FAPEX_IR_Ioctl( fd_IR0,
                              FAPEX_IOCCOM_IR_SET_MAPTABLE,
                              &mapping_table );
#endif
        appIrThreadHandle = FAPEX_SYS_CreateThread(
                NULL,
                APP_IR_THREAD_STACKSIZE,
                APP_IR_THREAD_PRIORITY,
                appIrThreadFunction,
                0, 0, APP_IR_THREAD_NAME );
        if( appIrThreadHandle == NULL )
        {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                APP_IR_THREAD_NAME );
            FAPEX_IR_Close( fd_IR0 );
            exit(1);
        }
    }

    // Open FPC
    fd_FPC0 = FAPEX_FPC_Open( DRV_ID_FPC0,
                              O_RDWR,
                              (FAPEXOPENARG3_T)NULL );
    if(fd_FPC0 < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open FPC driver. %d\n", fd_FPC0);
    }
    else {
        (void)FAPEX_FPC_Ioctl( fd_FPC0,
                               FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE,
                               (FAPEXIOCARG3_T)1 );

        appFpcThreadHandle = FAPEX_SYS_CreateThread(
                NULL,
                APP_FPC_THREAD_STACKSIZE,
                APP_FPC_THREAD_PRIORITY,
                appFpcThreadFunction,
                0, 0, APP_FPC_THREAD_NAME );
        if( appFpcThreadHandle == NULL )
        {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                APP_FPC_THREAD_NAME );
            FAPEX_FPC_Close( fd_FPC0 );
            exit(1);
        }
    }
	#endif
	
    // Open HDMI
    appHdmiEnable = HDMI_Open();

    if(appHdmiEnable == 0) {
        appHdmiThreadHandle = FAPEX_SYS_CreateThread(
                NULL,
                APP_HDMI_THREAD_STACKSIZE,
                APP_HDMI_THREAD_PRIORITY,
                appHdmiThreadFunction,
                0, 0, APP_HDMI_THREAD_NAME );
        if( appHdmiThreadHandle == NULL )
        {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                APP_HDMI_THREAD_NAME );
            exit(1);
        }
    }

    FAPI_SYS_PRINT_MSG( "%s\n", APP_CLEARSCREEN );
    FAPI_SYS_PRINT_MSG( "##%s\n", APP_SEPARATOR );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_DESCRIPTION );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# Target....: %s (%s)\n", APP_TARGET, APP_CPU );
    FAPI_SYS_PRINT_MSG( "# Version...: v%s\n", APP_VERSION );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_COPYRIGHT );
    FAPI_SYS_PRINT_MSG( "# %s\n", APP_COPYRIGHT2 );
    FAPI_SYS_PRINT_MSG( "#\n" );
    FAPI_SYS_PRINT_MSG( "##%s\n", APP_SEPARATOR );
    FAPI_SYS_PRINT_MSG( "\n" );

    FAPI_SYS_PRINT_MSG( "%s creating KBD/RC input handler\n", APP_PREFIX );
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT,
                                                             50,
                                                             NULL );
    if( !appShellInputHandler ) {
        exit(1);
    }
#else
    for( index=0; index < APP_CommandTableSize; index++ )
    {
        commandTable[index].command     = NULL;
        commandTable[index].brief       = NULL;
        commandTable[index].description = NULL;
        commandTable[index].function    = NULL;
    }
    memset( inputCommand, '\0', APP_CommandStringSize );
    for( index=0; index < APP_HistoryTableSize; index++ )
        memset( historyTable[index], '\0', APP_CommandStringSize );
    historyIndex = 0;
    updownIndex  = 0;

    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n" \
        "as optional argument to ?\n"                                         \
        "Example: ?       displays all available commands, one per line\n"    \
        "Example: ? !     displays a more detailed help for the ! command\n"
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n"
    );

    (void)appRegisterCommand(
        "kbd:cursor-up",
        SHELL_CursorUpDownCommand,
        "loops backward through the command history",
        NULL
    );

    (void)appRegisterCommand(
        "kbd:cursor-down",
        SHELL_CursorUpDownCommand,
        "loops foreward through the command history",
        NULL
    );

    (void)appRegisterCommand(
        "kbd:backspace",
        SHELL_CursorUpDownCommand,
        "deletes the character left of cursor",
        NULL
    );
#endif /* !_LINUX_FAPI */

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "start" );
    (void)appRegisterCommand(
        "start",
        APP_StartCommand,
        "Start audio/video/teletext decoder",
        "Usage: start <audioPID> <videoPID> <teletextPID (optional)> or\n" \
        "       start <service list index>\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "stop" );
    (void)appRegisterCommand(
        "stop",
        APP_StopCommand,
        "Stop audio/video decoder",
        "Usage: stop\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "list" );
    (void)appRegisterCommand(
        "list",
        APP_ServiceList,
        "Show service list on terminal",
        "Usage: list\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "up" );
    (void)appRegisterCommand(
        "up",
        APP_ServiceUp,
        "Go to the next service in the actual service list.",
        "Usage: up\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "down" );
    (void)appRegisterCommand(
        "down",
        APP_ServiceDown,
        "Go to the previous service in the actual service list.",
        "Usage: down\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "exit" );
    (void)appRegisterCommand(
        "exit",
        APP_HaltCommand,
        "Exit application.",
        "Usage: exit\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "scan" );
    (void)appRegisterCommand(
        "scan",
        APP_Scan,
        "Scan for services.",
        "Usage: scan\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "standby" );
    (void)appRegisterCommand(
        "standby",
        APP_Standby,
        "Put the system in standby / wake up.",
        "Usage: standby\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "play" );
    (void)appRegisterCommand(
        "play",
        APP_PlaybackStart,
        "Start PVR playback",
        "Usage: play <fileName> <startOffsetPos (optional)>\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "live" );
    (void)appRegisterCommand(
        "live",
        APP_KeyStop,
        "Stop key: Stop PVR playback, Change to Live mode",
        "Usage: live\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "p" );
    (void)appRegisterCommand(
        "p",
        APP_KeyPause,
        "Pause key: Pause/unpause playback",
        "Usage: p\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "dir" );
    (void)appRegisterCommand(
        "dir",
        APP_Dir,
        "list the contents of <pathName>",
        "Usage: dir <pathName>\n"
    );

#ifdef _LINUX_FAPI
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "sys" );
    (void)appRegisterCommand(
        "sys",
        APP_Sys,
        "Execute the shell command",
        "Usage: sys <command>\n"
    );
#endif

#ifdef USE_HDMICEC
    /*******************************************************************************************************
    ** HDMI Test Progam
    *******************************************************************************************************/
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "HDMI_ReadEDID" );
    (void)appRegisterCommand(
        "readedid",
        APP_HDMI_ReadEDID,
        "Read EDID",
        "Usage: readedid\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "HDMI_CheckHPD" );
    (void)appRegisterCommand(
        "checkhpd",
        APP_HDMI_CheckHPD,
        "Check HPD signal",
        "Usage: checkhpd\n"
    );

    /*******************************************************************************************************
    ** HDMI-CEC Test Progam
    ********************************************************************************************************/
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "CEC" );
    (void)appRegisterCommand(
        "cec",
        APP_CEC_MessaageControlInManual,
        "Communicate CEC message for test",
        "Usage: cec [Opcode] [send_flag] [rec_flag] [InitiatorLogicalAddr] [FollowerLogicalAddr]\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Check sending status" );
    (void)appRegisterCommand(
        "cecsendstatus",
        APP_CEC_CheckSendStatus,
        "Check CEC sending status",
        "Usage: cecsendstatus\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Check receiving status" );
    (void)appRegisterCommand(
        "cecrecstatus",
        APP_CEC_CheckReceiveStatus,
        "Check CEC receiving status",
        "Usage: cecrecstatus\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Get message" );
    (void)appRegisterCommand(
        "cecgetmsg",
        APP_CEC_GetMsg,
        "Get CEC message",
        "Usage: cecgetmsg\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Receiving message" );
    (void)appRegisterCommand(
        "cecrecmsg",
        APP_CEC_RecMsg,
        "Receive CEC Message",
        "Usage: cecrecmsg\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Set Logical Address" );
    (void)appRegisterCommand(
        "cecsetlogicaladdr",
        APP_CEC_SetLogicalAddress,
        "Set Logical Address for CEC",
        "Usage: cecsetlogicaladdr [My device logical address]\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Get Logical Address" );
    (void)appRegisterCommand(
        "cecgetlogicaladdr",
        APP_CEC_GetLogicalAddress,
        "Get Logical Address for CEC",
        "Usage: cecgetlogicaladdr\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Allocate Logical Address" );
    (void)appRegisterCommand(
        "allocatelogicaladdr",
        APP_CEC_AllocateLogicalAddress,
        "Allocate Logical Address for CEC",
        "Usage: allocatelogicalAddress\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "DeviceInstrationAndAddressing" );
    (void)appRegisterCommand(
        "devinsaddr",
        APP_CEC_DeviceInstrationAndAddressing,
        "Device Instration And Addressing for CEC",
        "Usage: devinsaddr\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Set the parameter of CEC message" );
    (void)appRegisterCommand(
        "cecsetparam",
        APP_CEC_SetCecMessageParameters,
        "Set the parameter of CEC message",
        "Usage: cecsetparam\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "send <Set OSD String> message" );
    (void)appRegisterCommand(
        "sendosdstring",
        APP_CEC_SendOSDStringMessage,
        "send <Set OSD String> message",
        "Usage: sendosdstring\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "One Touch Play" );
    (void)appRegisterCommand(
        "onetouchplay",
        APP_CEC_OneTouchPlay,
        "One Touch Play",
        "Usage: onetouchplay\n"
    );
#endif /* USE_HDMICEC */


    ///////////////////////////////////////////////////////////////////////////
    //                       PVR Command
    ///////////////////////////////////////////////////////////////////////////
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Playback" );
    (void)appRegisterCommand(
        "pb",
        APP_PvrPlayback,
        "Pvr Playback",
        "Usage: pb fileName\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Record" );
    (void)appRegisterCommand(
        "re",
        APP_PvrRecord,
        "Pvr Record",
        "Usage: re fileName time(sec)\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Pause" );
    (void)appRegisterCommand(
        "pa",
        APP_PvrPause,
        "Pvr Pause",
        "Usage: pa\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Resume" );
    (void)appRegisterCommand(
        "rs",
        APP_PvrResume,
        "Pvr Resume",
        "Usage: rs\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr FastFoward" );
    (void)appRegisterCommand(
        "ff",
        APP_PvrFastFoward,
        "Pvr FastFoward",
        "Usage: ff [speed_factor] speed_factor = 2, 4, 8, 16, 32, 64, 128, 256\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr FastBackward" );
    (void)appRegisterCommand(
        "fb",
        APP_PvrFastBackward,
        "Pvr FastBackward",
        "Usage: fb\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Normal" );
    (void)appRegisterCommand(
        "nm",
        APP_PvrFastToNormal,
        "Pvr Normal",
        "Usage: nm\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr PlayStop" );
    (void)appRegisterCommand(
        "ps",
        APP_PvrPlayStop,
        "Pvr PlayStop",
        "Usage: ps\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr PlaySeek" );
    (void)appRegisterCommand(
        "se",
        APP_PvrPlaySeek,
        "Pvr PlaySeek",
        "Usage: se pos\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr RecordStop" );
    (void)appRegisterCommand(
        "xs",
        APP_PvrRecordStop,
        "Pvr RecordStop: when 2 recording running, specify 1 or 2",
        "Usage: xs [1/2]\n"
    );

    
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr SetMntPath" );
    (void)appRegisterCommand(
        "sm",
        APP_PvrSetMntPath,
        "Pvr SetMntPath",
        "Usage: sm\n"
    );

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr SlowPlay" );
    (void)appRegisterCommand(
        "sp",
        APP_PvrSlowPlay,
        "Pvr SlowPlay",
        "Usage: sp [speed_factor] speed_factor = 2, 4, 8\n"
    );

    
	FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "App Magic key" );
		(void)appRegisterCommand(
			"magic",
			APP_Magic,
			"App Magic key",
			"Usage: magic [n]: n = 0 stop last frame\n"
		);

#if 0
    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr TimeShifting" );
    (void)appRegisterCommand(
        "ts",
        APP_PvrTimeShifting,
        "Pvr TimeShifting",
        "Usage: ts\n"
    );
#endif

    FAPI_SYS_PRINT_MSG( "%s register command \"%s\"\n", APP_PREFIX, "Pvr Enable USB" );
    (void)appRegisterCommand(
        "eu",
        usb_enable,
        "Pvr Enable USB [For SDMC boards]",
        "Usage: eu\n"
    );
    
    if (FAPI_OK != PVRDEC_Init()){
        printf("[ERROR] PVRDEC_Init Failed\n");
        exit(-1);
    }
    if (FAPI_OK != PVR_Init()){
        printf("[ERROR] PVR_Init Failed\n");
        exit(-1);
    }

    PVRSYS_Init();
    
    FAPI_SYS_PRINT_MSG( "%s starting command shell\n", APP_PREFIX );

#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#endif
}

/*
*****************************************************************************
**
** \brief Shell thread
**
** This function monitors the input command.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appShellThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
    appInitialize();

#else
    int32_t   i, c;
    char_t    args_buff[APP_CommandStringSize];

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appShellThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
    appInitialize();

    FAPI_SYS_PRINT_MSG( "%s%s", APP_ColorBgWhite, APP_ColorFgBlack );
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );
    strcpy( args_buff, "? ?" );
    SHELL_DoCommand( args_buff );
    inputCommand[0] = '\0';
    FAPI_SYS_PRINT_MSG( "\n" );

    while(1) {
        i = 0;
        APP_PrintPrompt();

        /* get command string */
        while( !feof(stdin) ) {
            c = getchar();
            args_buff[i] = c;
            if( (c == '\n') || (i >= (APP_CommandStringSize-1)) ) {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) ) {
                i --;
            }
            else {
                i++;
            }
        }

        SHELL_DoCommand( args_buff );

        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */

    FAPEX_SYS_WaitEndThread( appVideoThreadHandle );
    FAPEX_SYS_WaitEndThread( appAudioThreadHandle );
    FAPEX_SYS_WaitEndThread( appTtxThreadHandle );
#if ENABLE_GUI_WINDOW == 0
    FAPEX_SYS_WaitEndThread( appOsdThreadHandle );
#endif
    FAPEX_SYS_WaitEndThread( appScanThreadHandle );
    FAPEX_SYS_WaitEndThread( appIrThreadHandle );
    FAPEX_SYS_WaitEndThread( appFpcThreadHandle );
    FAPEX_SYS_WaitEndThread( appHdmiThreadHandle );

    FAPEX_SYS_WaitEndThread( appPvrThreadHandle );

    appVideoExit();
}

#ifdef USE_WATCHDOG
/*
*****************************************************************************
**
** \brief Watchdog thread
**
** This function will call the watchdog reset to prevent a system restart.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appWatchdogThreadFunction(void* optArg)
{
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appWatchdogThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    for(;;)
    {
        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_CLKPWR_WATCHDOG_RESET,
                                     (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] Watchdog Reset %d\n", result);
            break;
        }

        FAPI_SYS_SLEEP( 1000 );
    }
}
#endif /* USE_WATCHDOG */

/*
*****************************************************************************
**
** \brief Start thread
**
** This function implements the very first thread which is used to create
** all other required threads and test cases.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appStartThreadFunction(void* optArg)
{
    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appStartThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    appShellThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_SHELL_THREAD_STACKSIZE,
            APP_SHELL_THREAD_PRIORITY,
            appShellThreadFunction,
            0, 0, APP_SHELL_THREAD_NAME );
    if( appShellThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                            APP_SHELL_THREAD_NAME );
        exit(1);
    }

    // Open CLKPWR
    fd_CLKPWR = FAPEX_CLKPWR_Open( DRV_ID_CLKPWR,
                                   O_RDWR,
                                   (FAPEXOPENARG3_T)NULL );
    if(fd_CLKPWR < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open CLKPWR driver.\n");
    }
#ifdef USE_WATCHDOG
    else {
        int32_t  result;

        result = FAPEX_CLKPWR_Ioctl( fd_CLKPWR,
                                     FAPEX_IOCCOM_CLKPWR_WATCHDOG_START,
                                     (FAPEXIOCARG3_T)FAPEX_CLKPWR_WATCHDOG_MIN_VALUE );
        if(result < 0) {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] Watchdog Start %d\n", result );
        }
        else {
            appWatchdogThreadHandle = FAPEX_SYS_CreateThread(
                    NULL,
                    APP_WATCHDOG_THREAD_STACKSIZE,
                    APP_WATCHDOG_THREAD_PRIORITY,
                    appWatchdogThreadFunction,
                    0, 0, APP_WATCHDOG_THREAD_NAME );
            if( appWatchdogThreadHandle == NULL )
            {
                FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                    APP_WATCHDOG_THREAD_NAME );
                FAPEX_CLKPWR_Close( fd_CLKPWR );
                exit(1);
            }
        }
    }
    FAPEX_SYS_WaitEndThread( appWatchdogThreadHandle );
#endif /* USE_WATCHDOG */

    if(fd_CLKPWR > 0)
    {
        int32_t result;
        uint64_t component = FAPEX_CLKPWR_DAC0 | FAPEX_CLKPWR_DAC1
                             | FAPEX_CLKPWR_DAC2 | FAPEX_CLKPWR_DAC3
                             | FAPEX_CLKPWR_DAC4 | FAPEX_CLKPWR_DAC5;

        result = FAPEX_CLKPWR_Ioctl(fd_CLKPWR,FAPEX_IOCCOM_CLKPWR_START, (FAPEXIOCARG3_T)&component);
        if(result < 0)
            FAPI_SYS_PRINT_MSG("ERROR:[main.c] Video DACS power up %d\n",result);
    }

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appStartThreadFunction() end\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_WaitEndThread( appShellThreadHandle );
}

//***********************************************************************
//***********************************************************************
//** External Functions
//***********************************************************************
//***********************************************************************

#ifdef _LINUX_FAPI
//***************************************************************************
static void print_usage(char *name)
{
    FAPI_SYS_PRINT_MSG("usage: %s [fileName (startOffsetPos)] "
                       "[-s hd_mode (sd_mode)]\n",
                       name);
}
#endif
/*
*****************************************************************************
** \brief Testprogram main function
*****************************************************************************
*/
int32_t main( int32_t argc, char_t* argv[] )
{
    int32_t  result = 0;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, main() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    //usb_mount();
    //usb_enable(argc, argv);
#ifndef _LINUX_FAPI
    ///////////////////////////////////////////////////////////////////
    // This ugly delay loop is required to get the entire system
    // also running from flash, becuase the ROM boot loader first
    // starts an ARM application, but "av_demo" expects the ARC based
    // audio firmware already running.
    // During "make run" the audio firmware will be executed first,
    // and the "av_demo" will be started after a 1 second delay!
    ///////////////////////////////////////////////////////////////////

    #if defined(__arm__) && !defined(__GNUC__)
        #pragma O0
    #endif
    {
        uint32_t  delay;

        for( delay=0; delay < 0x00100000; delay++ );
    }
    #if defined(__arm__) && !defined(__GNUC__)
        #pragma O2
    #endif

    result = FAPEX_common_init();
    if ( result != FAPI_OK ) {
        exit( result );
    }
#else
    /* initial input settings */
    if(argc > 1) {
        int32_t  i;

        for(i=1; i<argc; i++) {
            if(strcmp("-s", argv[i]) == 0) {
                i ++;
                appHdSystemType = atoi(argv[i]);
                if(argc > (i+1)) {
                    if( (strlen(argv[i+1]) == 1)
                     && (isdigit(*argv[i+1])) ) {
                        i ++;
                        appSdSystemType = atoi(argv[i]);
                    }
                }
            }
            else if(*argv[i] == '-') {
                print_usage(argv[0]);
                exit(1);
            }
            else {
                strncpy(appTsInputFileName, argv[i], FAPEX_SYS_FMAXPATH);
                if( (argc > (i+1))
                 && (*argv[i+1] != '-') ) {
                    i ++;
                    appTsInputStartOffset = atoi(argv[i]);
                }
                else {
                    appTsInputStartOffset = 0;
                }
                appSetTsInput(FAPEX_TSD_INPUT_LINEAR_AUTO_MODE);
            }
        }
    }
    else
#endif
    {
        appTsInputFileName[0] = '\0';
        appSetTsInput(FAPEX_TSD_INPUT_CH_A);
    }

    appStartThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_START_THREAD_STACKSIZE,
            APP_START_THREAD_PRIORITY,
            appStartThreadFunction,
            0, 0, APP_START_THREAD_NAME );
    if( appStartThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR: creating thread \"%s\" failed\n",
                            APP_START_THREAD_NAME );
        exit(1);
    }

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, thread start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_StopPrintThread();
    FAPEX_SYS_StartMultitasking();

    FAPEX_SYS_WaitEndThread( appStartThreadHandle );

    exit( result );
}
