/*
*******************************************************************************
**
** \file        ./subsystems/audio/aal/src/aal.c
**
**              $Id: aal.c 5933 2011-03-30 08:28:54Z rseng $
**
** \brief       Audio abstraction layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
*******************************************************************************
*/

//#define APT_VOL_CTRL
//#define APT_11DB_ATTENUATE
#define APT_SIMPLE_SRC

/*
*******************************************************************************
**
** \note The following LINT statements have been suppressed:
- Warning 641: Converting enum to int.
- Info    717: do ... while(0);
- Info    788: enum constant 'anyConst' not used within defaulted switch
**
*******************************************************************************
*/
/*lint -save -e641 -e717 -e788 */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/


#define DEBUG 0

#include <stdint.h>
#include <string.h>


#include <fapex/abst_types.h>
#include <fapex/abst_libaudec.h>
#include <fapex/abst_libaudec_mpeg.h>
#include <fapex/abst_libaudec_ac3.h>
#include <fapex/abst_libaudec_ddpdcv.h>
#include <fapex/abst_libaudec_heaac.h>
#include <fapex/abst_libaudec_mixer.h>
#include <fapex/abst_libaudec_pcm.h>
#include <fapex/abst_libauout.h>
#include <fapex/abst_libtsd.h>
#include <fapex/abst_libhdmi.h>
#include <fapex/sys_config_mb86h61_devkit.h>

#define AAL_DEF_STREAM_INFO_TYPE            AAL_AUOUT_STREAM_PCM
#define AAL_DEF_STREAM_INFO_SAMPLEFREQ     AAL_AUOUT_48000_HZ
#define AAL_DEF_STREAM_INFO_LFEON           0
#define AAL_DEF_STREAM_INFO_CHANNELS        AAL_AUOUT_CHANNEL_STEREO
#define AAL_DEF_STREAM_INFO_SAMPLEBITS      16

#define AUDIO_TSD_CHANNEL     15
#define AUDIO_TSD_CHANNEL1    14
#define DRV_ID_TSD0_AUDIO_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + AUDIO_TSD_CHANNEL)
#define DRV_ID_TSD0_AUDIO_CH1 (FAPEXDEV_TSD0_SUB_MINOR_FIRST + AUDIO_TSD_CHANNEL1)
#define DRV_ID_AUDEC_0        (FAPEXDEV_AUDEC_MINOR_FIRST) /* for main data path */
#define DRV_ID_AUDEC_1        (FAPEXDEV_AUDEC_MINOR_FIRST + 1) /* for spdif alone data path */

// mixer index
#define GET_MIXER_INDEX(mixPath) ((mixPath) - AAL_PATH_MIX0)
// tsd device id
#define TSD_CH_MIXPATH_AUTO -1
#define DRV_ID_TSD0_MIXPATH_BASE (DRV_ID_TSD0_AUDIO_CH + 1)
#define GET_MIXPATH_TSD0_DEV_ID(mixPath) (GET_MIXER_INDEX(mixPath) + DRV_ID_TSD0_MIXPATH_BASE)
// audec device id
#define DRV_ID_AUDEC_MIXER_BASE (FAPEXDEV_AUDEC_MINOR_FIRST+ 2)
#define GET_MIXER_AUDEC_DEV_ID(mixPath) (GET_MIXER_INDEX(mixPath) + DRV_ID_AUDEC_MIXER_BASE)
#define DRV_ID_AUDEC_MIXPATH_BASE (DRV_ID_AUDEC_MIXER_BASE + AAL_PATH_NUMS - AAL_PATH_MIX0) // keep for mixer decoder
#define GET_MIXPATH_AUDEC_DEV_ID(mixPath) (GET_MIXER_INDEX(mixPath) + DRV_ID_AUDEC_MIXPATH_BASE)
// auout device id
#define DRV_ID_AUOUT_FINAL FAPEXDEV_AUOUT_MINOR_FIRST   // with mixer, final output AUOUT(i2s)
#define DRV_ID_AUOUT_MIXER_INPUT_BASE FAPEXDEV_AUOUT_PCM_MINOR_FIRST
#define GET_MIXER_INPUT_DEV_ID_MAIN(mixPath) (GET_MIXER_INDEX(mixPath) * 2 + DRV_ID_AUOUT_MIXER_INPUT_BASE)
#define GET_MIXER_INPUT_DEV_ID_SUB(mixPath) (GET_MIXER_INPUT_DEV_ID_MAIN(mixPath) + 1)
#define DRV_ID_AUOUT_MAINPATH GET_MIXER_INPUT_DEV_ID_MAIN(AAL_PATH_NUMS - 1)

extern int32_t fd_TSD[];


#include "aal.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#if defined(__GNUC__)

#define AAL_PARAMSBUF_SIZE      128

#endif /* defined(__GNUC__) */

#define AAL_DOLBY_LINE_MODE     2
#define AAL_DOLBY_RF_MODE       3

#define AAL_BM_4_BYTES          4

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{

    int32_t auoutMixerInput[2] ;     // input auout handle
    int32_t mixerInputBuffer[2] ;    // input auout's BM handle
    int32_t mixerHandle ;
    void*     previousMixer;
    void*     nextMixer;

    AAL_AudioPathT  mixerID ;
    uint32_t  paramsSize;
    void*     paramsPtr;

    int32_t   mixer_used ;
    int32_t   mixer_started ;
}AAL_MixerT;

typedef struct
{
   AAL_MixerT     aal_mixer[ AAL_PATH_NUMS -AAL_PATH_MIX0 ];
   AAL_MixerT*     firstMixerInChain;
   AAL_MixerT*     lastMixerInChain;

   uint32_t       mixerOpenNum;        //can use NUM,( means mix path can be used)
   uint32_t       mixerUsedNum;        // used NUM ,( means mix path is working )
}AAL_MixerDataT;

typedef struct
{
 #ifndef _LINUX_FAPI
    FAPI_SYS_HandleT        handleBm;
    FAPI_SYS_HandleT        handleTsd;
    FAPI_SYS_HandleT        handleOut;
    FAPI_SYS_HandleT        handleDec;
#else
    int32_t                handleBm;
    int32_t                handleTsd;
    int32_t                handleOut;   // NOTE: after add mixer decoder, mainpath handleOut will be replaced by a PCM buffer AUOUT as mixer main input
                                                // any want to access original AUOUT must use spdif path's AUOUT
    int32_t                handleDec;
#endif

    uint8_t                 started;
    AAL_AudioFormatT        format;
    AAL_AUDEC_DecoderEnumT decoder;
    uint32_t                pid;
#ifdef APT_VOL_CTRL
    AAL_AUOUT_AptVolParamsT aptData;
#endif

    /* parameters valid in 'started' mode/ */
    uint32_t                bmSize;
    uint8_t                 enableSpdif;
    uint8_t*                mp3MapBufAddr;  // TSD BM buffer maped
} AAL_AudioPathOpenT;

/*!
*******************************************************************************
**
**  \brief  AAL Open Parameter structure.
**
*******************************************************************************
 */
typedef struct
{
    AAL_OpenParamsT         params;
    uint8_t                 open;

    AAL_AudioPathOpenT      path[AAL_PATH_NUMS];
    AAL_MixerDataT          aalMixerData;
    /* Parameters for main audio decoding. */

    uint8_t                 audecRunning;
    uint8_t                 audecRestart;
    uint8_t                 decodingStarted;

    /* Parameters for SPDIF output. */
    AAL_SPDIFModeT          spdifMode;

    /* Parameters for audio decoding start notification */
    uint8_t                 startNotifyCbCalled;
    void                    (*startNotifyCb) (void* optData);
    void*                   startNotifyCbArg;

    AAL_DescrCtrlCb         descrCtrlCb;
    void*                   descrCtrlCbArg;

    /* Parameters for HDMI output */
    AAL_HDMISourceT         hdmiSource;
    uint8_t                 hdmiRunning;
    AAL_AUOUT_StreamInfoT   hdmiStreamInfo;

} AAL_OpenT;


typedef struct
{
    uint8_t                 muted;
    AAL_MixPathVolumeT      targetPathVolume;
    AAL_AUOUT_SampleFreqEnumT sampleFreq;
} AAL_AudioPathDataT;

/*!
*******************************************************************************
**
**  \brief  AAL Data structure.
**
*******************************************************************************
 */
typedef struct
{
    FAPEX_SYS_SemaphoreT    mutexHandle;

    AAL_AUDEC_DecoderEnumT  targetDecoder[AAL_AUDIOFORMAT_NUMS];
                            //!< holds corresponding decoder for each audio
                            //!< format.
    AAL_AudioPathDataT      path[AAL_PATH_NUMS];

    AAL_StereoModeT         stereoMode;
    uint8_t                 lvlLeft;
    uint8_t                 lvlRight;

    uint8_t                 hdmiEnabled;
    uint8_t                 spdifCpFlag;    //!< SPDIF copy protection status:
                                            //!< 1: copy protection is asserted,
                                            //!< 0: copy protection is cleared.
    uint8_t                 spdifLBitFlag;  //!< SPDIF status of L-Bit:
                                            //!< 1: set, 0: cleared.
    AAL_InitParamsT         initParams;
    AAL_OpenT               openData;

    // uint8_t*                mp3MapBufAddr;   // have move to mainpath AAL_AudioPathOpenT::mp3MapBufAddr

    #if defined(__GNUC__)
    uint8_t                 paramsBuf[AAL_PARAMSBUF_SIZE];
    #endif /* defined(__GNUC__) */

} AAL_DataT;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static AAL_DataT* audioDatPtr = NULL;
static uint32_t   audioDolbyCompMode = AAL_DOLBY_LINE_MODE;


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

static AAL_OpenT*       aalCheckHandle      (FAPI_SYS_HandleT handle,
                                             int32_t* errCodePtr);
static int32_t          aalStartHdmi        (AAL_DataT* inst,
                                             AAL_OpenT* openPtr);
static int32_t          aalStopHdmi         (AAL_OpenT* openPtr);
static int32_t          aalOpenDecoder   (AAL_DataT* inst,
                                             AAL_AudioFormatT format,
                                             int32_t audecDevId,
                                             int32_t handleTsd,
                                             int32_t handleOut,
                                             AAL_AUDEC_DecoderEnumT* decoderPtr,
                                             int32_t* errCodePtr);
static int32_t          aalSetDecoderParams(AAL_AUDEC_DecoderEnumT target_decoder,
                                             int32_t handleDec,
                                             uint8_t ac3ToSpdif,
                                             uint8_t suppressErrorRecognition, AAL_AudioFormatT format,
                                             AAL_AUOUT_SampleFreqEnumT sampleFreq); // sampleFreq used only format is AAL_AUDIOFORMAT_PCM
static int32_t          aalOpenTsd          (AAL_DataT* inst,
                                             AAL_AudioPathT path,
                                             FAPI_SYS_HandleT handleBm,
                                             int32_t* errCodePtr);
static int32_t          aalIntStartMixPath    (AAL_DataT* inst,
                                             AAL_OpenT* openPtr,
                                             AAL_AudioPathT mixerID,
                                             AAL_MixPathPramsT*  mixerPathParams);
static int32_t          aalIntStart         (AAL_DataT* inst,
                                             AAL_OpenT* openPtr,
                                             AAL_AudioFormatT format,
                                             uint32_t pid);
static int32_t          aalIntStartSPDIF    (AAL_DataT* inst,
                                             AAL_OpenT* openPtr,
                                             AAL_AudioFormatT format,
                                             uint32_t pid);
static int32_t          aalIntStop          (AAL_OpenT* openPtr,
                                             AAL_AudioPathT path);
static int32_t          aalIntStopModules   (AAL_OpenT* openPtr,
                                             AAL_AudioPathT path);
static int32_t          aalIntSetStereoMode (AAL_DataT* inst,
                                                  AAL_OpenT* openPtr);
static int32_t          aalIntSetVolume (AAL_DataT* inst,
                                             AAL_OpenT* openPtr);
int32_t                 aalAddDefaultAptsCb( int32_t audioOutHandle,
                                                  AAL_AudioFormatT format );
static int32_t          aalRemoveDefaultAptsCb (int32_t audioOuthandle);

static int32_t          audioBmWrite        (uint8_t *bmMapAddr,
                                             const void* writeBuffer,
                                             uint32_t length,
                                             uint32_t size,
                                             uint32_t timeout,
                                             int32_t handleTsd);
static int32_t aalOpenMixer( AAL_OpenT* openPtr, FAPI_SYS_HandleT audioOutHandle);
static int32_t aalCloseAllMixer(  AAL_OpenT* openPtr );
static void aalInitMixerID( AAL_DataT* inst  );
static int32_t aalIntAllMixerStart (  AAL_OpenT* openPtr  );
static int32_t aalIntAllMixerStop (  AAL_OpenT* openPtr  );
static int32_t aalIntSetMixPathVolume ( AAL_OpenT* openPtr , AAL_AudioPathT pathCnt ,
                                           AAL_MixPathVolumeT* targetPathVolume);
static int32_t aalGetAuoutPath( AAL_DataT* inst, int32_t audioOutHandle, AAL_AudioPathT* path );
// common for mainpath and mixpath call
static int32_t aalIntFeedPathBuffer (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId,
                                const void* bufferPtr, int32_t bytes, uint32_t timeout );
static int32_t aalIntClearPathBuffer (FAPI_SYS_HandleT handle , AAL_AudioPathT pathId);
static int32_t aalIntGetPathDecStatus (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId ,
                                 AAL_AUDEC_StatusEnumT* statusPtr);
static int32_t aalIntGetPathStreamInfo (FAPI_SYS_HandleT handle,  AAL_AudioPathT pathId ,
                            AAL_AUOUT_StreamInfoT* infoPtr);
static uint32_t aalIntGetPathBufferSize (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId);
static uint32_t aalIntGetPathBufferLevel (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId);
static int32_t aalAddMixPathAptsCb( FAPI_SYS_HandleT audioOutHandle,AAL_MixPathPramsT* mixerPathParams
                                               ,AAL_AUOUT_SampleFreqEnumT APTsampleFreq );
static int32_t aalRemoveMixPathAptsCb(FAPI_SYS_HandleT audioOutHandle);
static int32_t aalIntPathBufMmap(FAPI_SYS_HandleT handle, AAL_AudioPathT pathId);
static int32_t aalIntPathBufMunmap(FAPI_SYS_HandleT handle, AAL_AudioPathT pathId);

#ifndef _LINUX_FAPI

#if defined(__GNUC__)

static void             aalSwapLongwords    (uint8_t* bufPtr);

#endif /* defined(__GNUC__) */

#endif /*_LINUX_FAPIs*/


//*****************************************************************************
//*****************************************************************************
//** API Functions
//*****************************************************************************
//*****************************************************************************

/*!
*******************************************************************************
** \brief Initialize AAL module.
**
** \param   paramsPtr   Pointer to init parameters.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \note    This function shall be called once at system startup.
**
** \sa      AAL_Exit
*******************************************************************************
*/
int32_t AAL_Init (const AAL_InitParamsT* paramsPtr)
{
    int32_t                     err_code = FAPEX_OK;
    FAPEX_AUDEC_OpenParamStrT   audecOpenParam;
    AAL_AudioPathT              pathCnt;
    AAL_AudioFormatT            formatCnt;
    int32_t                     handleDec ;

    if ( audioDatPtr != NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED;

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Init: err %i \n", err_code);

        return err_code;
    }

    do
    {
        if ( paramsPtr == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            break;
        }

        audioDatPtr = (AAL_DataT*)FAPI_SYS_MALLOC_CACHED(sizeof(AAL_DataT));

        if ( audioDatPtr == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY;
            break;
        }

        memset(audioDatPtr, 0, sizeof(AAL_DataT));

        audioDatPtr->mutexHandle = FAPEX_SYS_CreateSemaphore(1);
        if ( audioDatPtr->mutexHandle == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_SEMAPHORE_CREATE;
            break;
        }

        audioDatPtr->hdmiEnabled                 = 1;
        audioDatPtr->spdifCpFlag                 = 0;
        audioDatPtr->spdifLBitFlag               = 0;
        audioDatPtr->initParams                  = *paramsPtr;

        audioDatPtr->openData.open               = 0;
        audioDatPtr->openData.audecRunning       = 0;
        audioDatPtr->openData.hdmiRunning        = 0;
        audioDatPtr->openData.decodingStarted    = 0;

        /* set default volume levels */
        audioDatPtr->stereoMode                  = AAL_STEREO_MODE_STEREO;
        audioDatPtr->lvlLeft                     = 0x80;
        audioDatPtr->lvlRight                    = 0x80;

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS; pathCnt++ )
        {
            /* set default mute status */
            audioDatPtr->path[pathCnt].muted = 0;
        }

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS; pathCnt++ )
        {
            audioDatPtr->openData.path[pathCnt].started = 0;
            audioDatPtr->openData.path[pathCnt].mp3MapBufAddr = NULL;
        }

        /* ----- Get list of available decoders ---------------------------- */

        for ( formatCnt = (AAL_AudioFormatT)0; formatCnt < AAL_AUDIOFORMAT_NUMS; formatCnt++ )
        {
            audioDatPtr->targetDecoder[formatCnt] = AAL_AUDEC_DECODER_UNDEF;
        }

        /* --------- MPEG ----------- */
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_MPEG;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1,"[AAL] Opening MPEG decoder: ");

       if(handleDec < 0) {
          FAPI_SYS_PRINT_DEBUG(1,"failed:%d\n", err_code);
          break;
       }
       else
       {
           FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_MPEG] = AAL_AUDEC_DECODER_MPEG;

            FAPEX_AUDEC_Close(handleDec);
        }

        /* ----- HE-AAC ----- */

        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = AAL_AUDEC_DECODER_HEAAC;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                          O_RDWR,
                                         (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening HE-AAC decoder: ");
        if(handleDec < 0) {
           	   FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                err_code = FAPEX_OK; /* This is a warning only! */
        }
         else
         {
             FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
             audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC] = AAL_AUDEC_DECODER_HEAAC;
            FAPEX_AUDEC_Close(handleDec);
        }

       /* ----- AAC ----- */
        if ( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC]
             != AAL_AUDEC_DECODER_UNDEF )
        {
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AAC] = AAL_AUDEC_DECODER_HEAAC;
        }

       /* ----- DD ----- */
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = AAL_AUDEC_DECODER_DD;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                        O_RDWR,
                                       (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening DD decoder:     ");
        if(handleDec < 0)
        {
               FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
               err_code = FAPEX_OK; /* This is a warning only! */
        }
        else{
               FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
               audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] = AAL_AUDEC_DECODER_DD;

               FAPEX_AUDEC_Close(handleDec);
       }

       /* ----- DD+ ----- */

        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = AAL_AUDEC_DECODER_DDPDCV;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                         O_RDWR,
                                        (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening DD+ decoder:    ");
        if(handleDec < 0)
        {
           	  err_code = FAPEX_OK;
              FAPI_SYS_PRINT_DEBUG(1,"NOT supported!\n");
        }
        else{
           	    FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] = AAL_AUDEC_DECODER_DDPDCV;

            FAPEX_AUDEC_Close(handleDec);
        }

        /* ----- AC-3 ----- */

        if ( (audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] == AAL_AUDEC_DECODER_UNDEF )
            &&(audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] == AAL_AUDEC_DECODER_UNDEF ))
        {
            memset(&audecOpenParam, 0, sizeof(audecOpenParam));

            audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
            audecOpenParam.codecType = AAL_AUDEC_DECODER_AC3;
            handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                             O_RDWR,
                                             (FAPEXOPENARG3_T)&audecOpenParam );
            FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening AC-3 decoder:   ");
            if(handleDec < 0)
            {
                    FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                    err_code = FAPEX_OK; /* This is a warning only! */
             }
             else{
                	  FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
                    audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_AC3;
                FAPEX_AUDEC_Close(handleDec);
            }
        }
        else
        {
            if( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] != AAL_AUDEC_DECODER_UNDEF )
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DD;
            else
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DDPDCV;

        }

         /* ----- WMA ----- */
         memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = AAL_AUDEC_DECODER_WMA;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                         O_RDWR,
                                        (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening WMA decoder:    ");
        if(handleDec < 0)
        {
            FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
            err_code = FAPEX_OK; /* This is a warning only! */
        }
        else
        {
            err_code = FAPEX_OK;
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_WMA] = AAL_AUDEC_DECODER_WMA;
            FAPEX_AUDEC_Close(handleDec);
        }

         /* ----- PCM ----- */
         memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = AAL_AUDEC_DECODER_PCMOUTPUT;
        handleDec = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                         O_RDWR,
                                        (FAPEXOPENARG3_T)&audecOpenParam );
        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening PCM decoder:    ");
        if(handleDec < 0)
        {
            FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
            err_code = FAPEX_OK; /* This is a warning only! */
        }
        else
        {
            err_code = FAPEX_OK;
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_PCM] = AAL_AUDEC_DECODER_PCMOUTPUT;
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_WAV] = AAL_AUDEC_DECODER_PCMOUTPUT;
            FAPEX_AUDEC_Close(handleDec);
        }

        /* ----- MIX ----- */
        aalInitMixerID( audioDatPtr );
        #if 0   // NOTE: may error
        err_code = aalOpenMixer( &audioDatPtr->openData, (FAPI_SYS_HandleT) DRV_ID_AUOUT_FINAL);

        FAPI_SYS_PRINT_DEBUG(1, "[AAL] aalOpenMixer:   ");

        if ( err_code != FAPI_OK )
        {
            /* mandatory decoder, opening must not fail! */

            FAPI_SYS_PRINT_DEBUG(1, "failed: %d\n", err_code);
            break;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            err_code = aalCloseAllMixer( &(audioDatPtr->openData) );
        }
        #endif
    }
    while ( 0 );
    if ( err_code != FAPEX_OK )
    {
        (void)AAL_Exit();
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Init: err %i \n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Exit/deinitialize AAL module.
**
** \param   -
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \note    This function is never needed if AAL_Init is called only
**          once at system startup, as recommended.
**
** \sa      AAL_Init
*******************************************************************************
*/
int32_t AAL_Exit (void)
{
    if ( audioDatPtr != NULL )
    {
        (void)AAL_Close(&(audioDatPtr->openData));

        if ( audioDatPtr->mutexHandle != NULL )
        {
            (void)FAPEX_SYS_DestroySemaphore(audioDatPtr->mutexHandle);
        }

        FAPI_SYS_FREE(audioDatPtr);

        audioDatPtr = NULL;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Exit: OK \n");

    return FAPEX_OK;
}

/*!
*******************************************************************************
** \brief Audio format check.
**
** This function checks if the given audio format is decodable.
**
** \param format        Audio format that needs to be checked
** \param resultPtr     Pointer to the result if the given format is decodable;
**                      1: decodable, 0: not decodable.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_FormatIsDecodable (AAL_AudioFormatT format, uint8_t* resultPtr)
{
    int32_t          err_code = FAPEX_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( (resultPtr == NULL)
           || (((int32_t)format) < 0)
           || (format >= AAL_AUDIOFORMAT_NUMS) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        if ( audioDatPtr->targetDecoder[format] == AAL_AUDEC_DECODER_UNDEF )
        {
            *resultPtr = 0;
        }
        else
        {
            *resultPtr = 1;
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Open AAL module.
**
** \param   paramsPtr   Pointer to open parameters.
** \param   errCodePtr  Optionally returns error_status. Set to NULL if not
**                      needed.
**
** \return  NULL in case of failure, handle for further usage on success.
**
** \note    This shall be called before any audio decoding (for live or one of
**          the some other playback modes like MP3) is started. The returned
**          pointer is used for all further operations related to decoding
**          until the module is closed by AAL_Close.
**
** \sa      AAL_Close
*******************************************************************************
*/
FAPI_SYS_HandleT AAL_Open (const AAL_OpenParamsT* paramsPtr, int32_t* errCodePtr)
{
    AAL_OpenT*                 openPtr  = NULL;
    int32_t             pathCnt;
    AAL_AudioPathOpenT*        pathPtr;
    FAPEX_AUOUT_OpenParamStrT  auout_open_params;
    int32_t                    handleOut = 0;;
    int32_t                    err_code  = FAPEX_OK;
    AAL_MixerT*         aal_mixer = NULL;
    uint8_t                    cnt;
    AAL_AUOUT_SpeakerEnumT speaker_id;
    FAPEX_AUOUT_PcmInfoStrT defaultPcmInfo;
    FAPEX_AUOUT_MuteParamStrT muteParam;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( paramsPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( paramsPtr->numSpeakers > AAL_AUOUT_SPEAKER_ALL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( (paramsPtr->addAptsCb && !paramsPtr->removeAptsCb)
              || (!paramsPtr->addAptsCb && paramsPtr->removeAptsCb))
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);


        if ( audioDatPtr->openData.open )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN;
        }
        else
        {
            openPtr = &(audioDatPtr->openData);

            openPtr->params = *paramsPtr;

            /* set default SPDIF mode */
            openPtr->spdifMode = AAL_SPDIFMODE_PCM;

            /* set default HDMI mode */
            openPtr->hdmiSource = AAL_HDMISOURCE_I2S;

             /* set default APT callbacks */
            if ( !paramsPtr->addAptsCb )
                openPtr->params.addAptsCb = aalAddDefaultAptsCb;

            if ( !paramsPtr->removeAptsCb )
                openPtr->params.removeAptsCb = aalRemoveDefaultAptsCb;

            /* set parameters for HDMI output */
            openPtr->hdmiStreamInfo.type         = AAL_AUOUT_STREAM_UNDEF;
            openPtr->hdmiStreamInfo.sampleFreq   = AAL_AUOUT_UNDEF_HZ;
            openPtr->hdmiStreamInfo.channels     = AAL_AUOUT_CHANNEL_UNDEF;
            openPtr->hdmiStreamInfo.lfeOn        = 0;
            openPtr->hdmiStreamInfo.sampleBits   = -1;

            /* open audio output */
            memset(&auout_open_params, 0, sizeof(auout_open_params));
            auout_open_params.enableFlag = FAPEX_AUOUT_ENABLE_SPDIF;
            if(openPtr->params.numSpeakers > 0 )
                auout_open_params.enableFlag |= FAPEX_AUOUT_ENABLE_I2S;
            auout_open_params.enableFlag |= FAPEX_AUOUT_ENABLE_HDMI;

            for ( cnt = 0; cnt < openPtr->params.numSpeakers; cnt++ )
            {
                /* NOTE: Security check: should never occur! speakerArr has
                         (AAL_AUOUT_SPEAKER_ALL + 1) entries */
                if ( cnt >= AAL_AUOUT_SPEAKER_ALL )
                {
                    break;
                }

                if(openPtr->params.speaker[cnt] != AAL_AUOUT_SPEAKER_UNDEF )
                    auout_open_params.channelFlag |= (1<<openPtr->params.speaker[cnt]);
            }

            handleOut = FAPEX_AUOUT_Open ( DRV_ID_AUOUT_FINAL,
                                           O_RDWR,
                                           (FAPEXOPENARG3_T)&auout_open_params );

            if ( handleOut < 0 )
            {
                FAPI_SYS_PRINT_MSG("[AAL]Open auout error(%d)\n",handleOut);
            }

             /*
                after mixer open, diagram like below
  TSD/BM --> DEC -- [SPDIF] --------------------------------
                TSD/BM --> DEC --> AUOUT(pcm) --> MIXER_00_sub   |          <-- lastMixerInChain
                                         --> AUOUT(pcm) --> MIXER_00_main --> AUOUT(i2s)
                                        |
                                        |--------------- OUT --------------
                                                                                                 |           ^: next        v: prev
                TSD/BM --> DEC --> AUOUT(pcm) --> MIXER_0X_sub     |
  TSD/BM --> DEC -- [MAIN] --> AUOUT(pcm) --> MIXER_0X_main --          <-- firstMixerInChain

                // NOTE:    1. like main and spdif, we use AUOUT device id constant. and auoutMixerInput[] will be file descriptor
                                 2. mixpath mean mixer sub input path TSD/BM --> DEC --> AUOUT(pcm) like mainpath, mixer mean mixer decoder
              */
             for ( pathCnt = AAL_PATH_MAIN; pathCnt < AAL_PATH_NUMS; pathCnt++ )
             {
                pathPtr = openPtr->path + pathCnt;
                pathPtr->started   = 0;
                pathPtr->format    = AAL_AUDIOFORMAT_NONE;
                pathPtr->decoder   = AAL_AUDEC_DECODER_UNDEF;
                pathPtr->pid       = 0;

                //path main or spdif default use the first mixer's 0 in chain.
                if ( pathCnt > AAL_PATH_SPDIF )
                { //mixer path
                    err_code = aalOpenMixer(openPtr, (FAPI_SYS_HandleT) DRV_ID_AUOUT_FINAL);

                    if ( err_code != FAPI_OK )
                    {
                        FAPI_SYS_PRINT_DEBUG(1, "failed: %d\n", err_code);
                        break;
                    }
                    //get the firs mixer(it will be the new mixer just added.)
                    pathPtr->handleOut = openPtr->aalMixerData.firstMixerInChain->auoutMixerInput[1];  //mixpath decoder output to mixer decoder sub input
                }
                else if ( pathCnt == AAL_PATH_SPDIF )
                {
                    //spdif stream direct to hardware output auout,because it's not PCM data,can't be mixed.
                    pathPtr->handleOut = handleOut;
                }
                else
                {
                    //main stream to first mixer's main input, and first mixer will open when loop finish
                    // so assign mainpath handleOut when loop finish
                }

                pathPtr->handleTsd = aalOpenTsd(audioDatPtr,pathCnt, NULL, &err_code);

                if ( pathPtr->handleTsd < 0 )
                {
                    err_code = pathPtr->handleTsd;
                    break;
                }
             }
             if( err_code != FAPEX_OK )
             {
                aalCloseAllMixer( openPtr );

                if ( handleOut > 0 )
                {
                    (void)FAPEX_AUOUT_Close(handleOut);
                }

                 for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS -1;
                      pathCnt++ )
                 {
                     pathPtr = openPtr->path + pathCnt;

                     if (pathPtr->handleTsd > 0 )
                     {

                         err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleTsd,
                                                    FAPEX_IOCCOM_CLOSE_SUBID,
                                                    (FAPEXIOCARG3_T)NULL );
                         if(err_code < 0)
                         {
                             FAPI_SYS_PRINT_ERR("ERROR Close Audio TSD channel. %d\n", err_code);
                         }

                         pathPtr->handleTsd = -1;
                     }
                 }
             }
             else
             {
                 //main stream to first mixer's main input
                 openPtr->path[AAL_PATH_MAIN].handleOut = openPtr->aalMixerData.firstMixerInChain->auoutMixerInput[0];
                 openPtr->open = 1;
                //init global setting.
                pathPtr = openPtr->path + AAL_PATH_SPDIF;
                FAPEX_AUOUT_Ioctl(pathPtr->handleOut, FAPEX_IOCCOM_AUOUT_SET_SPDIFCOPYPROTECT, audioDatPtr->spdifCpFlag);
                FAPEX_AUOUT_Ioctl(pathPtr->handleOut, FAPEX_IOCCOM_AUOUT_SET_SPDIFLBIT, audioDatPtr->spdifLBitFlag);
                // we use default HW auout setting,if main path not start,APP will get these message.
                defaultPcmInfo.sampleFreq =  AAL_DEF_STREAM_INFO_SAMPLEFREQ;
                defaultPcmInfo.channelCount =  AAL_DEF_STREAM_INFO_CHANNELS;
                defaultPcmInfo.sampleBits =  AAL_DEF_STREAM_INFO_SAMPLEBITS;
                FAPEX_AUOUT_Ioctl(pathPtr->handleOut, FAPEX_IOCCOM_AUOUT_SET_STREAM_TYPE, AAL_DEF_STREAM_INFO_TYPE);
                FAPEX_AUOUT_Ioctl(pathPtr->handleOut, FAPEX_IOCCOM_AUOUT_SET_PCM_INFO, &defaultPcmInfo);
                //add needed APTs for SPDIF(HW auout) ,can use customer's addAptsCb.
                //(always use AAL_AUDIOFORMAT_MPEG format to add APT_11DB_ATTENUATE control for HW auout )
                if( openPtr->params.addAptsCb )
                    err_code = openPtr->params.addAptsCb(pathPtr->handleOut, AAL_AUDIOFORMAT_MPEG);
                /* set current mute settings */
                if ( audioDatPtr->path[AAL_PATH_MAIN].muted )
                {
                    muteParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
                    muteParam.muteFlag = 0;
                    FAPEX_AUOUT_Ioctl(pathPtr->handleOut, FAPEX_IOCCOM_AUOUT_SET_MUTE, &muteParam);
                }
                //init all mixer path's default volume = max,pan=center. (val of spdif path will not work)
                for (pathCnt = AAL_PATH_MAIN; pathCnt<AAL_PATH_NUMS ; pathCnt++)
                {
                    for (speaker_id=0; speaker_id<AAL_AUOUT_SPEAKER_ALL ;speaker_id++)
                    {
                        audioDatPtr->path[pathCnt].targetPathVolume.mixerPathVolume[speaker_id] = AAL_VOLUMELEVEL_MAX;
                    }
                    audioDatPtr->path[pathCnt].targetPathVolume.pathPan = 0;  //center
                }

                (void)aalIntSetStereoMode (audioDatPtr, openPtr);  //HW auout
                (void)aalIntSetVolume (audioDatPtr, openPtr);  //HW auout
                //start all mixer decoder
                if ( FAPI_OK == err_code )
                {
                    err_code = aalIntAllMixerStart( openPtr );
                }
             }

        }
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Open: err %i \n", err_code);

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return (FAPI_SYS_HandleT)openPtr;
}

/*!
*******************************************************************************
** \brief   Close AAL module.
**
** \param   handle     Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \sa      AAL_Open
*******************************************************************************
*/
int32_t AAL_Close (FAPI_SYS_HandleT handle)
{
    int32_t             err_code = FAPEX_OK;
    AAL_OpenT*          openPtr;
    AAL_AudioPathT      pathCnt;
    AAL_AudioPathOpenT* pathPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL)
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        for (pathCnt=(AAL_AudioPathT)0 ; pathCnt<(AAL_PATH_NUMS ); pathCnt++ )
        {//stop all path
            (void)aalIntStop (openPtr, pathCnt );
        }

        /* close AUOUT */
        pathPtr = openPtr->path + AAL_PATH_SPDIF;
        if (pathPtr->handleOut > 0)
        {
            FAPEX_AUOUT_Close( pathPtr->handleOut );
            pathPtr->handleOut   = -1;
            pathPtr->enableSpdif = 0;
        }

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS; pathCnt++ )
        {
            pathPtr = openPtr->path + pathCnt;

            if( pathPtr->handleTsd > 0)
            {
                err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleTsd,
                                             FAPEX_IOCCOM_CLOSE_SUBID,
                                            (FAPEXIOCARG3_T)NULL );
                if(err_code < 0) {
                    FAPI_SYS_PRINT_ERR("ERROR Close Audio TSD channel. %d\n", err_code);
                }
                pathPtr->handleTsd = -1;
            }

            if ( pathPtr->handleDec > 0 )
            {
                FAPEX_AUDEC_Close(pathPtr->handleDec);
                pathPtr->handleDec = -1;
            }
        }
        openPtr->open = 0;
        //close all mixer opened
        aalCloseAllMixer( openPtr );

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }


    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Close: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Start audio decoding and output for main (I2S) output path.
**
** \param handle    Audio handle returned by AAL_Open.
** \param format    Format of given audio stream.
** \param pid       Audio PID to decode.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Start (FAPI_SYS_HandleT handle, AAL_AudioFormatT format, uint32_t pid)
{
    AAL_OpenT*  openPtr;
    int32_t     err_code = FAPEX_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL)
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        err_code = aalIntStart(audioDatPtr, openPtr, format, pid);
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Start(format %i pid %i): %i\n",
                         format, pid, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Start audio decoding and output in mix path
**
** \param handle    Audio handle returned by AAL_Open.
** \param mixerID   Mixer path will be started.
** \param mixerPathParams    Mixer path parameter.
** \NOTE: must not call AAL_StartMixPath() after call AAL_Start() immediately because final AUOUT stream information have not updated by AUDEC,
**            and when mixpath have started, mainpath AUDEC update final AUOUT will cause mixpath don't work.
**            so, should call AAL_StartMixPath() delay 1s after call AAL_Start() or don't call AAL_Start() or call AAL_Start() with not exist audio stream pid

** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_StartMixPath (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                            AAL_MixPathPramsT* mixerPathParams)
{
    AAL_OpenT*  openPtr;
    int32_t     err_code = FAPI_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( openPtr != NULL)
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        err_code = aalIntStartMixPath(audioDatPtr, openPtr, mixerID, mixerPathParams);

       (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_StartMixPath(mixerID %d format %d pid %d): %d\n",
                         mixerID ,mixerPathParams->format, mixerPathParams->pid, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Stop audio decoding and output for main (I2S) output path.
**
** \param handle    Audio handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Stop (FAPI_SYS_HandleT handle)
{
    AAL_OpenT*          openPtr;
    int32_t             err_code   = FAPEX_OK;
    AAL_AudioFormatT    currFormat = AAL_AUDIOFORMAT_NONE;
    AAL_AudioPathT      pathCnt;
    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

         if ( (openPtr->path[AAL_PATH_MAIN].started)
          && (openPtr->spdifMode == AAL_SPDIFMODE_PASSTHROUGH)
          && (!(openPtr->path[AAL_PATH_SPDIF].started))
          && (openPtr->path[AAL_PATH_MAIN].pid
          == openPtr->path[AAL_PATH_SPDIF].pid) )
        {
            currFormat = openPtr->path[AAL_PATH_MAIN].format;
        }

        for (pathCnt=AAL_PATH_MAIN ; pathCnt<AAL_PATH_MIX0; pathCnt++ )
        {//stop all path
            (void)aalIntStop (openPtr, pathCnt );
        }

        if ( currFormat != AAL_AUDIOFORMAT_NONE )
        {
            (void)aalIntStartSPDIF (audioDatPtr, openPtr, currFormat,
                                     openPtr->path[AAL_PATH_SPDIF].pid);
        }

       (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Stop: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Stop audio decoding and output for mixer path.
**
** \param handle    Audio handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_StopMixPath (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID )
{
    AAL_OpenT*          openPtr;
    int32_t             err_code   = FAPI_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        aalRemoveMixPathAptsCb(openPtr->path[mixerID].handleOut);
        err_code = aalIntStop (openPtr, mixerID);

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_StopMixPath[%d]: %d\n", mixerID , err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Set SPDIF operation mode.
**
** \param handle    Audio handle returned by AAL_Open.
** \param mode      Sets SPDIF output either to off, passthrough mode or I2S
**                  mode.
** \param format    Format of passthrough stream, only needed for mode
**                  AAL_SPDIFMODE_PASSTHROUGH.
** \param pid       PID of passthrough stream, only needed for mode
**                  AAL_SPDIFMODE_PASSTHROUGH.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetSPDIFMode (FAPI_SYS_HandleT handle, AAL_SPDIFModeT mode,
                 AAL_AudioFormatT format, uint32_t pid)
{
    AAL_OpenT*          openPtr;
    AAL_AudioPathOpenT* pathMainPtr;
    AAL_AudioPathOpenT* pathSpdifPtr;
    int32_t             err_code = FAPEX_OK;


    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetSPDIFMode(mode %i format %i pid %i\n",
                         mode, format, pid);

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        pathMainPtr  = openPtr->path + AAL_PATH_MAIN;
        pathSpdifPtr = openPtr->path + AAL_PATH_SPDIF;

        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        switch ( mode )
        {
            case AAL_SPDIFMODE_OFF:
                err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                if ( err_code == FAPEX_OK )
                {
                    openPtr->spdifMode = mode;
                    /* Lock PCM output on SPDIF */
                    err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                  FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK,
                                                  (FAPEXIOCARG3_T)AAL_SPDIFPCM_LOCK);
                    /* disable passthrough mode in main decoder */
                    if ( (err_code == FAPI_OK) && pathMainPtr->started )
                    {
                        if ((pathMainPtr->decoder == AAL_AUDEC_DECODER_AC3)
                          || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DD)
                          || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DDPDCV))
                        {
                            // don't stop the main path. instead change the decoder parameters only
                            err_code = aalSetDecoderParams( pathMainPtr->decoder,
                                                            pathMainPtr->handleDec,
                                                            0,
                                                            openPtr->params.suppressErrorRecognition,
                                                            pathMainPtr->format,
                                                            AAL_AUOUT_UNDEF_HZ);
                            if ( err_code == FAPI_OK )
                            {
                                pathMainPtr->enableSpdif = 0;
                            }
                        }
                    }
                }
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:
                if ( (openPtr->spdifMode == AAL_SPDIFMODE_PCM)
                      && (openPtr->hdmiSource == AAL_HDMISOURCE_SPDIF) )
                {
                    (void)aalStopHdmi(openPtr);
                }

                if ( pathSpdifPtr->started )
                {
                    /* SPDIF audio is already running in passthrough mode. */
                    if ( pathSpdifPtr->pid != pid )
                    {
                        err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                        if ( err_code != FAPEX_OK )
                        {
                            break;
                        }
                    }
                    else
                    {
                        /* nothing to do */
                        break;
                    }
                }

                if ( (pathMainPtr->started) && (pathMainPtr->pid == pid) )
                {
                    if ( (pathMainPtr->format != AAL_AUDIOFORMAT_AC3)
                      && (pathMainPtr->format != AAL_AUDIOFORMAT_DDP) )
                    {
                        err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
                    }
                    else
                    {
                        openPtr->spdifMode   = mode;

                        pathSpdifPtr->format = format;
                        pathSpdifPtr->pid    = pid;

                        // don't stop the main path. instead change the decoder parameters to
                        // go in passthrough mode
                        if ( (pathMainPtr->decoder == AAL_AUDEC_DECODER_AC3)
                              || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DD)
                              || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DDPDCV))
                        {
                            err_code = aalSetDecoderParams( pathMainPtr->decoder,
                                                            pathMainPtr->handleDec,
                                                            1,
                                                            openPtr->params.suppressErrorRecognition,
                                                            pathMainPtr->format,
                                                            AAL_AUOUT_UNDEF_HZ);

                            if ( err_code == FAPI_OK )
                            {
                                pathMainPtr->enableSpdif = 1;
                            }
                        }
                        else
                        {
                            err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                          FAPEX_IOCCOM_AUOUT_SET_SPDIFMODE,
                                                          (FAPEXIOCARG3_T)AAL_AUOUT_SPDIF_PASSTHROUGH);

                            err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                          FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK,
                                                          (FAPEXIOCARG3_T)AAL_SPDIFPCM_LOCK);

                            if ( err_code == FAPI_OK )
                            {
                                pathMainPtr->enableSpdif = 0;
                            }
                        }
                        // re-enable previously stopped SPDIF
                        err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                      FAPEX_IOCCOM_RESET,
                                                     (FAPEXIOCARG3_T)NULL);
                        if (err_code != FAPEX_OK)
                        {
                            FAPI_SYS_PRINT_ERR("ERROR Reset AUDEC, err_code %d\n", err_code);
                        }

                    }
                }
                else
                {
                    if ( format != AAL_AUDIOFORMAT_AC3 )
                    {
                        err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
                    }
                    else
                    {
                        openPtr->spdifMode   = mode;

                        pathSpdifPtr->format = format;
                        pathSpdifPtr->pid    = pid;

                        err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                      FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK,
                                                      (FAPEXIOCARG3_T)AAL_SPDIFPCM_LOCK);

                        if ( (err_code == FAPI_OK) && pathMainPtr->started )
                        {
                            if ((pathMainPtr->decoder == AAL_AUDEC_DECODER_AC3)
                              || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DD)
                              || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DDPDCV))
                            {
                                // don't stop the main path. instead change the decoder parameters only
                                err_code = aalSetDecoderParams( pathMainPtr->decoder,
                                                                pathMainPtr->handleDec,
                                                                0,
                                                                openPtr->params.suppressErrorRecognition,
                                                                pathMainPtr->format,
                                                                AAL_AUOUT_UNDEF_HZ);
                                if ( err_code == FAPI_OK )
                                {
                                    pathMainPtr->enableSpdif = 0;
                                }
                            }
                        }
                        if ( err_code == FAPI_OK )
                        {
                            err_code = aalIntStartSPDIF (audioDatPtr, openPtr,
                                                         format, pid);
                        }
                    }
                }
                break;

            case AAL_SPDIFMODE_PCM:

                if ( openPtr->spdifMode != mode )
                {
                    /* Stop passthrough SPDIF audio. HDMI*/
                    if ( (openPtr->spdifMode == AAL_SPDIFMODE_PASSTHROUGH)
                          && (openPtr->hdmiSource == AAL_HDMISOURCE_SPDIF) )
                    {
                        (void)aalStopHdmi(openPtr);
                    }
                    /* Stop passthrough SPDIF audio. */
                    err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                    if ( err_code != FAPEX_OK )
                    {
                        break;
                    }

                    /* UnLock PCM output on SPDIF */
                    err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                  FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK,
                                                  (FAPEXIOCARG3_T)AAL_SPDIFPCM_UNLOCK);

                    err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                  FAPEX_IOCCOM_AUOUT_SET_SPDIFMODE,
                                                  (FAPEXIOCARG3_T)AAL_AUOUT_SPDIF_PCM);

                    /* disable passthrough mode in main decoder */
                    if ( (err_code == FAPI_OK) && pathMainPtr->started )
                    {
                        if ((pathMainPtr->decoder == AAL_AUDEC_DECODER_AC3)
                          || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DD)
                          || (pathMainPtr->decoder == AAL_AUDEC_DECODER_DDPDCV))
                        {
                            /* don't stop the main path. instead change the decoder parameters only */
                            err_code = aalSetDecoderParams( pathMainPtr->decoder,
                                                            pathMainPtr->handleDec,
                                                            0,
                                                            openPtr->params.suppressErrorRecognition,
                                                            pathMainPtr->format,
                                                            AAL_AUOUT_UNDEF_HZ);
                            if ( err_code == FAPI_OK )
                            {
                                pathMainPtr->enableSpdif = 0;
                            }
                        }
                    }

                    /* re-enable previously stopped SPDIF */
                    err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                                  FAPEX_IOCCOM_RESET,
                                                 (FAPEXIOCARG3_T)NULL);
                    if (err_code != FAPEX_OK)
                    {
                        FAPI_SYS_PRINT_ERR("ERROR Reset AUOUT, err_code %d\n", err_code);
                    }


                    openPtr->spdifMode   = mode;
                    pathSpdifPtr->format = AAL_AUDIOFORMAT_NONE;
                    pathSpdifPtr->pid    = 0;
                }
                break;

            default:
                err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
                break;
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetSPDIFMode(mode %i format %i pid %i: %i\n",
                         mode, format, pid, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Set stereo mode.
**
** \param mode      Requested stereo mode.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetStereoMode (AAL_StereoModeT mode)
{
    int32_t err_code = FAPEX_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( mode >= AAL_STEREO_MODE_NUMS )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

         audioDatPtr->stereoMode = mode;

        if ( audioDatPtr->openData.open)
        {
            (void)aalIntSetStereoMode (audioDatPtr, &(audioDatPtr->openData));
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetStereoMode(%i): %i\n",
                         mode, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Switch actual stereo mode.
**        This function switches the actuel stereo mode. That means:
**        - Dual Mono Stereo becomes Dual Mono Swapped
**        - Dual Mono Sawpped becomes Dual Mono  Stereo
**        - Dual Mono Left becomes Dual Mono Right
**        - Dual Mono Right becomes Dual Mono Left
**
** \note This API only affects the analog (I2S) output.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SwitchStereoMode   (void)
{
    int32_t err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);


        if ( audioDatPtr->openData.open )
        {

            switch( audioDatPtr->stereoMode )
            {
                default:
                case AAL_STEREO_MODE_STEREO:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_STEREO_SWAPPED;
                     break;
                case AAL_STEREO_MODE_MONO_LEFT:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_MONO_RIGHT;
                     break;
                case AAL_STEREO_MODE_MONO_RIGHT:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_MONO_LEFT;
                     break;
                case AAL_STEREO_MODE_MONO_LEFT_ONLY:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_MONO_RIGHT_ONLY;
                     break;
                case AAL_STEREO_MODE_MONO_RIGHT_ONLY:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_MONO_LEFT_ONLY;
                     break;
                case AAL_STEREO_MODE_STEREO_SWAPPED:
                     audioDatPtr->stereoMode = AAL_STEREO_MODE_STEREO;
                     break;
            }

            (void)aalIntSetStereoMode (audioDatPtr, &(audioDatPtr->openData));
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SwitchStereoMode: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Feed the input buffer for main audio (I2S) decoding.
**
** \param   handle      Handle returned by AAL_Open.
** \param   bufferPtr   The pointer points to the data buffer feeding to
**                      the audio buffer.
** \param   bytes       The size of the audio data buffer in bytes.
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
#if 0
int32_t AAL_FeedBuffer (FAPI_SYS_HandleT handle, const void* bufferPtr, int32_t bytes)
{
    int32_t     err_code = FAPEX_OK;
    int32_t     retVal;
    FAPEX_TSD_BmWriteParmStrT  bmParam;

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        if ( (openPtr->params.feedMode == AAL_FEEDMODE_BM)
          && (bytes >= 0) )
        {
        	/* fixed me. Please use the mmap function */
            memset(&bmParam, 0, sizeof(FAPEX_TSD_BmWriteParmStrT));

            bmParam.bufferLen = bytes;
            bmParam.bufferPtr =(void*)bufferPtr;
            bmParam.timeout   = 500;

            /* write Audio BM buffer */
            retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                       FAPEX_IOCCOM_TSD_BM_WRITE,
                                       (FAPEXIOCARG3_T)0);

            if(retVal < 0) {
                FAPI_SYS_PRINT_ERR("ERROR Write Audio BM buffer (code %d).\n", retVal);
                err_code = retVal;
             }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }
    }
    return err_code;
}
#else /* add by Jim.Kang to replace FAPEX_IOCCOM_TSD_BM_WRITE with mmap */
int32_t AAL_FeedBuffer (FAPI_SYS_HandleT handle, const void* bufferPtr, int32_t bytes)
{
#if 0
    int32_t err_code = FAPEX_OK;
    uint32_t    timeout  = 500;
    AAL_DataT*  inst = audioDatPtr;
    AAL_OpenT*  openPtr;
    AAL_AudioPathOpenT *pMainPath;


    /*check param */
    if((NULL == bufferPtr) || (0 == bytes))
    {
        return -1;
    }

    /*check param */
    if(NULL == inst)
    {
        return SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        pMainPath = openPtr->path + AAL_PATH_MAIN;
        err_code = audioBmWrite(pMainPath->mp3MapBufAddr,
                                bufferPtr,
                                bytes,
                                FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES,
                                timeout,
                                pMainPath->handleTsd);
        if (err_code != bytes)
        {
            return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    }
    else
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    return FAPEX_OK;
#else
    return aalIntFeedPathBuffer(handle, AAL_PATH_MAIN, bufferPtr, bytes, 500);
#endif
}
#endif

/*!
*******************************************************************************
** \brief   Feed the input buffer for mixer path
**
** \param   handle      Handle returned by AAL_Open.
** \param   mixerID     Mixpath will be feeded.
** \param   bufferPtr   The pointer points to the data buffer feeding to
**                      the audio buffer.
** \param   bytes       The size of the audio data buffer in bytes.
** \param   timeout
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_FeedMixPathBuffer (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID,
                                const void* bufferPtr, int32_t bytes, uint32_t timeout ) {

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntFeedPathBuffer(handle, mixerID, bufferPtr, bytes, timeout);
}

/*!
*******************************************************************************
** \brief   Clear the input buffer for main audio (I2S) decoding.
**
** \param   handle      Handle returned by AAL_Open.
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_ClearBuffer (FAPI_SYS_HandleT handle)
{
#if 0
    int32_t     err_code = FAPEX_OK;

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {

        err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                   FAPEX_IOCCOM_TSD_RESET_BUFFER,
                                   0);
        if(err_code < 0) {
                FAPI_SYS_PRINT_ERR("ERROR Write Audio BM buffer (code %d).\n", err_code);
             }

    }
    else
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    return err_code;
#else
    return aalIntClearPathBuffer(handle, AAL_PATH_MAIN);
#endif
}

/*!
*******************************************************************************
** \brief   Clear the input buffer for mixer path decoding.
**
** \param   handle      Handle returned by AAL_Open.
** \param   mixerID     Mixpath buffer will be clear
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_ClearMixPathBuffer (FAPI_SYS_HandleT handle , AAL_AudioPathT mixerID)
{
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntClearPathBuffer(handle, mixerID);
}

/*!
*****************************************************************************
** \brief Get status of audio decoder.
**
** This function provides the status of the main audio (I2S) decoding.
**
** \param handle    Audio handle returned by AAL_Open.
** \param statusPtr Pointer to the current decoder status.
**
** \return  Possible return codes:
**          - FAPEX_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE Decoding is inactive.
*****************************************************************************
*/
int32_t AAL_GetDecStatus (FAPI_SYS_HandleT handle, AAL_AUDEC_StatusEnumT* statusPtr)
{
#if 0
    int32_t     err_code = FAPEX_OK;
    AAL_OpenT*  openPtr;
    FAPEX_AUDEC_StatusStrT status;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].handleDec <= 0 )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        }
        else if ( statusPtr != NULL )
        {
            err_code = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)(FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleDec,
                                    FAPEX_IOCCOM_AUDEC_GET_STATUS,
                                    (FAPEXIOCARG3_T)&status);
            if(err_code == 0)
                *statusPtr = status.firmStatus;
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }
    return err_code;
#else
    return aalIntGetPathDecStatus(handle, AAL_PATH_MAIN, statusPtr);
#endif
}

/*!
*****************************************************************************
** \brief Get status of audio decoder.
**
** This function provides the status of the main audio (I2S) decoding.
**
** \param handle    Audio handle returned by AAL_Open.
** \param   mixerID Mixpath will process
** \param statusPtr Pointer to the current decoder status.
**
** \return  Possible return codes:
**          - FAPI_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_DATA_NOT_AVAILABLE Decoding is inactive.
*****************************************************************************
*/
int32_t AAL_GetMixPathDecStatus (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,
                                 AAL_AUDEC_StatusEnumT* statusPtr)
{
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntGetPathDecStatus(handle, mixerID, statusPtr);
}

/*!
*****************************************************************************
** \brief Get stream information on currently decoded main audio (I2S) stream.
**
** \param handle    Audio handle returned by AAL_Open.
** \param infoPtr   Pointer to the current decoder info.
**
** \return  Possible return codes:
**          - FAPEX_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE Decoding has not been started.
*****************************************************************************
*/
int32_t AAL_GetStreamInfo (FAPI_SYS_HandleT handle, AAL_AUOUT_StreamInfoT* infoPtr)
{
#if 0
    int32_t                 err_code = FAPEX_OK;
    int32_t                 stream_type;
    AAL_AUOUT_StreamInfoT   streamInfo;
    AAL_OpenT*              openPtr;
    FAPEX_AUOUT_PcmInfoStrT pcmInfoParam;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {

            err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleOut,
                                          FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE,
                                          (FAPEXIOCARG3_T)&stream_type );

            err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleOut,
                                          FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                         (FAPEXIOCARG3_T)&pcmInfoParam );

           streamInfo.sampleBits = pcmInfoParam.sampleBits;
           streamInfo.sampleFreq = pcmInfoParam.sampleFreq;
           streamInfo.channels   = pcmInfoParam.channelCount;
           streamInfo.lfeOn      = 0;
           streamInfo.type       = stream_type;

            if ( infoPtr != NULL )
            {
                *infoPtr = streamInfo;
            }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
#else
    return aalIntGetPathStreamInfo(handle, AAL_PATH_SPDIF, infoPtr);    // use spdifpath to access final AUOUT
#endif
}

/*!
*****************************************************************************
** \brief Get mixer stream information on currently decoded main audio (I2S) stream.
**
** \param handle    Audio handle returned by AAL_Open.
** \param mixerID   Mixer path will be feeded.
** \param infoPtr   Pointer to the current decoder info.
** \return  Possible return codes:
**          - FAPI_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_DATA_NOT_AVAILABLE Decoding has not been started.
*****************************************************************************
*/
int32_t AAL_GetMixPathStreamInfo (FAPI_SYS_HandleT handle,  AAL_AudioPathT mixerID ,
                            AAL_AUOUT_StreamInfoT* infoPtr)
{
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntGetPathStreamInfo(handle, mixerID, infoPtr);
}

/* added for VLC Player */
int32_t AAL_FeedPts (FAPI_SYS_HandleT handle, uint64_t pts)
{
    int32_t     err_code = FAPEX_OK;
    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        if ( (openPtr->params.feedMode == AAL_FEEDMODE_BM) )
        {
            //FAPI_AUDEC_PtsNotifier(pts, openPtr->path[AAL_PATH_MAIN].handleBm);
            err_code = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleDec,
                                          FAPEX_IOCCOM_AUDEC_SET_PTS,
                                          (FAPEXIOCARG3_T)pts);
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }
    }
    return err_code;
}
/*!
*******************************************************************************
** \brief   Function that handles audio-related tasks in application context.
**
** Should be regularly called by application, handles starting audec and HDMI
** audio check
**
** \param   handle           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Task (FAPI_SYS_HandleT handle)
{
    int32_t     err_code = FAPEX_OK;
    AAL_OpenT*  openPtr;
    FAPEX_AUOUT_PcmInfoStrT  streamInfo;

    openPtr = aalCheckHandle(handle, &err_code);
    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {
            if ( (!(openPtr->audecRunning))
                && (openPtr->audecRestart) )
            {
                err_code = FAPEX_AUDEC_Ioctl(openPtr->path[AAL_PATH_MAIN].handleDec,
                                              FAPEX_IOCCOM_START,
                                              0);

                if ( err_code == FAPEX_OK )
                {
                    openPtr->audecRunning = 1;
                    openPtr->audecRestart = 0;
                }
            }

            if ( (openPtr->audecRunning)
                && !(openPtr->decodingStarted))
            {
                err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                                      FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                      (FAPEXIOCARG3_T)&streamInfo);

                if ( (streamInfo.channelCount   != AAL_AUOUT_CHANNEL_UNDEF)
                      && (streamInfo.sampleFreq != AAL_AUOUT_UNDEF_HZ))
                {
                    openPtr->decodingStarted = 1;
                    if ( (openPtr->startNotifyCb != NULL)
                        && !(openPtr->startNotifyCbCalled))
                    {
                        // release mutex here in order to allow the usage of AAL calls from
                        // the callback
                        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
                        openPtr->startNotifyCb(openPtr->startNotifyCbArg);
                        openPtr->startNotifyCbCalled = 1;
                        return err_code;
                    }
                }
            }
        }
        (void)aalStartHdmi(audioDatPtr, openPtr);
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Start/Pause audio decoding.
**
** \param   handle            Handle returned by AAL_Open.
** \param   enterOrExit       The command to play or pause audio decoding.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Pause (FAPI_SYS_HandleT handle, uint8_t enterOrExit)
{
    int32_t             err_code = FAPEX_OK;
    AAL_AudioPathOpenT* pathPtr;
    AAL_OpenT*          openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        /* handle main audio */
        pathPtr = openPtr->path + AAL_PATH_SPDIF;   // with mixer, use spdifpath to access final AUOUT

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {
            if ( enterOrExit ) /* enter pause */
            {

                err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleOut,
                                              FAPEX_IOCCOM_PAUSE,
                                              (FAPEXIOCARG3_T)NULL );
            }
            else /* exit pause */
            {
                err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleOut,
                                              FAPEX_IOCCOM_RESET,
                                              (FAPEXIOCARG3_T)NULL );
            }
        }

        /* handle SPDIF output */
        pathPtr = openPtr->path + AAL_PATH_SPDIF;

        if ( pathPtr->started )
        {
            if ( enterOrExit ) /* enter pause */
            {
            	  err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleOut,
                                              FAPEX_IOCCOM_PAUSE,
                                              (FAPEXIOCARG3_T)NULL );
            }
            else /* exit pause */
            {
            	   err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleOut,
                                                FAPEX_IOCCOM_RESET,
                                               (FAPEXIOCARG3_T)NULL );
            }
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Pause: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Mute/Unmute audio output.
**
** This function mutes or unmutes the audio output.
**
** \param mute    Mute or unmute command.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Mute (uint8_t mute)
{
    int32_t         err_code = FAPEX_OK;
    AAL_AudioPathT  pathCnt;
    FAPEX_AUOUT_MuteParamStrT muteParam;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
              pathCnt++ )
        {
            audioDatPtr->path[pathCnt].muted = mute;

            if ( (audioDatPtr->openData.open)
                    && (audioDatPtr->openData.path[pathCnt].started ||
                    (pathCnt == AAL_PATH_SPDIF && audioDatPtr->openData.path[AAL_PATH_MAIN].started)) )   // spdifpath access final AUOUT
            {
            	  memset(&muteParam, 0, sizeof(muteParam));
            	  muteParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;

                if ( mute )
                {
                    muteParam.muteFlag = 0;
                    err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioDatPtr->openData.path[pathCnt].handleOut,
                                                  FAPEX_IOCCOM_AUOUT_SET_MUTE,
                                                  (FAPEXIOCARG3_T)&muteParam );

                }
                else
                {
                    muteParam.muteFlag = 1;
                    err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioDatPtr->openData.path[pathCnt].handleOut,
                                                 FAPEX_IOCCOM_AUOUT_SET_MUTE,
                                                 (FAPEXIOCARG3_T)&muteParam );
                }
            }
        }
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }
    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Mute(%i): %i\n", mute, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Enable/Disable HDMI output.
**
** This function enables or disables the HDMI output.
**
** \param enable    The command to enable or disable the HDMI output.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_EnableHdmi (uint8_t enable)
{
    int32_t err_code = FAPEX_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( audioDatPtr->hdmiEnabled != enable )
        {
            audioDatPtr->hdmiEnabled = enable;

            if (audioDatPtr->openData.open)
            {
                if ( enable )
                {
                   FAPI_SYS_PRINT_MSG("AAL_EnableHDMI start hdmi\n");
                   err_code = aalStartHdmi (audioDatPtr,
                                              &(audioDatPtr->openData));
                }
                else
                {
                    err_code = aalStopHdmi (&(audioDatPtr->openData));
                }
            }
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_EnableHdmi(%i): %i\n", enable,
                         err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Set audio volume.
**
** This function sets the audio volume on the left and right speaker.
**
** \param lvlLeft    The volume level on the left speaker.
** \param lvlRight   The volume level on the right speaker.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetVolume (uint8_t lvlLeft, uint8_t lvlRight)
{
    int32_t err_code = FAPEX_OK;

   if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {

        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        /* Avoid 0xFF being passed to the audio output. */
        if ( lvlLeft > AAL_VOLUMELEVEL_MAX )
        {
            lvlLeft = AAL_VOLUMELEVEL_MAX;
        }

        if ( lvlRight > AAL_VOLUMELEVEL_MAX )
        {
            lvlRight = AAL_VOLUMELEVEL_MAX;
        }

        audioDatPtr->lvlLeft  = lvlLeft;
        audioDatPtr->lvlRight = lvlRight;

        if ( audioDatPtr->openData.open)
        {
            (void)aalIntSetVolume (audioDatPtr, &(audioDatPtr->openData));
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetVolume(%i, %i): %i\n",
                         lvlLeft, lvlRight, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return buffer size of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The size of the audio input buffer.
**
** \sa      AAL_GetBufferLevel
*******************************************************************************
*/
uint32_t AAL_GetBufferSize (FAPI_SYS_HandleT handle)
{
#if 0
    uint32_t    bufSize;
    AAL_OpenT*  openPtr;
    FAPEX_TSD_ChannelInfoStrT  ch_info;
    int32_t result;

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[AAL_PATH_MAIN].started) )
    {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                   FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                  (FAPEXIOCARG3_T)&ch_info );
        if(result == 0) {
            bufSize = ch_info.bufferSize;
        }
        else
            bufSize = 0;
    }
    else
    {
        bufSize = 0;
    }

    return bufSize;
#else
    return aalIntGetPathBufferSize(handle, AAL_PATH_MAIN);
#endif
}

/*!
*******************************************************************************
** \brief   Return buffer size of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
** \param   mixerID         mixpath
**
** \return  The size of the audio input buffer.
**
** \sa      AAL_GetMixPathBufferLevel
*******************************************************************************
*/
uint32_t AAL_GetMixPathBufferSize (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID)
{
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntGetPathBufferSize(handle, mixerID);
}

/*!
*******************************************************************************
** \brief   Return current filling level of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The filling level of the audio input buffer.
** \sa      AAL_GetBufferSize
*******************************************************************************
*/
uint32_t AAL_GetBufferLevel (FAPI_SYS_HandleT handle)
{
#if 0
    int32_t     result;
    AAL_OpenT*  openPtr;
    uint32_t    level = 0;

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[AAL_PATH_MAIN].started) )
    {
        AAL_Assert (openPtr->path[AAL_PATH_MAIN].handleTsd > 0);

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                  FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                 (FAPEXIOCARG3_T)&level );

    }

    return level;
#else
    return aalIntGetPathBufferLevel(handle, AAL_PATH_MAIN);
#endif
}

/*!
*******************************************************************************
** \brief   Return current filling level of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
** \param   mixerID         mixpath
**
** \return  The filling level of the audio input buffer.
**
** \sa      AAL_GetMixPathBufferSize
*******************************************************************************
*/
uint32_t AAL_GetMixPathBufferLevel (FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID)
{
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntGetPathBufferLevel(handle, mixerID);
}

/*!
*******************************************************************************
** \brief   Return buffer size of SPDIF input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The size of the SPDIF input buffer.
**
** \sa      AAL_GetBufferLevel
*******************************************************************************
*/
uint32_t AAL_GetSPDIFBufferSize (FAPI_SYS_HandleT handle)
{
    uint32_t    bufSize = 0;
    AAL_OpenT*  openPtr;
    FAPEX_TSD_ChannelInfoStrT  ch_info;
    int32_t result;

    openPtr = aalCheckHandle(handle, NULL);

    if ( openPtr != NULL )
    {
        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                bufSize = 0;
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:

                if ( openPtr->path[AAL_PATH_SPDIF].started )
                {

                   result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleTsd,
                                              FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                              (FAPEXIOCARG3_T)&ch_info );
                   if(result == 0) {
                       bufSize = ch_info.bufferSize;
                   }
                   else
                   	   bufSize = 0;
               }
                else if ( (openPtr->path[AAL_PATH_MAIN].started)
                       && (openPtr->path[AAL_PATH_MAIN].enableSpdif) )
                {
                    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                               FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                               (FAPEXIOCARG3_T)&ch_info );
                    if(result == 0) {
                       bufSize = ch_info.bufferSize;
                   }
                   else
                   	   bufSize = 0;
                }
                break;

            case AAL_SPDIFMODE_PCM:
                if ( openPtr->path[AAL_PATH_MAIN].started )
                {
                    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                               FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                               (FAPEXIOCARG3_T)&ch_info );
                    if(result == 0) {
                       bufSize = ch_info.bufferSize;
                   }
                   else
                   	   bufSize = 0;
                }
                break;
        }
    }

    return bufSize;
}

/*!
*******************************************************************************
** \brief   Return current filling level of SPDIF input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The filling level of the SPDIF input buffer.
**
** \sa      AAL_GetBufferSize
*******************************************************************************
*/
uint32_t AAL_GetSPDIFBufferLevel (FAPI_SYS_HandleT handle)
{
    int32_t             result;
    AAL_OpenT*          openPtr;
    uint32_t            level    = 0;

    openPtr = aalCheckHandle(handle, NULL);

    if ( openPtr != NULL )
    {
        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                /* do nothing */
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:

                if ( openPtr->path[AAL_PATH_SPDIF].started )
                {
                	  result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleTsd,
                                              FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                              (FAPEXIOCARG3_T)&level );
                }
                else if ( (openPtr->path[AAL_PATH_MAIN].started)
                       && (openPtr->path[AAL_PATH_MAIN].enableSpdif) )
                {
                    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                              FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                              (FAPEXIOCARG3_T)&level );
                }
                break;

            case AAL_SPDIFMODE_PCM:
                if ( openPtr->path[AAL_PATH_MAIN].started )
                {
                    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleTsd,
                                              FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                              (FAPEXIOCARG3_T)&level );
                }
                break;
        }
    }
    return level;
}

/*!
*******************************************************************************
** \brief   Set the delay in audio.
**
** \param   handle            Handle returned by AAL_Open.
** \param   outputDelayMsec   Requested the delay in milliseconds
**
** \return  Possible return codes:
**          - FAPEX_OK                              Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t AAL_SetOutputDelay (FAPI_SYS_HandleT handle, int32_t outputDelayMsec)
{
    int32_t     err_code = FAPEX_OK;
    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }
    else
    {

        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].handleDec < 0 )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
        }
        else
        {
            err_code = FAPEX_AUDEC_Ioctl((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_MAIN].handleDec,
                                          FAPEX_IOCCOM_AUDEC_SET_OUTPUT_DELAY,
                                          (FAPEXIOCARG3_T)outputDelayMsec);
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get the audio synchronization status.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - #AAL_SYNC_UNDEF  Audio is not in sync.
**          - #AAL_SYNCHRONISING Audio synchronization is in progress.
**          - #AAL_SYNCHRONIZED Audio is in sync.
*******************************************************************************
*/
AAL_SyncStateT AAL_GetSyncState (FAPI_SYS_HandleT handle)
{
    AAL_OpenT*  openPtr;
    uint32_t syncStatus=0;
    int32_t result;


    openPtr = aalCheckHandle(handle, NULL);
    if ( openPtr != NULL )
    {

    	  result = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                                          FAPEX_IOCCOM_AUOUT_GET_SYNCSTATUS,
                                          (FAPEXIOCARG3_T)syncStatus);

        switch( syncStatus )
        {
            case FAPEX_AUOUT_SYNC_SYNCHRONISING: return( AAL_SYNCHRONISING );
            case FAPEX_AUOUT_SYNC_SYNCHRONIZED:  return( AAL_SYNCHRONIZED  );
            default:                            return( AAL_SYNC_UNDEF    );
        }
    }

    return( AAL_SYNC_UNDEF );
}

/*!
*******************************************************************************
** \brief   Set SPDIF channel status bits for copy protection and generation
**          management
**
** \param   cpFlag      Status of SPDIF copyright bit: 1: copyright asserted,
**                      0: copyright not asserted
** \param   lBitFlag    Status of SPDIF "L-Bit": 1: set, 0: cleared
**
** \return  Possible return codes:
**          - FAPEX_OK                              Successful completion.
*******************************************************************************
*/
int32_t AAL_SetSPDIFCopyrightFlags (uint8_t cpFlag, uint8_t lBitFlag)
{
    int32_t         err_code = FAPEX_OK;
    AAL_AudioPathT  pathCnt;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        audioDatPtr->spdifCpFlag   = cpFlag;
        audioDatPtr->spdifLBitFlag = lBitFlag;
        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
              pathCnt++ )
        {
            if ( (audioDatPtr->openData.open)
              && (audioDatPtr->openData.path[pathCnt].started ||
                    (pathCnt == AAL_PATH_SPDIF && audioDatPtr->openData.path[AAL_PATH_MAIN].started)) )   // use spdifpath access final AUOUT
            {
            	 err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)audioDatPtr->openData.path[pathCnt].handleOut,
                                          FAPEX_IOCCOM_AUOUT_SET_SPDIFCOPYPROTECT,
                                          (FAPEXIOCARG3_T)cpFlag);
               if(err_code < 0)
               {
                   FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_SetSPDIFCopyrightFlags(%i): %i\n",
                         cpFlag, err_code);
               }
               else{
                       err_code = FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)audioDatPtr->openData.path[pathCnt].handleOut,
                                                      FAPEX_IOCCOM_AUOUT_SET_SPDIFLBIT,
                                                      lBitFlag);
               }
            }
        }


        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetSPDIFCopyrightFlags(%i, %i): %i\n",
                         cpFlag, lBitFlag, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Start audio decoding
**
** \param handle                Audio handle returned by AAL_Open.
** \param *startNotification    pointer to the start notification
**                              callback function AVINT_LiveTvAudioStartNotify.
*******************************************************************************
*/
int32_t AAL_SetStartNotificationCb (FAPI_SYS_HandleT handle,
                                    void (*startNotifyCb)(void* optData),
                                    void* startNotifyCbArg)
{
    AAL_OpenT*  openPtr;
    int32_t     err_code = FAPI_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        openPtr->startNotifyCb    = startNotifyCb;
        openPtr->startNotifyCbArg = startNotifyCbArg;
        openPtr->startNotifyCbCalled = 0;

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set notification function to be called if a TSD has been started
**          or stopped internally for filtering of audio ES. This can be used
**          for controlling descramblers corresponding to this video stream.
**
** \param   handle          Audio handle returned by AAL_Open.
** \param   descrCtrlCb     Notification function, NULL disables notification.
** \param   descrCtrlCbArg  Optional parameter passed with descrCtrlCb.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t AAL_SetDescramblerControlCb (FAPI_SYS_HandleT handle,
                                     AAL_DescrCtrlCb descrCtrlCb,
                                     void* descrCtrlCbArg)
{
    AAL_OpenT*  openPtr;
    int32_t     err_code = FAPI_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        openPtr->descrCtrlCb    = descrCtrlCb;
        openPtr->descrCtrlCbArg = descrCtrlCbArg;

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}


/* Added for smartgo app */
int32_t AAL_RebootAudec(void)
{
    int32_t err_code = FAPEX_OK;

    if (audioDatPtr == NULL ||audioDatPtr->openData.path[AAL_PATH_MAIN].handleDec == -1)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        FAPI_SYS_PRINT_MSG("[AAL]AAL_RebootAudec error:audec not open\n");
    }
    else
    {
        err_code = FAPEX_AUDEC_Ioctl((FAPEXIOCARG1_T)audioDatPtr->openData.path[AAL_PATH_MAIN].handleDec,
                                    FAPEX_IOCCOM_RESET,
                                    (FAPEXIOCARG3_T)NULL);
        if (err_code != FAPEX_OK)
        {
            FAPI_SYS_PRINT_ERR("ERROR Reset AUDEC, err_code %d\n", err_code);
        }
        else
        {
             FAPI_SYS_PRINT_MSG("RebootAudec success\n");
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief
**
** \param   none
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t AAL_BufMmap (FAPI_SYS_HandleT handle)
{
#if 0
    AAL_DataT*  inst = audioDatPtr;
    AAL_OpenT*  openPtr;
    AAL_AudioPathOpenT*  pMainPath = NULL;
    int32_t     err_code = FAPEX_OK;

    /*check param */
    if(NULL == inst)
    {
        return SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }

    openPtr = aalCheckHandle(handle, &err_code);
    if ( openPtr != NULL )
    {
        pMainPath = openPtr->path + AAL_PATH_MAIN;
        if(NULL != pMainPath->mp3MapBufAddr)
        {
            FAPI_SYS_PRINT_MSG("[AAL] AAL_BufMmap: already map\n");
            return SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED;
        }
        pMainPath->mp3MapBufAddr = (uint8_t*)FAPEX_TSD_Mmap( NULL, FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES,
                                                        (PROT_READ | PROT_WRITE), MAP_SHARED,
                                                        pMainPath->handleTsd, 0 );
        if (pMainPath->mp3MapBufAddr == (void*)-1){
            FAPI_SYS_PRINT_MSG("[AAL] Error: TSD audio bm buffer mmap failed. \n");
            return -1;
        }
    }
    memset(pMainPath->mp3MapBufAddr,0,FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES);

    FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_BufMmap >> mp3MapBufAddr:[0x%x]\n",pMainPath->mp3MapBufAddr);
    return err_code;
#else
    return aalIntPathBufMmap(handle, AAL_PATH_MAIN);
#endif
}

int32_t AAL_MixPathBufMmap(FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID) {
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntPathBufMmap(handle, mixerID);
}

/*!
*******************************************************************************
** \brief
**
** \param   fd_audec        Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t AAL_BufMumap (void)
{
#if 0
    AAL_DataT*  inst = audioDatPtr;
    AAL_AudioPathOpenT *pMainPath = NULL;
    /*check param */
    if(NULL == inst)
    {
        return SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }

    pMainPath = inst->openData.path + AAL_PATH_MAIN;
    if(NULL == pMainPath->mp3MapBufAddr)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    FAPEX_TSD_Munmap(pMainPath->mp3MapBufAddr, FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES);
    pMainPath->mp3MapBufAddr = NULL;
    return 0;
#else
    return aalIntPathBufMunmap(&audioDatPtr->openData, AAL_PATH_MAIN);
#endif
}

int32_t AAL_MixPathBufMumap(AAL_AudioPathT mixerID) {
    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    return aalIntPathBufMunmap(&audioDatPtr->openData, mixerID);
}

/*!
*******************************************************************************
** \brief Set HDMI  audio source..
**
** \param handle    Audio handle returned by AAL_Open.
** \param source    Sets HDMI audio source either to I2S (default) or SPDIF.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetHDMISource (FAPI_SYS_HandleT handle, AAL_HDMISourceT source)
{
    AAL_OpenT*          openPtr;
    int32_t             err_code = FAPI_OK;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( (source != AAL_HDMISOURCE_I2S) && (source != AAL_HDMISOURCE_SPDIF) )
    {
        return(SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER);
    }

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        if ( openPtr->hdmiRunning && (openPtr->hdmiSource != source) )
        {
            (void)aalStopHdmi(openPtr);
            openPtr->hdmiSource = source;
            (void)aalStartHdmi(audioDatPtr, openPtr);
        }
        else
        {
            openPtr->hdmiSource = source;
        }
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }
    else
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }

    return( err_code );
}

//set mixer path's sample rate to target val,
int32_t AAL_SetMixPathFreq ( FAPI_SYS_HandleT handle , AAL_AudioPathT mixerID ,
                                        AAL_AUOUT_SampleFreqEnumT sampleFreq )
{
    int32_t          err_code = FAPI_OK;
    AAL_OpenT*       openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( openPtr != NULL )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[AAL] AAL_SetMixPathFreq 1 .\n" );
        err_code = FAPEX_AUOUT_Ioctl(openPtr->path[mixerID].handleOut,
                FAPEX_IOCCOM_AUOUT_SET_APTSAMPLESRC, sampleFreq);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   set mixer decoder's params.
**
** \param   handle      Audio handle returned by AAL_Open.
** \param   mixerID     corresponding mixer decoder.
** \param   pan         mixer's pan to be set.    //(L)ebH~ffH~0~015H (R) ,(only pan sub-stream.)
** \param   fade        mixer's fade to be set. (only fade main stream.)
**
** \return  error code.
**
** \sa      AAL_Close
*******************************************************************************
*/
int32_t  AAL_SetMixer ( FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID ,uint8_t pan ,uint8_t fade )
{
    FAPEX_AUDEC_MixerParamStrT* mix_params;
    FAPEX_AUDEC_CodecSpecificParamStrT decoderParam;
    int32_t                 err_code = FAPI_OK;
    AAL_OpenT*              openPtr;
    AAL_MixerT*             aal_mixer ;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        aal_mixer = &(openPtr->aalMixerData.aal_mixer[GET_MIXER_INDEX(mixerID)]);

        if (aal_mixer->mixer_used)
        {
            if ( aal_mixer->paramsPtr == NULL )
            {
                aal_mixer->paramsPtr = FAPI_SYS_MALLOC_CACHED( sizeof(FAPEX_AUDEC_MixerParamStrT) );
                if ( aal_mixer->paramsPtr == NULL )
                {
                    err_code = SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY;

                    return err_code;
                }
                aal_mixer->paramsSize = sizeof(FAPEX_AUDEC_MixerParamStrT);
            }

            mix_params = (FAPEX_AUDEC_MixerParamStrT *)aal_mixer->paramsPtr;

            memset (mix_params, 0, sizeof(FAPEX_AUDEC_MixerParamStrT));

            mix_params->decoder = AAL_AUDEC_DECODER_MIXER;
            mix_params->fade = fade;
            mix_params->pan  = pan;
            decoderParam.params = &mix_params;
            decoderParam.length = sizeof(FAPEX_AUDEC_MixerParamStrT);
            err_code = FAPEX_AUDEC_Ioctl( aal_mixer->mixerHandle, FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS,
                                                  &decoderParam);
//            FAPI_SYS_SLEEP(100);

            if(err_code == FAPI_OK)
            {
                FAPI_SYS_PRINT_MSG( "[AAL] mixer set pan = %d ,  fade = %d.\n", pan ,fade );
            }
            else
            {
                FAPI_SYS_PRINT_MSG( "[AAL] mixer set pan fade error: %d.\n", err_code);
            }
        }
    }
        return err_code;
}


/*!
*******************************************************************************
** \brief   set main & mix path's params.
**
** \param   handle      Audio handle returned by AAL_Open.
** \param   mixerID     = 0xff: main path ,other's to corresponding mixer path.
** \param   targetPathVolume  path's params to be set.
**
** \return  error code.
**
** \sa      AAL_Close
*******************************************************************************
*/
int32_t  AAL_SetMixPathVol ( FAPI_SYS_HandleT handle, AAL_AudioPathT mixerID, AAL_MixPathVolumeT* targetPathVolume )
{
    int32_t                 err_code = FAPI_OK;
    AAL_OpenT*              openPtr;
    AAL_AudioPathT          pathCnt;
    AAL_AUOUT_SpeakerEnumT temp_cnt;
    AAL_MixPathVolumeT*     mixPathParamPtr;

    openPtr = aalCheckHandle(handle, &err_code);
    FAPI_SYS_PRINT_DEBUG(2,"[AAL] AAL_SetMixPathVol mixerID= %08x \n",mixerID);

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS)
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( openPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        pathCnt = mixerID;
        FAPI_SYS_PRINT_DEBUG(2,"[AAL] AAL_SetMixPathVol pathCnt= %d \n",pathCnt);
        if (pathCnt < AAL_PATH_NUMS)
        {
            //setting take effect.
            (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

            FAPI_SYS_PRINT_DEBUG(2,"[AAL] AAL_SetMixPathVol vol= %d, \n",targetPathVolume->mixerPathVolume[0]);
            err_code = aalIntSetMixPathVolume ( openPtr , pathCnt , targetPathVolume);
            FAPI_SYS_PRINT_DEBUG(2,"[AAL] aalIntSetMixPathVolume err_code =%d.\n", err_code);

            (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
            if (err_code == FAPEX_OK) {
                mixPathParamPtr = &(audioDatPtr->path[pathCnt].targetPathVolume);
                for (temp_cnt = (AAL_AUOUT_SpeakerEnumT)0; temp_cnt < AAL_AUOUT_SPEAKER_ALL ; temp_cnt++)
                {
                    mixPathParamPtr->mixerPathVolume[temp_cnt] =
                              targetPathVolume->mixerPathVolume[temp_cnt];
                }
                mixPathParamPtr->pathPan = targetPathVolume->pathPan ;
            }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }
    }
    return err_code;
}


/*!
********************************************************************************
********************************************************************************
*/
int32_t  AAL_GetMixPathInfo ( AAL_MixInfoT* mix_info )
{
    int32_t    err_code = FAPI_OK;
    uint32_t    ii;

    (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        if (audioDatPtr->openData.open)
        {
            mix_info->openPtr = (FAPI_SYS_HandleT)(&(audioDatPtr->openData));
            mix_info->mixerOpenNum = audioDatPtr->openData.aalMixerData.mixerUsedNum ;
            mix_info->mixerUsedNum = 0;

            for(ii=0;ii<mix_info->mixerOpenNum ; ii++)
            {
                if(audioDatPtr->openData.path[ii + AAL_PATH_MIX0].started)
                {
                    mix_info->mixerUsedNum++;
                    FAPI_SYS_PRINT_DEBUG(4,"[AAL] AAL_GetMixPathInfo mixerOpenNum=%d .mixer[%d] is started. \n",mix_info->mixerOpenNum,ii);
                }
            }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

    return err_code;
}

/*!
********************************************************************************
//stop all mixer path ,except main path
********************************************************************************
*/
int32_t AAL_StopAllMixPath( FAPI_SYS_HandleT handle )
{
    AAL_OpenT*          openPtr;
    int32_t             err_code   = FAPI_OK;
    int32_t             pathId;

    openPtr = aalCheckHandle(handle, &err_code);

    for (pathId = AAL_PATH_MIX0 ; pathId < AAL_PATH_NUMS; pathId++)
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        aalRemoveMixPathAptsCb(openPtr->path[pathId].handleOut);
        err_code = aalIntStop (openPtr, pathId);

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

        if (err_code != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_stopAllMixPath [%d] error: %d.\n",pathId, err_code);
        }
    }
    return err_code;
}

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*
*******************************************************************************
** \brief   Internal function, checks validity of AAL handle.
**
** \param   handle           Pointer to be checked.
** \param   errCodePtr       Possible error codes:
**          - SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED  AAL is not initialized.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE   Handle is a NULL pointer.
            - SUBSYS_AUDIO_AAL_ERR_NOT_OPEN         AAL is not open.
**
** \return  Possible return codes:
            - Driver handle (may be NULL)
**
*******************************************************************************
*/
static AAL_OpenT* aalCheckHandle (FAPI_SYS_HandleT handle, int32_t* errCodePtr)
{
    AAL_OpenT*  openPtr  = (AAL_OpenT*)handle;
    int32_t     err_code = FAPEX_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( (openPtr == NULL)
           || (openPtr != &(audioDatPtr->openData)) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }
    else if ( !(openPtr->open) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    if ( err_code != FAPEX_OK )
    {
        openPtr = NULL;
    }

    return openPtr;
}

/*
*******************************************************************************
** \brief Internal function, starts mixer path's decoding and output.
**
** This function starts the audio decoding process for the given PID in Mixer path.
**
** \param openPtr   Handle returned by AAL_Open.
** \param mixerID   Mixer path will be used.
** \param mixerPathParams    mixpath parameter.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
static int32_t aalIntStartMixPath (AAL_DataT* inst, AAL_OpenT* openPtr, AAL_AudioPathT mixerID,
                                 AAL_MixPathPramsT*  mixerPathParams)
{
    int32_t                 err_code        = FAPEX_OK;
    AAL_AUDEC_DecoderEnumT decoder         = AAL_AUDEC_DECODER_UNDEF;
    AAL_AudioPathOpenT*     pathPtr         = openPtr->path + mixerID;
    AAL_AUOUT_StreamInfoT mainPathStreamInfo;

    if ( mixerID  < AAL_PATH_MIX0 || mixerID >= AAL_PATH_NUMS) {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    do
    {
        /* already started? */
        if ( pathPtr->started )
        {
            /* do not return an error code */
            FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalIntStartMixPath path already started. \n" );
            break;
        }

        pathPtr->enableSpdif = 0;

        //can just use default addAptsCb.
        if ((err_code = AAL_GetStreamInfo(openPtr, &mainPathStreamInfo)) != FAPEX_OK) {
            mainPathStreamInfo.sampleFreq = mixerPathParams->sampleFreq;   // use input as output sample frequence
        }
        // mixerPathParams->sampleFreq is path input audio data sampel frequence,
        // handleOut need output sample frequence same as mainPathStreamInfo.sampleFreq
        aalAddMixPathAptsCb(pathPtr->handleOut, mixerPathParams ,mainPathStreamInfo.sampleFreq);
        // volume, must after APT have been added
        memcpy(&inst->path[mixerID].targetPathVolume, &mixerPathParams->targetPathVolume, sizeof(AAL_MixPathVolumeT));
        err_code = aalIntSetMixPathVolume( openPtr, mixerID,  &(inst->path[mixerID].targetPathVolume) );

        /* open decoder */
        pathPtr->handleDec = aalOpenDecoder ( inst,
                                                  mixerPathParams->format,
                                                  GET_MIXPATH_AUDEC_DEV_ID(mixerID),
                                                  GET_MIXPATH_TSD0_DEV_ID(mixerID),
                                                  GET_MIXER_INPUT_DEV_ID_SUB(mixerID),
                                                  &decoder,
                                                  &err_code);

        if ( pathPtr->handleDec < 0 )
        {
            FAPI_SYS_PRINT_MSG("[AAL]aalOpenDecoder error(%d)\n",err_code);
            break;
        }

        err_code = aalSetDecoderParams(decoder,
                                        pathPtr->handleDec,
                                        0,
                                        openPtr->params.suppressErrorRecognition,
                                        mixerPathParams->format,
                                        mixerPathParams->sampleFreq);   // pcm decoder need audio data sample frequence
        if  ( err_code != FAPI_OK )
        {
            break;
        }

        /* In streaming mode, AUDEC driver should starts manually by
         * AAL_StartAudecStreamingMedia() after certain level of data is
         * feed into buffer. */
        if ( openPtr->params.feedMode != AAL_FEEDMODE_BM_STREAMMEDIA )
        {
             FAPEX_TSD_PidParamStrT  pidParam;

             memset((void*)&pidParam, 0, sizeof(pidParam));
            /* set Audio PES PID */
            pidParam.pid   = mixerPathParams->pid;
            pidParam.flags = 0;
            err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleTsd,
                                       FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                                      (FAPEXIOCARG3_T)&pidParam );
            if(err_code < 0)
            {
                FAPI_SYS_PRINT_ERR("ERROR Set Audio Pid (code %d).\n", err_code);
                break;
            }

            /* start TSD channel */
           err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleTsd,
                                       FAPEX_IOCCOM_START,
                                       (FAPEXIOCARG3_T)NULL );
           if(err_code < 0)
           {
                FAPI_SYS_PRINT_ERR("ERROR Start Audio TSD channel (code %d).\n", err_code);
                break;
           }

        }

        err_code = FAPEX_AUDEC_Ioctl ((FAPEXIOCARG1_T)pathPtr->handleDec,
                                        FAPEX_IOCCOM_START,
                                       (FAPEXIOCARG3_T)NULL );

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Decoder start with status %d.\n",
                             err_code);

        if ( err_code != FAPEX_OK )
        {
            if ( err_code == FAPEX_AUDEC_ERR_BAD_PARAMETER )
            {
                openPtr->audecRestart = 1;

                err_code = FAPEX_OK;
            }
        }

        pathPtr->format  = mixerPathParams->format;
        pathPtr->decoder = decoder;
        pathPtr->pid     = mixerPathParams->pid;
        pathPtr->started = 1;

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            /* Notify for optional setup of descramblers */
            if ( openPtr->descrCtrlCb != NULL )
            {
                openPtr->descrCtrlCb (1, pathPtr->handleTsd, mixerPathParams->pid,
                                      openPtr->descrCtrlCbArg);
            }
        }
    }
    while ( 0 );

    if( err_code != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[AAL] mixer FAPI_AUDEC_Stop() %d.\n", err_code);
        (void)aalIntStopModules (openPtr, mixerID );

    }

    return err_code;
}



/*
*******************************************************************************
** \brief Internal function, starts audio decoding and output.
**
** This function starts the audio decoding process for the given PID.
**
** \param openPtr   Handle returned by AAL_Open.
** \param format    Format of given audio stream.
** \param pid       Audio PID to decode.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
*******************************************************************************
*/
static int32_t aalIntStart (AAL_DataT* inst, AAL_OpenT* openPtr,
                             AAL_AudioFormatT format, uint32_t pid)
{
    int32_t                    err_code        = FAPEX_OK;
    AAL_AUDEC_DecoderEnumT     decoder         = AAL_AUDEC_DECODER_UNDEF;
    AAL_AudioPathOpenT*        pathMainPtr     = openPtr->path + AAL_PATH_MAIN;
    AAL_AudioPathOpenT*        pathSpdifPtr    = openPtr->path + AAL_PATH_SPDIF;
    uint8_t                    enableSpdif     = 0;
    uint8_t                    ac3ToSpdif      = 0;
    AAL_AudioFormatT           currSpdifFormat = AAL_AUDIOFORMAT_NONE;
    uint32_t                   currSpdifPid    = 0;
    FAPEX_AUOUT_MuteParamStrT  muteParam;
    AAL_AudioPathT          pathCnt;
    AAL_AUOUT_SpeakerEnumT speaker_id;

    do
    {
        /* already started? */
        if ( pathMainPtr->started )
        {
            /* do not return an error code */
            break;
        }

        openPtr->audecRunning       = 0;
        openPtr->decodingStarted    = 0;
        openPtr->audecRestart       = 0;
        //openPtr->hdmiRunning             = 0;

        FAPI_SYS_PRINT_DEBUG(1, "spdif mode=%d\n",openPtr->spdifMode);
        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                enableSpdif = 0;
                ac3ToSpdif  = 0;
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:
                if ( pathSpdifPtr->pid == pid )
                {
                    /* Stop an independently running SPDIF passthrough decoder.*/
                    if ( pathSpdifPtr->started )
                    {
                        currSpdifFormat = pathSpdifPtr->format;
                        currSpdifPid    = pathSpdifPtr->pid;

                        (void)aalIntStop (openPtr, AAL_PATH_SPDIF);
                    }

                    enableSpdif = 1;

                    if ( (format == AAL_AUDIOFORMAT_AC3)
                      || (format == AAL_AUDIOFORMAT_DDP) )
                    {
                        ac3ToSpdif = 1;
                    }
                    else
                    {
                        /* not a format compliant for passthrough, use PCM
                           mode as fallback. */
                        ac3ToSpdif = 0;
                    }
                }
                else
                {
                    enableSpdif = 0;
                    ac3ToSpdif  = 0;
                }
                break;

            case AAL_SPDIFMODE_PCM:
                enableSpdif = 1;
                ac3ToSpdif  = 0;
                break;
        }

        /* ----- open modules ---------------------------------------------- */

        if ( pathMainPtr->handleTsd < 0 )
        {
            err_code = pathMainPtr->handleTsd;
            FAPI_SYS_PRINT_MSG("[AAL]aalIntStart tsd channel don't open\n");
            break;
        }

        #if 0
        memset(&auout_open_params, 0, sizeof(auout_open_params));
        if(enableSpdif)
            auout_open_params.enableFlag = FAPEX_AUOUT_ENABLE_SPDIF;
        if(openPtr->params.numSpeakers > 0 )
            auout_open_params.enableFlag |= FAPEX_AUOUT_ENABLE_I2S;
        auout_open_params.enableFlag |= FAPEX_AUOUT_ENABLE_HDMI;

        for ( cnt = 0; cnt < openPtr->params.numSpeakers; cnt++ )
        {
            /* NOTE: Security check: should never occur! speakerArr has
                     (AAL_AUOUT_SPEAKER_ALL + 1) entries */
            if ( cnt >= AAL_AUOUT_SPEAKER_ALL )
            {
                break;
            }

            if(openPtr->params.speaker[cnt] != AAL_AUOUT_SPEAKER_UNDEF )
                auout_open_params.channelFlag |= (1<<openPtr->params.speaker[cnt]);
        }

        pathMainPtr->handleOut = FAPEX_AUOUT_Open ( DRV_ID_AUOUT_0,
                                                    O_RDWR,
                                                    (FAPEXOPENARG3_T)&auout_open_params );

        if ( pathMainPtr->handleOut < 0 )
        {
            err_code = pathMainPtr->handleOut;
            FAPI_SYS_PRINT_MSG("[AAL]Open auout error(%d)\n",err_code);
            break;
        }

        /*spdif channel and main channel use the same audiou output*/
        pathSpdifPtr->handleOut = pathMainPtr->handleOut;
        #endif

        pathMainPtr->enableSpdif = enableSpdif;
        // aalAddDefaultAptsCb(pathMainPtr->handleOut, format); // AAL_Open have done this

        //init all mixer path's default volume = max. (val of spdif path will not work)
        for (pathCnt = AAL_PATH_MAIN; pathCnt<AAL_PATH_NUMS ; pathCnt++)
        {
            for (speaker_id=(AAL_AUOUT_SpeakerEnumT)0; speaker_id<AAL_AUOUT_SPEAKER_ALL ;speaker_id++)
            {
                inst->path[pathCnt].targetPathVolume.mixerPathVolume[speaker_id] = AAL_VOLUMELEVEL_MAX;
            }
            inst->path[pathCnt].targetPathVolume.pathPan = 0;  //center
        }

        /* open decoder */
        pathMainPtr->handleDec = aalOpenDecoder ( inst,
                                                  format,
                                                  DRV_ID_AUDEC_0,
                                                  DRV_ID_TSD0_AUDIO_CH,
                                                  DRV_ID_AUOUT_MAINPATH,
                                                  &decoder,
                                                  &err_code);

        if ( pathMainPtr->handleDec < 0 )
        {
            FAPI_SYS_PRINT_MSG("[AAL]aalOpenDecoder error(%d)\n",err_code);
            break;
        }

        err_code = aalSetDecoderParams(decoder,
                                       pathMainPtr->handleDec,
                                       ac3ToSpdif,
                                       openPtr->params.suppressErrorRecognition,
                                       pathMainPtr->format,
                                       AAL_AUOUT_UNDEF_HZ);
        if ( err_code < 0 )
        {
            FAPI_SYS_PRINT_ERR("ERROR set decoder params (code %d).\n", err_code);
            break;
        }

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
             FAPEX_TSD_PidParamStrT  pidParam;

             memset((void*)&pidParam, 0, sizeof(pidParam));
            /* set Audio PES PID */
            pidParam.pid   = pid;
            pidParam.flags = 0;
            err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathMainPtr->handleTsd,
                                       FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                                      (FAPEXIOCARG3_T)&pidParam );
            if(err_code < 0)
            {
                FAPI_SYS_PRINT_ERR("ERROR Set Audio Pid (code %d).\n", err_code);
                break;
            }

            /* start TSD channel */
           err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathMainPtr->handleTsd,
                                       FAPEX_IOCCOM_START,
                                       (FAPEXIOCARG3_T)NULL );
           if(err_code < 0)
           {
                FAPI_SYS_PRINT_ERR("ERROR Start Audio TSD channel (code %d).\n", err_code);
                break;
           }

        }

        err_code = FAPEX_AUDEC_Ioctl ((FAPEXIOCARG1_T)pathMainPtr->handleDec,
                                        FAPEX_IOCCOM_START,
                                       (FAPEXIOCARG3_T)NULL );

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Decoder start with status %d.\n",
                             err_code);

        if ( err_code != FAPEX_OK )
        {
            if ( err_code == FAPEX_AUDEC_ERR_BAD_PARAMETER )
            {
                openPtr->audecRestart = 1;

                err_code = FAPEX_OK;
            }
        }
        else
        {
            err_code = FAPI_OK;
            openPtr->audecRunning = 1;
            (void)aalStartHdmi(inst, openPtr);
            FAPI_SYS_PRINT_DEBUG(1, "aalIntStart starthdmi\n");
       }

        pathMainPtr->format  = format;
        pathMainPtr->decoder = decoder;
        pathMainPtr->pid     = pid;
        pathMainPtr->started = 1;

        if ( ac3ToSpdif )
        {
            pathSpdifPtr->format = format;
            pathSpdifPtr->pid    = pid;
        }

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            /* Notify for optional setup of descramblers */
            if ( openPtr->descrCtrlCb != NULL )
            {
                openPtr->descrCtrlCb (1, pathMainPtr->handleTsd, pid,
                                      openPtr->descrCtrlCbArg);
            }
        }
    }
    while ( 0 );

    if ( err_code != FAPEX_OK )
    {
        (void)aalIntStopModules (openPtr, AAL_PATH_MAIN);

        /* Restart independently running SPDIF passthrough decoder if it was
           running before. */
        if ( currSpdifFormat != AAL_AUDIOFORMAT_NONE )
        {
            (void)aalIntStartSPDIF (inst, openPtr, currSpdifFormat,
                                     currSpdifPid);
        }
    }

    return err_code;
}

static int32_t aalIntStartSPDIF (AAL_DataT* inst, AAL_OpenT* openPtr,
                                  AAL_AudioFormatT format, uint32_t pid)
{
    FAPEX_AUDEC_OpenParamStrT    audec_open_params;
    FAPEX_AUDEC_OutputParamStrT  audecOutParam;
    int32_t                      err_code     = FAPEX_OK;
    AAL_AudioPathOpenT*          pathSpdifPtr = openPtr->path + AAL_PATH_SPDIF;
    FAPEX_AUOUT_MuteParamStrT    muteParam;


    FAPI_SYS_PRINT_DEBUG(1, "[aalIntStartSPDIF] format=%d pid=%d\n",format,pid);

    do
    {
        /* already started? */
        if ( pathSpdifPtr->started )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            break;
        }
        else if ( openPtr->spdifMode != AAL_SPDIFMODE_PASSTHROUGH )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            break;
        }
        else if ( format != AAL_AUDIOFORMAT_AC3 )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
            break;
        }

        if ( pathSpdifPtr->handleOut < 0 )
        {
            FAPI_SYS_PRINT_MSG("[AAL]aalIntStartSPDIF spdif audio out not open\n");
            break;
        }

        FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                          FAPEX_IOCCOM_AUOUT_SET_SPDIFCOPYPROTECT,
                                          (FAPEXIOCARG3_T)inst->spdifCpFlag);

        FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                          FAPEX_IOCCOM_AUOUT_SET_SPDIFCOPYPROTECT,
                                          (FAPEXIOCARG3_T)inst->spdifLBitFlag);

        pathSpdifPtr->enableSpdif = 1;

        /* Lock PCM output on SPDIF */
        err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                                      FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK,
                                      (FAPEXIOCARG3_T)AAL_SPDIFPCM_LOCK);
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_MSG("[AAL]aalIntStartSPDIF set spdif pcm lock error=%d\n",err_code);
            break;
        }

        /* set current volume and mute settings */
        if ( inst->path[AAL_PATH_SPDIF].muted )
        {
            memset((void*)&muteParam , 0, sizeof(muteParam));
        	muteParam.muteFlag = 0;
        	muteParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
        	FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)pathSpdifPtr->handleOut,
                               FAPEX_IOCCOM_AUOUT_SET_MUTE,
                               (FAPEXIOCARG3_T)&muteParam);
        }

        /* ----- open decoder ---------------------------------------------- */
        memset((void*)&audec_open_params, 0, sizeof(audec_open_params));
        audec_open_params.inputId   = DRV_ID_TSD0_AUDIO_CH1;
        audec_open_params.codecType = AAL_AUDEC_DECODER_AC3SPDIF;
        pathSpdifPtr->handleDec = FAPEX_AUDEC_Open (DRV_ID_AUDEC_1,
                                                    O_RDWR,
                                                    &audec_open_params);

        if ( pathSpdifPtr->handleDec < 0 )
        {
            err_code = pathSpdifPtr->handleDec;
            FAPI_SYS_PRINT_MSG("[AAL]aalIntStartSPDIF open decoder error(%d)\n",err_code);
            break;
        }

        /* set Audio Decoder Output parameter */
        memset(&audecOutParam, 0, sizeof(audecOutParam));
        audecOutParam.outputId   = DRV_ID_AUOUT_FINAL;
        audecOutParam.sampleBits = 16;
        err_code = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)pathSpdifPtr->handleDec,
                                      FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                      (FAPEXIOCARG3_T)&audecOutParam);
        if(err_code != 0 )
        {
            FAPI_SYS_PRINT_MSG("[AAL]aalIntStartSPDIF set output error(%d)\n",err_code);
            break;
        }

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            FAPEX_TSD_PidParamStrT  pidParam;

             memset((void*)&pidParam, 0, sizeof(pidParam));
            /* set Audio PES PID */
            pidParam.pid   = pid;
            pidParam.flags = FAPEX_TSD_FLTFLAGS_PCR;
            err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathSpdifPtr->handleTsd,
                                       FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                                      (FAPEXIOCARG3_T)&pidParam );
            if(err_code < 0)
            {
                FAPI_SYS_PRINT_ERR("[AAl]aalIntStartSPDIF ERROR Set Audio Pid (code %d).\n", err_code);
                break;
            }

            /* start TSD channel */
           err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathSpdifPtr->handleTsd,
                                      FAPEX_IOCCOM_START,
                                      (FAPEXIOCARG3_T)NULL );
           if(err_code < 0)
           {
                FAPI_SYS_PRINT_ERR("ERROR Start Audio TSD channel (code %d).\n", err_code);
                break;
           }

        }

        err_code = FAPEX_AUDEC_Ioctl ((FAPEXIOCARG1_T)pathSpdifPtr->handleDec,
                                        FAPEX_IOCCOM_START,
                                       (FAPEXIOCARG3_T)NULL );

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Decoder start with status %d.\n",
                             err_code);


        if ( err_code != FAPEX_OK )
        {
            break;
        }

        pathSpdifPtr->format  = format;
        pathSpdifPtr->decoder = audec_open_params.codecType;
        pathSpdifPtr->pid     = pid;
        pathSpdifPtr->started = 1;

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            /* Notify for optional setup of descramblers */
            if ( openPtr->descrCtrlCb != NULL )
            {
                openPtr->descrCtrlCb (1, pathSpdifPtr->handleTsd, pid,
                                      openPtr->descrCtrlCbArg);
            }
        }
    }
    while ( 0 );

    if ( err_code != FAPEX_OK )
    {
        (void)aalIntStopModules (openPtr, AAL_PATH_SPDIF);
    }

    return err_code;
}

/*
*******************************************************************************
** \brief   Internal function, stop audio decoding and output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \sa      AAL_Stop
*******************************************************************************
*/
static int32_t aalIntStop (AAL_OpenT* openPtr, AAL_AudioPathT path)
{
    AAL_AudioPathOpenT* pathPtr = openPtr->path + path;

    if ( pathPtr->started )
    {
        (void)aalIntStopModules (openPtr, path);

        pathPtr->started = 0;
        pathPtr->format  = AAL_AUDIOFORMAT_NONE;
        pathPtr->decoder = AAL_AUDEC_DECODER_UNDEF;
        pathPtr->pid     = 0;
    }

    return FAPEX_OK;
}

static int32_t aalIntStopModules (AAL_OpenT* openPtr, AAL_AudioPathT path)
{
    AAL_AudioPathOpenT* pathPtr = openPtr->path + path;
    int32_t result;

    if ( path == AAL_PATH_MAIN )
    {
        /* remove registered APTs */
        if ( openPtr->params.removeAptsCb )
            (void)openPtr->params.removeAptsCb(openPtr->path[AAL_PATH_SPDIF].handleOut);    // use spdifpath access final AUOUT

        (void)aalStopHdmi (openPtr);
    }

    if ( pathPtr->handleDec > 0 )
    {
         result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleDec,
                                      FAPEX_IOCCOM_STOP,
                                     (FAPEXIOCARG3_T)NULL );

         FAPEX_AUDEC_Close(pathPtr->handleDec);

        pathPtr->handleDec = -1;
    }

    if ( pathPtr->handleTsd > 0 )
    {

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)pathPtr->handleTsd,
                                 FAPEX_IOCCOM_STOP,
                                (FAPEXIOCARG3_T)NULL );
        if(result < 0)
        {
            FAPI_SYS_PRINT_ERR("ERROR Stop Audio TSD channel (code %d).\n", result);
        }

        /* Notify for optional stopping of descramblers */
        if ( openPtr->descrCtrlCb != NULL )
        {
            openPtr->descrCtrlCb (0, pathPtr->handleTsd, pathPtr->pid,
                                  openPtr->descrCtrlCbArg);
        }

    }

    if ( path == AAL_PATH_MAIN )
    {
        openPtr->audecRunning       = 0;
        openPtr->decodingStarted    = 0;
    }

    return FAPEX_OK;
}

static int32_t aalIntSetStereoMode (AAL_DataT* inst, AAL_OpenT* openPtr)
{
    FAPEX_AUOUT_DualMonoParamStrT  params;
    int32_t                        handleOut;
    uint8_t                        lvlLeft;
    uint8_t                        lvlRight;

    memset((void*)&params, 0 ,sizeof(params));
    handleOut = openPtr->path[AAL_PATH_SPDIF].handleOut;

    if ( handleOut > 0 )
    {
        switch ( inst->stereoMode )
        {
            default:
            case AAL_STEREO_MODE_STEREO:
            	params.enableFlag = 0; /* disable dualmono */
            	params.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
                FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)handleOut,
                                   FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                   (FAPEXIOCARG3_T)&params);
                break;

            case AAL_STEREO_MODE_MONO_LEFT:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                params.enableFlag = 1; /* disable dualmono */
            	params.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
                FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)handleOut,
                                   FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                   (FAPEXIOCARG3_T)&params);
                break;

            case AAL_STEREO_MODE_MONO_RIGHT:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                params.enableFlag = 1; /* disable dualmono */
            	params.channelFlag = FAPEX_AUOUT_CHANNEL_RIGHT;
                FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)handleOut,
                                   FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                   (FAPEXIOCARG3_T)&params);
                break;

            case AAL_STEREO_MODE_MONO_LEFT_ONLY:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                params.enableFlag = 1; /* disable dualmono */
            	params.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
                FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)handleOut,
                                   FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                   (FAPEXIOCARG3_T)&params);
                lvlRight = inst->lvlRight;
                inst->lvlRight = 0;
                (void) aalIntSetVolume (inst, openPtr);
                inst->lvlRight = lvlRight;
                break;

            case AAL_STEREO_MODE_MONO_RIGHT_ONLY:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                params.enableFlag = 1; /* disable dualmono */
            	params.channelFlag = FAPEX_AUOUT_CHANNEL_RIGHT;
                FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)handleOut,
                                   FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                   (FAPEXIOCARG3_T)&params);
                lvlLeft = inst->lvlLeft;
                inst->lvlLeft=0;
                (void)aalIntSetVolume (inst, openPtr);
                inst->lvlLeft = lvlLeft;
                break;
            case AAL_STEREO_MODE_STEREO_SWAPPED:
                break;

        }
    }

    return FAPEX_OK;
}

static int32_t aalIntSetVolume (AAL_DataT* inst, AAL_OpenT* openPtr)
{
    FAPEX_AUOUT_VolumeParamStrT    volParam;
    int32_t                        handleOut;
    uint8_t                        lvlLeft;
    uint8_t                        lvlRight;
    uint8_t                        cnt;

    memset((void*)&volParam, 0, sizeof(volParam));
    handleOut = openPtr->path[AAL_PATH_SPDIF].handleOut;

    if ( handleOut > 0 )
    {
        lvlLeft  = inst->lvlLeft;
        lvlRight = inst->lvlRight;

        /* ---- set volume for all active speakers ------------------------- */

        for ( cnt = 0; cnt < openPtr->params.numSpeakers; cnt++ )
        {
            switch ( openPtr->params.speaker[cnt] )
            {
                case AAL_AUOUT_SPEAKER_LEFT:
                case AAL_AUOUT_SPEAKER_SUR_LEFT:
                case AAL_AUOUT_SPEAKER_BACK_LEFT:
#ifdef APT_VOL_CTRL
                   audioAptVolParamsPtr.targetVolume[openPtr->params.speaker[cnt]] = lvlLeft;
#endif
                	volParam.channelFlag =(1<< openPtr->params.speaker[cnt]);
                	volParam.volume = lvlLeft;

                    FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)handleOut,
                                        FAPEX_IOCCOM_AUOUT_SET_VOLUME,
                                        (FAPEXIOCARG3_T)&volParam);
                    break;

                case AAL_AUOUT_SPEAKER_RIGHT:
                case AAL_AUOUT_SPEAKER_SUR_RIGHT:
                case AAL_AUOUT_SPEAKER_BACK_RIGHT:
#ifdef APT_VOL_CTRL
                    audioAptVolParamsPtr.targetVolume[openPtr->params.speaker[cnt]] = lvlRight;
#endif
                    volParam.channelFlag = (1<<openPtr->params.speaker[cnt]);
                	volParam.volume = lvlRight;

                    FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)handleOut,
                                        FAPEX_IOCCOM_AUOUT_SET_VOLUME,
                                        (FAPEXIOCARG3_T)&volParam);
                    break;

                case AAL_AUOUT_SPEAKER_CENTER:
                case AAL_AUOUT_SPEAKER_LEF:

                    /* NOTE: For speakers not assigned to left or right side,
                             an average volume level is set. */

#ifdef APT_VOL_CTRL
                    audioAptVolParamsPtr.targetVolume[openPtr->params.speaker[cnt]] =
                                                                          (lvlLeft + lvlRight) / 2;
#endif
                    volParam.channelFlag = (1<<openPtr->params.speaker[cnt]);
                    volParam.volume = ((uint32_t)lvlLeft + (uint32_t)lvlRight) / 2;
                    FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)handleOut,
                                        FAPEX_IOCCOM_AUOUT_SET_VOLUME,
                                        (FAPEXIOCARG3_T)&volParam);
                    break;

                default:
                    /* do nothing */
                    break;
            }
        }
#ifdef APT_VOL_CTRL
       FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)handleOut,
                          FAPEX_IOCCOM_AUOUT_SET_APTVOLPAR,
                          (FAPEXIOCARG3_T)&audioAptVolParamsPtr);
#endif
    }

    return FAPI_OK;
}

//mixer path's volume which is a SW AUOUT, can only use APT
//Note: spdif path control HW auout, it can't use this function
static int32_t aalIntSetMixPathVolume ( AAL_OpenT* openPtr , AAL_AudioPathT pathCnt ,
                                           AAL_MixPathVolumeT* targetPathVolume)
{
    int32_t err_code = FAPI_OK;
    FAPEX_AUOUT_AptVolParamsT aptVolParam;

    memset(&aptVolParam, 0, sizeof(FAPEX_AUOUT_AptVolParamsT));
    memcpy(aptVolParam.targetVolume, targetPathVolume->mixerPathVolume, AAL_AUOUT_SPEAKER_ALL);
    err_code = FAPEX_AUOUT_Ioctl (openPtr->path[pathCnt].handleOut,
                FAPEX_IOCCOM_AUOUT_SET_APTVOLPAR,
                (FAPEXIOCARG3_T)&aptVolParam);
    return err_code;
}

/*
*******************************************************************************
** \brief   Internal function, start HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \sa      AAL_Start
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStartHdmi (AAL_DataT* inst, AAL_OpenT* openPtr)
{
    FAPEX_AUOUT_PcmInfoStrT  streamInfo;
    int32_t                  err_code;
    if (inst->hdmiEnabled)
    {
        err_code = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                                      FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                      (FAPEXIOCARG3_T)&streamInfo);

        if ( !(openPtr->hdmiRunning) )
        {
            if ( (streamInfo.channelCount != AAL_AUOUT_CHANNEL_UNDEF)
                  && (streamInfo.sampleFreq != AAL_AUOUT_UNDEF_HZ))
            {
                openPtr->hdmiStreamInfo.sampleFreq = streamInfo.sampleFreq;
                openPtr->hdmiStreamInfo.sampleBits = streamInfo.sampleBits;
                openPtr->hdmiStreamInfo.channels   = streamInfo.channelCount;
                err_code = FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                                               FAPEX_IOCCOM_AUOUT_START_HDMI,
                                               (FAPEXIOCARG3_T)openPtr->hdmiSource);
                if ( err_code == FAPEX_OK )
                {
                    openPtr->hdmiRunning = 1;
                }

                FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStartHdmi: start hdmi %i\n", err_code);
            }
        }
        else
        {
            if ( ((streamInfo.channelCount != openPtr->hdmiStreamInfo.channels)
                   || (streamInfo.sampleFreq != openPtr->hdmiStreamInfo.sampleFreq))
                   && (streamInfo.channelCount != AAL_AUOUT_CHANNEL_UNDEF)
                   && (streamInfo.sampleFreq != AAL_AUOUT_UNDEF_HZ))
            {
                openPtr->hdmiStreamInfo.sampleFreq = streamInfo.sampleFreq;
                openPtr->hdmiStreamInfo.sampleBits = streamInfo.sampleBits;
                openPtr->hdmiStreamInfo.channels   = streamInfo.channelCount;
                err_code = FAPEX_AUOUT_Ioctl ((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                                               FAPEX_IOCCOM_AUOUT_START_HDMI,
                                               (FAPEXIOCARG3_T)openPtr->hdmiSource);
                if ( err_code == FAPEX_OK )
                {
                    openPtr->hdmiRunning = 1;
                }

                FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStartHdmi: %i\n", err_code);

            }
        }
    }
    return FAPEX_OK;
}

/*
*******************************************************************************
** \brief   Internal function, stop HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPEX_OK  Successful completion.
**
** \sa      AAL_Pause
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStopHdmi (AAL_OpenT* openPtr)
{
    if ( openPtr->hdmiRunning )
    {
        FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)openPtr->path[AAL_PATH_SPDIF].handleOut,
                           FAPEX_IOCCOM_AUOUT_STOP_HDMI,
                           (FAPEXIOCARG3_T)NULL);

        openPtr->hdmiRunning = 0;

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_AUOUT_StopHdmi.\n");
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStopHdmi.\n");

    return FAPEX_OK;
}

static int32_t aalOpenTsd (AAL_DataT* inst,
                           AAL_AudioPathT path,
                           FAPI_SYS_HandleT handleBm,
                           int32_t* errCodePtr)
{
    FAPEX_TSD_OpenSubIdParamStrT    tsd_open_params;
    int32_t                 handleTsd = -1;
    FAPEX_TSD_PesParamStrT       pesParam;
    int32_t                 err_code = FAPEX_OK;
   FAPI_SYS_PRINT_MSG("aalOpenTsd:path %d\n",path);
    memset (&tsd_open_params, 0, sizeof(tsd_open_params));
    do
    {
       if(path == AAL_PATH_MAIN)
       {
            tsd_open_params.channelNo = AUDIO_TSD_CHANNEL;
            tsd_open_params.sub_id    = DRV_ID_TSD0_AUDIO_CH;
        }
        else if(path == AAL_PATH_SPDIF)
        {
             tsd_open_params.channelNo = AUDIO_TSD_CHANNEL1;
             tsd_open_params.sub_id    = DRV_ID_TSD0_AUDIO_CH1;
        }
        else
        {
            tsd_open_params.channelNo = TSD_CH_MIXPATH_AUTO;
            tsd_open_params.sub_id = DRV_ID_TSD0_MIXPATH_BASE + GET_MIXER_INDEX(path);
        }

        err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                                 FAPEX_IOCCOM_TSD_OPEN_SUBID,
                                (FAPEXIOCARG3_T)&tsd_open_params);


        if ( err_code < 0 )
        {
            break;
        }
         handleTsd = tsd_open_params.sub_id;

        /* set Audio PES paser */

        if(path == AAL_PATH_MAIN)
        {
            pesParam.bufferSize  = FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES;
            pesParam.streamType       = FAPEX_TSD_STREAMTYPE_AUDIO;
        }
        else if (path == AAL_PATH_SPDIF) {
            pesParam.bufferSize = FAPEX_SYS_MMAP_BM_AUDIO2_STREAM_BYTES;
            pesParam.streamType       = FAPEX_TSD_STREAMTYPE_AUDIO_SPDIF;
        }
        else {  // mixpath
            // FIXME: mixpath bm buffer size maybe different
            pesParam.bufferSize = FAPEX_SYS_MMAP_BM_MIX00_STREAM_BYTES;
            pesParam.streamType = FAPEX_TSD_STREAMTYPE_AUDIO_MIX0 + GET_MIXER_INDEX(path);
        }

        pesParam.pid   = 0x1FFF;  /* dummy */
        pesParam.flags = FAPEX_TSD_FLTFLAGS_RM_PES_HEADER | FAPEX_TSD_FLTFLAGS_PTS |
                         FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
        pesParam.bufferThresholdH = pesParam.bufferSize;
        pesParam.bufferThresholdL = 0;
        err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)handleTsd,
                                  FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                                 (FAPEXIOCARG3_T)&pesParam );

        if(err_code < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Set Audio PES filter (code %d).\n", err_code);
            break;
        }
    }
    while ( 0 );

    if ( err_code != FAPEX_OK )
    {
        if(handleTsd > 0 )
        {
            FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)handleTsd,
                             FAPEX_IOCCOM_CLOSE_SUBID,
                             (FAPEXIOCARG3_T)NULL);
        }
        handleTsd = -1;
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return handleTsd;
}

static int32_t aalOpenDecoder (AAL_DataT* inst,
                                    AAL_AudioFormatT format,
                                    int32_t audecDevId,
                                    int32_t handleTsd,
                                    int32_t handleOut,
                                    AAL_AUDEC_DecoderEnumT* decoderPtr,
                                    int32_t* errCodePtr)
{
    FAPEX_AUDEC_OpenParamStrT    audec_open_params;
    FAPEX_AUDEC_OutputParamStrT  audecOutParam;
    int32_t                      handleDec  = -1;
    AAL_AUDEC_DecoderEnumT       target_decoder = AAL_AUDEC_DECODER_UNDEF;
    int32_t                      err_code   = FAPEX_OK;

    if ( (format > AAL_AUDIOFORMAT_NONE) && (format < AAL_AUDIOFORMAT_NUMS) )
    {
        target_decoder = inst->targetDecoder[format];
    }

    if ( target_decoder == AAL_AUDEC_DECODER_UNDEF )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        memset (&audec_open_params, 0, sizeof(audec_open_params));

       FAPI_SYS_PRINT_MSG("opendecoder type:%d\n",target_decoder);

       audec_open_params.inputId   = handleTsd;
       audec_open_params.codecType = target_decoder;
       handleDec = FAPEX_AUDEC_Open( audecDevId,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audec_open_params );
       if(handleDec < 0)
       {
         FAPI_SYS_PRINT_MSG("[AAL]aalOpenDecoder error %d\n",handleDec);
       }
       if(handleDec > 0) /* set the output */
       {
       	    memset(&audecOutParam, 0, sizeof(audecOutParam));
       	    audecOutParam.outputId = handleOut;
       	    audecOutParam.sampleBits = 16;
           err_code = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)handleDec,
                                FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                (FAPEXIOCARG3_T)&audecOutParam);
           if(err_code != 0 )
           {
               FAPI_SYS_PRINT_MSG("[AAL]aalOpenDecoder Set output error(%d)\n",err_code);
               FAPEX_AUDEC_Close(handleDec);
               handleDec = -1;
           }
           else
           {
               if ( decoderPtr != NULL )
               {
                   *decoderPtr = inst->targetDecoder[format];
               }
           }
       }
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return handleDec;
}

int32_t aalAddDefaultAptsCb( int32_t audioOutHandle, AAL_AudioFormatT format )
{
    int32_t retVal = FAPI_OK;

#ifdef APT_VOL_CTRL
    memset( &audioAptVolParamsPtr, 0, sizeof(AAL_AUOUT_AptVolParamsT));

    retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_ADD_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_VOLUME_CONTROL);

    if ( retVal < 0 )
    {
        return( retVal );
    }
#endif

#ifdef APT_11DB_ATTENUATE
    // NOTE: The 11dB attenuator is applied only to SPDIF. This is valid for the
    //       default parameters:
    //       -  audioDolbyCompMode = AAL_DOLBY_RF_MODE
    //       -  heaacParams.drcTargetRefLevel = 80

    switch (format)
    {
        case AAL_AUDIOFORMAT_MPEG:
        case AAL_AUDIOFORMAT_AAC:
        case AAL_AUDIOFORMAT_HEAAC:
        case AAL_AUDIOFORMAT_AC3:
        case AAL_AUDIOFORMAT_DDP:
        case AAL_AUDIOFORMAT_DD:
        {

            retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                        FAPEX_IOCCOM_AUOUT_ADD_APT,
                                        (FAPEXIOCARG3_T)AAL_AUOUT_APT_11DB_ATTENUATION);
            if ( retVal < 0 )
            {
                return( retVal );
            }
            break;
        }

        default: break;
    }
#endif

#ifdef APT_SIMPLE_SRC
        retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                    FAPEX_IOCCOM_AUOUT_ADD_APT,
                                    (FAPEXIOCARG3_T)AAL_AUOUT_APT_SIMPLE_SRC);
        if ( retVal < 0  )
        {
            return( retVal );
        }
#endif


    return( FAPI_OK );

}

int32_t  aalRemoveDefaultAptsCb( int32_t audioOutHandle )
{
    int32_t retVal = FAPI_OK;

#ifdef APT_VOL_CTRL
    retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_REMOVE_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_VOLUME_CONTROL);

    if ( retVal < 0 )
    {
        return( retVal );
    }


#endif

#ifdef APT_11DB_ATTENUATE
    retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_REMOVE_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_11DB_ATTENUATION);
    if ( retVal < 0 )
    {
        return( retVal );
    }

#endif

#ifdef APT_SIMPLE_SRC
    retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_REMOVE_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_SIMPLE_SRC);
    if ( retVal < 0  )
    {
        return( retVal );
    }

#endif
    return( FAPI_OK );

}

#ifndef _LINUX_FAPI

#if defined(__GNUC__)

static void aalSwapLongwords (uint8_t* bufPtr)
{
    uint32_t    cnt;
    uint8_t     tmpByte;

    for ( cnt = 0; cnt < 4; cnt++ )
    {
        tmpByte = bufPtr[cnt];

        bufPtr[cnt]     = bufPtr[cnt + 4];
        bufPtr[cnt + 4] = tmpByte;
    }
}

#endif /* defined(__GNUC__) */

#endif/*_LINUX_FAPI*/

static int32_t audioBmWrite (uint8_t *bmMapAddr, const void* writeBuffer, uint32_t length,
                             uint32_t size, uint32_t timeout, int32_t handleTsd)
{
    int32_t                 ret = 0;
    int32_t                 time_difference    = 0;
    uint32_t                bytes_written      = 0;
    uint32_t                write_ptr_position = 0;
    uint32_t                buffer_addr        = 0;
    uint64_t                start_time         = 0;
    uint64_t                stop_time          = 0;
    volatile uint8_t*       data8_ptr          = NULL;
    volatile uint32_t*      data32_ptr         = NULL;
    FAPEX_TSD_WriteInfoStrT wr_info;
    uint32_t level = 0;

    /* check param */
    if((NULL == bmMapAddr) || (length == 0) || (length > 0x7FFFFFFF))
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;

    /* get the current write pointer position */
    memset(&wr_info, 0, sizeof(FAPEX_TSD_WriteInfoStrT));
    ret = FAPEX_TSD_Ioctl(handleTsd, FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO, (FAPEXIOCARG3_T)&wr_info);
    if(ret < 0)
    {
        FAPI_SYS_PRINT_ERR("[AAL] audioBmWrite >> Get BM buffer write info ERROR(%d).\n", ret);
        return ret;
    }

    write_ptr_position = wr_info.writeOffset;

    /* out of bounds check */
    if(wr_info.writeOffset >= size)
        return SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY;

    /* check for suspend mode */
    if(timeout == FAPEX_SYS_NO_SUSPEND)
    {
        /* not enough data inside the buffer */
        level = 0;
        ret = FAPEX_TSD_Ioctl(handleTsd, FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                                          (FAPEXIOCARG3_T)&level);
        if((size - level) < (length+1))
            return(0);
    }
    else
    {
        /* get time stamp */
        start_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        /* wait for enough bytes to write
           NOTE: We wait for one byte more to avoid read/write pointer getting the
                 same value. Otherwise this sould stop the HWCBs.

        */
        level = 0;
        ret = FAPEX_TSD_Ioctl(handleTsd, FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                                          (FAPEXIOCARG3_T)&level);
        while(((size - level) < (length+1)) &&
              ((timeout == FAPEX_SYS_SUSPEND) || ((uint32_t)time_difference < timeout)))
        {
            stop_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
            time_difference = (int32_t)(stop_time - start_time);
            if(start_time > stop_time)
                time_difference = ((time_difference < 0L) ? -time_difference : time_difference); /* C-L
IB code for labs() */

            level = 0;
            ret = FAPEX_TSD_Ioctl(handleTsd, FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                                              (FAPEXIOCARG3_T)&level);
        }

        /* timeout occurred */
        if((timeout != FAPEX_SYS_SUSPEND) && ((uint32_t)time_difference >= timeout))
            return SUBSYS_AUDIO_AAL_ERR_TIMEOUT;
    }

    /* get address to write to */
    data8_ptr = bmMapAddr;
    buffer_addr = (uint32_t)data8_ptr;
    data8_ptr += write_ptr_position;

    bytes_written = 0;
    /* check alignment */

    if ( !((uint32_t)data8_ptr & 0x3) && !((uint32_t)writeBuffer & 0x3))
    {
        data32_ptr = (volatile uint32_t*)(void*)data8_ptr;

        /* read bytes from buffer (4 bytes at a time) */
        while((bytes_written + AAL_BM_4_BYTES) <= length)
        {
            /* divide amount of bytes by 4 for 32 bit index */
            *data32_ptr++ = ((uint32_t*)writeBuffer)[(bytes_written >> 2)];
            /* increase data8_ptr for later usage */
            data8_ptr += AAL_BM_4_BYTES;
            write_ptr_position += AAL_BM_4_BYTES;
            bytes_written += AAL_BM_4_BYTES;

            /* reset if end of buffer is reached */
            if(write_ptr_position == size)
            {
                write_ptr_position = 0;
                data8_ptr = bmMapAddr;

                /* check alignment */
                if(!((uint32_t)data8_ptr & 0x3))
                    /* is aligned */
                    data32_ptr = (volatile uint32_t*)(void*)data8_ptr;
                else
                    break;
            }
        }
    }

    /* write remaining bytes in buffer (1 byte at a time) */
    for(; bytes_written < length; bytes_written++)
    {
        *data8_ptr++ = ((uint8_t*)writeBuffer)[bytes_written];
        write_ptr_position++;

        /* reset if end of buffer is reached */
        if(write_ptr_position == size)
        {
            write_ptr_position = 0;
            data8_ptr = bmMapAddr;
        }
    }

    ret = FAPEX_TSD_Ioctl(handleTsd, FAPEX_IOCCOM_TSD_SET_BUFFER_WRITEPOINTEROFFSET, (FAPEXIOCARG3_T)write_ptr_position);
    if(ret < 0)
    {
        FAPI_SYS_PRINT_ERR("[AAL] audioBmWrite >> Set BM buffer write position ERROR(%d).\n", ret);
        return ret;
    }
    return((int32_t)bytes_written);
}


/*
*******************************************************************************
** \brief   Internal function, set the decoder parameters.
**
** \param   pathPtr Pointer to the audio path.
** \param   ac3ToSpdif Set if AC3 should output via SPDIF, otherwise 0.
** \param   suppressErrorRecognition Flag to enable/disable stream error recognistion
**                                   by some decoders.
** \param   format  audio format
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          -  otherwise error code
**
** \sa      aalIntStart
*******************************************************************************
*/
static int32_t aalSetDecoderParams( AAL_AUDEC_DecoderEnumT target_decoder,
                                          int32_t handleDec,
                                          uint8_t ac3ToSpdif,
                                          uint8_t suppressErrorRecognition, AAL_AudioFormatT format,
                                          AAL_AUOUT_SampleFreqEnumT sampleFreq) // sampleFreq used only format is AAL_AUDIOFORMAT_PCM
{
    FAPEX_AUDEC_CodecSpecificParamStrT  codecParam;
    FAPEX_AUDEC_MpegParamStrT           mpegParams;
    FAPEX_AUDEC_Ac3ParamStrT            ac3Params;
    FAPEX_AUDEC_DdpdcvParamStrT         ddpdcvParams;
    FAPEX_AUDEC_HeaacParamsT            heaacParams;
    FAPEX_AUDEC_PcmParamStrT            pcmParam;
    uint32_t                            paramsSize = 0;
    int32_t                             err_code   = FAPEX_OK;
    uint8_t*                            paramsPtr  = NULL;
    uint8_t                             swap       = 0;

    switch ( target_decoder )
    {
        case AAL_AUDEC_DECODER_MPEG:
            memset (&mpegParams, 0, sizeof(mpegParams));
            mpegParams.decoder = AAL_AUDEC_DECODER_MPEG;
            mpegParams.muteDuration          = 1200;
            if ( suppressErrorRecognition )
            {
                mpegParams.startInsyncFrameCount = 0;
            }
            else
            {
                mpegParams.startInsyncFrameCount = 10;
            }

            paramsPtr  = (uint8_t*)(&(mpegParams));
            paramsSize = sizeof(mpegParams);
          break;

        case AAL_AUDEC_DECODER_AC3:
            memset (&ac3Params, 0, sizeof(ac3Params));
            ac3Params.decoder       = AAL_AUDEC_DECODER_AC3;
            ac3Params.compMode      = audioDolbyCompMode;
            ac3Params.dynsclh       = 1.0;
            ac3Params.dynscll       = 1.0;
            ac3Params.pcmscl        = 1.0;
            ac3Params.outChannelCfg = -1;
            ac3Params.ac3ToSpdif    = ac3ToSpdif;

            paramsPtr  = (uint8_t*)(&(ac3Params));
            paramsSize = sizeof(ac3Params);
            swap = 1;
            break;

        case AAL_AUDEC_DECODER_DD:
           memset (&ddpdcvParams, 0, sizeof(ddpdcvParams));

            ddpdcvParams.decoder       = AAL_AUDEC_DECODER_DD;
            ddpdcvParams.compMode      = audioDolbyCompMode;
            ddpdcvParams.dynsclh       = 1.0;
            ddpdcvParams.dynscll       = 1.0;
            ddpdcvParams.pcmscl        = 1.0;
            ddpdcvParams.outChannelCfg = -1;
            ddpdcvParams.ddToSpdif     = ac3ToSpdif;

            paramsPtr  = (uint8_t*)(&(ddpdcvParams));
            paramsSize = sizeof(ddpdcvParams);
            swap       = 1;
            break;

        case AAL_AUDEC_DECODER_DDPDCV:
            memset (&ddpdcvParams, 0, sizeof(ddpdcvParams));

            ddpdcvParams.decoder       = AAL_AUDEC_DECODER_DDPDCV;
            ddpdcvParams.compMode      = audioDolbyCompMode;
            ddpdcvParams.dynsclh       = 1.0;
            ddpdcvParams.dynscll       = 1.0;
            ddpdcvParams.pcmscl        = 1.0;
            ddpdcvParams.outChannelCfg = -1;
            ddpdcvParams.ddToSpdif    = ac3ToSpdif;

            paramsPtr  = (uint8_t*)(&(ddpdcvParams));
            paramsSize = sizeof(ddpdcvParams);
            swap       = 1;
            break;

        case AAL_AUDEC_DECODER_HEAAC:
            memset (&heaacParams, 0, sizeof(heaacParams));

            /* NOTE: All values except startInsyncFrameCount are set
                     to default. */

            heaacParams.decoder              = AAL_AUDEC_DECODER_HEAAC;
            heaacParams.extSamplingRate      = -1;
            heaacParams.maxOutSamplingRate   = 96000;
            heaacParams.enableDSSBR          = 0;
            heaacParams.enableDMSBR          = 0;
            heaacParams.sbrMode              = 2;
            heaacParams.enableDOWNMIX        = 1;
            heaacParams.downmixMode          = 0;
            heaacParams.enableDRC            = 1;
            heaacParams.drcTargetRefLevel    = 80;
            heaacParams.fadeOutFrameNum      = 5;
            heaacParams.fadeInFrameNum       = 5;
            heaacParams.muteFrameCount       = 5;

            if ( suppressErrorRecognition )
            {
                heaacParams.startInsyncFrameCount = 0;
            }
            else
            {
                heaacParams.startInsyncFrameCount = 10;
            }
            paramsPtr  = (uint8_t*)(&(heaacParams));
            paramsSize = sizeof(heaacParams);
            break;

        case AAL_AUDEC_DECODER_PCMOUTPUT:
            memset(&pcmParam, 0, sizeof(FAPEX_AUDEC_PcmParamStrT));
            pcmParam.decoder = FAPEX_AUDEC_CODEC_PCMOUTPUT;
            pcmParam.sampleFreq = sampleFreq;
            pcmParam.sampleBits = AAL_AUDEC_SAMPLE_16_BIT;
            pcmParam.channels = AAL_AUOUT_CHANNEL_STEREO;
            pcmParam.channelFormatArr[0] = 0;
            pcmParam.channelFormatArr[1] = 1;
            pcmParam.channelFormatArr[2] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.channelFormatArr[3] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.channelFormatArr[4] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.channelFormatArr[5] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.channelFormatArr[6] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.channelFormatArr[7] = AAL_AUOUT_SPEAKER_UNDEF;
            pcmParam.ifWavFormat = (format == AAL_AUDIOFORMAT_WAV);

            paramsPtr = (uint8_t *) &pcmParam;
            paramsSize = sizeof(FAPEX_AUDEC_PcmParamStrT);

            break;

        default:
            break;
    }

    if ( paramsPtr != NULL )
    {
    #ifndef _LINUX_FAPI
        if ( swap )
        {
        #if defined(__GNUC__)

            /* NOTE: Temporary workaround, ARC/Metaware interprets
                     'double' entries in codec structure different than
                     ARM/GCC (ARM/Realview is OK). */
            AAL_Assert(paramsSize <= AAL_PARAMSBUF_SIZE);

            /*lint -save -e669 -e670 */
            memcpy (inst->paramsBuf, paramsPtr, paramsSize);
            /*lint -restore */

            paramsPtr = inst->paramsBuf;

            /* swap 'double' entries in structure (last three entries,
               8 bytes each) */

            /* pcmscl */
            aalSwapLongwords(paramsPtr + paramsSize - 8);

            /* dynscll */
            aalSwapLongwords(paramsPtr + paramsSize - 16);

            /* dynsclh */
            aalSwapLongwords(paramsPtr + paramsSize - 24);

      #endif /* defined(__GNUC__) */
      }
      #endif
        FAPI_SYS_PRINT_DEBUG(2,"[AAL]Set the codec params \n");
        codecParam.params =(void*)paramsPtr;
        codecParam.length = paramsSize;

        err_code = FAPEX_AUDEC_Ioctl((FAPEXIOCARG1_T)handleDec,
                                     FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS,
                                     (FAPEXIOCARG3_T)&codecParam );
        if(err_code < 0)
        {
            (void)FAPEX_AUDEC_Close(handleDec);
            handleDec = -1;
            FAPI_SYS_PRINT_ERR("ERROR: set decoder parameter setting failed (ret=%d).\n",err_code);
        }

        FAPI_SYS_PRINT_DEBUG(2,"[aalSetDecoderParams]:set decoder parameter:%d\n",err_code);
    }

    return err_code;
}

static int32_t aalOpenMixerInput( AAL_OpenT* openPtr, AAL_AudioPathT path )
{
    FAPEX_AUOUT_OpenParamStrT auout_open_params;
    AAL_MixerT*         aal_mixer ;
    int32_t             err_code = FAPI_OK;

    if ( ( path < AAL_PATH_MIX0 ) ||  ( path >= AAL_PATH_NUMS ) )
    {
        err_code = FAPI_ERR_BAD_PARAMETER;
    }
    else
    {
        /* open audio output */
        aal_mixer = openPtr->aalMixerData.aal_mixer + GET_MIXER_INDEX(path);
        memset(&auout_open_params, 0, sizeof(FAPEX_AUOUT_OpenParamStrT));
        auout_open_params.enableFlag = FAPEX_AUOUT_ENABLE_PCM;
        auout_open_params.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT | FAPEX_AUOUT_CHANNEL_RIGHT;
        // mixer input main
        aal_mixer->auoutMixerInput[0] = FAPEX_AUOUT_Open ( GET_MIXER_INPUT_DEV_ID_MAIN(path),
                                   O_RDWR, (FAPEXOPENARG3_T)&auout_open_params );
        // mixer input sub
        aal_mixer->auoutMixerInput[1] = FAPEX_AUOUT_Open ( GET_MIXER_INPUT_DEV_ID_SUB(path),
                                   O_RDWR, (FAPEXOPENARG3_T)&auout_open_params );

        if ( aal_mixer->auoutMixerInput[0] < 0 || aal_mixer->auoutMixerInput[1] < 0)
        {
            FAPI_SYS_PRINT_DEBUG(2, "aalOpenMixerInput ERROR Open AUOUT driver.\n");
        }
    }

    return( err_code );
}

/*
*****************************************************************************
** \brief I
** \ audioOutHandle : the final auout handle ,will output to HCB BMs.
*****************************************************************************
*/
static int32_t aalOpenMixer( AAL_OpenT* openPtr, FAPI_SYS_HandleT audioOutHandle )
{
    FAPEX_AUDEC_OpenParamStrT mixerOpenParam;
    FAPEX_AUDEC_OutputParamStrT mixerOutputParam;
    AAL_AudioPathT  path = AAL_PATH_MIX0;
    AAL_MixerT*  aal_mixer = NULL;
    AAL_MixerT*  aal_mixer_next = NULL;
    int32_t      ii ;
    int32_t      err_code ;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        memset(&mixerOpenParam, 0, sizeof(FAPEX_AUDEC_OpenParamStrT));
        mixerOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_MIXER;
        for ( ii=0 ; ii<AAL_PATH_NUMS -AAL_PATH_MIX0 ; ii++)
        {
            aal_mixer = &(openPtr->aalMixerData.aal_mixer[GET_MIXER_INDEX(path)]);
//to find one empty mixer path,and open it.
            if ( !(aal_mixer->mixer_used ) )
            {
                FAPI_SYS_PRINT_DEBUG(2, "[AAL]FAPI_AUDEC_OpenMixer  aal_mixer[%d] .\n" ,GET_MIXER_INDEX(path) );

                err_code = aalOpenMixerInput( openPtr , path );

                if ( err_code != FAPI_OK )   //init mixer 2 input auout
                {
                    FAPI_SYS_PRINT_DEBUG(2, "aalOpenMixerInput path %d failed .\n" , path );
                    break;
                }

		    	      if  ( aal_mixer->paramsPtr )
                {
                    FAPI_SYS_FREE( aal_mixer->paramsPtr);
                    aal_mixer->paramsPtr = NULL;
                    aal_mixer->paramsSize = 0;
                }

                mixerOpenParam.inputId = GET_MIXER_INPUT_DEV_ID_MAIN(path);
                mixerOpenParam.subInputDevId = GET_MIXER_INPUT_DEV_ID_SUB(path);
                if ( path != AAL_PATH_MIX0 )
                {  //others output to previous mixer path's input0
                    aal_mixer_next = openPtr->aalMixerData.firstMixerInChain ;
                }
                aal_mixer->mixerHandle = FAPEX_AUDEC_Open(GET_MIXER_AUDEC_DEV_ID(path), O_RDWR, &mixerOpenParam);
                if (aal_mixer->mixerHandle < 0)
                {
                    FAPI_SYS_PRINT_MSG("[AAL]aalOpenDecoder error %d\n",aal_mixer->mixerHandle);
                } else {
                    memset(&mixerOutputParam, 0, sizeof(FAPEX_AUDEC_OutputParamStrT));
                    mixerOutputParam.sampleBits = AAL_AUDEC_SAMPLE_16_BIT;
                    if (path == AAL_PATH_MIX0) {
                        mixerOutputParam.outputId = (int32_t) audioOutHandle; // final output
                    } else {
                        mixerOutputParam.outputId = GET_MIXER_INPUT_DEV_ID_MAIN(path - 1);  // next mixer main input
                    }
                    err_code = FAPEX_AUDEC_Ioctl( aal_mixer->mixerHandle,
                                        FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                        (FAPEXIOCARG3_T)&mixerOutputParam);
                    if(err_code != 0 )
                    {
                        FAPI_SYS_PRINT_MSG("[AAL]aalOpenMixer Set output error(%d)\n",err_code);
                        FAPEX_AUDEC_Close(aal_mixer->mixerHandle);
                        aal_mixer->mixerHandle = -1;
                    }
                }

                break;
            }
            path ++ ;
        }

        if ( err_code != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2, "error  .\n" );
        }
        else
        {
            if ( path == AAL_PATH_MIX0 )
            {
                openPtr->aalMixerData.firstMixerInChain = aal_mixer ;
                openPtr->aalMixerData.lastMixerInChain = aal_mixer ;

                aal_mixer->nextMixer = NULL;
            }
            else
            {
                openPtr->aalMixerData.firstMixerInChain = aal_mixer;

                aal_mixer_next->previousMixer = aal_mixer;

                aal_mixer->nextMixer = aal_mixer_next;  //update the chain
            }
//all new mix will be added to the beginning of the chain,
//and be executed at first.
//this ensure final output is last executed,
//and data will be output with no delay.
            aal_mixer->previousMixer = NULL;

            aal_mixer->mixer_used = 1;
            aal_mixer->mixer_started = 0;

            openPtr->aalMixerData.mixerOpenNum ++;

            FAPI_SYS_PRINT_DEBUG(2, "ok  .\n" );
        }

    }
    return( err_code );
}

/*
*****************************************************************************
** \brief I
** \ audioOutHandle : the final auout handle ,will output to HCB BMs.
*****************************************************************************
*/
static int32_t aalCloseAllMixer(  AAL_OpenT* openPtr )
{
    int32_t      err_code = FAPI_OK;
    AAL_MixerT*  aal_mixer = NULL;
    uint32_t     mixer_Close_Num = 0;
    uint32_t     ii,jj;

    if ( openPtr->aalMixerData.mixerOpenNum == 0 )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalCloseAllMixer : no mixer have opened .\n" );
    }
    else
    {
//  Start from the first mixer in chain.
        aal_mixer = openPtr->aalMixerData.firstMixerInChain ;

        for (ii=0 ; ii < (AAL_PATH_NUMS - AAL_PATH_MIX0); ii++)
        {
            if (aal_mixer->mixerHandle > 0) {
                FAPEX_AUDEC_Close( aal_mixer->mixerHandle );
            }

            if  ( aal_mixer->paramsPtr )
            {
                FAPI_SYS_FREE( aal_mixer->paramsPtr);
                aal_mixer->paramsPtr = NULL;
                aal_mixer->paramsSize = 0;
                FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalCloseAllMixer : mixer params freed .\n" );
            }

            for ( jj=0 ; jj<2 ; jj++)
            {
                if( aal_mixer->auoutMixerInput[jj] ) {
                    FAPEX_AUOUT_Close( aal_mixer->auoutMixerInput[jj] );
                }
            }
            aal_mixer->mixer_used = 0;

            FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalCloseAllMixer : mixer %d .\n", aal_mixer->mixerID );

            mixer_Close_Num++;

            if ( aal_mixer->nextMixer != NULL)
            {
                aal_mixer = aal_mixer->nextMixer;
            }
            else
            { //the last one in chain.
                break;
            }
        }

        if ( mixer_Close_Num != openPtr->aalMixerData.mixerOpenNum )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalCloseAllMixer :  not all closed .\n" );
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }

        openPtr->aalMixerData.mixerOpenNum -= mixer_Close_Num;
        openPtr->aalMixerData.mixerUsedNum -= mixer_Close_Num;
        openPtr->aalMixerData.firstMixerInChain = NULL ;
        openPtr->aalMixerData.lastMixerInChain = NULL ;
    }
    return( err_code );
}

static void aalInitMixerID( AAL_DataT* inst  )
{
    uint32_t     ii;

    for (ii=0 ; ii < (AAL_PATH_NUMS -AAL_PATH_MIX0); ii++)
    {
        inst->openData.aalMixerData.aal_mixer[ii].mixerID = ii + AAL_PATH_MIX0;
    }
}

static int32_t aalIntAllMixerStart (  AAL_OpenT* openPtr  )
{
    int32_t      err_code = FAPI_OK;
    AAL_MixerT*  aal_mixer ;
    AAL_MixerT*  aal_mixer_next ;
    uint32_t     mixer_start_Num = 0;

    aal_mixer_next = openPtr->aalMixerData.firstMixerInChain;

    do
    {
        aal_mixer = aal_mixer_next;

        aal_mixer_next = aal_mixer->nextMixer ;

        if ( (aal_mixer->mixer_used) && (!aal_mixer->mixer_started) )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalIntAllMixerStart mixer %d ",aal_mixer->mixerID );
            err_code = FAPEX_AUDEC_Ioctl ((FAPEXIOCARG1_T)aal_mixer->mixerHandle,
                                        FAPEX_IOCCOM_START, (FAPEXIOCARG3_T)NULL );
            if (err_code == FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(2, "ok!  \n");
                aal_mixer->mixer_started = 1;
                mixer_start_Num ++;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(2, "failed!  \n");
                break;
            }
        }
    }
    while (aal_mixer_next);

    openPtr->aalMixerData.mixerUsedNum += mixer_start_Num;
    if (openPtr->aalMixerData.mixerUsedNum != openPtr->aalMixerData.mixerOpenNum)
    {
        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Opened mixer not all started or some have already started! started num=%d ,used =%d, opened =%d \n",mixer_start_Num, openPtr->aalMixerData.mixerUsedNum, openPtr->aalMixerData.mixerOpenNum);
    }

    return( err_code );
}

static int32_t aalIntAllMixerStop (  AAL_OpenT* openPtr  )
{
    int32_t      err_code = FAPI_OK;
    AAL_MixerT*  aal_mixer ;
    AAL_MixerT*  aal_mixer_next ;
    uint32_t     mixer_stop_Num = 0;

    aal_mixer_next = openPtr->aalMixerData.firstMixerInChain;

    do
    {
        aal_mixer = aal_mixer_next;

        aal_mixer_next = aal_mixer->nextMixer ;

        if ( (aal_mixer->mixer_used) && (aal_mixer->mixer_started) )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalIntAllMixerStop mixer %d ",aal_mixer->mixerID );
            err_code = FAPEX_AUDEC_Ioctl( aal_mixer->mixerHandle, FAPEX_IOCCOM_STOP, NULL );

            if (err_code == FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(2, "ok!  \n");
                aal_mixer->mixer_started = 0;

                mixer_stop_Num ++;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(2, "failed!  \n");
                break;
            }
        }
    }
    while (aal_mixer_next);

    openPtr->aalMixerData.mixerUsedNum -= mixer_stop_Num;
    if (openPtr->aalMixerData.mixerUsedNum != 0)
    {
        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Opened mixer not all stoped! closed num=%d ,use =%d \n",mixer_stop_Num ,openPtr->aalMixerData.mixerUsedNum);
    }

    return( err_code );
}

static int32_t aalIntFeedPathBuffer (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId,
                                const void* bufferPtr, int32_t bytes, uint32_t timeout )
{
    int32_t     err_code = FAPEX_OK;
    int32_t     retVal;
    AAL_DataT*  inst = audioDatPtr;
    AAL_OpenT*  openPtr;
    AAL_AudioPathOpenT *pPath;

    /*check param */
    if((NULL == bufferPtr) || (0 == bytes))
    {
        return -1;
    }

    /*check param */
    if(NULL == inst)
    {
        return SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        pPath = openPtr->path + pathId;
        retVal = audioBmWrite(pPath->mp3MapBufAddr,
                                bufferPtr,
                                bytes,
                                FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES,
                                timeout,
                                pPath->handleTsd);
        if ( retVal != bytes )
        {
            err_code = retVal;
        }
    }
    else
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_FeedBuffer: Out(%d)\n",err_code);

    return err_code;
}

static int32_t aalIntClearPathBuffer (FAPI_SYS_HandleT handle , AAL_AudioPathT pathId)
{
    int32_t     err_code = FAPI_OK;
    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {

        err_code = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[pathId].handleTsd,
                                   FAPEX_IOCCOM_TSD_RESET_BUFFER,
                                   0);
        if(err_code < 0) {
                FAPI_SYS_PRINT_ERR("ERROR Write Audio BM buffer (code %d).\n", err_code);
             }

    }
    else
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    return err_code;
}

static int32_t aalIntGetPathDecStatus (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId ,
                                 AAL_AUDEC_StatusEnumT* statusPtr)
{
    int32_t     err_code = FAPI_OK;
    AAL_OpenT*  openPtr;
    FAPEX_AUDEC_StatusStrT status;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( openPtr->path[pathId].handleDec <= 0 )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        }
        else if ( statusPtr != NULL )
        {
            err_code = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T) openPtr->path[pathId].handleDec,
                                    FAPEX_IOCCOM_AUDEC_GET_STATUS,
                                    (FAPEXIOCARG3_T)&status);
            if(err_code == 0)
                *statusPtr = status.firmStatus;
        }

        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

static int32_t aalIntGetPathStreamInfo (FAPI_SYS_HandleT handle,  AAL_AudioPathT pathId ,
                            AAL_AUOUT_StreamInfoT* infoPtr)
{
    int32_t                 err_code = FAPEX_OK;
    int32_t                 stream_type;
    AAL_AUOUT_StreamInfoT   streamInfo;
    AAL_OpenT*              openPtr;
    FAPEX_AUOUT_PcmInfoStrT pcmInfoParam;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)openPtr->path[pathId].handleOut,
                                      FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE,
                                      (FAPEXIOCARG3_T)&stream_type );

        err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)openPtr->path[pathId].handleOut,
                                      FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                     (FAPEXIOCARG3_T)&pcmInfoParam );

       streamInfo.sampleBits = pcmInfoParam.sampleBits;
       streamInfo.sampleFreq = pcmInfoParam.sampleFreq;
       streamInfo.channels   = pcmInfoParam.channelCount;
       streamInfo.lfeOn      = 0;
       streamInfo.type       = stream_type;

        if ( infoPtr != NULL )
        {
            *infoPtr = streamInfo;
        }
    }

    return err_code;
}

static uint32_t aalIntGetPathBufferSize (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId)
{
    uint32_t    bufSize;
    AAL_OpenT*  openPtr;
    FAPEX_TSD_ChannelInfoStrT  ch_info;
    int32_t result;

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[pathId].started) )
    {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[pathId].handleTsd,
                                   FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                  (FAPEXIOCARG3_T)&ch_info );
        if(result == 0) {
            bufSize = ch_info.bufferSize;
        }
        else
            bufSize = 0;
    }
    else
    {
        bufSize = 0;
    }

    return bufSize;
}

static uint32_t aalIntGetPathBufferLevel (FAPI_SYS_HandleT handle, AAL_AudioPathT pathId)
{
    int32_t     result;
    AAL_OpenT*  openPtr;
    uint32_t    level = 0;

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[pathId].started) )
    {
        AAL_Assert (openPtr->path[pathId].handleTsd > 0);

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)openPtr->path[pathId].handleTsd,
                                  FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                 (FAPEXIOCARG3_T)&level );
    }

    return level;
}

static int32_t aalAddMixPathAptsCb( FAPI_SYS_HandleT audioOutHandle,AAL_MixPathPramsT* mixerPathParams
                                               ,AAL_AUOUT_SampleFreqEnumT APTsampleFreq ) {
    int retVal = FAPEX_OK;

    retVal = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_ADD_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_SIMPLE_SRC);
    if (retVal == FAPEX_OK) {
        retVal = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T) audioOutHandle,
                FAPEX_IOCCOM_AUOUT_SET_APTSAMPLESRC, APTsampleFreq);
    }

    if (retVal == FAPEX_OK) {
        retVal = FAPEX_AUOUT_Ioctl((FAPEXIOCARG1_T)audioOutHandle,
                                FAPEX_IOCCOM_AUOUT_ADD_APT,
                                (FAPEXIOCARG3_T)AAL_AUOUT_APT_VOLUME_CONTROL);
    }
    return retVal;
}

static int32_t aalRemoveMixPathAptsCb(FAPI_SYS_HandleT audioOutHandle) {
    FAPEX_AUOUT_Ioctl(audioOutHandle, FAPEX_IOCCOM_AUOUT_REMOVE_APT, AAL_AUOUT_APT_SIMPLE_SRC);
    return FAPEX_AUOUT_Ioctl(audioOutHandle, FAPEX_IOCCOM_AUOUT_REMOVE_APT, AAL_AUOUT_APT_VOLUME_CONTROL);
}

static int32_t aalIntPathBufMmap(FAPI_SYS_HandleT handle, AAL_AudioPathT pathId) {
    AAL_OpenT*  openPtr;
    int32_t     err_code = FAPEX_OK;
    AAL_AudioPathOpenT *pPath = NULL;
    int32_t buffSize = -1;

    openPtr = aalCheckHandle(handle, &err_code);
    if ( openPtr != NULL )
    {
        pPath = openPtr->path + pathId;
        if(NULL != pPath->mp3MapBufAddr)
        {
            FAPI_SYS_PRINT_MSG("[AAL] AAL_BufMmap: already map\n");
            return SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED;
        }

        buffSize = aalIntGetPathBufferSize(handle, pathId);
        if (buffSize <= 0) {
            return -1;
        }
        pPath->mp3MapBufAddr = (uint8_t*)FAPEX_TSD_Mmap( NULL, buffSize,
                                                        (PROT_READ | PROT_WRITE), MAP_SHARED,
                                                        pPath->handleTsd, 0 );
        if (pPath->mp3MapBufAddr == MAP_FAILED){
            FAPI_SYS_PRINT_MSG("[AAL] Error: TSD audio bm buffer mmap failed. \n");
            return -1;
        }
    }
    memset(pPath->mp3MapBufAddr,0,FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES);

    FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_BufMmap >> mp3MapBufAddr:[0x%x]\n",pPath->mp3MapBufAddr);
    return err_code;
}

static int32_t aalIntPathBufMunmap(FAPI_SYS_HandleT handle, AAL_AudioPathT pathId) {
    int32_t     err_code = FAPEX_OK;
    AAL_AudioPathOpenT *pPath = NULL;
    int32_t buffSize = -1;
    AAL_OpenT*  openPtr = aalCheckHandle(handle, &err_code);
    if (openPtr == NULL) {
        return -1;
    }

    pPath = openPtr->path + pathId;
    if(NULL == pPath->mp3MapBufAddr)
    {
        return SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

    buffSize = aalIntGetPathBufferSize(handle, pathId);
    if (buffSize <= 0) {
        return -1;
    }
    FAPEX_TSD_Munmap(pPath->mp3MapBufAddr, buffSize);
    pPath->mp3MapBufAddr = NULL;
    return 0;
}

/*****************************************************************************/
/*lint -restore */
/* END OF FILE */
