/****************************************************************************
*
*  Front-end device driver for Fujitsu MB86A22 ISDB-T frontend.
*
*  Copyright (c) 2009 Fujitsu Microelectronics 
*  All rights reserved. 
*
*  Created 09/16/2009
*
****************************************************************************/
#include "../src/fe_priv.h" //for gnuarm compiler

#include "MB86A22_API_version.h"
#include "MB86A22_API_defs.h"
#include "MB86A22_Tuner.h"
#include "MB86A22_API_structs.h"
#include "MB86A22_Tuner_default_setting.h"
#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
#include "FC2581/FC2581_driver_v1411.h"
#endif
#include "MB86A22_user_define.h"
#include "MB86A22_API.h"
#include "MB86A22_print.h"
#include "stdio.h"

MB86A2x_ERR_MSG MB86A22_Init_Adr( MB86A2x_PARAM *param )
{
#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_MXL136RF)
    {
        param->nim_param.MB86A2x_Adr = MB86A22_MXL136RF_PARAM_TABLE.nim_param.MB86A2x_Adr;
        param->nim_param.tuner_param.Tuner_Adr = MB86A22_MXL136RF_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18211)
    {
        param->nim_param.MB86A2x_Adr = MB86A22_TDA182I1_PARAM_TABLE.nim_param.MB86A2x_Adr;
        param->nim_param.tuner_param.Tuner_Adr = MB86A22_TDA182I1_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18212)
    {
        param->nim_param.MB86A2x_Adr = MB86A22_TDA182I2_PARAM_TABLE.nim_param.MB86A2x_Adr;
        param->nim_param.tuner_param.Tuner_Adr = MB86A22_TDA182I2_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_MAX3580)
    {
        param->nim_param.MB86A2x_Adr = MB86A22_Max3580_PARAM_TABLE.nim_param.MB86A2x_Adr;
        param->nim_param.tuner_param.Tuner_Adr = MB86A22_Max3580_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    }
#endif

#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_FC2581)
    {
        param->nim_param.MB86A2x_Adr = MB86A22_FC2581_PARAM_TABLE.nim_param.MB86A2x_Adr;
        param->nim_param.tuner_param.Tuner_Adr = MB86A22_FC2581_PARAM_TABLE.nim_param.tuner_param.Tuner_Adr;
    }
#endif

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_Version( MB86A2x_NIM_PARAM *nim_param, MB86A2x_VERSION_INFO *version_info )
{
    UINT32 Status ;
    UINT8  read_val ;

    version_info->DRV_VERSION = MB86A2x_DRIVER_VERSION ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x00, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    version_info->LSI_VERSION = read_val ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_DRV_Version( MB86A2x_VERSION_INFO *version_info )
{

    version_info->DRV_VERSION = MB86A2x_DRIVER_VERSION ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_LSI_Version( MB86A2x_NIM_PARAM *nim_param, MB86A2x_VERSION_INFO *version_info )
{
    UINT32 Status ;
    UINT8  read_val ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x00, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    version_info->LSI_VERSION = read_val ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Init_Param( MB86A2x_PARAM *param )
{
    #if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_MXL136RF)
    {
        // Editing for MxL
        memcpy(param, &MB86A22_MXL136RF_PARAM_TABLE, sizeof(MB86A2x_PARAM));
    }
    #endif  

    #if (MB86A22_TUNER_SUPPORT_FC2581 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_FC2581)
    {
        memcpy(param, &MB86A22_FC2581_PARAM_TABLE, sizeof(MB86A2x_PARAM));
    }
    #endif

    #if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_MAX3580)
    {
        memcpy(param, &MB86A22_Max3580_PARAM_TABLE, sizeof(MB86A2x_PARAM)); 
    }
    #endif

    #if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18211)
    {
        // Tuner RF setting
        memcpy(param, &MB86A22_TDA182I1_PARAM_TABLE, sizeof(MB86A2x_PARAM));                
    }
    #endif  

    #if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18212)
    {
       memcpy(param, &MB86A22_TDA182I2_PARAM_TABLE, sizeof(MB86A2x_PARAM));
    }
    #endif          

    #if (MB86A22_TUNER_SUPPORT_NI386X> 0)
    if(param->nim_param.TUNER_TYPE == MB86A22_ISDBT_NI386X)
    {
       memcpy(param, &MB86A22_NI386X_PARAM_TABLE, sizeof(MB86A2x_PARAM));
    }
    #endif            

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Init( MB86A2x_PARAM *param, int RS_EN )
{
    UINT32 Status ;
    MB86A2x_ERR_MSG ERR_MSG ;

    if (param->pll_param.BANDSEL == 0){  // ISDB-T 6MHz
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0x00 ) ;   //execute I2C Reg reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xff ) ;   //release I2C Reg reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x01 ) ;   //sequencer stop
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_Power_Mode( &param->nim_param, &param->agc_param ) ;  //Power Down Setting
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x06, 0xff ) ;   //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1f, 0xf ) ;    //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        
    if (param->pll_param.BANDSEL == 0)
    {  // ISDB-T 6MHz
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x71, 0x00 ) ;   
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;  
    }

    if( param->nim_param.tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_000_kHz ) 
    {
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x02 ) ;   //Set IF input mode=IQ
        if( Status==I2C_NG ) 
            return MB86A2x_ERR_INIT ;
    }
    else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_040_kHz) ) 
    {
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x01) ;    //Set IF input mode=IF
        if( Status==I2C_NG ) 
            return MB86A2x_ERR_INIT ;
    }
    else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_570_kHz) ) 
    {
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x01 ) ;   //Set IF input mode=IF
        if( Status==I2C_NG ) 
            return MB86A2x_ERR_INIT ;
    }
    else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_440_kHz) ) 
    {
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x11 ) ;   //Set IF input mode=IF
        if( Status==I2C_NG ) 
            return MB86A2x_ERR_INIT ;
    }
    else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_360_kHz) ) 
    {
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x21 ) ;   //Set IF input mode=IF
        if( Status==I2C_NG ) 
            return MB86A2x_ERR_INIT ;
    }

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x29 ) ;   // Manual Setting for Clock Recovery Offset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x25 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x1f ) ;   // Manual Setting for Wide Carrier Offset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x04 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Calc_MASTER_CLK( &param->pll_param, &param->nim_param.tuner_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_STR_OFS( &param->nim_param, &param->pll_param, &param->recov_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_CRR_OFS( &param->nim_param, &param->pll_param, &param->recov_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_WIDE_FREQ_DET( &param->nim_param, &param->pll_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_AGC( &param->nim_param, &param->agc_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_IFAGC( &param->nim_param, &param->ifagc_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Set_TS_Output( &param->nim_param, &param->tsout_param, RS_EN ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x3b, 0xd1 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x3c, 0xa1 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x50, 0x38 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x51, 0xa0 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0xee ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0xc0 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x04 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1a, 0x09 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1b, 0x43 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x4f ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0xff ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x20, 0xab ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x21, 0x89 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x22, 0x37 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x59 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x0f ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x64 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x81 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x23, 0x81 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x24, 0x91 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x25, 0x93 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x26, 0x92 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x27, 0x94 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x68 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x61 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x22 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x22 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x3e ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0xb1 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x82 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x42 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x3d ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x06 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0xc4 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        //state4 timer
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x54 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x40 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x52 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x03 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x2c ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x44 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x43 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x33 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x4a ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, 0x27 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, 0x1a ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    }
    else if (param->pll_param.BANDSEL == 1 || param->pll_param.BANDSEL == 2 || param->pll_param.BANDSEL == 3 )
    { // DVB-T 6MHz, DVB-T 7MHz, DVB-T 8MHz
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0x00 ) ;   //execute I2C Reg reset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xff ) ;   //release I2C Reg reset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x01 ) ;   //sequencer stop
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
        ERR_MSG = MB86A22_Set_Power_Mode( &param->nim_param, &param->agc_param ) ;  //Power Down Setting
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x06, 0xff ) ;   //Set all mode detect(1/2)
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1f, 0x0f ) ;   //Set all mode detect(1/2)
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        
        if (param->pll_param.BANDSEL == 1){  // DVB-T 6MHz
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x71, 0x61 ) ;   
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;  
        }
        else if (param->pll_param.BANDSEL == 2){  // DVB-T 7MHz
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x71, 0x01 ) ;   
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;  
        }
        else if (param->pll_param.BANDSEL == 3){  // DVB-T 8MHz
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x71, 0x21 ) ;   
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;  
        }

        // Editing for MxL
        if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_000_kHz) ) {
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x02 ) ;   //Set IF input mode=IQ
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        }
        else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_040_kHz) ) {
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x01) ;    //Set IF input mode=IF
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        }
        else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_570_kHz) ) {
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x01 ) ;   //Set IF input mode=IF
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        }
        else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_440_kHz) ) {
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x11 ) ;   //Set IF input mode=IF
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        }
        else if( param->nim_param.tuner_param.IF_freq_kHz==(MB86A2x_IF_FREQ_360_kHz) ) {
            Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1c, 0x21 ) ;   //Set IF input mode=IF
            if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        }

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x29 ) ;   // Manual Setting for Clock Recovery Offset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x25 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x1f ) ;   // Manual Setting for Wide Carrier Offset
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, 0x04 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Calc_MASTER_CLK( &param->pll_param, &param->nim_param.tuner_param ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Set_STR_OFS( &param->nim_param, &param->pll_param, &param->recov_param ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Set_CRR_OFS( &param->nim_param, &param->pll_param, &param->recov_param ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Set_WIDE_FREQ_DET( &param->nim_param, &param->pll_param ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

        ERR_MSG = MB86A22_Set_TS_Output( &param->nim_param, &param->tsout_param, RS_EN ) ;
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;
        
/*
    ERR_MSG = MB86A22_Set_RFAGC( &param->nim_param, &param->rfagc_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A2x_Set_Opt_Param( &param->nim_param, &param->tun_dep_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_TS_Output( &param->nim_param, &param->tsout_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;
*/
}       

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x52, 0x01 ) ;   //Pre Viterbi BER Switch On
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x53, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x53, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x5e, 0x07 ) ;   //Post Viterbi BER Switch On
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x5f, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x5f, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x45, 0x04 ) ;   //set C/N Monitor mode to manual
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x50, 0x50 ) ;   //set MER Monitor mode to manual(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x51, 0x02 ) ;   //set MER Monitor mode to manual(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x50, 0xb0 ) ;   //Post RS BER Switch On(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x51, 0x07 ) ;   //Post RS BER Switch On(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x50, 0xb1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x51, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x51, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    //added from ver5
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x6d, 0xfb ) ;   //Accept Layer-A > Layer-B
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x6e, 0xbf ) ;   //Accept Layer-A > Layer-B
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_IRQ_All_Off( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Set_ModeGuard_fixed( MB86A2x_PARAM *param, MB86A2x_CH_SCH_INFO *ch_sch_info, UINT8 ch_num )
{
    UINT32 Status ;
    UINT32 MG = 0;

    if( ch_sch_info->MODE[ch_num]==0 ) {    //MODE1
        if(      ch_sch_info->GUARD[ch_num]==0 ) MG = 0x001 ;   //MODE1, GI=1/4
        else if( ch_sch_info->GUARD[ch_num]==1 ) MG = 0x002 ;   //MODE1, GI=1/8
        else if( ch_sch_info->GUARD[ch_num]==2 ) MG = 0x004 ;   //MODE1, GI=1/16
        else if( ch_sch_info->GUARD[ch_num]==3 ) MG = 0x008 ;   //MODE1, GI=1/32
    }
    else if( ch_sch_info->MODE[ch_num]==1 ) {   //MODE2
        if(      ch_sch_info->GUARD[ch_num]==0 ) MG = 0x010 ;   //MODE2, GI=1/4
        else if( ch_sch_info->GUARD[ch_num]==1 ) MG = 0x020 ;   //MODE2, GI=1/8
        else if( ch_sch_info->GUARD[ch_num]==2 ) MG = 0x040 ;   //MODE2, GI=1/16
        else if( ch_sch_info->GUARD[ch_num]==3 ) MG = 0x080 ;   //MODE2, GI=1/32
    }
    else if( ch_sch_info->MODE[ch_num]==2 ) {   //MODE3
        if(      ch_sch_info->GUARD[ch_num]==0 ) MG = 0x100 ;   //MODE3, GI=1/4
        else if( ch_sch_info->GUARD[ch_num]==1 ) MG = 0x200 ;   //MODE3, GI=1/8
        else if( ch_sch_info->GUARD[ch_num]==2 ) MG = 0x400 ;   //MODE3, GI=1/16
        else if( ch_sch_info->GUARD[ch_num]==3 ) MG = 0x800 ;   //MODE3, GI=1/32
    }

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x06, (UINT8)((MG&0xff0)>>4) ) ; //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1f, (UINT8)(MG&0x00f) ) ;  //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Set_ModeGuard_all( MB86A2x_PARAM *param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x06, 0xff ) ;   //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x1f, 0xf ) ;    //Set all mode detect(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Calc_MASTER_CLK( MB86A2x_PLL_PARAM *pll_param, TUNER_PARAM *tuner_param )
{
    int temp = 0;

    if( pll_param->PLLPASS==1 ) {
        pll_param->MASTER_CLK_FREQ_kHz = pll_param->INPUT_CLK_FREQ_kHz ;
    }
    else {
        if( pll_param->REFSEL==MB86A2x_REFSEL_32MHz || pll_param->REFSEL==MB86A2x_REFSEL_26MHz )
            temp = pll_param->INPUT_CLK_FREQ_kHz / 8;
        else if( pll_param->REFSEL==MB86A2x_REFSEL_16MHz )
            temp = pll_param->INPUT_CLK_FREQ_kHz / 4;
        else
            temp = pll_param->INPUT_CLK_FREQ_kHz;

        if( pll_param->REFSEL==MB86A2x_REFSEL_04MHz
            || pll_param->REFSEL==MB86A2x_REFSEL_16MHz
            || pll_param->REFSEL==MB86A2x_REFSEL_32MHz )
            temp = temp * 114;
        else
            temp = temp * 141;

        // Editing for MxL
        //if (tuner_param->IF_freq_kHz == MB86A2x_IF_FREQ_360_kHz){
        if (tuner_param->IF_freq_kHz == MB86A2x_IF_FREQ_360_kHz){
            temp = temp / 10;
        }
        else{
            if( pll_param->BANDSEL==MB86A2x_BANDSEL_ISDBT_6MHz ){
                if( pll_param->FREQSEL==MB86A2x_FREQSEL_0 )
                    temp = temp / 14;
                else if( pll_param->FREQSEL==MB86A2x_FREQSEL_1 )
                    temp = temp / 12;
                else if( pll_param->FREQSEL==MB86A2x_FREQSEL_2 )
                    temp = temp / 11;
            }
            else if( pll_param->BANDSEL==MB86A2x_BANDSEL_DVBT_6MHz )
                temp = temp / 16;
            else if( pll_param->BANDSEL==MB86A2x_BANDSEL_DVBT_7MHz )
                temp = temp / 14;
            else if( pll_param->BANDSEL==MB86A2x_BANDSEL_DVBT_8MHz )
                temp = temp / 12;       
        }
        pll_param->MASTER_CLK_FREQ_kHz = temp;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_STR_OFS( MB86A2x_NIM_PARAM *nim_param, MB86A2x_PLL_PARAM *pll_param, MB86A2x_RECOV_PARAM *recov_param )
{
    UINT32 Status ;
    REAL32 temp = ((32.0*64.0/63.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;

    //temp = ((REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*63.0 - 2048000.0)/125.0*1024.0 ;
    if (pll_param->BANDSEL == MB86A2x_BANDSEL_ISDBT_6MHz)
        temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((32.0*64.0/63.0)*1000.0) )/((32.0*64.0/63.0)*1000.0)*16777216.0 ;
    //else if (pll_param->BANDSEL == MB86A2x_BANDSEL_ISDBT_7MHz)
    //  temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((32.0*64.0/63.0*7.0/6.0)*1000.0) )/((32.0*64.0/63.0*7.0/6.0)*1000.0)*16777216.0 ;
    //else if (pll_param->BANDSEL == MB86A2x_BANDSEL_ISDBT_8MHz)
    //  temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((32.0*64.0/63.0*8.0/6.0)*1000.0) )/((32.0*64.0/63.0*8.0/6.0)*1000.0)*16777216.0 ;  
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_6MHz)
        temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((4.0*48.0/7.0)*1000.0) )/((4.0*48.0/7.0)*1000.0)*16777216.0 ;
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_7MHz)
        temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((4.0*48.0/7.0*7.0/6.0)*1000.0) )/((4.0*48.0/7.0*7.0/6.0)*1000.0)*16777216.0 ;  
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_8MHz)
        temp = ( (REAL32)(pll_param->MASTER_CLK_FREQ_kHz) - ((4.0*48.0/7.0*8.0/6.0)*1000.0) )/((4.0*48.0/7.0*8.0/6.0)*1000.0)*16777216.0 ;  
    recov_param->STR_OFS = (UINT32)(temp+0.50) ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x2a ) ; //Timing recoevery setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, (UINT8)((recov_param->STR_OFS)>>16) ) ;  //Timing recoevery setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, (UINT8)(((recov_param->STR_OFS)&0x00ff00)>>8) ) ;    //Timing recoevery setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, (UINT8)((recov_param->STR_OFS)&0x0000ff) ) ; //Timing recoevery setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_STR_OFS2( MB86A2x_NIM_PARAM *nim_param, MB86A2x_CH_SCH_INFO *ch_sch_info, UINT8 ch_num, MB86A2x_RECOV_PARAM *recov_param )
{
    UINT32 Status ;

    recov_param->STR_OFS = ch_sch_info->STR[ch_num] ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x2a ) ; //Timing recoevery setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, (UINT8)((recov_param->STR_OFS)>>16) ) ;  //Timing recoevery setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, (UINT8)(((recov_param->STR_OFS)&0x00ff00)>>8) ) ;    //Timing recoevery setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, (UINT8)((recov_param->STR_OFS)&0x0000ff) ) ; //Timing recoevery setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_CRR_OFS( MB86A2x_NIM_PARAM *nim_param, MB86A2x_PLL_PARAM *pll_param, MB86A2x_RECOV_PARAM *recov_param )
{
    UINT32 Status ;
    REAL32 temp = 0;

    if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_570_kHz ) {
        temp = ((REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*2.0-57000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*16777216.0 ;
    }
    else if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_440_kHz ) {
        temp = (44000.0-(REAL32)(pll_param->MASTER_CLK_FREQ_kHz))/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*16777216.0 ;
    }
    else if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_040_kHz ) {
        temp = (REAL32)(MB86A2x_IF_FREQ_040_kHz)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*16777216.0 ;
    }
    else if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_033_kHz ) {
        temp = (REAL32)(MB86A2x_IF_FREQ_033_kHz)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*16777216.0 ;
    }

    else if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_000_kHz ) {
        temp = 0.0 ;
    }

    recov_param->CRR_OFS = (UINT32)(temp+0.50) ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x20 ) ; //Carrier recoevery setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, (UINT8)((recov_param->CRR_OFS)>>16) ) ;  //Carrier recoevery setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, (UINT8)(((recov_param->CRR_OFS)&0x00ff00)>>8) ) ;    //Carrier recoevery setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, (UINT8)((recov_param->CRR_OFS)&0x0000ff) ) ; //Carrier recoevery setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_WIDE_FREQ_DET( MB86A2x_NIM_PARAM *nim_param, MB86A2x_PLL_PARAM *pll_param )
{
    UINT32 Status ;
    REAL32 temp  = ((32.0*64.0/63.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;
    UINT32 temp_value ;

    if (pll_param->BANDSEL == MB86A2x_BANDSEL_ISDBT_6MHz)
        temp = ((32.0*64.0/63.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_6MHz)
        temp = ((4.0*48.0/7.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_7MHz)
        temp = ((4.0*48.0/7.0*7.0/6.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;
    else if (pll_param->BANDSEL == MB86A2x_BANDSEL_DVBT_8MHz)
        temp = ((4.0*48.0/7.0*8.0/6.0)*1000.0)/(REAL32)(pll_param->MASTER_CLK_FREQ_kHz)*8192.0 ;
    temp_value = (UINT32)(temp+0.50) ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x22 ) ; //Wide freq detector data conversion setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, (UINT8)((temp_value)>>16) ) ;    //Wide freq detector data conversion setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, (UINT8)(((temp_value)&0x00ff00)>>8) ) ;  //Wide freq detector data conversion setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, (UINT8)((temp_value)&0x0000ff) ) ;   //Wide freq detector data conversion setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_AGC(  MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_PARAM * agc_param )
{
    UINT32 Status ;
    UINT8 value ;

    if( agc_param->RFAGC_ON==1 ) value = 0x00 ;
    else                         value = 0x04 ;

    value = value | (UINT8)((agc_param->MAIF)<<4) | (UINT8)((agc_param->IFAGC_POL)<<3) | (UINT8)((agc_param->MARF)<<1) | (UINT8)(agc_param->RFAGC_POL) ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x01, value ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IFAGC( MB86A2x_NIM_PARAM *nim_param, MB86A2x_IFAGC_PARAM *ifagc_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x08 ) ; //DTS setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->DTS)&0x00ff) ) ;   //DTS setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    //Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x1b ) ;   //DTS setting(3/4)
    //if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    //Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->DTS)>>8) ) ; //DTS setting(4/4)
    //if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x0e ) ; //VIFREF setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->VIFREF)>>8) ) ;    //VIFREF setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x0f ) ; //VIFREF setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->VIFREF)&0x00ff) ) ;    //VIFREF setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x0b ) ; //AGAIN setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->AGAIN) ) ;  //AGAIN setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x00 ) ; //IFA setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->IFA)>>8) ) ;   //IFA setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x01 ) ; //IFA setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->IFA)&0x00ff) ) ;   //IFA setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x02 ) ; //IFB setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->IFB)>>8) ) ;   //IFB setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x03 ) ; //IFB setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((ifagc_param->IFB)&0x00ff) ) ;   //IFB setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x09 ) ; //IFAGCO setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->IFAGCO) ) ; //IFAGCO setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x0a ) ; //MAXIFAGC setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->MAXIFAGC) ) ;   //MAXIFAGC setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x29 ) ; //IFSAMPLE setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->IFSAMPLE) ) ;   //IFSAMPLE setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x32 ) ; //OUTSAMPLE setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->OUTSAMPLE) ) ;  //OUTSAMPLE setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x27 ) ; //VMAIF setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(ifagc_param->VMAIF) ) ;  //VMAIF setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_RFAGC( MB86A2x_NIM_PARAM *nim_param, MB86A2x_RFAGC_PARAM *rfagc_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x14 ) ; //BGAIN setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(rfagc_param->BGAIN) ) ;  //BGAIN setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x04 ) ; //RFA setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->RFA)>>8) ) ;   //RFA setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x05 ) ; //RFA setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->RFA)&0x00ff) ) ;   //RFA setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x06 ) ; //RFB setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->RFB)>>8) ) ;   //RFB setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x07 ) ; //RFB setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->RFB)&0x00ff) ) ;   //RFB setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x12 ) ; //RFAGCO setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(rfagc_param->RFAGCO) ) ; //RFAGCO setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x13 ) ; //MAXRFAGC setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(rfagc_param->MAXRFAGC) ) ;   //MAXRFAGC setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x15 ) ; //DRF setting(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->DRF)>>8) ) ;   //DRF setting(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x16 ) ; //DRF setting(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)((rfagc_param->DRF)&0x00ff) ) ;   //DRF setting(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x1e ) ; //VMARF setting(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x05, (UINT8)(rfagc_param->VMARF) ) ;  //VMARF setting(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Opt_Param( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TU_DEP_PARAM *tun_dep_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x05 ) ; //Parameter Optimization-1(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x02 ) ; //Parameter Optimization-1(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x1f ) ; //Parameter Optimization-2(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x08 ) ; //Parameter Optimization-2(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x03 ) ; //Parameter Optimization-3(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-3(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x62 ) ; //Parameter Optimization-3(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x11 ) ; //Parameter Optimization-3(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x3c ) ; //Parameter Optimization-4(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-4(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x02 ) ; //Parameter Optimization-4(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x00 ) ; //Parameter Optimization-4(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x7f ) ; //Parameter Optimization-5(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-5(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x00 ) ; //Parameter Optimization-5(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x00 ) ; //Parameter Optimization-5(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x79 ) ; //Parameter Optimization-6(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-6(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x00 ) ; //Parameter Optimization-6(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x03 ) ; //Parameter Optimization-6(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x4f, 0x13 ) ; //Parameter Optimization-7
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x3e ) ; //Parameter Optimization-8(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-8(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x1c ) ; //Parameter Optimization-8(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x84 ) ; //Parameter Optimization-8(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x23 ) ; //Parameter Optimization-9(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x00 ) ; //Parameter Optimization-9(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x48 ) ; //Parameter Optimization-10(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x02 ) ; //Parameter Optimization-10(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x0c ) ; //Parameter Optimization-11(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x59 ) ; //Parameter Optimization-11(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x26 ) ; //Parameter Optimization-12(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x01 ) ; //Parameter Optimization-12(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x27 ) ; //Parameter Optimization-12(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x04 ) ; //Parameter Optimization-12(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x6a ) ; //Parameter Optimization-13(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-13(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, (UINT8)((0x00ff00&(tun_dep_param->S1CNT))>>8) ) ;    //Parameter Optimization-13(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, (UINT8)( 0x0000ff&(tun_dep_param->S1CNT)) ) ;    //Parameter Optimization-13(4/4)
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x40, 0x6e ) ; //Parameter Optimization-14(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x41, 0x70 ) ; //Parameter Optimization-14(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x5a ) ; //Parameter Optimization-15(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-15(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x00 ) ; //Parameter Optimization-15(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x28 ) ; //Parameter Optimization-15(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x14, 0x0a ) ; //Parameter Optimization-16(1/1)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x0a ) ; //Parameter Optimization-17(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x02 ) ; //Parameter Optimization-17(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x0b ) ; //Parameter Optimization-18(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x20 ) ; //Parameter Optimization-18(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x40, 0x6c ) ; //Parameter Optimization-19(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x41, 0x60 ) ; //Parameter Optimization-19(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x6c ) ; //Parameter Optimization-20(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x72 ) ; //Parameter Optimization-20(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x74 ) ; //Parameter Optimization-21(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x20 ) ; //Parameter Optimization-21(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x65 ) ; //Parameter Optimization-22(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x05 ) ; //Parameter Optimization-22(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x7d ) ; //Parameter Optimization-23(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x02 ) ; //Parameter Optimization-23(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x1e ) ; //Parameter Optimization-24(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x8f ) ; //Parameter Optimization-24(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x20 ) ; //Parameter Optimization-25(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x28 ) ; //Parameter Optimization-25(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x24, 0x80 ) ; //Parameter Optimization-26(1/1)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x25, 0xa2 ) ; //Parameter Optimization-27(1/1)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x26, 0x91 ) ; //Parameter Optimization-28(1/1)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x27, 0xa2 ) ; //Parameter Optimization-29(1/1)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x46 ) ; //Parameter Optimization-30(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-30(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x2d ) ; //Parameter Optimization-30(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x0a ) ; //Parameter Optimization-30(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x24 ) ; //Parameter Optimization-31(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x01 ) ; //Parameter Optimization-31(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0xff ) ; //Parameter Optimization-32(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, 0x01 ) ; //Parameter Optimization-32(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3b, 0x22 ) ; //Parameter Optimization-33(1/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x3c, 0x03 ) ; //Parameter Optimization-33(2/2)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0xd9 ) ; //Parameter Optimization-34(1/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x29, 0x00 ) ; //Parameter Optimization-34(2/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, 0x00 ) ; //Parameter Optimization-34(3/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, 0x81 ) ; //Parameter Optimization-34(4/4)
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_TS_Output( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TSOUT_PARAM *tsout_param, int RS_EN )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xd5 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    /* Here FMC board is high-active??? */
    ///Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(0x00|(tsout_param->TS_ERR_MASK<<1)|(tsout_param->TS_OUT_SEL)) ) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(0x00|(tsout_param->TS_ERR_MASK<<1)|(tsout_param->TS_OUT_SEL)) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( tsout_param->TS_OUT_SEL==MB86A2x_TS_OUT_PARALLEL ) {
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xd6 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        temp = (UINT8)(0x00|(tsout_param->TS_SYNC<<4)|(tsout_param->TS_PARITY<<3)|(tsout_param->TS_LAYER)) ;
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xd2 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        if( tsout_param->TS_CLK==MB86A2x_TS_CLK_NEG ) temp = 0x07 ;
        else                                          temp = 0x03 ;
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    }
    else {
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xd7 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
        //temp = (UINT8)(0x20|(tsout_param->TS_CLK<<7)|(tsout_param->TS_SYNC<<4)|(tsout_param->TS_PARITY<<3)|(tsout_param->TS_LAYER)) ;
        temp = (UINT8)(0x00|(tsout_param->TS_CLK<<7)|(RS_EN<<5)|
            (tsout_param->TS_SYNC<<4)|(tsout_param->TS_PARITY<<3)|(tsout_param->TS_LAYER)) ;
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Channel( MB86A2x_PARAM *param )
{
    UINT32 Status ;
    MB86A2x_ERR_MSG ERR_MSG ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x01 ) ;   //sequencer stop
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( param->nim_param.RF_CONT_EN==1 ) {
        ERR_MSG = MB86A22_Set_RF( &param->nim_param, param->pll_param.BANDSEL ) ;
        if( ERR_MSG==MB86A2x_ERR_RFTUNE ) return MB86A2x_ERR_RFTUNE ;
    }

    ERR_MSG = MB86A22_Set_IQ( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_RECEP_SEG( &param->nim_param, &param->layer_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xf0 ) ;   //execute logic reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xff ) ;   //release logic reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x00 ) ;   //sequencer re-start
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Channel_1( MB86A2x_PARAM *param, MB86A2x_VERSION_INFO *version_info )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x01 ) ;   //sequencer stop
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Channel_2( MB86A2x_PARAM *param, MB86A2x_VERSION_INFO *version_info )
{
    UINT32 Status ;
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Set_IQ( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_RECEP_SEG( &param->nim_param, &param->layer_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xf0 ) ;   //execute logic reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x70, 0xff ) ;   //release logic reset
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x08, 0x00 ) ;   //sequencer re-start
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Monitor_Timer( MB86A2x_PARAM *param )
{
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Set_VBER_Timer( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_SBER_Timer( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_PBER_Timer( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_CN_Timer( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_MER_Timer( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_Monitor_Timer( MB86A2x_PARAM *param )
{
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Reset_VBER_ALL_Counter( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Reset_SBER_ALL_Counter( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Reset_PBER_ALL_Counter( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Reset_CN_Counter( &param->nim_param, &param->mon_tim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Reset_MER_Counter( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Calc_VBER_NUM( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT32 VBER_T_A, UINT32 VBER_T_B, UINT32 VBER_T_C,
                                        UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    REAL32 carrier_num ;
    REAL32 symbol_bit ;
    REAL32 R ;
    REAL32 temp ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;
    carrier_num = MB86A22_calc_carrier_num( ch_sch_info->MODE[ch_num] ) ;

    symbol_bit = MB86A22_calc_symbol_bit( ch_sch_info->MOD_A[ch_num] ) ;
    R = MB86A22_calc_data_ratio( ch_sch_info->RATE_A[ch_num] ) ;
    temp = (REAL32)(VBER_T_A)*carrier_num*symbol_bit*R*(REAL32)(ch_sch_info->NUM_A[ch_num])/sym_length/26.0 ;
    if( temp>16777215.0 ) temp = 16777215.0 ;
    if( temp<0 )          temp = 1.0 ;
    mon_tim_param->VBER_A_NUM = (UINT32)( temp ) ;

    if( (1<=ch_sch_info->NUM_B[ch_num])&&(ch_sch_info->NUM_B[ch_num]<=12) ) {   //Layer-B exsits
        symbol_bit = MB86A22_calc_symbol_bit( ch_sch_info->MOD_B[ch_num] ) ;
        R = MB86A22_calc_data_ratio( ch_sch_info->RATE_B[ch_num] ) ;
        temp = (REAL32)(VBER_T_B)*carrier_num*symbol_bit*R*(REAL32)(ch_sch_info->NUM_B[ch_num])/sym_length/26.0 ;
        if( temp>16777215.0 ) temp = 16777215.0 ;
        if( temp<0 )          temp = 1.0 ;
        mon_tim_param->VBER_B_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->VBER_B_NUM = 0x7fffff ;
    }

    if( (1<=ch_sch_info->NUM_C[ch_num])&&(ch_sch_info->NUM_C[ch_num]<=11) ) {   //Layer-C exsits
        symbol_bit = MB86A22_calc_symbol_bit( ch_sch_info->MOD_C[ch_num] ) ;
        R = MB86A22_calc_data_ratio( ch_sch_info->RATE_C[ch_num] ) ;
        temp = (REAL32)(VBER_T_C)*carrier_num*symbol_bit*R*(REAL32)(ch_sch_info->NUM_C[ch_num])/sym_length/26.0 ;
        if( temp>16777215.0 ) temp = 16777215.0 ;
        if( temp<0 )          temp = 1.0 ;
        mon_tim_param->VBER_C_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->VBER_C_NUM = 0x7fffff ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Calc_SBER_NUM( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT32 SBER_T_A, UINT32 SBER_T_B, UINT32 SBER_T_C,
                                        UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    REAL32 TSP_num ;
    REAL32 temp ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_A[ch_num], ch_sch_info->RATE_A[ch_num] ) ;
    temp = (REAL32)(SBER_T_A)*TSP_num*(REAL32)(ch_sch_info->NUM_A[ch_num])/sym_length/2652.0 ;
    if( temp>65535.0 ) temp = 65535.0 ;
    if( temp<0 )       temp = 1.0 ;
    mon_tim_param->SBER_A_NUM = (UINT32)( temp ) ;

    if( (1<=ch_sch_info->NUM_B[ch_num])&&(ch_sch_info->NUM_B[ch_num]<=12) ) {   //Layer-B exsits
        TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_B[ch_num], ch_sch_info->RATE_B[ch_num] ) ;
        temp = (REAL32)(SBER_T_B)*TSP_num*(REAL32)(ch_sch_info->NUM_B[ch_num])/sym_length/2652.0 ;
        if( temp>65535.0 ) temp = 65535.0 ;
        if( temp<0 )       temp = 1.0 ;
        mon_tim_param->SBER_B_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->SBER_B_NUM = 0x7fff ;
    }

    if( (1<=ch_sch_info->NUM_C[ch_num])&&(ch_sch_info->NUM_C[ch_num]<=11) ) {   //Layer-C exsits
        TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_C[ch_num], ch_sch_info->RATE_C[ch_num] ) ;
        temp = (REAL32)(SBER_T_C)*TSP_num*(REAL32)(ch_sch_info->NUM_C[ch_num])/sym_length/2652.0 ;
        if( temp>65535.0 ) temp = 65535.0 ;
        if( temp<0 )       temp = 1.0 ;
        mon_tim_param->SBER_C_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->SBER_C_NUM = 0x7fff ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Calc_PBER_NUM( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT32 PBER_T_A, UINT32 PBER_T_B, UINT32 PBER_T_C,
                                        UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    REAL32 TSP_num ;
    REAL32 temp ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_A[ch_num], ch_sch_info->RATE_A[ch_num] ) ;
    temp = (REAL32)(PBER_T_A)*TSP_num*(REAL32)(ch_sch_info->NUM_A[ch_num])/sym_length/2652.0 ;
    if( temp>65535.0 ) temp = 65535.0 ;
    if( temp<0 )       temp = 1.0 ;
    mon_tim_param->PBER_A_NUM = (UINT32)( temp ) ;

    if( (1<=ch_sch_info->NUM_B[ch_num])&&(ch_sch_info->NUM_B[ch_num]<=12) ) {   //Layer-B exsits
        TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_B[ch_num], ch_sch_info->RATE_B[ch_num] ) ;
        temp = (REAL32)(PBER_T_B)*TSP_num*(REAL32)(ch_sch_info->NUM_B[ch_num])/sym_length/2652.0 ;
        if( temp>65535.0 ) temp = 65535.0 ;
        if( temp<0 )       temp = 1.0 ;
        mon_tim_param->PBER_B_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->PBER_B_NUM = 0x7fff ;
    }

    if( (1<=ch_sch_info->NUM_C[ch_num])&&(ch_sch_info->NUM_C[ch_num]<=11) ) {   //Layer-C exsits
        TSP_num = MB86A22_calc_TSP_num( ch_sch_info->MODE[ch_num], ch_sch_info->MOD_C[ch_num], ch_sch_info->RATE_C[ch_num] ) ;
        temp = (REAL32)(PBER_T_C)*TSP_num*(REAL32)(ch_sch_info->NUM_C[ch_num])/sym_length/2652.0 ;
        if( temp>65535.0 ) temp = 65535.0 ;
        if( temp<0 )       temp = 1.0 ;
        mon_tim_param->PBER_C_NUM = (UINT32)( temp ) ;
    }
    else {
        mon_tim_param->PBER_C_NUM = 0x7fff ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Calc_MER_SYMBOL( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT32 MER_T, UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    UINT32  MER_T_temp ;
    UINT8  mer_symbol_temp ;
    UINT8  loop ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    loop = 1 ;
    mer_symbol_temp = 0 ;
    while( loop==1 ) {
        MER_T_temp = (UINT32)( sym_length * (REAL32)(1<<mer_symbol_temp) ) ;    //calculation is different from C/N
        if( MER_T_temp>=MER_T ) loop = 0 ;
        else mer_symbol_temp++ ;
        if( mer_symbol_temp>7 ) loop = 0 ;
    }
    if(      mer_symbol_temp==0 ) mon_tim_param->MER_SYMBOL = mer_symbol_temp ;
    else if( mer_symbol_temp>7  ) mon_tim_param->MER_SYMBOL = 7 ;
    else if( MER_T_temp==MER_T  ) mon_tim_param->MER_SYMBOL = mer_symbol_temp ;
    else if( MER_T_temp>MER_T   ) mon_tim_param->MER_SYMBOL = mer_symbol_temp - 1 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Calc_CN_SYMBOL( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT32 CN_T, UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    UINT32  CN_T_temp ;
    UINT8  cn_symbol_temp ;
    UINT8  loop ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    loop = 1 ;
    cn_symbol_temp = 0 ;
    while( loop==1 ) {
        CN_T_temp = (UINT32)( sym_length * (REAL32)(1<<(cn_symbol_temp+1)) ) ;  //calculation is different from MER
        if( CN_T_temp>=CN_T ) loop = 0 ;
        else cn_symbol_temp++ ;
        if( cn_symbol_temp>15 ) loop = 0 ;
    }
    if(      cn_symbol_temp==0 ) mon_tim_param->CN_SYMBOL = cn_symbol_temp ;
    else if( cn_symbol_temp>15 ) mon_tim_param->CN_SYMBOL = 15 ;
    else if( CN_T_temp==CN_T   ) mon_tim_param->CN_SYMBOL = cn_symbol_temp ;
    else if( CN_T_temp>CN_T    ) mon_tim_param->CN_SYMBOL = cn_symbol_temp - 1 ;

    return MB86A2x_OK ;
}


UINT32 MB86A22_Calc_MER_Period( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    UINT32  MER_T ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    MER_T = (UINT32)( sym_length * (REAL32)(1<<mon_tim_param->MER_SYMBOL) ) ;   //calculation is different from C/N

    return MER_T ;
}


UINT32 MB86A22_Calc_CN_Period( MB86A2x_CH_SCH_INFO *ch_sch_info, UINT8 ch_num, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    REAL32 sym_length ;
    UINT32  CN_T ;

    sym_length = MB86A22_calc_symbol_length( ch_sch_info->MODE[ch_num], ch_sch_info->GUARD[ch_num] ) ;

    CN_T = (UINT32)( sym_length * (REAL32)(1<<(mon_tim_param->CN_SYMBOL+1)) ) ;

    return CN_T ;
}


REAL32 MB86A22_calc_symbol_length( UINT8 MODE, UINT8 GUARD )
{
    REAL32 sym_length_e= 0, sym_length ;
    REAL32 guard_r = 0;

    if(      MODE==0 ) sym_length_e = 0.252 ;
    else if( MODE==1 ) sym_length_e = 0.504 ;
    else if( MODE==2 ) sym_length_e = 1.008 ;

    if(      GUARD==0 ) guard_r = 1.0+1/4.0 ;
    else if( GUARD==1 ) guard_r = 1.0+1/8.0 ;
    else if( GUARD==2 ) guard_r = 1.0+1/16.0 ;
    else if( GUARD==3 ) guard_r = 1.0+1/32.0 ;

    sym_length = sym_length_e * guard_r ;

    return sym_length ;
}


REAL32 MB86A22_calc_carrier_num( UINT8 MODE )
{
    REAL32 carrier_num = 0;

    if(      MODE==0 ) carrier_num = 1248.0 ;
    else if( MODE==1 ) carrier_num = 2496.0 ;
    else if( MODE==2 ) carrier_num = 4992.0 ;

    return carrier_num ;
}


REAL32 MB86A22_calc_symbol_bit( UINT8 modulation )
{
    REAL32 symbol_bit = 0;

    if(      modulation==0 ) symbol_bit = 2.0 ;
    else if( modulation==1 ) symbol_bit = 2.0 ;
    else if( modulation==2 ) symbol_bit = 4.0 ;
    else if( modulation==3 ) symbol_bit = 6.0 ;

    return symbol_bit ;
}


REAL32 MB86A22_calc_data_ratio( UINT8 code_rate )
{
    REAL32 data_ratio = 0;

    if(      code_rate==0 ) data_ratio = 1.0 ;
    else if( code_rate==1 ) data_ratio = 2.0/3.0 ;
    else if( code_rate==2 ) data_ratio = 1.0 ;
    else if( code_rate==3 ) data_ratio = 1.0 ;
    else if( code_rate==4 ) data_ratio = 0.750 ;

    return data_ratio ;
}


REAL32 MB86A22_calc_TSP_num( UINT8 mode, UINT8 modulation, UINT8 code_rate )
{
    REAL32 temp1= 0, temp2 = 0;
    REAL32 TSP_num ;

    if( (modulation==0)||(modulation==1) ) {
        if(      code_rate==0 ) temp1 = 156.0 ;
        else if( code_rate==1 ) temp1 = 208.0 ;
        else if( code_rate==2 ) temp1 = 234.0 ;
        else if( code_rate==3 ) temp1 = 260.0 ;
        else if( code_rate==4 ) temp1 = 273.0 ;
    }
    else if( modulation==2 ) {
        if(      code_rate==0 ) temp1 = 312.0 ;
        else if( code_rate==1 ) temp1 = 416.0 ;
        else if( code_rate==2 ) temp1 = 468.0 ;
        else if( code_rate==3 ) temp1 = 520.0 ;
        else if( code_rate==4 ) temp1 = 546.0 ;
    }
    else if( modulation==3 ) {
        if(      code_rate==0 ) temp1 = 468.0 ;
        else if( code_rate==1 ) temp1 = 624.0 ;
        else if( code_rate==2 ) temp1 = 702.0 ;
        else if( code_rate==3 ) temp1 = 780.0 ;
        else if( code_rate==4 ) temp1 = 819.0 ;
    }

    if(      mode==0 ) temp2 = 1.0 ;
    else if( mode==1 ) temp2 = 2.0 ;
    else if( mode==2 ) temp2 = 4.0 ;

    TSP_num = temp1*temp2 ;

    return TSP_num ;
}


MB86A2x_ERR_MSG MB86A22_Set_RF( MB86A2x_NIM_PARAM *nim_param, MB86A2x_BANDSEL BANDSEL )
{
    //UINT32 Status ;
    // Editing for MxL
    //Status = MxL_TUNER_INIT( (UINT8)(nim_param->MB86A2x_Adr), &nim_param->tuner_param, BANDSEL ) ;    //tuner initialization
    //if( Status==I2C_NG ) return MB86A2x_ERR_RFTUNE ;
  
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IQ( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x09, (UINT8)(0x3a|((nim_param->IQ_pol)<<2)) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Init_LOCKNG_INFO( MB86A2x_LOCKNG_INFO *locking_info )
{
    locking_info->S8_REACH = 0 ;
    locking_info->S8_REACH_his = 0 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Init_SIG_QUAL_INFO( MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{

    sig_qual_info->VBER_A_current = -1.0 ;
    sig_qual_info->VBER_B_current = -1.0 ;
    sig_qual_info->VBER_C_current = -1.0 ;
    sig_qual_info->SBER_A_current = -1.0 ;
    sig_qual_info->SBER_B_current = -1.0 ;
    sig_qual_info->SBER_C_current = -1.0 ;
    sig_qual_info->PBER_A_current = -1.0 ;
    sig_qual_info->PBER_B_current = -1.0 ;
    sig_qual_info->PBER_C_current = -1.0 ;
    sig_qual_info->CN_current     = -1.0 ;
    sig_qual_info->MER_A_current  = -1.0 ;
    sig_qual_info->MER_B_current  = -1.0 ;
    sig_qual_info->MER_C_current  = -1.0 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Store_Channel_Info( UINT32 ch, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_CH_SCH_INFO *ch_sch_info )
{
    ch_sch_info->CH_num[69] = ch ;
    
    ch_sch_info->MODE[69] = tmcc_info->MODE_current ;
    ch_sch_info->GUARD[69] = tmcc_info->GUARD_current ;

    ch_sch_info->MOD_A[69] = tmcc_info->MOD_A_current ;
    ch_sch_info->MOD_B[69] = tmcc_info->MOD_B_current ;
    ch_sch_info->MOD_C[69] = tmcc_info->MOD_C_current ;

    ch_sch_info->RATE_A[69] = tmcc_info->RATE_A_current ;
    ch_sch_info->RATE_B[69] = tmcc_info->RATE_B_current ;
    ch_sch_info->RATE_C[69] = tmcc_info->RATE_C_current ;

    ch_sch_info->IL_A[69] = tmcc_info->IL_A_current ;
    ch_sch_info->IL_B[69] = tmcc_info->IL_B_current ;
    ch_sch_info->IL_C[69] = tmcc_info->IL_C_current ;

    ch_sch_info->NUM_A[69] = tmcc_info->NUM_A_current ;
    ch_sch_info->NUM_B[69] = tmcc_info->NUM_B_current ;
    ch_sch_info->NUM_C[69] = tmcc_info->NUM_C_current ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Load_CH_SCH_Result( MB86A2x_CH_SCH_INFO *ch_sch_info )
{
#if 0
    UINT8 i ;
    FILE *f1 ;
    f1 = fopen( "Channel_Search_Result.txt", "rt" ) ;

    fscanf( f1, "Detected_ch_num=%d\n", &(ch_sch_info->Detected_ch_num) ) ;
    for( i=0 ; i<ch_sch_info->Detected_ch_num ; i++ ) {
        fscanf( f1, "CH_num=%d,", &(ch_sch_info->CH_num[i]) ) ;
        fscanf( f1, "MODE=%d,", &(ch_sch_info->MODE[i]) ) ;
        fscanf( f1, "GUARD=%d,", &(ch_sch_info->GUARD[i]) ) ;

        fscanf( f1, "MOD_A=%d,", &(ch_sch_info->MOD_A[i]) ) ;
        fscanf( f1, "RATE_A=%d,", &(ch_sch_info->RATE_A[i]) ) ;
        fscanf( f1, "IL_A=%d,", &(ch_sch_info->IL_A[i]) ) ;
        fscanf( f1, "NUM_A=%d,", &(ch_sch_info->NUM_A[i]) ) ;

        fscanf( f1, "MOD_B=%d,", &(ch_sch_info->MOD_B[i]) ) ;
        fscanf( f1, "RATE_B=%d,", &(ch_sch_info->RATE_B[i]) ) ;
        fscanf( f1, "IL_B=%d,", &(ch_sch_info->IL_B[i]) ) ;
        fscanf( f1, "NUM_B=%d,", &(ch_sch_info->NUM_B[i]) ) ;

        fscanf( f1, "MOD_C=%d,", &(ch_sch_info->MOD_C[i]) ) ;
        fscanf( f1, "RATE_C=%d,", &(ch_sch_info->RATE_C[i]) ) ;
        fscanf( f1, "IL_C=%d,", &(ch_sch_info->IL_C[i]) ) ;
        fscanf( f1, "NUM_C=%d,", &(ch_sch_info->NUM_C[i]) ) ;

        fscanf( f1, "STR=%d\n", &(ch_sch_info->STR[i]) ) ;
    }
    fclose( f1 ) ;
#endif    

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_RECEP_SEG( MB86A2x_NIM_PARAM *nim_param, MB86A2x_LAYER_PARAM *recep_layer )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x39, (UINT8)(recep_layer->RECEP_LAYER) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_VBER_Timer( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xa7 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_A_NUM)&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xa8 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_A_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xa9 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(( mon_tim_param->VBER_A_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xaa ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_B_NUM)&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xab ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_B_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xac ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(( mon_tim_param->VBER_B_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xad ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_C_NUM)&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xae ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->VBER_C_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xaf ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(( mon_tim_param->VBER_C_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_SBER_Timer( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdc ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->SBER_A_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdd ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(( mon_tim_param->SBER_A_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xde ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->SBER_B_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdf ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(( mon_tim_param->SBER_B_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe0 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->SBER_C_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((mon_tim_param->SBER_C_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_PBER_Timer( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->PBER_A_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((mon_tim_param->PBER_A_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb4 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->PBER_B_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb5 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((mon_tim_param->PBER_B_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb6 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(((mon_tim_param->PBER_C_NUM)&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb7 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((mon_tim_param->PBER_C_NUM)&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_CN_Timer( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;
    UINT8  temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x45, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    temp = (temp&0xf0) | (UINT8)(mon_tim_param->CN_SYMBOL) ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x45, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_MER_Timer( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x51 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(mon_tim_param->MER_SYMBOL) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_SEQ_value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_LOCKNG_INFO *locking_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x0a, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    locking_info->STATE = read_val ;

    if( locking_info->STATE>=8 ) {
        if( locking_info->S8_REACH_his==0 ) {
            locking_info->S8_REACH = 1 ;
            locking_info->S8_REACH_his = 1 ;
        }
        else {
            locking_info->S8_REACH = 0 ;
        }
    }
    else {
        locking_info->S8_REACH = 0 ;
        locking_info->S8_REACH_his = 0 ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IF_level( MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_INFO *agc_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x41 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x05, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x42 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x05, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    agc_info->IF_INPUT_LEVEL = (read_val1<<8) + read_val2 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IFAGCDAC( MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_INFO *agc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x3a ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x05, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    agc_info->IFAGCDAC = read_val ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_RFAGCDAC( MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_INFO *agc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x04, 0x3b ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x05, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    agc_info->RFAGCDAC = read_val ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_STR_Freq_Error( MB86A2x_PARAM *param, MB86A2x_FREQ_ERR_INFO *freq_err_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 value ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x2b ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    value = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    if( value>=8388608 ) value = value - 16777216 ;

    freq_err_info->STR_freq_error_Hz = (SINT32)((REAL32)(param->pll_param.MASTER_CLK_FREQ_kHz)*((REAL32)(value)-(REAL32)(param->recov_param.STR_OFS))/ 16777.22) ;

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Get_STR_Register_Value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 value ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x28, 0x2b ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x29, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x2a, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x2b, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    value = (read_val1<<16) + (read_val2<<8) + read_val3 ;

    tmcc_info->STR_current = value ;
    
    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Get_CRR_Freq_Error( MB86A2x_PARAM *param, MB86A2x_FREQ_ERR_INFO *freq_err_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 value ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(param->nim_param.MB86A2x_Adr), 0x28, 0x21 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x29, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2a, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(param->nim_param.MB86A2x_Adr), 0x2b, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    value = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    if( value>=8388608 ) value = value - 16777216 ;

    freq_err_info->CRR_freq_error_kHz = (SINT32)((REAL32)(param->pll_param.MASTER_CLK_FREQ_kHz)/8.0*((REAL32)(value)-(REAL32)(param->recov_param.CRR_OFS))/ 2097152.0) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_Mode_Guard( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x07, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->MODE_current = (0x0c&read_val)>>2 ;
    tmcc_info->GUARD_current = 0x03&read_val ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Info( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Get_TMCC_Sys_Emerg( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Part_Recep( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Mod_Lay_A( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Mod_Lay_B( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Mod_Lay_C( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Rate_Lay_A( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Rate_Lay_B( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Rate_Lay_C( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_IL_Lay_A( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_IL_Lay_B( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_IL_Lay_C( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Num_Lay_A( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Num_Lay_B( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Get_TMCC_Num_Lay_C( nim_param, tmcc_info ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Sys_Emerg( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x84 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->SYSTEM_current = (read_val&0x60)>>5 ;
    tmcc_info->EMERG_current  =  read_val&0x01 ;
    tmcc_info->CNTDN_current = (read_val&0x1e)>>1 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Part_Recep( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x85 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->PARTIAL_current = (read_val&0x10)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Mod_Lay_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x86 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->MOD_A_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Mod_Lay_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8a ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->MOD_B_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Mod_Lay_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8e ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->MOD_C_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Rate_Lay_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x87 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->RATE_A_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Rate_Lay_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8b ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->RATE_B_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Rate_Lay_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8f ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->RATE_C_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_IL_Lay_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x88 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->IL_A_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_IL_Lay_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8c ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->IL_B_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_IL_Lay_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x90 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->IL_C_current = (read_val&0x70)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Num_Lay_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x89 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->NUM_A_current = (read_val&0xf0)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Num_Lay_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x8d ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->NUM_B_current = (read_val&0xf0)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_TMCC_Num_Lay_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x91 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    tmcc_info->NUM_C_current = (read_val&0xf0)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_VBER_A_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x06 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_VBER_B_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x05 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_VBER_C_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x03 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_VBER_ALL_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x53, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_VBER_Flag( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x54, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    sig_qual_info->VBER_A_update =  read_val&0x01 ;
    sig_qual_info->VBER_B_update = (read_val&0x02)>>1 ;
    sig_qual_info->VBER_C_update = (read_val&0x04)>>2 ;

    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_VBER_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x55, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x56, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x57, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->VBER_A_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->VBER_A_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_VBER_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x58, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x59, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x5a, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->VBER_B_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->VBER_B_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_VBER_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x5b, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x5c, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x5d, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->VBER_C_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->VBER_C_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_SBER_A_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x06 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_SBER_B_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x05 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_SBER_C_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x03 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_SBER_ALL_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x5f, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_SBER_Flag( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x60, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    sig_qual_info->SBER_A_update =  read_val&0x01 ;
    sig_qual_info->SBER_B_update = (read_val&0x02)>>1 ;
    sig_qual_info->SBER_C_update = (read_val&0x04)>>2 ;

    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_SBER_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x64, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x65, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x66, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->SBER_A_current = (REAL32)(error_num)/((REAL32)(mon_tim_param->SBER_A_NUM)*204.0*8.0) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_SBER_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x67, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x68, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x69, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->SBER_B_current = (REAL32)(error_num)/((REAL32)(mon_tim_param->SBER_B_NUM)*204.0*8.0) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_SBER_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2, read_val3 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6a, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6b, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6c, &read_val3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<16) + (read_val2<<8) + read_val3 ;
    sig_qual_info->SBER_C_current = (REAL32)(error_num)/((REAL32)(mon_tim_param->SBER_C_NUM)*204.0*8.0) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_PBER_A_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x01 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_PBER_B_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x02 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_PBER_C_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x04 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_PBER_ALL_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_PBER_Flag( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb8 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    sig_qual_info->PBER_A_update =  read_val&0x01 ;
    sig_qual_info->PBER_B_update = (read_val&0x02)>>1 ;
    sig_qual_info->PBER_C_update = (read_val&0x04)>>2 ;

    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_PBER_A( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xb9 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xba ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<8) + read_val2 ;
    sig_qual_info->PBER_A_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->PBER_A_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_PBER_B( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xbb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xbc ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<8) + read_val2 ;
    sig_qual_info->PBER_B_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->PBER_B_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_PBER_C( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 error_num ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xbd ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xbe ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    error_num = (read_val1<<8) + read_val2 ;
    sig_qual_info->PBER_C_current = (REAL32)(error_num)/(REAL32)(mon_tim_param->PBER_C_NUM) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_CN_Counter( MB86A2x_NIM_PARAM *nim_param, MB86A2x_MON_TIM_PARAM *mon_tim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x45, (UINT8)(0x10|mon_tim_param->CN_SYMBOL) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x45, (UINT8)(0x00|mon_tim_param->CN_SYMBOL) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_CN_Flag( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x45, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    sig_qual_info->CN_update = (read_val & 0x40)>>6 ; ;

    return MB86A2x_OK ;

}


MB86A2x_ERR_MSG MB86A22_Get_CN_Value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 cn_read_vale ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x46, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x47, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    cn_read_vale = (read_val1<<8) + read_val2 ;
    sig_qual_info->CN_current = (REAL32)(MB86A22_CNval_to_CN10(cn_read_vale))/10.0 ;

    return MB86A2x_OK ;
}


UINT32 MB86A22_CNval_to_CN10( UINT32 cn_read_value )
{
    UINT32 ret ;
    SINT32 A= 0, B= 0, C= 0 ;

    if(      cn_read_value>=18187 ) { A= 10 ; B=1461 ; C=18187 ; }
    else if( cn_read_value>=16534 ) { A= 20 ; B=1653 ; C=16534 ; }
    else if( cn_read_value>=14823 ) { A= 30 ; B=1711 ; C=14823 ; }
    else if( cn_read_value>=13161 ) { A= 40 ; B=1662 ; C=13161 ; }
    else if( cn_read_value>=11622 ) { A= 50 ; B=1539 ; C=11622 ; }
    else if( cn_read_value>=10279 ) { A= 60 ; B=1343 ; C=10279 ; }
    else if( cn_read_value>= 9089 ) { A= 70 ; B=1190 ; C= 9089 ; }
    else if( cn_read_value>= 8042 ) { A= 80 ; B=1047 ; C= 8042 ; }
    else if( cn_read_value>= 7137 ) { A= 90 ; B= 905 ; C= 7137 ; }
    else if( cn_read_value>= 6342 ) { A=100 ; B= 795 ; C= 6342 ; }
    else if( cn_read_value>= 5641 ) { A=110 ; B= 701 ; C= 5641 ; }
    else if( cn_read_value>= 5030 ) { A=120 ; B= 611 ; C= 5030 ; }
    else if( cn_read_value>= 4474 ) { A=130 ; B= 556 ; C= 4474 ; }
    else if( cn_read_value>= 3988 ) { A=140 ; B= 486 ; C= 3988 ; }
    else if( cn_read_value>= 3556 ) { A=150 ; B= 432 ; C= 3556 ; }
    else if( cn_read_value>= 3180 ) { A=160 ; B= 376 ; C= 3180 ; }
    else if( cn_read_value>= 2841 ) { A=170 ; B= 339 ; C= 2841 ; }
    else if( cn_read_value>= 2541 ) { A=180 ; B= 300 ; C= 2541 ; }
    else if( cn_read_value>= 2276 ) { A=190 ; B= 265 ; C= 2276 ; }
    else if( cn_read_value>= 2038 ) { A=200 ; B= 238 ; C= 2038 ; }
    else if( cn_read_value>= 1800 ) { A=210 ; B= 238 ; C= 1800 ; }
    else if( cn_read_value>= 1625 ) { A=220 ; B= 175 ; C= 1625 ; }
    else if( cn_read_value>= 1462 ) { A=230 ; B= 163 ; C= 1462 ; }
    else if( cn_read_value>= 1324 ) { A=240 ; B= 138 ; C= 1324 ; }
    else if( cn_read_value>= 1175 ) { A=250 ; B= 149 ; C= 1175 ; }
    else if( cn_read_value>= 1063 ) { A=260 ; B= 112 ; C= 1063 ; }
    else if( cn_read_value>=  980 ) { A=270 ; B=  83 ; C=  980 ; }
    else if( cn_read_value>=  907 ) { A=280 ; B=  73 ; C=  907 ; }
    else if( cn_read_value>=  840 ) { A=290 ; B=  67 ; C=  840 ; }
    else if( cn_read_value>=  788 ) { A=300 ; B=  52 ; C=  788 ; }

    if(      cn_read_value>=19648 ) ret = 0 ;
    else if( cn_read_value<   788 ) ret = 300 ;
    else                            ret = (UINT32)( A-(10*((SINT32)(cn_read_value)-C)-B/2)/B) ;

    return ret ;
}


MB86A2x_ERR_MSG MB86A22_Reset_MER_Counter( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x50 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x03 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x02 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_MER_Flag( MB86A2x_NIM_PARAM *nim_param, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val ;
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x5b ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    sig_qual_info->MER_update = read_val & 0x01 ;

    return MB86A2x_OK ;

}


MB86A2x_ERR_MSG MB86A22_Get_MER_A_Value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 mer_read_value ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x52 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x53 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    mer_read_value = (read_val1<<8) + read_val2 ;
    if( tmcc_info->MODE_current==0 ) mer_read_value = mer_read_value * 2 ;

    if(      tmcc_info->MOD_A_current==0 ) sig_qual_info->MER_A_current = (REAL32)(MB86A22_MERval_to_CN10_DQPSK( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_A_current==1 ) sig_qual_info->MER_A_current = (REAL32)(MB86A22_MERval_to_CN10_QPSK(  mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_A_current==2 ) sig_qual_info->MER_A_current = (REAL32)(MB86A22_MERval_to_CN10_16QAM( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_A_current==3 ) sig_qual_info->MER_A_current = (REAL32)(MB86A22_MERval_to_CN10_64QAM( mer_read_value ))/10.0 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_MER_B_Value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 mer_read_value ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x55 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x56 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    mer_read_value = (read_val1<<8) + read_val2 ;
    if( tmcc_info->MODE_current==0 ) mer_read_value = mer_read_value * 2 ;

    if(      tmcc_info->MOD_B_current==0 ) sig_qual_info->MER_B_current = (REAL32)(MB86A22_MERval_to_CN10_DQPSK( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_B_current==1 ) sig_qual_info->MER_B_current = (REAL32)(MB86A22_MERval_to_CN10_QPSK(  mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_B_current==2 ) sig_qual_info->MER_B_current = (REAL32)(MB86A22_MERval_to_CN10_16QAM( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_B_current==3 ) sig_qual_info->MER_B_current = (REAL32)(MB86A22_MERval_to_CN10_64QAM( mer_read_value ))/10.0 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_MER_C_Value( MB86A2x_NIM_PARAM *nim_param, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_SIG_QUAL_INFO *sig_qual_info )
{
    UINT8  read_val1, read_val2 ;
    UINT32 Status ;
    UINT32 mer_read_value ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x58 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0x59 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &read_val2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    mer_read_value = (read_val1<<8) + read_val2 ;
    if( tmcc_info->MODE_current==0 ) mer_read_value = mer_read_value * 2 ;

    if(      tmcc_info->MOD_C_current==0 ) sig_qual_info->MER_C_current = (REAL32)(MB86A22_MERval_to_CN10_DQPSK( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_C_current==1 ) sig_qual_info->MER_C_current = (REAL32)(MB86A22_MERval_to_CN10_QPSK(  mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_C_current==2 ) sig_qual_info->MER_C_current = (REAL32)(MB86A22_MERval_to_CN10_16QAM( mer_read_value ))/10.0 ;
    else if( tmcc_info->MOD_C_current==3 ) sig_qual_info->MER_C_current = (REAL32)(MB86A22_MERval_to_CN10_64QAM( mer_read_value ))/10.0 ;

    return MB86A2x_OK ;
}


UINT32 MB86A22_MERval_to_CN10_DQPSK( UINT32 mer_read_value )
{
    SINT32 ret ;
    SINT32 A= 0, B= 0, C= 0 ;

    if(      mer_read_value>=16293 ) { A= 10 ; B=  50 ; C=16293 ; }
    else if( mer_read_value>=16177 ) { A= 20 ; B= 116 ; C=16177 ; }
    else if( mer_read_value>=15994 ) { A= 30 ; B= 183 ; C=15994 ; }
    else if( mer_read_value>=15650 ) { A= 40 ; B= 344 ; C=15650 ; }
    else if( mer_read_value>=15091 ) { A= 50 ; B= 559 ; C=15091 ; }
    else if( mer_read_value>=14265 ) { A= 60 ; B= 826 ; C=14265 ; }
    else if( mer_read_value>=13142 ) { A= 70 ; B=1123 ; C=13142 ; }
    else if( mer_read_value>=11760 ) { A= 80 ; B=1382 ; C=11760 ; }
    else if( mer_read_value>=10202 ) { A= 90 ; B=1558 ; C=10202 ; }
    else if( mer_read_value>= 8607 ) { A=100 ; B=1595 ; C= 8607 ; }
    else if( mer_read_value>= 7090 ) { A=110 ; B=1517 ; C= 7090 ; }
    else if( mer_read_value>= 5744 ) { A=120 ; B=1346 ; C= 5744 ; }
    else if( mer_read_value>= 4608 ) { A=130 ; B=1136 ; C= 4608 ; }
    else if( mer_read_value>= 3685 ) { A=140 ; B= 923 ; C= 3685 ; }
    else if( mer_read_value>= 2950 ) { A=150 ; B= 735 ; C= 2950 ; }
    else if( mer_read_value>= 2365 ) { A=160 ; B= 585 ; C= 2365 ; }
    else if( mer_read_value>= 1903 ) { A=170 ; B= 462 ; C= 1903 ; }
    else if( mer_read_value>= 1538 ) { A=180 ; B= 365 ; C= 1538 ; }
    else if( mer_read_value>= 1262 ) { A=190 ; B= 276 ; C= 1262 ; }
    else if( mer_read_value>= 1028 ) { A=200 ; B= 234 ; C= 1028 ; }
    else if( mer_read_value>=  822 ) { A=210 ; B= 206 ; C=  822 ; }
    else if( mer_read_value>=  683 ) { A=220 ; B= 139 ; C=  683 ; }
    else if( mer_read_value>=  580 ) { A=230 ; B= 103 ; C=  580 ; }
    else if( mer_read_value>=  487 ) { A=240 ; B=  93 ; C=  487 ; }
    else if( mer_read_value>=  401 ) { A=250 ; B=  86 ; C=  401 ; }
    else if( mer_read_value>=  348 ) { A=260 ; B=  53 ; C=  348 ; }
    else if( mer_read_value>=  312 ) { A=270 ; B=  36 ; C=  312 ; }
    else if( mer_read_value>=  279 ) { A=280 ; B=  33 ; C=  279 ; }
    else if( mer_read_value>=  254 ) { A=290 ; B=  25 ; C=  254 ; }
    else if( mer_read_value>=  235 ) { A=300 ; B=  19 ; C=  235 ; }

    if(      mer_read_value>=16343 ) ret = 0 ;
    else if( mer_read_value<   235 ) ret = 300 ;
    else                    ret = (UINT32)( A-(10*((SINT32)(mer_read_value)-C)-B/2)/B) ;
    return ret ;
}


UINT32 MB86A22_MERval_to_CN10_QPSK( UINT32 mer_read_value )
{
    SINT32 ret ;
    SINT32 A= 0, B= 0, C= 0 ;

    if(      mer_read_value>=10483 ) { A= 10 ; B= 588 ; C=10483 ; }
    else if( mer_read_value>= 9910 ) { A= 20 ; B= 573 ; C= 9910 ; }
    else if( mer_read_value>= 9343 ) { A= 30 ; B= 567 ; C= 9343 ; }
    else if( mer_read_value>= 8333 ) { A= 40 ; B=1010 ; C= 8333 ; }
    else if( mer_read_value>= 7446 ) { A= 50 ; B= 887 ; C= 7446 ; }
    else if( mer_read_value>= 6510 ) { A= 60 ; B= 936 ; C= 6510 ; }
    else if( mer_read_value>= 5555 ) { A= 70 ; B= 955 ; C= 5555 ; }
    else if( mer_read_value>= 4647 ) { A= 80 ; B= 908 ; C= 4647 ; }
    else if( mer_read_value>= 3814 ) { A= 90 ; B= 833 ; C= 3814 ; }
    else if( mer_read_value>= 3087 ) { A=100 ; B= 727 ; C= 3087 ; }
    else if( mer_read_value>= 2474 ) { A=110 ; B= 613 ; C= 2474 ; }
    else if( mer_read_value>= 1975 ) { A=120 ; B= 499 ; C= 1975 ; }
    else if( mer_read_value>= 1574 ) { A=130 ; B= 401 ; C= 1574 ; }
    else if( mer_read_value>= 1253 ) { A=140 ; B= 321 ; C= 1253 ; }
    else if( mer_read_value>=  998 ) { A=150 ; B= 255 ; C=  998 ; }
    else if( mer_read_value>=  797 ) { A=160 ; B= 201 ; C=  797 ; }
    else if( mer_read_value>=  637 ) { A=170 ; B= 160 ; C=  637 ; }
    else if( mer_read_value>=  509 ) { A=180 ; B= 128 ; C=  509 ; }
    else if( mer_read_value>=  411 ) { A=190 ; B=  98 ; C=  411 ; }
    else if( mer_read_value>=  326 ) { A=200 ; B=  85 ; C=  326 ; }
    else if( mer_read_value>=  254 ) { A=210 ; B=  72 ; C=  254 ; }
    else if( mer_read_value>=  205 ) { A=220 ; B=  49 ; C=  205 ; }
    else if( mer_read_value>=  167 ) { A=230 ; B=  38 ; C=  167 ; }
    else if( mer_read_value>=  135 ) { A=240 ; B=  32 ; C=  135 ; }
    else if( mer_read_value>=  106 ) { A=250 ; B=  29 ; C=  106 ; }
    else if( mer_read_value>=   86 ) { A=260 ; B=  20 ; C=   86 ; }
    else if( mer_read_value>=   72 ) { A=270 ; B=  14 ; C=   72 ; }
    else if( mer_read_value>=   61 ) { A=280 ; B=  11 ; C=   61 ; }
    else if( mer_read_value>=   52 ) { A=290 ; B=   9 ; C=   52 ; }
    else if( mer_read_value>=   45 ) { A=300 ; B=   7 ; C=   45 ; }

    if(      mer_read_value>=11071 ) ret = 0 ;
    else if( mer_read_value<    45 ) ret = 300 ;
    else                    ret = (UINT32)( A-(10*((SINT32)(mer_read_value)-C)-B/2)/B) ;
    return ret ;
}


UINT32 MB86A22_MERval_to_CN10_16QAM( UINT32 mer_read_value )
{
    SINT32 ret ;
    SINT32 A= 0, B= 0, C= 0 ;

    if(      mer_read_value>=20387 ) { A= 10 ; B= 374 ; C=20387 ; }
    else if( mer_read_value>=19995 ) { A= 20 ; B= 392 ; C=19995 ; }
    else if( mer_read_value>=19527 ) { A= 30 ; B= 468 ; C=19527 ; }
    else if( mer_read_value>=19045 ) { A= 40 ; B= 482 ; C=19045 ; }
    else if( mer_read_value>=18500 ) { A= 50 ; B= 545 ; C=18500 ; }
    else if( mer_read_value>=17985 ) { A= 60 ; B= 515 ; C=17985 ; }
    else if( mer_read_value>=17417 ) { A= 70 ; B= 568 ; C=17417 ; }
    else if( mer_read_value>=16798 ) { A= 80 ; B= 619 ; C=16798 ; }
    else if( mer_read_value>=15988 ) { A= 90 ; B= 810 ; C=15988 ; }
    else if( mer_read_value>=14985 ) { A=100 ; B=1003 ; C=14985 ; }
    else if( mer_read_value>=13754 ) { A=110 ; B=1231 ; C=13754 ; }
    else if( mer_read_value>=12328 ) { A=120 ; B=1426 ; C=12328 ; }
    else if( mer_read_value>=10769 ) { A=130 ; B=1559 ; C=10769 ; }
    else if( mer_read_value>= 9168 ) { A=140 ; B=1601 ; C= 9168 ; }
    else if( mer_read_value>= 7637 ) { A=150 ; B=1531 ; C= 7637 ; }
    else if( mer_read_value>= 6225 ) { A=160 ; B=1382 ; C= 6255 ; }
    else if( mer_read_value>= 5070 ) { A=170 ; B=1185 ; C= 5070 ; }
    else if( mer_read_value>= 4078 ) { A=180 ; B= 992 ; C= 4078 ; }
    else if( mer_read_value>= 3280 ) { A=190 ; B= 798 ; C= 3280 ; }
    else if( mer_read_value>= 2626 ) { A=200 ; B= 654 ; C= 2626 ; }
    else if( mer_read_value>= 2043 ) { A=210 ; B= 583 ; C= 2043 ; }
    else if( mer_read_value>= 1659 ) { A=220 ; B= 384 ; C= 1659 ; }
    else if( mer_read_value>= 1348 ) { A=230 ; B= 311 ; C= 1348 ; }
    else if( mer_read_value>= 1094 ) { A=240 ; B= 254 ; C= 1094 ; }
    else if( mer_read_value>=  865 ) { A=250 ; B= 229 ; C=  865 ; }
    else if( mer_read_value>=  702 ) { A=260 ; B= 163 ; C=  702 ; }
    else if( mer_read_value>=  590 ) { A=270 ; B= 112 ; C=  590 ; }
    else if( mer_read_value>=  502 ) { A=280 ; B=  88 ; C=  502 ; }
    else if( mer_read_value>=  430 ) { A=290 ; B=  72 ; C=  430 ; }
    else if( mer_read_value>=  374 ) { A=300 ; B=  56 ; C=  374 ; }

    if(      mer_read_value>=20761 ) ret = 0 ;
    else if( mer_read_value<   374 ) ret = 300 ;
    else                    ret = (UINT32)( A-(10*((SINT32)(mer_read_value)-C)-B/2)/B) ;
    return ret ;
}


UINT32 MB86A22_MERval_to_CN10_64QAM( UINT32 mer_read_value )
{
    SINT32 ret ;
    SINT32 A= 0, B= 0, C= 0 ;

    if(      mer_read_value>=15314 ) { A= 10 ; B=   9 ; C=15314 ; }
    else if( mer_read_value>=15245 ) { A= 20 ; B=  69 ; C=15245 ; }
    else if( mer_read_value>=15214 ) { A= 30 ; B=  31 ; C=15214 ; }
    else if( mer_read_value>=15166 ) { A= 40 ; B=  48 ; C=15166 ; }
    else if( mer_read_value>=15128 ) { A= 50 ; B=  38 ; C=15128 ; }
    else if( mer_read_value>=15074 ) { A= 60 ; B=  54 ; C=15074 ; }
    else if( mer_read_value>=15045 ) { A= 70 ; B=  29 ; C=15045 ; }
    else if( mer_read_value>=14996 ) { A= 80 ; B=  49 ; C=14996 ; }
    else if( mer_read_value>=14958 ) { A= 90 ; B=  38 ; C=14958 ; }
    else if( mer_read_value>=14915 ) { A=100 ; B=  43 ; C=14915 ; }
    else if( mer_read_value>=14871 ) { A=110 ; B=  44 ; C=14871 ; }
    else if( mer_read_value>=14812 ) { A=120 ; B=  59 ; C=14812 ; }
    else if( mer_read_value>=14715 ) { A=130 ; B=  97 ; C=14715 ; }
    else if( mer_read_value>=14535 ) { A=140 ; B= 180 ; C=14535 ; }
    else if( mer_read_value>=14209 ) { A=150 ; B= 326 ; C=14209 ; }
    else if( mer_read_value>=13666 ) { A=160 ; B= 543 ; C=13666 ; }
    else if( mer_read_value>=12875 ) { A=170 ; B= 791 ; C=12875 ; }
    else if( mer_read_value>=11840 ) { A=180 ; B=1035 ; C=11840 ; }
    else if( mer_read_value>=10607 ) { A=190 ; B=1233 ; C=10607 ; }
    else if( mer_read_value>= 9229 ) { A=200 ; B=1378 ; C= 9229 ; }
    else if( mer_read_value>= 7669 ) { A=210 ; B=1560 ; C= 7669 ; }
    else if( mer_read_value>= 6444 ) { A=220 ; B=1225 ; C= 6444 ; }
    else if( mer_read_value>= 5339 ) { A=230 ; B=1105 ; C= 5339 ; }
    else if( mer_read_value>= 4378 ) { A=240 ; B= 961 ; C= 4378 ; }
    else if( mer_read_value>= 3480 ) { A=250 ; B= 898 ; C= 3480 ; }
    else if( mer_read_value>= 2826 ) { A=260 ; B= 654 ; C= 2826 ; }
    else if( mer_read_value>= 2391 ) { A=270 ; B= 435 ; C= 2391 ; }
    else if( mer_read_value>= 2027 ) { A=280 ; B= 364 ; C= 2027 ; }
    else if( mer_read_value>= 1751 ) { A=290 ; B= 276 ; C= 1751 ; }
    else if( mer_read_value>= 1519 ) { A=300 ; B= 232 ; C= 1519 ; }

    if(      mer_read_value>=15323 ) ret = 0 ;
    else if( mer_read_value<  1519 ) ret = 300 ;
    else                    ret = (UINT32)( A-(10*((SINT32)(mer_read_value)-C)-B/2)/B) ;
    return ret ;
}


MB86A2x_ERR_MSG MB86A22_Set_Init_CH_SCH( MB86A2x_PARAM *param, MB86A2x_CH_SCH_INFO *ch_sch_info, int RS_EN )
{
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Set_Init( param, RS_EN ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_Monitor_Timer( param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Reset_Monitor_Timer( param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_IRQ_All_Off( &param->nim_param ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ERR_MSG = MB86A22_Set_IRQ_CH_SCH_END_Off( &param->nim_param, 0 ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Init_CH_SCH_Result( MB86A2x_CH_SCH_INFO *ch_sch_info )
{
    UINT32 i ;

    ch_sch_info->Detected_ch_num = 0 ;

    for( i=0 ; i<70 ; i++ ) {
        ch_sch_info->CH_num[i] = 0 ;
        ch_sch_info->MODE[i] = 7 ;
        ch_sch_info->GUARD[i] = 7 ;
        ch_sch_info->MOD_A[i] = 7 ;
        ch_sch_info->MOD_B[i] = 7 ;
        ch_sch_info->MOD_C[i] = 7 ;
        ch_sch_info->RATE_A[i] = 7 ;
        ch_sch_info->RATE_B[i] = 7 ;
        ch_sch_info->RATE_C[i] = 7 ;
        ch_sch_info->IL_A[i] = 32 ;
        ch_sch_info->IL_B[i] = 32 ;
        ch_sch_info->IL_C[i] = 32 ;
        ch_sch_info->NUM_A[i] = 15 ;
        ch_sch_info->NUM_B[i] = 15 ;
        ch_sch_info->NUM_C[i] = 15 ;
        ch_sch_info->STR[i] = 0 ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Start_CH_SCH( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xe7, 0x01 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xe6, 0x20 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xe6, 0x21 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_CH_SCH_Status( MB86A2x_NIM_PARAM *nim_param, MB86A2x_CH_SCH_INFO *ch_sch_info )
{
    MB86A2x_ERR_MSG ERR_MSG ;
    UINT8  chend;

    ERR_MSG = MB86A22_Get_IRQ_CH_SCH_END( nim_param, &chend ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    ch_sch_info->CH_SCH_status = chend ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_CH_SCH_Result( MB86A2x_NIM_PARAM *nim_param, MB86A2x_CH_SCH_INFO *ch_sch_info )
{
    UINT32 Status ;
    UINT8  read_val ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xe8, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xe9, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    ch_sch_info->CH_SCH_result = read_val &0x01 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Store_CH_SCH_Result( UINT32 ch, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_CH_SCH_INFO *ch_sch_info )
{
    ch_sch_info->CH_num[ch_sch_info->Detected_ch_num] = ch ;
    
    ch_sch_info->MODE[ch_sch_info->Detected_ch_num] = tmcc_info->MODE_current ;
    ch_sch_info->GUARD[ch_sch_info->Detected_ch_num] = tmcc_info->GUARD_current ;

    ch_sch_info->MOD_A[ch_sch_info->Detected_ch_num] = tmcc_info->MOD_A_current ;
    ch_sch_info->MOD_B[ch_sch_info->Detected_ch_num] = tmcc_info->MOD_B_current ;
    ch_sch_info->MOD_C[ch_sch_info->Detected_ch_num] = tmcc_info->MOD_C_current ;

    ch_sch_info->RATE_A[ch_sch_info->Detected_ch_num] = tmcc_info->RATE_A_current ;
    ch_sch_info->RATE_B[ch_sch_info->Detected_ch_num] = tmcc_info->RATE_B_current ;
    ch_sch_info->RATE_C[ch_sch_info->Detected_ch_num] = tmcc_info->RATE_C_current ;

    ch_sch_info->IL_A[ch_sch_info->Detected_ch_num] = tmcc_info->IL_A_current ;
    ch_sch_info->IL_B[ch_sch_info->Detected_ch_num] = tmcc_info->IL_B_current ;
    ch_sch_info->IL_C[ch_sch_info->Detected_ch_num] = tmcc_info->IL_C_current ;

    ch_sch_info->NUM_A[ch_sch_info->Detected_ch_num] = tmcc_info->NUM_A_current ;
    ch_sch_info->NUM_B[ch_sch_info->Detected_ch_num] = tmcc_info->NUM_B_current ;
    ch_sch_info->NUM_C[ch_sch_info->Detected_ch_num] = tmcc_info->NUM_C_current ;

    ch_sch_info->STR[ch_sch_info->Detected_ch_num] = tmcc_info->STR_current ;

    ch_sch_info->Detected_ch_num++ ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Save_CH_SCH_Result( MB86A2x_CH_SCH_INFO *ch_sch_info )
{

    UINT8 i ;
    FILE *f1 ;
    f1 = fopen( "Channel_Search_Result.txt", "wt" ) ;

    fprintf( f1, "Detected_ch_num=%d\n", ch_sch_info->Detected_ch_num ) ;
    for( i=0 ; i<ch_sch_info->Detected_ch_num ; i++ ) {
        fprintf( f1, "CH_num=%d,", ch_sch_info->CH_num[i] ) ;
        fprintf( f1, "MODE=%d,", ch_sch_info->MODE[i] ) ;
        fprintf( f1, "GUARD=%d,", ch_sch_info->GUARD[i] ) ;

        fprintf( f1, "MOD_A=%d,", ch_sch_info->MOD_A[i] ) ;
        fprintf( f1, "RATE_A=%d,", ch_sch_info->RATE_A[i] ) ;
        fprintf( f1, "IL_A=%d,", ch_sch_info->IL_A[i] ) ;
        fprintf( f1, "NUM_A=%d,", ch_sch_info->NUM_A[i] ) ;

        fprintf( f1, "MOD_B=%d,", ch_sch_info->MOD_B[i] ) ;
        fprintf( f1, "RATE_B=%d,", ch_sch_info->RATE_B[i] ) ;
        fprintf( f1, "IL_B=%d,", ch_sch_info->IL_B[i] ) ;
        fprintf( f1, "NUM_B=%d,", ch_sch_info->NUM_B[i] ) ;

        fprintf( f1, "MOD_C=%d,", ch_sch_info->MOD_C[i] ) ;
        fprintf( f1, "RATE_C=%d,", ch_sch_info->RATE_C[i] ) ;
        fprintf( f1, "IL_C=%d,", ch_sch_info->IL_C[i] ) ;
        fprintf( f1, "NUM_C=%d,", ch_sch_info->NUM_C[i] ) ;

        fprintf( f1, "STR=%d", ch_sch_info->STR[i] ) ;

        fprintf( f1, "\n" ) ;
    }
    fclose( f1 ) ;


    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Power_Mode( MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_PARAM *agc_param ) {

    MB86A2x_ERR_MSG ERR_MSG ;

    if( nim_param->POWER_DOWN==0 ) {
        ERR_MSG = MB86A22_Set_Normal_Power( nim_param, agc_param ) ;    //Power Down Setting
        if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;
    }
    else {
        MB86A22_Set_Low_Power( nim_param ) ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Low_Power( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xd0, 0x33 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x08, 0x01 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x73, 0x01 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_Normal_Power( MB86A2x_NIM_PARAM *nim_param, MB86A2x_AGC_PARAM *agc_param )
{
    UINT32 Status ;
    UINT8  write_val ;
    UINT8  RFAGCDAC_OFF ;
    UINT8  ADCQ_OFF ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x73, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x08, 0x00 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( nim_param->tuner_param.IF_freq_kHz==MB86A2x_IF_FREQ_000_kHz ) {
        ADCQ_OFF = 0 ;
    }
    else {
        ADCQ_OFF = 1 ;
    }
    if( agc_param->RFAGC_ON==1 ) {
        RFAGCDAC_OFF = 0 ;
    }
    else {
        RFAGCDAC_OFF = 1 ;
    }
    write_val = 0x00 | (RFAGCDAC_OFF<<5) | (ADCQ_OFF<<2);

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xd0, write_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_GPIO_Mode( MB86A2x_NIM_PARAM *nim_param, UINT8 Port_No, UINT8 Port_mode )
{
    UINT32 Status ;
    UINT8  read_val ;
    UINT8  write_val ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( Port_No==0 ) {
        write_val = ((read_val & 0xfe) | (Port_mode)) ;
    }
    else if( Port_No==1 ) {
        write_val = ((read_val & 0xfd) | (Port_mode<<1)) ;
    }
    else if( Port_No==2 ) {
        write_val = ((read_val & 0xfb) | (Port_mode<<2)) ;
    }
    else {
        write_val = ((read_val & 0xf7) | (Port_mode<<3)) ;
    }

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, write_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_GPIO_Mode( MB86A2x_NIM_PARAM *nim_param, UINT8 Port_No, UINT8 *Port_mode )
{
    UINT32 Status ;
    UINT8  read_val ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( Port_No==0 ) {
        *Port_mode = read_val&0x01 ;
    }
    else if( Port_No==1 ) {
        *Port_mode = (read_val&0x02)>>1 ;
    }
    else if( Port_No==2 ) {
        *Port_mode = (read_val&0x04)>>2 ;
    }
    else {
        *Port_mode = (read_val&0x08)>>3 ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_GPIO_Value( MB86A2x_NIM_PARAM *nim_param, UINT8 Port_No, UINT8 Port_val )
{
    UINT32 Status ;
    UINT8  read_val ;
    UINT8  write_val ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xeb, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( Port_No==0 ) {
        write_val = ((read_val & 0x0e) | (Port_val)) ;
    }
    else if( Port_No==1 ) {
        write_val = ((read_val & 0x0d) | (Port_val<<1)) ;
    }
    else if( Port_No==2 ) {
        write_val = ((read_val & 0x0b) | (Port_val<<2)) ;
    }
    else {
        write_val = ((read_val & 0x07) | (Port_val<<3)) ;
    }

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xeb, write_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_GPIO_Value( MB86A2x_NIM_PARAM *nim_param, UINT8 Port_No, UINT8 *Port_val )
{
    UINT32 Status ;
    UINT8  read_val ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xeb, &read_val ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    if( Port_No==0 ) {
        *Port_val = read_val&0x01 ;
    }
    else if( Port_No==1 ) {
        *Port_val = (read_val&0x02)>>1 ;
    }
    else if( Port_No==2 ) {
        *Port_val = (read_val&0x04)>>2 ;
    }
    else {
        *Port_val = (read_val&0x08)>>3 ;
    }

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_All_Off( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x35 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0xff ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, 0x07 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, 0x03 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = 0xf0 | temp ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_TMCC_EMG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;
    MB86A2x_ERR_MSG ERR_MSG ;

    ERR_MSG = MB86A22_Set_IRQ_TMCC_EMG_INV( nim_param, 0 ) ;
    if( ERR_MSG==MB86A2x_ERR_INIT ) return MB86A2x_ERR_INIT ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x35 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0x03)|(disable<<2) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x35 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_TMCC_EMG( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x04 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0x03 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_TMCC_EMG_INV( MB86A2x_NIM_PARAM *nim_param, UINT8 inv )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x38 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, (UINT8)((inv<<4)|0x01) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_TMCC_EMG( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_tmcc_emg )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x63, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_tmcc_emg = (temp&0x04)>>2 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_TMCC_CNTDN_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x35 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0x05)|(disable<<1) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x35 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_TMCC_CNTDN( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x02 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0x05 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6d, 0x36 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x6e, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_TMCC_CNTDN( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_tmcc_cntdn )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x63, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_tmcc_cntdn = (temp&0x02)>>1 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_A_FIN_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfe)|disable ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_B_FIN_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfd)|(disable<<1) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_C_FIN_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfb)|(disable<<2) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_A_TH_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xf7)|(disable<<3) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_THRES_SBER_A( MB86A2x_NIM_PARAM *nim_param, UINT32 thres_a )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_a&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe3 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_a&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe4 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(thres_a&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_SBER_THRES_A( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_sber_thres_a )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6f, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_sber_thres_a = temp&0x01 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_B_TH_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xef)|(disable<<4) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_THRES_SBER_B( MB86A2x_NIM_PARAM *nim_param, UINT32 thres_b )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe5 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_b&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe6 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_b&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe7 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(thres_b&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_SBER_THRES_B( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_sber_thres_b )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6f, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_sber_thres_b = (temp&0x02)>>1 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_SBER_C_TH_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xdf)|(disable<<5) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xda ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_SBER_THRES_C( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_sber_thres_c )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x6f, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_sber_thres_c = (temp&0x04)>>2 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_PAC_ERR_A_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfe)|disable ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_THRES_SBER_C( MB86A2x_NIM_PARAM *nim_param, UINT32 thres_c )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe8 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_c&0x0ff0000)>>16) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xe9 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)((thres_c&0x000ff00)>>8) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xea ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(thres_c&0x00000ff) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_PAC_ERR_A( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x62, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x10 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0xe0 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_PAC_ERR_A( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_tserr_a )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x63, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_tserr_a = (temp&0x10)>>4 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_PAC_ERR_B_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfd)|(disable<<1) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_PAC_ERR_B( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x62, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x20 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0xd0 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_PAC_ERR_B( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_tserr_b )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x63, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_tserr_b = (temp&0x20)>>5 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_PAC_ERR_C_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x51, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfb)|(disable<<2) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xdb ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_PAC_ERR_C( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x62, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x40 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0xb0 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x62, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_PAC_ERR_C( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_tserr_c )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0x63, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_tserr_c = (temp&0x40)>>6 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_GPIO_CNG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xee, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfd)|(disable<<1) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_GPIO3_CNG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0x7f)|(disable<<7) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_GPIO2_CNG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xbf)|(disable<<6) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_GPIO1_CNG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xdf)|(disable<<5) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_GPIO0_CNG_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xec, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xef)|(disable<<4) ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xec, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_GPIO_CNG( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xee, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x20 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0xdf ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_GPIO_CNG( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_gpio_cng )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xed, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_gpio_cng = (temp&0x02)>>1 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_IRQ_CH_SCH_END_Off( MB86A2x_NIM_PARAM *nim_param, UINT8 disable )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xee, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp = (temp&0xfe)|disable ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    
    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reset_IRQ_CH_SCH_END( MB86A2x_NIM_PARAM *nim_param )
{
    UINT32 Status ;
    UINT8 temp1, temp2 ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xee, &temp1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 | 0x10 ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    temp2 = temp1 & 0xef ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0xee, temp2 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Get_IRQ_CH_SCH_END( MB86A2x_NIM_PARAM *nim_param, UINT8 *irq_ch_sch_end )
{
    UINT32 Status ;
    UINT8 temp ;

    Status = MB86A22_read( (UINT8)(nim_param->MB86A2x_Adr), 0xed, &temp ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    *irq_ch_sch_end = temp&0x01 ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_RS_SW( MB86A2x_NIM_PARAM *nim_param, UINT8 RS_SW )
{
    UINT32 Status ;

    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x50, 0xd1 ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x51, (UINT8)(0x20|(RS_SW<<1)) ) ;
    if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Set_RECEP_BW( MB86A2x_NIM_PARAM *nim_param, UINT8 BW )
{
    UINT32 Status ;

    if( BW==0 ) {   //input signal BW = 13seg
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x71, 0x00 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    }
    else {          //input signal BW = 1seg
        Status = MB86A22_write( (UINT8)(nim_param->MB86A2x_Adr), 0x71, 0x02 ) ;
        if( Status==I2C_NG ) return MB86A2x_ERR_INIT ;
    }

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Monitor1( MB86A2x_PARAM myfe, MB86A2x_LOCKNG_INFO *locking_info )
{
    MB86A2x_AGC_INFO      agc_info ;        /* prepare pointer(4/8) */
    MB86A2x_FREQ_ERR_INFO freq_err_info ;   /* prepare pointer(5/8) */

    //Read the register value to check IF input level.
    (void) MB86A22_Get_IF_level( &myfe.nim_param, &agc_info  ) ;
    MB86A22_printf_IF_input_level( &agc_info ) ;

    //Read the register value to check IFAGCDAC output level.
    (void) MB86A22_Get_IFAGCDAC( &myfe.nim_param, &agc_info ) ;
    MB86A22_printf_IFAGCDAC( &agc_info ) ;

    //Read register values to check state value.
    (void) MB86A22_Get_SEQ_value( &myfe.nim_param, locking_info ) ;
    MB86A22_printf_SEQUENCE_value( locking_info ) ;

    //Read register values to check frequency error detected by timing recovery.
    (void) MB86A22_Get_STR_Freq_Error( &myfe, &freq_err_info ) ;
    MB86A22_printf_STR_Freq_Error( &freq_err_info ) ;

    //Read register values to check frequency error detected by carrier recovery.
    (void) MB86A22_Get_CRR_Freq_Error( &myfe, &freq_err_info ) ;
    MB86A22_printf_CRR_Freq_Error( &freq_err_info ) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Monitor2( MB86A2x_PARAM myfe, MB86A2x_SIG_QUAL_INFO *sign_qual_info )
{
    //Read register values to check whether C/N measurement is finished or not and store the result to CN_update.
    (void) MB86A22_Get_CN_Flag( &myfe.nim_param, sign_qual_info ) ;
    if( sign_qual_info->CN_update==1 ) {    //Check whether whether C/N measurement is finished or not by checking the value of CN_update.
        //Read register values to know C/N value and calculate estimated C/N.
        (void) MB86A22_Get_CN_Value( &myfe.nim_param, sign_qual_info ) ;
        //Reset C/N counter.
        (void) MB86A22_Reset_CN_Counter( &myfe.nim_param, &myfe.mon_tim_param ) ;
    }

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Monitor3( MB86A2x_PARAM myfe, MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_SIG_QUAL_INFO *sign_qual_info )
{
    if( (1<=tmcc_info->NUM_A_current)&&(tmcc_info->NUM_A_current<=13) ) {   //Check whether the segment number of layer-A is within the range of 1~13.
        //Read register values to check whether pre-Viterbi BER of layer-A is finished or not and store the result to VBER_A_update.
        (void) MB86A22_Get_VBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->VBER_A_update==1 ) {    //Check whether pre-Viterbi BER of layer-A is finished or not by checking the value of VBER_A_update.
            //Read register values to know pre-Viterbi BER of layer-A and calculate pre-Viterbi BER.
            (void) MB86A22_Get_VBER_A( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for pre-Viterbi of layer-A.
            (void) MB86A22_Reset_VBER_A_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-Viterbi BER of layer-A is finished or not and store the result to SBER_A_update.
        (void) MB86A22_Get_SBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->SBER_A_update==1 ) {    //Check whether post-Viterbi BER of layer-A is finished or not by checking the value of SBER_A_update.
            //Read register values to know post-Viterbi BER of layer-A and calculate post-Viterbi BER.
            (void) MB86A22_Get_SBER_A( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-Viterbi of layer-A.
            (void) MB86A22_Reset_SBER_A_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-RS BER of layer-A is finished or not and store the result to PBER_A_update.
        (void) MB86A22_Get_PBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->PBER_A_update==1 ) {    //Check whether post-RS BER of layer-A is finished or not by checking the value of PBER_A_update.
            //Read register values to know post-RS BER of layer-A and calculate post-RS BER.
            (void) MB86A22_Get_PBER_A( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-RS of layer-A.
            (void) MB86A22_Reset_PBER_A_Counter( &myfe.nim_param ) ;
        }
    }
    MB86A22_printf_BER_Lay_A( sign_qual_info, tmcc_info ) ;
        if( (1<=tmcc_info->NUM_B_current)&&(tmcc_info->NUM_B_current<=12) ) {   //Check whether the segment number of layer-B is within the range of 1~12.
        //Read register values to check whether pre-Viterbi BER of layer-B is finished or not and store the result to VBER_update.
            (void) MB86A22_Get_VBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->VBER_B_update==1 ) {    //Check whether pre-Viterbi BER of layer-B is finished or not by checking the value of VBER_B_update.
            //Read register values to know pre-Viterbi BER of layer-B and calculate pre-Viterbi BER.
            (void) MB86A22_Get_VBER_B( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for pre-Viterbi of layer-B.
            (void) MB86A22_Reset_VBER_B_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-Viterbi BER of layer-B is finished or not and store the result to SBER_B_update.
        (void) MB86A22_Get_SBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->SBER_B_update==1 ) {    //Check whether post-Viterbi BER of layer-B is finished or not by checking the value of SBER_B_update.
            //Read register values to know post-Viterbi BER of layer-B and calculate post-Viterbi BER.
            (void) MB86A22_Get_SBER_B( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-Viterbi of layer-B.
            (void) MB86A22_Reset_SBER_B_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-RS BER of layer-B is finished or not and store 
        (void) MB86A22_Get_PBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->PBER_B_update==1 ) {    //Check whether post-RS BER of layer-B is finished or not by checking the value of PBER_B_update.
            //Read register values to know post-RS BER of layer-B and calculate post-RS BER.
            (void) MB86A22_Get_PBER_B( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-RS of layer-B.
            (void) MB86A22_Reset_PBER_B_Counter( &myfe.nim_param ) ;
        }
    }
    MB86A22_printf_BER_Lay_B( sign_qual_info, tmcc_info ) ;

    if( (1<=tmcc_info->NUM_C_current)&&(tmcc_info->NUM_C_current<=11) ) {   //Check whether the segment number of layer-C is within the range of 1~11.
        //Read register values to check whether pre-Viterbi BER of layer-C is finished or not and store the result to VBER_update.
        (void) MB86A22_Get_VBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->VBER_C_update==1 ) {    //Check whether pre-Viterbi BER of layer-C is finished or not by checking the value of VBER_C_update.
            //Read register values to know pre-Viterbi BER of layer-C and calculate pre-Viterbi BER.
            (void) MB86A22_Get_VBER_C( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for pre-Viterbi of layer-C.
            (void) MB86A22_Reset_VBER_C_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-Viterbi BER of layer-C is finished or not and store the result to SBER_C_update.
        (void) MB86A22_Get_SBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->SBER_C_update==1 ) {    //Check whether post-Viterbi BER of layer-C is finished or not by checking the value of SBER_C_update.
            //Read register values to know post-Viterbi BER of layer-C and calculate post-Viterbi BER.
            (void) MB86A22_Get_SBER_C( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-Viterbi of Layer-C.
            (void) MB86A22_Reset_SBER_C_Counter( &myfe.nim_param ) ;
        }
        //Read register values to check whether post-RS BER of layer-C is finished or not and store the result to PBER_C_update.
        (void) MB86A22_Get_PBER_Flag( &myfe.nim_param, sign_qual_info ) ;
        if( sign_qual_info->PBER_C_update==1 ) {    //Check whether post-RS BER of layer-C is finished or not by checking the value of PBER_C_update.
            //Read register values to know post-RS BER of layer-C and calculate post-RS BER.
            (void) MB86A22_Get_PBER_C( &myfe.nim_param, &myfe.mon_tim_param, sign_qual_info ) ;
            //Reset BER counter for post-RS of layer-C.
            (void) MB86A22_Reset_PBER_C_Counter( &myfe.nim_param ) ;
        }
    }
    MB86A22_printf_BER_Lay_C( sign_qual_info, tmcc_info ) ;

    //Read register values to check whether MER measurement is finished or not and store the result to MER_update.
    (void) MB86A22_Get_MER_Flag( &myfe.nim_param, sign_qual_info ) ;
    if( sign_qual_info->MER_update==1 ) {   //Check whether whether MER measurement is finished or not by checking the value of MER_update.
        //Read register values to know MER value and calculate estimated C/N.
        (void) MB86A22_Get_MER_A_Value( &myfe.nim_param, tmcc_info, sign_qual_info ) ;
        (void) MB86A22_Get_MER_B_Value( &myfe.nim_param, tmcc_info, sign_qual_info ) ;
        (void) MB86A22_Get_MER_C_Value( &myfe.nim_param, tmcc_info, sign_qual_info ) ;
        //Reset MER counter.
        (void) MB86A22_Reset_MER_Counter( &myfe.nim_param ) ;
    }

    MB86A22_printf_MER( sign_qual_info ) ;

    return MB86A2x_OK ;
}

MB86A2x_ERR_MSG MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_1_freq( 
                MB86A2x_PARAM *myfe, MB86A2x_LOCKNG_INFO *locking_info,
                MB86A2x_SIG_QUAL_INFO *sign_qual_info, MB86A2x_VERSION_INFO *version_info,
                UINT32 TunerUnit, UINT32 uItemId, UINT32 uValue,
                UINT32 freq )
{
    UINT32 Status = I2C_OK;
    #if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    int i2cerr ;
    #endif
    #if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    tmErrorCode_t err = TM_OK;
    #endif

    //Set BER monitor timer parameters and MER monitor parameter.
    myfe->mon_tim_param.VBER_A_NUM = 0x0000ff ;
    myfe->mon_tim_param.VBER_B_NUM = 0x00ffff ;
    myfe->mon_tim_param.VBER_C_NUM = 0x00ffff ;
    myfe->mon_tim_param.SBER_A_NUM = 0x0000ff ;
    myfe->mon_tim_param.SBER_B_NUM = 0x003fff ;
    myfe->mon_tim_param.SBER_C_NUM = 0x003fff ;
    myfe->mon_tim_param.PBER_A_NUM = 0x0000ff ;
    myfe->mon_tim_param.PBER_B_NUM = 0x003fff ;
    myfe->mon_tim_param.PBER_C_NUM = 0x003fff ;
    myfe->mon_tim_param.CN_SYMBOL  = 0x04 ;
    myfe->mon_tim_param.MER_SYMBOL = 0x04 ;
    //You can also set these values by chaning the values in "MB86A22_***_default_setting.h" instead of above codes.

    //Write the timer parameters.
    (void) MB86A22_Set_Monitor_Timer( myfe ) ;

    //Initialize the parameters of LOCKING_INFO.
    (void) MB86A22_Init_LOCKNG_INFO( locking_info ) ;

    //Initialize the parameters of SIG_QUAL_INFO.
    (void) MB86A22_Init_SIG_QUAL_INFO( sign_qual_info ) ;

    //Set Channel frequency value.
    myfe->nim_param.tuner_param.RF_freq_kHz = freq ;

    //Set RESEP_SEG value to MB86A2x_RECEP_SEG_A_OTHER.
    //myfe->seg_param.RECEP_SEG = MB86A2x_RECEP_SEG_A_OTHER ;

    //Set mode and guard value for all mods and all guards
    (void) MB86A22_Set_ModeGuard_all( myfe ) ;

    //Set timing recovery to default value
    (void) MB86A22_Set_STR_OFS( &(myfe->nim_param), &(myfe->pll_param), &(myfe->recov_param) ) ;

    //Write channel setting parameters.
    (void) MB86A22_Set_Channel_1( myfe, version_info ) ;
    
#if (MB86A22_TUNER_SUPPORT_TDA18211 > 0)
    if(myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18211)
    {
        // Tuner RF setting
        i2cerr = tmbslTDA18271SetRf(TunerUnit, myfe->nim_param.tuner_param.RF_freq_kHz*1000);
        if(i2cerr >= 1)
        {
            return MB86A2x_ERR_RFTUNE;
        }
        i2cerr = tmbslTDA18271SetConfig(TunerUnit, uItemId, uValue) ; // set IF_Level -12 dB
        if(i2cerr >= 1)
        {
            return MB86A2x_ERR_RFTUNE;
         }
    }
#endif

#if (MB86A22_TUNER_SUPPORT_TDA18212 > 0)
    if( myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_TDA18212 )
    {
        err = tmbslTDA182I2SetRf(TunerUnit, myfe->nim_param.tuner_param.RF_freq_kHz*1000);
        if(err != TM_OK)
        {
            return MB86A2x_ERR_RFTUNE;
        }
    }
#endif

#if (MB86A22_TUNER_SUPPORT_NI386X > 0)    
    if( myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_NI386X )
    {
      Status = MB86A22_NXP_TUNER_INIT_2( &(myfe->nim_param.tuner_param) ) ;   //calculate tuner parameter
      Status = MB86A22_tuner_write( myfe->nim_param.tuner_param.Tuner_Adr, 4 , myfe->nim_param.tuner_param.Tuner_Data ); //write data to tuner
    }
#endif

#if (MB86A22_TUNER_SUPPORT_MAX3580 > 0)    
    if( myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_MAX3580 )
    {
      Status = MB86A22_Max3580_TUNER_INIT((UINT8)(myfe->nim_param.MB86A2x_Adr), myfe->nim_param.tuner_param.Tuner_Adr, &(myfe->nim_param.tuner_param));
    }
#endif

#if (MB86A22_TUNER_SUPPORT_MXL136RF > 0)    
    if( myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_MXL136RF )
    {
      Status = MB86A22_MxL_TUNER_INIT(&(myfe->nim_param.tuner_param), myfe->pll_param.BANDSEL);
    }
#endif

#if (MB86A22_TUNER_SUPPORT_FC2581 > 0)    
    if( myfe->nim_param.TUNER_TYPE == MB86A22_ISDBT_FC2581)
    {
      fc2581_set_init(myfe->nim_param.tuner_param.Tuner_Adr, 2);
      fc2581_set_freq(myfe->nim_param.tuner_param.Tuner_Adr, myfe->nim_param.tuner_param.RF_freq_kHz);
      fc2581_set_filter(myfe->nim_param.tuner_param.Tuner_Adr, 6);
    }
#endif

#if 0    
    Status = MB86A22_ALPS_TDAL_TUNER_INIT( &(myfe->nim_param.tuner_param) ) ;   //calculate tuner parameter
    Status = MB86A22_tuner_write( myfe->nim_param.tuner_param.Tuner_Adr, 5 , myfe->nim_param.tuner_param.Tuner_Data ); //write data to tuner
#endif

    if( Status == I2C_NG )
        return MB86A2x_ERR_RFTUNE ;

    //Write channel setting parameters.
    (void) MB86A22_Set_Channel_2(myfe, version_info) ;

    //Write the data to reset monitor timers.
    (void) MB86A22_Reset_Monitor_Timer(myfe) ;

    return MB86A2x_OK ;
}


MB86A2x_ERR_MSG MB86A22_Reception_before_Channel_Search_Fixed_Timer_Register_Values_2( 
                MB86A2x_PARAM *myfe, MB86A2x_LOCKNG_INFO *locking_info, MB86A2x_SIG_QUAL_INFO *sign_qual_info,
                MB86A2x_TMCC_INFO *tmcc_info, MB86A2x_CH_SCH_INFO *ch_sch_info, MB86A2x_VERSION_INFO *version_info,
                MB86A2x_RECEP_LAYER recep_layer )
{
    //UINT8 ch_number ;       //Channel number of detected channel(0,1,2, ... ,N-1)
                            //N:The number of detected channel 

    //Set the channel number to store channel information to 69.
   // ch_number = 69 ;    //This value should be 69 in spite of reception channel.
    (void) MB86A22_Monitor1( *myfe, locking_info ) ;

    //Read register values to check the mode, GI.
    (void) MB86A22_Get_Mode_Guard( &(myfe->nim_param), tmcc_info ) ;
    MB86A22_printf_Mode_Guard( tmcc_info ) ;

    if( locking_info->STATE>=7 ) {
        MB86A22_Monitor2( *myfe, sign_qual_info ) ;
    }
    MB86A22_printf_CN( sign_qual_info ) ;

    if( locking_info->STATE>=8 ) {  //Check whether the state value has reached to 8 or not.
        //Read register values to check the TMCC information.
        (void) MB86A22_Get_TMCC_Info( &(myfe->nim_param), tmcc_info ) ;
        MB86A22_printf_TMCC( tmcc_info ) ;
    }
    if( locking_info->S8_REACH==1 ) {
        //Store TMCC information (channel information) to pointer.
        (void) MB86A22_Store_Channel_Info( myfe->nim_param.tuner_param.CH, tmcc_info, ch_sch_info ) ;
#if 0
        //Define reception layer
        if(recep_layer==MB86A2x_RECEP_HIGHEST ) {
            (void) MB86A2x_Init_RECEP_SEG_Highest( ch_sch_info, &(myfe->seg_param), ch_number ) ;
        }
        else if( recep_layer==MB86A2x_RECEP_LAYER_A ) {
            (void) MB86A2x_Init_RECEP_SEG_Lay_A( ch_sch_info, &(myfe->seg_param), ch_number ) ;
        }
        else if( recep_layer==MB86A2x_RECEP_LAYER_B ) {
            (void) MB86A2x_Init_RECEP_SEG_Lay_B( ch_sch_info, &(myfe->seg_param), ch_number ) ;
        }
        else if( recep_layer==MB86A2x_RECEP_LAYER_C ) {
            (void) MB86A2x_Init_RECEP_SEG_Lay_C( ch_sch_info, &(myfe->seg_param), ch_number ) ;
        }
        //Write reception segment parameters.
        (void) MB86A22_Set_RECEP_SEG( &(myfe->nim_param), &(myfe->seg_param) ) ;
#endif
                
    }
    if( locking_info->STATE>=9 ) {  //Check whether the state value has reached to 9 or not.
        (void) MB86A22_Monitor3( *myfe, tmcc_info, sign_qual_info ) ;
    }

    return MB86A2x_OK ;
}

